@@ -91,7 +91,18 @@ namespace IKVM.Reflection.Reader
 
 			internal Type GetType(ModuleReader module)
 			{
-				return type ?? (type = module.ResolveExportedType(index));
+				// guard against circular type forwarding
+				if (type == MarkerType.Pinned)
+				{
+					TypeName typeName = module.GetTypeName(module.ExportedType.records[index].TypeNamespace, module.ExportedType.records[index].TypeName);
+					return module.universe.GetMissingTypeOrThrow(module, module, null, typeName).SetCyclicTypeForwarder();
+				}
+				else if (type == null)
+				{
+					type = MarkerType.Pinned;
+					type = module.ResolveExportedType(index);
+				}
+				return type;
 			}
 		}
 
@@ -864,7 +875,11 @@ namespace IKVM.Reflection.Reader
 				{
 					return field;
 				}
+#if CORECLR
+				throw new MissingFieldException(org.ToString() + "." + name);
+#else
 				throw new MissingFieldException(org.ToString(), name);
+#endif
 			}
 			else
 			{
@@ -883,7 +898,11 @@ namespace IKVM.Reflection.Reader
 				{
 					return method;
 				}
+#if CORECLR
+				throw new MissingMethodException(org.ToString() + "." + name);
+#else
 				throw new MissingMethodException(org.ToString(), name);
+#endif
 			}
 		}
 
