@@ -13,7 +13,12 @@ struct thread_stop_info {
 /* the instrumented function uses any callee saved registers, they may  */
 /* be pushed to the stack much earlier.  Also, on amd64 'push' puts 8   */
 /* bytes on the stack even though our pointers are 4 bytes.             */
+#ifdef __arm__
+/* For ARM we save r4-r8, r10-r12, r14 */
+#define NACL_GC_REG_STORAGE_SIZE 9
+#else
 #define NACL_GC_REG_STORAGE_SIZE 20
+#endif
     ptr_t reg_storage[NACL_GC_REG_STORAGE_SIZE];
 #endif
 };
