@@ -37,6 +37,7 @@ namespace System.Windows.Forms.RTF {
 		#region	Local Variables
 		internal const char	EOF = unchecked((char)-1);
 		internal const int	NoParam = -1000000;
+		internal const int	DefaultEncodingCodePage = 1252;
 
 		private TokenClass	rtf_class;
 		private Major		major;
@@ -44,7 +45,7 @@ namespace System.Windows.Forms.RTF {
 		private int		param;
 		private string encoded_text;
 		private Encoding encoding;
-		private int encoding_code_page = 1252;
+		private int encoding_code_page = DefaultEncodingCodePage;
 		private StringBuilder	text_buffer;
 		private Picture picture;
 		private int		line_num;
@@ -100,6 +101,8 @@ namespace System.Windows.Forms.RTF {
 			line_pos = 0;
 			prev_char = unchecked((char)-1);
 			bump_line = false;
+			font_list = null;
+			charset_stack = null;
 
 			cur_charset = new Charset();
 
@@ -253,13 +256,20 @@ namespace System.Windows.Forms.RTF {
 			font.Name = name;
 		}
 
+		/// <summary>Read the next character from the input - skip any crlf</summary>
+		private char GetChar ()
+		{
+			return GetChar (true);
+		}
+
 		/// <summary>Read the next character from the input</summary>
-		private char GetChar() {
-			char	c;
+		private char GetChar(bool skipCrLf) 
+		{
+			int	c;
 			bool	old_bump_line;
 
 SkipCRLF:
-			if ((c = (char) source.Read()) != -1) {
+			if ((c = source.Read()) != -1) {
 				this.text_buffer.Append((char) c);
 			}
 
@@ -270,18 +280,20 @@ SkipCRLF:
 			old_bump_line = bump_line;
 			bump_line = false;
 
-			if (c == '\r') {
-				bump_line = true;
-				text_buffer.Length--;
-				goto SkipCRLF;
-			} else if (c == '\n') {
-				bump_line = true;
-				if (this.prev_char == '\r') {
-					old_bump_line = false;
+			if (skipCrLf) {
+				if (c == '\r') {
+					bump_line = true;
+					text_buffer.Length--;
+					goto SkipCRLF;
+				} else if (c == '\n') {
+					bump_line = true;
+					if (this.prev_char == '\r') {
+						old_bump_line = false;
+					}
+
+					text_buffer.Length--;
+					goto SkipCRLF;
 				}
-				
-				text_buffer.Length--;
-				goto SkipCRLF;
 			}
 
 			this.line_pos ++;
@@ -323,7 +335,7 @@ SkipCRLF:
 			
 		}
 
-		/// <summary>Skip to the end of the current group</summary>
+		/// <summary>Skip to the end of the next group to start or current group we are in</summary>
 		public void SkipGroup() {
 			int	level;
 
@@ -370,6 +382,10 @@ SkipCRLF:
 
 			if (CheckCMM (TokenClass.Control, Major.Unicode, Minor.UnicodeAnsiCodepage)) {
 				encoding_code_page = param;
+
+				// fallback to the default one in case we have an invalid value
+				if (encoding_code_page < 0 || encoding_code_page > 65535)
+					encoding_code_page = DefaultEncodingCodePage;
 			}
 
 			if (((this.cur_charset.Flags & CharsetFlags.Read) != 0) && CheckCM(TokenClass.Control, Major.CharSet)) {
@@ -480,7 +496,7 @@ SkipCRLF:
 			}
 
 			while (Char.IsLetter(c)) {
-				if ((c = GetChar()) == EOF) {
+				if ((c = GetChar(false)) == EOF) {
 					break;
 				}
 			}
@@ -513,7 +529,7 @@ SkipCRLF:
 			}
 
 			if (c != EOF) {
-				if (c != ' ') {
+				if (c != ' ' && c != '\r' && c != '\n') {
 					this.pushed_char = c;
 				}
 				this.text_buffer.Length--;
@@ -601,19 +617,30 @@ SkipCRLF:
 
 		private void HandleOptDest (RTF rtf)
 		{
+			int group_levels = 1;
+
 			while (true) {
 				GetToken ();
 
-				if (rtf.CheckCM (TokenClass.Group, Major.EndGroup))
-					break;
-
+				// Here is where we should handle recognised optional
+				// destinations.
 				//
-				// We don't want to skip all optional dests
+				// Handle a picture group 
 				//
 				if (rtf.CheckCMM (TokenClass.Control, Major.Destination, Minor.Pict)) {
 					ReadPictGroup (rtf);
 					return;
 				}
+
+				if (rtf.CheckCM (TokenClass.Group, Major.EndGroup)) {
+					if ((--group_levels) == 0) {
+						break;
+					}
+				}
+
+				if (rtf.CheckCM (TokenClass.Group, Major.BeginGroup)) {
+					group_levels++;
+				}
 			}
 		}
 
@@ -949,7 +976,7 @@ SkipCRLF:
 
 					picture.Data.Seek (0, SeekOrigin.Begin);
 
-					char c = (char) rtf.major;
+					//char c = (char) rtf.major;
 
 					uint digitValue1;
 					uint digitValue2;
