@@ -25,7 +25,6 @@
 //
 
 
-#if NET_2_0
 
 using System;
 using System.Drawing;
@@ -223,7 +222,7 @@ namespace System.Windows.Forms {
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		public DataGridViewColumn OwningColumn {
 			get {
-				if (DataGridView == null || columnIndex == -1)
+				if (DataGridView == null || columnIndex < 0 || columnIndex >= DataGridView.Columns.Count)
 					return null;
 					
 				return DataGridView.Columns[columnIndex];
@@ -388,7 +387,16 @@ namespace System.Windows.Forms {
 
 		[Browsable (false)]
 		public virtual Type ValueType {
-			get { return valueType; }
+			get { 
+				if (valueType == null) {
+					if (DataProperty != null)
+						valueType = DataProperty.PropertyType;
+					else if (OwningColumn != null)
+						valueType = OwningColumn.ValueType;
+				}
+
+				return valueType;
+			}
 			set { valueType = value; }
 		}
 
@@ -716,7 +724,7 @@ namespace System.Windows.Forms {
 		}
 
 		public override string ToString () {
-			return String.Format("{0} {RowIndex = {1}, ColumnIndex = {2}}", this.GetType().FullName, RowIndex, columnIndex);
+			return String.Format("{0} {{ ColumnIndex={1}, RowIndex={2} }}", this.GetType().Name, ColumnIndex, RowIndex);
 		}
 
 		protected virtual Rectangle BorderWidths (DataGridViewAdvancedBorderStyle advancedBorderStyle)
@@ -796,7 +804,7 @@ namespace System.Windows.Forms {
 			string value = null;
 			
 			if (Selected) {
-				DataGridViewCellStyle style = GetInheritedStyle (null, rowIndex, false);
+//				DataGridViewCellStyle style = GetInheritedStyle (null, rowIndex, false);
 				value = GetEditedFormattedValue (rowIndex, DataGridViewDataErrorContexts.ClipboardContent | DataGridViewDataErrorContexts.Formatting) as string;
 			}
 
@@ -878,6 +886,11 @@ namespace System.Windows.Forms {
 				value = e.Value;
 			}
 			
+			if (value == null || (cellStyle != null && value == cellStyle.DataSourceNullValue)) {
+				if (FormattedValueType == typeof (string))
+					return String.Empty;
+			}
+
 			if (FormattedValueType == typeof(string) && value is IFormattable && !String.IsNullOrEmpty (cellStyle.Format))
 				return ((IFormattable) value).ToString (cellStyle.Format, cellStyle.FormatProvider);
 			if (value != null && FormattedValueType.IsAssignableFrom (value.GetType()))
@@ -912,8 +925,11 @@ namespace System.Windows.Forms {
 		protected virtual object GetValue (int rowIndex) {
 			if (DataGridView != null && (RowIndex < 0 || RowIndex >= DataGridView.Rows.Count))
 				throw new ArgumentOutOfRangeException ("rowIndex", "Specified argument was out of the range of valid values.");
-				
-			if (DataProperty != null)
+		
+			if (OwningRow != null && DataGridView != null && OwningRow.Index == DataGridView.NewRowIndex)
+				return DefaultNewRowValue;
+
+			if (DataProperty != null && OwningRow.DataBoundItem != null)
 				return DataProperty.GetValue (OwningRow.DataBoundItem);
 			
 			if (valuex != null)
@@ -927,9 +943,9 @@ namespace System.Windows.Forms {
 		
 		private PropertyDescriptor DataProperty {
 			get {
-				if (OwningColumn != null && !String.IsNullOrEmpty (OwningColumn.DataPropertyName) && 
-				    OwningRow != null && OwningRow.DataBoundItem != null)
-					return TypeDescriptor.GetProperties (OwningRow.DataBoundItem)[OwningColumn.DataPropertyName];
+				if (OwningColumn != null && OwningColumn.DataColumnIndex != -1 && 
+				    DataGridView != null && DataGridView.DataManager != null)
+					return DataGridView.DataManager.GetItemProperties()[OwningColumn.DataColumnIndex];
 				return null;
 			}
 		}
@@ -1160,11 +1176,11 @@ namespace System.Windows.Forms {
 					if (DataGridView.CellBorderStyle != DataGridViewCellBorderStyle.Single)
 						graphics.DrawLine (pen, bounds.X, bounds.Y, bounds.X, bounds.Y + bounds.Height - 1);
 					break;
+				case DataGridViewAdvancedCellBorderStyle.Outset:
 				case DataGridViewAdvancedCellBorderStyle.Inset:
 					graphics.DrawLine(pen, bounds.X, bounds.Y, bounds.X, bounds.Y + bounds.Height - 1);
 					break;
 				case DataGridViewAdvancedCellBorderStyle.InsetDouble:
-				case DataGridViewAdvancedCellBorderStyle.Outset:
 				case DataGridViewAdvancedCellBorderStyle.OutsetDouble:
 					graphics.DrawLine(pen, bounds.X, bounds.Y, bounds.X, bounds.Y + bounds.Height - 1);
 					graphics.DrawLine(pen, bounds.X + 2, bounds.Y, bounds.X + 2, bounds.Y + bounds.Height - 1);
@@ -1176,7 +1192,10 @@ namespace System.Windows.Forms {
 				case DataGridViewAdvancedCellBorderStyle.Single:
 					graphics.DrawLine(pen, bounds.X + bounds.Width - 1, bounds.Y, bounds.X + bounds.Width - 1, bounds.Y + bounds.Height - 1);
 					break;
+				case DataGridViewAdvancedCellBorderStyle.Outset:
 				case DataGridViewAdvancedCellBorderStyle.Inset:
+				case DataGridViewAdvancedCellBorderStyle.InsetDouble:
+				case DataGridViewAdvancedCellBorderStyle.OutsetDouble:
 					graphics.DrawLine(pen, bounds.X + bounds.Width, bounds.Y, bounds.X + bounds.Width, bounds.Y + bounds.Height - 1);
 					break;
 			}
@@ -1187,14 +1206,21 @@ namespace System.Windows.Forms {
 					if (DataGridView.CellBorderStyle != DataGridViewCellBorderStyle.Single)
 						graphics.DrawLine(pen, bounds.X, bounds.Y, bounds.X + bounds.Width - 1, bounds.Y);
 					break;
+				case DataGridViewAdvancedCellBorderStyle.Outset:
 				case DataGridViewAdvancedCellBorderStyle.Inset:
+				case DataGridViewAdvancedCellBorderStyle.InsetDouble:
+				case DataGridViewAdvancedCellBorderStyle.OutsetDouble:
 					graphics.DrawLine(pen, bounds.X, bounds.Y, bounds.X + bounds.Width - 1, bounds.Y);
 					break;
 			}
 			
 			// Paint the bottom border, if any
 			switch (advancedBorderStyle.Bottom) {
+				case DataGridViewAdvancedCellBorderStyle.Outset:
+				case DataGridViewAdvancedCellBorderStyle.Inset:
 				case DataGridViewAdvancedCellBorderStyle.Single:
+				case DataGridViewAdvancedCellBorderStyle.InsetDouble:
+				case DataGridViewAdvancedCellBorderStyle.OutsetDouble:
 					graphics.DrawLine(pen, bounds.X, bounds.Y + bounds.Height - 1, bounds.X + bounds.Width - 1, bounds.Y + bounds.Height - 1);
 					break;
 			}
@@ -1298,10 +1324,15 @@ namespace System.Windows.Forms {
 		{
 			object oldValue = this.Value;
 
-			if (DataProperty != null && !DataProperty.IsReadOnly)
+			if (DataProperty != null && !DataProperty.IsReadOnly) {
 				DataProperty.SetValue (OwningRow.DataBoundItem, value);
-			else
+			} else if (DataGridView != null && DataGridView.VirtualMode) {
+				DataGridViewCellValueEventArgs ea = new DataGridViewCellValueEventArgs(ColumnIndex, RowIndex);
+				ea.Value = value;
+				DataGridView.OnCellValuePushed(ea);
+			} else {
 				valuex = value;
+			}
 
 			if (!Object.ReferenceEquals (oldValue, value) || !Object.Equals (oldValue, value)) {
 				RaiseCellValueChanged (new DataGridViewCellEventArgs (ColumnIndex, RowIndex));
@@ -1560,5 +1591,3 @@ namespace System.Windows.Forms {
 	}
 
 }
-
-#endif
