@@ -31,9 +31,6 @@
 
 namespace System.Windows.Forms
 {
-#if !NET_2_0
-	[Serializable]
-#endif
 	public enum ToolBarTextAlign
 	{
 		Underneath = 0,
