@@ -25,7 +25,6 @@
 // Authors:
 //	Jonathan Pobst (monkey@jpobst.com)
 //
-#if NET_2_0
 using System;
 using System.Collections.Generic;
 using System.Text;
@@ -36,7 +35,7 @@ using System.Windows.Forms;
 namespace MonoTests.System.Windows.Forms
 {
 	[TestFixture]
-	public class ToolStripManagerTest
+	public class ToolStripManagerTest : TestHelper
 	{
 		[Test]
 		[Ignore ("This has to run all by itself.  Toolstrips in other tests register themselves and mess this up.")]
@@ -503,6 +502,31 @@ namespace MonoTests.System.Windows.Forms
 			Assert.AreEqual ("Open 2", tsmi2.DropDownItems[1].Text, "M12-4");
 		}
 
+		[Test]  // For bug #81477
+		public void MethodMergeRecursive ()
+		{
+			MenuStrip ms1 = new MenuStrip ();
+			MenuStrip ms2 = new MenuStrip ();
+
+			ToolStripMenuItem tsmi1 = (ToolStripMenuItem)ms1.Items.Add ("File");
+			ToolStripMenuItem tsmi2 = (ToolStripMenuItem)ms2.Items.Add ("File");
+
+			tsmi1.DropDownItems.Add ("New 1");
+			tsmi1.DropDownItems.Add ("Open 1");
+
+			tsmi2.DropDownItems.Add ("New 2");
+			tsmi2.DropDownItems.Add ("Open 2");
+			
+			tsmi2.DropDownItems[0].MergeAction = MergeAction.Insert;
+			tsmi2.DropDownItems[0].MergeIndex = 0;
+
+			tsmi2.MergeAction = MergeAction.MatchOnly;
+
+			ToolStripManager.Merge (ms2, ms1);
+
+			Assert.AreEqual ("New 2", tsmi1.DropDownItems[0].Text, "M13");
+		}
+		
 		[Test]
 		[ExpectedException (typeof (ArgumentNullException))]
 		public void MethodMergeANE1 ()
@@ -571,6 +595,326 @@ namespace MonoTests.System.Windows.Forms
 			
 			Assert.AreEqual (false, ToolStripManager.Merge (ts1, ts2), "K1");
 		}
+		
+		[Test]  // Should not throw AOORE
+		public void Bug347669 ()
+		{
+			Form main = new MainForm ();
+			main.Show ();
+			Form frm = new Childform ();
+			frm.MdiParent = main;
+			frm.Show ();
+			main.Dispose ();
+		}
+		
+		private class MainForm : Form
+		{
+
+			public MainForm ()
+			{
+				mainMenuStrip1 = new MenuStrip ();
+				File = new ToolStripMenuItem ();
+				openToolStripMenuItem = new ToolStripMenuItem ();
+				toolStripSeparator2 = new ToolStripSeparator ();
+				Dokument = new ToolStripMenuItem ();
+				arveToolStripMenuItem = new ToolStripMenuItem ();
+
+				WareHouse = new ToolStripMenuItem ();
+				Personnel = new ToolStripMenuItem ();
+				Payroll = new ToolStripMenuItem ();
+				FixedAssets = new ToolStripMenuItem ();
+				Supplies = new ToolStripMenuItem ();
+				GeneralLedger = new ToolStripMenuItem ();
+				Manufacturing = new ToolStripMenuItem ();
+				PointOfSale = new ToolStripMenuItem ();
+				CardTerminal = new ToolStripMenuItem ();
+				Rent = new ToolStripMenuItem ();
+				WayBill = new ToolStripMenuItem ();
+				CustomerRelationManagement = new ToolStripMenuItem ();
+				toolStripSeparator8 = new ToolStripSeparator ();
+				NewUserToolStripMenuItem = new ToolStripMenuItem ();
+				exitToolStripMenuItem = new ToolStripMenuItem ();
+
+				mainMenuStrip1.Items.AddRange (new ToolStripItem[] {
+            File
+        });
+				mainMenuStrip1.Location = new Point (0, 0);
+				mainMenuStrip1.Name = "mainMenuStrip1";
+				mainMenuStrip1.Size = new Size (644, 24);
+				mainMenuStrip1.TabIndex = 3;
+				mainMenuStrip1.Text = "menuStrip1";
+				File.DropDownItems.AddRange (new ToolStripItem[] {
+            openToolStripMenuItem,
+            toolStripSeparator2,
+            Dokument,
+            WareHouse,
+            Personnel,
+            Payroll,
+            FixedAssets,
+            Supplies,
+            GeneralLedger,
+            Manufacturing,
+            PointOfSale,
+            CardTerminal,
+            Rent,
+            WayBill,
+            CustomerRelationManagement,
+            toolStripSeparator8,
+            NewUserToolStripMenuItem,
+            exitToolStripMenuItem});
+				File.MergeAction = MergeAction.Insert;
+				File.MergeIndex = 1;
+				File.Name = "File";
+				File.Size = new Size (35, 20);
+				File.Text = "&File";
+				openToolStripMenuItem.MergeIndex = 1;
+				openToolStripMenuItem.Name = "openToolStripMenuItem";
+				openToolStripMenuItem.Size = new Size (196, 22);
+				openToolStripMenuItem.Text = "Open";
+				toolStripSeparator2.MergeIndex = 2;
+				toolStripSeparator2.Name = "toolStripSeparator2";
+				toolStripSeparator2.Size = new Size (193, 6);
+				Dokument.DropDownItems.AddRange (new ToolStripItem[] {
+            arveToolStripMenuItem
+        });
+				Dokument.MergeIndex = 3;
+				Dokument.Name = "Dokument";
+				Dokument.Size = new Size (196, 22);
+				Dokument.Text = "&Dokument";
+				arveToolStripMenuItem.Name = "arveToolStripMenuItem";
+				arveToolStripMenuItem.ShortcutKeyDisplayString = "Ctrl+E";
+				arveToolStripMenuItem.ShortcutKeys = ((Keys)((Keys.Control | Keys.E)));
+				arveToolStripMenuItem.Size = new Size (146, 22);
+				arveToolStripMenuItem.Text = "Arve";
+				WareHouse.MergeIndex = 4;
+				WareHouse.Name = "WareHouse";
+				WareHouse.Size = new Size (196, 22);
+				WareHouse.Text = "&Ladu";
+				Personnel.MergeIndex = 5;
+				Personnel.Name = "Personnel";
+				Personnel.Size = new Size (196, 22);
+				Personnel.Text = "&Kaader";
+				Payroll.MergeIndex = 6;
+				Payroll.Name = "Payroll";
+				Payroll.Size = new Size (196, 22);
+				Payroll.Text = "&Palk";
+				FixedAssets.MergeIndex = 7;
+				FixedAssets.Name = "FixedAssets";
+				FixedAssets.Size = new Size (196, 22);
+				FixedAssets.Text = "Test";
+				Supplies.MergeIndex = 8;
+				Supplies.Name = "Supplies";
+				Supplies.Size = new Size (196, 22);
+				Supplies.Text = "Sample";
+				GeneralLedger.MergeIndex = 9;
+				GeneralLedger.Name = "GeneralLedger";
+				GeneralLedger.Size = new Size (196, 22);
+				GeneralLedger.Text = "Blah";
+				Manufacturing.MergeIndex = 10;
+				Manufacturing.Name = "Manufacturing";
+				Manufacturing.Size = new Size (196, 22);
+				Manufacturing.Text = "&Tootmine";
+				PointOfSale.MergeIndex = 11;
+				PointOfSale.Name = "PointOfSale";
+				PointOfSale.Size = new Size (196, 22);
+				PointOfSale.Text = "IceCream";
+				CardTerminal.MergeIndex = 12;
+				CardTerminal.Name = "CardTerminal";
+				CardTerminal.Size = new Size (196, 22);
+				CardTerminal.Text = "Terminal";
+				Rent.MergeIndex = 13;
+				Rent.Name = "Rent";
+				Rent.Size = new Size (196, 22);
+				Rent.Text = "Rent";
+				WayBill.MergeIndex = 14;
+				WayBill.Name = "WayBill";
+				WayBill.Size = new Size (196, 22);
+				WayBill.Text = "WayBill";
+				CustomerRelationManagement.MergeIndex = 15;
+				CustomerRelationManagement.Name = "CustomerRelationManagement";
+				CustomerRelationManagement.Size = new Size (196, 22);
+				CustomerRelationManagement.Text = "CustomerRelationManagement";
+				toolStripSeparator8.MergeAction = MergeAction.Insert;
+				toolStripSeparator8.MergeIndex = 20;
+				toolStripSeparator8.Name = "toolStripSeparator8";
+				toolStripSeparator8.Size = new Size (193, 6);
+				NewUserToolStripMenuItem.MergeIndex = 21;
+				NewUserToolStripMenuItem.Name = "NewUserToolStripMenuItem";
+				NewUserToolStripMenuItem.ShortcutKeys = ((Keys)((Keys.Shift |
+			Keys.F2)));
+				NewUserToolStripMenuItem.Size = new Size (196, 22);
+				NewUserToolStripMenuItem.Text = "Uus kasutaja";
+				exitToolStripMenuItem.MergeIndex = 22;
+				exitToolStripMenuItem.Name = "exitToolStripMenuItem";
+				exitToolStripMenuItem.Size = new Size (196, 22);
+				exitToolStripMenuItem.Text = "Exit";
+
+				AutoScaleDimensions = new SizeF (6F, 13F);
+				AutoScaleMode = AutoScaleMode.Font;
+				BackgroundImageLayout = ImageLayout.Center;
+				ClientSize = new Size (644, 396);
+				Controls.Add (mainMenuStrip1);
+				IsMdiContainer = true;
+				KeyPreview = false;
+				MainMenuStrip = mainMenuStrip1;
+				Name = "MainForm";
+			}
+
+			MenuStrip mainMenuStrip1;
+			ToolStripMenuItem File;
+			ToolStripMenuItem openToolStripMenuItem;
+			ToolStripSeparator toolStripSeparator2;
+			ToolStripMenuItem exitToolStripMenuItem;
+			ToolStripMenuItem Dokument;
+			ToolStripMenuItem arveToolStripMenuItem;
+			ToolStripMenuItem WareHouse;
+			ToolStripMenuItem Personnel;
+			ToolStripMenuItem Payroll;
+			ToolStripMenuItem FixedAssets;
+			ToolStripMenuItem Supplies;
+			ToolStripMenuItem GeneralLedger;
+			ToolStripMenuItem Manufacturing;
+			ToolStripMenuItem PointOfSale;
+			ToolStripMenuItem CardTerminal;
+			ToolStripMenuItem Rent;
+			ToolStripMenuItem WayBill;
+			ToolStripMenuItem CustomerRelationManagement;
+			ToolStripSeparator toolStripSeparator8;
+			ToolStripMenuItem NewUserToolStripMenuItem;
+		}
+
+		private class Childform : Form
+		{
+			public Childform ()
+			{
+
+				menuStrip1 = new MenuStrip ();
+				fileToolStripMenuItem = new ToolStripMenuItem ();
+				toolStripSeparator = new ToolStripSeparator ();
+				printToolStripMenuItem = new ToolStripMenuItem ();
+				editToolStripMenuItem = new ToolStripMenuItem ();
+				toolStripSeparator3 = new ToolStripSeparator ();
+				copyToolStripMenuItem = new ToolStripMenuItem ();
+				toolStripSeparator4 = new ToolStripSeparator ();
+				selectAllToolStripMenuItem = new ToolStripMenuItem ();
+				toolsToolStripMenuItem = new ToolStripMenuItem ();
+				addToolStripMenuItem = new ToolStripMenuItem ();
+				deleteToolStripMenuItem = new ToolStripMenuItem ();
+				filterToolStripMenuItem = new ToolStripMenuItem ();
+				helpToolStripMenuItem = new ToolStripMenuItem ();
+				searchToolStripMenuItem = new ToolStripMenuItem ();
+				menuStrip1.SuspendLayout ();
+				SuspendLayout ();
+
+				menuStrip1.Items.AddRange (new ToolStripItem[] {
+            fileToolStripMenuItem,
+	    editToolStripMenuItem,
+	    toolsToolStripMenuItem,
+	    helpToolStripMenuItem});
+				menuStrip1.Location = new Point (0, 0);
+				menuStrip1.Name = "menuStrip1";
+				menuStrip1.RenderMode = ToolStripRenderMode.System;
+				menuStrip1.Size = new Size (337, 24);
+				menuStrip1.TabIndex = 2;
+				menuStrip1.Text = "menuStrip1";
+				menuStrip1.Visible = false;
+
+				fileToolStripMenuItem.DropDownItems.AddRange (new ToolStripItem[] {
+            toolStripSeparator,
+            printToolStripMenuItem});
+				fileToolStripMenuItem.MergeAction = MergeAction.MatchOnly;
+				fileToolStripMenuItem.Name = "fileToolStripMenuItem";
+				fileToolStripMenuItem.Size = new Size (35, 20);
+				fileToolStripMenuItem.Text = "&File";
+				toolStripSeparator.MergeAction = MergeAction.Insert;
+				toolStripSeparator.MergeIndex = 3;
+				toolStripSeparator.Name = "toolStripSeparator";
+				toolStripSeparator.Size = new Size (149, 6);
+				printToolStripMenuItem.MergeAction = MergeAction.Insert;
+				printToolStripMenuItem.MergeIndex = 2;
+				printToolStripMenuItem.Name = "printToolStripMenuItem";
+				printToolStripMenuItem.ShortcutKeys = ((Keys)((Keys.Control |
+			Keys.P)));
+				printToolStripMenuItem.Size = new Size (152, 22);
+				printToolStripMenuItem.Text = "&Print";
+				editToolStripMenuItem.DropDownItems.AddRange (new ToolStripItem[] {
+	    toolStripSeparator3,
+	    copyToolStripMenuItem,
+	    toolStripSeparator4,
+	    selectAllToolStripMenuItem});
+				editToolStripMenuItem.MergeAction = MergeAction.Insert;
+				editToolStripMenuItem.MergeIndex = 1;
+				editToolStripMenuItem.Name = "editToolStripMenuItem";
+				editToolStripMenuItem.Size = new Size (59, 20);
+				editToolStripMenuItem.Text = "&Paranda";
+				toolStripSeparator3.Name = "toolStripSeparator3";
+				toolStripSeparator3.Size = new Size (157, 6);
+				copyToolStripMenuItem.Name = "copyToolStripMenuItem";
+				copyToolStripMenuItem.ShortcutKeys = ((Keys)((Keys.Control | Keys.C)));
+				copyToolStripMenuItem.Size = new Size (160, 22);
+				copyToolStripMenuItem.Text = "Kopeeri";
+				toolStripSeparator4.Name = "toolStripSeparator4";
+				toolStripSeparator4.Size = new Size (157, 6);
+				selectAllToolStripMenuItem.Name = "selectAllToolStripMenuItem";
+				selectAllToolStripMenuItem.Size = new Size (160, 22);
+				selectAllToolStripMenuItem.Text = "selectAllToolStripMenuItem";
+				toolsToolStripMenuItem.DropDownItems.AddRange (new ToolStripItem[] {
+	    addToolStripMenuItem,
+	    deleteToolStripMenuItem,
+	    filterToolStripMenuItem});
+				toolsToolStripMenuItem.MergeAction = MergeAction.Insert;
+				toolsToolStripMenuItem.MergeIndex = 2;
+				toolsToolStripMenuItem.Name = "toolsToolStripMenuItem";
+				toolsToolStripMenuItem.Size = new Size (60, 20);
+				toolsToolStripMenuItem.Text = "&Tegevus";
+				addToolStripMenuItem.Name = "addToolStripMenuItem";
+				addToolStripMenuItem.ShortcutKeys = ((Keys)((Keys.Control | Keys.N)));
+				addToolStripMenuItem.Size = new Size (160, 22);
+				addToolStripMenuItem.Text = "Lisa";
+				deleteToolStripMenuItem.Name = "deleteToolStripMenuItem";
+				deleteToolStripMenuItem.ShortcutKeys = ((Keys)((Keys.Control |
+			Keys.T)));
+				deleteToolStripMenuItem.Size = new Size (160, 22);
+				deleteToolStripMenuItem.Text = "Kustuta";
+				filterToolStripMenuItem.ImageTransparentColor =
+			Color.Silver;
+				filterToolStripMenuItem.Name = "filterToolStripMenuItem";
+				filterToolStripMenuItem.ShortcutKeys = Keys.F11;
+				filterToolStripMenuItem.Size = new Size (160, 22);
+				filterToolStripMenuItem.Text = "Tingimus";
+				helpToolStripMenuItem.DropDownItems.AddRange (new ToolStripItem[] {
+	    searchToolStripMenuItem});
+				helpToolStripMenuItem.MergeAction = MergeAction.MatchOnly;
+				helpToolStripMenuItem.Name = "helpToolStripMenuItem";
+				helpToolStripMenuItem.Size = new Size (34, 20);
+				helpToolStripMenuItem.Text = "&Abi";
+				searchToolStripMenuItem.MergeAction = MergeAction.Insert;
+				searchToolStripMenuItem.MergeIndex = 1;
+				searchToolStripMenuItem.Name = "searchToolStripMenuItem";
+				searchToolStripMenuItem.ShortcutKeys = Keys.F1;
+				searchToolStripMenuItem.Size = new Size (160, 22);
+				searchToolStripMenuItem.Text = "&Otsi teemat";
+
+				AutoScaleDimensions = new SizeF (6F, 13F);
+				ClientSize = new Size (337, 272);
+				Controls.Add (menuStrip1);
+			}
+
+			MenuStrip menuStrip1;
+			ToolStripMenuItem fileToolStripMenuItem;
+			ToolStripSeparator toolStripSeparator;
+			ToolStripMenuItem printToolStripMenuItem;
+			ToolStripMenuItem editToolStripMenuItem;
+			ToolStripSeparator toolStripSeparator3;
+			ToolStripMenuItem copyToolStripMenuItem;
+			ToolStripSeparator toolStripSeparator4;
+			ToolStripMenuItem selectAllToolStripMenuItem;
+			ToolStripMenuItem toolsToolStripMenuItem;
+			ToolStripMenuItem addToolStripMenuItem;
+			ToolStripMenuItem deleteToolStripMenuItem;
+			ToolStripMenuItem filterToolStripMenuItem;
+			ToolStripMenuItem helpToolStripMenuItem;
+			ToolStripMenuItem searchToolStripMenuItem;
+		}
 	}
 }
-#endif
\ No newline at end of file
