@@ -1,12 +1,14 @@
 //
 // ToolTask.cs: Base class for command line tool tasks. 
 //
-// Author:
+// Authors:
 //   Marek Sieradzki (marek.sieradzki@gmail.com)
 //   Ankit Jain (jankit@novell.com)
+//   Marek Safar (marek.safar@gmail.com)
 //
 // (C) 2005 Marek Sieradzki
 // Copyright 2009 Novell, Inc (http://www.novell.com)
+// Copyright 2014 Xamarin Inc
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -38,15 +40,14 @@ using System.Text.RegularExpressions;
 using Microsoft.Build.Framework;
 using Mono.XBuild.Utilities;
 using System.Threading;
+using System.Collections.Generic;
 
 using SCS = System.Collections.Specialized;
 
 namespace Microsoft.Build.Utilities
 {
 	public abstract class ToolTask : Task
-#if NET_4_0
 		, ICancelableTask
-#endif	
 	{
 		int			exitCode;
 		int			timeout;
@@ -56,9 +57,7 @@ namespace Microsoft.Build.Utilities
 		MessageImportance	standardOutputLoggingImportance;
 		StringBuilder toolOutput;
 		bool typeLoadException;
-#if NET_4_0
 		ManualResetEvent canceled;
-#endif
 
 		protected ToolTask ()
 			: this (null, null)
@@ -79,9 +78,7 @@ namespace Microsoft.Build.Utilities
 			this.HelpKeywordPrefix = helpKeywordPrefix;
 			this.responseFileEncoding = Encoding.UTF8;
 			this.timeout = Int32.MaxValue;
-#if NET_4_0
 			canceled = new ManualResetEvent (false);
-#endif
 		}
 
 		[MonoTODO]
@@ -90,12 +87,45 @@ namespace Microsoft.Build.Utilities
 			return true;
 		}
 
+		string CreateToolPath ()
+		{
+			string tp;
+			if (string.IsNullOrEmpty (ToolPath)) {
+				tp = GenerateFullPathToTool ();
+				if (string.IsNullOrEmpty (tp))
+					return null;
+
+				//
+				// GenerateFullPathToTool can return path including tool name
+				//
+				if (string.IsNullOrEmpty (ToolExe))
+					return tp;
+
+				tp = Path.GetDirectoryName (tp);
+			} else {
+				tp = ToolPath;
+			}
+
+			var	path = Path.Combine (tp, ToolExe);
+			if (!File.Exists (path)) {
+				if (Log != null)
+					Log.LogError ("Tool executable '{0}' could not be found", path);
+				return null;
+			}
+
+			return path;
+		}
+
 		public override bool Execute ()
 		{
 			if (SkipTaskExecution ())
 				return true;
 
-			exitCode = ExecuteTool (GenerateFullPathToTool (), GenerateResponseFileCommands (),
+			var tool_path = CreateToolPath ();
+			if (tool_path == null)
+				return false;
+
+			exitCode = ExecuteTool (tool_path, GenerateResponseFileCommands (),
 				GenerateCommandLineCommands ());
 
 			// HandleTaskExecutionErrors is called only if exitCode != 0
@@ -289,14 +319,14 @@ namespace Microsoft.Build.Utilities
 
 		protected virtual string GenerateCommandLineCommands ()
 		{
-			return null;
+			return "";
 		}
 
 		protected abstract string GenerateFullPathToTool ();
 
 		protected virtual string GenerateResponseFileCommands ()
 		{
-			return null;
+			return "";
 		}
 
 		protected virtual string GetResponseFileSwitch (string responseFilePath)
@@ -371,29 +401,29 @@ namespace Microsoft.Build.Utilities
 		// EnvironmentOverride is Obsolete'd in 4.0
 		//
 		// Returns the final set of environment variables and logs them
-		SCS.StringDictionary GetAndLogEnvironmentVariables ()
+		Dictionary<string, string> GetAndLogEnvironmentVariables ()
 		{
 			var env_vars = GetEnvironmentVariables ();
 			if (env_vars == null)
 				return env_vars;
 
 			Log.LogMessage (MessageImportance.Low, "Environment variables being passed to the tool:");
-			foreach (DictionaryEntry entry in env_vars)
+			foreach (var entry in env_vars)
 				Log.LogMessage (MessageImportance.Low, "\t{0}={1}", (string)entry.Key, (string)entry.Value);
 
 			return env_vars;
 		}
 
-		SCS.StringDictionary GetEnvironmentVariables ()
+		Dictionary<string, string> GetEnvironmentVariables ()
 		{
-			if (EnvironmentVariables == null || EnvironmentVariables.Length == 0)
-				return EnvironmentOverride;
+			var env_vars = new Dictionary<string, string> (StringComparer.InvariantCultureIgnoreCase);
 
-			var env_vars = new SCS.ProcessStringDictionary ();
-			foreach (string pair in EnvironmentVariables) {
-				string [] key_value = pair.Split ('=');
-				if (!String.IsNullOrEmpty (key_value [0]))
-					env_vars [key_value [0]] = key_value.Length > 1 ? key_value [1] : String.Empty;
+			if (EnvironmentVariables != null) {
+				foreach (string pair in EnvironmentVariables) {
+					string [] key_value = pair.Split ('=');
+					if (!String.IsNullOrEmpty (key_value [0]))
+						env_vars [key_value [0]] = key_value.Length > 1 ? key_value [1] : String.Empty;
+				}
 			}
 
 			if (EnvironmentOverride != null)
@@ -471,7 +501,6 @@ namespace Microsoft.Build.Utilities
 			set { toolPath  = value; }
 		}
 
-#if NET_4_0
 		protected ManualResetEvent ToolCanceled {
 			get {
 				return canceled;
@@ -482,7 +511,6 @@ namespace Microsoft.Build.Utilities
 		{
 			canceled.Set ();
 		}
-#endif
 
 #if XBUILD_12
 		protected MessageImportance StandardErrorImportanceToUse {
@@ -498,6 +526,7 @@ namespace Microsoft.Build.Utilities
 		}
 
 		public bool LogStandardErrorAsError { get; set; }
+		public string StandardOutputImportance { get; set; }
 #endif
 	}
 }
