@@ -41,7 +41,9 @@ using System.ComponentModel;
 using System.Diagnostics;
 using System.Net.Sockets;
 using System.Globalization;
+using System.Security;
 using System.Text;
+using System.Runtime.InteropServices;
 
 namespace Mono.Data.Tds.Protocol
 {
@@ -64,6 +66,9 @@ namespace Mono.Data.Tds.Protocol
 		int databaseMajorVersion;
 		CultureInfo locale = CultureInfo.InvariantCulture;
 
+		readonly int lifeTime;
+		readonly DateTime created = DateTime.Now;
+
 		string charset;
 		string language;
 
@@ -147,7 +152,7 @@ namespace Mono.Data.Tds.Protocol
 			get { return dataSource; }
 		}
 
-		public bool IsConnected {
+		public virtual bool IsConnected {
 			get { return connected && comm != null && comm.IsConnected (); }
 			set { connected = value; }
 		}
@@ -235,13 +240,9 @@ namespace Mono.Data.Tds.Protocol
 				throw new Exception ("Cannot Skip to a colindex less than the curr index");
 
 			while (colIndex != StreamColumnIndex) {
-#if NET_2_0
 				TdsColumnType? colType = Columns[StreamColumnIndex].ColumnType;
 				if (colType == null)
 					throw new Exception ("Column type unset.");
-#else
-				TdsColumnType colType = (TdsColumnType) Columns [StreamColumnIndex]["ColumnType"];
-#endif
 				if (!(colType == TdsColumnType.Image ||
 					colType == TdsColumnType.Text ||
 					colType == TdsColumnType.NText)) {
@@ -268,11 +269,7 @@ namespace Mono.Data.Tds.Protocol
 			if (colIndex != StreamColumnIndex)
 				SkipToColumnIndex (colIndex);
 
-#if NET_2_0
 			object o = GetColumnValue (Columns[colIndex].ColumnType, false, colIndex);
-#else
-			object o = GetColumnValue ((TdsColumnType)Columns[colIndex]["ColumnType"], false, colIndex);
-#endif
 			StreamColumnIndex++;
 			return o;
 		}
@@ -286,11 +283,7 @@ namespace Mono.Data.Tds.Protocol
 					SkipToColumnIndex (colIndex);
 
 				if (!LoadInProgress) {
-#if NET_2_0
 					BeginLoad (Columns[colIndex].ColumnType);
-#else
-					BeginLoad ((TdsColumnType)Columns[colIndex]["ColumnType"]);
-#endif
 				}
 
 				if (buffer == null)
@@ -303,11 +296,7 @@ namespace Mono.Data.Tds.Protocol
 		}
 
 		private void BeginLoad (
-#if NET_2_0
 			TdsColumnType? colType
-#else
-			TdsColumnType colType
-#endif
 		) 
 		{
 			if (LoadInProgress)
@@ -315,10 +304,8 @@ namespace Mono.Data.Tds.Protocol
 
 			StreamLength = 0;
 
-#if NET_2_0
 			if (colType == null)
 				throw new ArgumentNullException ("colType");
-#endif
 
 			switch (colType) {
 			case TdsColumnType.Text :
@@ -416,12 +403,23 @@ namespace Mono.Data.Tds.Protocol
 		#region Constructors
 
 		public Tds (string dataSource, int port, int packetSize, int timeout, TdsVersion tdsVersion)
+			: this  (dataSource, port, packetSize, timeout, 0, tdsVersion)
+		{
+		}
+
+		public Tds (string dataSource, int port, int packetSize, int timeout, int lifeTime, TdsVersion tdsVersion)
 		{
 			this.tdsVersion = tdsVersion;
 			this.packetSize = packetSize;
 			this.dataSource = dataSource;
 			this.columns = new TdsDataColumnCollection ();
+			this.lifeTime = lifeTime;
 
+			InitComm (port, timeout);
+		}
+
+		protected virtual void InitComm (int port, int timeout)
+		{
 			comm = new TdsComm (dataSource, port, packetSize, timeout, tdsVersion);
 		}
 
@@ -429,6 +427,14 @@ namespace Mono.Data.Tds.Protocol
 
 		#region Public Methods
 
+		internal bool Expired {
+			get {
+				if (lifeTime == 0)
+					return false;
+				return DateTime.Now > (created + TimeSpan.FromSeconds (lifeTime));
+			}
+		}
+
 		internal protected void InitExec () 
 		{
 			// clean up
@@ -477,7 +483,7 @@ namespace Mono.Data.Tds.Protocol
 			return new TdsTimeoutException (0, 0, message, -2, method, dataSource, "Mono TdsClient Data Provider", 0);
 		}
 
-		public void Disconnect ()
+		public virtual void Disconnect ()
 		{
 			try {
 				comm.StartPacket (TdsPacketType.Logoff);
@@ -733,22 +739,14 @@ namespace Mono.Data.Tds.Protocol
 		}
 		
 		protected object GetColumnValue (
-#if NET_2_0
 			TdsColumnType? colType,
-#else
-			TdsColumnType colType,
-#endif
 			bool outParam)
 		{
 			return GetColumnValue (colType, outParam, -1);
 		}
 
 		private object GetColumnValue (
-#if NET_2_0
 			TdsColumnType? colType,
-#else
-			TdsColumnType colType,
-#endif
 			bool outParam, int ordinal)
 		{
 			int len;
@@ -756,18 +754,11 @@ namespace Mono.Data.Tds.Protocol
 			Encoding enc = null;
 			int lcid = 0, sortId = 0;
 
-#if NET_2_0
 			if (colType == null)
 				throw new ArgumentNullException ("colType");
-#endif
 			if (ordinal > -1 && tdsVersion > TdsVersion.tds70) {
-#if NET_2_0
 				lcid = (int) columns[ordinal].LCID;
 				sortId = (int) columns[ordinal].SortOrder; 
-#else
-				lcid = (int) columns[ordinal]["LCID"];
-				sortId = (int) columns[ordinal]["SortOrder"];
-#endif 			
 			}
 			
 			switch (colType) {
@@ -873,13 +864,8 @@ namespace Mono.Data.Tds.Protocol
 					scale = comm.GetByte ();
 				}
 				else {
-#if NET_2_0
 					precision = (byte) columns[ordinal].NumericPrecision;
 					scale = (byte) columns[ordinal].NumericScale;
-#else
-					precision = (byte) columns[ordinal]["NumericPrecision"];
-					scale = (byte) columns[ordinal]["NumericScale"];
-#endif
 				}
 
 				element = GetDecimalValue (precision, scale);
@@ -971,20 +957,14 @@ namespace Mono.Data.Tds.Protocol
 		}
 
 		private object GetDateTimeValue (
-#if NET_2_0
 			TdsColumnType? type
-#else
-			TdsColumnType type
-#endif
 		)
 		{
 			int len = 0;
 			object result;
 
-#if NET_2_0
 			if (type == null)
 				throw new ArgumentNullException ("type");
-#endif
 			switch (type) {
 			case TdsColumnType.DateTime4:
 				len = 4;
@@ -1096,17 +1076,11 @@ namespace Mono.Data.Tds.Protocol
 		}
 
 		private object GetFloatValue (
-#if NET_2_0
 			TdsColumnType? columnType
-#else
-			TdsColumnType columnType
-#endif
 		)
 		{
-#if NET_2_0
 			if (columnType == null)
 				throw new ArgumentNullException ("columnType");
-#endif
 			int columnSize = 0;
 
 			switch (columnType) {
@@ -1148,19 +1122,13 @@ namespace Mono.Data.Tds.Protocol
 		}
 
 		private object GetIntValue (
-#if NET_2_0
 			TdsColumnType? type
-#else
-			TdsColumnType type
-#endif
 		)
 		{
 			int len;
 
-#if NET_2_0
 			if (type == null)
 				throw new ArgumentNullException ("type");
-#endif
 			switch (type) {
 			case TdsColumnType.BigInt :
 				len = 8;
@@ -1196,19 +1164,13 @@ namespace Mono.Data.Tds.Protocol
 		}
 
 		private object GetMoneyValue (
-#if NET_2_0
 			TdsColumnType? type
-#else
-			TdsColumnType type
-#endif
 		)
 		{
 			int len;
 
-#if NET_2_0
 			if (type == null)
 				throw new ArgumentNullException ("type");
-#endif
 			switch (type) {
 			case TdsColumnType.SmallMoney :
 			case TdsColumnType.Money4 :
@@ -1232,7 +1194,7 @@ namespace Mono.Data.Tds.Protocol
 					val = ~(val - 1);
 				return new Decimal (val, 0, 0, negative, 4);
 			}
-			case 8:
+			case 8:	{
 				int hi = Comm.GetTdsInt ();
 				int lo = Comm.GetTdsInt ();
 				bool negative = hi < 0;
@@ -1242,17 +1204,14 @@ namespace Mono.Data.Tds.Protocol
 					lo = ~(lo - 1);
 				}
 				return new Decimal (lo, hi, 0, negative, 4);
+			}
 			default:
 				return DBNull.Value;
 			}
 		}
 
 		protected object GetStringValue (
-#if NET_2_0
 			TdsColumnType? colType,
-#else
-			TdsColumnType colType,
-#endif
 		    bool wideChars, bool outputParam, Encoding encoder)
 		{
 			bool shortLen = false;
@@ -1391,11 +1350,7 @@ namespace Mono.Data.Tds.Protocol
 
 			int i = 0;
 			foreach (TdsDataColumn column in columns) {
-#if NET_2_0
 				object o = GetColumnValue (column.ColumnType, false, i);
-#else
-				object o = GetColumnValue ((TdsColumnType)column["ColumnType"], false, i);
-#endif
 				currentRow.Add (o);
 				if (doneProc)
 					outputParameters.Add (o);
@@ -1440,13 +1395,12 @@ namespace Mono.Data.Tds.Protocol
 			// 0x0200	Negotiate NTLM
 			// 0x8000	Negotiate Always Sign
 
-			Type3Message t3 = new Type3Message ();
-			t3.Challenge = t2.Nonce;
+			Type3Message t3 = new Type3Message (t2);
 			
 			t3.Domain = this.connectionParms.DefaultDomain;
 			t3.Host = this.connectionParms.Hostname;
 			t3.Username = this.connectionParms.User;
-			t3.Password = this.connectionParms.Password;
+			t3.Password = GetPlainPassword(this.connectionParms.Password);
 
 			Comm.StartPacket (TdsPacketType.SspAuth); // 0x11
 			Comm.Append (t3.GetBytes ());
@@ -1490,21 +1444,12 @@ namespace Mono.Data.Tds.Protocol
 				bool isExpression = ((values[2] & (byte) TdsColumnStatus.IsExpression) != 0);
 
 				TdsDataColumn column = columns [index];
-#if NET_2_0
 				column.IsHidden = ((values[2] & (byte) TdsColumnStatus.Hidden) != 0);
 				column.IsExpression = isExpression;
 				column.IsKey = ((values[2] & (byte) TdsColumnStatus.IsKey) != 0);
 				column.IsAliased = isAlias;
 				column.BaseColumnName = ((isAlias) ? baseColumnName : null);
 				column.BaseTableName = ((!isExpression) ? (string) tableNames [tableIndex] : null);
-#else
-				column ["IsHidden"] = ((values [2] & (byte) TdsColumnStatus.Hidden) != 0);
-				column ["IsExpression"] = isExpression;
-				column ["IsKey"] = ((values [2] & (byte) TdsColumnStatus.IsKey) != 0);
-				column ["IsAliased"] = isAlias;
-				column ["BaseColumnName"] = ((isAlias) ? baseColumnName : null);
-				column ["BaseTableName"] = ((!isExpression) ? tableNames [tableIndex] : null);
-#endif
 			}
 		}
 
@@ -1647,17 +1592,18 @@ namespace Mono.Data.Tds.Protocol
 				comm.Skip (1);
 				srvVersion = (uint)comm.GetTdsInt ();
 
+				//Console.WriteLine ("srvVersion: {0}", srvVersion);
 				switch (srvVersion) {
-				case 0x07000000: 
+				case 0x00000007: 
 					tdsVersion = TdsVersion.tds70;
 					break;
-				case 0x07010000:
+				case 0x00000107:
 					tdsVersion = TdsVersion.tds80;
 					break;
-				case 0x71000001:
+				case 0x01000071:
 					tdsVersion = TdsVersion.tds81;
 					break;
-				case 0x72090002:
+				case 0x02000972:
 					tdsVersion = TdsVersion.tds90;
 					break;
 				}
@@ -1687,6 +1633,7 @@ namespace Mono.Data.Tds.Protocol
 			}
 
 			connected = true;
+			//Console.WriteLine ("databaseProductVersion:{0}", databaseProductVersion);
 		}
 
 		protected void OnTdsErrorMessage (TdsInternalErrorMessageEventArgs e)
@@ -1895,9 +1842,22 @@ namespace Mono.Data.Tds.Protocol
 			comm.Skip(4);
 		}
 
+		public static string GetPlainPassword(SecureString secPass)
+		{
+			IntPtr plainString = IntPtr.Zero;
+			try
+			{
+				plainString = Marshal.SecureStringToGlobalAllocUnicode(secPass);
+				return Marshal.PtrToStringUni(plainString);
+			}
+			finally
+			{
+				Marshal.ZeroFreeGlobalAllocUnicode(plainString);
+			}
+		}
+
 		#endregion // Private Methods
 
-#if NET_2_0
                 #region asynchronous methods
                 protected IAsyncResult BeginExecuteQueryInternal (string sql, bool wantResults, 
                                                           AsyncCallback callback, object state)
@@ -2008,7 +1968,6 @@ namespace Mono.Data.Tds.Protocol
                 }
 
                 #endregion // asynchronous methods
-#endif // NET_2_0
 
 
 	}
