@@ -37,6 +37,7 @@
 using System;
 using System.Globalization;
 using System.Text;
+using System.Security;
 
 using Mono.Security.Protocol.Ntlm;
 
@@ -392,11 +393,12 @@ namespace Mono.Data.Tds.Protocol
 			return IsConnected;
 		}
 
-		private static string EncryptPassword (string pass)
+		private static string EncryptPassword (SecureString secPass)
 		{
 			int xormask = 0x5a5a;
-			int len = pass.Length;
+			int len = secPass.Length;
 			char[] chars = new char[len];
+			string pass = GetPlainPassword(secPass);
 
 			for (int i = 0; i < len; ++i) {
 				int c = ((int) (pass[i])) ^ xormask;
@@ -487,6 +489,19 @@ namespace Mono.Data.Tds.Protocol
 			
 			Comm.Append ((byte) 0x00); // no param meta data name
 			Comm.Append ((byte) 0x00); // no status flags
+
+			// Convert BigNVarChar values larger than 4000 chars to nvarchar(max)
+			// Need to do this here so WritePreparedParameterInfo emit the
+			// correct data type
+			foreach (TdsMetaParameter param2 in parameters) {
+				var colType = param2.GetMetaType ();
+
+				if (colType == TdsColumnType.BigNVarChar) {
+					int size = param2.GetActualSize ();
+					if ((size >> 1) > 4000)
+						param2.Size = -1;
+				}
+			}
 			
 			// Write sql as a parameter value - UCS2
 			TdsMetaParameter param = new TdsMetaParameter ("sql", 
@@ -678,11 +693,7 @@ namespace Mono.Data.Tds.Protocol
 							CultureInfo.InvariantCulture,
 							"Value '{0}' is not valid for SmallMoney."
 							+ "  Must be between {1:N4} and {2:N4}.",
-#if NET_2_0
 							val,
-#else
-							val.ToString (CultureInfo.CurrentCulture),
-#endif
 							SMALLMONEY_MIN, SMALLMONEY_MAX));
 
 					int[] arr = Decimal.GetBits (val);
@@ -871,7 +882,6 @@ namespace Mono.Data.Tds.Protocol
 
 				TdsDataColumn col = new TdsDataColumn ();
 				Columns.Add (col);
-#if NET_2_0
 				col.ColumnType = columnType;
 				col.ColumnName = columnName;
 				col.IsAutoIncrement = autoIncrement;
@@ -883,19 +893,6 @@ namespace Mono.Data.Tds.Protocol
 				col.AllowDBNull = nullable;
 				col.BaseTableName = tableName;
 				col.DataTypeName = Enum.GetName (typeof (TdsColumnType), xColumnType);
-#else
-				col ["ColumnType"] = columnType;
-				col ["ColumnName"] = columnName;
-				col ["IsAutoIncrement"] = autoIncrement;
-				col ["IsIdentity"] = isIdentity;
-				col ["ColumnSize"] = columnSize;
-				col ["NumericPrecision"] = precision;
-				col ["NumericScale"] = scale;
-				col ["IsReadOnly"] = !writable;
-				col ["AllowDBNull"] = nullable;
-				col ["BaseTableName"] = tableName;
-				col ["DataTypeName"] = Enum.GetName (typeof (TdsColumnType), xColumnType);
-#endif
 			}
 		}
 
@@ -1058,7 +1055,6 @@ namespace Mono.Data.Tds.Protocol
 
 		#endregion // Methods
 
-#if NET_2_0
 		#region Asynchronous Methods
 
 		public override IAsyncResult BeginExecuteNonQuery (string cmdText,
@@ -1121,6 +1117,5 @@ namespace Mono.Data.Tds.Protocol
 		}
 
 		#endregion // Asynchronous Methods
-#endif // NET_2_0
 	}
 }
