@@ -3,8 +3,10 @@
 //
 // Author:
 //   Tim Coleman (tim@timcoleman.com)
+//   Gonzalo Paniagua Javier (gonzalo@novell.com)
 //
 // Copyright (C) 2002 Tim Coleman
+// Copyright (c) 2009 Novell, Inc.
 //
 
 //
@@ -29,6 +31,7 @@
 //
 
 using System;
+using System.IO;
 using System.Net;
 using System.Net.Sockets;
 using System.Text;
@@ -51,6 +54,7 @@ namespace Mono.Data.Tds.Protocol {
 		byte[] outBuffer;
 		int outBufferLength;
 		int nextOutBufferIndex = 0;
+		bool lsb;
 
 		byte[] inBuffer;
 		int inBufferLength;
@@ -61,7 +65,7 @@ namespace Mono.Data.Tds.Protocol {
 		byte[] tmpBuf = new byte[8];
 		byte[] resBuffer = new byte[256];
 
-		int packetsSent = 0;
+		int packetsSent;
 		int packetsReceived = 0;
 
 		Socket socket;
@@ -83,10 +87,12 @@ namespace Mono.Data.Tds.Protocol {
 			outBufferLength = packetSize;
 			inBufferLength = packetSize;
 
+			lsb = true;
+			
 			IPEndPoint endPoint;
+			bool have_exception = false;
 			
 			try {
-#if NET_2_0
 				IPAddress ip;
 				if(IPAddress.TryParse(this.dataSource, out ip)) {
 					endPoint = new IPEndPoint(ip, port);
@@ -94,10 +100,6 @@ namespace Mono.Data.Tds.Protocol {
 					IPHostEntry hostEntry = Dns.GetHostEntry (this.dataSource);
 					endPoint = new IPEndPoint(hostEntry.AddressList [0], port);
 				}
-#else
-				IPHostEntry hostEntry = Dns.Resolve (this.dataSource);
-				endPoint = new IPEndPoint (hostEntry.AddressList [0], port);
-#endif
 			} catch (SocketException e) {
 				throw new TdsInternalException ("Server does not exist or connection refused.", e);
 			}
@@ -105,7 +107,8 @@ namespace Mono.Data.Tds.Protocol {
 			try {
 				socket = new Socket (AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
 				IAsyncResult ares = socket.BeginConnect (endPoint, null, null);
-				if (timeout > 0 && !ares.IsCompleted && !ares.AsyncWaitHandle.WaitOne (timeout * 1000, false))
+				int timeout_ms = timeout * 1000;
+				if (timeout > 0 && !ares.IsCompleted && !ares.AsyncWaitHandle.WaitOne (timeout_ms, false))
 					throw Tds.CreateTimeoutException (dataSource, "Open()");
 				socket.EndConnect (ares);
 				try {
@@ -115,21 +118,34 @@ namespace Mono.Data.Tds.Protocol {
 					// Some platform may throw an exception, so
 					// eat all socket exception, yeaowww! 
 				}
-				
+
+				try {
+					socket.NoDelay = true;
+					socket.SetSocketOption (SocketOptionLevel.Socket, SocketOptionName.SendTimeout, timeout_ms);
+					socket.SetSocketOption (SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, timeout_ms);
+				} catch {
+					// Ignore exceptions here for systems that do not support these options.
+				}
 				// Let the stream own the socket and take the pleasure of closing it
 				stream = new NetworkStream (socket, true);
 			} catch (SocketException e) {
-				if (socket != null) {
+				have_exception = true;
+				throw new TdsInternalException ("Server does not exist or connection refused.", e);
+			} catch (Exception) {
+				have_exception = true;
+				throw;
+			} finally {
+				if (have_exception && socket != null) {
 					try {
 						Socket s = socket;
 						socket = null;
 						s.Close ();
 					} catch {}
 				}
-				throw new TdsInternalException ("Server does not exist or connection refused.", e);
 			}
 			if (!socket.Connected)
 				throw new TdsInternalException ("Server does not exist or connection refused.", null);
+			packetsSent = 1;
 		}
 		
 		#endregion // Constructors
@@ -151,6 +167,10 @@ namespace Mono.Data.Tds.Protocol {
 			set { packetSize = value; }
 		}
 		
+		public bool TdsByteOrder {
+			get { return !lsb; }
+			set { lsb = !value; }
+		}
 		#endregion // Properties
 		
 		#region Methods
@@ -171,6 +191,13 @@ namespace Mono.Data.Tds.Protocol {
 			}
 		}
 
+		public void SendIfFull (int reserve) 
+		{
+			if (nextOutBufferIndex+reserve > outBufferLength) {
+				SendPhysicalPacket (false);
+				nextOutBufferIndex = headerLength;
+			}
+		}
 
 		public void Append (object o)
 		{
@@ -178,6 +205,7 @@ namespace Mono.Data.Tds.Protocol {
 				Append ((byte)0);
 				return ;
 			}
+
 			switch (Type.GetTypeCode (o.GetType ())) {
 			case TypeCode.Byte :
 				Append ((byte) o);
@@ -231,19 +259,37 @@ namespace Mono.Data.Tds.Protocol {
 		{
 			DateTime epoch = new DateTime (1900,1,1);
 			
-			TimeSpan span = t - epoch;
-			int days = span.Days ;
+			TimeSpan span = t - epoch; //new TimeSpan (t.Ticks - epoch.Ticks);
+			int days, hours, minutes, secs;
+			long msecs;
 			int val = 0;	
 
+			days = span.Days;
+			hours = span.Hours;
+			minutes = span.Minutes;
+			secs = span.Seconds;
+			msecs = span.Milliseconds;
+			
+			if (epoch > t) {
+				// If t.Hour/Min/Sec/MSec is > 0, days points to the next day and hence, 
+				// we move it back by a day - otherwise, no change
+				days = (t.Hour > 0 || t.Minute > 0 || t.Second > 0 || t.Millisecond > 0) ? days-1: days;
+				hours = t.Hour;
+				minutes = t.Minute;
+				secs = t.Second;
+				msecs = t.Millisecond;
+			}
+
+			SendIfFull (bytes);
 			if (bytes == 8) {
-				long ms = (span.Hours * 3600 + span.Minutes * 60 + span.Seconds)*1000L + (long)span.Milliseconds;
+				long ms = (hours * 3600 + minutes * 60 + secs)*1000L + (long)msecs;
 				val = (int) ((ms*300)/1000);
-				Append ((int) days);
-				Append ((int) val);
+				AppendInternal ((int) days);
+				AppendInternal ((int) val);
 			} else if (bytes ==4) {
 				val = span.Hours * 60 + span.Minutes;
-				Append ((ushort) days);
-				Append ((short) val);
+				AppendInternal ((short) days);
+				AppendInternal ((short) val);
 			} else {
 				throw new Exception ("Invalid No of bytes");
 			}
@@ -293,42 +339,70 @@ namespace Mono.Data.Tds.Protocol {
 			}
 		}
 
+		private void AppendInternal (short s)
+		{
+			if (!lsb) {
+				outBuffer[nextOutBufferIndex++] = (byte) (((byte) (s >> 8)) & 0xff);
+				outBuffer[nextOutBufferIndex++] = (byte) ((byte) (s & 0xff));
+			} else {
+				outBuffer[nextOutBufferIndex++] = (byte) ((byte) (s & 0xff));
+				outBuffer[nextOutBufferIndex++] = (byte) (((byte) (s >> 8)) & 0xff);
+			}
+		}
+
 		public void Append (short s)
 		{
-			if(!BitConverter.IsLittleEndian)
-				Append (Swap (BitConverter.GetBytes(s)), sizeof(short), (byte)0);
-			else 
-				Append (BitConverter.GetBytes (s), sizeof(short), (byte)0);
+			SendIfFull (sizeof (short));
+			AppendInternal (s);
 		}
 
 		public void Append (ushort s)
 		{
-			if(!BitConverter.IsLittleEndian)
-				Append (Swap (BitConverter.GetBytes(s)), sizeof(short), (byte)0);
-			else 
-				Append (BitConverter.GetBytes (s), sizeof(short), (byte)0);
+			SendIfFull (sizeof (short));
+			AppendInternal ((short) s);
+		}
+
+		private void AppendInternal (int i)
+		{
+			if (!lsb) {
+				AppendInternal ((short) (((short) (i >> 16)) & 0xffff));
+				AppendInternal ((short) ((short) (i & 0xffff)));
+			} else {
+				AppendInternal ((short) ((short) (i & 0xffff)));
+				AppendInternal ((short) (((short) (i >> 16)) & 0xffff));
+			}				
 		}
 
 		public void Append (int i)
 		{
-			if(!BitConverter.IsLittleEndian)
-				Append (Swap (BitConverter.GetBytes(i)), sizeof(int), (byte)0);
-			else
-				Append (BitConverter.GetBytes (i), sizeof(int), (byte)0);
+			SendIfFull (sizeof (int));
+			AppendInternal (i);
 		}
 
 		public void Append (string s)
 		{
-			if (tdsVersion < TdsVersion.tds70) 
+			if (tdsVersion < TdsVersion.tds70) { 
 				Append (encoder.GetBytes (s));
-			else 
-				foreach (char c in s)
-					if(!BitConverter.IsLittleEndian)
-						Append (Swap (BitConverter.GetBytes (c)));
-					else
-						Append (BitConverter.GetBytes (c));
+			} else {
+				for (int i = 0; i < s.Length; i++) {
+					SendIfFull (sizeof(short));
+					AppendInternal ((short)s[i]);
+				}
+			}
 		}	
 
+		public void AppendNonUnicode (string s)
+		{
+			if (tdsVersion < TdsVersion.tds70) { 
+				Append (encoder.GetBytes (s));
+			} else {
+				for (int i = 0; i < s.Length; i++) {
+					SendIfFull (sizeof(byte));
+					Append ((byte)s[i]);
+				}
+			}
+		}
+
 		// Appends with padding
 		public byte[] Append (string s, int len, byte pad)
 		{
@@ -342,7 +416,7 @@ namespace Mono.Data.Tds.Protocol {
 
 		public void Append (double value)
 		{
-			if (!BitConverter.IsLittleEndian)
+			if (!lsb)
 				Append (Swap (BitConverter.GetBytes (value)), sizeof(double), (byte)0);
 			else
 				Append (BitConverter.GetBytes (value), sizeof(double), (byte)0);
@@ -350,7 +424,7 @@ namespace Mono.Data.Tds.Protocol {
 
 		public void Append (float value)
 		{
-			if (!BitConverter.IsLittleEndian)
+			if (!lsb)
 				Append (Swap (BitConverter.GetBytes (value)), sizeof(float), (byte)0);
 			else
 				Append (BitConverter.GetBytes (value), sizeof(float), (byte)0);
@@ -358,28 +432,78 @@ namespace Mono.Data.Tds.Protocol {
 
 		public void Append (long l)
 		{
-			if (!BitConverter.IsLittleEndian)
-				Append (Swap (BitConverter.GetBytes (l)), sizeof(long), (byte)0);
-			else
-				Append (BitConverter.GetBytes (l), sizeof(long), (byte)0);
+			SendIfFull (sizeof (long));
+			if (!lsb) {
+				AppendInternal ((int) (((int) (l >> 32)) & 0xffffffff));
+				AppendInternal ((int) ((int) (l & 0xffffffff)));
+			} else {
+				AppendInternal ((int) ((int) (l & 0xffffffff)));
+				AppendInternal ((int) (((int) (l >> 32)) & 0xffffffff));
+			}				
 		}
 
 		public void Append (decimal d, int bytes)
 		{
 			int[] arr = Decimal.GetBits (d);
-			byte sign =  (d > 0 ? (byte)1 : (byte)0);
-			Append (sign) ;
-			Append (arr[0]);
-			Append (arr[1]);
-			Append (arr[2]);
-			Append ((int)0);
+			byte sign = (d > 0 ? (byte)1 : (byte)0);
+			SendIfFull (bytes);
+			Append (sign);
+			AppendInternal (arr[0]);
+			AppendInternal (arr[1]);
+			AppendInternal (arr[2]);
+			AppendInternal ((int)0);
+		}
+
+		public void AppendMoney (decimal d, int size)
+		{
+			// The method for this is to simply multiply by 10^4 and then stuff
+			// the value into either a int or long value depending on the size
+
+			SendIfFull (size);
+
+			decimal tmpD = Decimal.Multiply(d, 10000m);
+			if (size > 4) {
+				long longValue = Decimal.ToInt64(tmpD);
+
+				int significantHalf = (int) ((longValue >> 32) & 0xffffffff);
+				int lessSignificantHalf = (int)(longValue & 0xffffffff);
+
+				AppendInternal (significantHalf);
+				AppendInternal (lessSignificantHalf);
+			} else {
+				int intValue = Decimal.ToInt32(tmpD);
+				AppendInternal (intValue);
+			}
+		} 
+
+		// A method for decimals that properly scales the decimal out before putting on the TDS steam
+		public void AppendDecimal (decimal d, int bytes, int scale)
+		{
+			decimal tmpD1 = Decimal.Multiply (d, (decimal)System.Math.Pow (10.0, scale));
+			decimal tmpD2 = System.Math.Abs(Decimal.Truncate (tmpD1));
+
+			int[] arr = Decimal.GetBits (tmpD2);
+			byte sign = (d > 0 ? (byte)1 : (byte)0);
+			SendIfFull (bytes);
+			Append (sign);
+			AppendInternal (arr[0]);
+			AppendInternal (arr[1]);
+			AppendInternal (arr[2]);
+			AppendInternal ((int)0);
 		}
 
 		public void Close ()
 		{
+			if (stream == null)
+				return;
+
 			connReset = false;
 			socket = null;
-			stream.Close ();
+			try {
+				stream.Close ();
+			} catch {
+			}
+			stream = null;
 		}
 
 		public bool IsConnected () 
@@ -423,7 +547,7 @@ namespace Mono.Data.Tds.Protocol {
 				int avail = inBufferLength - inBufferIndex;
 				avail = avail>len-i ? len-i : avail;
 
-				System.Array.Copy (inBuffer, inBufferIndex, result, i, avail);
+				Buffer.BlockCopy (inBuffer, inBufferIndex, result, i, avail);
 				i += avail;
 				inBufferIndex += avail;
 			}
@@ -431,15 +555,23 @@ namespace Mono.Data.Tds.Protocol {
 			return result;
 		}
 
+		public string GetString (int len, Encoding enc)
+		{
+			if (tdsVersion >= TdsVersion.tds70) 
+				return GetString (len, true, null);
+			else
+				return GetString (len, false, null);
+		}
+		
 		public string GetString (int len)
 		{
-			if (tdsVersion == TdsVersion.tds70) 
+			if (tdsVersion >= TdsVersion.tds70) 
 				return GetString (len, true);
 			else
 				return GetString (len, false);
 		}
 
-		public string GetString (int len, bool wide)
+		public string GetString (int len, bool wide, Encoding enc)
 		{
 			if (wide) {
 				char[] chars = new char[len];
@@ -453,9 +585,18 @@ namespace Mono.Data.Tds.Protocol {
 			else {
 				byte[] result = new byte[len];
 				Array.Copy (GetBytes (len, false), result, len);
-				return (encoder.GetString (result));
+				// Use the passed encoder, if available
+				if (enc != null)
+					return (enc.GetString (result));
+				else
+					return (encoder.GetString (result));
 			}
 		}
+		
+		public string GetString (int len, bool wide)
+		{
+			return GetString (len, wide, null);
+		}
 
 		public int GetNetShort ()
 		{
@@ -507,13 +648,32 @@ namespace Mono.Data.Tds.Protocol {
                         GetPhysicalPacketData (dataLength);
 		}
 
+		int Read (byte [] buffer, int offset, int count)
+		{
+			try {
+				return stream.Read (buffer, offset, count);
+			} catch {
+				socket = null;
+				stream.Close ();
+				throw;
+			}
+		}
+
                 private int GetPhysicalPacketHeader ()
                 {
                         int nread = 0;
 
+			int n;
 			// read the header
-			while (nread < 8)
-				nread += stream.Read (tmpBuf, nread, 8 - nread);
+			while (nread < 8) {
+				n = Read (tmpBuf, nread, 8 - nread);
+				if (n <= 0) {
+					socket = null;
+					stream.Close ();
+					throw new IOException (n == 0 ? "Connection lost" : "Connection error");
+				}
+				nread += n;
+			}
 
 			TdsPacketType packetType = (TdsPacketType) tmpBuf[0];
 			if (packetType != TdsPacketType.Logon && packetType != TdsPacketType.Query && packetType != TdsPacketType.Reply) 
@@ -538,9 +698,16 @@ namespace Mono.Data.Tds.Protocol {
                 {
                         // now get the data
 			int nread = 0;
+			int n;
 
 			while (nread < length) {
-				nread += stream.Read (inBuffer, nread, length - nread);
+				n = Read (inBuffer, nread, length - nread);
+				if (n <= 0) {
+					socket = null;
+					stream.Close ();
+					throw new IOException (n == 0 ? "Connection lost" : "Connection error");
+				}
+				nread += n;
 			}
 
 			packetsReceived++;
@@ -592,7 +759,7 @@ namespace Mono.Data.Tds.Protocol {
 		{
 			if (newSize != outBufferLength) {
 				byte[] newBuf = new byte [newSize];
-				Array.Copy (outBuffer, 0, newBuf, 0, newSize);
+				Buffer.BlockCopy (outBuffer, 0, newBuf, 0, newSize);
 				outBufferLength = newSize;
 				outBuffer = newBuf;
 			}
@@ -615,23 +782,34 @@ namespace Mono.Data.Tds.Protocol {
 			// Reset connection-reset flag to false - as any exception would anyway close 
 			// the whole connection
 			connReset = false;
+			packetsSent = 1;
 		}
 		
 		private void SendPhysicalPacket (bool isLastSegment)
 		{
 			if (nextOutBufferIndex > headerLength || packetType == TdsPacketType.Cancel) {
 				byte status =  (byte) ((isLastSegment ? 0x01 : 0x00) | (connReset ? 0x08 : 0x00)); 
+
 				// packet type
 				Store (0, (byte) packetType);
 				Store (1, status);
 				Store (2, (short) nextOutBufferIndex );
 				Store (4, (byte) 0);
 				Store (5, (byte) 0);
-				Store (6, (byte) (tdsVersion == TdsVersion.tds70 ? 0x1 : 0x0));
+				if (tdsVersion >= TdsVersion.tds70)
+					Store (6, (byte) packetsSent);
+				else	
+					Store (6, (byte) 0);
 				Store (7, (byte) 0);
 
 				stream.Write (outBuffer, 0, nextOutBufferIndex);
 				stream.Flush ();
+
+				if (!isLastSegment && packetType == TdsPacketType.Bulk)
+				{
+					System.Threading.Thread.Sleep (100);    
+				}
+
 				packetsSent++;
 			}
 		}
@@ -663,7 +841,6 @@ namespace Mono.Data.Tds.Protocol {
 		}
 
 		#endregion // Methods
-#if NET_2_0
                 #region Async Methods
 
                 public IAsyncResult BeginReadPacket (AsyncCallback callback, object stateObject)
@@ -687,9 +864,17 @@ namespace Mono.Data.Tds.Protocol {
                 {
                         TdsAsyncResult ar = (TdsAsyncResult) socketAsyncResult.AsyncState;
                         int nread = stream.EndRead (socketAsyncResult);
+			int n;
                         
-                        while (nread < 8)
-                                nread += stream.Read (tmpBuf, nread, 8 - nread);
+			while (nread < 8) {
+				n = Read (tmpBuf, nread, 8 - nread);
+				if (n <= 0) {
+					socket = null;
+					stream.Close ();
+					throw new IOException (n == 0 ? "Connection lost" : "Connection error");
+				}
+				nread += n;
+			}
 
 			TdsPacketType packetType = (TdsPacketType) tmpBuf[0];
 			if (packetType != TdsPacketType.Logon && packetType != TdsPacketType.Query && packetType != TdsPacketType.Reply) 
@@ -714,7 +899,6 @@ namespace Mono.Data.Tds.Protocol {
                 }
                 
                 #endregion // Async Methods
-#endif // NET_2_0
 
 	}
 
