@@ -33,15 +33,18 @@ using System.Threading;
 
 namespace Mono.Security.Protocol.Tls
 {
-	public abstract class SslStreamBase: Stream, IDisposable
+#if INSIDE_SYSTEM
+	internal
+#else
+	public
+#endif
+	abstract class SslStreamBase: Stream, IDisposable
 	{
 		private delegate void AsyncHandshakeDelegate(InternalAsyncResult asyncResult, bool fromWrite);
 		
 		#region Fields
 
-		static ManualResetEvent record_processing = new ManualResetEvent (true);
-
-		private const int WaitTimeOut = 5 * 60 * 1000;
+		static ManualResetEvent record_processing = new ManualResetEvent (true);	
 
 		internal Stream innerStream;
 		internal MemoryStream inputBuffer;
@@ -93,18 +96,11 @@ namespace Mono.Security.Protocol.Tls
 			{
 				try
 				{
-					this.OnNegotiateHandshakeCallback(asyncResult);
-				}
-				catch (TlsException ex)
-				{
-					this.protocol.SendAlert(ex.Alert);
-
-					throw new IOException("The authentication or decryption has failed.", ex);
+					this.EndNegotiateHandshake(asyncResult);
 				}
 				catch (Exception ex)
 				{
-					this.protocol.SendAlert(AlertDescription.InternalError);
-
+					this.protocol.SendAlert(ref ex);
 					throw new IOException("The authentication or decryption has failed.", ex);
 				}
 
@@ -176,8 +172,8 @@ namespace Mono.Security.Protocol.Tls
 
 		#region Abstracts/Virtuals
 
-		internal abstract IAsyncResult OnBeginNegotiateHandshake(AsyncCallback callback, object state);
-		internal abstract void OnNegotiateHandshakeCallback(IAsyncResult asyncResult);
+		internal abstract IAsyncResult BeginNegotiateHandshake (AsyncCallback callback, object state);
+		internal abstract void EndNegotiateHandshake (IAsyncResult result);
 
 		internal abstract X509Certificate OnLocalCertificateSelection(X509CertificateCollection clientCertificates,
 															X509Certificate serverCertificate,
@@ -185,10 +181,8 @@ namespace Mono.Security.Protocol.Tls
 															X509CertificateCollection serverRequestedCertificates);
 
 		internal abstract bool OnRemoteCertificateValidation(X509Certificate certificate, int[] errors);
-#if NET_2_0
 		internal abstract ValidationResult OnRemoteCertificateValidation2 (Mono.Security.X509.X509CertificateCollection collection);
 		internal abstract bool HaveRemoteValidation2Callback { get; }
-#endif
 
 		internal abstract AsymmetricAlgorithm OnLocalPrivateKeySelection(X509Certificate certificate, string targetHost);
 
@@ -209,12 +203,10 @@ namespace Mono.Security.Protocol.Tls
 			return OnRemoteCertificateValidation(certificate, errors);
 		}
 
-#if NET_2_0
 		internal ValidationResult RaiseRemoteCertificateValidation2 (Mono.Security.X509.X509CertificateCollection collection)
 		{
 			return OnRemoteCertificateValidation2 (collection);
 		}
-#endif
 
 		internal AsymmetricAlgorithm RaiseLocalPrivateKeySelection(
 			X509Certificate certificate,
@@ -493,7 +485,7 @@ namespace Mono.Security.Protocol.Tls
 				{
 					if (this.context.HandshakeState == HandshakeState.None)
 					{
-						this.OnBeginNegotiateHandshake(new AsyncCallback(AsyncHandshakeCallback), asyncResult);
+						this.BeginNegotiateHandshake(new AsyncCallback(AsyncHandshakeCallback), asyncResult);
 
 						return true;
 					}
@@ -503,17 +495,10 @@ namespace Mono.Security.Protocol.Tls
 					}
 				}
 			}
-			catch (TlsException ex)
-			{
-				this.negotiationComplete.Set();
-				this.protocol.SendAlert(ex.Alert);
-
-				throw new IOException("The authentication or decryption has failed.", ex);
-			}
 			catch (Exception ex)
 			{
 				this.negotiationComplete.Set();
-				this.protocol.SendAlert(AlertDescription.InternalError);
+				this.protocol.SendAlert(ref ex);
 
 				throw new IOException("The authentication or decryption has failed.", ex);
 			}
@@ -616,7 +601,7 @@ namespace Mono.Security.Protocol.Tls
 				{
 					asyncResult.SetComplete(preReadSize);
 				}
-				else if (!this.context.ConnectionEnd)
+				else if (!this.context.ReceivedConnectionEnd)
 				{
 					// this will read data from the network until we have (at least) one
 					// record to send back to the caller
@@ -629,15 +614,10 @@ namespace Mono.Security.Protocol.Tls
 					asyncResult.SetComplete(0);
 				}
 			}
-			catch (TlsException ex)
-			{
-				this.protocol.SendAlert(ex.Alert);
-
-				throw new IOException("The authentication or decryption has failed.", ex);
-			}
 			catch (Exception ex)
 			{
-				throw new IOException("IO exception during read.", ex);
+				this.protocol.SendAlert(ref ex);
+				throw new IOException("The authentication or decryption has failed.", ex);
 			}
 		}
 
@@ -735,11 +715,15 @@ namespace Mono.Security.Protocol.Tls
 
 				if (!dataToReturn && (n > 0))
 				{
-					// there is no record to return to caller and (possibly) more data waiting
-					// so continue reading from network (and appending to stream)
-					recordStream.Position = recordStream.Length;
-					this.innerStream.BeginRead(recbuf, 0, recbuf.Length,
-						new AsyncCallback(InternalReadCallback), state);
+					if (context.ReceivedConnectionEnd) {
+						internalResult.SetComplete (0);
+					} else {
+						// there is no record to return to caller and (possibly) more data waiting
+						// so continue reading from network (and appending to stream)
+						recordStream.Position = recordStream.Length;
+						this.innerStream.BeginRead(recbuf, 0, recbuf.Length,
+							new AsyncCallback(InternalReadCallback), state);
+					}
 				}
 				else
 				{
@@ -778,17 +762,13 @@ namespace Mono.Security.Protocol.Tls
 						record, 0, record.Length, new AsyncCallback(InternalWriteCallback), asyncResult);
 				}
 			}
-			catch (TlsException ex)
+			catch (Exception ex)
 			{
-				this.protocol.SendAlert(ex.Alert);
+				this.protocol.SendAlert (ref ex);
 				this.Close();
 
 				throw new IOException("The authentication or decryption has failed.", ex);
 			}
-			catch (Exception ex)
-			{
-				throw new IOException("IO exception during Write.", ex);
-			}
 		}
 
 		private void InternalWriteCallback(IAsyncResult ar)
@@ -875,7 +855,7 @@ namespace Mono.Security.Protocol.Tls
 			// Always wait until the read is complete
 			if (!asyncResult.IsCompleted)
 			{
-				if (!asyncResult.AsyncWaitHandle.WaitOne (WaitTimeOut, false))
+				if (!asyncResult.AsyncWaitHandle.WaitOne ())
 					throw new TlsException (AlertDescription.InternalError, "Couldn't complete EndRead");
 			}
 
@@ -900,7 +880,7 @@ namespace Mono.Security.Protocol.Tls
 
 			if (!asyncResult.IsCompleted)
 			{
-				if (!internalResult.AsyncWaitHandle.WaitOne (WaitTimeOut, false))
+				if (!internalResult.AsyncWaitHandle.WaitOne ())
 					throw new TlsException (AlertDescription.InternalError, "Couldn't complete EndWrite");
 			}
 
@@ -912,11 +892,7 @@ namespace Mono.Security.Protocol.Tls
 
 		public override void Close()
 		{
-#if NET_2_0
 			base.Close ();
-#else
-			((IDisposable)this).Dispose();
-#endif
 		}
 
 		public override void Flush()
@@ -1122,15 +1098,11 @@ namespace Mono.Security.Protocol.Tls
 					byte[] record = this.protocol.EncodeRecord (ContentType.ApplicationData, buffer, offset, count);
 					this.innerStream.Write (record, 0, record.Length);
 				}
-				catch (TlsException ex)
-				{
-					this.protocol.SendAlert(ex.Alert);
-					this.Close();
-					throw new IOException("The authentication or decryption has failed.", ex);
-				}
 				catch (Exception ex)
 				{
-					throw new IOException("IO exception during Write.", ex);
+					this.protocol.SendAlert(ref ex);
+					this.Close();
+					throw new IOException("The authentication or decryption has failed.", ex);
 				}
 			}
 		}
@@ -1175,17 +1147,7 @@ namespace Mono.Security.Protocol.Tls
 			this.Dispose(false);
 		}
 
-#if !NET_2_0
-		public void Dispose()
-		{
-			this.Dispose(true);
-			GC.SuppressFinalize(this);
-		}
-
-		protected virtual void Dispose (bool disposing)
-#else
 		protected override void Dispose (bool disposing)
-#endif
 		{
 			if (!this.disposed)
 			{
@@ -1194,10 +1156,13 @@ namespace Mono.Security.Protocol.Tls
 					if (this.innerStream != null)
 					{
 						if (this.context.HandshakeState == HandshakeState.Finished &&
-							!this.context.ConnectionEnd)
+							!this.context.SentConnectionEnd)
 						{
-							// Write close notify							
-							this.protocol.SendAlert(AlertDescription.CloseNotify);
+							// Write close notify
+							try {
+								this.protocol.SendAlert(AlertDescription.CloseNotify);
+							} catch {
+							}
 						}
 
 						if (this.ownsStream)
@@ -1211,9 +1176,7 @@ namespace Mono.Security.Protocol.Tls
 				}
 
 				this.disposed = true;
-#if NET_2_0
 				base.Dispose (disposing);
-#endif
 			}
 		}
 
