@@ -30,7 +30,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 using System.IO;
 
@@ -40,10 +39,9 @@ namespace Microsoft.SqlServer.Server {
 		#region Methods
 
 		void Read (BinaryReader r);
-		void Write (BinaryWriter r);
+		void Write (BinaryWriter w);
 
 		#endregion // Methods
 	}
 }
 
-#endif
