@@ -30,13 +30,12 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0 || TARGET_JVM
-
 using System.ComponentModel;
 using System.Data;
-#if NET_2_0 && !TARGET_JVM
 using System.Transactions;
-#endif
+
+using System.Threading;
+using System.Threading.Tasks;
 
 namespace System.Data.Common {
 	public abstract class DbConnection : Component, IDbConnection, IDisposable
@@ -50,7 +49,6 @@ namespace System.Data.Common {
 		#endregion // Constructors
 
 		#region Properties
-
 		[RecommendedAsConfigurable (true)]
 		[RefreshProperties (RefreshProperties.All)]
 		[DefaultValue ("")]
@@ -95,15 +93,11 @@ namespace System.Data.Common {
 
 		protected abstract DbCommand CreateDbCommand ();
 
-#if NET_2_0 && !TARGET_JVM
-		[MonoTODO]
 		public virtual void EnlistTransaction (Transaction transaction)
 		{
 			throw new NotSupportedException ();                        
 		}
-#endif
 
-#if NET_2_0
 		static class DataTypes
 		{
 			static readonly ColumnInfo [] columns = {
@@ -243,7 +237,7 @@ namespace System.Data.Common {
 			}
 		}
 
-		static class MetaDataCollections
+		internal static class MetaDataCollections
 		{
 			static readonly ColumnInfo [] columns = {
 				new ColumnInfo ("CollectionName", typeof (string)),
@@ -711,13 +705,17 @@ namespace System.Data.Common {
 			for (int i = 0; i < length; i++) {
 				command.Parameters [i].Value = restrictionValues [i];
 			}
-			DbProviderFactory dbProvider = DbProviderFactories.GetFactory (this.GetType (). ToString ());
-			DbDataAdapter dataAdapter = dbProvider.CreateDataAdapter ();
+			DbDataAdapter dataAdapter = DbProviderFactory.CreateDataAdapter ();
 			dataAdapter.SelectCommand = command;
 			dataAdapter.Fill (dataTable);
 			return dataTable;
 		}
-#endif
+
+		protected internal virtual DbProviderFactory DbProviderFactory {
+			get {
+				return null;
+			}
+		}
 
 		IDbTransaction IDbConnection.BeginTransaction ()
 		{
@@ -736,10 +734,29 @@ namespace System.Data.Common {
 		
 		public abstract void Open ();
 
-		protected virtual void OnStateChange (StateChangeEventArgs stateChanged)
+		protected virtual void OnStateChange (StateChangeEventArgs stateChange)
 		{
 			if (StateChange != null)
-				StateChange (this, stateChanged);
+				StateChange (this, stateChange);
+		}
+		
+		public Task OpenAsync ()
+		{
+			return OpenAsync (CancellationToken.None);
+		}
+		
+		public virtual Task OpenAsync (CancellationToken cancellationToken)
+		{
+			if (cancellationToken.IsCancellationRequested) {
+				return TaskHelper.CreateCanceledTask ();
+			}
+			
+			try {
+				Open ();
+				return TaskHelper.CreateVoidTask ();
+			} catch (Exception e) {
+				return TaskHelper.CreateExceptionTask (e);
+			}
 		}
 
 		#endregion // Methods
@@ -748,5 +765,3 @@ namespace System.Data.Common {
 
 	}
 }
-
-#endif
