@@ -30,7 +30,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 namespace System.Data.Common {
 	public enum IdentifierCase 
@@ -41,4 +40,3 @@ namespace System.Data.Common {
 	}
 }
 
-#endif
