@@ -42,17 +42,11 @@ namespace System.Data.Odbc
 	/// Provides a means of automatically generating single-table commands used to reconcile changes made to a DataSet with the associated database. This class cannot be inherited.
 	/// </summary>
 
-#if NET_2_0
 	public sealed class OdbcCommandBuilder : DbCommandBuilder
-#else // 1_1
-	public sealed class OdbcCommandBuilder : Component
-#endif // NET_2_0
 	{
 		#region Fields
 
 		private OdbcDataAdapter _adapter;
-		private string 			_quotePrefix;
-		private string 			_quoteSuffix;
 
 		private DataTable		_schema;
 		private string			_tableName;
@@ -70,8 +64,6 @@ namespace System.Data.Odbc
 		
 		public OdbcCommandBuilder ()
 		{
-			_quotePrefix = string.Empty;
-			_quoteSuffix = string.Empty;
 		}
 
 		public OdbcCommandBuilder (OdbcDataAdapter adapter)
@@ -87,9 +79,7 @@ namespace System.Data.Odbc
 		[OdbcDescriptionAttribute ("The DataAdapter for which to automatically generate OdbcCommands")]
 		[DefaultValue (null)]
 		public
-#if NET_2_0
 		new
-#endif // NET_2_0
 		OdbcDataAdapter DataAdapter {
 			get {
 				return _adapter;
@@ -145,39 +135,6 @@ namespace System.Data.Odbc
 			}
 		}
 
-		[BrowsableAttribute (false)]
-		[OdbcDescriptionAttribute ("The prefix string wrapped around sql objects")]
-		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
-#if ONLY_1_1
-		public
-#else
-		new
-#endif
-		string QuotePrefix {
-			get {
-				return _quotePrefix;
-			}
-			set {
-				_quotePrefix = value;
-			}
-		}
-
-		[BrowsableAttribute (false)]
-		[OdbcDescriptionAttribute ("The suffix string wrapped around sql objects")]
-		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
-#if ONLY_1_1
-		public
-#else
-		new
-#endif // NET_2_0
-		string QuoteSuffix {
-			get {
-				return _quoteSuffix;
-			}
-			set {
-				_quoteSuffix = value;
-			}
-		}
 
 		#endregion // Properties
 
@@ -189,11 +146,7 @@ namespace System.Data.Odbc
 			throw new NotImplementedException ();
 		}
 
-#if ONLY_1_1
-		protected override
-#else
 		new
-#endif
 		void Dispose (bool disposing)
 		{
 			if (_disposed)
@@ -208,7 +161,7 @@ namespace System.Data.Odbc
 				if (_deleteCommand != null)
 					_deleteCommand.Dispose ();
 				if (_schema != null)
-					_insertCommand.Dispose ();
+					_schema.Dispose ();
 
 				_insertCommand = null;
 				_updateCommand = null;
@@ -221,10 +174,9 @@ namespace System.Data.Odbc
 		private bool IsUpdatable (DataRow schemaRow)
 		{
 			if ( (! schemaRow.IsNull ("IsAutoIncrement") && (bool) schemaRow ["IsAutoIncrement"])
-			     || (! schemaRow.IsNull ("IsHidden") && (bool) schemaRow ["IsHidden"])
-			     || (! schemaRow.IsNull ("IsExpression") && (bool) schemaRow ["IsExpression"])
 			     || (! schemaRow.IsNull ("IsRowVersion") && (bool) schemaRow ["IsRowVersion"])
 			     || (! schemaRow.IsNull ("IsReadOnly") && (bool) schemaRow ["IsReadOnly"])
+			     || (schemaRow.IsNull ("BaseTableName") || ((string) schemaRow ["BaseTableName"]).Length == 0)
 			     )
 				return false;
 			return true;
@@ -253,23 +205,18 @@ namespace System.Data.Odbc
 		/*
 		 * creates where clause for optimistic concurrency
 		 */
-		private string CreateOptWhereClause (OdbcCommand command, bool option)
+		private string CreateOptWhereClause (OdbcCommand command, int paramCount)
 		{
 			string [] whereClause = new string [Schema.Rows.Count];
 
-			int count = 0;
+			int partCount = 0;
 
 			foreach (DataRow schemaRow in Schema.Rows) {
 				// exclude non updatable columns
 				if (! IsUpdatable (schemaRow))
 					continue;
 
-				string columnName = null;
-				if (option)
-					columnName = GetColumnName (schemaRow);
-				else
-					columnName = String.Format ("@p{0}", count);
-				
+				string columnName = GetColumnName (schemaRow);
 				if (columnName == String.Empty)
 					throw new InvalidOperationException ("Cannot form delete command. Column name is missing!");
 
@@ -278,19 +225,29 @@ namespace System.Data.Odbc
 				int 	length 	   = schemaRow.IsNull ("ColumnSize") ? -1 : (int) schemaRow ["ColumnSize"];
 
 				if (allowNull) {
-					whereClause [count] = String.Format ("((? = 1 AND {0} IS NULL) OR ({0} = ?))",
-									      columnName);
-					AddParameter (command, columnName, sqlDbType, length, columnName, DataRowVersion.Original);
-					AddParameter (command, columnName, sqlDbType, length, columnName, DataRowVersion.Original);
+					whereClause [partCount++] = String.Format ("((? = 1 AND {0} IS NULL) OR ({0} = ?))",
+						GetQuotedString (columnName));
+					OdbcParameter nullParam = AddParameter (
+						command,
+						GetParameterName (++paramCount),
+						OdbcType.Int,
+						length,
+						columnName,
+						DataRowVersion.Original);
+					nullParam.Value = 1;
+					AddParameter (command, GetParameterName (++paramCount),
+						sqlDbType, length, columnName,
+						DataRowVersion.Original);
 				} else {
-					whereClause [count] = String.Format ( "({0} = ?)", columnName);
-					AddParameter (command, columnName, sqlDbType, length, columnName, DataRowVersion.Original);
+					whereClause [partCount++] = String.Format ("({0} = ?)",
+						GetQuotedString (columnName));
+					AddParameter (command, GetParameterName (++paramCount),
+						sqlDbType, length, columnName,
+						DataRowVersion.Original);
 				}
-
-				count++;
 			}
 
-			return String.Join (" AND ", whereClause, 0, count);
+			return String.Join (" AND ", whereClause, 0, partCount);
 		}
 
 		private void CreateNewCommand (ref OdbcCommand command)
@@ -310,8 +267,8 @@ namespace System.Data.Odbc
 		private OdbcCommand CreateInsertCommand (bool option)
 		{
 			CreateNewCommand (ref _insertCommand);
-			
-			string query = String.Format ("INSERT INTO {0}", QuoteIdentifier (TableName));
+
+			string query = String.Format ("INSERT INTO {0}", GetQuotedString (TableName));
 			string [] columns = new string [Schema.Rows.Count];
 			string [] values  = new string [Schema.Rows.Count];
 
@@ -322,39 +279,33 @@ namespace System.Data.Odbc
 				if (! IsUpdatable (schemaRow))
 					continue;
 
-				string columnName = null;
-				
-				if (option)
-					columnName = GetColumnName (schemaRow);
-				else
-					columnName = String.Format ("@p{0}", count); 
-				
+				string columnName = GetColumnName (schemaRow);
 				if (columnName == String.Empty)
 					throw new InvalidOperationException ("Cannot form insert command. Column name is missing!");
 
 				// create column string & value string
-				columns [count] = QuoteIdentifier(columnName);
+				columns [count] = GetQuotedString (columnName);
 				values [count++] = "?";
 
 				// create parameter and add
 				OdbcType sqlDbType = schemaRow.IsNull ("ProviderType") ? OdbcType.VarChar : (OdbcType) schemaRow ["ProviderType"];
 				int length = schemaRow.IsNull ("ColumnSize") ? -1 : (int) schemaRow ["ColumnSize"];
 
-				AddParameter (_insertCommand, columnName, sqlDbType, length, columnName, DataRowVersion.Current);
+				AddParameter (_insertCommand, GetParameterName (count),
+					sqlDbType, length, columnName, DataRowVersion.Current);
 			}
 
-			query = String.Format ("{0} ({1}) VALUES ({2})", 
-					       query, 
-					       String.Join (", ", columns, 0, count),
-					       String.Join (", ", values, 0, count) );
+			query = String.Format (
+				"{0} ({1}) VALUES ({2})", 
+				query, 
+				String.Join (", ", columns, 0, count),
+				String.Join (", ", values, 0, count));
 			_insertCommand.CommandText = query;
 			return _insertCommand;
 		}
 
 		public
-#if NET_2_0
 		new
-#endif // NET_2_0
 		OdbcCommand GetInsertCommand ()
 		{
 			// FIXME: check validity of adapter
@@ -367,7 +318,6 @@ namespace System.Data.Odbc
 			return CreateInsertCommand (false);
 		}
 
-#if NET_2_0
 		public new OdbcCommand GetInsertCommand (bool useColumnsForParameterNames)
 		{
 			// FIXME: check validity of adapter
@@ -379,13 +329,12 @@ namespace System.Data.Odbc
 
 			return CreateInsertCommand (useColumnsForParameterNames);
 		}
-#endif // NET_2_0
 
 		private OdbcCommand CreateUpdateCommand (bool option)
 		{
 			CreateNewCommand (ref _updateCommand);
 
-			string query = String.Format ("UPDATE {0} SET", QuoteIdentifier (TableName));
+			string query = String.Format ("UPDATE {0} SET", GetQuotedString (TableName));
 			string [] setClause = new string [Schema.Rows.Count];
 
 			int count = 0;
@@ -395,12 +344,7 @@ namespace System.Data.Odbc
 				if (! IsUpdatable (schemaRow))
 					continue;
 
-				string columnName = null; 
-				if (option)
-					columnName = GetColumnName (schemaRow);
-				else
-					columnName = String.Format ("@p{0}", count);
-				
+				string columnName = GetColumnName (schemaRow);
 				if (columnName == String.Empty)
 					throw new InvalidOperationException ("Cannot form update command. Column name is missing!");
 
@@ -408,27 +352,26 @@ namespace System.Data.Odbc
 				int length = schemaRow.IsNull ("ColumnSize") ? -1 : (int) schemaRow ["ColumnSize"];
 
 				// create column = value string
-				setClause [count] = String.Format ("{0} = ?", QuoteIdentifier(columnName));
-				AddParameter (_updateCommand, columnName, sqlDbType, length, columnName, DataRowVersion.Current);
-				count++;
+				setClause [count++] = String.Format ("{0} = ?", GetQuotedString (columnName));
+				AddParameter (_updateCommand, GetParameterName (count),
+					sqlDbType, length, columnName, DataRowVersion.Current);
 			}
 
 			// create where clause. odbc uses positional parameters. so where class
 			// is created seperate from the above loop.
-			string whereClause = CreateOptWhereClause (_updateCommand, option);
+			string whereClause = CreateOptWhereClause (_updateCommand, count);
 			
-			query = String.Format ("{0} {1} WHERE ({2})", 
-					       query, 
-					       String.Join (", ", setClause, 0, count),
-					       whereClause);
+			query = String.Format (
+				"{0} {1} WHERE ({2})",
+				query,
+				String.Join (", ", setClause, 0, count),
+				whereClause);
 			_updateCommand.CommandText = query;
 			return _updateCommand;
 		}
 		
 		public
-#if NET_2_0
 		new
-#endif // NET_2_0
 		OdbcCommand GetUpdateCommand ()
 		{
 			// FIXME: check validity of adapter
@@ -441,7 +384,6 @@ namespace System.Data.Odbc
 			return CreateUpdateCommand (false);
 		}
 
-#if NET_2_0
 		public new OdbcCommand GetUpdateCommand (bool useColumnsForParameterNames)
 		{
 			// FIXME: check validity of adapter
@@ -453,24 +395,26 @@ namespace System.Data.Odbc
 
 			return CreateUpdateCommand (useColumnsForParameterNames);
 		}
-#endif // NET_2_0
 
 		private OdbcCommand CreateDeleteCommand (bool option)
 		{
 			CreateNewCommand (ref _deleteCommand);
 
-			string query = String.Format ("DELETE FROM {0}", QuoteIdentifier (TableName));
-			string whereClause = CreateOptWhereClause (_deleteCommand, option);
+			string query = String.Format (
+				"DELETE FROM {0}",
+				GetQuotedString (TableName));
+			string whereClause = CreateOptWhereClause (_deleteCommand, 0);
 			
-			query = String.Format ("{0} WHERE ({1})", query, whereClause);
+			query = String.Format (
+				"{0} WHERE ({1})",
+				query,
+				whereClause);
 			_deleteCommand.CommandText = query;
 			return _deleteCommand;
 		}
 
 		public
-#if NET_2_0
 		new
-#endif // NET_2_0
 		OdbcCommand GetDeleteCommand ()
 		{
 			// FIXME: check validity of adapter
@@ -483,7 +427,6 @@ namespace System.Data.Odbc
 			return CreateDeleteCommand (false);
 		}
 
-#if NET_2_0
 		public new OdbcCommand GetDeleteCommand (bool useColumnsForParameterNames)
 		{
 			// FIXME: check validity of adapter
@@ -495,13 +438,8 @@ namespace System.Data.Odbc
 
 			return CreateDeleteCommand (useColumnsForParameterNames);
 		}
-#endif // NET_2_0
 
-#if ONLY_1_1
-		public
-#else
 		new
-#endif // NET_2_0
 		void RefreshSchema ()
 		{
 			// creates metadata
@@ -527,24 +465,25 @@ namespace System.Data.Odbc
 			_tableName	= String.Empty;
 		}
 
-#if NET_2_0
+		protected override
+		string GetParameterName (int parameterOrdinal)
+		{
+			return String.Format ("p{0}", parameterOrdinal);
+		}
+
+
 		protected override void ApplyParameterInfo (DbParameter parameter,
-		                                            DataRow row,
+		                                            DataRow datarow,
 		                                            StatementType statementType,
 		                                            bool whereClause)
 		{
 			OdbcParameter odbcParam = (OdbcParameter) parameter;
-			odbcParam.Size = int.Parse (row ["ColumnSize"].ToString ());
-			if (row ["NumericPrecision"] != DBNull.Value)
-				odbcParam.Precision = byte.Parse (row ["NumericPrecision"].ToString ());
-			if (row ["NumericScale"] != DBNull.Value)
-				odbcParam.Scale = byte.Parse (row ["NumericScale"].ToString ());
-			odbcParam.DbType = (DbType) row ["ProviderType"];
-		}
-
-		protected override string GetParameterName (int parameterOrdinal)
-		{
-			return String.Format("@p{0}", parameterOrdinal);
+			odbcParam.Size = int.Parse (datarow ["ColumnSize"].ToString ());
+			if (datarow ["NumericPrecision"] != DBNull.Value)
+				odbcParam.Precision = byte.Parse (datarow ["NumericPrecision"].ToString ());
+			if (datarow ["NumericScale"] != DBNull.Value)
+				odbcParam.Scale = byte.Parse (datarow ["NumericScale"].ToString ());
+			odbcParam.DbType = (DbType) datarow ["ProviderType"];
 		}
 
 		protected override string GetParameterName (string parameterName)
@@ -570,40 +509,46 @@ namespace System.Data.Odbc
 
 			((OdbcDataAdapter) adapter).RowUpdating += rowUpdatingHandler;
 		}
-#endif // NET_2_0
 
-#if NET_2_0
-		public override
-#else
-		private
-#endif
-		string QuoteIdentifier (string unquotedIdentifier)
+		public override string QuoteIdentifier (string unquotedIdentifier)
 		{
-			if (unquotedIdentifier == null || unquotedIdentifier.Length == 0)
-				return unquotedIdentifier;
-			return String.Format ("{0}{1}{2}", QuotePrefix, 
-				unquotedIdentifier, QuoteSuffix);
+			return QuoteIdentifier (unquotedIdentifier, null);
 		}
 
-#if NET_2_0
-		// FIXME:  Not sure what the extra "connection" param does!
 		public string QuoteIdentifier (string unquotedIdentifier, OdbcConnection connection)
 		{
-			return QuoteIdentifier (unquotedIdentifier);
+			if (unquotedIdentifier == null)
+				throw new ArgumentNullException ("unquotedIdentifier");
+
+			string prefix = QuotePrefix;
+			string suffix = QuoteSuffix;
+
+			if (QuotePrefix.Length == 0) {
+				if (connection == null)
+					throw new InvalidOperationException (
+						"An open connection is required if "
+						+ "QuotePrefix is not set.");
+				prefix = suffix = GetQuoteCharacter (connection);
+			}
+
+			if (prefix.Length > 0 && prefix != " ") {
+				string escaped;
+				if (suffix.Length > 0)
+					escaped = unquotedIdentifier.Replace (
+						suffix, suffix + suffix);
+				else
+					escaped = unquotedIdentifier;
+				return string.Concat (prefix, escaped, suffix);
+			}
+			return unquotedIdentifier;
 		}
 
 		public string UnquoteIdentifier (string quotedIdentifier, OdbcConnection connection)
 		{
 			return UnquoteIdentifier (quotedIdentifier);
 		}
-#endif
 
-#if NET_2_0
-		public override
-#else
-		private
-#endif
-		string UnquoteIdentifier (string quotedIdentifier)
+		public override string UnquoteIdentifier (string quotedIdentifier)
 		{
 			if (quotedIdentifier == null || quotedIdentifier.Length == 0)
 				return quotedIdentifier;
@@ -639,6 +584,29 @@ namespace System.Data.Odbc
 			}
 		}
 
+		string GetQuotedString (string unquotedIdentifier)
+		{
+			string prefix = QuotePrefix;
+			string suffix = QuoteSuffix;
+
+			if (prefix.Length == 0 && suffix.Length == 0)
+				return unquotedIdentifier;
+
+			return String.Format ("{0}{1}{2}", prefix,
+				unquotedIdentifier, suffix);
+		}
+
+		bool IsCommandGenerated {
+			get {
+				return (_insertCommand != null || _updateCommand != null || _deleteCommand != null);
+			}
+		}
+
+		string GetQuoteCharacter (OdbcConnection conn)
+		{
+			return conn.GetInfo (OdbcInfo.IdentifierQuoteChar);
+		}
+
 		#endregion // Methods
 	}
 }
