@@ -42,11 +42,7 @@ using System.Text;
 
 namespace System.Data.Odbc
 {
-#if NET_2_0
 	public sealed class OdbcDataReader : DbDataReader
-#else
-	public sealed class OdbcDataReader : MarshalByRefObject, IDataReader, IDisposable, IDataRecord, IEnumerable
-#endif
 	{
 		#region Fields
 		
@@ -95,9 +91,7 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int Depth {
 			get {
 				return 0; // no nested selects supported
@@ -105,19 +99,17 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int FieldCount {
 			get {
+				if (IsClosed)
+					throw new InvalidOperationException ("The reader is closed.");
 				return cols.Length;
 			}
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool IsClosed {
 			get {
 				return !open;
@@ -125,39 +117,24 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		object this [string value] {
 			get {
-				int pos;
-
-				if (currentRow == -1)
-					throw new InvalidOperationException ();
-
-				pos = ColIndex (value);
-				
-				if (pos == -1)
-					throw new IndexOutOfRangeException ();
-
+				int pos = GetOrdinal (value);
 				return this [pos];
 			}
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		object this [int i] {
 			get {
-				return (object) GetValue (i);
+				return GetValue (i);
 			}
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int RecordsAffected {
 			get {
 				return _recordsAffected;
@@ -166,9 +143,7 @@ namespace System.Data.Odbc
 
 		[MonoTODO]
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool HasRows {
 			get { throw new NotImplementedException(); }
 		}
@@ -234,9 +209,7 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		void Close ()
 		{
 			// FIXME : have to implement output parameter binding
@@ -249,37 +222,30 @@ namespace System.Data.Odbc
 				this.command.Connection.Close ();
 		}
 
-#if ONLY_1_1
-		~OdbcDataReader ()
-		{
-			this.Dispose (false);
-		}
-#endif
 
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool GetBoolean (int i)
 		{
 			return (bool) GetValue (i);
 		}
 
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		byte GetByte (int i)
 		{
 			return Convert.ToByte (GetValue (i));
 		}
 
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		long GetBytes (int i, long dataIndex, byte[] buffer, int bufferIndex, int length)
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("Reader is not open.");
+			if (currentRow == -1)
+				throw new InvalidOperationException ("No data available.");
+
 			OdbcReturn ret = OdbcReturn.Error;
 			bool copyBuffer = false;
 			int returnVal = 0, outsize = 0;
@@ -323,21 +289,26 @@ namespace System.Data.Odbc
 			}
 
 			if (copyBuffer) {
-				int j = 0;
-				while (tbuff [j] != libodbc.C_NULL) {
-					buffer [bufferIndex + i] = tbuff [j];
-					j++;
+				if (outsize == (int) OdbcLengthIndicator.NoTotal) {
+					int j = 0;
+					while (tbuff [j] != libodbc.C_NULL) {
+						buffer [bufferIndex + j] = tbuff [j];
+						j++;
+					}
+					returnVal = j;
+				} else {
+					int read_bytes = Math.Min (outsize, length);
+					for (int j = 0; j < read_bytes; j++)
+						buffer [bufferIndex + j] = tbuff [j];
+					returnVal = read_bytes;
 				}
-				returnVal = j;
 			}
 			return returnVal;
 		}
 		
 		[MonoTODO]
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		char GetChar (int i)
 		{
 			throw new NotImplementedException ();
@@ -345,33 +316,34 @@ namespace System.Data.Odbc
 
 		[MonoTODO]
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		long GetChars (int i, long dataIndex, char[] buffer, int bufferIndex, int length)
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
+			if (currentRow == -1)
+				throw new InvalidOperationException ("No data available.");
+			if (i < 0 || i >= FieldCount)
+				throw new IndexOutOfRangeException ();
 			throw new NotImplementedException ();
 		}
 
 		[MonoTODO]
 		[EditorBrowsableAttribute (EditorBrowsableState.Never)]
-#if ONLY_1_1
-		public
-#endif
-#if NET_2_0
 		new
-#endif
 		IDataReader GetData (int i)
 		{
 			throw new NotImplementedException ();
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		string GetDataTypeName (int i)
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
+			if (i < 0 || i >= FieldCount)
+				throw new IndexOutOfRangeException ();
 			return GetColumnAttributeStr (i + 1, FieldIdentifier.TypeName);
 		}
 
@@ -381,45 +353,37 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		DateTime GetDateTime (int i)
 		{
 			return (DateTime) GetValue (i);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		decimal GetDecimal (int i)
 		{
 			return (decimal) GetValue (i);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		double GetDouble (int i)
 		{
 			return (double) GetValue (i);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		Type GetFieldType (int i)
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
 			return GetColumn (i).DataType;
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		float GetFloat (int i)
 		{
 			return (float) GetValue (i);
@@ -427,71 +391,65 @@ namespace System.Data.Odbc
 
 		[MonoTODO]
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		Guid GetGuid (int i)
 		{
 			throw new NotImplementedException ();
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		short GetInt16 (int i)
 		{
 			return (short) GetValue (i);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int GetInt32 (int i)
 		{
 			return (int) GetValue (i);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		long GetInt64 (int i)
 		{
 			return (long) GetValue (i);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		string GetName (int i)
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
 			return GetColumn (i).ColumnName;
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int GetOrdinal (string value)
 		{
-			int i = ColIndex (value);
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
+			if (value == null)
+				throw new ArgumentNullException ("fieldName");
 
+			int i = ColIndex (value);
 			if (i == -1)
 				throw new IndexOutOfRangeException ();
-			else
-				return i;
+			return i;
 		}
 
 		[MonoTODO]
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		DataTable GetSchemaTable ()
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
+
 			// FIXME : 
 			// * Map OdbcType to System.Type and assign to DataType.
 			//   This will eliminate the need for IsStringType in
@@ -535,8 +493,6 @@ namespace System.Data.Odbc
 				DataRow schemaRow;
 
 				for (int i = 0; i < cols.Length; i += 1 ) {
-					string baseTableName = String.Empty;
-					bool isKey = false;
 					OdbcColumn col=GetColumn(i);
 
 					schemaRow = dataTableSchema.NewRow ();
@@ -609,9 +565,7 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		string GetString (int i)
 		{
 			object ret = GetValue (i);
@@ -629,14 +583,13 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		object GetValue (int i)
 		{
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
 			if (currentRow == -1)
-				throw new IndexOutOfRangeException ();
-
+				throw new InvalidOperationException ("No data available.");
 			if (i > cols.Length-1 || i < 0)
 				throw new IndexOutOfRangeException ();
 
@@ -708,6 +661,8 @@ namespace System.Data.Odbc
 					bufsize = (col.MaxLength < 127 ? (col.MaxLength*2+1) : 255);
 					buffer = new byte[bufsize];  // According to sqlext.h, use SQL_CHAR for both char and varchar
 					StringBuilder sb = new StringBuilder ();
+					char[] charBuffer = new char[bufsize];
+					Decoder unicodeDecoder = Encoding.Unicode.GetDecoder ();
 					do {
 						ret = libodbc.SQLGetData (hstmt, ColIndex, col.SqlCType, buffer, bufsize, ref outsize);
 						if (ret == OdbcReturn.Error)
@@ -716,24 +671,24 @@ namespace System.Data.Odbc
 						if (ret == OdbcReturn.Success && outsize==-1)
 							ret = OdbcReturn.NoData;
 
-						if (ret != OdbcReturn.NoData && outsize > 0) {
-							string value = null;
-
-							if (outsize < bufsize)
-								value = Encoding.Unicode.GetString (buffer, 0, outsize);
-							else
-								value = Encoding.Unicode.GetString (buffer, 0, bufsize);
-
-							sb.Append (RemoveTrailingNullChar (value));
+						if (ret == OdbcReturn.Success || ret == OdbcReturn.SuccessWithInfo) {
+							if (outsize >= bufsize || outsize == (int)OdbcLengthIndicator.NoTotal)
+								outsize = bufsize;
+							int charCount = unicodeDecoder.GetChars (buffer, 0, outsize, charBuffer, 0);
+							string strValue = new String (charBuffer, 0, charCount);
+							sb.Append (RemoveTrailingNullChar (strValue));
 						}
 					} while (ret != OdbcReturn.NoData);
 					DataValue = sb.ToString ();
+					charBuffer = null;
 					break;
 				case OdbcType.Text:
 				case OdbcType.VarChar:
 					bufsize = (col.MaxLength < 255 ? (col.MaxLength+1) : 255);
 					buffer = new byte[bufsize];  // According to sqlext.h, use SQL_CHAR for both char and varchar
 					StringBuilder sb1 = new StringBuilder ();
+					charBuffer = new char[bufsize];
+					Decoder defaultDecoder = Encoding.Default.GetDecoder();
 					do { 
 						ret = libodbc.SQLGetData (hstmt, ColIndex, col.SqlCType, buffer, bufsize, ref outsize);
 						if (ret == OdbcReturn.Error)
@@ -741,11 +696,11 @@ namespace System.Data.Odbc
 						// Fix for strance ODBC drivers (like psqlODBC)
 						if (ret == OdbcReturn.Success && outsize==-1)
 							ret = OdbcReturn.NoData;
-						if (ret != OdbcReturn.NoData && outsize > 0) {
-							if (outsize < bufsize)
-								sb1.Append (Encoding.Default.GetString (buffer, 0, outsize));
-							else
-								sb1.Append (Encoding.Default.GetString (buffer, 0, bufsize - 1));
+						if (ret == OdbcReturn.Success || ret == OdbcReturn.SuccessWithInfo) {
+							if (outsize >= bufsize || outsize == (int)OdbcLengthIndicator.NoTotal)
+								outsize = bufsize - 1;
+							int charCount = defaultDecoder.GetChars(buffer, 0, outsize, charBuffer, 0);
+							sb1.Append(charBuffer, 0, charCount);
 						}
 					} while (ret != OdbcReturn.NoData);
 					DataValue = sb1.ToString ();
@@ -784,25 +739,31 @@ namespace System.Data.Odbc
 					bufsize = (col.MaxLength < 255 && col.MaxLength > 0 ? col.MaxLength : 255);
 					buffer= new byte [bufsize];
 					ArrayList al = new ArrayList ();
-					do {
-						ret = libodbc.SQLGetData (hstmt, ColIndex, SQL_C_TYPE.BINARY, buffer, bufsize, ref outsize);
-						if (ret == OdbcReturn.Error)
-							break;
-						if (ret != OdbcReturn.NoData && outsize!=-1) {
-							if (outsize < bufsize) {
-								byte[] tmparr = new byte [outsize];
-								Array.Copy (buffer, 0, tmparr, 0, outsize);
-								al.AddRange (tmparr);
-							} else
-								al.AddRange (buffer);
-						}
-					} while (ret != OdbcReturn.NoData);
+					//get the size of data to be returned.
+					ret = libodbc.SQLGetData (hstmt, ColIndex, SQL_C_TYPE.BINARY, buffer, 0, ref outsize);
+					if (outsize != (int) OdbcLengthIndicator.NullData) {
+						do {
+							ret = libodbc.SQLGetData (hstmt, ColIndex, SQL_C_TYPE.BINARY, buffer, bufsize, ref outsize);
+							if (ret == OdbcReturn.Error)
+								break;
+							if (ret != OdbcReturn.NoData && outsize != -1) {
+								if (outsize < bufsize) {
+									byte [] tmparr = new byte [outsize];
+									Array.Copy (buffer, 0, tmparr, 0, outsize);
+									al.AddRange (tmparr);
+								} else
+									al.AddRange (buffer);
+							} else {
+								break;
+							}
+						} while (ret != OdbcReturn.NoData);
+					}
 					DataValue = al.ToArray (typeof (byte));
 					break;
 				case OdbcType.Binary :
 					bufsize = col.MaxLength;
 					buffer = new byte [bufsize];
-					long read = GetBytes (i, 0, buffer, 0, bufsize);
+					GetBytes (i, 0, buffer, 0, bufsize);
 					ret = OdbcReturn.Success;
 					DataValue = buffer;
 					break;
@@ -829,13 +790,16 @@ namespace System.Data.Odbc
 		}
 		
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int GetValues (object [] values)
 		{
 			int numValues = 0;
 
+			if (IsClosed)
+				throw new InvalidOperationException ("The reader is closed.");
+			if (currentRow == -1)
+				throw new InvalidOperationException ("No data available.");
+
 			// copy values
 			for (int i = 0; i < values.Length; i++) {
 				if (i < FieldCount) {
@@ -856,29 +820,13 @@ namespace System.Data.Odbc
 			return numValues;
 		}
 
-#if ONLY_1_1
-		void IDisposable.Dispose ()
-		{
-			Dispose (true);
-			GC.SuppressFinalize (this);
-		}
 
-		IEnumerator IEnumerable.GetEnumerator ()
-		{
-			return new DbEnumerator (this);
-		}
-#endif // ONLY_1_1
-
-#if NET_2_0
 		public override IEnumerator GetEnumerator ()
 		{
 			return new DbEnumerator (this);
 		}
-#endif
 
-#if NET_2_0
 		protected override
-#endif
 		void Dispose (bool disposing)
 		{
 			if (disposed)
@@ -896,9 +844,7 @@ namespace System.Data.Odbc
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool IsDBNull (int i)
 		{
 			return (GetValue (i) is DBNull);
@@ -908,9 +854,7 @@ namespace System.Data.Odbc
 		/// Move to the next result set.
 		/// </remarks>
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool NextResult ()
 		{
 			OdbcReturn ret = OdbcReturn.Success;
@@ -1021,7 +965,6 @@ namespace System.Data.Odbc
 				if (ret != OdbcReturn.Success && ret != OdbcReturn.SuccessWithInfo)
 					throw Connection.CreateOdbcException (OdbcHandleType.Stmt, handle);
 
-				int i = 0;
 				while (true) {
 					ret = libodbc.SQLFetch (handle);
 					if (ret != OdbcReturn.Success && ret != OdbcReturn.SuccessWithInfo)
@@ -1066,7 +1009,7 @@ namespace System.Data.Odbc
 				// NON_UNIQUE
 				int  nonUniqueLength = 0;
 				short nonUnique = libodbc.SQL_FALSE;
-				ret = libodbc.SQLBindCol (handle, 4, SQL_C_TYPE.SHORT, ref (short) nonUnique, sizeof (short), ref nonUniqueLength);
+				ret = libodbc.SQLBindCol (handle, 4, SQL_C_TYPE.SHORT, ref nonUnique, sizeof (short), ref nonUniqueLength);
 				if (ret != OdbcReturn.Success && ret != OdbcReturn.SuccessWithInfo)
 					throw Connection.CreateOdbcException (OdbcHandleType.Stmt, handle);
 
@@ -1077,7 +1020,6 @@ namespace System.Data.Odbc
 				if (ret != OdbcReturn.Success && ret != OdbcReturn.SuccessWithInfo)
 					throw Connection.CreateOdbcException (OdbcHandleType.Stmt, handle);
 			
-				int i = 0;
 				while (true) {
 					ret = libodbc.SQLFetch (handle);
 					if (ret != OdbcReturn.Success && ret != OdbcReturn.SuccessWithInfo)
@@ -1103,9 +1045,7 @@ namespace System.Data.Odbc
 		}
 		
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool Read ()
 		{
 			return NextRow ();
