@@ -5,11 +5,12 @@
 //   Rodrigo Moya (rodrigo@ximian.com)
 //   Daniel Morgan (danmorg@sc.rr.com)
 //   Tim Coleman (tim@timcoleman.com)
+//	 Veerapuram Varadhan  (vvaradhan@novell.com)
 //
 // (C) Ximian, Inc 2002
 // Copyright (C) 2002 Tim Coleman
 //
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004, 2009 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -40,20 +41,12 @@ namespace System.Data.SqlClient {
 	[DefaultEvent ("RowUpdated")]
 	[DesignerAttribute ("Microsoft.VSDesigner.Data.VS.SqlDataAdapterDesigner, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.ComponentModel.Design.IDesigner")]
 	[ToolboxItemAttribute ("Microsoft.VSDesigner.Data.VS.SqlDataAdapterToolboxItem, "+ Consts.AssemblyMicrosoft_VSDesigner)]
-	public sealed class SqlDataAdapter : DbDataAdapter, IDbDataAdapter, ICloneable
+
+	public sealed class SqlDataAdapter : DbDataAdapter, IDbDataAdapter, IDataAdapter, ICloneable
 	{
 		#region Fields
 
-#if !NET_2_0
-		bool disposed;
-#endif
-		SqlCommand deleteCommand;
-		SqlCommand insertCommand;
-		SqlCommand selectCommand;
-		SqlCommand updateCommand;
-#if NET_2_0
 		int updateBatchSize;
-#endif
 		#endregion
 
 		#region Constructors
@@ -65,9 +58,7 @@ namespace System.Data.SqlClient {
 		public SqlDataAdapter (SqlCommand selectCommand) 
 		{
 			SelectCommand = selectCommand;
-#if NET_2_0
 			UpdateBatchSize = 1;
-#endif
 		}
 
 		public SqlDataAdapter (string selectCommandText, SqlConnection selectConnection) 
@@ -84,67 +75,69 @@ namespace System.Data.SqlClient {
 
 		#region Properties
 
-#if !NET_2_0
-		[DataSysDescription ("Used during Update for deleted rows in DataSet.")]
-#endif
 		[DefaultValue (null)]
 		[EditorAttribute ("Microsoft.VSDesigner.Data.Design.DBCommandEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
 		public new SqlCommand DeleteCommand {
-			get { return deleteCommand; }
-			set { deleteCommand = value; }
+			get { 
+				return (SqlCommand)base.DeleteCommand; 
+			}
+			set { 
+				base.DeleteCommand = value; 
+			}
 		}
 
-#if !NET_2_0
-		[DataSysDescription ("Used during Update for new rows in DataSet.")]
-#endif
 		[DefaultValue (null)]
 		[EditorAttribute ("Microsoft.VSDesigner.Data.Design.DBCommandEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
 		public new SqlCommand InsertCommand {
-			get { return insertCommand; }
-			set { insertCommand = value; }
+			get { 
+				return (SqlCommand)base.InsertCommand; 
+			}
+			set { 
+				base.InsertCommand = value; 
+			}
 		}
 
-#if !NET_2_0
-		[DataSysDescription ("Used during Fill/FillSchema.")]
-#endif
 		[DefaultValue (null)]
 		[EditorAttribute ("Microsoft.VSDesigner.Data.Design.DBCommandEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
 		public new SqlCommand SelectCommand {
-			get { return selectCommand; }
-			set { selectCommand = value; }
+			get { 
+				return (SqlCommand)base.SelectCommand; 
+			}
+			set { 
+				base.SelectCommand = value; 
+			}
 		}
 
-#if !NET_2_0
-		[DataSysDescription ("Used during Update for modified rows in DataSet.")]
-#endif
 		[DefaultValue (null)]
 		[EditorAttribute ("Microsoft.VSDesigner.Data.Design.DBCommandEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
-		public new SqlCommand UpdateCommand {
-			get { return updateCommand; }
-			set { updateCommand = value; }
+		public new  SqlCommand UpdateCommand {
+			get { 
+				return (SqlCommand)base.UpdateCommand; 
+			}
+			set { 
+				base.UpdateCommand = value; 
+			}
+		}
+		
+		IDbCommand IDbDataAdapter.SelectCommand {
+			get { return SelectCommand; }
+			set { SelectCommand = (SqlCommand) value; }
+		}
+		
+		IDbCommand IDbDataAdapter.InsertCommand {
+			get { return InsertCommand; }
+			set { InsertCommand = (SqlCommand) value; }
+		}
+		
+		IDbCommand IDbDataAdapter.UpdateCommand {
+			get { return UpdateCommand; }
+			set { UpdateCommand = (SqlCommand) value; }
 		}
-
 		IDbCommand IDbDataAdapter.DeleteCommand {
 			get { return DeleteCommand; }
 			set { DeleteCommand = (SqlCommand) value; }
 		}
 
-		IDbCommand IDbDataAdapter.InsertCommand {
-			get { return InsertCommand; }
-			set { InsertCommand = (SqlCommand) value; }
-		}
-
-		IDbCommand IDbDataAdapter.SelectCommand {
-			get { return SelectCommand; }
-			set { SelectCommand = (SqlCommand) value; }
-		}
-
-		IDbCommand IDbDataAdapter.UpdateCommand {
-			get { return UpdateCommand; }
-			set { UpdateCommand = (SqlCommand) value; }
-		}
-
-#if NET_2_0
 		public override int UpdateBatchSize {
 			get { return updateBatchSize; }
 			set {
@@ -153,7 +146,6 @@ namespace System.Data.SqlClient {
 				updateBatchSize = value; 
 			}
 		}
-#endif
 
 		#endregion // Properties
 
@@ -170,19 +162,6 @@ namespace System.Data.SqlClient {
 			return new SqlRowUpdatingEventArgs (dataRow, command, statementType, tableMapping);
 		}
 
-#if !NET_2_0
-		protected override void Dispose (bool disposing)
-		{
-			if (!disposed) {
-				if (disposing) {
-					// Release managed resources
-				}
-				// Release unmanaged resources
-				disposed = true;
-			}
-			base.Dispose (disposing);
-		}
-#endif
 
 		protected override void OnRowUpdated (RowUpdatedEventArgs value) 
 		{
@@ -202,7 +181,6 @@ namespace System.Data.SqlClient {
 			throw new NotImplementedException ();
 		}
 
-#if NET_2_0
 		// All the batch methods, should be implemented, if supported,
 		// by individual providers 
 
@@ -241,19 +219,12 @@ namespace System.Data.SqlClient {
 		{
 			throw new NotImplementedException ();
 		}
-#endif
 		#endregion // Methods
 
 		#region Events and Delegates
 
-#if ONLY_1_1
-		[DataSysDescription ("Event triggered before every DataRow during Update.")]
-#endif
 		public event SqlRowUpdatedEventHandler RowUpdated;
 
-#if ONLY_1_1
-		[DataSysDescription ("Event triggered after every DataRow during Update.")]
-#endif
 		public event SqlRowUpdatingEventHandler RowUpdating;
 
 		#endregion // Events and Delegates
