@@ -70,6 +70,8 @@ namespace System.Data
 		private DataViewManager defaultView;
 		private CultureInfo locale;
 		internal XmlDataDocument _xmlDataDocument;
+
+		internal TableAdapterSchemaInfo tableAdapterSchemaInfo;
 		bool initInProgress;
 
 		#region Constructors
@@ -91,6 +93,15 @@ namespace System.Data
 		protected DataSet (SerializationInfo info, StreamingContext context)
 			: this ()
 		{
+			if (IsBinarySerialized (info, context)) {
+				BinaryDeserialize (info);
+				return;
+			}
+			string s = info.GetValue ("XmlSchema", typeof (String)) as String;
+			XmlTextReader reader = new XmlTextReader (new StringReader (s));
+			ReadXmlSchema (reader);
+			reader.Close ();
+
 			GetSerializationData (info, context);
 		}
 
@@ -99,9 +110,6 @@ namespace System.Data
 		#region Public Properties
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("Indicates whether comparing strings within the DataSet is case sensitive.")]
-#endif
 		[DefaultValue (false)]
 		public bool CaseSensitive {
 			get { return caseSensitive; }
@@ -122,18 +130,12 @@ namespace System.Data
 		}
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("The name of this DataSet.")]
-#endif
 		[DefaultValue ("")]
 		public string DataSetName {
 			get { return dataSetName; }
 			set { dataSetName = value; }
 		}
 
-#if !NET_2_0
-		[DataSysDescription ("Indicates a custom \"view\" of the data contained by the DataSet. This view allows filtering, searching, and navigating through the custom data view.")]
-#endif
 		[Browsable (false)]
 		public DataViewManager DefaultViewManager {
 			get {
@@ -143,9 +145,6 @@ namespace System.Data
 			}
 		}
 
-#if !NET_2_0
-		[DataSysDescription ("Indicates whether constraint rules are to be followed.")]
-#endif
 		[DefaultValue (true)]
 		public bool EnforceConstraints {
 			get { return enforceConstraints; }
@@ -154,17 +153,11 @@ namespace System.Data
 
 		[Browsable (false)]
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("The collection that holds custom user information.")]
-#endif
 		public PropertyCollection ExtendedProperties {
 			get { return properties; }
 		}
 
 		[Browsable (false)]
-#if !NET_2_0
-		[DataSysDescription ("Indicates that the DataSet has errors.")]
-#endif
 		public bool HasErrors {
 			get {
 				for (int i = 0; i < Tables.Count; i++) {
@@ -176,9 +169,6 @@ namespace System.Data
 		}
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("Indicates a locale under which to compare strings within the DataSet.")]
-#endif
 		public CultureInfo Locale {
 			get { return locale != null ? locale : Thread.CurrentThread.CurrentCulture; }
 			set {
@@ -194,6 +184,11 @@ namespace System.Data
 			get { return locale != null; }
 		}
 
+		
+		internal TableAdapterSchemaInfo TableAdapterSchemaData {
+			get { return tableAdapterSchemaInfo; }
+		}
+		
 		internal void InternalEnforceConstraints (bool value,bool resetIndexes)
 		{
 			if (value == enforceConstraints)
@@ -284,9 +279,6 @@ namespace System.Data
 		}
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("Indicates the XML uri namespace for the root element pointed at by this DataSet.")]
-#endif
 		[DefaultValue ("")]
 		public string Namespace {
 			get { return _namespace; }
@@ -301,9 +293,6 @@ namespace System.Data
 		}
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("Indicates the prefix of the namespace used for this DataSet.")]
-#endif
 		[DefaultValue ("")]
 		public string Prefix {
 			get { return prefix; }
@@ -323,9 +312,6 @@ namespace System.Data
 		}
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("The collection that holds the relations for this DatSet.")]
-#endif
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Content)]
 		public DataRelationCollection Relations {
 			get { return relationCollection; }
@@ -339,9 +325,6 @@ namespace System.Data
 		}
 
 		[DataCategory ("Data")]
-#if !NET_2_0
-		[DataSysDescription ("The collection that holds the tables for this DataSet.")]
-#endif
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Content)]
 		public DataTableCollection Tables {
 			get { return tableCollection; }
@@ -546,7 +529,8 @@ namespace System.Data
 			// add the current row
 			DataRow newRow = copyTable.NewNotInitializedRow ();
 			copyTable.Rows.AddInternal (newRow);
-			row.CopyValuesToRow (newRow);
+			// Don't check for ReadOnly, when cloning data to new uninitialized row.
+			row.CopyValuesToRow (newRow, false);
 			newRow.XmlRowID = row.XmlRowID;
 			addedRows.Add (row, row);
 		}
@@ -820,7 +804,9 @@ namespace System.Data
 		public void ReadXmlSchema (XmlReader reader)
 		{
 #if true
-			new XmlSchemaDataImporter (this, reader, true).Process ();
+			XmlSchemaDataImporter xsdImporter = new XmlSchemaDataImporter (this, reader, true);
+			xsdImporter.Process ();
+			tableAdapterSchemaInfo = xsdImporter.CurrentAdapter;
 #else
 			XmlSchemaMapper SchemaMapper = new XmlSchemaMapper (this);
 			SchemaMapper.Read (reader);
@@ -1050,9 +1036,6 @@ namespace System.Data
 		#region Public Events
 
 		[DataCategory ("Action")]
-#if !NET_2_0
-		[DataSysDescription ("Occurs when it is not possible to merge schemas for two tables with the same name.")]
-#endif
 		public event MergeFailedEventHandler MergeFailed;
 
 		#endregion // Public Events
@@ -1080,9 +1063,7 @@ namespace System.Data
 		public void BeginInit ()
 		{
 			InitInProgress = true;
-#if NET_2_0
 			dataSetInitialized = false;
-#endif
 		}
 
 		public void EndInit ()
@@ -1098,26 +1079,18 @@ namespace System.Data
 
 			Relations.PostAddRange ();
 			InitInProgress = false;
-#if NET_2_0
 			dataSetInitialized = true;
 			DataSetInitialized ();
-#endif
 		}
 		#endregion
 
 		#region ISerializable
-#if NET_2_0
 		public virtual
-#endif
 		void
-#if !NET_2_0
-		ISerializable.
-#endif
 		GetObjectData (SerializationInfo info, StreamingContext context)
 		{
-#if NET_2_0
 			if (RemotingFormat == SerializationFormat.Xml) {
-#endif
+				info.AddValue ("SchemaSerializationMode.DataSet", this.SchemaSerializationMode);
 				StringWriter sw = new StringWriter ();
 				XmlTextWriter writer = new XmlTextWriter (sw);
 				DoWriteXmlSchema (writer);
@@ -1129,30 +1102,17 @@ namespace System.Data
 				WriteXml (writer, XmlWriteMode.DiffGram);
 				writer.Flush ();
 				info.AddValue ("XmlDiffGram", sw.ToString ());
-#if NET_2_0
 			} else /*if (DataSet.RemotingFormat == SerializationFormat.Binary)*/ {
 				BinarySerialize (info);
 			}
-#endif
 		}
 		#endregion
 
 		#region Protected Methods
 		protected void GetSerializationData (SerializationInfo info, StreamingContext context)
 		{
-#if NET_2_0
-			if (IsBinarySerialized (info, context)) {
-				BinaryDeserialize (info);
-				return;
-			}
-#endif
-			string s = info.GetValue ("XmlSchema", typeof (String)) as String;
+			string s = info.GetValue ("XmlDiffGram", typeof (String)) as String;
 			XmlTextReader reader = new XmlTextReader (new StringReader (s));
-			ReadXmlSchema (reader);
-			reader.Close ();
-
-			s = info.GetValue ("XmlDiffGram", typeof (String)) as String;
-			reader = new XmlTextReader (new StringReader (s));
 			ReadXml (reader, XmlReadMode.DiffGram);
 			reader.Close ();
 		}
@@ -1245,11 +1205,7 @@ namespace System.Data
 				case TypeCode.Char:
 					return XmlConvert.ToString ((Char) o);
 				case TypeCode.DateTime:
-#if NET_2_0
 					return XmlConvert.ToString ((DateTime) o, XmlDateTimeSerializationMode.Unspecified);
-#else
-					return XmlConvert.ToString ((DateTime) o);
-#endif
 				case TypeCode.Decimal:
 					return XmlConvert.ToString ((Decimal) o);
 				case TypeCode.Double:
@@ -1312,7 +1268,6 @@ namespace System.Data
 
 			SplitColumns (table, out atts, out elements, out simple);
 			//sort out the namespacing
-			int relationCount = table.ParentRelations.Count;
 
 			foreach (DataRow row in rows) {
 				if (skipIfNested) {
@@ -1386,8 +1341,12 @@ namespace System.Data
 
 			//TODO check if I can get away with write element string
 			WriteStartElement (writer, mode, colnspc, col.Prefix, XmlHelper.Encode (col.ColumnName));	
-			if (typeof (IXmlSerializable).IsAssignableFrom (col.DataType)) {
-				((IXmlSerializable)rowObject).WriteXml (writer);
+			if (typeof (IXmlSerializable).IsAssignableFrom (col.DataType) 
+			    || col.DataType == typeof (object)) {
+				IXmlSerializable serializableObj = rowObject as IXmlSerializable;
+				if (serializableObj == null)
+					throw new InvalidOperationException ();
+				((IXmlSerializable)rowObject).WriteXml (writer);				
 			} else {
 				writer.WriteString (WriteObjectXml (rowObject));
 			}
@@ -1525,17 +1484,32 @@ namespace System.Data
 		#endregion //Private Xml Serialisation
 	}
 
-#if NET_2_0
 	[XmlSchemaProvider ("GetDataSetSchema")]
 	[XmlRoot ("DataSet")]
 	partial class DataSet : ISupportInitializeNotification {
 		private bool dataSetInitialized = true;
 		public event EventHandler Initialized;
 
-		[MonoTODO]
-		protected DataSet (SerializationInfo info, StreamingContext context, bool constructSchema)
-			: this (info, context)
+		protected DataSet (SerializationInfo info, StreamingContext context, bool ConstructSchema)
+			: this ()
 		{
+			if (DetermineSchemaSerializationMode (info, context) == SchemaSerializationMode.ExcludeSchema) {
+				InitializeDerivedDataSet ();
+			}
+			
+			if (IsBinarySerialized (info, context)) {
+				BinaryDeserialize (info);
+				return;
+			}
+			
+			if (ConstructSchema) {
+				string s = info.GetValue ("XmlSchema", typeof (String)) as String;
+				XmlTextReader reader = new XmlTextReader (new StringReader (s));
+				ReadXmlSchema (reader);
+				reader.Close ();
+				
+				GetSerializationData (info, context);
+			}
 		}
 
 		SerializationFormat remotingFormat = SerializationFormat.Xml;
@@ -1736,10 +1710,8 @@ namespace System.Data
 			OnDataSetInitialized (e);
 		}
 
-		[MonoTODO]
 		protected virtual void InitializeDerivedDataSet ()
 		{
-			throw new NotImplementedException ();
 		}
 
 		protected SchemaSerializationMode DetermineSchemaSerializationMode (XmlReader reader)
@@ -1749,6 +1721,13 @@ namespace System.Data
 
 		protected SchemaSerializationMode DetermineSchemaSerializationMode (SerializationInfo info, StreamingContext context)
 		{
+			SerializationInfoEnumerator e = info.GetEnumerator ();
+			while (e.MoveNext ()) {
+				if (e.Name == "SchemaSerializationMode.DataSet") {
+					return (SchemaSerializationMode) e.Value;
+				}
+			}
+			
 			return SchemaSerializationMode.IncludeSchema;
 		}
 
@@ -1762,5 +1741,4 @@ namespace System.Data
 			return false;
 		}
 	}
-#endif
 }
