@@ -118,7 +118,6 @@ namespace MonoTests.System.Drawing{
 			public string lfFaceName;
 		}
 
-#if !TARGET_JVM
 		[Test]
 		[Category ("CAS")]
 		[ExpectedException (typeof (SecurityException))]
@@ -189,24 +188,13 @@ namespace MonoTests.System.Drawing{
 
 		[Test]
 		[SecurityPermission (SecurityAction.Assert, UnmanagedCode = true)]
-	#if NET_2_0
 		[ExpectedException (typeof (AccessViolationException))]
-	#else
-		[ExpectedException (typeof (NullReferenceException))]
-	#endif
 		public void ToLogFont_Null ()
 		{
 			Font f = new Font ("Arial", 10);
 			f.ToLogFont (null);
 		}
-#endif
 		[Test]
-#if ONLY_1_1
-		[ExpectedException (typeof (ArgumentNullException))]
-#endif
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_StringNull_Float ()
 		{
 			string family = null;
@@ -219,9 +207,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_String_Float ()
 		{
 			Font f = new Font (name, 12.5f);
@@ -240,9 +225,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_String_Float_FontStyle ()
 		{
 			Font f = new Font (name, 12.5f, FontStyle.Bold);
@@ -261,9 +243,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_String_Float_FontStyle_GraphicsUnit ()
 		{
 			Font f = new Font (name, 12.5f, FontStyle.Italic, GraphicsUnit.Pixel);
@@ -282,18 +261,12 @@ namespace MonoTests.System.Drawing{
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_String_Float_FontStyle_GraphicsUnit_Display ()
 		{
 			new Font (name, 12.5f, FontStyle.Italic, GraphicsUnit.Display);
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_String_Float_FontStyle_GraphicsUnit_Byte ()
 		{
 			Font f = new Font (name, 12.5f, FontStyle.Strikeout, GraphicsUnit.Inch, Byte.MaxValue);
@@ -312,9 +285,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_String_Float_FontStyle_GraphicsUnit_Byte_Bool ()
 		{
 			Font f = new Font (name, 12.5f, FontStyle.Underline, GraphicsUnit.Document, Byte.MinValue, true);
@@ -341,9 +311,14 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
+		[ExpectedException (typeof (NullReferenceException))]
+		public void Font_FontNull_FontStyle ()
+		{
+			Font f = null;
+			new Font (f, FontStyle.Bold);
+		}
+
+		[Test]
 		public void Font_FontFamily_Float ()
 		{
 			Font f = new Font (FontFamily.GenericMonospace, 12.5f);
@@ -362,9 +337,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_FontFamily_Float_FontStyle ()
 		{
 			Font f = new Font (FontFamily.GenericMonospace, 12.5f, FontStyle.Bold);
@@ -383,9 +355,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_FontFamily_Float_FontStyle_GraphicsUnit ()
 		{
 			Font f = new Font (FontFamily.GenericMonospace, 12.5f, FontStyle.Italic, GraphicsUnit.Millimeter);
@@ -405,18 +374,12 @@ namespace MonoTests.System.Drawing{
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_FontFamily_Float_FontStyle_GraphicsUnit_Display ()
 		{
 			new Font (FontFamily.GenericMonospace, 12.5f, FontStyle.Italic, GraphicsUnit.Display);
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_FontFamily_Float_FontStyle_GraphicsUnit_Byte ()
 		{
 			Font f = new Font (FontFamily.GenericMonospace, 12.5f, FontStyle.Strikeout, GraphicsUnit.Inch, Byte.MaxValue);
@@ -435,9 +398,6 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Font_FontFamily_Float_FontStyle_GraphicsUnit_Byte_Bool ()
 		{
 			Font f = new Font (FontFamily.GenericMonospace, 12.5f, FontStyle.Underline, GraphicsUnit.Document, Byte.MinValue, true);
@@ -476,9 +436,6 @@ namespace MonoTests.System.Drawing{
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Dispose_Height ()
 		{
 			Font f = new Font (name, 12.5f);
@@ -488,26 +445,17 @@ namespace MonoTests.System.Drawing{
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Dispose_ToLogFont ()
 		{
-#if !TARGET_JVM
 			Font f = new Font (name, 12.5f);
 			f.Dispose ();
 			LOGFONT	lf = new LOGFONT();
 			f.ToLogFont (lf);
-#endif
 		}
 
 		[Test]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Dispose_ToLogFont_LoopCharSet ()
 		{
-#if !TARGET_JVM
 			Font f = new Font (name, 12.5f);
 			f.Dispose ();
 			LOGFONT lf = new LOGFONT ();
@@ -552,21 +500,15 @@ namespace MonoTests.System.Drawing{
 					Assert.Fail ("Unexcepted exception {0} at iteration {1}", e, i);
 				}
 			}
-#endif
 		}
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
-#if TARGET_JVM
-		[Category ("NotWorking")]
-#endif
 		public void Dispose_ToHFont ()
 		{
-#if !TARGET_JVM
 			Font f = new Font (name, 12.5f);
 			f.Dispose ();
 			f.ToHfont ();
-#endif
 		}
 		
 		[Test]
@@ -603,5 +545,55 @@ namespace MonoTests.System.Drawing{
 				}
 			}
 		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void GetHeight_Graphics_Null ()
+		{
+			using (Font f = new Font (name, 12.5f)) {
+				Assert.AreEqual (0, f.GetHeight (null), "0");
+			}
+		}
+
+		[Test]
+		public void FontUniqueHashCode ()
+		{
+			Font f1 = new Font ("Arial", 14);
+			Font f2 = new Font ("Arial", 12);
+			Font f3 = new Font (f1, FontStyle.Bold);
+
+			Assert.IsFalse (f1.GetHashCode () == f2.GetHashCode (), "1) Fonts with different sizes should have different HashCodes");
+			Assert.IsFalse (f1.GetHashCode () == f3.GetHashCode (), "2) Fonts with different styles should have different HashCodes");
+		}
+
+        [Test]
+        public void GetHashCode_UnitDiffers_HashesNotEqual()
+        {
+            Font f1 = new Font("Arial", 8.25F, GraphicsUnit.Point);
+            Font f2 = new Font("Arial", 8.25F, GraphicsUnit.Pixel);
+
+            Assert.IsFalse(f1.GetHashCode() == f2.GetHashCode(),
+                "Hashcodes should differ if _unit member differs");
+        }
+
+        [Test]
+        public void GetHashCode_NameDiffers_HashesNotEqual()
+        {
+            Font f1 = new Font("Arial", 8.25F, GraphicsUnit.Point);
+            Font f2 = new Font("Courier New", 8.25F, GraphicsUnit.Point);
+
+            Assert.IsFalse(f1.GetHashCode() == f2.GetHashCode(),
+                "Hashcodes should differ if _name member differs");
+        }
+
+        [Test]
+        public void GetHashCode_StyleEqualsGdiCharSet_HashesNotEqual()
+        {
+            Font f1 = new Font("Arial", 8.25F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(0)));
+            Font f2 = new Font("Arial", 8.25F, FontStyle.Bold, GraphicsUnit.Point, ((byte)(1)));
+
+            Assert.IsFalse(f1.GetHashCode() == f2.GetHashCode(),
+                "Hashcodes should differ if _style member differs");
+        }
 	}
 }
