@@ -13,10 +13,10 @@
 // distribute, sublicense, and/or sell copies of the Software, and to
 // permit persons to whom the Software is furnished to do so, subject to
 // the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be
 // included in all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -25,7 +25,6 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-#if NET_2_0
 using System;
 using System.Collections;
 using System.Collections.Generic;
@@ -74,14 +73,9 @@ namespace System.Runtime.Serialization
 	  exists (and raises InvalidOperationException if required).
 
 */
+
 	internal static class TypeExtensions
 	{
-		public static T GetCustomAttribute<T> (this MemberInfo type, bool inherit)
-		{
-			var arr = type.GetCustomAttributes (typeof (T), inherit);
-			return arr != null && arr.Length == 1 ? (T) arr [0] : default (T);
-		}
-
 		public static IEnumerable<Type> GetInterfacesOrSelfInterface (this Type type)
 		{
 			if (type.IsInterface)
@@ -89,6 +83,20 @@ namespace System.Runtime.Serialization
 			foreach (var t in type.GetInterfaces ())
 				yield return t;
 		}
+
+		public static bool ImplementsInterface (this Type type, Type iface)
+		{
+			foreach (var t in type.GetInterfacesOrSelfInterface ()) {
+				if (t == iface)
+					return true;
+			}
+
+			var baseType = type.BaseType;
+			if (baseType != null)
+				return baseType.ImplementsInterface (iface);
+			
+			return false;
+		}
 	}
 
 	internal sealed class KnownTypeCollection : Collection<Type>
@@ -115,6 +123,9 @@ namespace System.Runtime.Serialization
 			// not in ms nsURI schema
 			dbnull_type, date_time_offset_type;
 
+		// XmlSchemaType.GetBuiltInPrimitiveType() does not exist in moonlight, so I had to explicitly add them. And now that we have it, it does not make much sense to use #if MOONLIGHT ... #endif for XmlSchemaType anymore :-(
+		static Dictionary<string,Type> xs_predefined_types = new Dictionary<string,Type> ();
+
 		static KnownTypeCollection ()
 		{
 			string s = MSSimpleNamespace;
@@ -142,6 +153,55 @@ namespace System.Runtime.Serialization
 
 			dbnull_type = new QName ("DBNull", DefaultClrNamespaceBase + "System");
 			date_time_offset_type = new QName ("DateTimeOffset", DefaultClrNamespaceBase + "System");
+
+			xs_predefined_types.Add ("string", typeof (string));
+			xs_predefined_types.Add ("boolean", typeof (bool));
+			xs_predefined_types.Add ("float", typeof (float));
+			xs_predefined_types.Add ("double", typeof (double));
+			xs_predefined_types.Add ("decimal", typeof (decimal));
+			xs_predefined_types.Add ("duration", typeof (TimeSpan));
+			xs_predefined_types.Add ("dateTime", typeof (DateTime));
+			xs_predefined_types.Add ("date", typeof (DateTime));
+			xs_predefined_types.Add ("time", typeof (DateTime));
+			xs_predefined_types.Add ("gYearMonth", typeof (DateTime));
+			xs_predefined_types.Add ("gYear", typeof (DateTime));
+			xs_predefined_types.Add ("gMonthDay", typeof (DateTime));
+			xs_predefined_types.Add ("gDay", typeof (DateTime));
+			xs_predefined_types.Add ("gMonth", typeof (DateTime));
+			xs_predefined_types.Add ("hexBinary", typeof (byte []));
+			xs_predefined_types.Add ("base64Binary", typeof (byte []));
+			xs_predefined_types.Add ("anyURI", typeof (Uri));
+			xs_predefined_types.Add ("QName", typeof (QName));
+			xs_predefined_types.Add ("NOTATION", typeof (string));
+
+			xs_predefined_types.Add ("normalizedString", typeof (string));
+			xs_predefined_types.Add ("token", typeof (string));
+			xs_predefined_types.Add ("language", typeof (string));
+			xs_predefined_types.Add ("IDREFS", typeof (string []));
+			xs_predefined_types.Add ("ENTITIES", typeof (string []));
+			xs_predefined_types.Add ("NMTOKEN", typeof (string));
+			xs_predefined_types.Add ("NMTOKENS", typeof (string []));
+			xs_predefined_types.Add ("Name", typeof (string));
+			xs_predefined_types.Add ("NCName", typeof (string));
+			xs_predefined_types.Add ("ID", typeof (string));
+			xs_predefined_types.Add ("IDREF", typeof (string));
+			xs_predefined_types.Add ("ENTITY", typeof (string));
+
+			xs_predefined_types.Add ("integer", typeof (decimal));
+			xs_predefined_types.Add ("nonPositiveInteger", typeof (int));
+			xs_predefined_types.Add ("negativeInteger", typeof (int));
+			xs_predefined_types.Add ("long", typeof (long));
+			xs_predefined_types.Add ("int", typeof (int));
+			xs_predefined_types.Add ("short", typeof (short));
+			xs_predefined_types.Add ("byte", typeof (sbyte));
+			xs_predefined_types.Add ("nonNegativeInteger", typeof (decimal));
+			xs_predefined_types.Add ("unsignedLong", typeof (ulong));
+			xs_predefined_types.Add ("unsignedInt", typeof (uint));
+			xs_predefined_types.Add ("unsignedShort", typeof (ushort));
+			xs_predefined_types.Add ("unsignedByte", typeof (byte));
+			xs_predefined_types.Add ("positiveInteger", typeof (decimal));
+
+			xs_predefined_types.Add ("anyType", typeof (object));
 		}
 
 		// FIXME: find out how QName and guid are processed
@@ -161,8 +221,6 @@ namespace System.Runtime.Serialization
 				return name;
 			if (type == typeof (DBNull))
 				return dbnull_type;
-			if (type == typeof (DateTimeOffset))
-				return date_time_offset_type;
 			return QName.Empty;
 		}
 
@@ -189,6 +247,8 @@ namespace System.Runtime.Serialization
 					return base64_type;
 				if (type == typeof (Uri))
 					return any_uri_type;
+				if (type == typeof (DateTimeOffset))
+					return date_time_offset_type;
 				return QName.Empty;
 			case TypeCode.Boolean:
 				return bool_type;
@@ -287,8 +347,8 @@ namespace System.Runtime.Serialization
 					return typeof (DateTimeOffset);
 				}
 				break;
-			// FIXME: xsd types and ms serialization types should be differentiated. (usage problem)
 			case XmlSchema.Namespace:
+				return xs_predefined_types.FirstOrDefault (p => p.Key == name.Name).Value;
 			case MSSimpleNamespace:
 				switch (name.Name) {
 				case "anyURI":
@@ -452,26 +512,6 @@ namespace System.Runtime.Serialization
 				base.InsertItem (index - 1, type);
 		}
 
-		internal SerializationMap FindUserMap (QName qname)
-		{
-			lock (this)
-				return contracts.FirstOrDefault (c => c.XmlName == qname);
-		}
-
-		internal Type GetSerializedType (Type type)
-		{
-			if (IsPrimitiveNotEnum (type))
-				return type;
-			Type element = GetCollectionElementType (type);
-			if (element == null)
-				return type;
-			QName name = GetQName (type);
-			var map = FindUserMap (name);
-			if (map != null)
-				return map.RuntimeType;
-			return type;
-		}
-
 		internal SerializationMap FindUserMap (Type type)
 		{
 			lock (this) {
@@ -482,12 +522,38 @@ namespace System.Runtime.Serialization
 			}
 		}
 
+		internal SerializationMap FindUserMap (QName qname)
+		{
+			lock (this)
+				return contracts.FirstOrDefault (c => c.XmlName == qname);
+		}
+
+		internal SerializationMap FindUserMap (QName qname, Type type)
+		{
+			lock (this)
+				return contracts.FirstOrDefault (c => c.XmlName == qname && c.RuntimeType == type);
+		}
+
+		internal Type GetSerializedType (Type type)
+		{
+			if (IsPrimitiveNotEnum (type))
+				return type;
+			Type element = GetCollectionElementType (type);
+			if (element == null)
+				return type;
+			QName name = GetQName (type);
+			var map = FindUserMap (name, type);
+			if (map != null)
+				return map.RuntimeType;
+			return type;
+		}
+
 		internal QName GetQName (Type type)
 		{
 			SerializationMap map = FindUserMap (type);
 			if (map != null)
 				// already mapped.
-				return map.XmlName; 
+				return map.XmlName;
 			return GetStaticQName (type);
 		}
 
@@ -504,7 +570,7 @@ namespace System.Runtime.Serialization
 				return qname;
 
 			if (type.GetInterface ("System.Xml.Serialization.IXmlSerializable") != null)
-				//FIXME: Reusing GetSerializableQName here, since we just 
+				//FIXME: Reusing GetSerializableQName here, since we just
 				//need name of the type..
 				return GetSerializableQName (type);
 
@@ -513,8 +579,10 @@ namespace System.Runtime.Serialization
 				return qname;
 
 			Type element = GetCollectionElementType (type);
-			if (element != null)
-				return GetCollectionQName (element);
+			if (element != null) {
+				if (type.IsInterface || IsCustomCollectionType (type, element))
+					return GetCollectionQName (element);
+			}
 
 			if (GetAttribute<SerializableAttribute> (type) != null)
 				return GetSerializableQName (type);
@@ -539,6 +607,12 @@ namespace System.Runtime.Serialization
 		{
 			if (name == null)
 				name = GetDefaultName (type);
+			else if (type.IsGenericType) {
+				var args = type.GetGenericArguments ();
+				for (int i = 0; i < args.Length; i++)
+					name = name.Replace ("{" + i + "}", GetStaticQName (args [i]).Name);
+			}
+
 			if (ns == null)
 				ns = GetDefaultNamespace (type);
 			return new QName (name, ns);
@@ -569,7 +643,7 @@ namespace System.Runtime.Serialization
 
 		internal static string GetDefaultName (Type type)
 		{
-			// FIXME: there could be decent ways to get 
+			// FIXME: there could be decent ways to get
 			// the same result...
 			string name = type.Namespace == null || type.Namespace.Length == 0 ? type.Name : type.FullName.Substring (type.Namespace.Length + 1).Replace ('+', '.');
 			if (type.IsGenericType) {
@@ -591,7 +665,7 @@ namespace System.Runtime.Serialization
 		static QName GetCollectionQName (Type element)
 		{
 			QName eqname = GetStaticQName (element);
-			
+
 			string ns = eqname.Namespace;
 			if (eqname.Namespace == MSSimpleNamespace)
 				//Arrays of Primitive types
@@ -604,10 +678,11 @@ namespace System.Runtime.Serialization
 
 		static QName GetSerializableQName (Type type)
 		{
-#if !NET_2_1
 			// First, check XmlSchemaProviderAttribute and try GetSchema() to see if it returns a schema in the expected format.
 			var xpa = type.GetCustomAttribute<XmlSchemaProviderAttribute> (true);
 			if (xpa != null) {
+				if (xpa.IsAny)
+					return XmlQualifiedName.Empty;
 				var mi = type.GetMethod (xpa.MethodName, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static);
 				if (mi != null) {
 					try {
@@ -618,7 +693,6 @@ namespace System.Runtime.Serialization
 					}
 				}
 			}
-#endif
 
 			string xmlName = type.Name;
 			if (type.IsGenericType) {
@@ -641,13 +715,11 @@ namespace System.Runtime.Serialization
 				return false;
 			if (Type.GetTypeCode (type) != TypeCode.Object) // explicitly primitive
 				return true;
-			if (type == typeof (Guid) || type == typeof (object) || type == typeof(TimeSpan) || type == typeof(byte[]) || type==typeof(Uri)) // special primitives
+			if (type == typeof (Guid) || type == typeof (object) || type == typeof(TimeSpan) || type == typeof(byte[]) || type == typeof(Uri) || type == typeof(DateTimeOffset)) // special primitives
 				return true;
-#if !MOONLIGHT
 			// DOM nodes
 			if (type == typeof (XmlElement) || type == typeof (XmlNode []))
 				return true;
-#endif
 			// nullable
 			if (type.IsGenericType && type.GetGenericTypeDefinition () == typeof (Nullable<>))
 				return IsPrimitiveNotEnum (type.GetGenericArguments () [0]);
@@ -711,7 +783,7 @@ namespace System.Runtime.Serialization
 				if (i.IsGenericType && i.GetGenericTypeDefinition ().Equals (typeof (IEnumerable<>)))
 					return i.GetGenericArguments () [0];
 			foreach (Type i in ifaces)
-				if (i == typeof (IList))
+				if (i == typeof (IEnumerable))
 					return typeof (object);
 			return null;
 		}
@@ -730,13 +802,15 @@ namespace System.Runtime.Serialization
 
 			Type element = GetCollectionElementType (type);
 			if (element == null)
-				throw new InvalidOperationException (String.Format ("Type '{0}' is marked as collection contract, but it is not a collection", type));
+				throw new InvalidDataContractException (String.Format ("Type '{0}' is marked as collection contract, but it is not a collection", type));
+			if (type.GetMethod ("Add", new Type[] { element }) == null)
+				throw new InvalidDataContractException (String.Format ("Type '{0}' is marked as collection contract, but missing a public \"Add\" method", type));
 
 			TryRegister (element); // must be registered before the name conflict check.
 
 			QName qname = GetCollectionContractQName (type);
 			CheckStandardQName (qname);
-			var map = FindUserMap (qname);
+			var map = FindUserMap (qname, type);
 			if (map != null) {
 				var cmap = map as CollectionContractTypeMap;
 				if (cmap == null) // The runtime type may still differ (between array and other IList; see bug #670560)
@@ -756,9 +830,20 @@ namespace System.Runtime.Serialization
 
 			TryRegister (element);
 
+			/*
+			 * To qualify as a custom collection type, a type must have
+			 * a public parameterless constructor and an "Add" method
+			 * with the correct parameter type in addition to implementing
+			 * one of the collection interfaces.
+			 * 
+			 */
+
+			if (!type.IsArray && type.IsClass && !IsCustomCollectionType (type, element))
+				return null;
+
 			QName qname = GetCollectionQName (element);
 
-			var map = FindUserMap (qname);
+			var map = FindUserMap (qname, element);
 			if (map != null) {
 				var cmap = map as CollectionTypeMap;
 				if (cmap == null) // The runtime type may still differ (between array and other IList; see bug #670560)
@@ -772,6 +857,86 @@ namespace System.Runtime.Serialization
 			return ret;
 		}
 
+		static bool IsCustomCollectionType (Type type, Type elementType)
+		{
+			if (!type.IsClass)
+				return false;
+			if (type.GetConstructor (new Type [0]) == null)
+				return false;
+			if (type.GetMethod ("Add", new Type[] { elementType }) == null)
+				return false;
+
+			return true;
+		}
+
+		internal static bool IsInterchangeableCollectionType (Type contractType, Type graphType,
+		                                                      out QName collectionQName)
+		{
+			collectionQName = null;
+			if (GetAttribute<CollectionDataContractAttribute> (contractType) != null)
+				return false;
+
+			var type = contractType;
+			if (type.IsGenericType)
+				type = type.GetGenericTypeDefinition ();
+
+			var elementType = GetCollectionElementType (contractType);
+			if (elementType == null)
+				return false;
+			
+			if (contractType.IsArray) {
+				if (!graphType.IsArray || !elementType.Equals (graphType.GetElementType ()))
+					throw new InvalidCastException (String.Format ("Type '{0}' cannot be converted into '{1}'.", graphType.GetElementType (), elementType));
+			} else if (!contractType.IsInterface) {
+				if (GetAttribute<SerializableAttribute> (contractType) == null)
+					return false;
+
+				var graphElementType = GetCollectionElementType (graphType);
+				if (elementType != graphElementType)
+					return false;
+
+				if (!IsCustomCollectionType (contractType, elementType))
+					return false;
+			} else if (type.Equals (typeof (IEnumerable)) || type.Equals (typeof (IList)) ||
+			           type.Equals (typeof (ICollection))) {
+				if (!graphType.ImplementsInterface (contractType))
+					return false;
+			} else if (type.Equals (typeof (IEnumerable<>)) || type.Equals (typeof (IList<>)) ||
+			           type.Equals (typeof (ICollection<>))) {
+				var graphElementType = GetCollectionElementType (graphType);
+				if (graphElementType != elementType)
+					throw new InvalidCastException (String.Format (
+						"Cannot convert type '{0}' into '{1}'.", graphType, contractType));
+
+				if (!graphType.ImplementsInterface (contractType))
+					return false;
+			} else {
+				return false;
+			}
+
+			collectionQName = GetCollectionQName (elementType);
+			return true;
+		}
+
+		static bool ImplementsInterface (Type type, Type iface)
+		{
+			foreach (var i in type.GetInterfacesOrSelfInterface ())
+				if (iface == i)
+					return true;
+					
+			return false;
+		}
+
+
+		static bool TypeImplementsIEnumerable (Type type)
+		{
+			foreach (var iface in type.GetInterfacesOrSelfInterface ())
+				if (iface == typeof (IEnumerable) || (iface.IsGenericType && iface.GetGenericTypeDefinition () == typeof (IEnumerable<>)))
+					return true;
+			
+			return false;
+		}
+
 		static bool TypeImplementsIDictionary (Type type)
 		{
 			foreach (var iface in type.GetInterfacesOrSelfInterface ())
@@ -795,7 +960,7 @@ namespace System.Runtime.Serialization
 			TryRegister (ret.KeyType);
 			TryRegister (ret.ValueType);
 
-			var map = FindUserMap (ret.XmlName);
+			var map = FindUserMap (ret.XmlName, type);
 			if (map != null) {
 				var dmap = map as DictionaryTypeMap;
 				if (dmap == null) // The runtime type may still differ (between array and other IList; see bug #670560)
@@ -810,7 +975,7 @@ namespace System.Runtime.Serialization
 		{
 			QName qname = GetSerializableQName (type);
 
-			if (FindUserMap (qname) != null)
+			if (FindUserMap (qname, type) != null)
 				throw new InvalidOperationException (String.Format ("There is already a registered type for XML name {0}", qname));
 
 			SharedTypeMap ret = new SharedTypeMap (type, qname, this);
@@ -826,7 +991,7 @@ namespace System.Runtime.Serialization
 
 			QName qname = GetSerializableQName (type);
 
-			if (FindUserMap (qname) != null)
+			if (!QName.Empty.Equals (qname) && FindUserMap (qname, type) != null)
 				throw new InvalidOperationException (String.Format ("There is already a registered type for XML name {0}", qname));
 
 			XmlSerializableMap ret = new XmlSerializableMap (type, qname, this);
@@ -853,7 +1018,7 @@ namespace System.Runtime.Serialization
 			if (qname == null)
 				return null;
 			CheckStandardQName (qname);
-			if (FindUserMap (qname) != null)
+			if (FindUserMap (qname, type) != null)
 				throw new InvalidOperationException (String.Format ("There is already a registered type for XML name {0}", qname));
 
 			SharedContractMap ret = new SharedContractMap (type, qname, this);
@@ -890,7 +1055,7 @@ namespace System.Runtime.Serialization
 			if (qname == null)
 				return null;
 
-			if (FindUserMap (qname) != null)
+			if (FindUserMap (qname, type) != null)
 				throw new InvalidOperationException (String.Format ("There is already a registered type for XML name {0}", qname));
 
 			EnumMap ret =
@@ -900,4 +1065,3 @@ namespace System.Runtime.Serialization
 		}
 	}
 }
-#endif
