@@ -29,7 +29,6 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-#if NET_2_0
 using System;
 using System.Collections;
 using System.Collections.Generic;
@@ -78,7 +77,7 @@ namespace System.Runtime.Serialization
 	  exists (and raises InvalidOperationException if required).
 
 */
-	internal abstract class SerializationMap
+	internal abstract partial class SerializationMap
 	{
 		public const BindingFlags AllInstanceFlags =
 			BindingFlags.Public | BindingFlags.NonPublic |
@@ -106,14 +105,37 @@ namespace System.Runtime.Serialization
 
 			XmlName = qname;
 			Members = new List<DataMemberInfo> ();
+
+			foreach (var mi in type.GetMethods (AllInstanceFlags)) {
+				if (mi.GetCustomAttributes (typeof (OnDeserializingAttribute), false).Length > 0)
+					OnDeserializing = mi;
+				else if (mi.GetCustomAttributes (typeof (OnDeserializedAttribute), false).Length > 0)
+					OnDeserialized = mi;
+			}
 		}
 
+		public MethodInfo OnDeserializing { get; set; }
+		public MethodInfo OnDeserialized { get; set; }
+
 		public virtual bool OutputXsiType {
 			get { return true; }
 		}
 
 		public QName XmlName { get; set; }
 
+		public abstract bool IsContractAllowedType { get; }
+
+		protected void HandleId (XmlReader reader, XmlFormatterDeserializer deserializer, object instance)
+		{
+			HandleId (reader.GetAttribute ("Id", KnownTypeCollection.MSSimpleNamespace), deserializer, instance);
+		}
+		
+		protected void HandleId (string id, XmlFormatterDeserializer deserializer, object instance)
+		{
+			if (id != null)
+				deserializer.References.Add (id, instance);
+		}
+
 		public CollectionDataContractAttribute GetCollectionDataContractAttribute (Type type)
 		{
 			object [] atts = type.GetCustomAttributes (
@@ -136,110 +158,6 @@ namespace System.Runtime.Serialization
 			return (Type.GetTypeCode (type) != TypeCode.Object || type == typeof (object));
 		}
 
-#if !NET_2_1
-		/* Returns the XmlSchemaType AND adds it to @schemas */
-		public virtual XmlSchemaType GetSchemaType (XmlSchemaSet schemas, Dictionary<QName, XmlSchemaType> generated_schema_types)
-		{
-			if (IsPrimitive (RuntimeType))
-				return null;
-
-			if (generated_schema_types.ContainsKey (XmlName)) // Caching  
-				return generated_schema_types [XmlName] as XmlSchemaType;
-
-			XmlSchemaComplexType complex_type = null;
-
-			complex_type = new XmlSchemaComplexType ();
-			complex_type.Name = XmlName.Name;
-			generated_schema_types [XmlName] = complex_type;
-
-			if (RuntimeType.BaseType == typeof (object)) {
-				complex_type.Particle = GetSequence (schemas, generated_schema_types);
-			} else {
-				//Has a non-System.Object base class
-				XmlSchemaComplexContentExtension extension = new XmlSchemaComplexContentExtension ();
-				XmlSchemaComplexContent content = new XmlSchemaComplexContent ();
-
-				complex_type.ContentModel = content;
-				content.Content = extension;
-
-				KnownTypes.Add (RuntimeType.BaseType);
-				SerializationMap map = KnownTypes.FindUserMap (RuntimeType.BaseType);
-				//FIXME: map == null ?
-				map.GetSchemaType (schemas, generated_schema_types);
-
-				extension.Particle = GetSequence (schemas, generated_schema_types);
-				extension.BaseTypeName = GetQualifiedName (RuntimeType.BaseType);
-			}
-			
-			XmlSchemaElement schemaElement = GetSchemaElement (XmlName, complex_type);
-			XmlSchema schema = GetSchema (schemas, XmlName.Namespace);
-			schema.Items.Add (complex_type);
-			schema.Items.Add (schemaElement);
-			schemas.Reprocess (schema);
-
-			return complex_type;
-		}
-
-		/* Returns the <xs:sequence> for the data members */
-		XmlSchemaSequence GetSequence (XmlSchemaSet schemas,
-				Dictionary<QName, XmlSchemaType> generated_schema_types)
-		{
-			List<DataMemberInfo> members = GetMembers ();
-
-			XmlSchema schema = GetSchema (schemas, XmlName.Namespace);
-			XmlSchemaSequence sequence = new XmlSchemaSequence ();
-			foreach (DataMemberInfo dmi in members) {
-				// delegates are not supported.
-				if (!dmi.MemberType.IsAbstract && typeof (System.Delegate).IsAssignableFrom (dmi.MemberType))
-					continue;
-
-				XmlSchemaElement element = new XmlSchemaElement ();
-				element.Name = dmi.XmlName;
-
-				KnownTypes.Add (dmi.MemberType);
-				SerializationMap map = KnownTypes.FindUserMap (dmi.MemberType);
-				if (map != null) {
-					XmlSchemaType schema_type = map.GetSchemaType (schemas, generated_schema_types);
-					if (schema_type is XmlSchemaComplexType)
-						element.IsNillable = true;
-				} else {
-					//Primitive type
-					if (dmi.MemberType == typeof (string))
-						element.IsNillable = true;
-				}
-
-				element.MinOccurs = 0;
-
-				element.SchemaTypeName = GetQualifiedName (dmi.MemberType);
-				AddImport (schema, element.SchemaTypeName.Namespace);
-
-				sequence.Items.Add (element);
-			}
-
-			schemas.Reprocess (schema);
-			return sequence;
-		}
-
-		//FIXME: Replace with a dictionary ?
-		void AddImport (XmlSchema schema, string ns)
-		{
-			if (ns == XmlSchema.Namespace || schema.TargetNamespace == ns)
-				return;
-
-			foreach (XmlSchemaObject o in schema.Includes) {
-				XmlSchemaImport import = o as XmlSchemaImport;
-				if (import == null)
-					continue;
-				if (import.Namespace == ns)
-					return;
-			}
-
-			XmlSchemaImport imp = new XmlSchemaImport ();
-			imp.Namespace = ns;
-			schema.Includes.Add (imp);
-		}
-#endif
-
 		//Returns list of data members for this type ONLY
 		public virtual List<DataMemberInfo> GetMembers ()
 		{
@@ -296,16 +214,9 @@ namespace System.Runtime.Serialization
 		public virtual void Serialize (object graph,
 			XmlFormatterSerializer serializer)
 		{
-			string label = null;
-			if (IsReference) {
-				label = (string) serializer.References [graph];
-				if (label != null) {
-					serializer.Writer.WriteAttributeString ("z", "Ref", KnownTypeCollection.MSSimpleNamespace, label);
-					return;
-				}
-				label = "i" + (serializer.References.Count + 1);
-				serializer.References.Add (graph, label);
-			}
+			string label;
+			if (serializer.TrySerializeAsReference (IsReference, graph, out label))
+				return;
 			else if (serializer.SerializingObjects.Contains (graph))
 				throw new SerializationException (String.Format ("Circular reference of an object in the object graph was found: '{0}' of type {1}", graph, graph.GetType ()));
 			serializer.SerializingObjects.Add (graph);
@@ -340,20 +251,21 @@ namespace System.Runtime.Serialization
 		public virtual object DeserializeObject (XmlReader reader, XmlFormatterDeserializer deserializer)
 		{
 			bool isEmpty = reader.IsEmptyElement;
+			string id = reader.GetAttribute ("Id", KnownTypeCollection.MSSimpleNamespace);
 			reader.ReadStartElement ();
 			reader.MoveToContent ();
 
 			object res;
 
 			if (isEmpty)
-				res = DeserializeEmptyContent (reader, deserializer);
+				res = DeserializeEmptyContent (reader, deserializer, id);
 			else
-				res = DeserializeContent (reader, deserializer);
+				res = DeserializeContent (reader, deserializer, id);
 
 			reader.MoveToContent ();
 			if (!isEmpty && reader.NodeType == XmlNodeType.EndElement)
 				reader.ReadEndElement ();
-			else if (!isEmpty && reader.NodeType != XmlNodeType.None) {
+			else if (!isEmpty && !reader.EOF && reader.NodeType != XmlNodeType.EndElement) {
 				var li = reader as IXmlLineInfo;
 				throw new SerializationException (String.Format ("Deserializing type '{3}'. Expecting state 'EndElement'. Encountered state '{0}' with name '{1}' with namespace '{2}'.{4}",
 					reader.NodeType,
@@ -368,53 +280,69 @@ namespace System.Runtime.Serialization
 		// This is sort of hack. The argument reader already moved ahead of
 		// the actual empty element.It's just for historical consistency.
 		public virtual object DeserializeEmptyContent (XmlReader reader,
-			XmlFormatterDeserializer deserializer)
+			XmlFormatterDeserializer deserializer, string id)
 		{
-			return DeserializeContent (reader, deserializer, true);
+			return DeserializeContent (reader, deserializer, id, true);
 		}
 
 		public virtual object DeserializeContent (XmlReader reader,
-			XmlFormatterDeserializer deserializer)
+			XmlFormatterDeserializer deserializer, string id)
 		{
-			return DeserializeContent (reader, deserializer, false);
+			return DeserializeContent (reader, deserializer, id, false);
 		}
 
-		object DeserializeContent (XmlReader reader,
-			XmlFormatterDeserializer deserializer, bool empty)
+		object DeserializeContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id, bool empty)
 		{
 			object instance = FormatterServices.GetUninitializedObject (RuntimeType);
+			HandleId (id, deserializer, instance);
+
+			if (OnDeserializing != null)
+				OnDeserializing.Invoke (instance, new object [] {new StreamingContext (StreamingContextStates.All)});
+
 			int depth = reader.NodeType == XmlNodeType.None ? reader.Depth : reader.Depth - 1;
 			bool [] filled = new bool [Members.Count];
+			bool [] nsmatched = new bool [Members.Count];
 			int memberInd = -1, ordered = -1;
 			while (!empty && reader.NodeType == XmlNodeType.Element && reader.Depth > depth) {
 				DataMemberInfo dmi = null;
 				int i = 0;
+				bool nsmatchedOne = false;
 				for (; i < Members.Count; i++) { // unordered
 					if (Members [i].Order >= 0)
 						break;
-					if (reader.LocalName == Members [i].XmlName &&
-						reader.NamespaceURI == Members [i].XmlRootNamespace) {
+					if (reader.LocalName == Members [i].XmlName) {
 						memberInd = i;
 						dmi = Members [i];
-						break;
+						nsmatchedOne = (dmi.XmlRootNamespace == null || reader.NamespaceURI == dmi.XmlRootNamespace);
+						if (nsmatchedOne)
+							break;
 					}
 				}
 				for (i = Math.Max (i, ordered); i < Members.Count; i++) { // ordered
 					if (dmi != null)
 						break;
-					if (reader.LocalName == Members [i].XmlName &&
-						reader.NamespaceURI == Members [i].XmlRootNamespace) {
-						memberInd = i;
+					if (reader.LocalName == Members [i].XmlName) {
 						ordered = i;
+						memberInd = i;
 						dmi = Members [i];
-						break;
+						nsmatchedOne = (dmi.XmlRootNamespace == null || reader.NamespaceURI == dmi.XmlRootNamespace);
+						if (nsmatchedOne)
+							break;
 					}
 				}
-
+				
 				if (dmi == null) {
 					reader.Skip ();
+					reader.MoveToContent ();
 					continue;
 				}
+				if (filled [memberInd] && nsmatched [memberInd]) {
+					// The strictly-corresponding member (i.e. that matches namespace URI too, not only local name) already exists, so skip this element.
+					reader.Skip ();
+					reader.MoveToContent ();
+					continue;
+				}
+				nsmatched [memberInd] = nsmatchedOne;
 				SetValue (dmi, instance, deserializer.Deserialize (dmi.MemberType, reader));
 				filled [memberInd] = true;
 				reader.MoveToContent ();
@@ -423,6 +351,9 @@ namespace System.Runtime.Serialization
 				if (!filled [i] && Members [i].IsRequired)
 					throw MissingRequiredMember (Members [i], reader);
 
+			if (OnDeserialized != null)
+				OnDeserialized.Invoke (instance, new object [] {new StreamingContext (StreamingContextStates.All)});
+
 			return instance;
 		}
 
@@ -449,20 +380,22 @@ namespace System.Runtime.Serialization
 			}
 		}
 
-		protected DataMemberInfo CreateDataMemberInfo (DataMemberAttribute dma, MemberInfo mi, Type type)
+		protected DataMemberInfo CreateDataMemberInfo (DataMemberAttribute dma, MemberInfo mi, Type memberType, string ownerNamespace)
 		{
-			KnownTypes.Add (type);
-			QName qname = KnownTypes.GetQName (type);
-			string rootNamespace = KnownTypes.GetQName (mi.DeclaringType).Namespace;
-			if (KnownTypeCollection.GetPrimitiveTypeFromName (qname.Name) != null)
-				return new DataMemberInfo (mi, dma, rootNamespace, null);
+			KnownTypes.Add (memberType);
+			QName qname = KnownTypes.GetQName (memberType);
+			
+			if (KnownTypeCollection.GetPrimitiveTypeFromName (qname) != null)
+				return new DataMemberInfo (mi, dma, ownerNamespace, null);
 			else
-				return new DataMemberInfo (mi, dma, rootNamespace, qname.Namespace);
+				return new DataMemberInfo (mi, dma, ownerNamespace, qname.Namespace);
 		}
 	}
 
-	internal class XmlSerializableMap : SerializationMap
+	internal partial class XmlSerializableMap : SerializationMap
 	{
+		public override bool IsContractAllowedType { get { return true; } }
+
 		public XmlSerializableMap (Type type, QName qname, KnownTypeCollection knownTypes)
 			: base (type, qname, knownTypes)
 		{
@@ -480,21 +413,20 @@ namespace System.Runtime.Serialization
 
 		public override object DeserializeObject (XmlReader reader, XmlFormatterDeserializer deserializer)
 		{
-			IXmlSerializable ixs = (IXmlSerializable) FormatterServices.GetUninitializedObject (RuntimeType);
+#if NET_2_1
+			IXmlSerializable ixs = (IXmlSerializable) Activator.CreateInstance (RuntimeType);
+#else
+			IXmlSerializable ixs = (IXmlSerializable) Activator.CreateInstance (RuntimeType, true);
+#endif
+
+			HandleId (reader, deserializer, ixs);
+
 			ixs.ReadXml (reader);
 			return ixs;
 		}
-
-#if !NET_2_1
-		// FIXME: verify return value sanity.
-		public override XmlSchemaType GetSchemaType (XmlSchemaSet schemas, Dictionary<QName, XmlSchemaType> generated_schema_types)
-		{
-			return null;
-		}
-#endif
 	}
 
-	internal class SharedContractMap : SerializationMap
+	internal partial class SharedContractMap : SerializationMap
 	{
 		public SharedContractMap (
 			Type type, QName qname, KnownTypeCollection knownTypes)
@@ -502,44 +434,42 @@ namespace System.Runtime.Serialization
 		{
 		}
 
+		public override bool IsContractAllowedType { get { return true; } }
+
 		internal void Initialize ()
 		{
-			Type baseType = RuntimeType;
-			List <DataMemberInfo> members = new List <DataMemberInfo> ();
-			object [] atts = baseType.GetCustomAttributes (
+			Type type = RuntimeType;
+			List <DataMemberInfo> members;
+			object [] atts = type.GetCustomAttributes (
 				typeof (DataContractAttribute), false);
 			IsReference = atts.Length > 0 ? (((DataContractAttribute) atts [0]).IsReference) : false;
 
-			while (baseType != null) {
-				QName bqname = KnownTypes.GetQName (baseType);
-					
-				members = GetMembers (baseType, bqname, true);
+			while (type != null) {
+				members = GetMembers (type);
 				members.Sort (DataMemberInfo.DataMemberInfoComparer.Instance);
 				Members.InsertRange (0, members);
 				members.Clear ();
 
-				baseType = baseType.BaseType;
+				type = type.BaseType;
 			}
 		}
 
-		List<DataMemberInfo> GetMembers (Type type, QName qname, bool declared_only)
+		List<DataMemberInfo> GetMembers (Type type)
 		{
 			List<DataMemberInfo> data_members = new List<DataMemberInfo> ();
-			BindingFlags flags = AllInstanceFlags;
-			if (declared_only)
-				flags |= BindingFlags.DeclaredOnly;
+			BindingFlags flags = AllInstanceFlags | BindingFlags.DeclaredOnly;
 
 			foreach (PropertyInfo pi in type.GetProperties (flags)) {
 				DataMemberAttribute dma =
 					GetDataMemberAttribute (pi);
 				if (dma == null)
 					continue;
-				KnownTypes.TryRegister (pi.PropertyType);
+				KnownTypes.Add (pi.PropertyType);
 				var map = KnownTypes.FindUserMap (pi.PropertyType);
 				if (!pi.CanRead || (!pi.CanWrite && !(map is ICollectionTypeMap)))
 					throw new InvalidDataContractException (String.Format (
 							"DataMember property '{0}' on type '{1}' must have both getter and setter.", pi, pi.DeclaringType));
-				data_members.Add (CreateDataMemberInfo (dma, pi, pi.PropertyType));
+				data_members.Add (CreateDataMemberInfo (dma, pi, pi.PropertyType, KnownTypeCollection.GetStaticQName (pi.DeclaringType).Namespace));
 			}
 
 			foreach (FieldInfo fi in type.GetFields (flags)) {
@@ -547,7 +477,7 @@ namespace System.Runtime.Serialization
 					GetDataMemberAttribute (fi);
 				if (dma == null)
 					continue;
-				data_members.Add (CreateDataMemberInfo (dma, fi, fi.FieldType));
+				data_members.Add (CreateDataMemberInfo (dma, fi, fi.FieldType, KnownTypeCollection.GetStaticQName (fi.DeclaringType).Namespace));
 			}
 
 			return data_members;
@@ -559,10 +489,16 @@ namespace System.Runtime.Serialization
 		}
 	}
 
-	internal class DefaultTypeMap : SerializationMap
+	internal partial class DefaultTypeMap : SerializationMap
 	{
 		public DefaultTypeMap (Type type, KnownTypeCollection knownTypes)
-			: base (type, KnownTypeCollection.GetContractQName (type, null, null), knownTypes)
+			: base (type, KnownTypeCollection.GetStaticQName (type), knownTypes)
+		{
+		}
+
+		public override bool IsContractAllowedType { get { return false; } }
+
+		internal void Initialize ()
 		{
 			Members.AddRange (GetDefaultMembers ());
 		}
@@ -581,35 +517,43 @@ namespace System.Runtime.Serialization
 					continue;
 				if (mi.GetCustomAttributes (typeof (IgnoreDataMemberAttribute), false).Length != 0)
 					continue;
-				l.Add (new DataMemberInfo (mi, new DataMemberAttribute (), null, null));
+				string ns = KnownTypeCollection.GetStaticQName (mi.DeclaringType).Namespace;
+				l.Add (CreateDataMemberInfo (new DataMemberAttribute (), mi, mt, ns));
 			}
 			l.Sort (DataMemberInfo.DataMemberInfoComparer.Instance);
 			return l;
 		}
 	}
 
-	// FIXME: it still needs to consider ItemName/KeyName/ValueName
+	// FIXME: it still needs to consider KeyName/ValueName
 	// (especially Dictionary collection is not likely considered yet.)
-	internal class CollectionContractTypeMap : CollectionTypeMap
+	internal partial class CollectionContractTypeMap : CollectionTypeMap
 	{
+		CollectionDataContractAttribute a;
+
 		public CollectionContractTypeMap (
 			Type type, CollectionDataContractAttribute a, Type elementType,
 			QName qname, KnownTypeCollection knownTypes)
 			: base (type, elementType, qname, knownTypes)
 		{
+			this.a = a;
 			IsReference = a.IsReference;
+			if (!String.IsNullOrEmpty (a.ItemName))
+				element_qname = new XmlQualifiedName (a.ItemName, a.Namespace ?? CurrentNamespace);
 		}
 
 		internal override string CurrentNamespace {
 			get { return XmlName.Namespace; }
 		}
+
+		public override bool IsContractAllowedType { get { return true; } }
 	}
 
 	internal interface ICollectionTypeMap
 	{
 	}
 
-	internal class CollectionTypeMap : SerializationMap, ICollectionTypeMap
+	internal partial class CollectionTypeMap : SerializationMap, ICollectionTypeMap
 	{
 		Type element_type;
 		internal QName element_qname;
@@ -641,13 +585,15 @@ namespace System.Runtime.Serialization
 
 		static Type GetGenericCollectionInterface (Type type)
 		{
-			foreach (var iface in type.GetInterfaces ())
-				if (iface.IsGenericType && iface.GetGenericTypeDefinition () == typeof (ICollection<>))
+			foreach (var iface in type.GetInterfacesOrSelfInterface ())
+				if (iface.IsGenericType && iface.GetGenericTypeDefinition () == typeof (IEnumerable<>))
 					return iface;
 
 			return null;
 		}
 
+		public override bool IsContractAllowedType { get { return false; } }
+
 		public override bool OutputXsiType {
 			get { return false; }
 		}
@@ -664,6 +610,9 @@ namespace System.Runtime.Serialization
 		public override void SerializeNonReference (object graph,
 			XmlFormatterSerializer serializer)
 		{
+			// output item xmlns in advance so that it does not result in excessive xmlns overwrites.
+			if (XmlName.Namespace != element_qname.Namespace && element_qname.Namespace != KnownTypeCollection.MSSimpleNamespace)
+				serializer.Writer.WriteXmlnsAttribute (null, element_qname.Namespace);
 
 			foreach (object o in (IEnumerable) graph) {
 				serializer.WriteStartElement (element_qname.Name, XmlName.Namespace, CurrentNamespace);
@@ -690,14 +639,29 @@ namespace System.Runtime.Serialization
 #endif
 		}
 
-		public override object DeserializeEmptyContent (XmlReader reader, XmlFormatterDeserializer deserializer)
+		public override object DeserializeEmptyContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id)
 		{
-			return CreateInstance ();
+			var instance = CreateInstance ();
+			HandleId (id, deserializer, instance);
+			if (OnDeserializing != null)
+				OnDeserializing.Invoke (instance, new object [] {new StreamingContext (StreamingContextStates.All)});
+			try {
+				if (RuntimeType.IsArray)
+					return ((ArrayList)instance).ToArray (element_type);
+				else
+					return instance;
+			} finally {
+				if (OnDeserialized != null)
+					OnDeserialized.Invoke (instance, new object [] {new StreamingContext (StreamingContextStates.All)});
+			}
 		}
 
-		public override object DeserializeContent (XmlReader reader, XmlFormatterDeserializer deserializer)
+		public override object DeserializeContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id)
 		{
 			object instance = CreateInstance ();
+			HandleId (id, deserializer, instance);
+			if (OnDeserializing != null)
+				OnDeserializing.Invoke (instance, new object [] {new StreamingContext (StreamingContextStates.All)});
 			int depth = reader.NodeType == XmlNodeType.None ? reader.Depth : reader.Depth - 1;
 			while (reader.NodeType == XmlNodeType.Element && reader.Depth > depth) {
 				object elem = deserializer.Deserialize (element_type, reader);
@@ -709,9 +673,14 @@ namespace System.Runtime.Serialization
 					throw new NotImplementedException (String.Format ("Type {0} is not supported", RuntimeType));
 				reader.MoveToContent ();
 			}
-			if (RuntimeType.IsArray)
-				return ((ArrayList)instance).ToArray (element_type);
-			return instance;
+			try {
+				if (RuntimeType.IsArray)
+					return ((ArrayList)instance).ToArray (element_type);
+				return instance;
+			} finally {
+				if (OnDeserialized != null)
+					OnDeserialized.Invoke (instance, new object [] {new StreamingContext (StreamingContextStates.All)});
+			}
 		}
 
 		public override List<DataMemberInfo> GetMembers ()
@@ -719,56 +688,12 @@ namespace System.Runtime.Serialization
 			//Shouldn't come here at all!
 			throw new NotImplementedException ();
 		}
-		
-#if !NET_2_1
-		public override XmlSchemaType GetSchemaType (XmlSchemaSet schemas, Dictionary<QName, XmlSchemaType> generated_schema_types)
-		{
-			if (generated_schema_types.ContainsKey (XmlName))
-				return null;
-
-			if (generated_schema_types.ContainsKey (XmlName))
-				return generated_schema_types [XmlName];
-
-			QName element_qname = GetQualifiedName (element_type);
-
-			XmlSchemaComplexType complex_type = new XmlSchemaComplexType ();
-			complex_type.Name = XmlName.Name;
-
-			XmlSchemaSequence sequence = new XmlSchemaSequence ();
-			XmlSchemaElement element = new XmlSchemaElement ();
-
-			element.MinOccurs = 0;
-			element.MaxOccursString = "unbounded";
-			element.Name = element_qname.Name;
-
-			KnownTypes.Add (element_type);
-			SerializationMap map = KnownTypes.FindUserMap (element_type);
-			if (map != null) {// non-primitive type
-				map.GetSchemaType (schemas, generated_schema_types);
-				element.IsNillable = true;
-			}
-
-			element.SchemaTypeName = element_qname;
-
-			sequence.Items.Add (element);
-			complex_type.Particle = sequence;
-
-			XmlSchema schema = GetSchema (schemas, XmlName.Namespace);
-			schema.Items.Add (complex_type);
-			schema.Items.Add (GetSchemaElement (XmlName, complex_type));
-			schemas.Reprocess (schema);
-
-			generated_schema_types [XmlName] = complex_type;
-
-			return complex_type;
-		}
-#endif
 	}
 
-	internal class DictionaryTypeMap : SerializationMap, ICollectionTypeMap
+	internal partial class DictionaryTypeMap : SerializationMap, ICollectionTypeMap
 	{
 		Type key_type, value_type;
-		QName dict_qname, item_qname, key_qname, value_qname;
+		QName item_qname, key_qname, value_qname;
 		MethodInfo add_method;
 		CollectionDataContractAttribute a;
 
@@ -804,7 +729,7 @@ namespace System.Runtime.Serialization
 
 		static Type GetGenericDictionaryInterface (Type type)
 		{
-			foreach (var iface in type.GetInterfaces ())
+			foreach (var iface in type.GetInterfacesOrSelfInterface ())
 				if (iface.IsGenericType && iface.GetGenericTypeDefinition () == typeof (IDictionary<,>))
 					return iface;
 
@@ -815,38 +740,53 @@ namespace System.Runtime.Serialization
 			get { return a != null && !String.IsNullOrEmpty (a.Namespace) ? a.Namespace : KnownTypeCollection.MSArraysNamespace; }
 		}
 
+		public override bool IsContractAllowedType { get { return a != null; } }
+
 		public Type KeyType { get { return key_type; } }
 		public Type ValueType { get { return value_type; } }
 
-		static readonly QName kvpair_key_qname = new QName ("Key", KnownTypeCollection.MSArraysNamespace);
-		static readonly QName kvpair_value_qname = new QName ("Value", KnownTypeCollection.MSArraysNamespace);
-
 		internal virtual QName GetDictionaryQName ()
 		{
-			if (a != null && !String.IsNullOrEmpty (a.Name))
-				return new QName (a.Name, ContractNamespace);
-			return new QName ("ArrayOf" + GetItemQName ().Name, KnownTypeCollection.MSArraysNamespace);
+			string name = a != null ? a.Name : null;
+			string ns = a != null ? a.Namespace : null;
+			if (RuntimeType.IsGenericType && RuntimeType.GetGenericTypeDefinition () != typeof (Dictionary<,>))
+				name = name ?? KnownTypeCollection.GetDefaultName (RuntimeType);
+			else
+				name = "ArrayOf" + GetItemQName ().Name;
+			ns = ns ?? KnownTypeCollection.MSArraysNamespace;
+
+			return new QName (name, ns);
 		}
 
 		internal virtual QName GetItemQName ()
 		{
-			if (a != null && !String.IsNullOrEmpty (a.ItemName))
-				return new QName (a.ItemName, ContractNamespace);
-			return new QName ("KeyValueOf" + KnownTypes.GetQName (key_type).Name + KnownTypes.GetQName (value_type).Name, KnownTypeCollection.MSArraysNamespace);
+			string name = a != null ? a.ItemName : null;
+			string ns = a != null ? a.Namespace : null;
+
+			name = name ?? "KeyValueOf" + KnownTypes.GetQName (key_type).Name + KnownTypes.GetQName (value_type).Name;
+			ns = ns ?? (a != null ? ContractNamespace : KnownTypeCollection.MSArraysNamespace);
+
+			return new QName (name, ns);
 		}
 
 		internal virtual QName GetKeyQName ()
 		{
-			if (a != null && !String.IsNullOrEmpty (a.KeyName))
-				return new QName (a.KeyName, ContractNamespace);
-			return kvpair_key_qname;
+			string name = a != null ? a.KeyName : null;
+			string ns = a != null ? a.Namespace : null;
+
+			name = name ?? "Key";
+			ns = ns ?? (a != null ? ContractNamespace : KnownTypeCollection.MSArraysNamespace);
+			return new QName (name, ns);
 		}
 
 		internal virtual QName GetValueQName ()
 		{
-			if (a != null && !String.IsNullOrEmpty (a.ValueName))
-				return new QName (a.ValueName, ContractNamespace);
-			return kvpair_value_qname;
+			string name = a != null ? a.ValueName : null;
+			string ns = a != null ? a.Namespace : null;
+
+			name = name ?? "Value";
+			ns = ns ?? (a != null ? ContractNamespace : KnownTypeCollection.MSArraysNamespace);
+			return new QName (name, ns);
 		}
 
 		internal virtual string CurrentNamespace {
@@ -911,14 +851,17 @@ namespace System.Runtime.Serialization
 #endif
 		}
 
-		public override object DeserializeEmptyContent (XmlReader reader, XmlFormatterDeserializer deserializer)
-		{
-			return DeserializeContent (reader, deserializer);
-		}
-
-		public override object DeserializeContent(XmlReader reader, XmlFormatterDeserializer deserializer)
+		public override object DeserializeEmptyContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id)
 		{
 			object instance = CreateInstance ();
+			HandleId (id, deserializer, instance);
+			return instance;
+		}
+
+		public override object DeserializeContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id)
+		{
+			object instance = CreateInstance ();
+			HandleId (id, deserializer, instance);
 			int depth = reader.NodeType == XmlNodeType.None ? reader.Depth : reader.Depth - 1;
 			while (reader.NodeType == XmlNodeType.Element && reader.Depth > depth) {
 				if (reader.IsEmptyElement)
@@ -929,7 +872,9 @@ namespace System.Runtime.Serialization
 				object key = deserializer.Deserialize (key_type, reader);
 				reader.MoveToContent ();
 				object val = deserializer.Deserialize (value_type, reader);
+				reader.MoveToContent ();
 				reader.ReadEndElement (); // of pair
+				reader.MoveToContent ();
 
 				if (instance is IDictionary)
 					((IDictionary)instance).Add (key, val);
@@ -946,41 +891,40 @@ namespace System.Runtime.Serialization
 			//Shouldn't come here at all!
 			throw new NotImplementedException ();
 		}
-		
-#if !NET_2_1
-		public override XmlSchemaType GetSchemaType (XmlSchemaSet schemas, Dictionary<QName, XmlSchemaType> generated_schema_types)
-		{
-			throw new NotImplementedException ();
-		}
-#endif
 	}
 
-	internal class SharedTypeMap : SerializationMap
+	internal partial class SharedTypeMap : SerializationMap
 	{
 		public SharedTypeMap (
 			Type type, QName qname, KnownTypeCollection knownTypes)
 			: base (type, qname, knownTypes)
 		{
-			Members = GetMembers (type, XmlName, false);
+		}
+
+		public override bool IsContractAllowedType { get { return true; } }
+
+		public void Initialize ()
+		{
+			Members = GetMembers (RuntimeType, XmlName, false);
 		}
 
 		List<DataMemberInfo> GetMembers (Type type, QName qname, bool declared_only)
 		{
 			List<DataMemberInfo> data_members = new List<DataMemberInfo> ();
-			BindingFlags flags = AllInstanceFlags;
-			if (declared_only)
-				flags |= BindingFlags.DeclaredOnly;
+			BindingFlags flags = AllInstanceFlags | BindingFlags.DeclaredOnly;
 			
-			foreach (FieldInfo fi in type.GetFields (flags)) {
-				if (fi.GetCustomAttributes (
-					typeof (NonSerializedAttribute),
-					false).Length > 0)
-					continue;
+			for (Type t = type; t != null; t = t.BaseType) {
+				foreach (FieldInfo fi in t.GetFields (flags)) {
+					if (fi.GetCustomAttributes (
+						typeof (NonSerializedAttribute),
+						false).Length > 0)
+						continue;
 
-				if (fi.IsInitOnly)
-					throw new InvalidDataContractException (String.Format ("DataMember field {0} must not be read-only.", fi));
-				DataMemberAttribute dma = new DataMemberAttribute ();
-				data_members.Add (CreateDataMemberInfo (dma, fi, fi.FieldType));
+					if (fi.IsInitOnly)
+						throw new InvalidDataContractException (String.Format ("DataMember field {0} must not be read-only.", fi));
+					DataMemberAttribute dma = new DataMemberAttribute ();
+					data_members.Add (CreateDataMemberInfo (dma, fi, fi.FieldType, KnownTypeCollection.GetStaticQName (fi.DeclaringType).Namespace));
+				}
 			}
 
 			data_members.Sort (DataMemberInfo.DataMemberInfoComparer.Instance); // alphabetic order.
@@ -996,7 +940,7 @@ namespace System.Runtime.Serialization
 		}
 	}
 
-	internal class EnumMap : SerializationMap
+	internal partial class EnumMap : SerializationMap
 	{
 		List<EnumMemberInfo> enum_members;
 		bool flag_attr;
@@ -1031,6 +975,8 @@ namespace System.Runtime.Serialization
 			}
 		}
 
+		public override bool IsContractAllowedType { get { return false; } }
+
 		private EnumMemberAttribute GetEnumMemberAttribute (
 			MemberInfo mi)
 		{
@@ -1041,45 +987,30 @@ namespace System.Runtime.Serialization
 			return (EnumMemberAttribute) atts [0];
 		}
 
-#if !NET_2_1
-		public override XmlSchemaType GetSchemaType (XmlSchemaSet schemas, Dictionary<QName, XmlSchemaType> generated_schema_types)
-		{
-			if (generated_schema_types.ContainsKey (XmlName))
-				return generated_schema_types [XmlName];
-
-			XmlSchemaSimpleType simpleType = new XmlSchemaSimpleType ();
-			simpleType.Name = XmlName.Name;
-
-			XmlSchemaSimpleTypeRestriction simpleRestriction = new XmlSchemaSimpleTypeRestriction ();
-			simpleType.Content = simpleRestriction;
-			simpleRestriction.BaseTypeName = new XmlQualifiedName ("string", XmlSchema.Namespace);
-
-			foreach (EnumMemberInfo emi in enum_members) {
-				XmlSchemaEnumerationFacet e = new XmlSchemaEnumerationFacet ();
-				e.Value = emi.XmlName;
-				simpleRestriction.Facets.Add (e);
-			}
-
-			generated_schema_types [XmlName] = simpleType;
-			
-			XmlSchema schema = GetSchema (schemas, XmlName.Namespace);
-			XmlSchemaElement element = GetSchemaElement (XmlName, simpleType);
-			element.IsNillable = true;
-
-			schema.Items.Add (simpleType);
-			schema.Items.Add (element);
-
-			return simpleType;
-		}
-#endif
-
 		public override void Serialize (object graph,
 			XmlFormatterSerializer serializer)
 		{
-			foreach (EnumMemberInfo emi in enum_members) {
-				if (Enum.Equals (emi.Value, graph)) {
-					serializer.Writer.WriteString (emi.XmlName);
-					return;
+			if (flag_attr) {
+				long val = Convert.ToInt64 (graph);
+				string s = null;
+				foreach (EnumMemberInfo emi in enum_members) {
+					long f = Convert.ToInt64 (emi.Value);
+					if (f == val) {
+						s = emi.XmlName;
+						break;
+					}
+					if ((f & val) == f)
+						s += (s != null ? " " : String.Empty) + emi.XmlName;
+				}
+				if (s != null)
+					serializer.Writer.WriteString (s);
+				return;
+			} else {
+				foreach (EnumMemberInfo emi in enum_members) {
+					if (Enum.Equals (emi.Value, graph)) {
+						serializer.Writer.WriteString (emi.XmlName);
+						return;
+					}
 				}
 			}
 
@@ -1087,23 +1018,38 @@ namespace System.Runtime.Serialization
 				"Enum value '{0}' is invalid for type '{1}' and cannot be serialized.", graph, RuntimeType));
 		}
 
-		public override object DeserializeEmptyContent (XmlReader reader,
-			XmlFormatterDeserializer deserializer)
+		public override object DeserializeEmptyContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id)
 		{
 			if (!flag_attr)
 				throw new SerializationException (String.Format ("Enum value '' is invalid for type '{0}' and cannot be deserialized.", RuntimeType));
-			return Enum.ToObject (RuntimeType, 0);
+			var instance = Enum.ToObject (RuntimeType, 0);
+			HandleId (id, deserializer, instance);
+			return instance;
 		}
 
-		public override object DeserializeContent (XmlReader reader,
-			XmlFormatterDeserializer deserializer)
+		public override object DeserializeContent (XmlReader reader, XmlFormatterDeserializer deserializer, string id)
 		{
 			string value = reader.NodeType != XmlNodeType.Text ? String.Empty : reader.ReadContentAsString ();
+			HandleId (id, deserializer, value);
 
 			if (value != String.Empty) {
-				foreach (EnumMemberInfo emi in enum_members)
-					if (emi.XmlName == value)
-						return emi.Value;
+				if (flag_attr && value.IndexOf (' ') != -1) {
+					long flags = 0L;
+					foreach (string flag in value.Split (' ')) {
+						foreach (EnumMemberInfo emi in enum_members) {
+							if (emi.XmlName == flag) {
+								flags |= Convert.ToInt64 (emi.Value);
+								break;
+							}
+						}
+					}
+					return Enum.ToObject (RuntimeType, flags);
+				}
+				else {
+					foreach (EnumMemberInfo emi in enum_members)
+						if (emi.XmlName == value)
+							return emi.Value;
+				}
 			}
 
 			if (!flag_attr)
@@ -1141,7 +1087,7 @@ namespace System.Runtime.Serialization
 			Order = dma.Order;
 			Member = member;
 			IsRequired = dma.IsRequired;
-			XmlName = dma.Name != null ? dma.Name : member.Name;
+			XmlName = XmlConvert.EncodeLocalName (dma.Name != null ? dma.Name : member.Name);
 			XmlNamespace = ns;
 			XmlRootNamespace = rootNamespce;
 			if (Member is FieldInfo)
@@ -1174,4 +1120,3 @@ namespace System.Runtime.Serialization
 		}
 	}
 }
-#endif
