@@ -29,7 +29,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 using System.IO;
 using System.Xml.Schema;
@@ -142,7 +141,7 @@ namespace System.Web.Services.Description
 
 		public override void Check (ConformanceCheckContext ctx, BindingCollection value) {
 			foreach (Binding b in value)
-				foreach (ServiceDescriptionFormatExtension ext in b.Extensions)
+				foreach (object ext in b.Extensions)
 					if (ext.GetType () == typeof (SoapBinding))
 						return;
 
@@ -424,7 +423,7 @@ namespace System.Web.Services.Description
 				if (msg == null)
 					continue; // it is doubtful, but R2305 is not to check such cases anyways.
 				foreach (MessagePart p in msg.Parts) {
-					if (Array.IndexOf (order, p.Name) >= 0)
+					if (order != null && Array.IndexOf (order, p.Name) >= 0)
 						continue;
 					if (omitted == null) {
 						omitted = msg;
@@ -956,4 +955,3 @@ namespace System.Web.Services.Description
 	}
 }
 
-#endif
