@@ -42,9 +42,7 @@ using MonoTests.SystemWeb.Framework;
 using MonoTests.stand_alone.WebHarness;
 using MonoTests.Common;
 
-#if NET_2_0
 using System.Web.UI.Adapters;
-#endif
 
 #if NET_4_0
 using System.Web.Routing;
@@ -232,7 +230,6 @@ namespace MonoTests.System.Web.UI
 			string uid1_3 = c1.UniqueID;
 			Assert.IsFalse (uid1_3 == uid1_2, "id was not reset");
 
-#if NET_2_0
 			EnsureIDControl c3 = new EnsureIDControl ();
 			nc.Controls.Add (c3);
 			string uid3_1 = c3.UniqueID;
@@ -243,7 +240,6 @@ namespace MonoTests.System.Web.UI
 			string uid3_2 = c3.UniqueID;
 			Assert.IsNull (c3.ID);
 			Assert.IsFalse (uid3_1 == uid3_2, "id was not reset");
-#endif
 		}
 
 		[Test]
@@ -258,11 +254,7 @@ namespace MonoTests.System.Web.UI
 			nc3.Controls.Add (nc2);
 			nc2.Controls.Add (nc);
 			nc.Controls.Add (control);
-#if NET_2_0
 			string expected = "ctl00_ctl00_ctl00";
-#else
-			string expected = "_ctl0__ctl0__ctl0";
-#endif
 			Assert.AreEqual (expected, control.ClientID, "ClientID");
 		}
 
@@ -292,7 +284,6 @@ namespace MonoTests.System.Web.UI
 			Assert.AreEqual (-1, html.IndexOf ("<span>label</span><span>label</span>"), "#A1");
 		}
 		
-#if NET_2_0
 		[Test]
 		[Category("NunitWeb")]
 		public void AppRelativeTemplateSourceDirectory ()
@@ -816,11 +807,7 @@ namespace MonoTests.System.Web.UI
 
 		public static void ResolveUrl_Load (Page p)
 		{
-#if TARGET_JVM
-			string appPath = "/MainsoftWebApp20";
-#else
 			string appPath = "/NunitWeb";
-#endif
 			Control ctrl = new Control ();
 			p.Controls.Add (ctrl);
 			Assert.AreEqual (appPath + "/MyPage.aspx", ctrl.ResolveUrl ("~/MyPage.aspx"), "ResolveClientUrl Failed");
@@ -866,11 +853,7 @@ namespace MonoTests.System.Web.UI
 
 		public static void ResolveUrl2_Load (Page p)
 		{
-#if TARGET_JVM
-			string appPath = "/MainsoftWebApp20";
-#else
 			string appPath = "/NunitWeb";
-#endif
 			Control uc = p.FindControl ("WebUserControl1");
 			Control ctrl = uc.FindControl ("Label");
 
@@ -924,7 +907,6 @@ namespace MonoTests.System.Web.UI
 			Assert.AreEqual (false, ctrl.EnableTheming, "EnableTheming#2");
 		}
 
-#endif
 		[Test]
 		public void BindingContainer ()
 		{
@@ -951,7 +933,6 @@ namespace MonoTests.System.Web.UI
 			Assert.IsTrue (Object.ReferenceEquals (child2.NamingContainer, nc), "NamingContainer #2");
 			Assert.IsTrue (Object.ReferenceEquals (child2.BindingContainer, nc), "BindingContainer #2");
 
-#if NET_2_0
 			// DetailsViewPagerRow marked to be not BindingContainer 
 			DetailsViewPagerRow row = new DetailsViewPagerRow (0, DataControlRowType.Pager, DataControlRowState.Normal);
 			TableCell cell = new TableCell ();
@@ -962,16 +943,13 @@ namespace MonoTests.System.Web.UI
 
 			Assert.IsTrue (Object.ReferenceEquals (child3.NamingContainer, row), "NamingContainer #3");
 			Assert.IsTrue (Object.ReferenceEquals (child3.BindingContainer, control), "BindingContainer #3");
-#endif
 		}
-#if NET_2_0
 		[Test]
 		public void Control_Adapter ()
 		{
 			MyNC ctr = new MyNC ();
 			Assert.AreEqual (null, ctr.Adapter (), "Adapter");
 		}
-#endif
 		[Test]
 		public void ChildControlsCreated () {
 			ChildControlsCreatedControl ctr = new ChildControlsCreatedControl ();
@@ -983,7 +961,6 @@ namespace MonoTests.System.Web.UI
 			Assert.AreEqual (1, ctr.Controls.Count, "ChildControlsCreated#2");
 		}
 
-#if NET_2_0
 		[Test (Description="Bug #594238")]
 		public void OverridenControlsPropertyAndPostBack_Bug594238 ()
 		{
@@ -1021,7 +998,6 @@ namespace MonoTests.System.Web.UI
 			WebTest.CopyResource (GetType (), "OverridenControlsPropertyAndPostBack_Bug594238.aspx", "OverridenControlsPropertyAndPostBack_Bug594238.aspx");
 		}
 
-#endif
 #if NET_4_0
 		[Test]
 		public void GetRouteUrl_Object ()
@@ -1142,7 +1118,6 @@ namespace MonoTests.System.Web.UI
 		}
 #endif
 		#region helpcalsses
-#if NET_2_0
 		class ControlWithState : Control
 		{
 			string _state;
@@ -1158,18 +1133,18 @@ namespace MonoTests.System.Web.UI
 				set { ViewState ["Viewstate"] = value; }
 			}
 
-			protected override void OnInit (EventArgs e)
+			protected internal override void OnInit (EventArgs e)
 			{
 				base.OnInit (e);
 				Page.RegisterRequiresControlState (this);
 			}
 
-			protected override object SaveControlState ()
+			protected internal override object SaveControlState ()
 			{
 				return State;
 			}
 
-			protected override void LoadControlState (object savedState)
+			protected internal override void LoadControlState (object savedState)
 			{
 				State = (string) savedState;
 			}
@@ -1195,10 +1170,8 @@ namespace MonoTests.System.Web.UI
 			}
 		}
 
-#endif
 		class MyNC : Control, INamingContainer
 		{
-			#if NET_2_0
 			public ControlAdapter Adapter ()
 			{
 				return base.Adapter;
@@ -1223,7 +1196,6 @@ namespace MonoTests.System.Web.UI
 			{
 				return base.HasEvents ();
 			}
-			#endif
 		}
 
 		class DerivedControl : Control
@@ -1240,7 +1212,6 @@ namespace MonoTests.System.Web.UI
 				get { return coll; }
 			}
 
-#if NET_2_0
 			public bool DoIsViewStateEnabled
 			{
 				get { return IsViewStateEnabled; }
@@ -1265,7 +1236,6 @@ namespace MonoTests.System.Web.UI
 			{
 				return base.ResolveAdapter ();
 			}
-#endif
 		}
 
 		class NamingContainer : Control, INamingContainer
@@ -1283,7 +1253,7 @@ namespace MonoTests.System.Web.UI
 				set { _template = value; }
 			}
 
-			protected override void CreateChildControls ()
+			protected internal override void CreateChildControls ()
 			{
 				Controls.Clear ();
 
@@ -1301,7 +1271,6 @@ namespace MonoTests.System.Web.UI
 		#endregion
 	}
 
-#if NET_2_0
 	public class Customadaptercontrol : Control
 	{
 		public new ControlAdapter Adapter {
@@ -1323,11 +1292,10 @@ namespace MonoTests.System.Web.UI
 			EnsureID ();
 		}
 	}
-#endif
 
 	public class ChildControlsCreatedControl : Control
 	{
-		protected override void CreateChildControls () {
+		protected internal override void CreateChildControls () {
 			Controls.Add (new Control ());
 		}
 
