@@ -38,7 +38,6 @@ using System.Xml.XPath;
 using Mono.Xml;
 #endif
 
-#if NET_2_0
 
 using System.Collections.Generic;
 using QName = System.Xml.XmlQualifiedName;
@@ -77,7 +76,7 @@ using ValException = System.Xml.Schema.XmlSchemaValidationException;
 namespace Mono.Xml.Schema
 {
 	internal class XmlSchemaValidatingReader : XmlReader, IXmlLineInfo,
-		IXmlSchemaInfo
+		IXmlSchemaInfo, IXmlNamespaceResolver, IHasXmlParserContext
 	{
 		static readonly XsAttr [] emptyAttributeArray =
 			new XsAttr [0];
@@ -92,19 +91,17 @@ namespace Mono.Xml.Schema
 		IXmlLineInfo readerLineInfo;
 		ValidationType validationType;
 		IXmlNamespaceResolver nsResolver;
-		int startDepth;
-
-		StringBuilder tmpBuilder = new StringBuilder ();
 
 		XsAttr [] defaultAttributes = emptyAttributeArray;
 		int currentDefaultAttribute = -1;
 		ArrayList defaultAttributesCache = new ArrayList ();
 		bool defaultAttributeConsumed;
 		XmlSchemaType currentAttrType;
+		bool validationDone;
 
 		// Extra for XmlSchemaValidtingReader
 		// (not in XsdValidatingReader)
-		XsElement element; // ... xsinfo.Element?
+		//XsElement element; // ... xsinfo.Element?
 
 		#endregion
 
@@ -127,13 +124,20 @@ namespace Mono.Xml.Schema
 				schemas,
 				nsResolver,
 				options);
+			if (reader.BaseURI != String.Empty && Uri.IsWellFormedUriString(reader.BaseURI, UriKind.Absolute))
+				v.SourceUri = new Uri (reader.BaseURI);
 
 			readerLineInfo = reader as IXmlLineInfo;
-			startDepth = reader.Depth;
-			getter = delegate () { return Value; };
+			getter = delegate () {
+				if (v.CurrentAttributeType != null)
+					return v.CurrentAttributeType.ParseValue (Value, NameTable, this);
+				else
+					return Value; 
+				};
 			xsinfo = new XmlSchemaInfo (); // transition cache
 			v.LineInfoProvider = this;
 			v.ValidationEventSender = reader;
+			this.nsResolver = nsResolver;
 #if !NON_MONO
 			ValidationEventHandler += delegate (object o, ValidationEventArgs e) {
 				settings.OnValidationError (o, e);
@@ -153,9 +157,10 @@ namespace Mono.Xml.Schema
 			remove { v.ValidationEventHandler -= value; }
 		}
 
+		[MonoTODO]
 		public XmlSchemaType ElementSchemaType {
 			get {
-				return element != null ? element.ElementSchemaType : null;
+				return null; // element != null ? element.ElementSchemaType : null;
 			}
 		}
 
@@ -167,6 +172,7 @@ namespace Mono.Xml.Schema
 			defaultAttributeConsumed = false;
 			currentAttrType = null;
 			defaultAttributes = emptyAttributeArray;
+			v.CurrentAttributeType = null;
 		}
 
 		#region Properties
@@ -357,11 +363,9 @@ namespace Mono.Xml.Schema
 			}
 		}
 
-#if !NON_MONO
 		public XmlParserContext ParserContext {
 			get { return XmlSchemaUtil.GetParserContext (reader); }
 		}
-#endif
 
 		public override string Prefix {
 			get {
@@ -387,7 +391,18 @@ namespace Mono.Xml.Schema
 		}
 
 		public override IXmlSchemaInfo SchemaInfo {
-			get { return this; }
+			get {
+				return new XmlSchemaInfo () {
+					//ContentType = this.ContentType,
+					IsDefault = this.IsDefault,
+					IsNil = this.IsNil,
+					MemberType = this.MemberType,
+					SchemaAttribute = this.SchemaAttribute,
+					SchemaElement = this.SchemaElement,
+					SchemaType = this.SchemaType,
+					Validity = this.Validity
+					};
+			}
 		}
 
 		public override string Value {
@@ -676,7 +691,10 @@ namespace Mono.Xml.Schema
 		public override bool Read ()
 		{
 			if (!reader.Read ()) {
-				v.EndValidation ();
+				if (!validationDone) {
+					v.EndValidation ();
+					validationDone = true;
+				}
 				return false;
 			}
 
@@ -765,26 +783,6 @@ namespace Mono.Xml.Schema
 			return true;
 		}
 
-#if NET_1_0
-		public override string ReadInnerXml ()
-		{
-			// MS.NET 1.0 has a serious bug here. It skips validation.
-			return ReadInnerXmlInternal ();
-		}
-
-		public override string ReadOuterXml ()
-		{
-			// MS.NET 1.0 has a serious bug here. It skips validation.
-			return ReadInnerXmlInternal ();
-		}
-
-		// XmlReader.ReadString() should call derived this.Read().
-		public override string ReadString ()
-		{
-			return ReadStringInternal ();
-		}
-#endif
-
 		// This class itself does not have this feature.
 		public override void ResolveEntity ()
 		{
@@ -819,4 +817,3 @@ namespace Mono.Xml.Schema
 	}
 }
 
-#endif
