@@ -29,9 +29,7 @@
 //
 using System;
 using System.Collections;
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 using System.Collections.Specialized;
 using System.IO;
 using System.Text;
@@ -39,11 +37,7 @@ using System.Xml;
 using System.Xml.Schema;
 using Mono.Xml;
 
-#if NET_2_0
 using ValException = System.Xml.Schema.XmlSchemaValidationException;
-#else
-using ValException = System.Xml.Schema.XmlSchemaException;
-#endif
 
 using QName = System.Xml.XmlQualifiedName;
 using ContentProc = System.Xml.Schema.XmlSchemaContentProcessing;
@@ -473,7 +467,7 @@ namespace Mono.Xml.Schema
 
 		private void ValidateEndElementParticle ()
 		{
-			if (Context.State != null) {
+			if (xsiNilDepth < 0 && Context.State != null) {
 				if (!Context.EvaluateEndElement ()) {
 					HandleError ("Invalid end element: " + reader.Name);
 				}
@@ -523,6 +517,16 @@ namespace Mono.Xml.Schema
 					dt = st.Datatype;
 				} else {
 					ComplexType ct = Context.ActualType as ComplexType;
+					var ctsm = ct.ContentModel as XmlSchemaSimpleContent;
+					if (ctsm != null) {
+						var scr = ctsm.Content as XmlSchemaSimpleContentRestriction;
+						if (scr != null)
+							st = FindSimpleBaseType (scr.BaseType ?? FindType (scr.BaseTypeName));
+						var sce = ctsm.Content as XmlSchemaSimpleContentExtension;
+						if (sce != null)
+							st = FindSimpleBaseType (FindType (sce.BaseTypeName));
+					}
+
 					dt = ct.Datatype;
 					switch (ct.ContentType) {
 					case XmlSchemaContentType.ElementOnly:
@@ -552,6 +556,16 @@ namespace Mono.Xml.Schema
 			shouldValidateCharacters = false;
 		}
 
+		SimpleType FindSimpleBaseType (XmlSchemaType xt)
+		{
+			var st = xt as SimpleType;
+			if (st != null)
+				return st;
+			if (xt == null)
+				return null;
+			return FindSimpleBaseType (xt.BaseXmlSchemaType);
+		}
+
 		// 3.14.4 String Valid 
 		private void AssessStringValid (SimpleType st,
 			XsDatatype dt, string value)
@@ -1660,28 +1674,10 @@ namespace Mono.Xml.Schema
 			return true;
 		}
 
-#if NET_1_0
-		public override string ReadInnerXml ()
-		{
-			// MS.NET 1.0 has a serious bug here. It skips validation.
-			return reader.ReadInnerXml ();
-		}
-
-		public override string ReadOuterXml ()
-		{
-			// MS.NET 1.0 has a serious bug here. It skips validation.
-			return reader.ReadOuterXml ();
-		}
-#endif
-
 		// XmlReader.ReadString() should call derived this.Read().
 		public override string ReadString ()
 		{
-#if NET_1_0
-			return reader.ReadString ();
-#else
 			return base.ReadString ();
-#endif
 		}
 
 		// This class itself does not have this feature.
@@ -1730,7 +1726,6 @@ namespace Mono.Xml.Schema
 			}
 		}
 
-#if NET_2_0
 		public XmlSchemaType ActualSchemaType {
 			get {
 				object at = ActualType;
@@ -1743,7 +1738,6 @@ namespace Mono.Xml.Schema
 				return st;
 			}
 		}
-#endif
 
 		public bool IsInvalid {
 			get { return State == XsdValidationState.Invalid; }
@@ -1809,14 +1803,14 @@ namespace Mono.Xml.Schema
 					MissingIDReferences.Remove (str);
 				break;
 			case XmlTokenizedType.IDREF:
-				if (!idList.Contains (str))
+				if (!idList.Contains (str) && !MissingIDReferences.Contains (str))
 					MissingIDReferences.Add (str);
 				break;
 			case XmlTokenizedType.IDREFS:
 				string [] idrefs = (string []) parsedValue;
 				for (int i = 0; i < idrefs.Length; i++) {
 					string id = idrefs [i];
-					if (!idList.Contains (id))
+					if (!idList.Contains (id) && !MissingIDReferences.Contains (str))
 						MissingIDReferences.Add (id);
 				}
 				break;
