@@ -35,11 +35,7 @@ using System.Xml.Schema;
 using System.Globalization;
 using System.Security.Cryptography;
 
-#if NET_2_0
 using NSResolver = System.Xml.IXmlNamespaceResolver;
-#else
-using NSResolver = System.Xml.XmlNamespaceManager;
-#endif
 
 
 namespace Mono.Xml.Schema
@@ -78,11 +74,13 @@ namespace Mono.Xml.Schema
 			get { return instance; }
 		}
 
-#if NET_2_0
+		protected XsdAnySimpleType ()
+		{
+		}
+
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.AnyAtomicType; }
 		}
-#endif
 
 		static readonly char [] whitespaceArray = new char [] {' '};
 
@@ -104,11 +102,12 @@ namespace Mono.Xml.Schema
 		}
 
 		public override Type ValueType {
-#if BUGGY_MS_COMPLIANT
-			get { return typeof (string); }
-#else
-			get { return typeof (object); }
-#endif
+			get {
+				if (XmlSchemaUtil.StrictMsCompliant)
+					return typeof (string);
+				else
+					return typeof (object);
+			}
 		}
 
 		public override XmlTokenizedType TokenizedType {
@@ -195,7 +194,6 @@ namespace Mono.Xml.Schema
 						 XmlSchemaFacet.Facet.enumeration | XmlSchemaFacet.Facet.whiteSpace; 
 	}
 
-#if NET_2_0
 	internal class XdtAnyAtomicType : XsdAnySimpleType
 	{
 		internal XdtAnyAtomicType ()
@@ -217,7 +215,6 @@ namespace Mono.Xml.Schema
 			get { return XmlTypeCode.UntypedAtomic; }
 		}
 	}
-#endif
 
 	// xs:string
 	internal class XsdString : XsdAnySimpleType
@@ -234,11 +231,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.String; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -273,11 +268,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.NormalizedString; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -298,11 +291,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Token; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -322,11 +313,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Language; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -346,11 +335,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.NMTOKEN; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.NmToken; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -381,12 +368,10 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.NMTOKENS; }
 		}
 
-#if NET_2_0
 		[MonoTODO]
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Item; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string []); }
@@ -423,11 +408,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Name; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -458,11 +441,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.NCName; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.NCName; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -493,11 +474,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.ID; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Id; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -522,11 +501,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.IDREF; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Idref; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -551,12 +528,10 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.IDREFS; }
 		}
 
-#if NET_2_0
 		[MonoTODO]
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Item; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string []); }
@@ -592,11 +567,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.ENTITY; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Entity; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -617,12 +590,10 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.ENTITIES; }
 		}
 
-#if NET_2_0
 		[MonoTODO]
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Item; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string []); }
@@ -663,11 +634,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.NOTATION; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Notation; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (string); }
@@ -711,11 +680,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.None; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Decimal; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (decimal); }
@@ -771,11 +738,9 @@ namespace Mono.Xml.Schema
 		{
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Integer; }
 		}
-#endif
 
 		// Here it may be bigger than int's (or long's) MaxValue.
 		public override Type ValueType {
@@ -803,11 +768,9 @@ namespace Mono.Xml.Schema
 	// xs:Long
 	internal class XsdLong : XsdInteger
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Long; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (long); }
@@ -843,11 +806,9 @@ namespace Mono.Xml.Schema
 	// xs:Int
 	internal class XsdInt : XsdLong
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Int; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (int); }
@@ -884,11 +845,9 @@ namespace Mono.Xml.Schema
 	// xs:Short
 	internal class XsdShort : XsdInt
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Short; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (short); }
@@ -924,11 +883,9 @@ namespace Mono.Xml.Schema
 	// xs:Byte
 	internal class XsdByte : XsdShort
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Byte; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (sbyte); }
@@ -965,11 +922,9 @@ namespace Mono.Xml.Schema
 //	[CLSCompliant (false)]
 	internal class XsdNonNegativeInteger : XsdInteger
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.NonNegativeInteger; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (decimal); }
@@ -992,11 +947,9 @@ namespace Mono.Xml.Schema
 //	[CLSCompliant (false)]
 	internal class XsdUnsignedLong : XsdNonNegativeInteger
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.UnsignedLong; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (ulong); }
@@ -1033,11 +986,9 @@ namespace Mono.Xml.Schema
 //	[CLSCompliant (false)]
 	internal class XsdUnsignedInt : XsdUnsignedLong
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.UnsignedInt; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (uint); }
@@ -1075,11 +1026,9 @@ namespace Mono.Xml.Schema
 //	[CLSCompliant (false)]
 	internal class XsdUnsignedShort : XsdUnsignedInt
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.UnsignedShort; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (ushort); }
@@ -1116,11 +1065,9 @@ namespace Mono.Xml.Schema
 //	[CLSCompliant (false)]
 	internal class XsdUnsignedByte : XsdUnsignedShort
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.UnsignedByte; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (byte); }
@@ -1158,11 +1105,9 @@ namespace Mono.Xml.Schema
 //	[CLSCompliant (false)]
 	internal class XsdPositiveInteger : XsdNonNegativeInteger
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.PositiveInteger; }
 		}
-#endif
 
 		// It returns decimal, instead of int or long.
 		// Maybe MS developers thought about big integer...
@@ -1185,11 +1130,9 @@ namespace Mono.Xml.Schema
 	// xs:nonPositiveInteger
 	internal class XsdNonPositiveInteger : XsdInteger
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.NonPositiveInteger; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (decimal); }
@@ -1210,11 +1153,9 @@ namespace Mono.Xml.Schema
 	// xs:negativeInteger
 	internal class XsdNegativeInteger : XsdNonPositiveInteger
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.NegativeInteger; }
 		}
-#endif
 
 		public override Type ValueType {
 
@@ -1236,11 +1177,9 @@ namespace Mono.Xml.Schema
 	// xs:float
 	internal class XsdFloat : XsdAnySimpleType
 	{
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Float; }
 		}
-#endif
 
 		internal XsdFloat ()
 		{
@@ -1323,11 +1262,9 @@ namespace Mono.Xml.Schema
 			get { return XsdOrderedFacet.Total; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Double; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (double); }
@@ -1368,11 +1305,9 @@ namespace Mono.Xml.Schema
 		{
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Base64Binary; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (byte[]); }
@@ -1464,11 +1399,9 @@ namespace Mono.Xml.Schema
 			get { return stringAllowedFacets; } 
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.HexBinary; }
 		}
-#endif
 
 		public override XmlTokenizedType TokenizedType {
 			get { return XmlTokenizedType.None; }
@@ -1509,11 +1442,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.QName; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.QName; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (XmlQualifiedName); }
@@ -1527,7 +1458,7 @@ namespace Mono.Xml.Schema
 				throw new ArgumentNullException ("name table");
 			if (nsmgr == null)
 				throw new ArgumentNullException ("namespace manager");
-			XmlQualifiedName name = XmlQualifiedName.Parse (s, nsmgr);
+			XmlQualifiedName name = XmlQualifiedName.Parse (s, nsmgr, true);
 			nameTable.Add (name.Name);
 			nameTable.Add (name.Namespace);
 			return name;
@@ -1552,18 +1483,17 @@ namespace Mono.Xml.Schema
 		}
 
 		public override XmlTokenizedType TokenizedType {
-#if BUGGY_MS_COMPLIANT
-			get { return XmlTokenizedType.None; }
-#else
-			get { return XmlTokenizedType.CDATA; }
-#endif
+			get {
+				if (XmlSchemaUtil.StrictMsCompliant)
+					return XmlTokenizedType.None;
+				else
+					return XmlTokenizedType.CDATA;
+			}
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Boolean; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (bool); }
@@ -1605,11 +1535,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.AnyUri; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (Uri); }
@@ -1709,11 +1637,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Duration; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (TimeSpan); }
@@ -1769,7 +1695,6 @@ namespace Mono.Xml.Schema
 		}
 	}
 
-#if NET_2_0
 	// xdt:dayTimeDuration
 	internal class XdtDayTimeDuration : XsdDuration
 	{
@@ -1853,7 +1778,6 @@ namespace Mono.Xml.Schema
 	 
 		}
 	}
-#endif
 
 	// xs:dateTime
 	internal class XsdDateTime : XsdAnySimpleType
@@ -1871,11 +1795,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.DateTime; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -1941,11 +1863,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Date; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -2028,11 +1948,9 @@ namespace Mono.Xml.Schema
 			get { return XmlTokenizedType.CDATA; }
 		}
 
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.Time; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -2084,11 +2002,9 @@ namespace Mono.Xml.Schema
 			get { return durationAllowedFacets; } 
 		}
 		
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.GYearMonth; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -2135,11 +2051,9 @@ namespace Mono.Xml.Schema
 			get { return durationAllowedFacets; } 
 		}
 		
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.GMonthDay; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -2186,11 +2100,9 @@ namespace Mono.Xml.Schema
 			get { return durationAllowedFacets; } 
 		}
 		
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.GYear; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -2239,11 +2151,9 @@ namespace Mono.Xml.Schema
 			get { return durationAllowedFacets; } 
 		}
 		
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.GMonth; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
@@ -2257,7 +2167,8 @@ namespace Mono.Xml.Schema
 
 		internal override ValueType ParseValueType (string s, XmlNameTable nameTable, NSResolver nsmgr) 
 		{
-			return DateTime.ParseExact (Normalize(s), "--MM--", null);
+			DateTime ret;
+			return DateTime.TryParseExact (Normalize(s), "--MM", null, DateTimeStyles.RoundtripKind, out ret) ? ret : DateTime.ParseExact (Normalize(s), "--MM--", null);
 		}
 		
 		internal override XsdOrdering Compare(object x, object y) {
@@ -2290,11 +2201,9 @@ namespace Mono.Xml.Schema
 			get { return durationAllowedFacets; } 
 		}
 		
-#if NET_2_0
 		public override XmlTypeCode TypeCode {
 			get { return XmlTypeCode.GDay; }
 		}
-#endif
 
 		public override Type ValueType {
 			get { return typeof (DateTime); }
