@@ -28,7 +28,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 using System.Collections;
 using System.Xml;
@@ -45,8 +44,10 @@ namespace System.Xml.Schema
 		double doubleValue;
 		int intValue;
 		long longValue;
+		ulong ulongValue;
 		object objectValue;
 		float floatValue;
+		byte [] bytesValue;
 		string stringValue;
 		XmlSchemaType schemaType;
 		XmlTypeCode xmlTypeCode;
@@ -68,6 +69,20 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
+		internal XmlAtomicValue (byte [] value, XmlSchemaType xmlType)
+		{
+			Init (value, xmlType);
+		}
+		
+		private void Init (byte [] value, XmlSchemaType xmlType)
+		{
+			if (xmlType == null)
+				throw new ArgumentNullException ("xmlType");
+			xmlTypeCode = XmlTypeCode.Base64Binary;
+			this.bytesValue = value;
+			schemaType = xmlType;
+		}
+
 		internal XmlAtomicValue (DateTime value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -138,6 +153,20 @@ namespace System.Xml.Schema
 			schemaType = xmlType;
 		}
 
+		internal XmlAtomicValue (ulong value, XmlSchemaType xmlType)
+		{
+			Init (value, xmlType);
+		}
+		
+		private void Init (ulong value, XmlSchemaType xmlType)
+		{
+			if (xmlType == null)
+				throw new ArgumentNullException ("xmlType");
+			xmlTypeCode = XmlTypeCode.UnsignedLong;
+			this.ulongValue = value;
+			schemaType = xmlType;
+		}
+
 		internal XmlAtomicValue (float value, XmlSchemaType xmlType)
 		{
 			Init (value, xmlType);
@@ -182,8 +211,18 @@ namespace System.Xml.Schema
 				throw new ArgumentNullException ("xmlType");
 
 			switch (Type.GetTypeCode (value.GetType ())) {
+			case TypeCode.Byte:
+				Init ((byte) value, xmlType);
+				return;
+			case TypeCode.SByte:
+				Init ((sbyte) value, xmlType);
+				return;
 			case TypeCode.Int16:
+				Init ((short) value, xmlType);
+				return;
 			case TypeCode.UInt16:
+				Init ((ushort) value, xmlType);
+				return;
 			case TypeCode.Int32:
 				Init ((int) value, xmlType);
 				return;
@@ -197,9 +236,14 @@ namespace System.Xml.Schema
 				Init ((float) value, xmlType);
 				return;
 			case TypeCode.Int64:
-			case TypeCode.UInt32:
 				Init ((long) value, xmlType);
 				return;
+			case TypeCode.UInt32:
+				Init ((uint) value, xmlType);
+				return;
+			case TypeCode.UInt64:
+				Init ((ulong) value, xmlType);
+				return;
 			case TypeCode.String:
 				Init ((string) value, xmlType);
 				return;
@@ -210,6 +254,10 @@ namespace System.Xml.Schema
 				Init ((bool) value, xmlType);
 				return;
 			}
+			if (value is byte []) {
+				Init ((byte []) value, xmlType);
+				return;
+			}
 
 			ICollection col = value as ICollection;
 			if (col != null && col.Count == 1) {
@@ -248,6 +296,9 @@ namespace System.Xml.Schema
 				case XmlTypeCode.Long:
 					Init (another.longValue, xmlType);
 					return;
+				case XmlTypeCode.UnsignedLong:
+					Init (another.ulongValue, xmlType);
+					return;
 				case XmlTypeCode.Float:
 					Init (another.floatValue, xmlType);
 					return;
@@ -285,6 +336,8 @@ namespace System.Xml.Schema
 			case XmlTypeCode.Short:
 			case XmlTypeCode.UnsignedShort:
 				return ValueAsInt;
+			case XmlTypeCode.Decimal:
+				return ValueAsDecimal;
 			case XmlTypeCode.Double:
 			case XmlTypeCode.Float:
 				return ValueAsDouble;
@@ -300,7 +353,12 @@ namespace System.Xml.Schema
 			case XmlTypeCode.Item:
 				return TypedValue;
 			case XmlTypeCode.QName:
-				return XmlQualifiedName.Parse (Value, nsResolver);
+				return XmlQualifiedName.Parse (Value, nsResolver, true);
+			case XmlTypeCode.Base64Binary:
+			case XmlTypeCode.HexBinary:
+				if (bytesValue != null)
+					return bytesValue;
+				break;
 			}
 			throw new NotImplementedException ();
 		}
@@ -340,6 +398,8 @@ namespace System.Xml.Schema
 					return ValueAsBoolean;
 				case XmlTypeCode.DateTime:
 					return ValueAsDateTime;
+				case XmlTypeCode.Decimal:
+					return ValueAsDecimal;
 				case XmlTypeCode.Float:
 				case XmlTypeCode.Double:
 					return ValueAsDouble;
@@ -361,19 +421,61 @@ namespace System.Xml.Schema
 				case XmlTypeCode.Boolean:
 					stringValue = XQueryConvert.BooleanToString (ValueAsBoolean);
 					break;
+				case XmlTypeCode.Date:
+				case XmlTypeCode.Time:
 				case XmlTypeCode.DateTime:
 					stringValue = XQueryConvert.DateTimeToString (ValueAsDateTime);
 					break;
+				case XmlTypeCode.Duration:
+					stringValue = XQueryConvert.DayTimeDurationToString (TimeSpan.FromMilliseconds (doubleValue));
+					break;
+				case XmlTypeCode.GYear:
+					stringValue = XQueryConvert.GYearToString (ValueAsDateTime);
+					break;
+				case XmlTypeCode.GYearMonth:
+					stringValue = XQueryConvert.GYearMonthToString (ValueAsDateTime);
+					break;
+				case XmlTypeCode.GMonth:
+					stringValue = XQueryConvert.GMonthToString (ValueAsDateTime);
+					break;
+				case XmlTypeCode.GMonthDay:
+					stringValue = XQueryConvert.GMonthDayToString (ValueAsDateTime);
+					break;
+				case XmlTypeCode.GDay:
+					stringValue = XQueryConvert.GDayToString (ValueAsDateTime);
+					break;
 				case XmlTypeCode.Float:
 				case XmlTypeCode.Double:
 					stringValue = XQueryConvert.DoubleToString (ValueAsDouble);
 					break;
+				case XmlTypeCode.Integer:
+				case XmlTypeCode.UnsignedInt:
+				case XmlTypeCode.UnsignedLong:
+				case XmlTypeCode.Decimal:
+					stringValue = XQueryConvert.DecimalToString (ValueAsDecimal);
+					break;
+				case XmlTypeCode.NonPositiveInteger:
+				case XmlTypeCode.NonNegativeInteger:
+				case XmlTypeCode.NegativeInteger:
 				case XmlTypeCode.Long:
+				case XmlTypeCode.PositiveInteger:
 					stringValue = XQueryConvert.IntegerToString (ValueAsLong);
 					break;
 				case XmlTypeCode.Int:
+				case XmlTypeCode.Short:
+				case XmlTypeCode.Byte:
+				case XmlTypeCode.UnsignedShort:
+				case XmlTypeCode.UnsignedByte:
 					stringValue = XQueryConvert.IntToString (ValueAsInt);
 					break;
+				case XmlTypeCode.Base64Binary:
+					stringValue = XQueryConvert.Base64BinaryToString (bytesValue);
+					break;
+				case XmlTypeCode.HexBinary:
+					stringValue = XQueryConvert.HexBinaryToString (bytesValue);
+					break;
+				case XmlTypeCode.QName:
+				case XmlTypeCode.AnyUri:
 				case XmlTypeCode.String:
 					return stringValue;
 
@@ -384,9 +486,18 @@ namespace System.Xml.Schema
 					case XmlTypeCode.String:
 						stringValue = (string) objectValue;
 						break;
+					case XmlTypeCode.Date:
+						stringValue = XQueryConvert.DateToString ((DateTime) objectValue);
+						break;
+					case XmlTypeCode.Time:
+						stringValue = XQueryConvert.TimeToString ((DateTime) objectValue);
+						break;
 					case XmlTypeCode.DateTime:
 						stringValue = XQueryConvert.DateTimeToString ((DateTime) objectValue);
 						break;
+					case XmlTypeCode.Duration:
+						stringValue = XQueryConvert.DayTimeDurationToString (TimeSpan.FromMilliseconds ((double) objectValue));
+						break;
 					case XmlTypeCode.Boolean:
 						stringValue = XQueryConvert.BooleanToString ((bool) objectValue);
 						break;
@@ -396,6 +507,7 @@ namespace System.Xml.Schema
 					case XmlTypeCode.Double:
 						stringValue = XQueryConvert.DoubleToString ((double) objectValue);
 						break;
+					case XmlTypeCode.Integer:
 					case XmlTypeCode.Decimal:
 						stringValue = XQueryConvert.DecimalToString ((decimal) objectValue);
 						break;
@@ -405,6 +517,12 @@ namespace System.Xml.Schema
 					case XmlTypeCode.Int:
 						stringValue = XQueryConvert.IntToString ((int) objectValue);
 						break;
+					case XmlTypeCode.Base64Binary:
+						stringValue = XQueryConvert.Base64BinaryToString ((byte []) objectValue);
+						break;
+					case XmlTypeCode.HexBinary:
+						stringValue = XQueryConvert.HexBinaryToString ((byte []) objectValue);
+						break;
 					}
 					break;
 				}
@@ -412,9 +530,9 @@ namespace System.Xml.Schema
 					return stringValue;
 
 				if (objectValue != null)
-					throw new InvalidCastException (String.Format ("Conversion from runtime type {0} to {1} is not supported", objectValue.GetType (), XmlTypeCode.String));
+					throw new InvalidCastException (String.Format ("Conversion from runtime type {0}, resolved as type code {1}, to {2} is not supported", objectValue.GetType (), ResolvedTypeCode, XmlTypeCode.String));
 				else
-					throw new InvalidCastException (String.Format ("Conversion from schema type {0} (type code {1}) to {2} is not supported", schemaType.QualifiedName, xmlTypeCode, XmlTypeCode.String));
+					throw new InvalidCastException (String.Format ("Conversion from schema type {0} (type code {1}, resolved type code {2}) to {3} is not supported.", schemaType.QualifiedName, xmlTypeCode, ResolvedTypeCode, XmlTypeCode.String));
 			}
 		}
 
@@ -456,6 +574,16 @@ namespace System.Xml.Schema
 					return dateTimeValue;
 				case XmlTypeCode.String:
 					return XQueryConvert.StringToDateTime (stringValue);
+				case XmlTypeCode.GYear:
+					return XQueryConvert.StringToGYear (stringValue);
+				case XmlTypeCode.GYearMonth:
+					return XQueryConvert.StringToGYearMonth (stringValue);
+				case XmlTypeCode.GMonth:
+					return XQueryConvert.StringToGMonth (stringValue);
+				case XmlTypeCode.GMonthDay:
+					return XQueryConvert.StringToGMonthDay (stringValue);
+				case XmlTypeCode.GDay:
+					return XQueryConvert.StringToGDay (stringValue);
 				case XmlTypeCode.None:
 				case XmlTypeCode.Item:
 				case XmlTypeCode.AnyAtomicType:
@@ -469,6 +597,39 @@ namespace System.Xml.Schema
 			}
 		}
 
+		// Unlike the other ValueAs...() functions, this is not part
+		// of the XPathItem abstract class, so it's not an override
+		public decimal ValueAsDecimal {
+			get {
+				switch (xmlTypeCode) {
+				case XmlTypeCode.Boolean:
+					return XQueryConvert.BooleanToDecimal (booleanValue);
+				case XmlTypeCode.Decimal:
+					return decimalValue;
+				case XmlTypeCode.Double:
+					return XQueryConvert.DoubleToDecimal (doubleValue);
+				case XmlTypeCode.Long:
+					return XQueryConvert.IntegerToDecimal (longValue);
+				case XmlTypeCode.UnsignedLong:
+					return XQueryConvert.IntegerToDecimal (ulongValue);
+				case XmlTypeCode.Int:
+					return XQueryConvert.IntToDecimal (intValue);
+				case XmlTypeCode.Float:
+					return XQueryConvert.FloatToDecimal (floatValue);
+				case XmlTypeCode.String:
+					return XQueryConvert.StringToDecimal (stringValue);
+				case XmlTypeCode.None:
+				case XmlTypeCode.Item:
+				case XmlTypeCode.AnyAtomicType:
+					if (objectValue is decimal)
+						return (decimal) objectValue;
+					break;
+				}
+
+				throw new InvalidCastException (String.Format ("Conversion from {0} to {1} is not supported", schemaType.QualifiedName, XmlSchemaSimpleType.XsDecimal.QualifiedName));
+			}
+		}
+
 		public override double ValueAsDouble {
 			get {
 				switch (xmlTypeCode) {
@@ -596,6 +757,9 @@ namespace System.Xml.Schema
 				return typeof (DateTime);
 			case XmlTypeCode.Boolean:
 				return typeof (bool);
+			case XmlTypeCode.Base64Binary:
+			case XmlTypeCode.HexBinary:
+				return typeof (byte []);
 			case XmlTypeCode.Item:
 				return typeof (object);
 			}
@@ -628,7 +792,10 @@ namespace System.Xml.Schema
 			case TypeCode.Boolean:
 				return XmlTypeCode.Boolean;
 			case TypeCode.Object:
-				return XmlTypeCode.Item;
+				if (cliType == typeof (byte []))
+					return XmlTypeCode.Base64Binary;
+				else
+					return XmlTypeCode.Item;
 			}
 			if (raiseError)
 				throw new NotSupportedException (String.Format ("XQuery internal error: Cannot infer XmlTypeCode from Runtime Type {0}", cliType));
@@ -639,4 +806,3 @@ namespace System.Xml.Schema
 	}
 }
 
-#endif
