@@ -43,9 +43,7 @@ namespace System.Xml.Schema
 		private XmlSchemaObjectCollection constraints;
 		private string defaultValue;
 		private object elementType;
-#if NET_2_0
 		private XmlSchemaType elementSchemaType;
-#endif
 		private XmlSchemaDerivationMethod final;
 		private string fixedValue;
 		private XmlSchemaForm form;
@@ -216,9 +214,7 @@ namespace System.Xml.Schema
 		}
 
 		[XmlIgnore]
-#if NET_2_0
 		[Obsolete]
-#endif
 		public object ElementType 
 		{
 			get {
@@ -229,7 +225,6 @@ namespace System.Xml.Schema
 			}
 		}
 
-#if NET_2_0
 		[XmlIgnore]
 		public XmlSchemaType ElementSchemaType
 		{
@@ -240,7 +235,6 @@ namespace System.Xml.Schema
 					return elementSchemaType;
 			}
 		}
-#endif
 
 		[XmlIgnore]
 		public XmlSchemaDerivationMethod BlockResolved 
@@ -322,6 +316,15 @@ namespace System.Xml.Schema
 
 		#endregion
 
+		internal override void SetParent (XmlSchemaObject parent)
+		{
+			base.SetParent (parent);
+			if (SchemaType != null)
+				SchemaType.SetParent (this);
+			foreach (XmlSchemaObject obj in Constraints)
+				obj.SetParent (this);
+		}
+
 		/// <remarks>
 		/// a) If Element has parent as schema:
 		///		1. name must be present and of type NCName.
@@ -354,12 +357,6 @@ namespace System.Xml.Schema
 				return 0;
 			InitPostCompileInformations ();
 			this.schema = schema;
-#if NET_2_0
-			if (SchemaType != null)
-				SchemaType.Parent = this;
-			foreach (XmlSchemaObject obj in Constraints)
-				obj.Parent = this;
-#endif
 
 			if(this.defaultValue != null && this.fixedValue != null)
 				error(h,"both default and fixed can't be present");
@@ -374,7 +371,7 @@ namespace System.Xml.Schema
 				else if(!XmlSchemaUtil.CheckNCName(this.name)) // b1.2
 					error(h,"attribute name must be NCName");
 				else
-					this.qName = new XmlQualifiedName (this.name, schema.TargetNamespace);
+					this.qName = new XmlQualifiedName (this.name, AncestorSchema.TargetNamespace);
 
 				if(form != XmlSchemaForm.None)
 					error(h,"form must be absent");
@@ -449,8 +446,8 @@ namespace System.Xml.Schema
 				{
 					string targetNamespace = String.Empty;
 
-					if(form == XmlSchemaForm.Qualified || (form == XmlSchemaForm.None && schema.ElementFormDefault == XmlSchemaForm.Qualified))
-						targetNamespace = schema.TargetNamespace;
+					if(form == XmlSchemaForm.Qualified || (form == XmlSchemaForm.None && AncestorSchema.ElementFormDefault == XmlSchemaForm.Qualified))
+						targetNamespace = AncestorSchema.TargetNamespace;
 
 					if(this.name == null)	//b1
 						error(h,"Required attribute name must be present");
@@ -742,7 +739,6 @@ namespace System.Xml.Schema
 			foreach (XmlSchemaIdentityConstraint ident in Constraints)
 				ident.Validate (h, schema);
 
-#if NET_2_0
 			if (elementType != null) {
 				elementSchemaType = elementType as XmlSchemaType;
 				if (elementType == XmlSchemaSimpleType.AnySimpleType)
@@ -750,7 +746,6 @@ namespace System.Xml.Schema
 				if (elementSchemaType == null)
 					elementSchemaType = XmlSchemaType.GetBuiltInSimpleType (SchemaTypeName);
 			}
-#endif
 
 			ValidationId = schema.ValidationId;
 			return errorCount;
@@ -886,14 +881,6 @@ namespace System.Xml.Schema
 			return true;
 		}
 
-		internal override void CheckRecursion (int depth, ValidationEventHandler h, XmlSchema schema)
-		{
-			XmlSchemaComplexType ct = this.ElementType as XmlSchemaComplexType;
-			if (ct == null || ct.Particle == null)
-				return;
-			ct.Particle.CheckRecursion (depth + 1, h, schema);
-		}
-
 		internal override void ValidateUniqueParticleAttribution (XmlSchemaObjectTable qnames, ArrayList nsNames,
 			ValidationEventHandler h, XmlSchema schema)
 		{
@@ -923,7 +910,7 @@ namespace System.Xml.Schema
 						}
 					} else {
 						if (any.TargetNamespace != this.QualifiedName.Namespace)
-							error (h, "Ambiguous element label which is contained by -any- particle with ##other value was detected: " + this.QualifiedName);
+							error (h, String.Format ("Ambiguous element label '{0}' which is contained by -any- particle with ##other value than '{1}' was detected: ", this.QualifiedName.Namespace, any.TargetNamespace));
 					}
 				}
 				qnames.Add (this.QualifiedName, this);
@@ -965,8 +952,12 @@ namespace System.Xml.Schema
 			XmlNamespaceManager nsmgr = null;
 			if (datatype.TokenizedType == XmlTokenizedType.QName) {
 				if (this.Namespaces != null)
-					foreach (XmlQualifiedName qname in Namespaces.ToArray ())
+					foreach (XmlQualifiedName qname in Namespaces.ToArray ()) {
+						if (nsmgr == null)
+							nsmgr = new XmlNamespaceManager (new NameTable());
+
 						nsmgr.AddNamespace (qname.Name, qname.Namespace);
+					}
 			}
 
 			try {
