@@ -47,7 +47,6 @@ namespace System.Xml.Schema
 		private bool recursed;
 		private XmlSchemaDerivationMethod variety;
 
-#if NET_2_0
 		// predefined simple types
 		internal static readonly XmlSchemaSimpleType XsAnySimpleType;
 		internal static readonly XmlSchemaSimpleType XsString;
@@ -99,7 +98,6 @@ namespace System.Xml.Schema
 		internal static readonly XmlSchemaSimpleType XdtAnyAtomicType;
 		internal static readonly XmlSchemaSimpleType XdtYearMonthDuration;
 		internal static readonly XmlSchemaSimpleType XdtDayTimeDuration;
-#endif
 
 		static XmlSchemaSimpleType ()
 		{
@@ -112,7 +110,6 @@ namespace System.Xml.Schema
 			st.variety = XmlSchemaDerivationMethod.List;
 			schemaLocationType = st;
 
-#if NET_2_0
 			// Built-In schema types
 			XsAnySimpleType = BuildSchemaType ("anySimpleType", null);
 			XsString = BuildSchemaType ("string", "anySimpleType");
@@ -179,10 +176,8 @@ namespace System.Xml.Schema
 			sl = new XmlSchemaSimpleTypeList ();
 			sl.ItemType = XsNMToken;
 			XsNMTokens.Content = sl;
-#endif
 		}
 
-#if NET_2_0
 		private static XmlSchemaSimpleType BuildSchemaType (string name, string baseName)
 		{
 			return BuildSchemaType (name, baseName, false, false);
@@ -199,7 +194,6 @@ namespace System.Xml.Schema
 			st.DatatypeInternal = XmlSchemaDatatype.FromName (st.QualifiedName);
 			return st;
 		}
-#endif
 
 		internal static XsdAnySimpleType AnySimpleType {
 			get { return XsdAnySimpleType.Instance; }
@@ -227,6 +221,13 @@ namespace System.Xml.Schema
 			get{ return variety; }
 		}
 
+		internal override void SetParent (XmlSchemaObject parent)
+		{
+			base.SetParent (parent);
+			if (Content != null)
+				Content.SetParent (this);
+		}
+
 		/// <remarks>
 		/// For a simple Type:
 		///		1. Content must be present
@@ -251,10 +252,6 @@ namespace System.Xml.Schema
 			// If this is already compiled this time, simply skip.
 			if (CompilationId == schema.CompilationId)
 				return 0;
-#if NET_2_0
-			if (Content != null)
-				Content.Parent = this;
-#endif
 
 			errorCount = 0;
 
@@ -263,7 +260,7 @@ namespace System.Xml.Schema
 				if(this.Name != null) // a.1
 					error(h,"Name is prohibited in a local simpletype");
 				else
-					this.QNameInternal = new XmlQualifiedName(this.Name,schema.TargetNamespace);
+					this.QNameInternal = new XmlQualifiedName(this.Name, AncestorSchema.TargetNamespace);
 				if(this.Final != XmlSchemaDerivationMethod.None) //a.2
 					error(h,"Final is prohibited in a local simpletype");
 			}
@@ -274,7 +271,7 @@ namespace System.Xml.Schema
 				else if(!XmlSchemaUtil.CheckNCName(this.Name)) // b.1.2
 					error(h,"name attribute of a simpleType must be NCName");
 				else
-					this.QNameInternal = new XmlQualifiedName(this.Name,schema.TargetNamespace);
+					this.QNameInternal = new XmlQualifiedName(this.Name, AncestorSchema.TargetNamespace);
 				
 				//NOTE: Although the FinalResolved can be Empty, it is not a valid value for Final
 				//DEVIATION: If an error occurs, the finaldefault is always consulted. This deviates
