@@ -30,12 +30,13 @@
 //
 
 using System.Xml;
-#if !TARGET_JVM && !MOBILE
+#if !MOBILE
 using System.CodeDom.Compiler;
 #endif
 using System.Xml.Schema;
 using System.Collections;
-#if NET_2_0 && CONFIGURATION_DEP
+using System.Collections.Generic;
+#if CONFIGURATION_DEP
 using System.Configuration;
 using System.Xml.Serialization.Configuration;
 #endif
@@ -43,7 +44,7 @@ using System.Xml.Serialization.Configuration;
 namespace System.Xml.Serialization 
 {
 	public class XmlSchemaImporter
-#if NET_2_0 && !MOBILE
+#if !MOBILE
 		: SchemaImporter
 #endif
 	{
@@ -62,9 +63,7 @@ namespace System.Xml.Serialization
 		SoapReflectionImporter auxSoapRefImporter;
 		bool anyTypeImported;
 
-#if NET_2_0
-		CodeGenerationOptions options;
-#endif
+//		CodeGenerationOptions options;
 
 		static readonly XmlQualifiedName anyType = new XmlQualifiedName ("anyType",XmlSchema.Namespace);
 		static readonly XmlQualifiedName arrayType = new XmlQualifiedName ("Array",XmlSerializer.EncodingNamespace);
@@ -99,13 +98,12 @@ namespace System.Xml.Serialization
 			this.typeIdentifiers = typeIdentifiers;
 		}
 		
-#if NET_2_0
-#if !TARGET_JVM && !MOBILE
+#if !MOBILE
 		[MonoTODO]
 		public XmlSchemaImporter (XmlSchemas schemas, CodeGenerationOptions options, CodeDomProvider codeProvider, ImportContext context)
 		{
 			this.schemas = schemas;
-			this.options = options;
+//			this.options = options;
 			if (context != null) {
 				typeIdentifiers = context.TypeIdentifiers;
 				InitSharedData (context);
@@ -115,12 +113,12 @@ namespace System.Xml.Serialization
 
 			InitializeExtensions ();
 		}
-#endif
 
+		[MonoTODO]
 		public XmlSchemaImporter (XmlSchemas schemas, CodeGenerationOptions options, ImportContext context)
 		{
 			this.schemas = schemas;
-			this.options = options;
+//			this.options = options;
 			if (context != null) {
 				typeIdentifiers = context.TypeIdentifiers;
 				InitSharedData (context);
@@ -131,12 +129,12 @@ namespace System.Xml.Serialization
 			InitializeExtensions ();
 		}
 		
-
+		[MonoTODO]
 		public XmlSchemaImporter (XmlSchemas schemas, CodeIdentifiers typeIdentifiers, CodeGenerationOptions options)
 		{
 			this.typeIdentifiers = typeIdentifiers;
 			this.schemas = schemas;
-			this.options = options;
+//			this.options = options;
 
 			InitializeExtensions ();
 		}
@@ -163,7 +161,7 @@ namespace System.Xml.Serialization
 
 		void InitializeExtensions ()
 		{
-#if NET_2_0 && CONFIGURATION_DEP
+#if CONFIGURATION_DEP
 			SerializationSectionGroup root = ConfigurationManager.GetSection ("system.xml.serialization") as SerializationSectionGroup;
 			if (root == null)
 				return;
@@ -348,7 +346,6 @@ namespace System.Xml.Serialization
 			return new XmlMembersMapping (mapping);
 		}
 		
-#if NET_2_0
 		[MonoTODO]
 		public XmlMembersMapping ImportMembersMapping (string name, string ns, SoapSchemaMember[] members)
 		{
@@ -373,7 +370,6 @@ namespace System.Xml.Serialization
 				(XmlSchemaType) schemas.Find (typeName, typeof (XmlSchemaSimpleType));
 			return ImportTypeCommon (typeName, typeName, stype, true);
 		}
-#endif
 		
 		internal XmlMembersMapping ImportEncodedMembersMapping (string name, string ns, SoapSchemaMember[] members, bool hasWrapperElement)
 		{
@@ -508,7 +504,7 @@ namespace System.Xml.Serialization
 				stype = elem.SchemaType;
 				qname = elem.QualifiedName;
 			}
-			else if (elem.ElementType == XmlSchemaComplexType.AnyType)
+			else if (elem.ElementSchemaType == XmlSchemaComplexType.AnyType)
 			{
 				qname = anyType;
 				return true;
@@ -747,6 +743,25 @@ namespace System.Xml.Serialization
 				}
 			}
 		}
+		
+		IEnumerable<XmlSchemaAttribute> EnumerateAttributes (XmlSchemaObjectCollection col, List<XmlSchemaAttributeGroup> recurse)
+		{
+			foreach (var o in col) {
+				if (o is XmlSchemaAttributeGroupRef) {
+					var gr = (XmlSchemaAttributeGroupRef) o;
+					var g = FindRefAttributeGroup (gr.RefName);
+					if (recurse.Contains (g))
+						continue;
+					recurse.Add (g);
+					if (g == null)
+						throw new InvalidOperationException (string.Format ("Referenced AttributeGroup '{0}' was not found.", gr.RefName));
+					foreach (var a in EnumerateAttributes (g.Attributes, recurse))
+						yield return a;
+				}
+				else
+					yield return (XmlSchemaAttribute) o;
+			}
+		}
 
 		// Attributes might be redefined, so there is an existing attribute for the same name, skip it.
 		// FIXME: this is nothing more than just a hack.
@@ -756,7 +771,7 @@ namespace System.Xml.Serialization
 			XmlSchemaObjectCollection src, ClassMap map)
 		{
 			XmlSchemaObjectCollection atts = new XmlSchemaObjectCollection ();
-			foreach (XmlSchemaAttribute a in src)
+			foreach (var a in EnumerateAttributes (src, new List<XmlSchemaAttributeGroup> ()))
 				if (map.GetAttribute (a.QualifiedName.Name, a.QualifiedName.Namespace) == null)
 					atts.Add (a);
 			return atts;
@@ -944,12 +959,8 @@ namespace System.Xml.Serialization
 							// It is a simple list (space separated list).
 							// Since this is not supported, map as a single item value
 							member = new XmlTypeMapMemberElement ();
-#if NET_2_0
 							// In MS.NET those types are mapped to a string
 							typeData = TypeTranslator.GetTypeData(typeof(string));
-#else
-							typeData = typeData.ListItemTypeData;
-#endif
 						}
 						else
 							member = new XmlTypeMapMemberList ();
@@ -1581,12 +1592,30 @@ namespace System.Xml.Serialization
 			einfo.ElementName = name;
 			einfo.Namespace = ns;
 			einfo.IsNullable = isNillable;
-			einfo.Form = form;
+			einfo.Form = GetForm (form, ns, true);
 			if (typeData.IsComplexType)
 				einfo.MappedType = emap;
 			einfo.ExplicitOrder = order;
 			return einfo;
 		}
+		
+		XmlSchemaForm GetForm (XmlSchemaForm form, string ns, bool forElement)
+		{
+			// Returns the schema form for an element or attribute, taking
+			// into account the schema defaults. If the form has not been explicitly
+			// set and there is no default, use Unqualified as default.
+			
+			if (form != XmlSchemaForm.None)
+				return form;
+			XmlSchema s = schemas [ns];
+			if (s == null)
+				return XmlSchemaForm.Unqualified;
+			XmlSchemaForm schemaForm = forElement ? s.ElementFormDefault : s.AttributeFormDefault;
+			if (schemaForm != XmlSchemaForm.None)
+				return schemaForm;
+			else
+				return XmlSchemaForm.Unqualified;
+		}
 
 		XmlTypeMapElementInfo CreateTextElementInfo (string ns, XmlTypeMapMember member, TypeData typeData)
 		{
