@@ -64,27 +64,19 @@ TODOs:
 
 using System;
 using System.Collections;
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 using System.IO;
 using System.Text;
 using System.Xml;
 using System.Xml.Schema;
 
-#if NET_2_0
 using XmlTextReaderImpl = Mono.Xml2.XmlTextReader;
-#else
-using XmlTextReaderImpl = System.Xml.XmlTextReader;
-#endif
 
 
 namespace Mono.Xml
 {
 	internal class DTDValidatingReader : XmlReader, IXmlLineInfo, 
-#if NET_2_0
 		IXmlNamespaceResolver,
-#endif
 		IHasXmlParserContext, IHasXmlSchemaInfo
 	{
 		public DTDValidatingReader (XmlReader reader)
@@ -95,9 +87,7 @@ namespace Mono.Xml
 		internal DTDValidatingReader (XmlReader reader,
 			XmlValidatingReader validatingReader)
 		{
-			IHasXmlParserContext container = reader as IHasXmlParserContext;
-			this.reader = new EntityResolvingXmlReader (reader,
-				container != null ? container.ParserContext : null);
+			this.reader = new EntityResolvingXmlReader (reader);
 			this.sourceTextReader = reader as XmlTextReader;
 			elementStack = new Stack ();
 			automataStack = new Stack ();
@@ -259,13 +249,11 @@ namespace Mono.Xml
 			return i < 0 ? null : attributes [i].Value;
 		}
 
-#if NET_2_0
 		IDictionary<string, string> IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			IXmlNamespaceResolver res = reader as IXmlNamespaceResolver;
 			return res != null ? res.GetNamespacesInScope (scope) : new Dictionary<string, string> ();
 		}
-#endif
 
 		bool IXmlLineInfo.HasLineInfo ()
 		{
@@ -282,13 +270,11 @@ namespace Mono.Xml
 			return s == String.Empty ? null : s;
 		}
 
-#if NET_2_0
 		string IXmlNamespaceResolver.LookupPrefix (string ns)
 		{
 			IXmlNamespaceResolver res = reader as IXmlNamespaceResolver;
 			return res != null ? res.LookupPrefix (ns) : null;
 		}
-#endif
 
 		public override void MoveToAttribute (int i)
 		{
@@ -397,9 +383,7 @@ namespace Mono.Xml
 
 			bool b = ReadContent () || currentTextValue != null;
 			if (!b &&
-#if NET_2_0
 			    (Settings == null || (Settings.ValidationFlags & XmlSchemaValidationFlags.ProcessIdentityConstraints) == 0) &&
-#endif
 			    this.missingIDReferences.Count > 0) {
 				this.HandleError ("Missing ID reference was found: " +
 					String.Join (",", missingIDReferences.ToArray (typeof (string)) as string []),
@@ -968,11 +952,7 @@ namespace Mono.Xml
 			}
 		}
 
-#if NET_2_1
-		internal
-#else
 		public
-#endif
 		override bool ReadAttributeValue ()
 		{
 			if (consumedAttribute)
@@ -990,11 +970,7 @@ namespace Mono.Xml
 				return reader.ReadAttributeValue ();
 		}
 
-#if NET_2_1
-		internal
-#else
 		public
-#endif
 		override void ResolveEntity ()
 		{
 			reader.ResolveEntity ();
@@ -1033,11 +1009,7 @@ namespace Mono.Xml
 			get { return reader.EOF; }
 		}
 
-#if NET_2_1
-		internal
-#else
 		public
-#endif
 		override bool HasValue {
 			get {
 				return currentAttribute >= 0 ? true :
