@@ -26,7 +26,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 namespace System.Xml {
 
@@ -40,4 +39,3 @@ namespace System.Xml {
 
 }
 
-#endif
