@@ -34,23 +34,15 @@
 //
 
 using System;
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 using System.Xml;
 using System.Text;
 using Mono.Xml;
-#if NET_2_0
 using System.Xml.Schema;
-#endif
 
 namespace System.Xml
 {
-#if NET_2_0
 	internal class XmlNodeReaderImpl : XmlReader, IHasXmlParserContext, IXmlNamespaceResolver
-#else
-	internal class XmlNodeReaderImpl : XmlReader, IHasXmlParserContext
-#endif
 	{
 		XmlDocument document;
 		XmlNode startNode;
@@ -109,7 +101,6 @@ namespace System.Xml
 			}
 		}
 
-#if NET_2_0
 		public override bool CanReadBinaryContent {
 			get { return true; }
 		}
@@ -117,15 +108,6 @@ namespace System.Xml
 		public override bool CanReadValueChunk {
 			get { return true; }
 		}
-#else
-		internal override bool CanReadBinaryContent {
-			get { return true; }
-		}
-
-		internal override bool CanReadValueChunk {
-			get { return true; }
-		}
-#endif
 
 		public override bool CanResolveEntity {
 			get { return false; }
@@ -161,7 +143,6 @@ namespace System.Xml
 			}
 		}
 
-#if !NET_2_1
 		public override bool HasValue {
 			get {
 				if (current == null)
@@ -182,7 +163,6 @@ namespace System.Xml
 			}
 			      
 		}
-#endif
 
 		public override bool IsDefault {
 			get {
@@ -210,20 +190,6 @@ namespace System.Xml
 			}
 		}
 
-#if NET_2_0
-#else
-		public override string this [int i] {
-			get { return GetAttribute (i); }
-		}
-
-		public override string this [string name] {
-			get { return GetAttribute (name); }
-		}
-
-		public override string this [string name, string namespaceURI] {
-			get { return GetAttribute (name, namespaceURI); }
-		}
-#endif
 
 		public override string LocalName {
 			get {
@@ -294,24 +260,14 @@ namespace System.Xml
 			}
 		}
 
-#if NET_2_0
-#else
-		public override char QuoteChar {
-			get {
-				return '"';
-			}
-		}
-#endif
 
 		public override ReadState ReadState {
 			get { return state; }
 		}
 
-#if NET_2_0
 		public override IXmlSchemaInfo SchemaInfo {
 			get { return current != null ? current.SchemaInfo : null; }
 		}
-#endif
 
 		public override string Value {
 			get {
@@ -467,18 +423,20 @@ namespace System.Xml
 			}
 		}
 
-#if NET_2_0
 		public IDictionary<string, string> GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			IDictionary<string, string> table = new Dictionary<string, string> ();
-			XmlNode n = current;
+			XmlNode n = current ?? startNode;
 			do {
 				if (n.NodeType == XmlNodeType.Document)
 					break;
-				for (int i = 0; i < current.Attributes.Count; i++) {
-					XmlAttribute a = current.Attributes [i];
-					if (a.NamespaceURI == XmlNamespaceManager.XmlnsXmlns)
-						table.Add (a.Prefix == XmlNamespaceManager.PrefixXmlns ? a.LocalName : String.Empty, a.Value);
+				for (int i = 0; i < n.Attributes.Count; i++) {
+					XmlAttribute a = n.Attributes [i];
+					if (a.NamespaceURI == XmlNamespaceManager.XmlnsXmlns) {
+						string key = a.Prefix == XmlNamespaceManager.PrefixXmlns ? a.LocalName : String.Empty;
+						if (!table.ContainsKey (key))
+							table.Add (key, a.Value);
+					}
 				}
 				if (scope == XmlNamespaceScope.Local)
 					return table;
@@ -488,7 +446,6 @@ namespace System.Xml
 				table.Add (XmlNamespaceManager.PrefixXml, XmlNamespaceManager.XmlnsXml);
 			return table;
 		}
-#endif
 
 		private XmlElement GetCurrentElement ()
 		{
@@ -544,7 +501,6 @@ namespace System.Xml
 			return null;
 		}
 
-#if NET_2_0
 		public string LookupPrefix (string ns)
 		{
 			return LookupPrefix (ns, false);
@@ -583,7 +539,6 @@ namespace System.Xml
 			}
 			return null;
 		}
-#endif
 
 		public override void MoveToAttribute (int attributeIndex)
 		{
@@ -664,15 +619,21 @@ namespace System.Xml
 			if (current == null)
 				return false;
 
-			if (current.NodeType != XmlNodeType.Attribute)
-				return MoveToFirstAttribute ();
-			else
+			XmlNode anode = current;
+			if (current.NodeType != XmlNodeType.Attribute) {
+				// then it's either an attribute child or anything on the tree.
+				if (current.ParentNode == null ||  // document, or non-tree node
+				    current.ParentNode.NodeType != XmlNodeType.Attribute) // not an attr value
+					return MoveToFirstAttribute ();
+				anode = current.ParentNode;
+			}
+
 			{
-				XmlAttributeCollection ac = ((XmlAttribute) current).OwnerElement.Attributes;
+				XmlAttributeCollection ac = ((XmlAttribute) anode).OwnerElement.Attributes;
 				for (int i=0; i<ac.Count-1; i++)
 				{
 					XmlAttribute attr = ac [i];
-					if (attr == current)
+					if (attr == anode)
 					{
 						i++;
 						if (i == ac.Count)
@@ -696,10 +657,8 @@ namespace System.Xml
 				return false;
 			}
 
-#if NET_2_0
 			if (Binary != null)
 				Binary.Reset ();
-#endif
 
 			bool ret = ReadContent ();
 			ownerLinkedNode = current;
@@ -789,7 +748,6 @@ namespace System.Xml
 			}
 		}
 
-#if !NET_2_1
 		public override bool ReadAttributeValue ()
 		{
 			if (current.NodeType == XmlNodeType.Attribute) {
@@ -805,19 +763,16 @@ namespace System.Xml
 			} else
 				return false;
 		}
-#endif
 
 		public override string ReadString ()
 		{
 			return base.ReadString ();
 		}
 
-#if !NET_2_1
 		public override void ResolveEntity ()
 		{
 			throw new NotSupportedException ("Should not happen.");
 		}
-#endif
 
 		public override void Skip ()
 		{
