@@ -41,32 +41,19 @@
 
 using System;
 using System.Collections;
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 using System.Globalization;
 using System.IO;
 using System.Security.Permissions;
 using System.Text;
 using System.Xml.Schema;
 using Mono.Xml;
-
-#if NET_2_0
 using System.Xml;
 
 namespace Mono.Xml2
-#else
-namespace System.Xml
-#endif
 {
-
-#if NET_2_0
-	internal class XmlTextReader : XmlReader,
+	class XmlTextReader : XmlReader,
 		IXmlLineInfo, IXmlNamespaceResolver, IHasXmlParserContext
-#else
-	[PermissionSet (SecurityAction.InheritanceDemand, Unrestricted = true)]
-	public class XmlTextReader : XmlReader, IXmlLineInfo, IHasXmlParserContext
-#endif
 	{
 		#region Constructors
 
@@ -111,13 +98,13 @@ namespace System.Xml
 
 		public XmlTextReader (string url, XmlNameTable nt)
 		{
-			string uriString;
-			Stream stream = GetStreamFromUrl (url, out uriString);
+			reader_uri = resolver.ResolveUri (null, url);
+			string uriString = (reader_uri == null) ? String.Empty : reader_uri.ToString ();
 			XmlParserContext ctx = new XmlParserContext (nt,
 				new XmlNamespaceManager (nt),
 				String.Empty,
 				XmlSpace.None);
-			this.InitializeContext (uriString, ctx, new XmlStreamReader (stream), XmlNodeType.Document);
+			this.InitializeContext (uriString, ctx, null, XmlNodeType.Document);
 		}
 
 		public XmlTextReader (TextReader input, XmlNameTable nt)
@@ -130,14 +117,11 @@ namespace System.Xml
 		internal XmlTextReader (bool dummy, XmlResolver resolver, string url, XmlNodeType fragType, XmlParserContext context)
 		{
 			if (resolver == null) {
-#if MOONLIGHT
-				resolver = new XmlXapResolver ();
-#else
 				resolver = new XmlUrlResolver ();
-#endif
 			}
 			this.XmlResolver = resolver;
 			string uriString;
+
 			Stream stream = GetStreamFromUrl (url, out uriString);
 			this.InitializeContext (uriString, context, new XmlStreamReader (stream), fragType);
 		}
@@ -180,7 +164,7 @@ namespace System.Xml
 			InitializeContext (url, context, fragment, fragType);
 		}
 
-		private Stream GetStreamFromUrl (string url, out string absoluteUriString)
+		Stream GetStreamFromUrl (string url, out string absoluteUriString)
 		{
 #if NET_2_1
 			if (url == null)
@@ -188,9 +172,13 @@ namespace System.Xml
 			if (url.Length == 0)
 				throw new ArgumentException ("url");
 #endif
-			Uri uri = resolver.ResolveUri (null, url);
+			//
+			// This needs to work even if resolver is explicitly set to null
+			//
+			var res = resolver ?? new XmlUrlResolver ();
+			var uri = res.ResolveUri (null, url);
 			absoluteUriString = uri != null ? uri.ToString () : String.Empty;
-			return resolver.GetEntity (uri, null, typeof (Stream)) as Stream;
+			return res.GetEntity (uri, null, typeof (Stream)) as Stream;
 		}
 
 		#endregion
@@ -207,7 +195,6 @@ namespace System.Xml
 			get { return parserContext.BaseURI; }
 		}
 
-#if NET_2_0
 		public override bool CanReadBinaryContent {
 			get { return true; }
 		}
@@ -215,15 +202,6 @@ namespace System.Xml
 		public override bool CanReadValueChunk {
 			get { return true; }
 		}
-#else
-		internal override bool CanReadBinaryContent {
-			get { return true; }
-		}
-
-		internal override bool CanReadValueChunk {
-			get { return true; }
-		}
-#endif
 
 		internal bool CharacterChecking {
 			get { return checkCharacters; }
@@ -252,12 +230,11 @@ namespace System.Xml
 		{
 			get { return parserContext.Encoding; }
 		}
-#if NET_2_0
+
 		public EntityHandling EntityHandling {
 			get { return entityHandling; }
 			set { entityHandling = value; }
 		}
-#endif
 
 		public override bool EOF {
 			get { return readState == ReadState.EndOfFile; }
@@ -276,21 +253,6 @@ namespace System.Xml
 			get { return cursorToken.IsEmptyElement; }
 		}
 
-#if NET_2_0
-#else
-		public override string this [int i] {
-			get { return GetAttribute (i); }
-		}
-
-		public override string this [string name] {
-			get { return GetAttribute (name); }
-		}
-
-		public override string this [string localName, string namespaceName] {
-			get { return GetAttribute (localName, namespaceName); }
-		}
-#endif
-
 		public int LineNumber {
 			get {
 				if (useProceedingLineInfo)
@@ -360,11 +322,9 @@ namespace System.Xml
 			get { return readState; }
 		}
 
-#if NET_2_0
 		public override XmlReaderSettings Settings {
 			get { return base.Settings; }
 		}
-#endif
 
 		public override string Value {
 			get { return cursorToken.Value != null ? cursorToken.Value : String.Empty; }
@@ -445,7 +405,6 @@ namespace System.Xml
 			return attributeTokens [idx].Value;
 		}
 
-#if NET_2_0
 		public IDictionary<string, string> GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			return nsmgr.GetNamespacesInScope (scope);
@@ -455,7 +414,6 @@ namespace System.Xml
 		{
 			return GetNamespacesInScope (scope);
 		}
-#endif
 
 		public TextReader GetRemainder ()
 		{
@@ -464,11 +422,7 @@ namespace System.Xml
 			return new StringReader (new string (peekChars, peekCharsIndex, peekCharsLength - peekCharsIndex) + reader.ReadToEnd ());
 		}
 
-#if NET_2_0
 		public bool HasLineInfo ()
-#else
-		bool IXmlLineInfo.HasLineInfo ()
-#endif
 		{
 			return true;
 		}
@@ -485,7 +439,6 @@ namespace System.Xml
 			return s == String.Empty ? null : s;
 		}
 
-#if NET_2_0
 		string IXmlNamespaceResolver.LookupPrefix (string ns)
 		{
 			return LookupPrefix (ns, false);
@@ -495,7 +448,6 @@ namespace System.Xml
 		{
 			return nsmgr.LookupPrefix (ns, atomizedName);
 		}
-#endif
 
 		public override void MoveToAttribute (int i)
 		{
@@ -714,13 +666,11 @@ namespace System.Xml
 			throw new InvalidOperationException ("XmlTextReader cannot resolve external entities.");
 		}
 
-#if NET_2_0
 		[MonoTODO] // FIXME: Implement, for performance improvement
 		public override void Skip ()
 		{
 			base.Skip ();
 		}
-#endif
 		#endregion
 
 		#region Internals
@@ -766,7 +716,6 @@ namespace System.Xml
 					if (valueCache != null)
 						return valueCache;
 					if (ValueBufferStart >= 0) {
-//Console.WriteLine (NodeType + " / " + ValueBuffer.Length + " / " + ValueBufferStart + " / " + ValueBufferEnd);
 						valueCache = Reader.valueBuffer.ToString (ValueBufferStart, ValueBufferEnd - ValueBufferStart);
 						return valueCache;
 					}
@@ -928,6 +877,7 @@ namespace System.Xml
 
 		private StringBuilder valueBuffer;
 
+		Uri reader_uri;
 		private TextReader reader;
 		private char [] peekChars;
 		private int peekCharsIndex;
@@ -959,11 +909,7 @@ namespace System.Xml
 		// These values are never re-initialized.
 		private bool namespaces = true;
 		private WhitespaceHandling whitespaceHandling = WhitespaceHandling.All;
-#if MOONLIGHT
-		private XmlResolver resolver = new XmlXapResolver ();
-#else
 		private XmlResolver resolver = new XmlUrlResolver ();
-#endif
 		private bool normalization = false;
 
 		private bool checkCharacters;
@@ -990,10 +936,8 @@ namespace System.Xml
 #endif
 
 			checkCharacters = true;
-#if NET_2_0
 			if (Settings != null)
 				checkCharacters = Settings.CheckCharacters;
-#endif
 			prohibitDtd = false;
 			closeInput = true;
 			entityHandling = EntityHandling.ExpandCharEntities;
@@ -1066,11 +1010,7 @@ namespace System.Xml
 #else
 				Uri uri = null;
 				try {
-#if NET_2_0
 					uri = new Uri (url, UriKind.RelativeOrAbsolute);
-#else
-					uri = new Uri (url);
-#endif
 				} catch (Exception) {
 					string path = Path.GetFullPath ("./a");
 					uri = new Uri (new Uri (path), url);
@@ -1098,7 +1038,6 @@ namespace System.Xml
 			}
 		}
 
-#if NET_2_0
 		internal ConformanceLevel Conformance {
 			get { return allowMultipleRoot ? ConformanceLevel.Fragment : ConformanceLevel.Document; }
 			set {
@@ -1119,7 +1058,6 @@ namespace System.Xml
 		{
 			parserContext.NameTable = nameTable;
 		}
-#endif
 
 		// Use this method rather than setting the properties
 		// directly so that all the necessary properties can
@@ -1239,6 +1177,15 @@ namespace System.Xml
 
 		private bool ReadTextReader (int remained)
 		{
+			if (reader == null && reader_uri != null) {
+				Uri uri = reader_uri;
+				reader_uri = null;
+				string uriString;
+				Stream stream = GetStreamFromUrl (uri.ToString (), out uriString);
+				if (stream == null)
+					return false;
+				reader = new XmlStreamReader (stream);
+			}
 			if (peekCharsLength < 0) {	// initialized buffer
 				peekCharsLength = reader.Read (peekChars, 0, peekChars.Length);
 				return peekCharsLength > 0;
@@ -1612,7 +1559,7 @@ namespace System.Xml
 					break;
 				if (whitespacePool == null)
 					whitespacePool = new NameTable ();
-#if NET_2_0 && !NET_2_1
+#if !NET_2_1
 				valueBuffer.CopyTo (0, whitespaceCache, 0, len);
 #else
 				for (int i = 0; i < len; i++)
@@ -1852,6 +1799,7 @@ namespace System.Xml
 				value,
 				false);
 			ati.Value = value;
+			ati.ValueTokenStartIndex = ati.ValueTokenEndIndex = currentAttributeValue;
 			attributeCount++;
 		}
 
@@ -1961,14 +1909,11 @@ namespace System.Xml
 					int predefined = XmlChar.GetPredefinedEntity (entName);
 					if (predefined < 0) {
 						CheckAttributeEntityReferenceWFC (entName);
-#if NET_2_0
 						if (entityHandling == EntityHandling.ExpandEntities) {
 							string value = DTD.GenerateEntityAttributeText (entName);
 							foreach (char c in (IEnumerable<char>) value)
 								AppendValueChar (c);
-						} else
-#endif
-						{
+						} else {
 							currentAttributeValueToken.ValueBufferEnd = valueBuffer.Length;
 							currentAttributeValueToken.NodeType = XmlNodeType.Text;
 							if (!isNewToken)
@@ -2259,13 +2204,11 @@ namespace System.Xml
 				}
 				// Encoding value should be checked inside XmlInputStream.
 			}
-#if NET_2_0
 			// this condition is to check if this instance is
 			// not created by XmlReader.Create() (which just
 			// omits strict text declaration check).
 			else if (Conformance == ConformanceLevel.Auto)
 				throw NotWFError ("Encoding declaration is mandatory in text declaration.");
-#endif
 
 			Expect ("?>");
 
