@@ -35,21 +35,15 @@ using System;
 using System.Collections;
 using System.IO;
 using System.Text;
-#if NET_2_0 && !NET_2_1
 using System.Xml.XPath;
-#endif
+using System.Threading;
+using System.Threading.Tasks;
 
 namespace System.Xml
 {
-#if NET_2_0
 	public abstract class XmlWriter : IDisposable
-#else
-	public abstract class XmlWriter
-#endif
 	{
-#if NET_2_0
 		XmlWriterSettings settings;
-#endif
 
 		#region Constructors
 
@@ -59,20 +53,12 @@ namespace System.Xml
 
 		#region Properties
 
-#if NET_2_0
 		public virtual XmlWriterSettings Settings {
-			get {
-				if (settings == null)
-					settings = new XmlWriterSettings ();
-				return settings;
-			}
+			get { return settings; }
 		}
-#endif
 
 		public abstract WriteState WriteState { get; }
-		
 
-#if NET_2_0
 		public virtual string XmlLang {
 			get { return null; }
 		}
@@ -80,74 +66,104 @@ namespace System.Xml
 		public virtual XmlSpace XmlSpace {
 			get { return XmlSpace.None; }
 		}
-#else
-		public abstract string XmlLang { get; }
-
-		public abstract XmlSpace XmlSpace { get; }
-#endif
 
 		#endregion
 
 		#region Methods
 
-		public abstract void Close ();
-
-#if NET_2_0
-		public static XmlWriter Create (Stream stream)
+		public virtual void Close ()
 		{
-			return Create (stream, null);
+			if (asyncRunning)
+				throw new InvalidOperationException ("An asynchronous operation is already in progress.");
 		}
 
-		public static XmlWriter Create (string file)
+		public static XmlWriter Create (Stream output)
 		{
-			return Create (file, null);
+			return Create (output, null);
 		}
 
-		public static XmlWriter Create (TextWriter writer)
+		public static XmlWriter Create (string outputFileName)
 		{
-			return Create (writer, null);
+			return Create (outputFileName, null);
 		}
 
-		public static XmlWriter Create (XmlWriter writer)
+		public static XmlWriter Create (TextWriter output)
 		{
-			return Create (writer, null);
+			return Create (output, null);
 		}
 
-		public static XmlWriter Create (StringBuilder builder)
+		public static XmlWriter Create (XmlWriter output)
 		{
-			return Create (builder, null);
+			return Create (output, null);
 		}
 
-		public static XmlWriter Create (Stream stream, XmlWriterSettings settings)
+		public static XmlWriter Create (StringBuilder output)
+		{
+			return Create (output, null);
+		}
+
+		public static XmlWriter Create (Stream output, XmlWriterSettings settings)
 		{
 			Encoding enc = settings != null ? settings.Encoding : Encoding.UTF8;
-			return Create (new StreamWriter (stream, enc), settings);
+			return Create (new StreamWriter (output, enc), settings);
 		}
 
-		public static XmlWriter Create (string file, XmlWriterSettings settings)
+		public static XmlWriter Create (string outputFileName, XmlWriterSettings settings)
 		{
 			Encoding enc = settings != null ? settings.Encoding : Encoding.UTF8;
-			return CreateTextWriter (new StreamWriter (file, false, enc), settings, true);
+			return CreateTextWriter (new StreamWriter (outputFileName, false, enc), settings, true);
 		}
 
-		public static XmlWriter Create (StringBuilder builder, XmlWriterSettings settings)
+		public static XmlWriter Create (StringBuilder output, XmlWriterSettings settings)
 		{
-			return Create (new StringWriter (builder), settings);
+			return Create (new StringWriter (output), settings);
 		}
 
-		public static XmlWriter Create (TextWriter writer, XmlWriterSettings settings)
+		public static XmlWriter Create (TextWriter output, XmlWriterSettings settings)
 		{
 			if (settings == null)
 				settings = new XmlWriterSettings ();
-			return CreateTextWriter (writer, settings, settings.CloseOutput);
+			return CreateTextWriter (output, settings, settings.CloseOutput);
 		}
 
-		public static XmlWriter Create (XmlWriter writer, XmlWriterSettings settings)
+		public static XmlWriter Create (XmlWriter output, XmlWriterSettings settings)
 		{
 			if (settings == null)
 				settings = new XmlWriterSettings ();
-			writer.settings = settings;
-			return writer;
+			else
+				settings = settings.Clone ();
+
+			var src = output.Settings;
+			if (src == null) {
+				settings.ConformanceLevel = ConformanceLevel.Document; // Huh? Why??
+				output = new DefaultXmlWriter (output);
+				settings.SetReadOnly ();
+				output.settings = settings;
+			} else {
+				ConformanceLevel dst = src.ConformanceLevel;
+				switch (src.ConformanceLevel) {
+				case ConformanceLevel.Auto:
+					dst = settings.ConformanceLevel;
+					break;
+				case ConformanceLevel.Document:
+				case ConformanceLevel.Fragment:
+					if (settings.ConformanceLevel != ConformanceLevel.Auto)
+						dst = settings.ConformanceLevel;
+					break;
+				}
+
+				settings.MergeFrom (src);
+
+				settings.SetReadOnly ();
+
+				// It returns a new XmlWriter instance if 1) Settings is null, or 2) Settings ConformanceLevel (or might be other members as well) give significant difference.
+				if (src.ConformanceLevel != dst) {
+					output = new DefaultXmlWriter (output, false);
+					output.settings = settings;
+				}
+			}
+
+			return output;
 		}
 
 		private static XmlWriter CreateTextWriter (TextWriter writer, XmlWriterSettings settings, bool closeOutput)
@@ -163,11 +179,10 @@ namespace System.Xml
 			Close ();
 		}
 
-		void IDisposable.Dispose ()
+		public void Dispose ()
 		{
 			Dispose (false);
 		}
-#endif
 
 		public abstract void Flush ();
 
@@ -179,10 +194,6 @@ namespace System.Xml
 				return;
 
 			WriteStartAttribute (reader.Prefix, reader.LocalName, reader.NamespaceURI);
-#if NET_2_1
-			// no ReadAttributeValue() in 2.1 profile.
-			WriteString (reader.Value);
-#else
 			while (reader.ReadAttributeValue ()) {
 				switch (reader.NodeType) {
 				case XmlNodeType.Text:
@@ -193,7 +204,6 @@ namespace System.Xml
 					break;
 				}
 			}
-#endif
 			WriteEndAttribute ();
 		}
 
@@ -248,16 +258,12 @@ namespace System.Xml
 
 		public abstract void WriteBase64 (byte[] buffer, int index, int count);
 
-#if NET_2_0
 		public virtual void WriteBinHex (byte [] buffer, int index, int count)
 		{
 			StringWriter sw = new StringWriter ();
 			XmlConvert.WriteBinHex (buffer, index, count, sw);
 			WriteString (sw.ToString ());
 		}
-#else
-		public abstract void WriteBinHex (byte[] buffer, int index, int count);
-#endif
 
 		public abstract void WriteCData (string text);
 
@@ -285,7 +291,6 @@ namespace System.Xml
 			WriteEndElement();
 		}
 
-#if NET_2_0
 		public void WriteElementString (string prefix, string localName, string ns, string value)
 		{
 			WriteStartElement(prefix, localName, ns);
@@ -293,7 +298,6 @@ namespace System.Xml
 				WriteString(value);
 			WriteEndElement();
 		}
-#endif
 
 		public abstract void WriteEndAttribute ();
 
@@ -305,7 +309,6 @@ namespace System.Xml
 
 		public abstract void WriteFullEndElement ();
 
-#if NET_2_0
 		public virtual void WriteName (string name)
 		{
 			WriteNameInternal (name);
@@ -320,42 +323,27 @@ namespace System.Xml
 		{
 			WriteQualifiedNameInternal (localName, ns);
 		}
-#else
-		public abstract void WriteName (string name);
-
-		public abstract void WriteNmToken (string name);
-
-		public abstract void WriteQualifiedName (string localName, string ns);
-#endif
 
 		internal void WriteNameInternal (string name)
 		{
-#if NET_2_0
 			switch (Settings.ConformanceLevel) {
 			case ConformanceLevel.Document:
 			case ConformanceLevel.Fragment:
 				XmlConvert.VerifyName (name);
 				break;
 			}
-#else
-			XmlConvert.VerifyName (name);
-#endif
 			WriteString (name);
 		}
 
 		internal virtual void WriteNmTokenInternal (string name)
 		{
 			bool valid = true;
-#if NET_2_0
 			switch (Settings.ConformanceLevel) {
 			case ConformanceLevel.Document:
 			case ConformanceLevel.Fragment:
 				valid = XmlChar.IsNmToken (name);
 					break;
 			}
-#else
-			valid = XmlChar.IsNmToken (name);
-#endif
 			if (!valid)
 				throw new ArgumentException ("Argument name is not a valid NMTOKEN.");
 			WriteString (name);
@@ -368,16 +356,16 @@ namespace System.Xml
 			if (ns == null)
 				ns = String.Empty;
 
-#if NET_2_0
-			switch (Settings.ConformanceLevel) {
-			case ConformanceLevel.Document:
-			case ConformanceLevel.Fragment:
-				XmlConvert.VerifyNCName (localName);
-				break;
+			if (Settings != null) {
+				switch (Settings.ConformanceLevel) {
+				case ConformanceLevel.Document:
+				case ConformanceLevel.Fragment:
+					XmlConvert.VerifyNCName (localName);
+					break;
+				}
 			}
-#else
-			XmlConvert.VerifyNCName (localName);
-#endif
+			else
+				XmlConvert.VerifyNCName (localName);
 
 			string prefix = ns.Length > 0 ? LookupPrefix (ns) : String.Empty;
 			if (prefix == null)
@@ -392,7 +380,6 @@ namespace System.Xml
 				WriteString (localName);
 		}
 
-#if NET_2_0 && !NET_2_1
 		public virtual void WriteNode (XPathNavigator navigator, bool defattr)
 		{
 			if (navigator == null)
@@ -462,7 +449,6 @@ namespace System.Xml
 				throw new NotSupportedException ();
 			}
 		}
-#endif
 
 		public virtual void WriteNode (XmlReader reader, bool defattr)
 		{
@@ -557,12 +543,10 @@ namespace System.Xml
 
 		public abstract void WriteRaw (char[] buffer, int index, int count);
 
-#if NET_2_0
 		public void WriteStartAttribute (string localName)
 		{
 			WriteStartAttribute (null, localName, null);
 		}
-#endif
 
 		public void WriteStartAttribute (string localName, string ns)
 		{
@@ -593,7 +577,6 @@ namespace System.Xml
 
 		public abstract void WriteWhitespace (string ws);
 
-#if NET_2_0
 		public virtual void WriteValue (bool value)
 		{
 			WriteString (XQueryConvert.BooleanToString (value));
@@ -655,6 +638,8 @@ namespace System.Xml
 				WriteValue ((float) value);
 			else if (value is TimeSpan) // undocumented
 				WriteString (XmlConvert.ToString ((TimeSpan) value));
+			else if (value is Uri)
+				WriteString (((Uri) value).ToString ());
 			else if (value is XmlQualifiedName) {
 				XmlQualifiedName qname = (XmlQualifiedName) value;
 				if (!qname.Equals (XmlQualifiedName.Empty)) {
@@ -688,7 +673,403 @@ namespace System.Xml
 		{
 			WriteString (value);
 		}
-#endif
+
+		public virtual void WriteValue (DateTimeOffset value)
+		{
+			WriteString (XmlConvert.ToString (value));
+		}
+
+		#endregion
+
+		#region .NET 4.5 Async Methods
+
+		bool asyncRunning;
+
+		void StartAsync ()
+		{
+			if (!settings.Async)
+				throw new InvalidOperationException ("Set XmlWriterSettings.Async to true if you want to use Async Methods.");
+			lock (this) {
+				if (asyncRunning)
+					throw new InvalidOperationException ("An asynchronous operation is already in progress.");
+				asyncRunning = true;
+			}
+		}
+
+		public virtual Task FlushAsync ()
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					Flush ();
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteAttributesAsync (XmlReader reader, bool defattr)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteAttributes (reader, defattr);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public Task WriteAttributeStringAsync (string prefix, string localName,
+		                                       string ns, string value)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteAttributeString (prefix, localName, ns, value);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteBase64Async (byte[] buffer, int index, int count)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteBase64 (buffer, index, count);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteBinHexAsync (byte[] buffer, int index, int count)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteBinHex (buffer, index, count);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteCDataAsync (string text)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteCData (text);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteCharEntityAsync (char ch)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteCharEntity (ch);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteCharsAsync (char[] buffer, int index, int count)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteChars (buffer, index, count);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteCommentAsync (string text)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteComment (text);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteDocTypeAsync (string name, string pubid, string sysid, string subset)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteDocType (name, pubid, sysid, subset);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public Task WriteElementStringAsync (string prefix, string localName, string ns, string value)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteElementString (prefix, localName, ns, value);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		protected internal virtual Task WriteEndAttributeAsync ()
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteEndAttribute ();
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteEndDocumentAsync ()
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteEndDocument ();
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteEndElementAsync ()
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteEndElement ();
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteEntityRefAsync (string name)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteEntityRef (name);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteFullEndElementAsync ()
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteFullEndElement ();
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteNameAsync (string name)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteName (name);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteNmTokenAsync (string name)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteNmToken (name);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteNodeAsync (XmlReader reader, bool defattr)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteNode (reader, defattr);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteNodeAsync (XPathNavigator navigator, bool defattr)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteNode (navigator, defattr);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteProcessingInstructionAsync (string name, string text)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteProcessingInstruction (name, text);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteQualifiedNameAsync (string localName, string ns)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteQualifiedName (localName, ns);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteRawAsync (string data)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteRaw (data);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+
+		}
+
+		public virtual Task WriteRawAsync (char[] buffer, int index, int count)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteRaw (buffer, index, count);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		protected internal virtual Task WriteStartAttributeAsync (
+			string prefix, string localName, string ns)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteStartAttribute (prefix, localName, ns);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteStartDocumentAsync ()
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteStartDocument ();
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteStartDocumentAsync (bool standalone)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteStartDocument (standalone);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteStartElementAsync (string prefix, string localName, string ns)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteStartElement (prefix, localName, ns);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteStringAsync (string text)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteString (text);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteSurrogateCharEntityAsync (char lowChar, char highChar)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteSurrogateCharEntity (lowChar, highChar);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
+
+		public virtual Task WriteWhitespaceAsync (string ws)
+		{
+			StartAsync ();
+			return Task.Run (() => {
+				try {
+					WriteWhitespace (ws);
+				} finally {
+					asyncRunning = false;
+				}
+			});
+		}
 
 		#endregion
 	}
