@@ -173,6 +173,7 @@ namespace MonoTests.System.Xml
 		}
 
 		[Test]
+		[Category ("MobileNotWorking")]
 		public void TestSimpleMutualImport ()
 		{
 			XmlReader r = new XmlTextReader ("Test/XmlFiles/xsd/inter-inc-1.xsd");
@@ -429,7 +430,6 @@ namespace MonoTests.System.Xml
 
 			XmlSchema schema = GetSchema ("Test/XmlFiles/xsd/extension-attr-redefine-1.xsd");
 
-#if NET_2_0
 			XmlSchemaSet xss = new XmlSchemaSet ();
 			xss.Add (schema);
 			if (StrictMsCompliant) {
@@ -449,23 +449,6 @@ namespace MonoTests.System.Xml
 			settings.ValidationType = ValidationType.Schema;
 			settings.Schemas = xss;
 			XmlReader vr = XmlReader.Create (sr, settings);
-#else
-			if (StrictMsCompliant) {
-				schema.Compile (null);
-			} else {
-				try {
-					schema.Compile (null);
-					Assert.Fail ();
-				} catch (XmlSchemaException) {
-				}
-				return;
-			}
-
-			XmlValidatingReader vr = new XmlValidatingReader (xml,
-				XmlNodeType.Document, null);
-			vr.Schemas.Add (schema);
-			vr.ValidationType = ValidationType.Schema;
-#endif
 
 			try {
 				vr.Read ();
@@ -481,7 +464,6 @@ namespace MonoTests.System.Xml
 
 			XmlSchema schema = GetSchema ("Test/XmlFiles/xsd/extension-attr-redefine-2.xsd");
 
-#if NET_2_0
 			XmlSchemaSet xss = new XmlSchemaSet ();
 			xss.Add (schema);
 			xss.Compile ();
@@ -492,14 +474,6 @@ namespace MonoTests.System.Xml
 			settings.ValidationType = ValidationType.Schema;
 			settings.Schemas = xss;
 			XmlReader vr = XmlReader.Create (sr, settings);
-#else
-			schema.Compile (null);
-
-			XmlValidatingReader vr = new XmlValidatingReader (xml,
-				XmlNodeType.Document, null);
-			vr.Schemas.Add (schema);
-			vr.ValidationType = ValidationType.Schema;
-#endif
 
 			while (vr.Read ()) ;
 		}
@@ -511,7 +485,6 @@ namespace MonoTests.System.Xml
 
 			XmlSchema schema = GetSchema ("Test/XmlFiles/xsd/extension-attr-redefine-3.xsd");
 
-#if NET_2_0
 			XmlSchemaSet xss = new XmlSchemaSet ();
 			xss.Add (schema);
 			if (StrictMsCompliant) {
@@ -531,28 +504,10 @@ namespace MonoTests.System.Xml
 			settings.ValidationType = ValidationType.Schema;
 			settings.Schemas = xss;
 			XmlReader vr = XmlReader.Create (sr, settings);
-#else
-			if (StrictMsCompliant) {
-				schema.Compile (null);
-			} else {
-				try {
-					schema.Compile (null);
-					Assert.Fail ();
-				} catch (XmlSchemaException) {
-				}
-				return;
-			}
-
-			XmlValidatingReader vr = new XmlValidatingReader (xml,
-				XmlNodeType.Document, null);
-			vr.Schemas.Add (schema);
-			vr.ValidationType = ValidationType.Schema;
-#endif
 
 			while (vr.Read ()) ;
 		}
 
-#if NET_2_0
 
 		internal class XmlTestResolver : XmlResolver
 		{			
@@ -585,7 +540,7 @@ namespace MonoTests.System.Xml
 		public void TestResolveUri ()
 		{
 			XmlSchemaSet schemaSet = new XmlSchemaSet ();
-			FileStream stream = new FileStream ("Test/XmlFiles/xsd/resolveUriSchema.xsd", FileMode.Open);
+			FileStream stream = new FileStream ("Test/XmlFiles/xsd/resolveUriSchema.xsd", FileMode.Open, FileAccess.Read);
 			schemaSet.Add ("http://tempuri.org/resolveUriSchema.xsd", new XmlTextReader (stream));
 
 			XmlTestResolver resolver = new XmlTestResolver ();		
@@ -626,7 +581,6 @@ namespace MonoTests.System.Xml
 			// Parse the file. 
 			while (reader.Read()) {}
 		}
-#endif
 
 		[Test]
 		public void TestImportSchemaThatIncludesAnother ()
