@@ -13,9 +13,7 @@
 
 using System;
 using System.Collections;
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 using System.ComponentModel;
 using System.Xml;
 using System.Xml.Schema;
@@ -68,7 +66,6 @@ namespace MonoTests.System.Xml.TestClasses
 
 	#region GenericsTestClasses
 
-#if NET_2_0
 	public class GenSimpleClass<T>
 	{
 		public T something = default (T);
@@ -166,7 +163,6 @@ namespace MonoTests.System.Xml.TestClasses
 		bb
 	}
 	
-#endif
 
 	#endregion // GenericsTestClasses
 
@@ -1117,5 +1113,83 @@ namespace MonoTests.System.Xml.TestClasses
 		[XmlElement ("Extra", Order=1)]
 		public string[] Extra;
 	}
+
+	public class SimpleObjectA
+	{
+		[XmlAttribute]
+		public string Text
+		{
+			get; set;
+		}
+
+		public static implicit operator SimpleObjectA (SimpleObjectB o)
+		{
+			return new SimpleObjectA { Text = o.Text };
+		}
+
+		public static implicit operator SimpleObjectB (SimpleObjectA o)
+		{
+		 return new SimpleObjectB { Text = o.Text };
+		}
+	}
+
+	public class SimpleObjectB
+	{
+		[XmlAttribute]
+		public string Text
+		{
+			get; set;
+		}
+	}
+
+	public class ObjectWithElementRequiringImplicitCast
+	{
+		public ObjectWithElementRequiringImplicitCast () { }
+		public ObjectWithElementRequiringImplicitCast (string text)
+		{
+			Object = new SimpleObjectB { Text = text };
+		}
+
+		[XmlElement(Type = typeof (SimpleObjectA))]
+		public SimpleObjectB Object
+		{
+			get; set;
+		}
+	}
+
+	public class ObjectWithNullableArrayItems
+	{
+		[XmlArrayItem ("Element", IsNullable = true)]
+		public List<SimpleClass> Elements;
+	}
+
+	public class ObjectWithNonNullableArrayItems
+	{
+		[XmlArrayItem ("Element", IsNullable = false)]
+		public List<SimpleClass> Elements;
+	}
+
+	public class ObjectWithNotSpecifiedNullableArrayItems
+	{
+		[XmlArrayItem ("Element")]
+		public List<SimpleClass> Elements;
+	}
+
+	[Serializable]
+	public sealed class ClassWithDefaultTextNotNull
+	{
+		[XmlText]
+		public string Value;
+
+		public const string DefaultValue = "NotNull";
+
+		public ClassWithDefaultTextNotNull (string v) {
+			Value = v;
+		}
+
+		public ClassWithDefaultTextNotNull () {
+			Value = DefaultValue;
+		}
+    }
 }
 
