@@ -7,7 +7,6 @@
 // Copyright (C) 2005 Novell, Inc.  http://www.novell.com
 //
 
-#if NET_2_0
 
 using System;
 using System.Xml;
@@ -780,7 +779,15 @@ namespace MonoTests.System.Xml.XPath
 			using (XmlWriter w = y.CreateNavigator ().AppendChild ())
 				w.WriteAttributeString ("test", "test1");
 		}
+
+		[Test] // bug #1558
+		public void CreateNavigatorReturnsEdidable ()
+		{
+			var document = new XmlDocument();
+			document.LoadXml ("<div>hello world</div>");
+			XPathNavigator navigator = document.CreateNavigator ().CreateNavigator ();
+			navigator.SelectSingleNode ("//div").SetValue ("hello world 2");
+		}
 	}
 }
 
-#endif
