@@ -18,11 +18,7 @@ using System.Text;
 
 using NUnit.Framework;
 
-#if NET_2_0
 using InvalidNodeTypeArgException = System.ArgumentException;
-#else // it makes less sense
-using InvalidNodeTypeArgException = System.ArgumentOutOfRangeException;
-#endif
 
 namespace MonoTests.System.Xml
 {
@@ -201,9 +197,7 @@ namespace MonoTests.System.Xml
 		}
 
 		[Test]
-#if NET_2_0
 		[Category ("NotDotNet")] // enbug in 2.0
-#endif
 		public void CreateNodeNodeTypeName ()
 		{
 			XmlNode node;
@@ -1198,5 +1192,24 @@ namespace MonoTests.System.Xml
 			Assert.IsNotNull (doc.DocumentElement.GetAttributeNode ("hijacked"));
 			Assert.IsNull (doc.DocumentElement.GetAttributeNode ("a"));
 		}
+
+		[Test]
+		public void LoadFromMiddleOfDocument ()
+		{
+			// bug #598953
+			string xml = @"<?xml version='1.0' encoding='utf-8' ?>
+<Racal>
+  <Ports>
+    <ConsolePort value='9998' />
+  </Ports>
+</Racal>";
+			var r = new XmlTextReader (new StringReader (xml));
+			r.WhitespaceHandling = WhitespaceHandling.All;
+			r.MoveToContent ();
+			r.Read ();
+			var doc = new XmlDocument ();
+			doc.Load (r);
+			Assert.AreEqual (XmlNodeType.EndElement, r.NodeType, "#1");
+		}
 	}
 }
