@@ -33,8 +33,6 @@ namespace System.Xml.Linq
 	{
 		None = 0,
 		DisableFormatting = 1,
-#if NET_4_0 || MOONLIGHT
 		OmitDuplicateNamespaces = 2
-#endif
 	}
 }
