@@ -4,9 +4,11 @@
 // Authors:
 //	Tim Coleman (tim@timcoleman.com)
 //	Atsushi Enomoto (atsushi@ximian.com)
+//	Marek Safar (marek.safar@gmail.com)
 //
 // Copyright (C) Tim Coleman, 2004
 // (c) 2004,2007 Novell, Inc. (http://www.novell.com)
+// Copyright 2011 Xamarin Inc.
 //
 
 //
@@ -30,20 +32,9 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0 && SECURITY_DEP
+#if SECURITY_DEP
 
-#if !MOONLIGHT
-extern alias PrebuiltSystem;
-using X509CertificateCollection = PrebuiltSystem::System.Security.Cryptography.X509Certificates.X509CertificateCollection;
-#endif
-
-using System;
-using System.IO;
-using System.Net;
-using System.Security.Authentication;
-using System.Security.Cryptography.X509Certificates;
-using System.Security.Principal;
-using System.Security.Cryptography;
+#if MONOTOUCH || MONODROID
 using Mono.Security.Protocol.Tls;
 
 using CipherAlgorithmType = System.Security.Authentication.CipherAlgorithmType;
@@ -54,6 +45,33 @@ using MonoCipherAlgorithmType = Mono.Security.Protocol.Tls.CipherAlgorithmType;
 using MonoHashAlgorithmType = Mono.Security.Protocol.Tls.HashAlgorithmType;
 using MonoExchangeAlgorithmType = Mono.Security.Protocol.Tls.ExchangeAlgorithmType;
 using MonoSecurityProtocolType = Mono.Security.Protocol.Tls.SecurityProtocolType;
+#else
+extern alias PrebuiltSystem;
+extern alias MonoSecurity;
+
+using X509CertificateCollection = PrebuiltSystem::System.Security.Cryptography.X509Certificates.X509CertificateCollection;
+
+using CipherAlgorithmType = System.Security.Authentication.CipherAlgorithmType;
+using HashAlgorithmType = System.Security.Authentication.HashAlgorithmType;
+using ExchangeAlgorithmType = System.Security.Authentication.ExchangeAlgorithmType;
+
+using MonoCipherAlgorithmType = MonoSecurity::Mono.Security.Protocol.Tls.CipherAlgorithmType;
+using MonoHashAlgorithmType = MonoSecurity::Mono.Security.Protocol.Tls.HashAlgorithmType;
+using MonoExchangeAlgorithmType = MonoSecurity::Mono.Security.Protocol.Tls.ExchangeAlgorithmType;
+using MonoSecurityProtocolType = MonoSecurity::Mono.Security.Protocol.Tls.SecurityProtocolType;
+
+using MonoSecurity::Mono.Security.Protocol.Tls;
+#endif
+
+using System;
+using System.IO;
+using System.Net;
+using System.Security.Authentication;
+using System.Security.Cryptography.X509Certificates;
+using System.Security.Principal;
+using System.Security.Cryptography;
+
+using System.Threading.Tasks;
 
 namespace System.Net.Security 
 {
@@ -75,24 +93,24 @@ namespace System.Net.Security
 		{
 		}
 
-		public SslStream (Stream innerStream, bool leaveStreamOpen)
-			: base (innerStream, leaveStreamOpen)
+		public SslStream (Stream innerStream, bool leaveInnerStreamOpen)
+			: base (innerStream, leaveInnerStreamOpen)
 		{
 		}
 
-		[MonoTODO ("certValidationCallback is not passed X509Chain and SslPolicyErrors correctly")]
-		public SslStream (Stream innerStream, bool leaveStreamOpen, RemoteCertificateValidationCallback certValidationCallback)
-			: this (innerStream, leaveStreamOpen, certValidationCallback, null)
+		[MonoTODO ("userCertificateValidationCallback is not passed X509Chain and SslPolicyErrors correctly")]
+		public SslStream (Stream innerStream, bool leaveInnerStreamOpen, RemoteCertificateValidationCallback userCertificateValidationCallback)
+			: this (innerStream, leaveInnerStreamOpen, userCertificateValidationCallback, null)
 		{
 		}
 
-		[MonoTODO ("certValidationCallback is not passed X509Chain and SslPolicyErrors correctly")]
-		public SslStream (Stream innerStream, bool leaveStreamOpen, RemoteCertificateValidationCallback certValidationCallback, LocalCertificateSelectionCallback certSelectionCallback)
-			: base (innerStream, leaveStreamOpen)
+		[MonoTODO ("userCertificateValidationCallback is not passed X509Chain and SslPolicyErrors correctly")]
+		public SslStream (Stream innerStream, bool leaveInnerStreamOpen, RemoteCertificateValidationCallback userCertificateValidationCallback, LocalCertificateSelectionCallback userCertificateSelectionCallback)
+			: base (innerStream, leaveInnerStreamOpen)
 		{
 			// they are nullable.
-			validation_callback = certValidationCallback;
-			selection_callback = certSelectionCallback;
+			validation_callback = userCertificateValidationCallback;
+			selection_callback = userCertificateSelectionCallback;
 		}
 		#endregion // Constructors
 
@@ -140,11 +158,7 @@ namespace System.Net.Security
 		}
 
 		public override bool IsServer { 
-#if MOONLIGHT
-			get { return false; }
-#else
 			get { return ssl_stream is SslServerStream; }
-#endif
 		}
 
 		public override bool IsSigned { 
@@ -278,11 +292,7 @@ namespace System.Net.Security
 		public virtual X509Certificate RemoteCertificate {
 			get {
 				CheckConnectionAuthenticated ();
-#if MOONLIGHT
-				return ssl_stream.ServerCertificate;
-#else
 				return !IsServer ? ssl_stream.ServerCertificate : ((SslServerStream) ssl_stream).ClientCertificate;
-#endif
 			}
 		}
 
@@ -329,12 +339,12 @@ namespace System.Net.Security
 			return BeginAuthenticateAsClient (targetHost, new X509CertificateCollection (), SslProtocols.Tls, false, asyncCallback, asyncState);
 		}
 
-		public virtual IAsyncResult BeginAuthenticateAsClient (string targetHost, X509CertificateCollection clientCertificates, SslProtocols sslProtocolType, bool checkCertificateRevocation, AsyncCallback asyncCallback, object asyncState)
+		public virtual IAsyncResult BeginAuthenticateAsClient (string targetHost, X509CertificateCollection clientCertificates, SslProtocols enabledSslProtocols, bool checkCertificateRevocation, AsyncCallback asyncCallback, object asyncState)
 		{
 			if (IsAuthenticated)
 				throw new InvalidOperationException ("This SslStream is already authenticated");
 
-			SslClientStream s = new SslClientStream (InnerStream,  targetHost, !LeaveInnerStreamOpen, GetMonoSslProtocol (sslProtocolType), clientCertificates);
+			SslClientStream s = new SslClientStream (InnerStream, targetHost, !LeaveInnerStreamOpen, GetMonoSslProtocol (enabledSslProtocols), clientCertificates);
 			s.CheckCertRevocationStatus = checkCertificateRevocation;
 
 			// Due to the Mono.Security internal, it cannot reuse
@@ -355,7 +365,15 @@ namespace System.Net.Security
 				return null;
 			};
 
-			if (validation_callback != null)
+#if MONOTOUCH || MONODROID
+			// Even if validation_callback is null this allows us to verify requests where the user
+			// does not provide a verification callback but attempts to authenticate with the website
+			// as a client (see https://bugzilla.xamarin.com/show_bug.cgi?id=18962 for an example)
+			var helper = new ServicePointManager.ChainValidationHelper (this, targetHost);
+			helper.ServerCertificateValidationCallback = validation_callback;
+			s.ServerCertValidation2 += new CertificateValidationCallback2 (helper.ValidateChain);
+#else
+			if (validation_callback != null) {
 				s.ServerCertValidationDelegate = delegate (X509Certificate cert, int [] certErrors) {
 					X509Chain chain = new X509Chain ();
 					X509Certificate2 x2 = (cert as X509Certificate2);
@@ -395,6 +413,8 @@ namespace System.Net.Security
 
 					return validation_callback (this, cert, chain, errors);
 				};
+			}
+#endif
 			if (selection_callback != null)
 				s.ClientCertSelectionDelegate = OnCertificateSelection;
 
@@ -409,18 +429,18 @@ namespace System.Net.Security
 
 			return ssl_stream.BeginRead (buffer, offset, count, asyncCallback, asyncState);
 		}
-#if !MOONLIGHT
-		public virtual IAsyncResult BeginAuthenticateAsServer (X509Certificate serverCertificate, AsyncCallback callback, object asyncState)
+
+		public virtual IAsyncResult BeginAuthenticateAsServer (X509Certificate serverCertificate, AsyncCallback asyncCallback, object asyncState)
 		{
-			return BeginAuthenticateAsServer (serverCertificate, false, SslProtocols.Tls, false, callback, asyncState);
+			return BeginAuthenticateAsServer (serverCertificate, false, SslProtocols.Tls, false, asyncCallback, asyncState);
 		}
 
-		public virtual IAsyncResult BeginAuthenticateAsServer (X509Certificate serverCertificate, bool clientCertificateRequired, SslProtocols sslProtocolType, bool checkCertificateRevocation, AsyncCallback callback, object asyncState)
+		public virtual IAsyncResult BeginAuthenticateAsServer (X509Certificate serverCertificate, bool clientCertificateRequired, SslProtocols enabledSslProtocols, bool checkCertificateRevocation, AsyncCallback asyncCallback, object asyncState)
 		{
 			if (IsAuthenticated)
 				throw new InvalidOperationException ("This SslStream is already authenticated");
 
-			SslServerStream s = new SslServerStream (InnerStream, serverCertificate, clientCertificateRequired, !LeaveInnerStreamOpen, GetMonoSslProtocol (sslProtocolType));
+			SslServerStream s = new SslServerStream (InnerStream, serverCertificate, clientCertificateRequired, !LeaveInnerStreamOpen, GetMonoSslProtocol (enabledSslProtocols));
 			s.CheckCertRevocationStatus = checkCertificateRevocation;
 			// Due to the Mono.Security internal, it cannot reuse
 			// the delegated argument, as Mono.Security creates 
@@ -447,9 +467,9 @@ namespace System.Net.Security
 
 			ssl_stream = s;
 
-			return BeginRead (new byte [0], 0, 0, callback, asyncState);
+			return BeginWrite (new byte[0], 0, 0, asyncCallback, asyncState);
 		}
-#endif
+
 		MonoSecurityProtocolType GetMonoSslProtocol (SslProtocols ms)
 		{
 			switch (ms) {
@@ -476,23 +496,23 @@ namespace System.Net.Security
 			AuthenticateAsClient (targetHost, new X509CertificateCollection (), SslProtocols.Tls, false);
 		}
 
-		public virtual void AuthenticateAsClient (string targetHost, X509CertificateCollection clientCertificates, SslProtocols sslProtocolType, bool checkCertificateRevocation)
+		public virtual void AuthenticateAsClient (string targetHost, X509CertificateCollection clientCertificates, SslProtocols enabledSslProtocols, bool checkCertificateRevocation)
 		{
 			EndAuthenticateAsClient (BeginAuthenticateAsClient (
-				targetHost, clientCertificates, sslProtocolType, checkCertificateRevocation, null, null));
+				targetHost, clientCertificates, enabledSslProtocols, checkCertificateRevocation, null, null));
 		}
-#if !MOONLIGHT
+
 		public virtual void AuthenticateAsServer (X509Certificate serverCertificate)
 		{
 			AuthenticateAsServer (serverCertificate, false, SslProtocols.Tls, false);
 		}
 
-		public virtual void AuthenticateAsServer (X509Certificate serverCertificate, bool clientCertificateRequired, SslProtocols sslProtocolType, bool checkCertificateRevocation)
+		public virtual void AuthenticateAsServer (X509Certificate serverCertificate, bool clientCertificateRequired, SslProtocols enabledSslProtocols, bool checkCertificateRevocation)
 		{
 			EndAuthenticateAsServer (BeginAuthenticateAsServer (
-				serverCertificate, clientCertificateRequired, sslProtocolType, checkCertificateRevocation, null, null));
+				serverCertificate, clientCertificateRequired, enabledSslProtocols, checkCertificateRevocation, null, null));
 		}
-#endif
+
 		protected override void Dispose (bool disposing)
 		{
 			if (disposing) {
@@ -512,7 +532,7 @@ namespace System.Net.Security
 			else
 				ssl_stream.EndWrite (asyncResult);
 		}
-#if !MOONLIGHT
+
 		public virtual void EndAuthenticateAsServer (IAsyncResult asyncResult)
 		{
 			CheckConnectionAuthenticated ();
@@ -522,7 +542,7 @@ namespace System.Net.Security
 			else
 				ssl_stream.EndWrite (asyncResult);
 		}
-#endif
+
 		public override int EndRead (IAsyncResult asyncResult)
 		{
 			CheckConnectionAuthenticated ();
@@ -575,6 +595,36 @@ namespace System.Net.Security
 				throw new InvalidOperationException ("This operation is invalid until it is successfully authenticated");
 		}
 
+		public virtual Task AuthenticateAsClientAsync (string targetHost)
+		{
+			return Task.Factory.FromAsync (BeginAuthenticateAsClient, EndAuthenticateAsClient, targetHost, null);
+		}
+
+		public virtual Task AuthenticateAsClientAsync (string targetHost, X509CertificateCollection clientCertificates, SslProtocols enabledSslProtocols, bool checkCertificateRevocation)
+		{
+			var t = Tuple.Create (targetHost, clientCertificates, enabledSslProtocols, checkCertificateRevocation, this);
+
+			return Task.Factory.FromAsync ((callback, state) => {
+				var d = (Tuple<string, X509CertificateCollection, SslProtocols, bool, SslStream>) state;
+				return d.Item5.BeginAuthenticateAsClient (d.Item1, d.Item2, d.Item3, d.Item4, callback, null);
+			}, EndAuthenticateAsClient, t);
+		}
+
+		public virtual Task AuthenticateAsServerAsync (X509Certificate serverCertificate)
+		{
+			return Task.Factory.FromAsync (BeginAuthenticateAsServer, EndAuthenticateAsServer, serverCertificate, null);
+		}
+
+		public virtual Task AuthenticateAsServerAsync (X509Certificate serverCertificate, bool clientCertificateRequired, SslProtocols enabledSslProtocols, bool checkCertificateRevocation)
+		{
+			var t = Tuple.Create (serverCertificate, clientCertificateRequired, enabledSslProtocols, checkCertificateRevocation, this);
+
+			return Task.Factory.FromAsync ((callback, state) => {
+				var d = (Tuple<X509Certificate, bool, SslProtocols, bool, SslStream>) state;
+				return d.Item5.BeginAuthenticateAsServer (d.Item1, d.Item2, d.Item3, d.Item4, callback, null);
+			}, EndAuthenticateAsServer, t);
+		}
+
 		#endregion // Methods
 	}
 }
