@@ -42,9 +42,7 @@ namespace System.Net.Sockets
 		int in_progress;
 		internal Socket.Worker Worker;
 		EndPoint remote_ep;
-#if NET_4_0
 		public Exception ConnectByNameError { get; internal set; }
-#endif
 
 		public event EventHandler<SocketAsyncEventArgs> Completed;
 
@@ -82,7 +80,6 @@ namespace System.Net.Sockets
 		public SocketFlags SocketFlags { get; set; }
 		public object UserToken { get; set; }
 		internal Socket curSocket;
-#if (NET_2_1 || NET_4_0)
 		public Socket ConnectSocket {
 			get {
 				switch (SocketError) {
@@ -101,7 +98,6 @@ namespace System.Net.Sockets
 		{
 			PolicyRestricted = policy;
 		}
-#endif
 		
 		public SocketAsyncEventArgs ()
 		{
@@ -260,6 +256,8 @@ namespace System.Net.Sockets
 				curSocket.EndConnect (ares);
  			} catch (SocketException se) {
 				SocketError = se.SocketErrorCode;
+			} catch (ObjectDisposedException) {
+				SocketError = SocketError.OperationAborted;
 			} finally {
 				OnCompleted (this);
 			}
