@@ -1,9 +1,17 @@
 // System.Net.Dns.cs
 //
+// Authors:
+//	Mads Pultz (mpultz@diku.dk)
+//	Lawrence Pit (loz@cable.a2000.nl)
+
 // Author: Mads Pultz (mpultz@diku.dk)
-// Author: Lawrence Pit (loz@cable.a2000.nl)
+// 	   Lawrence Pit (loz@cable.a2000.nl)
+//	   Marek Safar (marek.safar@gmail.com)
+// 	   Gonzalo Paniagua Javier (gonzalo.mono@gmail.com)
 //
 // (C) Mads Pultz, 2001
+// Copyright (c) 2011 Novell, Inc.
+// Copyright (c) 2011 Xamarin, Inc.
 
 //
 // Permission is hereby granted, free of charge, to any person obtaining
@@ -33,60 +41,125 @@ using System.Collections;
 using System.Threading;
 using System.Runtime.CompilerServices;
 using System.Runtime.Remoting.Messaging;
+using System.Threading.Tasks;
+
+#if !MOBILE
+using Mono.Net.Dns;
+#endif
 
 namespace System.Net {
-#if NET_2_0
 	public static class Dns {
-#else
-	public sealed class Dns {
-
-		private Dns () {}
+#if !MOBILE
+		static bool use_mono_dns;
+		static SimpleResolver resolver;
 #endif
 
 		static Dns ()
 		{
 			System.Net.Sockets.Socket.CheckProtocolSupport();
+
+#if !MOBILE
+			if (Environment.GetEnvironmentVariable ("MONO_DNS") != null) {
+				resolver = new SimpleResolver ();
+				use_mono_dns = true;
+			}
+#endif
 		}
 
-#if !NET_2_1 // global remove of async methods
+#if !MOBILE
+		internal static bool UseMonoDns {
+			get { return use_mono_dns; }
+		}
+#endif
 
 		private delegate IPHostEntry GetHostByNameCallback (string hostName);
 		private delegate IPHostEntry ResolveCallback (string hostName);
-#if NET_2_0
 		private delegate IPHostEntry GetHostEntryNameCallback (string hostName);
 		private delegate IPHostEntry GetHostEntryIPCallback (IPAddress hostAddress);
 		private delegate IPAddress [] GetHostAddressesCallback (string hostName);
+
+#if !MOBILE
+		static void OnCompleted (object sender, SimpleResolverEventArgs e)
+		{
+			DnsAsyncResult ares = (DnsAsyncResult) e.UserToken;
+			IPHostEntry entry = e.HostEntry;
+			if (entry == null || e.ResolverError != 0) {
+				ares.SetCompleted (false, new Exception ("Error: " + e.ResolverError));
+				return;
+			}
+			ares.SetCompleted (false, entry);
+		}
+
+		static IAsyncResult BeginAsyncCallAddresses (string host, AsyncCallback callback, object state)
+		{
+			SimpleResolverEventArgs e = new SimpleResolverEventArgs ();
+			e.Completed += OnCompleted;
+			e.HostName = host;
+			DnsAsyncResult ares = new DnsAsyncResult (callback, state);
+			e.UserToken = ares;
+			if (resolver.GetHostAddressesAsync (e) == false)
+				ares.SetCompleted (true, e.HostEntry); // Completed synchronously
+			return ares;
+		}
+
+		static IAsyncResult BeginAsyncCall (string host, AsyncCallback callback, object state)
+		{
+			SimpleResolverEventArgs e = new SimpleResolverEventArgs ();
+			e.Completed += OnCompleted;
+			e.HostName = host;
+			DnsAsyncResult ares = new DnsAsyncResult (callback, state);
+			e.UserToken = ares;
+			if (resolver.GetHostEntryAsync (e) == false)
+				ares.SetCompleted (true, e.HostEntry); // Completed synchronously
+			return ares;
+		}
+
+		static IPHostEntry EndAsyncCall (DnsAsyncResult ares)
+		{
+			if (ares == null)
+				throw new ArgumentException ("Invalid asyncResult");
+			if (!ares.IsCompleted)
+				ares.AsyncWaitHandle.WaitOne ();
+			if (ares.Exception != null)
+				throw ares.Exception;
+			IPHostEntry entry = ares.HostEntry;
+			if (entry == null || entry.AddressList == null || entry.AddressList.Length == 0)
+				Error_11001 (entry.HostName);
+			return entry;
+		}
 #endif
 
-#if NET_2_0
 		[Obsolete ("Use BeginGetHostEntry instead")]
-#endif
-		public static IAsyncResult BeginGetHostByName (string hostName,
-			AsyncCallback requestCallback, object stateObject)
+		public static IAsyncResult BeginGetHostByName (string hostName, AsyncCallback requestCallback, object stateObject)
 		{
 			if (hostName == null)
 				throw new ArgumentNullException ("hostName");
 
+#if !MOBILE
+			if (use_mono_dns)
+				return BeginAsyncCall (hostName, requestCallback, stateObject);
+#endif
+
 			GetHostByNameCallback c = new GetHostByNameCallback (GetHostByName);
 			return c.BeginInvoke (hostName, requestCallback, stateObject);
 		}
 
-#if NET_2_0
 		[Obsolete ("Use BeginGetHostEntry instead")]
-#endif
-		public static IAsyncResult BeginResolve (string hostName,
-			AsyncCallback requestCallback, object stateObject)
+		public static IAsyncResult BeginResolve (string hostName, AsyncCallback requestCallback, object stateObject)
 		{
 			if (hostName == null)
 				throw new ArgumentNullException ("hostName");
 
+#if !MOBILE
+			if (use_mono_dns)
+				return BeginAsyncCall (hostName, requestCallback, stateObject);
+#endif
+
 			ResolveCallback c = new ResolveCallback (Resolve);
 			return c.BeginInvoke (hostName, requestCallback, stateObject);
 		}
 
-#if NET_2_0
-		public static IAsyncResult BeginGetHostAddresses (string hostNameOrAddress,
-			AsyncCallback requestCallback, object stateObject)
+		public static IAsyncResult BeginGetHostAddresses (string hostNameOrAddress, AsyncCallback requestCallback, object state)
 		{
 			if (hostNameOrAddress == null)
 				throw new ArgumentNullException ("hostName");
@@ -96,12 +169,16 @@ namespace System.Net {
 					"cannot use them as target address.",
 					"hostNameOrAddress");
 
+#if !MOBILE
+			if (use_mono_dns)
+				return BeginAsyncCallAddresses (hostNameOrAddress, requestCallback, state);
+#endif
+
 			GetHostAddressesCallback c = new GetHostAddressesCallback (GetHostAddresses);
-			return c.BeginInvoke (hostNameOrAddress, requestCallback, stateObject);
+			return c.BeginInvoke (hostNameOrAddress, requestCallback, state);
 		}
 
-		public static IAsyncResult BeginGetHostEntry (string hostNameOrAddress,
-			AsyncCallback requestCallback, object stateObject)
+		public static IAsyncResult BeginGetHostEntry (string hostNameOrAddress, AsyncCallback requestCallback, object stateObject)
 		{
 			if (hostNameOrAddress == null)
 				throw new ArgumentNullException ("hostName");
@@ -111,53 +188,75 @@ namespace System.Net {
 					"cannot use them as target address.",
 					"hostNameOrAddress");
 
+#if !MOBILE
+			if (use_mono_dns)
+				return BeginAsyncCall (hostNameOrAddress, requestCallback, stateObject);
+#endif
+
 			GetHostEntryNameCallback c = new GetHostEntryNameCallback (GetHostEntry);
 			return c.BeginInvoke (hostNameOrAddress, requestCallback, stateObject);
 		}
 
-		public static IAsyncResult BeginGetHostEntry (IPAddress address,
-			AsyncCallback requestCallback, object stateObject)
+		public static IAsyncResult BeginGetHostEntry (IPAddress address, AsyncCallback requestCallback, object stateObject)
 		{
 			if (address == null)
 				throw new ArgumentNullException ("address");
 
+#if !MOBILE
+			if (use_mono_dns)
+				return BeginAsyncCall (address.ToString (), requestCallback, stateObject);
+#endif
+
 			GetHostEntryIPCallback c = new GetHostEntryIPCallback (GetHostEntry);
 			return c.BeginInvoke (address, requestCallback, stateObject);
 		}
-#endif
 
-#if NET_2_0
 		[Obsolete ("Use EndGetHostEntry instead")]
-#endif
 		public static IPHostEntry EndGetHostByName (IAsyncResult asyncResult) 
 		{
 			if (asyncResult == null)
 				throw new ArgumentNullException ("asyncResult");
 
+#if !MOBILE
+			if (use_mono_dns)
+				return EndAsyncCall (asyncResult as DnsAsyncResult);
+#endif
+
 			AsyncResult async = (AsyncResult) asyncResult;
 			GetHostByNameCallback cb = (GetHostByNameCallback) async.AsyncDelegate;
 			return cb.EndInvoke(asyncResult);
 		}
 
-#if NET_2_0
 		[Obsolete ("Use EndGetHostEntry instead")]
-#endif
 		public static IPHostEntry EndResolve (IAsyncResult asyncResult) 
 		{
 			if (asyncResult == null)
 				throw new ArgumentNullException ("asyncResult");
+
+#if !MOBILE
+			if (use_mono_dns)
+				return EndAsyncCall (asyncResult as DnsAsyncResult);
+#endif
+
 			AsyncResult async = (AsyncResult) asyncResult;
 			ResolveCallback cb = (ResolveCallback) async.AsyncDelegate;
 			return cb.EndInvoke(asyncResult);
 		}
 
-#if NET_2_0
-
 		public static IPAddress [] EndGetHostAddresses (IAsyncResult asyncResult) 
 		{
 			if (asyncResult == null)
 				throw new ArgumentNullException ("asyncResult");
 
+#if !MOBILE
+			if (use_mono_dns) {
+				IPHostEntry entry = EndAsyncCall (asyncResult as DnsAsyncResult);
+				if (entry == null)
+					return null;
+				return entry.AddressList;
+			}
+#endif
+
 			AsyncResult async = (AsyncResult) asyncResult;
 			GetHostAddressesCallback cb = (GetHostAddressesCallback) async.AsyncDelegate;
 			return cb.EndInvoke(asyncResult);
@@ -167,19 +266,20 @@ namespace System.Net {
 		{
 			if (asyncResult == null)
 				throw new ArgumentNullException ("asyncResult");
+
+#if !MOBILE
+			if (use_mono_dns)
+				return EndAsyncCall (asyncResult as DnsAsyncResult);
+#endif
+
 			AsyncResult async = (AsyncResult) asyncResult;
-#if NET_2_0
 			if (async.AsyncDelegate is GetHostEntryIPCallback)
 				return ((GetHostEntryIPCallback) async.AsyncDelegate).EndInvoke (asyncResult);
-#endif
 			GetHostEntryNameCallback cb = (GetHostEntryNameCallback) async.AsyncDelegate;
 			return cb.EndInvoke(asyncResult);
 		}
-#endif
-		
-#endif // !NET_2_1: global remove of async methods
 
-#if !TARGET_JVM
+
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static bool GetHostByName_internal(string host, out string h_name, out string[] h_aliases, out string[] h_addr_list);
 
@@ -188,9 +288,14 @@ namespace System.Net {
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static bool GetHostName_internal(out string h_name);
-#endif	
 
-		private static IPHostEntry hostent_to_IPHostEntry(string h_name, string[] h_aliases, string[] h_addrlist) 
+		static void Error_11001 (string hostName)
+		{
+			throw new SocketException(11001, string.Format ("Could not resolve host '{0}'", hostName));
+
+		}
+
+		private static IPHostEntry hostent_to_IPHostEntry(string originalHostName, string h_name, string[] h_aliases, string[] h_addrlist) 
 		{
 			IPHostEntry he = new IPHostEntry();
 			ArrayList addrlist = new ArrayList();
@@ -214,15 +319,13 @@ namespace System.Net {
 			}
 
 			if(addrlist.Count == 0)
-				throw new SocketException(11001);
+				Error_11001 (originalHostName);
 
 			he.AddressList = addrlist.ToArray(typeof(IPAddress)) as IPAddress[];
 			return he;
 		}
 
-#if NET_2_0
 		[Obsolete ("Use GetHostEntry instead")]
-#endif
 		public static IPHostEntry GetHostByAddress(IPAddress address)
 		{
 			if (address == null)
@@ -231,9 +334,7 @@ namespace System.Net {
 			return GetHostByAddressFromString (address.ToString (), false);
 		}
 
-#if NET_2_0
 		[Obsolete ("Use GetHostEntry instead")]
-#endif
 		public static IPHostEntry GetHostByAddress(string address)
 		{
 			if (address == null)
@@ -257,34 +358,13 @@ namespace System.Net {
 
 			string h_name;
 			string[] h_aliases, h_addrlist;
-#if TARGET_JVM
-			h_name = null;
-			h_aliases = null;
-			h_addrlist = null;
-            		try {
-                		java.net.InetAddress[] iaArr = 
-					java.net.InetAddress.getAllByName(address);
-		                if (iaArr != null && iaArr.Length > 0)
-                		    h_name = iaArr[0].getHostName();
-		                if (iaArr != null && iaArr.Length > 0)
-                		{
-		                    h_addrlist = new String[iaArr.Length];
-                		    for (int i = 0; i < h_addrlist.Length; i++)
-		                        h_addrlist[i] = iaArr[i].getHostAddress();
-                		}
-            		} catch (java.net.UnknownHostException jUHE) {
-		                throw new SocketException((int)SocketError.HostNotFound, jUHE.Message);
-            		}
-#else
 			bool ret = GetHostByAddr_internal(address, out h_name, out h_aliases, out h_addrlist);
 			if (!ret)
-				throw new SocketException(11001);
-#endif
-			return (hostent_to_IPHostEntry (h_name, h_aliases, h_addrlist));
+				Error_11001 (address);
+			return (hostent_to_IPHostEntry (address, h_name, h_aliases, h_addrlist));
 			
 		}
 
-#if NET_2_0
 		public static IPHostEntry GetHostEntry (string hostNameOrAddress)
 		{
 			if (hostNameOrAddress == null)
@@ -327,65 +407,35 @@ namespace System.Net {
 
 			return GetHostEntry (hostNameOrAddress).AddressList;
 		}
-#endif
 
-#if NET_2_0
 		[Obsolete ("Use GetHostEntry instead")]
-#endif
 		public static IPHostEntry GetHostByName (string hostName)
 		{
 			if (hostName == null)
 				throw new ArgumentNullException ("hostName");
-#if TARGET_JVM
-			if (hostName.Length == 0)
-				hostName = "localhost";
-		        try {
-				java.net.InetAddress[] iaArr = java.net.InetAddress.getAllByName(hostName);
-				IPHostEntry host = new IPHostEntry();
-				if (iaArr != null && iaArr.Length > 0)
-                		{
-					host.HostName = iaArr[0].getHostName();
-                		    	IPAddress[] ipArr = new IPAddress[iaArr.Length];
-		                    	for (int i = 0; i < iaArr.Length; i++)
-                		        	ipArr[i] = IPAddress.Parse(iaArr[i].getHostAddress());
-
-					host.AddressList = ipArr;
-                		}
-		                return host;
-			} catch (java.net.UnknownHostException jUHE) {
-				throw new SocketException((int)SocketError.HostNotFound, jUHE.Message);
-			}
-#else
 			string h_name;
 			string[] h_aliases, h_addrlist;
 
 			bool ret = GetHostByName_internal(hostName, out h_name, out h_aliases, out h_addrlist);
 			if (ret == false)
-				throw new SocketException(11001);
+				Error_11001 (hostName);
 
-			return(hostent_to_IPHostEntry(h_name, h_aliases, h_addrlist));
-#endif
+			return(hostent_to_IPHostEntry(hostName, h_name, h_aliases, h_addrlist));
 		}
 
 		public static string GetHostName ()
 		{
-#if TARGET_JVM
-			return java.net.InetAddress.getLocalHost ().getHostName ();
-#else
 			string hostName;
 
 			bool ret = GetHostName_internal(out hostName);
 
 			if (ret == false)
-				throw new SocketException(11001);
+				Error_11001 (hostName);
 
 			return hostName;
-#endif
 		}
 
-#if NET_2_0
 		[Obsolete ("Use GetHostEntry instead")]
-#endif
 		public static IPHostEntry Resolve(string hostName) 
 		{
 			if (hostName == null)
@@ -403,6 +453,21 @@ namespace System.Net {
 
 			return ret;
 		}
+
+		public static Task<IPAddress[]> GetHostAddressesAsync (string hostNameOrAddress)
+		{
+			return Task<IPAddress[]>.Factory.FromAsync (BeginGetHostAddresses, EndGetHostAddresses, hostNameOrAddress, null);
+		}
+
+		public static Task<IPHostEntry> GetHostEntryAsync (IPAddress address)
+		{
+			return Task<IPHostEntry>.Factory.FromAsync (BeginGetHostEntry, EndGetHostEntry, address, null);
+		}
+
+		public static Task<IPHostEntry> GetHostEntryAsync (string hostNameOrAddress)
+		{
+			return Task<IPHostEntry>.Factory.FromAsync (BeginGetHostEntry, EndGetHostEntry, hostNameOrAddress, null);
+		}
 	}
 }
 
