@@ -1951,6 +1951,51 @@ namespace MonoTests.System
 			Assert.IsTrue (Uri.TryCreate (mainUri, uriPath, out result), "#1");
 			Assert.AreEqual ("http://www.imdb.com/title/tt0106521", result.ToString (), "#2");
 		}
+
+		[Test]
+		public void GetSerializationInfoStringOnRelativeUri ()
+		{
+			var uri = new Uri ("/relative/path", UriKind.Relative);
+			var result = uri.GetComponents (UriComponents.SerializationInfoString, UriFormat.UriEscaped);
+
+			Assert.AreEqual (uri.OriginalString, result);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentOutOfRangeException))]
+		public void GetSerializationInfoStringException ()
+		{
+			var uri = new Uri ("/relative/path", UriKind.Relative);
+			uri.GetComponents (UriComponents.SerializationInfoString  | UriComponents.Host, UriFormat.UriEscaped);
+		}
+
+		[Test]
+		public void UserInfo_EscapedLetter ()
+		{
+			var uri = new Uri ("https://first%61second@host");
+			Assert.AreEqual ("firstasecond", uri.UserInfo);
+		}
+
+		[Test]
+		public void UserInfo_EscapedAt ()
+		{
+			var userinfo =  "first%40second";
+			var uri = new Uri ("https://" + userinfo + "@host");
+			Assert.AreEqual (userinfo, uri.UserInfo);
+		}
+
+		[Test]
+		public void UserInfo_EscapedChars ()
+		{
+			for (var ch = (char) 1; ch < 128; ch++) {
+				var userinfo = Uri.EscapeDataString (ch.ToString ());
+				try {
+					new Uri (string.Format("http://{0}@localhost:80/", userinfo));
+				} catch (Exception e) {
+					Assert.Fail (string.Format("Unexpected {0} while building URI with username {1}", e.GetType ().Name, userinfo));
+				}
+			}
+		}
 	}
 
 	// Tests non default IriParsing
@@ -1961,7 +2006,7 @@ namespace MonoTests.System
 		private bool originalIriParsing;
 
 		[TestFixtureSetUp]
-		public void GetReady ()
+		public void GetReady2 ()
 		{
 			isWin32 = (Path.DirectorySeparatorChar == '\\');
 
