@@ -29,6 +29,8 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+#if !NET_2_1
+
 using System;
 using System.IO;
 using System.Collections;
@@ -36,19 +38,16 @@ using System.Diagnostics;
 using System.Runtime.InteropServices;
 using System.Security;
 using System.Text;
-
-#if NET_2_0
 using System.Security.AccessControl;
-#endif
+using System.Security.Permissions;
+using Microsoft.Win32.SafeHandles;
 
 namespace Microsoft.Win32
 {
 	/// <summary>
 	///	Wrapper class for Windows Registry Entry.
 	/// </summary>
-#if NET_2_0
 	[ComVisible (true)]
-#endif
 	public sealed class RegistryKey : MarshalByRefObject, IDisposable 
 	{
 		//
@@ -56,6 +55,7 @@ namespace Microsoft.Win32
 		// RegistryKey object
 		//
 		object handle;
+		SafeRegistryHandle safe_handle;
 
 		object hive; // the RegistryHive if the key represents a base key
 		readonly string qname;	// the fully qualified registry key name
@@ -108,7 +108,7 @@ namespace Microsoft.Win32
 		///	Dispose of registry key object. Close the 
 		///	key if it's still open.
 		/// </summary>
-		void IDisposable.Dispose ()
+		public void Dispose ()
 		{
 			GC.SuppressFinalize (this);
 			Close ();
@@ -157,6 +157,7 @@ namespace Microsoft.Win32
 			
 			RegistryApi.Close (this);
 			handle = null;
+			safe_handle = null;
 		}
 		
 		
@@ -183,6 +184,29 @@ namespace Microsoft.Win32
 			}
 		}
 
+		[ComVisible (false)]
+		[MonoTODO ("Not implemented in Unix")]
+		public SafeRegistryHandle Handle {
+			get {
+				AssertKeyStillValid ();
+
+				if (safe_handle == null) {
+					IntPtr h = RegistryApi.GetHandle (this);
+					safe_handle = new SafeRegistryHandle (h, true);
+				}
+
+				return safe_handle;
+			}
+		}
+
+		[ComVisible (false)]
+		[MonoLimitation ("View is ignored in Mono.")]
+		public RegistryView View {
+			get {
+				return RegistryView.Default;
+			}
+		}
+
 		
 		/// <summary>
 		///	Set a registry value.
@@ -203,7 +227,6 @@ namespace Microsoft.Win32
 			RegistryApi.SetValue (this, name, value);
 		}
 
-#if NET_2_0
 		[ComVisible (false)]
 		public void SetValue (string name, object value, RegistryValueKind valueKind)
 		{
@@ -220,7 +243,6 @@ namespace Microsoft.Win32
 
 			RegistryApi.SetValue (this, name, value, valueKind);
 		}
-#endif
 
 		/// <summary>
 		///	Open the sub key specified, for read access.
@@ -267,7 +289,6 @@ namespace Microsoft.Win32
 				RegistryValueOptions.None);
 		}
 
-#if NET_2_0
 		[ComVisible (false)]
 		public object GetValue (string name, object defaultValue, RegistryValueOptions options)
 		{
@@ -279,9 +300,8 @@ namespace Microsoft.Win32
 		[ComVisible (false)]
 		public RegistryValueKind GetValueKind (string name)
 		{
-			throw new NotImplementedException ();
+			return RegistryApi.GetValueKind (this, name);
 		}
-#endif
 
 		/// <summary>
 		///	Create a sub key.
@@ -297,19 +317,42 @@ namespace Microsoft.Win32
 			return RegistryApi.CreateSubKey (this, subkey);
 		}
 
-#if NET_2_0
 		[ComVisible (false)]
+		[MonoLimitation ("permissionCheck is ignored in Mono")]
 		public RegistryKey CreateSubKey (string subkey, RegistryKeyPermissionCheck permissionCheck)
 		{
-			throw new NotImplementedException ();
+			return CreateSubKey (subkey);
 		}
 
 		[ComVisible (false)]
+		[MonoLimitation ("permissionCheck and registrySecurity are ignored in Mono")]
 		public RegistryKey CreateSubKey (string subkey, RegistryKeyPermissionCheck permissionCheck, RegistrySecurity registrySecurity)
 		{
-			throw new NotImplementedException ();
+			return CreateSubKey (subkey);
 		}
-#endif
+
+		[ComVisible (false)]
+		[MonoLimitation ("permissionCheck is ignored in Mono")]
+		public RegistryKey CreateSubKey (string subkey, RegistryKeyPermissionCheck permissionCheck, RegistryOptions options)
+		{
+			AssertKeyStillValid ();
+			AssertKeyNameNotNull (subkey);
+			AssertKeyNameLength (subkey);
+
+			if (!IsWritable)
+				throw new UnauthorizedAccessException ("Cannot write to the registry key.");
+
+			return RegistryApi.CreateSubKey (this, subkey, options);
+		}
+
+		[ComVisible (false)]
+		[MonoLimitation ("permissionCheck and registrySecurity are ignored in Mono")]
+		public RegistryKey CreateSubKey (string subkey, RegistryKeyPermissionCheck permissionCheck, RegistryOptions registryOptions,
+			RegistrySecurity registrySecurity)
+		{
+			return CreateSubKey (subkey, permissionCheck, registryOptions);
+		}
+
 		
 		/// <summary>
 		///	Delete the specified subkey.
@@ -356,6 +399,12 @@ namespace Microsoft.Win32
 		///	Delete a sub tree (node, and values alike).
 		/// </summary>
 		public void DeleteSubKeyTree(string subkey)
+		{
+			DeleteSubKeyTree (subkey, true);
+		}
+
+		public
+		void DeleteSubKeyTree (string subkey, bool throwOnMissingSubKey)
 		{
 			// Note: this is done by deleting sub-nodes recursively.
 			// The preformance is not very good. There may be a 
@@ -366,9 +415,13 @@ namespace Microsoft.Win32
 			AssertKeyNameLength (subkey);
 			
 			RegistryKey child = OpenSubKey (subkey, true);
-			if (child == null)
+			if (child == null) {
+				if (!throwOnMissingSubKey)
+					return;
+
 				throw new ArgumentException ("Cannot delete a subkey tree"
 					+ " because the subkey does not exist.");
+			}
 
 			child.DeleteChildKeysAndValues ();
 			child.Close ();
@@ -401,17 +454,17 @@ namespace Microsoft.Win32
 			RegistryApi.DeleteValue (this, name, throwOnMissingValue);
 		}
 
-#if NET_2_0
 		public RegistrySecurity GetAccessControl ()
 		{
-			throw new NotImplementedException ();
+			return GetAccessControl (AccessControlSections.Owner |
+						 AccessControlSections.Group |
+						 AccessControlSections.Access);
 		}
 		
 		public RegistrySecurity GetAccessControl (AccessControlSections includeSections)
 		{
-			throw new NotImplementedException ();
+			return new RegistrySecurity (Name, includeSections);
 		}
-#endif
 		
 		
 		/// <summary>
@@ -433,6 +486,25 @@ namespace Microsoft.Win32
 			AssertKeyStillValid ();
 			return RegistryApi.GetValueNames (this);
 		}
+
+		[ComVisible (false)]
+		[SecurityPermission (SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
+		[MonoTODO ("Not implemented on unix")]
+		public static RegistryKey FromHandle (SafeRegistryHandle handle)
+		{
+			if (handle == null)
+				throw new ArgumentNullException ("handle");
+
+			return RegistryApi.FromHandle (handle);
+		}
+
+		[ComVisible (false)]
+		[SecurityPermission (SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
+		[MonoTODO ("Not implemented on unix")]
+		public static RegistryKey FromHandle (SafeRegistryHandle handle, RegistryView view)
+		{
+			return FromHandle (handle);
+		}
 		
 		
 		[MonoTODO ("Not implemented on unix")]
@@ -443,24 +515,59 @@ namespace Microsoft.Win32
 			return RegistryApi.OpenRemoteBaseKey (hKey, machineName);
 		}
 
-#if NET_2_0
+		[ComVisible (false)]
+		[MonoTODO ("Not implemented on unix")]
+		public static RegistryKey OpenRemoteBaseKey (RegistryHive hKey, string machineName, RegistryView view)
+		{
+			if (machineName == null)
+				throw new ArgumentNullException ("machineName");
+			return RegistryApi.OpenRemoteBaseKey (hKey, machineName);
+		}
+
+		[ComVisible (false)]
+		[MonoLimitation ("View is ignored in Mono")]
+		public static RegistryKey OpenBaseKey (RegistryHive hKey, RegistryView view)
+		{
+			switch (hKey) {
+				case RegistryHive.ClassesRoot:
+					return Registry.ClassesRoot;
+				case RegistryHive.CurrentConfig:
+					return Registry.CurrentConfig;
+				case RegistryHive.CurrentUser:
+					return Registry.CurrentUser;
+				case RegistryHive.DynData:
+					return Registry.DynData;
+				case RegistryHive.LocalMachine:
+					return Registry.LocalMachine;
+				case RegistryHive.PerformanceData:
+					return Registry.PerformanceData;
+				case RegistryHive.Users:
+					return Registry.Users;
+			}
+
+			throw new ArgumentException ("hKey");
+		}
+
 		[ComVisible (false)]
 		public RegistryKey OpenSubKey (string name, RegistryKeyPermissionCheck permissionCheck)
 		{
-			throw new NotImplementedException ();
+			return OpenSubKey (name, permissionCheck == RegistryKeyPermissionCheck.ReadWriteSubTree);
 		}
 		
 		[ComVisible (false)]
+		[MonoLimitation ("rights are ignored in Mono")]
 		public RegistryKey OpenSubKey (string name, RegistryKeyPermissionCheck permissionCheck, RegistryRights rights)
 		{
-			throw new NotImplementedException ();
+			return OpenSubKey (name, permissionCheck == RegistryKeyPermissionCheck.ReadWriteSubTree);
 		}
 		
 		public void SetAccessControl (RegistrySecurity registrySecurity)
 		{
-			throw new NotImplementedException ();
+			if (null == registrySecurity)
+				throw new ArgumentNullException ("registrySecurity");
+				
+			registrySecurity.PersistModifications (Name);
 		}
-#endif
 		
 		
 		/// <summary>
@@ -495,7 +602,7 @@ namespace Microsoft.Win32
 
 		// returns the key handle for the win32 implementation and the
 		// KeyHandler for the unix implementation
-		internal object Handle {
+		internal object InternalHandle {
 			get { return handle; }
 		}
 
@@ -516,22 +623,13 @@ namespace Microsoft.Win32
 		private void AssertKeyNameNotNull (string subKeyName)
 		{
 			if (subKeyName == null)
-#if NET_2_0
 				throw new ArgumentNullException ("name");
-#else
-				throw new ArgumentNullException ("subkey");
-#endif
 		}
 
 		private void AssertKeyNameLength (string name)
 		{
-#if NET_2_0
 			if (name.Length > 255)
 				throw new ArgumentException ("Name of registry key cannot be greater than 255 characters");
-#else
-			if (name.Length >= 255)
-				throw new ArgumentException ("Name of registry key cannot be greater than or equal to 255 characters");
-#endif
 		}
 
 		/// <summary>
@@ -603,3 +701,5 @@ namespace Microsoft.Win32
 	}
 }
 
+#endif // NET_2_1
+
