@@ -5,9 +5,11 @@
 // 	Dietmar Maurer (dietmar@ximian.com)
 // 	Dan Lewis (dihlewis@yahoo.co.uk)
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
+//  Marek Safar (marek.safar@gmail.com)
 //
 // (C) 2001-2003 Ximian, Inc.  http://www.ximian.com
-// Copyright (C) 2004-2005, 2008 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2005, 2008, 2010 Novell, Inc (http://www.novell.com)
+// Copyright 2011 Xamarin Inc (http://www.xamarin.com).
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -34,96 +36,53 @@ using System.Globalization;
 using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
 using System.Runtime.Remoting.Messaging;
+using System.Security;
 using System.Security.Permissions;
 using System.Threading;
-
-#if NET_2_0
 using Microsoft.Win32.SafeHandles;
+
 #if NET_2_1
 using System.IO.IsolatedStorage;
-using System.Security;
 #else
 using System.Security.AccessControl;
 #endif
-#endif
+
+using System.Threading.Tasks;
 
 namespace System.IO
 {
-#if NET_2_0
 	[ComVisible (true)]
-#endif
 	public class FileStream : Stream
 	{
 		// construct from handle
 		
 		[Obsolete ("Use FileStream(SafeFileHandle handle, FileAccess access) instead")]
 		public FileStream (IntPtr handle, FileAccess access)
-			: this (handle, access, true, DefaultBufferSize, false) {}
+			: this (handle, access, true, DefaultBufferSize, false, false) {}
 
 		[Obsolete ("Use FileStream(SafeFileHandle handle, FileAccess access) instead")]
 		public FileStream (IntPtr handle, FileAccess access, bool ownsHandle)
-			: this (handle, access, ownsHandle, DefaultBufferSize, false) {}
+			: this (handle, access, ownsHandle, DefaultBufferSize, false, false) {}
 		
 		[Obsolete ("Use FileStream(SafeFileHandle handle, FileAccess access, int bufferSize) instead")]
 		public FileStream (IntPtr handle, FileAccess access, bool ownsHandle, int bufferSize)
-			: this (handle, access, ownsHandle, bufferSize, false) {}
+			: this (handle, access, ownsHandle, bufferSize, false, false) {}
 
 		[Obsolete ("Use FileStream(SafeFileHandle handle, FileAccess access, int bufferSize, bool isAsync) instead")]
 		public FileStream (IntPtr handle, FileAccess access, bool ownsHandle, int bufferSize, bool isAsync)
 			: this (handle, access, ownsHandle, bufferSize, isAsync, false) {}
 
 		[SecurityPermission (SecurityAction.Demand, UnmanagedCode = true)]
-		internal FileStream (IntPtr handle, FileAccess access, bool ownsHandle, int bufferSize, bool isAsync, bool noBuffering)
+		internal FileStream (IntPtr handle, FileAccess access, bool ownsHandle, int bufferSize, bool isAsync, bool isZeroSize)
 		{
-			this.handle = MonoIO.InvalidHandle;
-			if (handle == this.handle)
+			if (handle == MonoIO.InvalidHandle)
 				throw new ArgumentException ("handle", Locale.GetText ("Invalid."));
 
-			if (access < FileAccess.Read || access > FileAccess.ReadWrite)
-				throw new ArgumentOutOfRangeException ("access");
-
-			MonoIOError error;
-			MonoFileType ftype = MonoIO.GetFileType (handle, out error);
-
-			if (error != MonoIOError.ERROR_SUCCESS) {
-				throw MonoIO.GetException (name, error);
-			}
-			
-			if (ftype == MonoFileType.Unknown) {
-				throw new IOException ("Invalid handle.");
-			} else if (ftype == MonoFileType.Disk) {
-				this.canseek = true;
-			} else {
-				this.canseek = false;
-			}
-
-			this.handle = handle;
-			this.access = access;
-			this.owner = ownsHandle;
-			this.async = isAsync;
-#if NET_2_1 && !MONOTOUCH
-			// default the browser to 'all' anonymous files and let other usage (like smcs) with 'normal'
-			// (i.e. non-anonymous except for isolated storage) files and paths
-			this.anonymous = SecurityManager.SecurityEnabled;
-#else
-			this.anonymous = false;
-#endif
-
-			InitBuffer (bufferSize, noBuffering);
-
-			if (canseek) {
-				buf_start = MonoIO.Seek (handle, 0, SeekOrigin.Current, out error);
-				if (error != MonoIOError.ERROR_SUCCESS) {
-					throw MonoIO.GetException (name, error);
-				}
-			}
-
-			/* Can't set append mode */
-			this.append_startpos=0;
+			Init (new SafeFileHandle (handle, false), access, ownsHandle, bufferSize, isAsync, isZeroSize);
 		}
 
 		// construct from filename
-		
+
 		public FileStream (string path, FileMode mode)
 			: this (path, mode, (mode == FileMode.Append ? FileAccess.Write : FileAccess.ReadWrite), FileShare.Read, DefaultBufferSize, false, FileOptions.None)
 		{
@@ -145,16 +104,16 @@ namespace System.IO
 		}
 
 		public FileStream (string path, FileMode mode, FileAccess access, FileShare share, int bufferSize, bool useAsync)
-			: this (path, mode, access, share, bufferSize, useAsync, FileOptions.None)
+			: this (path, mode, access, share, bufferSize, useAsync ? FileOptions.Asynchronous : FileOptions.None)
 		{
 		}
 
-#if NET_2_0 && !NET_2_1
 		public FileStream (string path, FileMode mode, FileAccess access, FileShare share, int bufferSize, FileOptions options)
 			: this (path, mode, access, share, bufferSize, false, options)
 		{
 		}
 
+#if !NET_2_1
 		public FileStream (SafeFileHandle handle, FileAccess access)
 			:this(handle, access, DefaultBufferSize, false)
 		{
@@ -165,31 +124,35 @@ namespace System.IO
 			:this(handle, access, bufferSize, false)
 		{
 		}
-		
-		[MonoLimitationAttribute("Need to use SafeFileHandle instead of underlying handle")]
-		public FileStream (SafeFileHandle handle, FileAccess access,
-				   int bufferSize, bool isAsync)
-			:this (handle.DangerousGetHandle (), access, false, bufferSize, isAsync)
+
+		public FileStream (SafeFileHandle handle, FileAccess access, int bufferSize, bool isAsync)
 		{
-			this.safeHandle = handle;
+			Init (handle, access, false, bufferSize, isAsync, false);
 		}
 
+		[MonoLimitation ("This ignores the rights parameter")]
 		public FileStream (string path, FileMode mode,
 				   FileSystemRights rights, FileShare share,
 				   int bufferSize, FileOptions options)
+			: this (path, mode, (mode == FileMode.Append ? FileAccess.Write : FileAccess.ReadWrite), share, bufferSize, false, options)
 		{
-			throw new NotImplementedException ();
 		}
 		
+		[MonoLimitation ("This ignores the rights and fileSecurity parameters")]
 		public FileStream (string path, FileMode mode,
 				   FileSystemRights rights, FileShare share,
 				   int bufferSize, FileOptions options,
 				   FileSecurity fileSecurity)
+			: this (path, mode, (mode == FileMode.Append ? FileAccess.Write : FileAccess.ReadWrite), share, bufferSize, false, options)
 		{
-			throw new NotImplementedException ();
 		}
 #endif
 
+		internal FileStream (string path, FileMode mode, FileAccess access, FileShare share, int bufferSize, FileOptions options, string msgPath, bool bFromProxy, bool useLongPath, bool checkHost)
+			: this (path, mode, access, share, bufferSize, false, options)
+		{
+		}
+
 		internal FileStream (string path, FileMode mode, FileAccess access, FileShare share, int bufferSize, bool isAsync, bool anonymous)
 			: this (path, mode, access, share, bufferSize, anonymous, isAsync ? FileOptions.Asynchronous : FileOptions.None)
 		{
@@ -205,12 +168,10 @@ namespace System.IO
 				throw new ArgumentException ("Path is empty");
 			}
 
-#if NET_2_0
+			this.anonymous = anonymous;
 			// ignore the Inheritable flag
 			share &= ~FileShare.Inheritable;
 
-#endif
-
 			if (bufferSize <= 0)
 				throw new ArgumentOutOfRangeException ("bufferSize", "Positive number required.");
 
@@ -224,24 +185,10 @@ namespace System.IO
 			}
 
 			if (access < FileAccess.Read || access > FileAccess.ReadWrite) {
-#if NET_2_1
-				if (anonymous)
-					throw new IsolatedStorageException ("Enum value for FileAccess was out of legal range.");
-				else
-#endif
 				throw new ArgumentOutOfRangeException ("access", "Enum value was out of legal range.");
 			}
 
-#if NET_2_0
 			if (share < FileShare.None || share > (FileShare.ReadWrite | FileShare.Delete)) {
-#else
-			if (share < FileShare.None || share > FileShare.ReadWrite) {
-#endif
-#if NET_2_1
-				if (anonymous)
-					throw new IsolatedStorageException ("Enum value for FileShare was out of legal range.");
-				else
-#endif
 				throw new ArgumentOutOfRangeException ("share", "Enum value was out of legal range.");
 			}
 
@@ -270,19 +217,20 @@ namespace System.IO
 				throw new ArgumentException (string.Format (msg, access, mode));
 			}
 
-			string dname = Path.GetDirectoryName (path);
+			SecurityManager.EnsureElevatedPermissions (); // this is a no-op outside moonlight
+
+			string dname;
+			if (Path.DirectorySeparatorChar != '/' && path.IndexOf ('/') >= 0)
+				dname = Path.GetDirectoryName (Path.GetFullPath (path));
+			else
+				dname = Path.GetDirectoryName (path);
 			if (dname.Length > 0) {
 				string fp = Path.GetFullPath (dname);
 				if (!Directory.Exists (fp)) {
 					// don't leak the path information for isolated storage
 					string msg = Locale.GetText ("Could not find a part of the path \"{0}\".");
 					string fname = (anonymous) ? dname : Path.GetFullPath (path);
-#if NET_2_1
-					// don't use GetSecureFileName for the directory name
-					throw new IsolatedStorageException (String.Format (msg, fname));
-#else
 					throw new DirectoryNotFoundException (String.Format (msg, fname));
-#endif
 				}
 			}
 
@@ -291,11 +239,7 @@ namespace System.IO
 				// don't leak the path information for isolated storage
 				string msg = Locale.GetText ("Could not find file \"{0}\".");
 				string fname = GetSecureFileName (path);
-#if NET_2_1
-				throw new IsolatedStorageException (String.Format (msg, fname));
-#else
 				throw new FileNotFoundException (String.Format (msg, fname), fname);
-#endif
 			}
 
 			// IsolatedStorage needs to keep the Name property to the default "[Unknown]"
@@ -306,19 +250,21 @@ namespace System.IO
 
 			MonoIOError error;
 
-			this.handle = MonoIO.Open (path, mode, access, share, options, out error);
-			if (handle == MonoIO.InvalidHandle) {
+			var nativeHandle = MonoIO.Open (path, mode, access, share, options, out error);
+
+			if (nativeHandle == MonoIO.InvalidHandle) {
 				// don't leak the path information for isolated storage
 				throw MonoIO.GetException (GetSecureFileName (path), error);
 			}
 
+			this.safeHandle = new SafeFileHandle (nativeHandle, false);
+
 			this.access = access;
 			this.owner = true;
-			this.anonymous = anonymous;
 
 			/* Can we open non-files by name? */
-			
-			if (MonoIO.GetFileType (handle, out error) == MonoFileType.Disk) {
+
+			if (MonoIO.GetFileType (safeHandle, out error) == MonoFileType.Disk) {
 				this.canseek = true;
 				this.async = (options & FileOptions.Asynchronous) != 0;
 			} else {
@@ -345,6 +291,44 @@ namespace System.IO
 			}
 		}
 
+		private void Init (SafeFileHandle safeHandle, FileAccess access, bool ownsHandle, int bufferSize, bool isAsync, bool isZeroSize)
+		{
+			if (access < FileAccess.Read || access > FileAccess.ReadWrite)
+				throw new ArgumentOutOfRangeException ("access");
+
+			MonoIOError error;
+			MonoFileType ftype = MonoIO.GetFileType (safeHandle, out error);
+
+			if (error != MonoIOError.ERROR_SUCCESS) {
+				throw MonoIO.GetException (name, error);
+			}
+
+			if (ftype == MonoFileType.Unknown) {
+				throw new IOException ("Invalid handle.");
+			} else if (ftype == MonoFileType.Disk) {
+				this.canseek = true;
+			} else {
+				this.canseek = false;
+			}
+
+			this.safeHandle = safeHandle;
+			ExposeHandle ();
+			this.access = access;
+			this.owner = ownsHandle;
+			this.async = isAsync;
+			this.anonymous = false;
+
+			if (canseek) {
+				buf_start = MonoIO.Seek (safeHandle, 0, SeekOrigin.Current, out error);
+				if (error != MonoIOError.ERROR_SUCCESS) {
+					throw MonoIO.GetException (name, error);
+				}
+			}
+
+			/* Can't set append mode */
+			this.append_startpos=0;
+		}
+
 		// properties
 		
 		public override bool CanRead {
@@ -375,13 +359,13 @@ namespace System.IO
 
 		public string Name {
 			get {
-				return name; 
+				return name;
 			}
 		}
 
 		public override long Length {
 			get {
-				if (handle == MonoIO.InvalidHandle)
+				if (safeHandle.IsClosed)
 					throw new ObjectDisposedException ("Stream has been closed");
 
 				if (!CanSeek)
@@ -391,9 +375,9 @@ namespace System.IO
 				FlushBufferIfDirty ();
 
 				MonoIOError error;
-				long length;
-				
-				length = MonoIO.GetLength (handle, out error);
+
+				long length = MonoIO.GetLength (safeHandle, out error);
+
 				if (error != MonoIOError.ERROR_SUCCESS) {
 					// don't leak the path information for isolated storage
 					throw MonoIO.GetException (GetSecureFileName (name), error);
@@ -405,22 +389,28 @@ namespace System.IO
 
 		public override long Position {
 			get {
-				if (handle == MonoIO.InvalidHandle)
+				if (safeHandle.IsClosed)
 					throw new ObjectDisposedException ("Stream has been closed");
 
-				if(CanSeek == false)
+				if (CanSeek == false)
 					throw new NotSupportedException("The stream does not support seeking");
-				
-				return(buf_start + buf_offset);
-			}
-			set {
-				if (handle == MonoIO.InvalidHandle)
-					throw new ObjectDisposedException ("Stream has been closed");
 
-				if(CanSeek == false) {
-					throw new NotSupportedException("The stream does not support seeking");
+				if (!isExposed)
+					return(buf_start + buf_offset);
+
+				// If the handle was leaked outside we always ask the real handle
+				MonoIOError error;
+
+				long ret = MonoIO.Seek (safeHandle, 0, SeekOrigin.Current, out error);
+
+				if (error != MonoIOError.ERROR_SUCCESS) {
+					// don't leak the path information for isolated storage
+					throw MonoIO.GetException (GetSecureFileName (name), error);
 				}
 
+				return ret;
+			}
+			set {
 				if(value < 0) {
 					throw new ArgumentOutOfRangeException("Attempt to set the position to a negative value");
 				}
@@ -429,47 +419,47 @@ namespace System.IO
 			}
 		}
 
-#if NET_2_0
 		[Obsolete ("Use SafeFileHandle instead")]
-#endif
 		public virtual IntPtr Handle {
 			[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 			[SecurityPermission (SecurityAction.InheritanceDemand, UnmanagedCode = true)]
 			get {
+				var handle = safeHandle.DangerousGetHandle ();
+				if (!isExposed)
+					ExposeHandle ();
 				return handle;
 			}
 		}
 
-#if NET_2_0
 		public virtual SafeFileHandle SafeFileHandle {
 			[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 			[SecurityPermission (SecurityAction.InheritanceDemand, UnmanagedCode = true)]
 			get {
-				SafeFileHandle ret;
-
-				if (safeHandle != null)
-					ret = safeHandle;
-				else
-					ret = new SafeFileHandle (handle, owner);
-
-				FlushBuffer ();
-				return ret;
+				if (!isExposed)
+					ExposeHandle ();
+				return safeHandle;
 			}
 		}
-#endif
+
+		void ExposeHandle ()
+		{
+			isExposed = true;
+			FlushBuffer ();
+			InitBuffer (0, true);
+		}
 
 		// methods
 
 		public override int ReadByte ()
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 
 			if (!CanRead)
 				throw new NotSupportedException ("Stream does not support reading");
-			
+
 			if (buf_size == 0) {
-				int n = ReadData (handle, buf, 0, 1);
+				int n = ReadData (safeHandle, buf, 0, 1);
 				if (n == 0) return -1;
 				else return buf[0];
 			}
@@ -485,7 +475,7 @@ namespace System.IO
 
 		public override void WriteByte (byte value)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 
 			if (!CanWrite)
@@ -511,7 +501,7 @@ namespace System.IO
 
 		public override int Read ([In,Out] byte[] array, int offset, int count)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 			if (array == null)
 				throw new ArgumentNullException ("array");
@@ -538,43 +528,31 @@ namespace System.IO
 
 		int ReadInternal (byte [] dest, int offset, int count)
 		{
-			int copied = 0;
-
 			int n = ReadSegment (dest, offset, count);
-			copied += n;
-			count -= n;
-			
-			if (count == 0) {
-				/* If there was already enough
-				 * buffered, no need to read
-				 * more from the file.
-				 */
-				return (copied);
+			if (n == count) {
+				return count;
 			}
+			
+			int copied = n;
+			count -= n;
 
 			if (count > buf_size) {
 				/* Read as much as we can, up
 				 * to count bytes
 				 */
 				FlushBuffer();
-				n = ReadData (handle, dest,
-					      offset+copied,
-					      count);
-			
+				n = ReadData (safeHandle, dest, offset+n, count);
+
 				/* Make the next buffer read
 				 * start from the right place
 				 */
 				buf_start += n;
 			} else {
 				RefillBuffer ();
-				n = ReadSegment (dest,
-						 offset+copied,
-						 count);
+				n = ReadSegment (dest, offset+copied, count);
 			}
 
-			copied += n;
-
-			return copied;
+			return copied + n;
 		}
 
 		delegate int ReadDelegate (byte [] buffer, int offset, int count);
@@ -582,7 +560,7 @@ namespace System.IO
 		public override IAsyncResult BeginRead (byte [] array, int offset, int numBytes,
 							AsyncCallback userCallback, object stateObject)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 
 			if (!CanRead)
@@ -629,7 +607,7 @@ namespace System.IO
 
 		public override void Write (byte[] array, int offset, int count)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 			if (array == null)
 				throw new ArgumentNullException ("array");
@@ -659,13 +637,16 @@ namespace System.IO
 				MonoIOError error;
 
 				FlushBuffer ();
+				int wcount = count;
 
-				MonoIO.Write (handle, src, offset, count, out error);
-				if (error != MonoIOError.ERROR_SUCCESS) {
-					// don't leak the path information for isolated storage
-					throw MonoIO.GetException (GetSecureFileName (name), error);
+				while (wcount > 0){
+					int n = MonoIO.Write (safeHandle, src, offset, wcount, out error);
+					if (error != MonoIOError.ERROR_SUCCESS)
+						throw MonoIO.GetException (GetSecureFileName (name), error);
+
+					wcount -= n;
+					offset += n;
 				}
-				
 				buf_start += count;
 			} else {
 
@@ -690,7 +671,7 @@ namespace System.IO
 		public override IAsyncResult BeginWrite (byte [] array, int offset, int numBytes,
 							AsyncCallback userCallback, object stateObject)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 
 			if (!CanWrite)
@@ -716,17 +697,20 @@ namespace System.IO
 			result.BytesRead = -1;
 			result.Count = numBytes;
 			result.OriginalCount = numBytes;
-
+/*
 			if (buf_dirty) {
 				MemoryStream ms = new MemoryStream ();
 				FlushBuffer (ms);
 				ms.Write (array, offset, numBytes);
-				offset = 0;
-				numBytes = (int) ms.Length;
-			}
 
-			WriteDelegate w = new WriteDelegate (WriteInternal);
-			return w.BeginInvoke (array, offset, numBytes, userCallback, stateObject);			
+				// Set arguments to new compounded buffer 
+				offset = 0;
+				array = ms.ToArray ();
+				numBytes = array.Length;
+			}
+*/
+			WriteDelegate w = WriteInternal;
+			return w.BeginInvoke (array, offset, numBytes, userCallback, stateObject);	
 		}
 		
 		public override void EndWrite (IAsyncResult asyncResult)
@@ -755,7 +739,7 @@ namespace System.IO
 		{
 			long pos;
 
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 			
 			// make absolute
@@ -796,10 +780,8 @@ namespace System.IO
 			FlushBuffer ();
 
 			MonoIOError error;
-		
-			buf_start = MonoIO.Seek (handle, pos,
-						 SeekOrigin.Begin,
-						 out error);
+
+			buf_start = MonoIO.Seek (safeHandle, pos, SeekOrigin.Begin, out error);
 
 			if (error != MonoIOError.ERROR_SUCCESS) {
 				// don't leak the path information for isolated storage
@@ -811,7 +793,7 @@ namespace System.IO
 
 		public override void SetLength (long value)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 
 			if(CanSeek == false)
@@ -823,11 +805,11 @@ namespace System.IO
 			if(value < 0)
 				throw new ArgumentOutOfRangeException("value is less than 0");
 			
-			Flush ();
+			FlushBuffer ();
 
 			MonoIOError error;
-			
-			MonoIO.SetLength (handle, value, out error);
+
+			MonoIO.SetLength (safeHandle, value, out error);
 			if (error != MonoIOError.ERROR_SUCCESS) {
 				// don't leak the path information for isolated storage
 				throw MonoIO.GetException (GetSecureFileName (name), error);
@@ -839,28 +821,29 @@ namespace System.IO
 
 		public override void Flush ()
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 
 			FlushBuffer ();
-			
-			// The flushing is not actually required, in
-			//the mono runtime we were mapping flush to
-			//`fsync' which is not the same.
-			//
-			//MonoIO.Flush (handle);
 		}
 
-#if !NET_2_0
-		public override void Close ()
+		public virtual void Flush (bool flushToDisk)
 		{
-			Dispose (true);
+			if (safeHandle.IsClosed)
+				throw new ObjectDisposedException ("Stream has been closed");
+
+			FlushBuffer ();
+
+			// This does the fsync
+			if (flushToDisk){
+				MonoIOError error;
+				MonoIO.Flush (safeHandle, out error);
+			}
 		}
-#endif
 
 		public virtual void Lock (long position, long length)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 			if (position < 0) {
 				throw new ArgumentOutOfRangeException ("position must not be negative");
@@ -868,13 +851,10 @@ namespace System.IO
 			if (length < 0) {
 				throw new ArgumentOutOfRangeException ("length must not be negative");
 			}
-			if (handle == MonoIO.InvalidHandle) {
-				throw new ObjectDisposedException ("Stream has been closed");
-			}
-				
+
 			MonoIOError error;
 
-			MonoIO.Lock (handle, position, length, out error);
+			MonoIO.Lock (safeHandle, position, length, out error);
 			if (error != MonoIOError.ERROR_SUCCESS) {
 				// don't leak the path information for isolated storage
 				throw MonoIO.GetException (GetSecureFileName (name), error);
@@ -883,7 +863,7 @@ namespace System.IO
 
 		public virtual void Unlock (long position, long length)
 		{
-			if (handle == MonoIO.InvalidHandle)
+			if (safeHandle.IsClosed)
 				throw new ObjectDisposedException ("Stream has been closed");
 			if (position < 0) {
 				throw new ArgumentOutOfRangeException ("position must not be negative");
@@ -891,10 +871,10 @@ namespace System.IO
 			if (length < 0) {
 				throw new ArgumentOutOfRangeException ("length must not be negative");
 			}
-				
+
 			MonoIOError error;
 
-			MonoIO.Unlock (handle, position, length, out error);
+			MonoIO.Unlock (safeHandle, position, length, out error);
 			if (error != MonoIOError.ERROR_SUCCESS) {
 				// don't leak the path information for isolated storage
 				throw MonoIO.GetException (GetSecureFileName (name), error);
@@ -908,15 +888,14 @@ namespace System.IO
 			Dispose (false);
 		}
 
-#if NET_2_0
 		protected override void Dispose (bool disposing)
-#else
-		protected virtual void Dispose (bool disposing)
-#endif
 		{
 			Exception exc = null;
-			if (handle != MonoIO.InvalidHandle) {
+			if (safeHandle != null && !safeHandle.IsClosed) {
 				try {
+					// If the FileStream is in "exposed" status
+					// it means that we do not have a buffer(we write the data without buffering)
+					// therefor we don't and can't flush the buffer becouse we don't have one.
 					FlushBuffer ();
 				} catch (Exception e) {
 					exc = e;
@@ -924,40 +903,78 @@ namespace System.IO
 
 				if (owner) {
 					MonoIOError error;
-				
-					MonoIO.Close (handle, out error);
+
+					MonoIO.Close (safeHandle.DangerousGetHandle (), out error);
 					if (error != MonoIOError.ERROR_SUCCESS) {
 						// don't leak the path information for isolated storage
 						throw MonoIO.GetException (GetSecureFileName (name), error);
 					}
 
-					handle = MonoIO.InvalidHandle;
+					safeHandle.DangerousRelease ();
 				}
 			}
 
 			canseek = false;
 			access = 0;
-			if (disposing) {
+			
+			if (disposing && buf != null) {
+				if (buf.Length == DefaultBufferSize && buf_recycle == null) {
+					lock (buf_recycle_lock) {
+						if (buf_recycle == null) {
+							buf_recycle = buf;
+						}
+					}
+				}
+				
 				buf = null;
-			}
-			if (disposing)
 				GC.SuppressFinalize (this);
- 			if (exc != null)
- 				throw exc;
+			}
+			if (exc != null)
+				throw exc;
 		}
 
-#if NET_2_0 && !NET_2_1
+#if !NET_2_1
 		public FileSecurity GetAccessControl ()
 		{
-			throw new NotImplementedException ();
+			if (safeHandle.IsClosed)
+				throw new ObjectDisposedException ("Stream has been closed");
+
+			return new FileSecurity (SafeFileHandle,
+						 AccessControlSections.Owner |
+						 AccessControlSections.Group |
+						 AccessControlSections.Access);
 		}
 		
 		public void SetAccessControl (FileSecurity fileSecurity)
 		{
-			throw new NotImplementedException ();
+			if (safeHandle.IsClosed)
+				throw new ObjectDisposedException ("Stream has been closed");
+
+			if (null == fileSecurity)
+				throw new ArgumentNullException ("fileSecurity");
+				
+			fileSecurity.PersistModifications (SafeFileHandle);
 		}
 #endif
 
+		public override Task FlushAsync (CancellationToken cancellationToken)
+		{
+			if (safeHandle.IsClosed)
+				throw new ObjectDisposedException ("Stream has been closed");
+
+			return base.FlushAsync (cancellationToken);
+		}
+
+		public override Task<int> ReadAsync (byte[] buffer, int offset, int count, CancellationToken cancellationToken)
+		{
+			return base.ReadAsync (buffer, offset, count, cancellationToken);
+		}
+
+		public override Task WriteAsync (byte[] buffer, int offset, int count, CancellationToken cancellationToken)
+		{
+			return base.WriteAsync (buffer, offset, count, cancellationToken);
+		}
+
 		// private.
 
 		// ReadSegment, WriteSegment, FlushBuffer,
@@ -967,18 +984,15 @@ namespace System.IO
 
 		private int ReadSegment (byte [] dest, int dest_offset, int count)
 		{
-			if (count > buf_length - buf_offset) {
-				count = buf_length - buf_offset;
-			}
+			count = Math.Min (count, buf_length - buf_offset);
 			
 			if (count > 0) {
-				Buffer.BlockCopy (buf, buf_offset,
-						  dest, dest_offset,
-						  count);
+				// Use the fastest method, all range checks has been done
+				Buffer.BlockCopyInternal (buf, buf_offset, dest, dest_offset, count);
 				buf_offset += count;
 			}
 			
-			return(count);
+			return count;
 		}
 
 		private int WriteSegment (byte [] src, int src_offset,
@@ -1003,31 +1017,35 @@ namespace System.IO
 			return(count);
 		}
 
-		void FlushBuffer (Stream st)
+		void FlushBuffer ()
 		{
 			if (buf_dirty) {
-				MonoIOError error;
+//				if (st == null) {
+					MonoIOError error;
 
-				if (CanSeek == true) {
-					MonoIO.Seek (handle, buf_start,
-						     SeekOrigin.Begin,
-						     out error);
-					if (error != MonoIOError.ERROR_SUCCESS) {
-						// don't leak the path information for isolated storage
-						throw MonoIO.GetException (GetSecureFileName (name), error);
-					}
-				}
-				if (st == null) {
-					MonoIO.Write (handle, buf, 0,
-						      buf_length, out error);
+					if (CanSeek == true && !isExposed) {
+						MonoIO.Seek (safeHandle, buf_start, SeekOrigin.Begin, out error);
 
-					if (error != MonoIOError.ERROR_SUCCESS) {
-						// don't leak the path information for isolated storage
-						throw MonoIO.GetException (GetSecureFileName (name), error);
+						if (error != MonoIOError.ERROR_SUCCESS) {
+							// don't leak the path information for isolated storage
+							throw MonoIO.GetException (GetSecureFileName (name), error);
+						}
 					}
-				} else {
-					st.Write (buf, 0, buf_length);
-				}
+
+					int wcount = buf_length;
+					int offset = 0;
+					while (wcount > 0){
+						int n = MonoIO.Write (safeHandle, buf, 0, buf_length, out error);
+						if (error != MonoIOError.ERROR_SUCCESS) {
+							// don't leak the path information for isolated storage
+							throw MonoIO.GetException (GetSecureFileName (name), error);
+						}
+						wcount -= n;
+						offset += n;
+					}
+//				} else {
+//					st.Write (buf, 0, buf_length);
+//				}
 			}
 
 			buf_start += buf_offset;
@@ -1035,26 +1053,20 @@ namespace System.IO
 			buf_dirty = false;
 		}
 
-		private void FlushBuffer ()
-		{
-			FlushBuffer (null);
-		}
-
 		private void FlushBufferIfDirty ()
 		{
 			if (buf_dirty)
-				FlushBuffer (null);
+				FlushBuffer ();
 		}
 
 		private void RefillBuffer ()
 		{
-			FlushBuffer (null);
-			
-			buf_length = ReadData (handle, buf, 0,
-					       buf_size);
+			FlushBuffer ();
+
+			buf_length = ReadData (safeHandle, buf, 0, buf_size);
 		}
 
-		private int ReadData (IntPtr handle, byte[] buf, int offset,
+		private int ReadData (SafeHandle safeHandle, byte[] buf, int offset,
 				      int count)
 		{
 			MonoIOError error;
@@ -1062,7 +1074,7 @@ namespace System.IO
 
 			/* when async == true, if we get here we don't suport AIO or it's disabled
 			 * and we're using the threadpool */
-			amount = MonoIO.Read (handle, buf, offset, count, out error);
+			amount = MonoIO.Read (safeHandle, buf, offset, count, out error);
 			if (error == MonoIOError.ERROR_BROKEN_PIPE) {
 				amount = 0; // might not be needed, but well...
 			} else if (error != MonoIOError.ERROR_SUCCESS) {
@@ -1078,26 +1090,40 @@ namespace System.IO
 			return(amount);
 		}
 				
-		private void InitBuffer (int size, bool noBuffering)
+		void InitBuffer (int size, bool isZeroSize)
 		{
-			if (noBuffering) {
+			if (isZeroSize) {
 				size = 0;
-				// We need a buffer for the ReadByte method. This buffer won't
-				// be used for anything else since buf_size==0.
-				buf = new byte [1];
-			}
-			else {
+				buf = new byte[1];
+			} else {
 				if (size <= 0)
 					throw new ArgumentOutOfRangeException ("bufferSize", "Positive number required.");
-				if (size < 8)
-					size = 8;
-				buf = new byte [size];
+
+				size = Math.Max (size, 8);
+
+				//
+				// Instead of allocating a new default buffer use the
+				// last one if there is any available
+				//		
+				if (size <= DefaultBufferSize && buf_recycle != null) {
+					lock (buf_recycle_lock) {
+						if (buf_recycle != null) {
+							buf = buf_recycle;
+							buf_recycle = null;
+						}
+					}
+				}
+
+				if (buf == null)
+					buf = new byte [size];
+				else
+					Array.Clear (buf, 0, size);
 			}
 					
 			buf_size = size;
-			buf_start = 0;
-			buf_offset = buf_length = 0;
-			buf_dirty = false;
+//			buf_start = 0;
+//			buf_offset = buf_length = 0;
+//			buf_dirty = false;
 		}
 
 		private string GetSecureFileName (string filename)
@@ -1113,27 +1139,30 @@ namespace System.IO
 
 		// fields
 
-		internal const int DefaultBufferSize = 8192;
+		internal const int DefaultBufferSize = 4096;
+
+		// Input buffer ready for recycling				
+		static byte[] buf_recycle;
+		static readonly object buf_recycle_lock = new object ();
+
+		private byte [] buf;			// the buffer
+		private string name = "[Unknown]";	// name of file.
+
+		private SafeFileHandle safeHandle;
+		private bool isExposed;
+
+		private long append_startpos;
 
 		private FileAccess access;
 		private bool owner;
 		private bool async;
 		private bool canseek;
-		private long append_startpos;
 		private bool anonymous;
+		private bool buf_dirty;			// true if buffer has been written to
 
-		private byte [] buf;			// the buffer
 		private int buf_size;			// capacity in bytes
 		private int buf_length;			// number of valid bytes in buffer
 		private int buf_offset;			// position of next byte
-		private bool buf_dirty;			// true if buffer has been written to
 		private long buf_start;			// location of buffer in file
-		private string name = "[Unknown]";	// name of file.
-
-		IntPtr handle;				// handle to underlying file
-#if NET_2_0
-		SafeFileHandle safeHandle;              // set only when using one of the
-							// constructors taking SafeFileHandle
-#endif
 	}
 }
