@@ -5,6 +5,7 @@
 // Copyright (C) 2001 Moonlight Enterprises, All Rights Reserved
 // Copyright (C) 2002 Ximian, Inc. (http://www.ximian.com)
 // Copyright (C) 2003 Ben Maurer
+// Copyright 2011 Xamarin Inc (http://www.xamarin.com).
 // 
 // Author:         Jim Richardson, develop@wtfo-guru.com
 //                 Dan Lewis (dihlewis@yahoo.co.uk)
@@ -221,6 +222,8 @@ namespace System.IO {
 
 				if (l >= 2 && DirectorySeparatorChar == '\\' && ret [l - 1] == VolumeSeparatorChar)
 					return ret + DirectorySeparatorChar;
+				else if (l == 1 && DirectorySeparatorChar == '\\' && path.Length >= 2 && path [nLast] == VolumeSeparatorChar)
+					return ret + VolumeSeparatorChar;
 				else {
 					//
 					// Important: do not use CanonicalizePath here, use
@@ -286,6 +289,29 @@ namespace System.IO {
 			return fullpath;
 		}
 
+		// http://msdn.microsoft.com/en-us/library/windows/desktop/aa364963%28v=vs.85%29.aspx
+		[DllImport("Kernel32.dll", SetLastError = true, CharSet = CharSet.Unicode)]
+		private static extern int GetFullPathName(string path, int numBufferChars, StringBuilder buffer, ref IntPtr lpFilePartOrNull); 
+
+		internal static string GetFullPathName(string path)
+		{
+			const int MAX_PATH = 260;
+			StringBuilder buffer = new StringBuilder(MAX_PATH);
+			IntPtr ptr = IntPtr.Zero;
+			int length = GetFullPathName(path, MAX_PATH, buffer, ref ptr);
+			if (length == 0)
+			{
+				int error = Marshal.GetLastWin32Error();
+				throw new IOException("Windows API call to GetFullPathName failed, Windows error code: " + error);
+			}
+			else if (length > MAX_PATH)
+			{
+				buffer = new StringBuilder(length);
+				GetFullPathName(path, length, buffer, ref ptr);
+			}
+			return buffer.ToString();
+		}
+
 		internal static string WindowsDriveAdjustment (string path)
 		{
 			// two special cases to consider when a drive is specified
@@ -294,14 +320,14 @@ namespace System.IO {
 			if ((path [1] != ':') || !Char.IsLetter (path [0]))
 				return path;
 
-			string current = Directory.GetCurrentDirectory ();
+			string current = Directory.InsecureGetCurrentDirectory ();
 			// first, only the drive is specified
 			if (path.Length == 2) {
 				// then if the current directory is on the same drive
 				if (current [0] == path [0])
 					path = current; // we return it
 				else
-					path += '\\';
+					path = GetFullPathName(path); // we have to use the GetFullPathName Windows API
 			} else if ((path [2] != Path.DirectorySeparatorChar) && (path [2] != Path.AltDirectorySeparatorChar)) {
 				// second, the drive + a directory is specified *without* a separator between them (e.g. C:dir).
 				// If the current directory is on the specified drive...
@@ -309,8 +335,8 @@ namespace System.IO {
 					// then specified directory is appended to the current drive directory
 					path = Path.Combine (current, path.Substring (2, path.Length - 2));
 				} else {
-					// if not, then just pretend there was a separator (Path.Combine won't work in this case)
-					path = String.Concat (path.Substring (0, 2), DirectorySeparatorStr, path.Substring (2, path.Length - 2));
+					// we have to use the GetFullPathName Windows API
+					path = GetFullPathName(path);
 				}
 			}
 			return path;
@@ -336,8 +362,8 @@ namespace System.IO {
 
 			var canonicalize = true;
 			if (path.Length >= 2 &&
-				IsDsc (path [0]) &&
-				IsDsc (path [1])) {
+				IsDirectorySeparator (path [0]) &&
+				IsDirectorySeparator (path [1])) {
 				if (path.Length == 2 || path.IndexOf (path [0], 2) < 0)
 					throw new ArgumentException ("UNC paths should be of the form \\\\server\\share.");
 
@@ -348,23 +374,29 @@ namespace System.IO {
 				if (!IsPathRooted (path)) {
 					
 					// avoid calling expensive CanonicalizePath when possible
-					var start = 0;
-					while ((start = path.IndexOf ('.', start)) != -1) {
-						if (++start == path.Length || path [start] == DirectorySeparatorChar || path [start] == AltDirectorySeparatorChar)
-							break;
+					if (!Environment.IsRunningOnWindows) {
+						var start = 0;
+						while ((start = path.IndexOf ('.', start)) != -1) {
+							if (++start == path.Length || path [start] == DirectorySeparatorChar || path [start] == AltDirectorySeparatorChar)
+								break;
+						}
+						canonicalize = start > 0;
 					}
-					canonicalize = start > 0;
-					
-					path = Directory.GetCurrentDirectory () + DirectorySeparatorStr + path;
+
+					var cwd = Directory.InsecureGetCurrentDirectory();
+					if (cwd [cwd.Length - 1] == DirectorySeparatorChar)
+						path = cwd + path;
+					else
+						path = cwd + DirectorySeparatorChar + path;					
 				} else if (DirectorySeparatorChar == '\\' &&
 					path.Length >= 2 &&
-					IsDsc (path [0]) &&
-					!IsDsc (path [1])) { // like `\abc\def'
-					string current = Directory.GetCurrentDirectory ();
+					IsDirectorySeparator (path [0]) &&
+					!IsDirectorySeparator (path [1])) { // like `\abc\def'
+					string current = Directory.InsecureGetCurrentDirectory();
 					if (current [1] == VolumeSeparatorChar)
 						path = current.Substring (0, 2) + path;
 					else
-						path = current.Substring (0, current.IndexOf ('\\', current.IndexOf ("\\\\") + 1));
+						path = current.Substring (0, current.IndexOf ('\\', current.IndexOfOrdinalUnchecked ("\\\\") + 1));
 				}
 			}
 			
@@ -372,16 +404,16 @@ namespace System.IO {
 			    path = CanonicalizePath (path);
 
 			// if the original ended with a [Alt]DirectorySeparatorChar then ensure the full path also ends with one
-			if (IsDsc (end) && (path [path.Length - 1] != DirectorySeparatorChar))
+			if (IsDirectorySeparator (end) && (path [path.Length - 1] != DirectorySeparatorChar))
 				path += DirectorySeparatorChar;
 
 			return path;
 		}
 
-		static bool IsDsc (char c) {
+		internal static bool IsDirectorySeparator (char c) {
 			return c == DirectorySeparatorChar || c == AltDirectorySeparatorChar;
 		}
-		
+
 		public static string GetPathRoot (string path)
 		{
 			if (path == null)
@@ -395,36 +427,36 @@ namespace System.IO {
 			
 			if (DirectorySeparatorChar == '/') {
 				// UNIX
-				return IsDsc (path [0]) ? DirectorySeparatorStr : String.Empty;
+				return IsDirectorySeparator (path [0]) ? DirectorySeparatorStr : String.Empty;
 			} else {
 				// Windows
 				int len = 2;
 
-				if (path.Length == 1 && IsDsc (path [0]))
+				if (path.Length == 1 && IsDirectorySeparator (path [0]))
 					return DirectorySeparatorStr;
 				else if (path.Length < 2)
 					return String.Empty;
 
-				if (IsDsc (path [0]) && IsDsc (path[1])) {
+				if (IsDirectorySeparator (path [0]) && IsDirectorySeparator (path[1])) {
 					// UNC: \\server or \\server\share
 					// Get server
-					while (len < path.Length && !IsDsc (path [len])) len++;
+					while (len < path.Length && !IsDirectorySeparator (path [len])) len++;
 
 					// Get share
 					if (len < path.Length) {
 						len++;
-						while (len < path.Length && !IsDsc (path [len])) len++;
+						while (len < path.Length && !IsDirectorySeparator (path [len])) len++;
 					}
 
 					return DirectorySeparatorStr +
 						DirectorySeparatorStr +
 						path.Substring (2, len - 2).Replace (AltDirectorySeparatorChar, DirectorySeparatorChar);
-				} else if (IsDsc (path [0])) {
+				} else if (IsDirectorySeparator (path [0])) {
 					// path starts with '\' or '/'
 					return DirectorySeparatorStr;
 				} else if (path[1] == VolumeSeparatorChar) {
 					// C:\folder
-					if (path.Length >= 3 && (IsDsc (path [2]))) len++;
+					if (path.Length >= 3 && (IsDirectorySeparator (path [2]))) len++;
 				} else
 					return Directory.GetCurrentDirectory ().Substring (0, 2);// + path.Substring (0, len);
 				return path.Substring (0, len);
@@ -438,33 +470,27 @@ namespace System.IO {
 			FileStream f = null;
 			string path;
 			Random rnd;
-			int num = 0;
+			int num;
+			int count = 0;
 
 			SecurityManager.EnsureElevatedPermissions (); // this is a no-op outside moonlight
 
 			rnd = new Random ();
+			var tmp_path = GetTempPath ();
 			do {
 				num = rnd.Next ();
 				num++;
-				path = Path.Combine (GetTempPath(), "tmp" + num.ToString("x") + ".tmp");
+				path = Path.Combine (tmp_path, "tmp" + num.ToString ("x", CultureInfo.InvariantCulture) + ".tmp");
 
 				try {
 					f = new FileStream (path, FileMode.CreateNew, FileAccess.ReadWrite, FileShare.Read,
 							    8192, false, (FileOptions) 1);
-				}
-				catch (SecurityException) {
-					// avoid an endless loop
-					throw;
-				}
-				catch (UnauthorizedAccessException) {
-					// This can happen if we don't have write permission to /tmp
-					throw;
-				}
-				catch (DirectoryNotFoundException) {
-					// This happens when TMPDIR does not exist
-					throw;
-				}
-				catch {
+				} catch (IOException ex){
+					if (ex.hresult != MonoIO.FileAlreadyExistsHResult || count ++ > 65536)
+						throw;
+				} catch (UnauthorizedAccessException ex) {
+					if (count ++ > 65536)
+						throw new IOException (ex.Message, ex);
 				}
 			} while (f == null);
 			
@@ -602,11 +628,11 @@ namespace System.IO {
 		static string GetServerAndShare (string path)
 		{
 			int len = 2;
-			while (len < path.Length && !IsDsc (path [len])) len++;
+			while (len < path.Length && !IsDirectorySeparator (path [len])) len++;
 
 			if (len < path.Length) {
 				len++;
-				while (len < path.Length && !IsDsc (path [len])) len++;
+				while (len < path.Length && !IsDirectorySeparator (path [len])) len++;
 			}
 
 			return path.Substring (2, len - 2).Replace (AltDirectorySeparatorChar, DirectorySeparatorChar);
@@ -620,8 +646,8 @@ namespace System.IO {
 				return false;
 
 			// UNC handling
-			if (IsDsc (root[0]) && IsDsc (root[1])) {
-				if (!(IsDsc (path[0]) && IsDsc (path[1])))
+			if (IsDirectorySeparator (root[0]) && IsDirectorySeparator (root[1])) {
+				if (!(IsDirectorySeparator (path[0]) && IsDirectorySeparator (path[1])))
 					return false;
 
 				string rootShare = GetServerAndShare (root);
@@ -638,7 +664,7 @@ namespace System.IO {
 				return false;
 			if ((root.Length > 2) && (path.Length > 2)) {
 				// but don't directory compare the directory separator
-				return (IsDsc (root[2]) && IsDsc (path[2]));
+				return (IsDirectorySeparator (root[2]) && IsDirectorySeparator (path[2]));
 			}
 			return true;
 		}
@@ -666,7 +692,7 @@ namespace System.IO {
 			int target = 0;
 
 			bool isUnc = Environment.IsRunningOnWindows &&
-				root.Length > 2 && IsDsc (root[0]) && IsDsc (root[1]);
+				root.Length > 2 && IsDirectorySeparator (root[0]) && IsDirectorySeparator (root[1]);
 
 			// Set an overwrite limit for UNC paths since '\' + server + share
 			// must not be eliminated by the '..' elimination algorithm.
@@ -702,7 +728,7 @@ namespace System.IO {
 
 					if (isUnc) {
 						return ret;
-					} else if (!IsDsc (path[0]) && SameRoot (root, path)) {
+					} else if (!IsDirectorySeparator (path[0]) && SameRoot (root, path)) {
 						if (ret.Length <= 2 && !ret.EndsWith (DirectorySeparatorStr)) // '\' after "c:"
 							ret += Path.DirectorySeparatorChar;
 						return ret;
@@ -710,14 +736,17 @@ namespace System.IO {
 						string current = Directory.GetCurrentDirectory ();
 						if (current.Length > 1 && current[1] == Path.VolumeSeparatorChar) {
 							// DOS local file path
-							if (ret.Length == 0 || IsDsc (ret[0]))
+							if (ret.Length == 0 || IsDirectorySeparator (ret[0]))
 								ret += '\\';
 							return current.Substring (0, 2) + ret;
-						} else if (IsDsc (current[current.Length - 1]) && IsDsc (ret[0]))
+						} else if (IsDirectorySeparator (current[current.Length - 1]) && IsDirectorySeparator (ret[0]))
 							return current + ret.Substring (1);
 						else
 							return current + ret;
 					}
+				} else {
+					if (root != "" && ret.Length > 0 && ret [0] != '/')
+						ret = root + ret;
 				}
 				return ret;
 			}
@@ -747,11 +776,7 @@ namespace System.IO {
 			return String.Compare (subset, slast, path, slast, subset.Length - slast) == 0;
 		}
 
-#if NET_4_0 || MOONLIGHT || MOBILE
 		public
-#else
-                internal
-#endif
 		static string Combine (params string [] paths)
 		{
 			if (paths == null)
@@ -761,13 +786,21 @@ namespace System.IO {
 			var ret = new StringBuilder ();
 			int pathsLen = paths.Length;
 			int slen;
+			need_sep = false;
+
 			foreach (var s in paths) {
-				need_sep = false;
 				if (s == null)
 					throw new ArgumentNullException ("One of the paths contains a null value", "paths");
+				if (s.Length == 0)
+					continue;
 				if (s.IndexOfAny (InvalidPathChars) != -1)
 					throw new ArgumentException ("Illegal characters in path.");
-				
+
+				if (need_sep) {
+					need_sep = false;
+					ret.Append (DirectorySeparatorStr);
+				}
+
 				pathsLen--;
 				if (IsPathRooted (s))
 					ret.Length = 0;
@@ -779,19 +812,12 @@ namespace System.IO {
 					if (p1end != DirectorySeparatorChar && p1end != AltDirectorySeparatorChar && p1end != VolumeSeparatorChar)
 						need_sep = true;
 				}
-				
-				if (need_sep)
-					ret.Append (DirectorySeparatorStr);
 			}
 
 			return ret.ToString ();
 		}
 
-#if NET_4_0 || MOONLIGHT || MOBILE
 		public
-#else
-                internal
-#endif
 		static string Combine (string path1, string path2, string path3)
 		{
 			if (path1 == null)
@@ -806,11 +832,7 @@ namespace System.IO {
 			return Combine (new string [] { path1, path2, path3 });
 		}
 
-#if NET_4_0 || MOONLIGHT || MOBILE
 		public
-#else
-                internal
-#endif
 		static string Combine (string path1, string path2, string path3, string path4)
 		{
 			if (path1 == null)
@@ -841,11 +863,11 @@ namespace System.IO {
 				throw new ArgumentException (Locale.GetText ("Path is empty"));
 			if (path.IndexOfAny (Path.InvalidPathChars) != -1)
 				throw new ArgumentException (Locale.GetText ("Path contains invalid chars"));
-#if MOONLIGHT
-			// On Moonlight (SL4+) there are some limitations in "Elevated Trust"
-			if (SecurityManager.HasElevatedPermissions) {
+			if (Environment.IsRunningOnWindows) {
+				int idx = path.IndexOf (':');
+				if (idx >= 0 && idx != 1)
+					throw new ArgumentException (parameterName);
 			}
-#endif
 		}
 	}
 }
