@@ -30,6 +30,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+#if !FULL_AOT_RUNTIME
 using System;
 using System.Reflection;
 using System.Reflection.Emit;
@@ -44,6 +45,7 @@ namespace System.Reflection.Emit {
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_ConstructorBuilder))]
 	[ClassInterface (ClassInterfaceType.None)]
+	[StructLayout (LayoutKind.Sequential)]
 	public sealed class ConstructorBuilder : ConstructorInfo, _ConstructorBuilder {
 	
 #pragma warning disable 169, 414
@@ -112,32 +114,36 @@ namespace System.Reflection.Emit {
 
 		public override ParameterInfo[] GetParameters ()
 		{
-			if (!type.is_created && !IsCompilerContext)
+			if (!type.is_created)
 				throw not_created ();
 
 			return GetParametersInternal ();
 		}
 
-		internal ParameterInfo [] GetParametersInternal ()
+		internal override ParameterInfo [] GetParametersInternal ()
 		{
 			if (parameters == null)
-				return new ParameterInfo [0];
+				return EmptyArray<ParameterInfo>.Value;
 
 			ParameterInfo [] retval = new ParameterInfo [parameters.Length];
 			for (int i = 0; i < parameters.Length; i++)
-				retval [i] = new ParameterInfo (pinfo == null ? null
+				retval [i] = ParameterInfo.New (pinfo == null ? null
 					: pinfo [i + 1], parameters [i], this, i + 1);
 
 			return retval;
 		}
 		
-		internal override int GetParameterCount ()
+		internal override int GetParametersCount ()
 		{
 			if (parameters == null)
 				return 0;
 			
 			return parameters.Length;
 		}
+
+		internal override Type GetParameterType (int pos) {
+			return parameters [pos];
+		}
 		
 		public override Object Invoke (Object obj, BindingFlags invokeAttr, Binder binder, Object[] parameters, CultureInfo culture)
 		{
@@ -173,9 +179,7 @@ namespace System.Reflection.Emit {
 			}
 		}
 
-#if NET_4_0
 		[Obsolete]
-#endif
 		public Type ReturnType {
 			get {
 				return null;
@@ -226,7 +230,7 @@ namespace System.Reflection.Emit {
 
 		public ParameterBuilder DefineParameter (int iSequence, ParameterAttributes attributes, string strParamName)
 		{
-			if (iSequence < 1 || iSequence > GetParameterCount ())
+			if (iSequence < 1 || iSequence > GetParametersCount ())
 				throw new ArgumentOutOfRangeException ("iSequence");
 			if (type.is_created)
 				throw not_after_created ();
@@ -245,23 +249,12 @@ namespace System.Reflection.Emit {
 
 		public override object [] GetCustomAttributes (bool inherit)
 		{
-			/*
-			 * On MS.NET, this always returns not_supported, but we can't do this
-			 * since there would be no way to obtain custom attributes of 
-			 * dynamically created ctors.
-			 */
-			if (type.is_created && IsCompilerContext)
-				return MonoCustomAttrs.GetCustomAttributes (this, inherit);
-			else
-				throw not_supported ();
+			throw not_supported ();
 		}
 
 		public override object [] GetCustomAttributes (Type attributeType, bool inherit)
 		{
-			if (type.is_created && IsCompilerContext)
-				return MonoCustomAttrs.GetCustomAttributes (this, attributeType, inherit);
-			else
-				throw not_supported ();
+			throw not_supported ();
 		}
 
 		public ILGenerator GetILGenerator ()
@@ -356,7 +349,7 @@ namespace System.Reflection.Emit {
 				throw new InvalidOperationException ("Method '" + Name + "' does not have a method body.");
 			}
 			if (ilgen != null)
-				ilgen.label_fixup ();
+				ilgen.label_fixup (this);
 		}
 		
 		internal void GenerateDebugInfo (ISymbolWriter symbolWriter)
@@ -375,14 +368,6 @@ namespace System.Reflection.Emit {
 			return type.get_next_table_index (obj, table, inc);
 		}
 
-		private bool IsCompilerContext {
-			get {
-				ModuleBuilder mb = (ModuleBuilder) TypeBuilder.Module;
-				AssemblyBuilder ab = (AssemblyBuilder) mb.Assembly;
-				return ab.IsCompilerContext;
-			}
-		}
-
 		private void RejectIfCreated ()
 		{
 			if (type.is_created)
@@ -425,3 +410,4 @@ namespace System.Reflection.Emit {
 		}
 	}
 }
+#endif
