@@ -31,6 +31,7 @@
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
 //
 
+#if !FULL_AOT_RUNTIME
 using System;
 using System.Reflection;
 using System.Reflection.Emit;
@@ -38,11 +39,10 @@ using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
 
 namespace System.Reflection.Emit {
-#if NET_2_0
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_CustomAttributeBuilder))]
-#endif
 	[ClassInterface (ClassInterfaceType.None)]
+	[StructLayout (LayoutKind.Sequential)]
 	public class CustomAttributeBuilder : _CustomAttributeBuilder {
 		ConstructorInfo ctor;
 		byte[] data;
@@ -93,9 +93,44 @@ namespace System.Reflection.Emit {
 			/* FIXME: Add more checks */
 			if (t.IsArray && t.GetArrayRank () > 1)
 				return false;
+			if (t is TypeBuilder && t.IsEnum) {
+				// Check that the enum is properly constructed, the unmanaged code
+				// depends on this
+				Enum.GetUnderlyingType (t);
+			}
+			if (t.IsClass && !(t.IsArray || t == typeof (object) || t == typeof (Type) || t == typeof (string) || t.Assembly.GetName ().Name == "mscorlib"))
+				return false;
+			if (t.IsValueType && !(t.IsPrimitive || t.IsEnum || ((t.Assembly is AssemblyBuilder) && t.Assembly.GetName ().Name == "mscorlib")))
+				return false;
 			return true;
 		}
 
+		private bool IsValidParam (object o, Type paramType)
+		{
+			Type t = o.GetType ();
+			if (!IsValidType (t))
+				return false;
+			if (paramType == typeof (object)) {
+				if (t.IsArray && t.GetArrayRank () == 1)
+					return IsValidType (t.GetElementType ());
+				if (!t.IsPrimitive && !typeof (Type).IsAssignableFrom (t) && t != typeof (string) && !t.IsEnum)
+					return false;
+			}
+			return true;
+		}
+
+		static bool IsValidValue (Type type, object value) {
+			if (type.IsValueType && value == null)
+				return false;
+			if (type.IsArray && type.GetElementType ().IsValueType) {
+				foreach (var v in (Array)value) {
+					if (v == null)
+						return false;
+				}
+			}
+			return true;
+		} 
+
 		private void Initialize (ConstructorInfo con, object [] constructorArgs,
 				PropertyInfo [] namedProperties, object [] propertyValues,
 				FieldInfo [] namedFields, object [] fieldValues)
@@ -113,7 +148,7 @@ namespace System.Reflection.Emit {
 				throw new ArgumentNullException ("namedFields");
 			if (fieldValues == null)
 				throw new ArgumentNullException ("fieldValues");
-			if (con.GetParameterCount () != constructorArgs.Length)
+			if (con.GetParametersCount () != constructorArgs.Length)
 				throw new ArgumentException ("Parameter count does not match " +
 						"passed in argument value count.");
 			if (namedProperties.Length != propertyValues.Length)
@@ -131,10 +166,12 @@ namespace System.Reflection.Emit {
 			i = 0;
 			foreach (FieldInfo fi in namedFields) {
 				Type t = fi.DeclaringType;
-				if (!IsValidType (t))
-					throw new ArgumentException ("Field '" + fi.Name + "' does not have a valid type.");
 				if ((atype != t) && (!t.IsSubclassOf (atype)) && (!atype.IsSubclassOf (t)))
 					throw new ArgumentException ("Field '" + fi.Name + "' does not belong to the same class as the constructor");
+				if (!IsValidType (fi.FieldType))
+					throw new ArgumentException ("Field '" + fi.Name + "' does not have a valid type.");
+				if (!IsValidValue (fi.FieldType, fieldValues [i]))
+					throw new ArgumentException ("Field " + fi.Name + " is not a valid value.");
 				// FIXME: Check enums and TypeBuilders as well
 				if (fieldValues [i] != null)
 					// IsEnum does not seem to work on TypeBuilders
@@ -154,10 +191,12 @@ namespace System.Reflection.Emit {
 				if (!pi.CanWrite)
 					throw new ArgumentException ("Property '" + pi.Name + "' does not have a setter.");
 				Type t = pi.DeclaringType;
-				if (!IsValidType (t))
-					throw new ArgumentException ("Property '" + pi.Name + "' does not have a valid type.");
 				if ((atype != t) && (!t.IsSubclassOf (atype)) && (!atype.IsSubclassOf (t)))
 					throw new ArgumentException ("Property '" + pi.Name + "' does not belong to the same class as the constructor");
+				if (!IsValidType (pi.PropertyType))
+					throw new ArgumentException ("Property '" + pi.Name + "' does not have a valid type.");
+				if (!IsValidValue (pi.PropertyType, propertyValues [i]))
+					throw new ArgumentException ("Property " + pi.Name + " is not a valid value.");
 				if (propertyValues [i] != null) {
 					if (!(pi.PropertyType is TypeBuilder) && !pi.PropertyType.IsEnum && !pi.PropertyType.IsInstanceOfType (propertyValues [i]))
 						if (!pi.PropertyType.IsArray)
@@ -171,11 +210,17 @@ namespace System.Reflection.Emit {
 				if (pi != null) {
 					Type paramType = pi.ParameterType;
 					if (!IsValidType (paramType))
-						throw new ArgumentException ("Argument " + i + " does not have a valid type.");
-					if (constructorArgs [i] != null)
+						throw new ArgumentException ("Parameter " + i + " does not have a valid type.");
+					if (!IsValidValue (paramType, constructorArgs [i]))
+						throw new ArgumentException ("Parameter " + i + " is not a valid value.");
+					
+					if (constructorArgs [i] != null) {
 						if (!(paramType is TypeBuilder) && !paramType.IsEnum && !paramType.IsInstanceOfType (constructorArgs [i]))
 							if (!paramType.IsArray)
 								throw new ArgumentException ("Value of argument " + i + " does not match parameter type: " + paramType + " -> " + constructorArgs [i]);
+						if (!IsValidParam (constructorArgs [i], paramType))
+							throw new ArgumentException ("Cannot emit a CustomAttribute with argument of type " + constructorArgs [i].GetType () + "."); 
+					}
 				}
 				i ++;
 			}
@@ -266,6 +311,12 @@ namespace System.Reflection.Emit {
 					hasSize = true;
 					break;
 				case "SafeArraySubType":
+					value = (int)data[pos++];
+					value |= ((int)data[pos++]) << 8;
+					value |= ((int)data[pos++]) << 16;
+					value |= ((int)data[pos++]) << 24;
+					subtype = (UnmanagedType)value;
+					break;
 				case "IidParameterIndex":
 					pos += 4;
 					break;
@@ -481,7 +532,9 @@ namespace System.Reflection.Emit {
 			ConstructorBuilder cb = ctor as ConstructorBuilder;
 			if (cb != null)
 				return cb.GetParametersInternal ();
-			return ctor.GetParameters ();
+				
+			return ctor.GetParametersInternal ();
 		}
 	}
 }
+#endif
