@@ -1,4 +1,3 @@
-
 //
 // Copyright (C) 2004 Novell, Inc (http://www.novell.com)
 //
@@ -31,6 +30,7 @@
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
 //
 
+#if !FULL_AOT_RUNTIME
 using System;
 using System.Reflection;
 using System.Collections;
@@ -46,6 +46,7 @@ namespace System.Reflection.Emit {
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_ModuleBuilder))]
 	[ClassInterface (ClassInterfaceType.None)]
+	[StructLayout (LayoutKind.Sequential)]
 	public class ModuleBuilder : Module, _ModuleBuilder {
 
 #pragma warning disable 169, 414
@@ -103,19 +104,12 @@ namespace System.Reflection.Emit {
 			}
 
 			if (emitSymbolInfo) {
-#if MOONLIGHT
-				symbolWriter = new Mono.CompilerServices.SymbolWriter.SymbolWriterImpl (this);
-#else
 				Assembly asm = Assembly.LoadWithPartialName ("Mono.CompilerServices.SymbolWriter");
 				if (asm == null)
-					throw new ExecutionEngineException ("The assembly for default symbol writer cannot be loaded");
-
-				Type t = asm.GetType ("Mono.CompilerServices.SymbolWriter.SymbolWriterImpl");
-				if (t == null)
-					throw new ExecutionEngineException ("The type that implements the default symbol writer interface cannot be found");
+					throw new TypeLoadException ("The assembly for default symbol writer cannot be loaded");
 
+				Type t = asm.GetType ("Mono.CompilerServices.SymbolWriter.SymbolWriterImpl", true);
 				symbolWriter = (ISymbolWriter) Activator.CreateInstance (t, new object[] { this });
-#endif
 				string fileName = fqname;
 				if (assemblyb.AssemblyDir != null)
 					fileName = Path.Combine (assemblyb.AssemblyDir, fileName);
@@ -539,7 +533,7 @@ namespace System.Reflection.Emit {
 			if (resourceFileName == String.Empty)
 				throw new ArgumentException ("resourceFileName");
 			if (!File.Exists (resourceFileName) || Directory.Exists (resourceFileName))
-				throw new FileNotFoundException ("File '" + resourceFileName + "' does not exists or is a directory.");
+				throw new FileNotFoundException ("File '" + resourceFileName + "' does not exist or is a directory.");
 
 			throw new NotImplementedException ();
 		}
@@ -589,8 +583,7 @@ namespace System.Reflection.Emit {
 		{
 			if (method == null)
 				throw new ArgumentNullException ("method");
-			if (method.DeclaringType.Module != this)
-				throw new InvalidOperationException ("The method is not in this module");
+
 			return new MethodToken (GetToken (method));
 		}
 
@@ -604,6 +597,8 @@ namespace System.Reflection.Emit {
 		{
 			if (con == null)
 				throw new ArgumentNullException ("con");
+			if (con.DeclaringType.Module != this)
+				throw new InvalidOperationException ("The constructor is not in this module");
 			return new MethodToken (GetToken (con));
 		}
 
@@ -656,10 +651,10 @@ namespace System.Reflection.Emit {
 		private static extern int getUSIndex (ModuleBuilder mb, string str);
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private static extern int getToken (ModuleBuilder mb, object obj);
+		private static extern int getToken (ModuleBuilder mb, object obj, bool create_open_instance);
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private static extern int getMethodToken (ModuleBuilder mb, MethodInfo method,
+		private static extern int getMethodToken (ModuleBuilder mb, MethodBase method,
 							  Type[] opt_param_types);
 
 		internal int GetToken (string str)
@@ -674,15 +669,19 @@ namespace System.Reflection.Emit {
 		}
 
 		internal int GetToken (MemberInfo member) {
-			return getToken (this, member);
+			return getToken (this, member, true);
 		}
 
-		internal int GetToken (MethodInfo method, Type[] opt_param_types) {
+		internal int GetToken (MemberInfo member, bool create_open_instance) {
+			return getToken (this, member, create_open_instance);
+		}
+
+		internal int GetToken (MethodBase method, Type[] opt_param_types) {
 			return getMethodToken (this, method, opt_param_types);
 		}
 
 		internal int GetToken (SignatureHelper helper) {
-			return getToken (this, helper);
+			return getToken (this, helper, true);
 		}
 
 		/*
@@ -821,7 +820,6 @@ namespace System.Reflection.Emit {
 			throw new NotImplementedException ();
 		}
 
-#if NET_4_0 || MOONLIGHT
 		public override	Assembly Assembly {
 			get { return assemblyb; }
 		}
@@ -916,7 +914,51 @@ namespace System.Reflection.Emit {
 				return Type.GetTypeFromHandle (new RuntimeTypeHandle (handle));
 		}
 
-#endif
+		public override bool Equals (object obj)
+		{
+			return base.Equals (obj);
+		}
+
+		public override int GetHashCode ()
+		{
+			return base.GetHashCode ();
+		}
+
+		public override bool IsDefined (Type attributeType, bool inherit)
+		{
+			return base.IsDefined (attributeType, inherit);
+		}
+
+		public override object[] GetCustomAttributes (bool inherit)
+		{
+			return base.GetCustomAttributes (inherit);
+		}
+
+		public override object[] GetCustomAttributes (Type attributeType, bool inherit)
+		{
+			return base.GetCustomAttributes (attributeType, inherit);
+		}
+
+		public override FieldInfo GetField (string name, BindingFlags bindingAttr)
+		{
+			return base.GetField (name, bindingAttr);
+		}
+
+		public override FieldInfo[] GetFields (BindingFlags bindingFlags)
+		{
+			return base.GetFields (bindingFlags);
+		}
+
+		public override MethodInfo[] GetMethods (BindingFlags bindingFlags)
+		{
+			return base.GetMethods (bindingFlags);
+		}
+
+		public override int MetadataToken {
+			get {
+				return base.MetadataToken;
+			}
+		}
 	}
 
 	internal class ModuleBuilderTokenGenerator : TokenGenerator {
@@ -931,11 +973,11 @@ namespace System.Reflection.Emit {
 			return mb.GetToken (str);
 		}
 
-		public int GetToken (MemberInfo member) {
-			return mb.GetToken (member);
+		public int GetToken (MemberInfo member, bool create_open_instance) {
+			return mb.GetToken (member, create_open_instance);
 		}
 
-		public int GetToken (MethodInfo method, Type[] opt_param_types) {
+		public int GetToken (MethodBase method, Type[] opt_param_types) {
 			return mb.GetToken (method, opt_param_types);
 		}
 
@@ -945,3 +987,4 @@ namespace System.Reflection.Emit {
 	}
 }
 
+#endif
