@@ -31,6 +31,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+#if !FULL_AOT_RUNTIME
 using System;
 using System.Text;
 using System.Reflection;
@@ -43,12 +44,14 @@ using System.Security;
 using System.Security.Permissions;
 using System.Diagnostics.SymbolStore;
 
+
 namespace System.Reflection.Emit
 {
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_TypeBuilder))]
 	[ClassInterface (ClassInterfaceType.None)]
-	public sealed class TypeBuilder : Type, _TypeBuilder
+	[StructLayout (LayoutKind.Sequential)]
+	public sealed class TypeBuilder : TypeInfo, _TypeBuilder
 	{
 #pragma warning disable 169		
 		#region Sync with reflection.h
@@ -74,7 +77,7 @@ namespace System.Reflection.Emit
 		private IntPtr generic_container;
 		private GenericTypeParameterBuilder[] generic_params;
 		private RefEmitPermissionSet[] permissions;
-		private Type created;
+		private TypeInfo created;
 		#endregion
 #pragma warning restore 169		
 		
@@ -144,7 +147,7 @@ namespace System.Reflection.Emit
 			}
 			pmodule = mb;
 
-			if (((attr & TypeAttributes.Interface) == 0) && (parent == null) && !IsCompilerContext)
+			if (((attr & TypeAttributes.Interface) == 0) && (parent == null))
 				this.parent = typeof (object);
 
 			// skip .<Module> ?
@@ -195,7 +198,7 @@ namespace System.Reflection.Emit
 				if (is_created)
 					return created.UnderlyingSystemType;
 
-				if (!IsCompilerContext && IsEnum) {
+				if (IsEnum) {
 					if (underlying_type != null)
 						return underlying_type;
 					throw new InvalidOperationException (
@@ -361,7 +364,7 @@ namespace System.Reflection.Emit
 
 		public override bool IsDefined (Type attributeType, bool inherit)
 		{
-			if (!is_created && !IsCompilerContext)
+			if (!is_created)
 				throw new NotSupportedException ();
 			/*
 			 * MS throws NotSupported here, but we can't because some corlib
@@ -664,7 +667,7 @@ namespace System.Reflection.Emit
 				create_internal_class (this);
 			}
 
-			if (IsEnum && !IsCompilerContext) {
+			if (IsEnum) {
 				if (underlying_type == null && (attributes & FieldAttributes.Static) == 0)
 					underlying_type = type;
 			}
@@ -677,12 +680,10 @@ namespace System.Reflection.Emit
 			return DefineProperty (name, attributes, 0, returnType, null, null, parameterTypes, null, null);
 		}
 		
-#if NET_4_0
 		public PropertyBuilder DefineProperty (string name, PropertyAttributes attributes, CallingConventions callingConvention, Type returnType, Type[] parameterTypes)
 		{
 			return DefineProperty (name, attributes, callingConvention, returnType , null, null, parameterTypes, null, null);
 		}	
-#endif
 
 		public PropertyBuilder DefineProperty (string name, PropertyAttributes attributes, Type returnType, Type[] returnTypeRequiredCustomModifiers, Type[] returnTypeOptionalCustomModifiers, Type[] parameterTypes, Type[][] parameterTypeRequiredCustomModifiers, Type[][] parameterTypeOptionalCustomModifiers)
 		{
@@ -719,7 +720,7 @@ namespace System.Reflection.Emit
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern Type create_runtime_class (TypeBuilder tb);
+		private extern TypeInfo create_runtime_class (TypeBuilder tb);
 
 		private bool is_nested_in (Type t)
 		{
@@ -745,8 +746,14 @@ namespace System.Reflection.Emit
 
 			return false;
 	    }
+
+		public Type CreateType ()
+		{
+			return CreateTypeInfo ();
+		}
 		
-		public Type CreateType()
+		public
+		TypeInfo CreateTypeInfo ()
 		{
 			/* handle nesting_type */
 			if (createTypeCalled)
@@ -794,6 +801,12 @@ namespace System.Reflection.Emit
 
 			if (parent == pmodule.assemblyb.corlib_enum_type && methods != null)
 				throw new TypeLoadException ("Could not load type '" + FullName + "' from assembly '" + Assembly + "' because it is an enum with methods.");
+			if (interfaces != null) {
+				foreach (var iface in interfaces) {
+					if (iface.IsNestedPrivate && iface.Assembly != Assembly)
+						throw new TypeLoadException ("Could not load type '" + FullName + "' from assembly '" + Assembly + "' because it is implements the inaccessible interface '" + iface.FullName + "'.");
+				}
+			}
 
 			if (methods != null) {
 				bool is_concrete = !IsAbstract;
@@ -847,10 +860,7 @@ namespace System.Reflection.Emit
 			if (is_created)
 				return created.GetConstructors (bindingAttr);
 
-			if (!IsCompilerContext)
-				throw new NotSupportedException ();
-
-			return GetConstructorsInternal (bindingAttr);
+			throw new NotSupportedException ();
 		}
 
 		internal ConstructorInfo[] GetConstructorsInternal (BindingFlags bindingAttr)
@@ -911,9 +921,7 @@ namespace System.Reflection.Emit
 		{
 			if (is_created)
 				return created.GetEvents (bindingAttr);
-			if (!IsCompilerContext)
-				throw new NotSupportedException ();
-			return new EventInfo [0]; /*FIXME shouldn't we return the events here?*/
+			throw new NotSupportedException ();
 		}
 
 		// This is only used from MonoGenericInst.initialize().
@@ -1252,7 +1260,7 @@ namespace System.Reflection.Emit
 
 		public override Type[] GetNestedTypes (BindingFlags bindingAttr)
 		{
-			if (!is_created && !IsCompilerContext)
+			if (!is_created)
 				throw new NotSupportedException ();
 
 			bool match;
@@ -1462,7 +1470,7 @@ namespace System.Reflection.Emit
 					throw new Exception ("Error in customattr");
 				}
 				
-				var ctor_type = customBuilder.Ctor is ConstructorBuilder ? ((ConstructorBuilder)customBuilder.Ctor).parameters[0] : customBuilder.Ctor.GetParameters()[0].ParameterType;
+				var ctor_type = customBuilder.Ctor is ConstructorBuilder ? ((ConstructorBuilder)customBuilder.Ctor).parameters[0] : customBuilder.Ctor.GetParametersInternal()[0].ParameterType;
 				int pos = 6;
 				if (ctor_type.FullName == "System.Int16")
 					pos = 4;
@@ -1642,12 +1650,6 @@ namespace System.Reflection.Emit
 			return created.GetInterfaceMap (interfaceType);
 		}
 
-		internal override bool IsCompilerContext {
-			get {
-				return pmodule.assemblyb.IsCompilerContext;
-			}
-		}
-
 		internal override Type InternalResolve ()
 		{
 			check_created ();
@@ -1930,5 +1932,21 @@ namespace System.Reflection.Emit
 		{
 			throw new NotImplementedException ();
 		}
+
+		internal override bool IsUserType {
+			get {
+				return false;
+			}
+		}
+
+		public override bool IsConstructedGenericType {
+			get { return false; }
+		}
+
+		public override bool IsAssignableFrom (TypeInfo typeInfo)
+		{
+			return base.IsAssignableFrom (typeInfo);
+		}
 	}
 }
+#endif
