@@ -6,6 +6,7 @@
 //
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
 // Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
+// Copyright 2011 Xamarin Inc (http://www.xamarin.com).
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -32,7 +33,6 @@ using System.Security.Policy;
 using System.Security.Permissions;
 using System.Runtime.Serialization;
 using System.Reflection;
-using System.Reflection.Emit;
 using System.IO;
 using System.Globalization;
 using System.Runtime.CompilerServices;
@@ -48,39 +48,61 @@ namespace System.Reflection {
 	[ComDefaultInterfaceAttribute (typeof (_Assembly))]
 	[Serializable]
 	[ClassInterface(ClassInterfaceType.None)]
+	[StructLayout (LayoutKind.Sequential)]
 #if MOBILE
-	public partial class Assembly : ICustomAttributeProvider, _Assembly {
-#elif MOONLIGHT
-	public abstract class Assembly : ICustomAttributeProvider, _Assembly {
-#elif NET_4_0
-	public abstract class Assembly : ICustomAttributeProvider, _Assembly, IEvidenceFactory, ISerializable {
+	public partial class Assembly : ICustomAttributeProvider {
 #else
-	public partial class Assembly : ICustomAttributeProvider, _Assembly, IEvidenceFactory, ISerializable {
+	public abstract class Assembly : ICustomAttributeProvider, _Assembly, IEvidenceFactory, ISerializable {
 #endif
 		internal class ResolveEventHolder {
 			public event ModuleResolveEventHandler ModuleResolve;
 		}
 
+		internal class UnmanagedMemoryStreamForModule : UnmanagedMemoryStream
+		{
+			Module module;
+
+			public unsafe UnmanagedMemoryStreamForModule (byte* pointer, long length, Module module)
+				: base (pointer, length)
+			{
+				this.module = module;
+			}
+
+			protected override void Dispose (bool disposing)
+			{
+				if (!closed) {
+					/* 
+					 * The returned pointer points inside metadata, so
+					 * we have to increase the refcount of the module, and decrease
+					 * it when the stream is finalized.
+					 */
+					module = null;
+				}
+
+				base.Dispose (disposing);
+			}
+		}
+
 		// Note: changes to fields must be reflected in _MonoReflectionAssembly struct (object-internals.h)
 #pragma warning disable 649
 		private IntPtr _mono_assembly;
 #pragma warning restore 649
 
 		private ResolveEventHolder resolve_event_holder;
+#if !MOBILE
 		private Evidence _evidence;
 		internal PermissionSet _minimum;	// for SecurityAction.RequestMinimum
 		internal PermissionSet _optional;	// for SecurityAction.RequestOptional
 		internal PermissionSet _refuse;		// for SecurityAction.RequestRefuse
 		private PermissionSet _granted;		// for the resolved assembly granted permissions
 		private PermissionSet _denied;		// for the resolved assembly denied permissions
+#else
+		object _evidence, _minimum, _optional, _refuse, _granted, _denied;
+#endif
 		private bool fromByteArray;
 		private string assemblyName;
 
-#if NET_4_0 || MOONLIGHT || MOBILE
 		protected
-#else
-		internal
-#endif
 		Assembly () 
 		{
 			resolve_event_holder = new ResolveEventHolder ();
@@ -151,7 +173,7 @@ namespace System.Reflection {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
 			get;
 		}
-#if !MOONLIGHT
+
 		public virtual Evidence Evidence {
 			[SecurityPermission (SecurityAction.Demand, ControlEvidence = true)]
 			get { return UnprotectedGetEvidence (); }
@@ -160,6 +182,9 @@ namespace System.Reflection {
 		// note: the security runtime requires evidences but may be unable to do so...
 		internal Evidence UnprotectedGetEvidence ()
 		{
+#if MOBILE
+			return null;
+#else
 			// if the host (runtime) hasn't provided it's own evidence...
 			if (_evidence == null) {
 				// ... we will provide our own
@@ -168,12 +193,12 @@ namespace System.Reflection {
 				}
 			}
 			return _evidence;
+#endif
 		}
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		internal extern bool get_global_assembly_cache ();
 
-#endif
 		internal bool FromByteArray {
 			set { fromByteArray = value; }
 		}
@@ -201,13 +226,9 @@ namespace System.Reflection {
 			}
 		}
 
-		[SecurityPermission (SecurityAction.LinkDemand, SerializationFormatter = true)]
 		public virtual void GetObjectData (SerializationInfo info, StreamingContext context)
 		{
-			if (info == null)
-				throw new ArgumentNullException ("info");
-
-			UnitySerializationHolder.GetAssemblyData (this, info, context);
+			throw new NotImplementedException ();
 		}
 
 		public virtual bool IsDefined (Type attributeType, bool inherit)
@@ -237,7 +258,7 @@ namespace System.Reflection {
 		{
 			string[] names = (string[]) GetFilesInternal (null, getResourceModules);
 			if (names == null)
-				return new FileStream [0];
+				return EmptyArray<FileStream>.Value;
 
 			string location = Location;
 
@@ -281,8 +302,13 @@ namespace System.Reflection {
 					"name");
 
 			ManifestResourceInfo info = GetManifestResourceInfo (name);
-			if (info == null)
-				return null;
+			if (info == null) {
+				Assembly a = AppDomain.CurrentDomain.DoResourceResolve (name, this);
+				if (a != null && a != this)
+					return a.GetManifestResourceStream (name);
+				else
+					return null;
+			}
 
 			if (info.ReferencedAssembly != null)
 				return info.ReferencedAssembly.GetManifestResourceStream (name);
@@ -292,12 +318,6 @@ namespace System.Reflection {
 
 				string location = Path.GetDirectoryName (Location);
 				string filename = Path.Combine (location, info.FileName);
-#if MOONLIGHT
-				// we don't control the content of 'info.FileName' so we want to make sure we keep to ourselves
-				filename = Path.GetFullPath (filename);
-				if (!filename.StartsWith (location))
-					throw new SecurityException ("non-rooted access to manifest resource");
-#endif
 				return new FileStream (filename, FileMode.Open, FileAccess.Read);
 			}
 
@@ -309,14 +329,8 @@ namespace System.Reflection {
 			else {
 				UnmanagedMemoryStream stream;
 				unsafe {
-					stream = new UnmanagedMemoryStream ((byte*) data, size);
+					stream = new UnmanagedMemoryStreamForModule ((byte*) data, size, module);
 				}
-				/* 
-				 * The returned pointer points inside metadata, so
-				 * we have to increase the refcount of the module, and decrease
-				 * it when the stream is finalized.
-				 */
-				stream.Closed += new EventHandler (new ResourceCloseHandler (module).OnClose);
 				return stream;
 			}
 		}
@@ -372,10 +386,12 @@ namespace System.Reflection {
 		[MonoTODO ("copiedName == true is not supported")]
 		public virtual AssemblyName GetName (Boolean copiedName)
 		{
+#if !MOBILE
 			// CodeBase, which is restricted, will be copied into the AssemblyName object so...
 			if (SecurityManager.SecurityEnabled) {
 				GetCodeBase (true); // this will ensure the Demand is made
 			}
+#endif
 			return UnprotectedGetName ();
 		}
 
@@ -449,21 +465,20 @@ namespace System.Reflection {
 			// Try the assembly directory
 			string location = Path.GetDirectoryName (Location);
 			string fullName = Path.Combine (location, Path.Combine (culture.Name, aname.Name + ".dll"));
-#if MOONLIGHT
-			// it's unlikely that culture.Name or aname.Name could contain stuff like ".." but...
-			fullName = Path.GetFullPath (fullName);
-			if (!fullName.StartsWith (location)) {
-				if (throwOnError)
-					throw new SecurityException ("non-rooted access to satellite assembly");
-				return null;
-			}
-#endif
 			if (!throwOnError && !File.Exists (fullName))
 				return null;
 
 			return LoadFrom (fullName);
 		}
-		
+
+#if !MOBILE
+		Type _Assembly.GetType ()
+		{
+			// Required or object::GetType becomes virtual final
+			return base.GetType ();
+		}		
+#endif
+
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		private extern static Assembly LoadFrom (String assemblyFile, bool refonly);
 
@@ -472,9 +487,7 @@ namespace System.Reflection {
 			return LoadFrom (assemblyFile, false);
 		}
 
-#if NET_4_0
 		[Obsolete]
-#endif
 		public static Assembly LoadFrom (String assemblyFile, Evidence securityEvidence)
 		{
 			Assembly a = LoadFrom (assemblyFile, false);
@@ -487,9 +500,7 @@ namespace System.Reflection {
 			return a;
 		}
 
-#if NET_4_0
 		[Obsolete]
-#endif
 		[MonoTODO("This overload is not currently implemented")]
 		// FIXME: What are we missing?
 		public static Assembly LoadFrom (String assemblyFile, Evidence securityEvidence, byte[] hashValue, AssemblyHashAlgorithm hashAlgorithm)
@@ -497,24 +508,18 @@ namespace System.Reflection {
 			throw new NotImplementedException ();
 		}
 
-#if NET_4_0
 		[MonoTODO]
 		public static Assembly LoadFrom (String assemblyFile, byte [] hashValue, AssemblyHashAlgorithm hashAlgorithm)
 		{
 			throw new NotImplementedException ();
 		}
-#endif
 
-#if NET_4_0
 		public static Assembly UnsafeLoadFrom (String assemblyFile)
 		{
 			return LoadFrom (assemblyFile);
 		}
-#endif
 
-#if NET_4_0
 		[Obsolete]
-#endif
 		public static Assembly LoadFile (String path, Evidence securityEvidence)
 		{
 			if (path == null)
@@ -535,9 +540,7 @@ namespace System.Reflection {
 			return AppDomain.CurrentDomain.Load (assemblyString);
 		}
 
-#if NET_4_0
 		[Obsolete]
-#endif		
 		public static Assembly Load (String assemblyString, Evidence assemblySecurity)
 		{
 			return AppDomain.CurrentDomain.Load (assemblyString, assemblySecurity);
@@ -548,9 +551,7 @@ namespace System.Reflection {
 			return AppDomain.CurrentDomain.Load (assemblyRef);
 		}
 
-#if NET_4_0
 		[Obsolete]
-#endif
 		public static Assembly Load (AssemblyName assemblyRef, Evidence assemblySecurity)
 		{
 			return AppDomain.CurrentDomain.Load (assemblyRef, assemblySecurity);
@@ -566,22 +567,18 @@ namespace System.Reflection {
 			return AppDomain.CurrentDomain.Load (rawAssembly, rawSymbolStore);
 		}
 
-#if NET_4_0
 		[Obsolete]
-#endif
 		public static Assembly Load (Byte[] rawAssembly, Byte[] rawSymbolStore,
 					     Evidence securityEvidence)
 		{
 			return AppDomain.CurrentDomain.Load (rawAssembly, rawSymbolStore, securityEvidence);
 		}
 
-#if NET_4_0
 		[MonoLimitation ("Argument securityContextSource is ignored")]
 		public static Assembly Load (byte [] rawAssembly, byte [] rawSymbolStore, SecurityContextSource securityContextSource)
 		{
 			return AppDomain.CurrentDomain.Load (rawAssembly, rawSymbolStore);
 		}
-#endif
 
 		public static Assembly ReflectionOnlyLoad (byte[] rawAssembly)
 		{
@@ -615,9 +612,7 @@ namespace System.Reflection {
 
 		[MonoTODO ("Not implemented")]
 		public
-#if NET_4_0 || MOONLIGHT || MOBILE
 		virtual
-#endif
 		Module LoadModule (string moduleName, byte [] rawModule, byte [] rawSymbolStore)
 		{
 			throw new NotImplementedException ();
@@ -670,9 +665,7 @@ namespace System.Reflection {
 		}
 
 		public
-#if NET_4_0 || MOONLIGHT || MOBILE
 		virtual
-#endif
 		Object CreateInstance (String typeName, Boolean ignoreCase,
 					      BindingFlags bindingAttr, Binder binder,
 					      Object[] args, CultureInfo culture,
@@ -701,10 +694,6 @@ namespace System.Reflection {
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		internal virtual extern Module[] GetModulesInternal ();
-
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal extern string[] GetNamespaces ();
 		
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		public extern virtual String[] GetManifestResourceNames ();
@@ -735,34 +724,10 @@ namespace System.Reflection {
 				return null;
 		}
 
-		private class ResourceCloseHandler {
-#pragma warning disable 169, 414
-			Module module;
-#pragma warning restore 169, 414			
-
-			public ResourceCloseHandler (Module module) {
-				this.module = module;
-			}
-
-			public void OnClose (object sender, EventArgs e) {
-				// The module dtor will take care of things
-				module = null;
-			}
-		}
-
-		//
-		// The following functions are only for the Mono Debugger.
-		//
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern int MonoDebugger_GetMethodToken (MethodBase method);
-
 		[MonoTODO ("Currently it always returns zero")]
 		[ComVisible (false)]
 		public
-#if NET_4_0 || MOONLIGHT || MOBILE
 		virtual
-#endif
 		long HostContext {
 			get { return 0; }
 		}
@@ -797,24 +762,8 @@ namespace System.Reflection {
 			Assembly other = (Assembly) o;
 			return other._mono_assembly == _mono_assembly;
 		}
-		
-#if NET_4_0
-#if MOONLIGHT || MOBILE
-		public virtual IList<CustomAttributeData> GetCustomAttributesData () {
-			return CustomAttributeData.GetCustomAttributes (this);
-		}
-#endif
-		public PermissionSet PermissionSet {
-			get { return this.GrantedPermissionSet; }
-		}
 
-		[MonoTODO]
-		public bool IsFullyTrusted {
-			get { return true; }
-		}
-#endif
-
-#if !MOONLIGHT
+#if !NET_2_1
 		// Code Access Security
 
 		internal void Resolve () 
@@ -895,13 +844,31 @@ namespace System.Reflection {
 				}
 			}
 		}
+		
+		public virtual PermissionSet PermissionSet {
+			get { return this.GrantedPermissionSet; }
+		}
+		
+		public virtual SecurityRuleSet SecurityRuleSet {
+			get { throw CreateNIE (); }
+		}
+
 #endif
 
-#if NET_4_0 || MOONLIGHT || MOBILE
 		static Exception CreateNIE ()
 		{
 			return new NotImplementedException ("Derived classes must implement it");
 		}
+		
+		public virtual IList<CustomAttributeData> GetCustomAttributesData ()
+		{
+			return CustomAttributeData.GetCustomAttributes (this);
+		}
+
+		[MonoTODO]
+		public bool IsFullyTrusted {
+			get { return true; }
+		}
 
 		public virtual Type GetType (string name, bool throwOnError, bool ignoreCase)
 		{
@@ -968,6 +935,25 @@ namespace System.Reflection {
 				return true;
 			return !left.Equals (right);
 		}
-#endif
+
+		public virtual IEnumerable<TypeInfo> DefinedTypes {
+			get {
+				foreach (var type in GetTypes ()) {
+					yield return type.GetTypeInfo ();
+				}
+			}
+		}
+
+		public virtual IEnumerable<Type> ExportedTypes {
+			get { return GetExportedTypes (); }
+		}
+
+		public virtual IEnumerable<Module> Modules {
+			get { return GetModules (); }
+		}
+
+		public virtual IEnumerable<CustomAttributeData> CustomAttributes {
+			get { return GetCustomAttributesData (); }
+		}
 	}
 }
