@@ -6,6 +6,7 @@
 //
 // (C) Ximian, Inc.  http://www.ximian.com
 // Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
+// Copyright 2011 Xamarin Inc (http://www.xamarin.com).
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,7 +28,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 using System.Diagnostics;
-using System.Reflection.Emit;
 using System.Globalization;
 using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
@@ -38,8 +38,11 @@ namespace System.Reflection {
 	[ComDefaultInterfaceAttribute (typeof (_FieldInfo))]
 	[Serializable]
 	[ClassInterface(ClassInterfaceType.None)]
+#if MOBILE
+	public abstract class FieldInfo : MemberInfo {
+#else
 	public abstract class FieldInfo : MemberInfo, _FieldInfo {
-
+#endif
 		public abstract FieldAttributes Attributes {get;}
 		public abstract RuntimeFieldHandle FieldHandle {get;}
 
@@ -190,13 +193,7 @@ namespace System.Reflection {
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern UnmanagedMarshal GetUnmanagedMarshal ();
-
-		internal virtual UnmanagedMarshal UMarshal {
-			get {
-				return GetUnmanagedMarshal ();
-			}
-		}
+		private extern MarshalAsAttribute get_marshal_info ();
 
 		internal object[] GetPseudoCustomAttributes ()
 		{
@@ -208,7 +205,7 @@ namespace System.Reflection {
 			if (DeclaringType.IsExplicitLayout)
 				count ++;
 
-			UnmanagedMarshal marshalAs = UMarshal;
+			MarshalAsAttribute marshalAs = get_marshal_info ();
 			if (marshalAs != null)
 				count ++;
 
@@ -222,7 +219,7 @@ namespace System.Reflection {
 			if (DeclaringType.IsExplicitLayout)
 				attrs [count ++] = new FieldOffsetAttribute (GetFieldOffset ());
 			if (marshalAs != null)
-				attrs [count ++] = marshalAs.ToMarshalAsAttribute ();
+				attrs [count ++] = marshalAs;
 
 			return attrs;
 		}
@@ -250,7 +247,6 @@ namespace System.Reflection {
 		}
 
 
-#if NET_4_0
 		public override bool Equals (object obj)
 		{
 			return obj == (object) this;
@@ -278,12 +274,37 @@ namespace System.Reflection {
 				return true;
 			return !left.Equals (right);
 		}
-#endif
+		
+		public virtual bool IsSecurityCritical {
+			get {
+				throw new NotImplementedException ();
+			}
+		}
+		
+		public virtual bool IsSecuritySafeCritical {
+			get {
+				throw new NotImplementedException ();
+			}
+		}
+
+		public virtual bool IsSecurityTransparent {
+			get {
+				throw new NotImplementedException ();
+			}
+		}
+
+#if !MOBILE
 		void _FieldInfo.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
 		{
 			throw new NotImplementedException ();
 		}
 
+		Type _FieldInfo.GetType ()
+		{
+			// Required or object::GetType becomes virtual final
+			return base.GetType ();
+		}
+
 		void _FieldInfo.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
 		{
 			throw new NotImplementedException ();
@@ -298,5 +319,6 @@ namespace System.Reflection {
 		{
 			throw new NotImplementedException ();
 		}
+#endif
 	}
 }
