@@ -37,8 +37,11 @@ namespace System.Reflection {
 	[ComDefaultInterfaceAttribute (typeof (_MethodInfo))]
 	[Serializable]
 	[ClassInterface(ClassInterfaceType.None)]
+#if MOBILE
+	public abstract class MethodInfo: MethodBase {
+#else
 	public abstract class MethodInfo: MethodBase, _MethodInfo {
-
+#endif
 		public abstract MethodInfo GetBaseDefinition();
 
 		internal virtual MethodInfo GetBaseMethod ()
@@ -58,22 +61,18 @@ namespace System.Reflection {
 
 		public abstract ICustomAttributeProvider ReturnTypeCustomAttributes { get; }
 
-		// FIXME: when this method is uncommented, corlib fails
-		// to build
-/*
-		[DebuggerStepThrough]
-		[DebuggerHidden]
-		public new object Invoke (object obj, object[] parameters)
-		{
-			return base.Invoke (obj, parameters);
-		}
-*/
-
+#if !MOBILE
 		void _MethodInfo.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
 		{
 			throw new NotImplementedException ();
 		}
 
+		Type _MethodInfo.GetType ()
+		{
+			// Required or object::GetType becomes virtual final
+			return base.GetType ();
+		}
+
 		void _MethodInfo.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
 		{
 			throw new NotImplementedException ();
@@ -88,6 +87,7 @@ namespace System.Reflection {
 		{
 			throw new NotImplementedException ();
 		}
+#endif
 
 		[ComVisible (true)]
 		public virtual MethodInfo GetGenericMethodDefinition ()
@@ -107,25 +107,6 @@ namespace System.Reflection {
 			return Type.EmptyTypes;
 		}
 
-#if !NET_4_0 && !MOONLIGHT
-		public override bool IsGenericMethod {
-			get {
-				return false;
-			}
-		}
-
-		public override bool IsGenericMethodDefinition {
-			get {
-				return false;
-			}
-		}
-
-		public override bool ContainsGenericParameters {
-			get {
-				return false;
-			}
-		}
-#endif
 
 		public virtual ParameterInfo ReturnParameter {
 			get {
@@ -133,10 +114,9 @@ namespace System.Reflection {
 			}
 		}
 
-#if NET_4_0
 		public override bool Equals (object obj)
 		{
-			return obj == this;
+			return obj == (object) this;
 		}
 
 		public override int GetHashCode ()
@@ -161,7 +141,15 @@ namespace System.Reflection {
 				return true;
 			return !left.Equals (right);
 		}
-#endif
 
+		public virtual Delegate CreateDelegate (Type delegateType)
+		{
+			return Delegate.CreateDelegate (delegateType, this);
+		}
+
+		public virtual Delegate CreateDelegate (Type delegateType, object target)
+		{
+			return Delegate.CreateDelegate (delegateType, target, this);
+		}
 	}
 }
