@@ -40,9 +40,7 @@ using System.Runtime.InteropServices;
 using System.Runtime.Serialization;
 using System.Runtime.Serialization.Formatters.Binary;
 using System.Security.Permissions;
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 
 namespace System.Resources
 {
@@ -70,9 +68,7 @@ namespace System.Resources
 		FistCustom	= 64
 	}
 
-#if NET_2_0
 	[System.Runtime.InteropServices.ComVisible (true)]
-#endif
 	public sealed class ResourceReader : IResourceReader, IEnumerable, IDisposable
 	{
 		struct ResourceInfo
@@ -100,6 +96,26 @@ namespace System.Resources
 				ResourceValue = value;
 			}
 		}
+
+		unsafe class HGlobalUnmanagedMemoryStream : UnmanagedMemoryStream
+		{
+			IntPtr ptr;
+
+			public HGlobalUnmanagedMemoryStream (byte* pointer, long length, IntPtr ptr)
+				: base (pointer, length, length, FileAccess.ReadWrite)
+			{
+				this.ptr = ptr;
+			}
+
+			protected override void Dispose (bool disposing)
+			{
+				if (!closed) {
+					Marshal.FreeHGlobal (ptr);
+				}
+
+				base.Dispose (disposing);
+			}
+		}
 		
 		BinaryReader reader;
 		object readerLock = new object ();
@@ -173,11 +189,7 @@ namespace System.Resources
 				/* Now read the ResourceReader header */
 				resource_ver = reader.ReadInt32();
 
-				if(resource_ver != 1
-#if NET_2_0
-					&& resource_ver != 2
-#endif
-					) {
+				if(resource_ver != 1 && resource_ver != 2) {
 					throw new NotSupportedException("This .resources file requires unsupported set class version: " + resource_ver.ToString());
 				}
 
@@ -285,7 +297,6 @@ namespace System.Resources
 			return ret;
 		}
 
-#if NET_2_0
 		object ReadValueVer2 (int type_index)
 		{
 			switch ((PredefinedResourceType)type_index)
@@ -354,7 +365,6 @@ namespace System.Resources
 			type_index -= (int)PredefinedResourceType.FistCustom;
 			return ReadNonPredefinedValue (Type.GetType (typeNames[type_index], true));
 		}
-#endif		
 
 		object ReadValueVer1 (Type type)
 		{
@@ -428,11 +438,9 @@ namespace System.Resources
 					}
 
 					reader.BaseStream.Seek (ri.ValuePosition, SeekOrigin.Begin);
-#if NET_2_0
 					if (resource_ver == 2)
 						value = ReadValueVer2 (ri.TypeIndex);
 					else
-#endif
 						value = ReadValueVer1 (Type.GetType (typeNames [ri.TypeIndex], true));
 
 					store [i] = new ResourceCacheItem (ri.ResourceName, value);
@@ -440,7 +448,6 @@ namespace System.Resources
 			}
 		}
 		
-#if NET_2_0
 		internal UnmanagedMemoryStream ResourceValueAsStream (string name, int index)
 		{
 			ResourceInfo ri = infos [index];
@@ -461,12 +468,7 @@ namespace System.Resources
 					} else {
 						IntPtr ptr = Marshal.AllocHGlobal ((int) slen);
 						byte* addr = (byte*) ptr.ToPointer ();
-						UnmanagedMemoryStream ms = new UnmanagedMemoryStream (addr, slen, slen, FileAccess.ReadWrite);
-						// The memory resource must be freed
-						// when the stream is disposed.
-						ms.Closed += delegate (object o, EventArgs e) {
-							Marshal.FreeHGlobal (ptr);
-						};
+						UnmanagedMemoryStream ms = new HGlobalUnmanagedMemoryStream (addr, slen, ptr);
 
 						byte [] bytes = new byte [slen < 1024 ? slen : 1024];
 						while (slen > 0 ) {
@@ -484,12 +486,16 @@ namespace System.Resources
 				}
 			}
 		}
-#endif
 
 		public void Close ()
 		{
 			Dispose(true);
 		}
+
+		public void Dispose ()
+		{
+			Dispose(true);
+		}
 		
 		public IDictionaryEnumerator GetEnumerator () {
 			if (reader == null){
@@ -504,7 +510,7 @@ namespace System.Resources
 		{
 			return ((IResourceReader) this).GetEnumerator();
 		}
-#if NET_2_0
+
 		public void GetResourceData (string resourceName, out string resourceType, out byte [] resourceData)
 		{
 			if (resourceName == null)
@@ -553,11 +559,6 @@ namespace System.Resources
 				reader.BaseStream.Read (data, 0, datalen);
 			}
 		}
-#endif
-		void IDisposable.Dispose ()
-		{
-			Dispose(true);
-		}
 
 		private void Dispose (bool disposing)
 		{
@@ -625,7 +626,6 @@ namespace System.Resources
 				}
 			}
 			
-#if NET_2_0
 			public UnmanagedMemoryStream ValueAsStream
 			{
 				get {
@@ -636,7 +636,6 @@ namespace System.Resources
 					return(reader.ResourceValueAsStream((string) Key, index));
 				}
 			}
-#endif
 			
 			public object Current
 			{
