@@ -33,9 +33,7 @@
 namespace System.Runtime.InteropServices
 {
 	[Flags]
-#if NET_2_0
 	[ComVisible(true)]
-#endif
 	public enum AssemblyRegistrationFlags {
 		None = 0,
 		SetCodeBase,
