@@ -2,10 +2,11 @@
 // HMACRIPEMD160.cs: HMAC implementation using RIPEMD160
 //
 // Author:
-//	Sebastien Pouliot  <sebastien@ximian.com>
+//	Sebastien Pouliot  <sebastien@xamarin.com>
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
 // Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
+// Copyright 2013 Xamarin Inc. (http://www.xamarin.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,8 +28,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0 && !NET_2_1 || MONOTOUCH
-
 using System.Runtime.InteropServices;
 
 using Mono.Security.Cryptography;
@@ -39,17 +38,19 @@ namespace System.Security.Cryptography {
 	public class HMACRIPEMD160 : HMAC {
 
 		public HMACRIPEMD160 () 
-			: this (KeyBuilder.Key (8))
+			: this (KeyBuilder.Key (64))
 		{
 		}
 
 		public HMACRIPEMD160 (byte[] key) : base () 
 		{
+#if FULL_AOT_RUNTIME
+			SetHash ("RIPEMD160", new RIPEMD160Managed ());
+#else
 			HashName = "RIPEMD160";
+#endif
 			HashSizeValue = 160;
 			Key = key;
 		}
 	}
 }
-
-#endif
