@@ -187,6 +187,13 @@ namespace System.Security.Cryptography {
 			_pos = 0;
 			_f = 0;
 		}
+		protected override void Dispose (bool disposing)
+		{
+			Array.Clear (_buffer, 0, _buffer.Length);
+			Array.Clear (_salt, 0, _salt.Length);
+			_hmac.Clear ();
+			base.Dispose (disposing);
+		}
 	} 
 }
 
