@@ -29,7 +29,10 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+#if !NET_2_1
+
 using System.Collections;
+using System.Diagnostics;
 using System.Globalization;
 using System.IO;
 using System.Reflection;
@@ -50,16 +53,8 @@ namespace System.Security {
 		public RuntimeDeclSecurityEntry choice;
 	}
 
-#if NET_2_0
 	[ComVisible (true)]
 	public static class SecurityManager {
-#else
-	public sealed class SecurityManager {
-
-		private SecurityManager ()
-		{
-		}
-#endif
 		private static object _lockObject;
 		private static ArrayList _hierarchy;
 		private static IPermission _unmanagedCode;
@@ -75,6 +70,7 @@ namespace System.Security {
 
 		// properties
 
+		[Obsolete]
 		extern public static bool CheckExecutionRights {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
 			get;
@@ -84,9 +80,7 @@ namespace System.Security {
 			set;
 		}
 
-#if NET_2_0
 		[Obsolete ("The security manager cannot be turned off on MS runtime")]
-#endif
 		extern public static bool SecurityEnabled {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
 			get;
@@ -96,9 +90,19 @@ namespace System.Security {
 			set;
 		}
 
+		internal static bool CheckElevatedPermissions ()
+		{
+			return true; // always true outside Moonlight
+		}
+
+		[Conditional ("ENABLE_SANDBOX")] //??
+		internal static void EnsureElevatedPermissions ()
+		{
+			// do nothing outside of Moonlight
+		}
+
 		// methods
 
-#if NET_1_1
 		// NOTE: This method doesn't show in the class library status page because
 		// it cannot be "found" with the StrongNameIdentityPermission for ECMA key.
 		// But it's there!
@@ -110,8 +114,8 @@ namespace System.Security {
 			zone = new ArrayList ();
 			origin = new ArrayList ();
 		}
-#endif
 
+		[Obsolete]
 		public static bool IsGranted (IPermission perm)
 		{
 			if (perm == null)
@@ -124,38 +128,10 @@ namespace System.Security {
 			// - Not affected by overrides (like Assert, Deny and PermitOnly)
 			// - calls IsSubsetOf even for non CAS permissions
 			//   (i.e. it does call Demand so any code there won't be executed)
-#if NET_2_0
 			// with 2.0 identity permission are unrestrictable
 			return IsGranted (Assembly.GetCallingAssembly (), perm);
-#else
-			if (perm is IUnrestrictedPermission)
-				return IsGranted (Assembly.GetCallingAssembly (), perm);
-			else
-				return IsGrantedRestricted (Assembly.GetCallingAssembly (), perm);
-#endif
 		}
 
-#if !NET_2_0
-		// only for permissions that do not implement IUnrestrictedPermission
-		internal static bool IsGrantedRestricted (Assembly a, IPermission perm)
-		{
-			PermissionSet granted = a.GrantedPermissionSet;
-			if (granted != null) {
-				CodeAccessPermission grant = (CodeAccessPermission) granted.GetPermission (perm.GetType ());
-				if (!perm.IsSubsetOf (grant)) {
-					return false;
-				}
-			}
-
-			PermissionSet denied = a.DeniedPermissionSet;
-			if (denied != null) {
-				CodeAccessPermission refuse = (CodeAccessPermission) a.DeniedPermissionSet.GetPermission (perm.GetType ());
-				if ((refuse != null) && perm.IsSubsetOf (refuse))
-					return false;
-			}
-			return true;
-		}
-#endif
 		// note: in 2.0 *all* permissions (including identity permissions) support unrestricted
 		internal static bool IsGranted (Assembly a, IPermission perm)
 		{
@@ -186,18 +162,8 @@ namespace System.Security {
 			foreach (IPermission p in ps) {
 				// note: this may contains non CAS permissions
 				if ((!noncas) && (p is CodeAccessPermission)) {
-#if NET_2_0
 					if (!IsGranted (a, p))
 						return p;
-#else
-					if (p is IUnrestrictedPermission) {
-						if (!IsGranted (a, p))
-							return p;
-					} else {
-						if (!IsGrantedRestricted (a, p))
-							return p;
-					}
-#endif
 				} else {
 					// but non-CAS will throw on failure...
 					try {
@@ -220,13 +186,8 @@ namespace System.Security {
 			PermissionSet granted = ad.GrantedPermissionSet;
 			if (granted == null)
 				return null;
-#if NET_2_0
 			if (granted.IsUnrestricted ())
 				return null;
-#else
-			if ((granted.Count == 0) && granted.IsUnrestricted ())
-				return null;
-#endif
 			if (ps.IsUnrestricted ())
 				return new SecurityPermission (SecurityPermissionFlag.NoFlags);
 
@@ -255,6 +216,7 @@ namespace System.Security {
 			return null;
 		}
 
+		[Obsolete]
 		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static PolicyLevel LoadPolicyLevelFromFile (string path, PolicyLevelType type)
 		{
@@ -272,6 +234,7 @@ namespace System.Security {
 			return pl;
 		}
 
+		[Obsolete]
 		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static PolicyLevel LoadPolicyLevelFromString (string str, PolicyLevelType type)
 		{
@@ -289,12 +252,14 @@ namespace System.Security {
 			return pl;
 		}
 
+		[Obsolete]
 		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static IEnumerator PolicyHierarchy ()
 		{
 			return Hierarchy;
 		}
 
+		[Obsolete]
 		public static PermissionSet ResolvePolicy (Evidence evidence)
 		{
 			// no evidence, no permission
@@ -316,7 +281,7 @@ namespace System.Security {
 			return ps;
 		}
 
-#if NET_2_0
+		[Obsolete]
 		[MonoTODO ("(2.0) more tests are needed")]
 		public static PermissionSet ResolvePolicy (Evidence[] evidences)
 		{
@@ -333,6 +298,7 @@ namespace System.Security {
 			return ps;
 		}
 
+		[Obsolete]
 		public static PermissionSet ResolveSystemPolicy (Evidence evidence)
 		{
 			// no evidence, no permission
@@ -353,10 +319,10 @@ namespace System.Security {
 			ResolveIdentityPermissions (ps, evidence);
 			return ps;
 		}
-#endif
 
 		static private SecurityPermission _execution = new SecurityPermission (SecurityPermissionFlag.Execution);
 
+		[Obsolete]
 		public static PermissionSet ResolvePolicy (Evidence evidence, PermissionSet reqdPset, PermissionSet optPset, PermissionSet denyPset, out PermissionSet denied)
 		{
 			PermissionSet resolved = ResolvePolicy (evidence);
@@ -391,6 +357,7 @@ namespace System.Security {
 			return resolved;
 		}
 
+		[Obsolete]
 		public static IEnumerator ResolvePolicyGroups (Evidence evidence)
 		{
 			if (evidence == null)
@@ -407,6 +374,7 @@ namespace System.Security {
 			return al.GetEnumerator ();
 		}
 
+		[Obsolete]
 		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static void SavePolicy () 
 		{
@@ -417,6 +385,7 @@ namespace System.Security {
 			}
 		}
 
+		[Obsolete]
 		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static void SavePolicyLevel (PolicyLevel level) 
 		{
@@ -440,7 +409,7 @@ namespace System.Security {
 		{
 			string machinePolicyPath = Path.GetDirectoryName (Environment.GetMachineConfigPath ());
 			// note: use InternalGetFolderPath to avoid recursive policy initialization
-			string userPolicyPath = Path.Combine (Environment.InternalGetFolderPath (Environment.SpecialFolder.ApplicationData), "mono");
+			string userPolicyPath = Path.Combine (Environment.UnixGetFolderPath (Environment.SpecialFolder.ApplicationData, Environment.SpecialFolderOption.Create), "mono");
 
 			PolicyLevel enterprise = new PolicyLevel ("Enterprise", PolicyLevelType.Enterprise);
 			_level = enterprise;
@@ -486,11 +455,10 @@ namespace System.Security {
 
 		internal static void ResolveIdentityPermissions (PermissionSet ps, Evidence evidence)
 		{
-#if NET_2_0
 			// in 2.0 identity permissions can now be unrestricted
 			if (ps.IsUnrestricted ())
 				return;
-#endif
+
 			// Only host evidence are used for policy resolution
 			IEnumerator ee = evidence.GetHostEnumerator ();
 			while (ee.MoveNext ()) {
@@ -734,17 +702,12 @@ namespace System.Security {
 			throw new SecurityException (message, an, granted, refused, method, SecurityAction.InheritanceDemand, null, null, null);
 		}
 
-#if NET_2_1
-		private static void MethodAccessException (IntPtr caller, IntPtr callee)
-		{
-			throw new MethodAccessException (Locale.GetText ("Method call not allowed."));
-		}
+		// called by the runtime when CoreCLR is enabled
 
-		private static void VerificationException ()
+		private static void ThrowException (Exception ex)
 		{
-			throw new VerificationException (Locale.GetText ("Unsafe code encountered."));
+			throw ex;
 		}
-#endif
 
 		// internal - get called by the class loader
 
@@ -798,6 +761,22 @@ namespace System.Security {
 		{
 			throw new SecurityException ("SecurityAction.DemandChoice was removed from 2.0");
 		}
-#pragma warning restore 169		
+#pragma warning restore 169
+
+		public static PermissionSet GetStandardSandbox (Evidence evidence)
+		{
+			if (evidence == null)
+				throw new ArgumentNullException ("evidence");
+
+			throw new NotImplementedException ();
+		}
+
+		public static bool CurrentThreadRequiresSecurityContextCapture ()
+		{
+			throw new NotImplementedException ();
+		}
 	}
 }
+
+#endif
+
