@@ -36,14 +36,14 @@ using System.Globalization;
 using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
 using System.IO;
-using System.Collections;
+using System.Collections.Generic;
 using System.Reflection;
 using System.Security;
 using System.Runtime.ConstrainedExecution;
 
 namespace System.Threading {
-
-	internal class InternalThread : CriticalFinalizerObject {
+	[StructLayout (LayoutKind.Sequential)]
+	sealed class InternalThread : CriticalFinalizerObject {
 #pragma warning disable 169, 414, 649
 		#region Sync with metadata/object-internals.h
 		int lock_thread_id;
@@ -51,9 +51,7 @@ namespace System.Threading {
 		internal IntPtr system_thread_handle;
 
 		/* Note this is an opaque object (an array), not a CultureInfo */
-		private object cached_culture_info; /*FIXME remove this on the next corlib version bump*/
-		private IntPtr unused0;
-		internal bool threadpool_thread;
+		private object cached_culture_info;
 		/* accessed only from unmanaged code */
 		private IntPtr name;
 		private int name_len; 
@@ -70,45 +68,46 @@ namespace System.Threading {
 		private IntPtr stack_ptr;
 		private UIntPtr static_data; /* GC-tracked */
 		private IntPtr jit_data;
-		private IntPtr lock_data;
+		private IntPtr runtime_thread_info;
 		/* current System.Runtime.Remoting.Contexts.Context instance
 		   keep as an object to avoid triggering its class constructor when not needed */
 		private object current_appcontext;
-		internal int stack_size;
+		private object pending_exception;
+		private object root_domain_thread;
+		internal byte[] _serialized_principal;
+		internal int _serialized_principal_version;
 		private IntPtr appdomain_refs;
 		private int interruption_requested;
 		private IntPtr suspend_event;
 		private IntPtr suspended_event;
 		private IntPtr resume_event;
 		private IntPtr synch_cs;
+		internal bool threadpool_thread;
 		private bool thread_dump_requested;
-		private IntPtr end_stack;
 		private bool thread_interrupt_requested;
+		private IntPtr end_stack;
+		/* These are used from managed code */
+		internal int stack_size;
 		internal byte apartment_state;
 		internal volatile int critical_region_level;
+		internal int managed_id;
 		private int small_id;
 		private IntPtr manage_callback;
-		private object pending_exception;
-		/* This is the ExecutionContext that will be set by
-		   start_wrapper() in the runtime. */
-		private ExecutionContext ec_to_set;
-
 		private IntPtr interrupt_on_stop;
-
+		private IntPtr flags;
+		private IntPtr android_tid;
+		private IntPtr thread_pinning_ref;
+		private int ignore_next_signal;
 		/* 
 		 * These fields are used to avoid having to increment corlib versions
 		 * when a new field is added to the unmanaged MonoThread structure.
 		 */
-		private IntPtr unused3;
-		private IntPtr unused4;
-		private IntPtr unused5;
-		internal int managed_id;
+		private IntPtr unused0;
+		private IntPtr unused1;
+		private IntPtr unused2;
 		#endregion
 #pragma warning restore 169, 414, 649
 
-		internal byte[] _serialized_principal;
-		internal int _serialized_principal_version;
-
 		// Closes the system thread handle
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern void Thread_free_internal(IntPtr handle);
@@ -122,7 +121,12 @@ namespace System.Threading {
 	[ClassInterface (ClassInterfaceType.None)]
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_Thread))]
+	[StructLayout (LayoutKind.Sequential)]
+#if MOBILE
+	public sealed class Thread : CriticalFinalizerObject {
+#else
 	public sealed class Thread : CriticalFinalizerObject, _Thread {
+#endif
 #pragma warning disable 414		
 		#region Sync with metadata/object-internals.h
 		private InternalThread internal_thread;
@@ -133,6 +137,8 @@ namespace System.Threading {
 
 		IPrincipal principal;
 		int principal_version;
+		bool current_culture_set;
+		bool current_ui_culture_set;
 		CultureInfo current_culture;
 		CultureInfo current_ui_culture;
 
@@ -150,6 +156,11 @@ namespace System.Threading {
 		[ThreadStatic]
 		static ExecutionContext _ec;
 
+		static NamedDataSlot namedDataSlot;		
+
+		static internal CultureInfo default_culture;
+		static internal CultureInfo default_ui_culture;
+
 		// can be both a ThreadStart and a ParameterizedThreadStart
 		private MulticastDelegate threadstart;
 		//private string thread_name=null;
@@ -184,7 +195,77 @@ namespace System.Threading {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static byte[] ByteArrayToCurrentDomain (byte[] arr);
 
-#if !MOONLIGHT
+		static void DeserializePrincipal (Thread th)
+		{
+			MemoryStream ms = new MemoryStream (ByteArrayToCurrentDomain (th.Internal._serialized_principal));
+			int type = ms.ReadByte ();
+			if (type == 0) {
+				BinaryFormatter bf = new BinaryFormatter ();
+				th.principal = (IPrincipal) bf.Deserialize (ms);
+				th.principal_version = th.Internal._serialized_principal_version;
+			} else if (type == 1) {
+				BinaryReader reader = new BinaryReader (ms);
+				string name = reader.ReadString ();
+				string auth_type = reader.ReadString ();
+				int n_roles = reader.ReadInt32 ();
+				string [] roles = null;
+				if (n_roles >= 0) {
+					roles = new string [n_roles];
+					for (int i = 0; i < n_roles; i++)
+						roles [i] = reader.ReadString ();
+				}
+				th.principal = new GenericPrincipal (new GenericIdentity (name, auth_type), roles);
+			} else if (type == 2 || type == 3) {
+				string [] roles = type == 2 ? null : new string [0];
+				th.principal = new GenericPrincipal (new GenericIdentity ("", ""), roles);
+			}
+		}
+
+		static void SerializePrincipal (Thread th, IPrincipal value)
+		{
+			MemoryStream ms = new MemoryStream ();
+			bool done = false;
+			if (value.GetType () == typeof (GenericPrincipal)) {
+				GenericPrincipal gp = (GenericPrincipal) value;
+				if (gp.Identity != null && gp.Identity.GetType () == typeof (GenericIdentity)) {
+					GenericIdentity id = (GenericIdentity) gp.Identity;
+					if (id.Name == "" && id.AuthenticationType == "") {
+						if (gp.Roles == null) {
+							ms.WriteByte (2);
+							done = true;
+						} else if (gp.Roles.Length == 0) {
+							ms.WriteByte (3);
+							done = true;
+						}
+					} else {
+						ms.WriteByte (1);
+						BinaryWriter br = new BinaryWriter (ms);
+						br.Write (gp.Identity.Name);
+						br.Write (gp.Identity.AuthenticationType);
+						string [] roles = gp.Roles;
+						if  (roles == null) {
+							br.Write ((int) (-1));
+						} else {
+							br.Write (roles.Length);
+							foreach (string s in roles) {
+								br.Write (s);
+							}
+						}
+						br.Flush ();
+						done = true;
+					}
+				}
+			}
+			if (!done) {
+				ms.WriteByte (0);
+				BinaryFormatter bf = new BinaryFormatter ();
+				try {
+					bf.Serialize (ms, value);
+				} catch {}
+			}
+			th.Internal._serialized_principal = ByteArrayToRootDomain (ms.ToArray ());
+		}
+
 		public static IPrincipal CurrentPrincipal {
 			get {
 				Thread th = CurrentThread;
@@ -197,13 +278,9 @@ namespace System.Threading {
 
 				if (th.Internal._serialized_principal != null) {
 					try {
-						BinaryFormatter bf = new BinaryFormatter ();
-						MemoryStream ms = new MemoryStream (ByteArrayToCurrentDomain (th.Internal._serialized_principal));
-						th.principal = (IPrincipal) bf.Deserialize (ms);
-						th.principal_version = th.Internal._serialized_principal_version;
+						DeserializePrincipal (th);
 						return th.principal;
-					} catch (Exception) {
-					}
+					} catch {}
 				}
 
 				th.principal = GetDomain ().DefaultPrincipal;
@@ -214,26 +291,33 @@ namespace System.Threading {
 			set {
 				Thread th = CurrentThread;
 
-				++th.Internal._serialized_principal_version;
-				try {
-					BinaryFormatter bf = new BinaryFormatter ();
-					MemoryStream ms = new MemoryStream ();
-					bf.Serialize (ms, value);
-					th.Internal._serialized_principal = ByteArrayToRootDomain (ms.ToArray ());
-				} catch (Exception) {
+				if (value != GetDomain ().DefaultPrincipal) {
+					++th.Internal._serialized_principal_version;
+					try {
+						SerializePrincipal (th, value);
+					} catch (Exception) {
+						th.Internal._serialized_principal = null;
+					}
+					th.principal_version = th.Internal._serialized_principal_version;
+				} else {
 					th.Internal._serialized_principal = null;
 				}
 
 				th.principal = value;
-				th.principal_version = th.Internal._serialized_principal_version;
 			}
 		}
-#endif
 
 		// Looks up the object associated with the current thread
+		// this is called by the JIT directly, too
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static InternalThread CurrentInternalThread_internal();
 
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		internal extern static uint AllocTlsData (Type type);
+
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		internal extern static void DestroyTlsData (uint offset);
+
 		public static Thread CurrentThread {
 			[ReliabilityContract (Consistency.WillNotCorruptState, Cer.MayFail)]
 			get {
@@ -248,52 +332,28 @@ namespace System.Threading {
 				return (int)(CurrentThread.internal_thread.thread_id);
 			}
 		}
-
-#if !MOONLIGHT
-		// Stores a hash keyed by strings of LocalDataStoreSlot objects
-		static Hashtable datastorehash;
-		private static object datastore_lock = new object ();
 		
-		private static void InitDataStoreHash () {
-			lock (datastore_lock) {
-				if (datastorehash == null) {
-					datastorehash = Hashtable.Synchronized(new Hashtable());
-				}
+		static NamedDataSlot NamedDataSlot {
+			get {
+				if (namedDataSlot == null)
+					Interlocked.CompareExchange (ref namedDataSlot, new NamedDataSlot (), null);
+
+				return namedDataSlot;
 			}
 		}
 		
-		public static LocalDataStoreSlot AllocateNamedDataSlot (string name) {
-			lock (datastore_lock) {
-				if (datastorehash == null)
-					InitDataStoreHash ();
-				LocalDataStoreSlot slot = (LocalDataStoreSlot)datastorehash [name];
-				if (slot != null) {
-					// This exception isnt documented (of
-					// course) but .net throws it
-					throw new ArgumentException("Named data slot already added");
-				}
-			
-				slot = AllocateDataSlot ();
-
-				datastorehash.Add (name, slot);
-
-				return slot;
-			}
+		public static LocalDataStoreSlot AllocateNamedDataSlot (string name)
+		{
+			return NamedDataSlot.Allocate (name);
 		}
 
-		public static void FreeNamedDataSlot (string name) {
-			lock (datastore_lock) {
-				if (datastorehash == null)
-					InitDataStoreHash ();
-				LocalDataStoreSlot slot = (LocalDataStoreSlot)datastorehash [name];
-
-				if (slot != null) {
-					datastorehash.Remove (slot);
-				}
-			}
+		public static void FreeNamedDataSlot (string name)
+		{
+			NamedDataSlot.Free (name);
 		}
 
-		public static LocalDataStoreSlot AllocateDataSlot () {
+		public static LocalDataStoreSlot AllocateDataSlot ()
+		{
 			return new LocalDataStoreSlot (true);
 		}
 
@@ -325,20 +385,10 @@ namespace System.Threading {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		internal extern static void FreeLocalSlotValues (int slot, bool thread_local);
 
-		public static LocalDataStoreSlot GetNamedDataSlot(string name) {
-			lock (datastore_lock) {
-				if (datastorehash == null)
-					InitDataStoreHash ();
-				LocalDataStoreSlot slot=(LocalDataStoreSlot)datastorehash[name];
-
-				if(slot==null) {
-					slot=AllocateNamedDataSlot(name);
-				}
-			
-				return(slot);
-			}
+		public static LocalDataStoreSlot GetNamedDataSlot(string name)
+	 	{
+	 		return NamedDataSlot.Get (name);
 		}
-#endif
 		
 		public static AppDomain GetDomain() {
 			return AppDomain.CurrentDomain;
@@ -356,11 +406,10 @@ namespace System.Threading {
 			ResetAbort_internal ();
 		}
 
-#if NET_4_0 || MOBILE
 		[HostProtectionAttribute (SecurityAction.LinkDemand, Synchronization = true, ExternalThreading = true)]
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		[ReliabilityContract (Consistency.WillNotCorruptState, Cer.Success)]
 		public extern static bool Yield ();
-#endif
 
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
@@ -397,8 +446,13 @@ namespace System.Threading {
 		private Thread (InternalThread it) {
 			internal_thread = it;
 		}
+		
+		// part of ".NETPortable,Version=v4.0,Profile=Profile3" i.e. FX4 and SL4
+		[ReliabilityContract (Consistency.WillNotCorruptState, Cer.Success)]
+		~Thread ()
+		{
+		}
 
-#if !MOONLIGHT
 		[Obsolete ("Deprecated in favor of GetApartmentState, SetApartmentState and TrySetApartmentState.")]
 		public ApartmentState ApartmentState {
 			get {
@@ -412,7 +466,6 @@ namespace System.Threading {
 				TrySetApartmentState (value);
 			}
 		}
-#endif // !NET_2_1
 
 		//[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		//private static extern int current_lcid ();
@@ -420,11 +473,13 @@ namespace System.Threading {
 		public CultureInfo CurrentCulture {
 			get {
 				CultureInfo culture = current_culture;
-				if (culture != null)
+				if (current_culture_set && culture != null)
 					return culture;
 
+				if (default_culture != null)
+					return default_culture;
+
 				current_culture = culture = CultureInfo.ConstructCurrentCulture ();
-				NumberFormatter.SetThreadCurrentCulture (culture);
 				return culture;
 			}
 			
@@ -435,16 +490,19 @@ namespace System.Threading {
 
 				value.CheckNeutral ();
 				current_culture = value;
-				NumberFormatter.SetThreadCurrentCulture (value);
+				current_culture_set = true;
 			}
 		}
 
 		public CultureInfo CurrentUICulture {
 			get {
 				CultureInfo culture = current_ui_culture;
-				if (culture != null)
+				if (current_ui_culture_set && culture != null)
 					return culture;
 
+				if (default_ui_culture != null)
+					return default_ui_culture;
+
 				current_ui_culture = culture = CultureInfo.ConstructCurrentUICulture ();
 				return culture;
 			}
@@ -453,6 +511,7 @@ namespace System.Threading {
 				if (value == null)
 					throw new ArgumentNullException ("value");
 				current_ui_culture = value;
+				current_ui_culture_set = true;
 			}
 		}
 
@@ -524,17 +583,16 @@ namespace System.Threading {
 			}
 		}
 
-#if !MOONLIGHT
 		public ThreadPriority Priority {
 			get {
-				return(ThreadPriority.Lowest);
+				return (ThreadPriority)GetPriority (Internal);
 			}
 			
 			set {
-				// FIXME: Implement setter.
+				// FIXME: This doesn't do anything yet
+				SetPriority (Internal, (int)value);
 			}
 		}
-#endif
 
 		public ThreadState ThreadState {
 			get {
@@ -545,13 +603,18 @@ namespace System.Threading {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static void Abort_internal (InternalThread thread, object stateInfo);
 
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		private extern static int GetPriority (InternalThread thread);
+
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		private extern static void SetPriority (InternalThread thread, int priority);
+
 		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
 		public void Abort () 
 		{
 			Abort_internal (Internal, null);
 		}
 
-#if !MOONLIGHT
 		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
 		public void Abort (object stateInfo) 
 		{
@@ -569,7 +632,6 @@ namespace System.Threading {
 		{
 			Interrupt_internal (Internal);
 		}
-#endif
 
 		// The current thread joins with 'this'. Set ms to 0 to block
 		// until this actually exits.
@@ -589,7 +651,6 @@ namespace System.Threading {
 			return Join_internal (Internal, millisecondsTimeout, Internal.system_thread_handle);
 		}
 
-#if !MOONLIGHT
 		public bool Join(TimeSpan timeout)
 		{
 			long ms = (long) timeout.TotalMilliseconds;
@@ -598,14 +659,10 @@ namespace System.Threading {
 
 			return Join_internal (Internal, (int) ms, Internal.system_thread_handle);
 		}
-#endif
 
-#if NET_1_1
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static void MemoryBarrier ();
-#endif
 
-#if !MOONLIGHT
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern void Resume_internal();
 
@@ -615,7 +672,6 @@ namespace System.Threading {
 		{
 			Resume_internal ();
 		}
-#endif // !NET_2_1
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		private extern static void SpinWait_nop ();
@@ -632,69 +688,6 @@ namespace System.Threading {
 			}
 		}
 
-#if MOONLIGHT
-		private void StartInternal ()
-		{
-			current_thread = this;
-
-			try {
-				if (threadstart is ThreadStart) {
-					((ThreadStart) threadstart) ();
-				} else {
-					((ParameterizedThreadStart) threadstart) (start_obj);
-				}
-			} catch (ThreadAbortException) {
-				// do nothing
-			} catch (Exception ex) {
-				MoonlightUnhandledException (ex);
-			}
-		}
-
-		static MethodInfo moonlight_unhandled_exception = null;
-
-		static internal void MoonlightUnhandledException (Exception e)
-		{
-			try {
-				if (moonlight_unhandled_exception == null) {
-					var assembly = System.Reflection.Assembly.Load ("System.Windows, Version=2.0.5.0, Culture=Neutral, PublicKeyToken=7cec85d7bea7798e");
-					var application = assembly.GetType ("System.Windows.Application");
-					moonlight_unhandled_exception = application.GetMethod ("OnUnhandledException", 
-						System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Static);
-				}
-				moonlight_unhandled_exception.Invoke (null, new object [] { null, e });
-			}
-			catch {
-				try {
-					Console.WriteLine ("Unexpected exception while trying to report unhandled application exception: {0}", e);
-				} catch {
-				}
-			}
-		}
-#elif MONOTOUCH
-		static ConstructorInfo nsautoreleasepool_ctor;
-		
-		IDisposable GetNSAutoreleasePool ()
-		{
-			if (nsautoreleasepool_ctor == null) {
-				Type t = Type.GetType ("MonoTouch.Foundation.NSAutoreleasePool, monotouch, Version=0.0.0.0, Culture=neutral, PublicKeyToken=null");
-				nsautoreleasepool_ctor = t.GetConstructor (Type.EmptyTypes);
-			}
-			return (IDisposable) nsautoreleasepool_ctor.Invoke (null);
-		}
-		
-		private void StartInternal ()
-		{
-			using (var pool = GetNSAutoreleasePool ()) {
-				current_thread = this;
-			
-				if (threadstart is ThreadStart) {
-					((ThreadStart) threadstart) ();
-				} else {
-					((ParameterizedThreadStart) threadstart) (start_obj);
-				}
-			}
-		}
-#else
 		private void StartInternal ()
 		{
 			current_thread = this;
@@ -705,11 +698,10 @@ namespace System.Threading {
 				((ParameterizedThreadStart) threadstart) (start_obj);
 			}
 		}
-#endif
+
 		public void Start() {
 			// propagate informations from the original thread to the new thread
-			if (!ExecutionContext.IsFlowSuppressed ())
-				ec_to_set = ExecutionContext.Capture ();
+			ec_to_set = ExecutionContext.Capture (false, true);
 			Internal._serialized_principal = CurrentThread.Internal._serialized_principal;
 
 			// Thread_internal creates and starts the new thread, 
@@ -717,7 +709,6 @@ namespace System.Threading {
 				throw new SystemException ("Thread creation failed.");
 		}
 
-#if !MOONLIGHT
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static void Suspend_internal(InternalThread thread);
 
@@ -727,7 +718,6 @@ namespace System.Threading {
 		{
 			Suspend_internal (Internal);
 		}
-#endif // !NET_2_1
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		extern private static void SetState (InternalThread thread, ThreadState set);
@@ -738,8 +728,6 @@ namespace System.Threading {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		extern private static ThreadState GetState (InternalThread thread);
 
-#if NET_1_1
-		
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		extern public static byte VolatileRead (ref byte address);
 		
@@ -828,7 +816,8 @@ namespace System.Threading {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		extern public static void VolatileWrite (ref UIntPtr address, UIntPtr value);
 		
-#endif
+		[MethodImplAttribute (MethodImplOptions.InternalCall)]
+		extern static int SystemMaxStackStize ();
 
 		static int CheckStackSize (int maxStackSize)
 		{
@@ -843,12 +832,8 @@ namespace System.Threading {
 			if ((maxStackSize % page_size) != 0) // round up to a divisible of page size
 				maxStackSize = (maxStackSize / (page_size - 1)) * page_size;
 
-			int default_stack_size = (IntPtr.Size / 4) * 1024 * 1024; // from wthreads.c
-
-			if (maxStackSize > default_stack_size)
-				return default_stack_size;
-
-			return maxStackSize; 
+			/* Respect the max stack size imposed by the system*/
+			return Math.Min (maxStackSize, SystemMaxStackStize ());
 		}
 
 		public Thread (ThreadStart start, int maxStackSize)
@@ -877,7 +862,6 @@ namespace System.Threading {
 			Internal.stack_size = CheckStackSize (maxStackSize);
 		}
 
-		[MonoTODO ("limited to CompressedStack support")]
 		public ExecutionContext ExecutionContext {
 			[ReliabilityContract (Consistency.WillNotCorruptState, Cer.MayFail)]
 			get {
@@ -885,6 +869,35 @@ namespace System.Threading {
 					_ec = new ExecutionContext ();
 				return _ec;
 			}
+			internal set {
+				_ec = value;
+			}
+		}
+
+		internal bool HasExecutionContext {
+			get {
+				return _ec != null;
+			}
+		}
+
+		internal void BranchExecutionContext (out ExecutionContext.Switcher switcher)
+		{
+			if (_ec == null) {
+				switcher =  new ExecutionContext.Switcher ();
+			} else {
+				switcher = new ExecutionContext.Switcher (_ec);
+				_ec.CopyOnWrite = true;
+			}
+		}
+
+		internal void RestoreExecutionContext (ref ExecutionContext.Switcher switcher)
+		{
+			if (switcher.IsEmpty) {
+				_ec = null;
+				return;
+			}
+
+			switcher.Restore (_ec);
 		}
 
 		public int ManagedThreadId {
@@ -918,7 +931,6 @@ namespace System.Threading {
 			// Managed and native threads are currently bound together.
 		}
 
-#if !MOONLIGHT
 		public ApartmentState GetApartmentState ()
 		{
 			return (ApartmentState)Internal.apartment_state;
@@ -932,21 +944,17 @@ namespace System.Threading {
 
 		public bool TrySetApartmentState (ApartmentState state) 
 		{
-			/* Only throw this exception when changing the
-			 * state of another thread.  See bug 324338
-			 */
-			if ((this != CurrentThread) &&
-			    (ThreadState & ThreadState.Unstarted) == 0)
+			if ((ThreadState & ThreadState.Unstarted) == 0)
 				throw new ThreadStateException ("Thread was in an invalid state for the operation being executed.");
 
-			if ((ApartmentState)Internal.apartment_state != ApartmentState.Unknown)
+			if ((ApartmentState)Internal.apartment_state != ApartmentState.Unknown && 
+			    (ApartmentState)Internal.apartment_state != state)
 				return false;
 
 			Internal.apartment_state = (byte)state;
 
 			return true;
 		}
-#endif // !NET_2_1
 		
 		[ComVisible (false)]
 		public override int GetHashCode ()
@@ -960,25 +968,23 @@ namespace System.Threading {
 			Start ();
 		}
 
-#if !MOONLIGHT
 		// NOTE: This method doesn't show in the class library status page because
 		// it cannot be "found" with the StrongNameIdentityPermission for ECMA key.
 		// But it's there!
 		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey="00000000000000000400000000000000")]
 		[Obsolete ("see CompressedStack class")]
-#if NET_1_1
-		public
-#else
-		internal
-#endif
-		CompressedStack GetCompressedStack ()
+		public CompressedStack GetCompressedStack ()
 		{
+#if MOBILE
+			throw new NotSupportedException ();
+#else			
 			// Note: returns null if no CompressedStack has been set.
 			// However CompressedStack.GetCompressedStack returns an 
 			// (empty?) CompressedStack instance.
 			CompressedStack cs = ExecutionContext.SecurityContext.CompressedStack;
 			return ((cs == null) || cs.IsEmpty ()) ? null : cs.CreateCopy ();
+#endif
 		}
 
 		// NOTE: This method doesn't show in the class library status page because
@@ -987,19 +993,16 @@ namespace System.Threading {
 		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey="00000000000000000400000000000000")]
 		[Obsolete ("see CompressedStack class")]
-#if NET_1_1
-		public
-#else
-		internal
-#endif
-		void SetCompressedStack (CompressedStack stack)
+		public void SetCompressedStack (CompressedStack stack)
 		{
+#if MOBILE
+			throw new NotSupportedException ();
+#else
 			ExecutionContext.SecurityContext.CompressedStack = stack;
+#endif
 		}
 
-#endif
-
-#if NET_1_1
+#if !MOBILE
 		void _Thread.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
 		{
 			throw new NotImplementedException ();
