@@ -7,7 +7,7 @@
 //
 // (C) Ximian, Inc.  http://www.ximian.com
 // (C) 2004,2005 Novell, Inc. (http://www.novell.com)
-//
+// Copyright 2013 Xamarin Inc. (http://www.xamarin.com)
 
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -39,7 +39,7 @@ using System.Text;
 
 namespace System
 {
-	public static class Console
+	public static partial class Console
 	{
 #if !NET_2_1
 		private class WindowsConsole
@@ -89,6 +89,7 @@ namespace System
 			}
 		}
 #endif
+
 		internal static TextWriter stdout;
 		private static TextWriter stderr;
 		private static TextReader stdin;
@@ -124,36 +125,65 @@ namespace System
 				// UTF-8 ZWNBSP (zero-width non-breaking space).
 				//
 				int code_page = 0;
-				Encoding.InternalCodePage (ref code_page);
+				EncodingHelper.InternalCodePage (ref code_page);
 
 				if (code_page != -1 && ((code_page & 0x0fffffff) == 3 // UTF8Encoding.UTF8_CODE_PAGE
 					|| ((code_page & 0x10000000) != 0)))
-					inputEncoding = outputEncoding = Encoding.UTF8Unmarked;
+					inputEncoding = outputEncoding = EncodingHelper.UTF8Unmarked;
 				else
 					inputEncoding = outputEncoding = Encoding.Default;
 			}
 
-			stderr = new UnexceptionalStreamWriter (OpenStandardError (0), outputEncoding); 
-			((StreamWriter)stderr).AutoFlush = true;
-			stderr = TextWriter.Synchronized (stderr, true);
+			SetupStreams (inputEncoding, outputEncoding);
+		}
 
+		static void SetupStreams (Encoding inputEncoding, Encoding outputEncoding)
+		{
 #if !NET_2_1
 			if (!Environment.IsRunningOnWindows && ConsoleDriver.IsConsole) {
-				StreamWriter w = new CStreamWriter (OpenStandardOutput (0), outputEncoding);
+				StreamWriter w = new CStreamWriter (OpenStandardOutput (0), outputEncoding, true);
 				w.AutoFlush = true;
-				stdout = TextWriter.Synchronized (w, true);
+				stdout = TextWriter.Synchronized (w);
+
+				w = new CStreamWriter (OpenStandardOutput (0), outputEncoding, true);
+				w.AutoFlush = true;
+				stderr = TextWriter.Synchronized (w);
+				
 				stdin = new CStreamReader (OpenStandardInput (0), inputEncoding);
 			} else {
 #endif
+// FULL_AOT_RUNTIME is used (instead of MONOTOUCH) since we only want this code when running on 
+// iOS (simulator or devices) and *not* when running tools (e.g. btouch #12179) that needs to use 
+// the mscorlib.dll shipped with Xamarin.iOS
+#if MONOTOUCH && FULL_AOT_RUNTIME
+				stdout = new NSLogWriter ();
+#else
 				stdout = new UnexceptionalStreamWriter (OpenStandardOutput (0), outputEncoding);
 				((StreamWriter)stdout).AutoFlush = true;
-				stdout = TextWriter.Synchronized (stdout, true);
+#endif
+				stdout = TextWriter.Synchronized (stdout);
+
+#if MONOTOUCH && FULL_AOT_RUNTIME
+				stderr = new NSLogWriter ();
+#else
+				stderr = new UnexceptionalStreamWriter (OpenStandardError (0), outputEncoding); 
+				((StreamWriter)stderr).AutoFlush = true;
+#endif
+				stderr = TextWriter.Synchronized (stderr);
+
 				stdin = new UnexceptionalStreamReader (OpenStandardInput (0), inputEncoding);
 				stdin = TextReader.Synchronized (stdin);
 #if !NET_2_1
 			}
 #endif
 
+#if MONODROID
+			if (LogcatTextWriter.IsRunningOnAndroid ()) {
+				stdout = TextWriter.Synchronized (new LogcatTextWriter ("mono-stdout", stdout));
+				stderr = TextWriter.Synchronized (new LogcatTextWriter ("mono-stderr", stderr));
+			}
+#endif  // MONODROID
+
 			GC.SuppressFinalize (stdout);
 			GC.SuppressFinalize (stderr);
 			GC.SuppressFinalize (stdin);
@@ -179,14 +209,10 @@ namespace System
 
 		private static Stream Open (IntPtr handle, FileAccess access, int bufferSize)
 		{
-#if MOONLIGHT
-			if (SecurityManager.SecurityEnabled && !Debugger.IsAttached && Environment.GetEnvironmentVariable ("MOONLIGHT_ENABLE_CONSOLE") == null)
-				return new NullStream ();
-#endif
 			try {
 				return new FileStream (handle, access, false, bufferSize, false, bufferSize == 0);
 			} catch (IOException) {
-				return new NullStream ();
+				return Stream.Null;
 			}
 		}
 
@@ -331,7 +357,10 @@ namespace System
 
 		public static void Write (string format, params object[] arg)
 		{
-			stdout.Write (format, arg);
+			if (arg == null)
+				stdout.Write (format);
+			else
+				stdout.Write (format, arg);
 		}
 
 		public static void Write (char[] buffer, int index, int count)
@@ -442,7 +471,10 @@ namespace System
 
 		public static void WriteLine (string format, params object[] arg)
 		{
-			stdout.WriteLine (format, arg);
+			if (arg == null)
+				stdout.WriteLine (format);
+			else
+				stdout.WriteLine (format, arg);
 		}
 
 		public static void WriteLine (char[] buffer, int index, int count)
@@ -517,12 +549,18 @@ namespace System
 
 		public static Encoding InputEncoding {
 			get { return inputEncoding; }
-			set { inputEncoding = value; }
+			set {
+				inputEncoding = value;
+				SetupStreams (inputEncoding, outputEncoding);
+			}
 		}
 
 		public static Encoding OutputEncoding {
 			get { return outputEncoding; }
-			set { outputEncoding = value; }
+			set {
+				outputEncoding = value;
+				SetupStreams (inputEncoding, outputEncoding);
+			}
 		}
 
 		public static ConsoleColor BackgroundColor {
@@ -623,6 +661,24 @@ namespace System
 			set { ConsoleDriver.WindowWidth = value; }
 		}
 
+		public static bool IsErrorRedirected {
+			get {
+				return ConsoleDriver.IsErrorRedirected;
+			}
+		}
+
+		public static bool IsOutputRedirected {
+			get {
+				return ConsoleDriver.IsOutputRedirected;
+			}
+		}
+
+		public static bool IsInputRedirected {
+			get {
+				return ConsoleDriver.IsInputRedirected;
+			}
+		}
+
 		public static void Beep ()
 		{
 			Beep (1000, 500);
