@@ -39,11 +39,13 @@ using System.Security;
 using System.Security.Permissions;
 using System.Text;
 using System.Runtime.InteropServices;
+using System.Threading;
+using System.Diagnostics.Contracts;
 
 namespace System {
 
 	[ComVisible (true)]
-	public static class Environment {
+	public static partial class Environment {
 
 		/*
 		 * This is the version number of the corlib-runtime interface. When
@@ -55,7 +57,7 @@ namespace System {
 		 * of icalls, do not require an increment.
 		 */
 #pragma warning disable 169
-		private const int mono_corlib_version = 94;
+		private const int mono_corlib_version = 117;
 #pragma warning restore 169
 
 		[ComVisible (true)]
@@ -84,10 +86,7 @@ namespace System {
 			ProgramFiles = 0x26,
 			MyPictures = 0x27,
 			CommonProgramFiles = 0x2b,
-#if NET_4_0 || MOONLIGHT || MOBILE
 			MyVideos = 0x0e,
-#endif
-#if NET_4_0
 			NetworkShortcuts = 0x13,
 			Fonts = 0x14,
 			CommonStartMenu = 0x16,
@@ -111,14 +110,9 @@ namespace System {
 			LocalizedResources = 0x39,
 			CommonOemLinks = 0x3a,
 			CDBurning = 0x3b,
-#endif
 		}
 
-#if NET_4_0
 		public
-#else
-		internal
-#endif
 		enum SpecialFolderOption {
 			None = 0,
 			DoNotVerify = 0x4000,
@@ -168,6 +162,12 @@ namespace System {
 				Directory.SetCurrentDirectory (value);
 			}
 		}
+		
+		public static int CurrentManagedThreadId {
+			get {
+				return Thread.CurrentThread.ManagedThreadId;
+			}
+		}
 
 		/// <summary>
 		/// Gets or sets the exit code of this process
@@ -197,12 +197,21 @@ namespace System {
 			get;
 		}
 
+		[MethodImplAttribute (MethodImplOptions.InternalCall)]
+		extern static string GetNewLine ();
+
+		static string nl;
 		/// <summary>
 		/// Gets the standard new line value
 		/// </summary>
-		public extern static string NewLine {
-			[MethodImplAttribute (MethodImplOptions.InternalCall)]
-			get;
+		public static string NewLine {
+			get {
+				if (nl != null)
+					return nl;
+
+				nl = GetNewLine ();
+				return nl;
+			}
 		}
 
 		//
@@ -224,7 +233,7 @@ namespace System {
 		public static OperatingSystem OSVersion {
 			get {
 				if (os == null) {
-					Version v = Version.CreateFromString (GetOSVersionString ());
+					Version v = CreateVersionFromString (GetOSVersionString ());
 					PlatformID p = Platform;
 					if (p == PlatformID.MacOSX)
 						p = PlatformID.Unix;
@@ -234,6 +243,75 @@ namespace System {
 			}
 		}
 
+
+		// a very gentle way to construct a Version object which takes 
+		// the first four numbers in a string as the version
+		internal static Version CreateVersionFromString (string info)
+		{
+			int major = 0;
+			int minor = 0;
+			int build = 0;
+			int revision = 0;
+			int state = 1;
+			int number = -1; // string may not begin with a digit
+
+			if (info == null)
+				return new Version (0, 0, 0, 0);
+
+			for (int i=0; i < info.Length; i++) {
+				char c = info [i];
+				if (Char.IsDigit (c)) {
+					if (number < 0) {
+						number = (c - '0');
+					}
+					else {
+						number = (number * 10) + (c - '0');
+					}
+				}
+				else if (number >= 0) {
+					// assign
+					switch (state) {
+					case 1:
+						major = number;
+						break;
+					case 2:
+						minor = number;
+						break;
+					case 3:
+						build = number;
+						break;
+					case 4:
+						revision = number;
+						break;
+					}
+					number = -1;
+					state ++;
+				}
+				// ignore end of string
+				if (state == 5)
+					break;
+			}
+
+			// Last number
+			if (number >= 0) {
+				switch (state) {
+				case 1:
+					major = number;
+					break;
+				case 2:
+					minor = number;
+					break;
+				case 3:
+					build = number;
+					break;
+				case 4:
+					revision = number;
+					break;
+				}
+			}
+			return new Version (major, minor, build, revision);
+		}
+
 		/// <summary>
 		/// Get StackTrace
 		/// </summary>
@@ -314,6 +392,11 @@ namespace System {
 		[SecurityPermission (SecurityAction.Demand, UnmanagedCode=true)]
 		public extern static void Exit (int exitCode);
 
+		internal static void _Exit (int exitCode)
+		{
+			Exit (exitCode);
+		}
+
 		/// <summary>
 		/// Substitute environment variables in the argument "name"
 		/// </summary>
@@ -348,6 +431,7 @@ namespace System {
 				// If value not found, add %FOO to stream,
 				//  and use the closing % for the next iteration.
 				// If value found, expand it in place of %FOO%
+				int realOldOff2 = off2;
 				if (value == null) {
 					result.Append ('%');
 					result.Append (var);
@@ -371,7 +455,7 @@ namespace System {
 				// If value not found in current iteration, but a % was found for next iteration,
 				//  use text from current closing % to the next %.
 				else
-					textLen = off1 - oldOff2;
+					textLen = off1 - realOldOff2;
 				if(off1 >= oldOff2 || off1 == -1)
 					result.Append (name, oldOff2+1, textLen);
 			} while (off2 > -1 && off2 < len);
@@ -456,9 +540,6 @@ namespace System {
 		}
 #endif
 
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		private extern static string GetWindowsFolderPath (int folder);
-
 		/// <summary>
 		/// Returns the fully qualified path of the
 		/// folder specified by the "folder" parameter
@@ -467,9 +548,13 @@ namespace System {
 		{
 			return GetFolderPath (folder, SpecialFolderOption.None);
 		}
-#if NET_4_0
+
+// for monotouch, not monotouch_runtime
+#if !(MONOTOUCH && FULL_AOT_RUNTIME)
+		[MethodImplAttribute (MethodImplOptions.InternalCall)]
+		private extern static string GetWindowsFolderPath (int folder);
+
 		public
-#endif
 		static string GetFolderPath(SpecialFolder folder, SpecialFolderOption option)
 		{
 			SecurityManager.EnsureElevatedPermissions (); // this is a no-op outside moonlight
@@ -512,10 +597,10 @@ namespace System {
 							string path = line.Substring (delim_index + 1).Trim ('"');
 							bool relative = false;
 							
-							if (path.StartsWith ("$HOME/")) {
+							if (path.StartsWithOrdinalUnchecked ("$HOME/")) {
 								relative = true;
 								path = path.Substring (6);
-							} else if (!path.StartsWith ("/")) {
+							} else if (!path.StartsWithOrdinalUnchecked ("/")) {
 								relative = true;
 							}
 							
@@ -558,38 +643,15 @@ namespace System {
 
 			// personal == ~
 			case SpecialFolder.Personal:
-#if MONOTOUCH
-				return Path.Combine (home, "Documents");
-#else
 				return home;
-#endif
+
 			// use FDO's CONFIG_HOME. This data will be synced across a network like the windows counterpart.
 			case SpecialFolder.ApplicationData:
-#if MONOTOUCH
-			{
-				string dir = Path.Combine (Path.Combine (home, "Documents"), ".config");
-				if (option == SpecialFolderOption.Create){
-					if (!Directory.Exists (dir))
-						Directory.CreateDirectory (dir);
-				}
-				return dir;
-			}
-#else
 				return config;
-#endif
+
 			//use FDO's DATA_HOME. This is *NOT* synced
 			case SpecialFolder.LocalApplicationData:
-#if MONOTOUCH
-			{
-				string dir = Path.Combine (home, "Documents");
-				if (!Directory.Exists (dir))
-					Directory.CreateDirectory (dir);
-
-				return dir;
-			}
-#else
 				return data;
-#endif
 
 			case SpecialFolder.Desktop:
 			case SpecialFolder.DesktopDirectory:
@@ -609,11 +671,8 @@ namespace System {
 			
 			case SpecialFolder.Templates:
 				return ReadXdgUserDir (config, home, "XDG_TEMPLATES_DIR", "Templates");
-#if NET_4_0 || MOONLIGHT || MOBILE
 			case SpecialFolder.MyVideos:
 				return ReadXdgUserDir (config, home, "XDG_VIDEOS_DIR", "Videos");
-#endif
-#if NET_4_0
 			case SpecialFolder.CommonTemplates:
 				return "/usr/share/templates";
 			case SpecialFolder.Fonts:
@@ -621,7 +680,6 @@ namespace System {
 					return Path.Combine (home, "Library", "Fonts");
 				
 				return Path.Combine (home, ".fonts");
-#endif
 			// these simply dont exist on Linux
 			// The spec says if a folder doesnt exist, we
 			// should return ""
@@ -642,7 +700,11 @@ namespace System {
 					return Path.Combine (home, "Library", "Caches");
 				else
 					return String.Empty;
-				
+
+				// #2873
+			case SpecialFolder.UserProfile:
+				return home;
+
 			case SpecialFolder.Programs:
 			case SpecialFolder.SendTo:
 			case SpecialFolder.StartMenu:
@@ -652,7 +714,6 @@ namespace System {
 			case SpecialFolder.Recent:
 			case SpecialFolder.CommonProgramFiles:
 			case SpecialFolder.System:
-#if NET_4_0
 			case SpecialFolder.NetworkShortcuts:
 			case SpecialFolder.CommonStartMenu:
 			case SpecialFolder.CommonPrograms:
@@ -660,7 +721,6 @@ namespace System {
 			case SpecialFolder.CommonDesktopDirectory:
 			case SpecialFolder.PrinterShortcuts:
 			case SpecialFolder.Windows:
-			case SpecialFolder.UserProfile:
 			case SpecialFolder.SystemX86:
 			case SpecialFolder.ProgramFilesX86:
 			case SpecialFolder.CommonProgramFilesX86:
@@ -674,15 +734,15 @@ namespace System {
 			case SpecialFolder.LocalizedResources:
 			case SpecialFolder.CommonOemLinks:
 			case SpecialFolder.CDBurning:
-#endif
 				return String.Empty;
 			// This is where data common to all users goes
 			case SpecialFolder.CommonApplicationData:
 				return "/usr/share";
 			default:
 				throw new ArgumentException ("Invalid SpecialFolder");
-                        }
-                }
+			}
+		}
+#endif
 
 		
 		[EnvironmentPermission (SecurityAction.Demand, Unrestricted=true)]
@@ -691,7 +751,7 @@ namespace System {
 			return GetLogicalDrivesInternal ();
 		}
 
-#if !NET_2_1
+#if !MOBILE
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		private static extern void internalBroadcastSettingChange ();
 
@@ -802,38 +862,54 @@ namespace System {
 				throw new ArgumentException ("target");
 			}
 		}
+#else
+		public static void SetEnvironmentVariable (string variable, string value)
+		{
+			if (variable == null)
+				throw new ArgumentNullException ("variable");
+			if (variable == String.Empty)
+				throw new ArgumentException ("String cannot be of zero length.", "variable");
+			if (variable.IndexOf ('=') != -1)
+				throw new ArgumentException ("Environment variable name cannot contain an equal character.", "variable");
+			if (variable[0] == '\0')
+				throw new ArgumentException ("The first char in the string is the null character.", "variable");
 
+			InternalSetEnvironmentVariable (variable, value);
+		}
+#endif
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		internal static extern void InternalSetEnvironmentVariable (string variable, string value);
-#endif
+
 		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode=true)]
 		public static void FailFast (string message)
 		{
 			throw new NotImplementedException ();
 		}
 
-#if NET_4_0 || MOONLIGHT || MOBILE
+		internal static void FailFast (String message, uint exitCode)
+		{
+			throw new NotImplementedException ();
+		}
+
 		[SecurityCritical]
 		public static void FailFast (string message, Exception exception)
 		{
 			throw new NotImplementedException ();
 		}
-#endif
 
-#if NET_4_0
 		public static bool Is64BitOperatingSystem {
 			get { return IntPtr.Size == 8; } // FIXME: is this good enough?
 		}
 
-		public static bool Is64BitProcess {
-			get { return Is64BitOperatingSystem; }
-		}
-
 		public static int SystemPageSize {
 			get { return GetPageSize (); }
 		}
-#endif
 
+		public
+		static bool Is64BitProcess {
+			get { return IntPtr.Size == 8; }
+		}
+		
 		public static extern int ProcessorCount {
 			[EnvironmentPermission (SecurityAction.Demand, Read="NUMBER_OF_PROCESSORS")]
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
@@ -841,10 +917,14 @@ namespace System {
 		}
 
 		// private methods
-
+#if (MONOTOUCH || MONODROID || XAMMAC)
+		internal const bool IsRunningOnWindows = false;
+#else
 		internal static bool IsRunningOnWindows {
 			get { return ((int) Platform < 4); }
 		}
+#endif
+
 #if !NET_2_1
 		//
 		// Used by gacutil.exe
@@ -887,6 +967,22 @@ namespace System {
 				return (platform == 4 || platform == 128 || platform == 6);
 			}
 		}
+		static internal bool IsMacOS {
+			get {
+				return Environment.Platform == PlatformID.MacOSX;
+			}
+		}
+
+		internal static bool IsCLRHosted {
+			get {
+				return false;
+			}
+		}
+
+		internal static void TriggerCodeContractFailure(ContractFailureKind failureKind, String message, String condition, String exceptionAsString)
+		{
+
+		}
 	}
 }
 
