@@ -26,7 +26,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_4_0 || MOBILE
 
 namespace System
 {
@@ -36,4 +35,3 @@ namespace System
 	}
 }
 
-#endif
