@@ -26,16 +26,14 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_4_0
 
 namespace System
 {
 	public interface IObserver<in T>
 	{
 		void OnCompleted ();
-		void OnError (Exception exception);
-		void OnNext(T arg);
+		void OnError (Exception error);
+		void OnNext(T value);
 	}
 }
 
-#endif
