@@ -6,9 +6,11 @@
 // 	Paolo Molaro (lupus@ximian.com)
 // 	Patrik Torstensson (patrik.torstensson@labs2.com)
 //	Gonzalo Paniagua (gonzalo@ximian.com)
+//  Marek Safar (marek.safar@gmail.com)
 //
 // (c) 2001-2003 Ximian, Inc.
 // Copyright (C) 2003-2005 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2013 Xamarin Inc (http://www.xamarin.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,17 +32,43 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+using System.Collections.Generic;
 using System.Globalization;
 using System.Reflection;
+using System.Runtime.InteropServices;
 using System.Runtime.CompilerServices;
 using System.Runtime.Serialization;
 using System.Security;
 
 namespace System
 {
-	[Serializable]
-	internal class MonoType : Type, ISerializable
+	// Contains information about the type which is expensive to compute
+	[StructLayout (LayoutKind.Sequential)]
+	internal class MonoTypeInfo {
+		public string full_name;
+		public MonoCMethod default_ctor;
+	}
+		
+	abstract class RuntimeType : TypeInfo
 	{
+		internal RuntimeAssembly GetRuntimeAssembly ()
+		{
+			return (RuntimeAssembly) Assembly;
+		}
+
+		internal virtual bool IsSzArray {
+			get {
+				return IsArrayImpl () && GetArrayRank () == 1;
+			}
+		}
+	}
+
+	[Serializable]
+	[StructLayout (LayoutKind.Sequential)]
+	sealed class MonoType : RuntimeType, ISerializable
+	{
+		[NonSerialized]
+		MonoTypeInfo type_info;
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private static extern void type_from_obj (MonoType type, Object obj);
@@ -55,7 +83,30 @@ namespace System
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private static extern TypeAttributes get_attributes (Type type);
-	
+
+		public MonoCMethod GetDefaultConstructor ()
+		{
+			MonoCMethod ctor = null;
+			
+			if (type_info == null)
+				type_info = new MonoTypeInfo ();
+			else
+				ctor = type_info.default_ctor;
+
+			if (ctor == null) {
+				var ctors = GetConstructors (BindingFlags.Public | BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
+
+				for (int i = 0; i < ctors.Length; ++i) {
+					if (ctors [i].GetParametersCount () == 0) {
+						type_info.default_ctor = ctor = (MonoCMethod) ctors [i];
+						break;
+					}
+				}
+			}
+
+			return ctor;
+		}
+
 		protected override TypeAttributes GetAttributeFlagsImpl ()
 		{
 			return get_attributes (this);
@@ -66,11 +117,20 @@ namespace System
 								       CallingConventions callConvention,
 								       Type[] types,
 								       ParameterModifier[] modifiers)
+		{
+			ConstructorInfo[] methods = GetConstructors (bindingAttr);
+			return GetConstructorImpl (methods, bindingAttr, binder, callConvention, types, modifiers);
+		}
+
+		internal static ConstructorInfo GetConstructorImpl (ConstructorInfo[] methods, BindingFlags bindingAttr,
+								       Binder binder,
+								       CallingConventions callConvention,
+								       Type[] types,
+								       ParameterModifier[] modifiers)
 		{
 			if (bindingAttr == BindingFlags.Default)
 				bindingAttr = BindingFlags.Public | BindingFlags.Instance;
 
-			ConstructorInfo[] methods = GetConstructors (bindingAttr);
 			ConstructorInfo found = null;
 			MethodBase[] match;
 			int count = 0;
@@ -150,9 +210,12 @@ namespace System
 			Type[] interfaces = GetInterfaces();
 
 			foreach (Type type in interfaces) {
-				if (String.Compare (type.Name, name, ignoreCase, CultureInfo.InvariantCulture) == 0)
+				/*We must compare against the generic type definition*/
+				Type t = type.IsGenericType ? type.GetGenericTypeDefinition () : type;
+
+				if (String.Compare (t.Name, name, ignoreCase, CultureInfo.InvariantCulture) == 0)
 					return type;
-				if (String.Compare (type.FullName, name, ignoreCase, CultureInfo.InvariantCulture) == 0)
+				if (String.Compare (t.FullName, name, ignoreCase, CultureInfo.InvariantCulture) == 0)
 					return type;
 			}
 
@@ -161,6 +224,28 @@ namespace System
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern override Type[] GetInterfaces();
+
+		public override InterfaceMapping GetInterfaceMap (Type interfaceType)
+		{
+			if (!IsSystemType)
+				throw new NotSupportedException ("Derived classes must provide an implementation.");
+			if (interfaceType == null)
+				throw new ArgumentNullException ("interfaceType");
+			if (!interfaceType.IsSystemType)
+				throw new ArgumentException ("interfaceType", "Type is an user type");
+			InterfaceMapping res;
+			if (!interfaceType.IsInterface)
+				throw new ArgumentException (Locale.GetText ("Argument must be an interface."), "interfaceType");
+			if (IsInterface)
+				throw new ArgumentException ("'this' type cannot be an interface itself");
+			res.TargetType = this;
+			res.InterfaceType = interfaceType;
+			GetInterfaceMapData (this, interfaceType, out res.TargetMethods, out res.InterfaceMethods);
+			if (res.TargetMethods == null)
+				throw new ArgumentException (Locale.GetText ("Interface not found"), "interfaceType");
+
+			return res;
+		}
 		
 		public override MemberInfo[] GetMembers( BindingFlags bindingAttr)
 		{
@@ -229,11 +314,15 @@ namespace System
 
 		internal override MethodInfo GetMethod (MethodInfo fromNoninstanciated)
                 {
+			if (fromNoninstanciated == null)
+				throw new ArgumentNullException ("fromNoninstanciated");
                         return GetCorrespondingInflatedMethod (fromNoninstanciated);
                 }
 
 		internal override ConstructorInfo GetConstructor (ConstructorInfo fromNoninstanciated)
 		{
+			if (fromNoninstanciated == null)
+				throw new ArgumentNullException ("fromNoninstanciated");
                         return GetCorrespondingInflatedConstructor (fromNoninstanciated);
 		}
 
@@ -315,11 +404,7 @@ namespace System
 						     ParameterModifier[] modifiers,
 						     CultureInfo culture, string[] namedParameters)
 		{
-#if NET_2_0
 			const string bindingflags_arg = "bindingFlags";
-#else
-			const string bindingflags_arg = "invokeAttr";
-#endif
 
 
 			if ((invokeAttr & BindingFlags.CreateInstance) != 0) {
@@ -351,22 +436,10 @@ namespace System
 				invokeAttr |= BindingFlags.Static|BindingFlags.Instance;
 
 			if (binder == null)
-				binder = Binder.DefaultBinder;
+				binder = DefaultBinder;
+
 			if ((invokeAttr & BindingFlags.CreateInstance) != 0) {
-				/* the name is ignored */
-				invokeAttr |= BindingFlags.DeclaredOnly;
-				ConstructorInfo[] ctors = GetConstructors (invokeAttr);
-				object state = null;
-				MethodBase ctor = binder.BindToMethod (invokeAttr, ctors, ref args, modifiers, culture, namedParameters, out state);
-				if (ctor == null) {
-					if (this.IsValueType && args == null)
-						return Activator.CreateInstanceInternal (this);
-					
-					throw new MissingMethodException ("Constructor on type '" + FullName + "' not found.");
-				}
-				object result = ctor.Invoke (target, invokeAttr, binder, args, culture);
-				binder.ReorderArgumentArray (ref args, state);
-				return result;
+				return Activator.CreateInstance (this, invokeAttr, binder, args, culture);
 			}
 			if (name == String.Empty && Attribute.IsDefined (this, typeof (DefaultMemberAttribute))) {
 				DefaultMemberAttribute attr = (DefaultMemberAttribute) Attribute.GetCustomAttribute (this, typeof (DefaultMemberAttribute));
@@ -380,7 +453,7 @@ namespace System
 				MethodInfo[] methods = GetMethodsByName (name, invokeAttr, ignoreCase, this);
 				object state = null;
 				if (args == null)
-					args = new object [0];
+					args = EmptyArray<object>.Value;
 				MethodBase m = binder.BindToMethod (invokeAttr, methods, ref args, modifiers, culture, namedParameters, out state);
 				if (m == null) {
 					if (methods.Length > 0)
@@ -388,15 +461,11 @@ namespace System
 					else
 						throwMissingMethodDescription = "Cannot find method " + name + ".";
 				} else {
-					ParameterInfo[] parameters = m.GetParameters();
+					ParameterInfo[] parameters = m.GetParametersInternal();
 					for (int i = 0; i < parameters.Length; ++i) {
 						if (System.Reflection.Missing.Value == args [i] && (parameters [i].Attributes & ParameterAttributes.HasDefault) != ParameterAttributes.HasDefault)
 							throw new ArgumentException ("Used Missing.Value for argument without default value", "parameters");
 					}
-					bool hasParamArray = parameters.Length > 0 ? Attribute.IsDefined (parameters [parameters.Length - 1], 
-						typeof (ParamArrayAttribute)) : false;
-					if (hasParamArray)
-						ReorderParamArrayArguments (ref args, m);
 					object result = m.Invoke (target, invokeAttr, binder, args, culture);
 					binder.ReorderArgumentArray (ref args, state);
 					return result;
@@ -413,10 +482,8 @@ namespace System
 			} else if ((invokeAttr & BindingFlags.SetField) != 0) {
 				FieldInfo f = GetField (name, invokeAttr);
 				if (f != null) {
-#if NET_2_0
 					if (args == null)
 						throw new ArgumentNullException ("providedArgs");
-#endif
 					if ((args == null) || args.Length != 1)
 						throw new ArgumentException ("Only the field value can be specified to set a field value.", bindingflags_arg);
 					f.SetValue (target, args [0]);
@@ -445,11 +512,6 @@ namespace System
 				if (m == null) {
 					throwMissingFieldException = true;
 				} else {
-					ParameterInfo[] parameters = m.GetParameters();
-					bool hasParamArray = parameters.Length > 0 ? Attribute.IsDefined (parameters [parameters.Length - 1], 
-						typeof (ParamArrayAttribute)) : false;
-					if (hasParamArray)
-						ReorderParamArrayArguments (ref args, m);
 					object result = m.Invoke (target, invokeAttr, binder, args, culture);
 					binder.ReorderArgumentArray (ref args, state);
 					return result;
@@ -473,11 +535,6 @@ namespace System
 				if (m == null) {
 					throwMissingFieldException = true;
 				} else {
-					ParameterInfo[] parameters = m.GetParameters();
-					bool hasParamArray = parameters.Length > 0 ? Attribute.IsDefined (parameters [parameters.Length - 1], 
-						typeof (ParamArrayAttribute)) : false;
-					if (hasParamArray)
-						ReorderParamArrayArguments (ref args, m);
 					object result = m.Invoke (target, invokeAttr, binder, args, culture);
 					binder.ReorderArgumentArray (ref args, state);
 					return result;
@@ -522,7 +579,14 @@ namespace System
 
 		public override string FullName {
 			get {
-				return getFullName (true, false);
+				string fullName;
+				// This doesn't need locking
+				if (type_info == null)
+					type_info = new MonoTypeInfo ();
+				if ((fullName = type_info.full_name) == null)
+					fullName = type_info.full_name = getFullName (true, false);
+
+				return fullName;
 			}
 		}
 
@@ -557,7 +621,7 @@ namespace System
 
 		public override MemberTypes MemberType {
 			get {
-				if (DeclaringType != null)
+				if (DeclaringType != null && !IsGenericParameter)
 					return MemberTypes.NestedType;
 				else
 					return MemberTypes.TypeInfo;
@@ -601,7 +665,7 @@ namespace System
 
 		public void GetObjectData(SerializationInfo info, StreamingContext context)
 		{
-			UnitySerializationHolder.GetTypeData (this, info, context);
+			UnitySerializationHolder.GetUnitySerializationInfo(info, this);
 		}
 
 		public override string ToString()
@@ -609,7 +673,6 @@ namespace System
 			return getFullName (false, false);
 		}
 
-#if NET_2_0 || BOOTSTRAP_NET_2_0
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern override Type [] GetGenericArguments ();
 
@@ -640,9 +703,28 @@ namespace System
 			[MethodImplAttribute(MethodImplOptions.InternalCall)]
 			get;
 		}
-#endif
 
-		private MethodBase CheckMethodSecurity (MethodBase mb)
+		public override Type GetGenericTypeDefinition () {
+			Type res = GetGenericTypeDefinition_impl ();
+			if (res == null)
+				throw new InvalidOperationException ();
+
+			return res;
+		}
+
+		public override IList<CustomAttributeData> GetCustomAttributesData () {
+			return CustomAttributeData.GetCustomAttributes (this);
+		}
+
+
+		public override Array GetEnumValues () {
+			if (!IsEnum)
+				throw new ArgumentException ("Type is not an enumeration", "enumType");
+
+			return Enum.GetValues (this);
+		}
+
+		static MethodBase CheckMethodSecurity (MethodBase mb)
 		{
 #if NET_2_1
 			return mb;
@@ -660,23 +742,41 @@ namespace System
 #endif
 		}
 
-		void ReorderParamArrayArguments(ref object[] args, MethodBase method)
+		//seclevel { transparent = 0, safe-critical = 1, critical = 2}
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		public extern int get_core_clr_security_level ();
+
+		public override bool IsSecurityTransparent
 		{
-			ParameterInfo[] parameters = method.GetParameters();
-			object[] newArgs = new object [parameters.Length];
-			Array paramArray = Array.CreateInstance(parameters[parameters.Length - 1].ParameterType.GetElementType(), 
-				args.Length - (parameters.Length - 1));
-			int paramArrayCount = 0;
-			for (int i = 0; i < args.Length; i++) {
-				if (i < (parameters.Length - 1))
-					newArgs [i] = args [i];
-				else {
-					paramArray.SetValue (args [i], paramArrayCount);
-					paramArrayCount ++;
-				}
+			get { return get_core_clr_security_level () == 0; }
+		}
+
+		public override bool IsSecurityCritical
+		{
+			get { return get_core_clr_security_level () > 0; }
+		}
+
+		public override bool IsSecuritySafeCritical
+		{
+			get { return get_core_clr_security_level () == 1; }
+		}
+
+		public override StructLayoutAttribute StructLayoutAttribute {
+			get {
+				return GetStructLayoutAttribute ();
+			}
+		}
+
+		internal override bool IsUserType {
+			get {
+				return false;
+			}
+		}
+
+		public override bool IsConstructedGenericType {
+			get {
+				return IsGenericType && !ContainsGenericParameters;
 			}
-			newArgs [parameters.Length - 1] = paramArray;
-			args = newArgs;
 		}
 	}
 }
