@@ -1,8 +1,9 @@
 //
-// Predicate.cs
+// System.Predicate.cs
 //
 // Authors:
 //  Ben Maurer (bmaurer@ximian.com)
+//  Marek Safar (marek.safar@gmail.com)
 //
 // Copyright (C) 2004 Novell
 //
@@ -25,8 +26,9 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-#if NET_2_0
-namespace System {
-	public delegate bool Predicate <T> (T value);
+
+namespace System
+{
+	public delegate bool Predicate <in T> (T obj);
 }
-#endif
+
