@@ -33,11 +33,14 @@
 
 using System.Diagnostics;
 using System.Reflection;
+#if !FULL_AOT_RUNTIME
 using System.Reflection.Emit;
+#endif
 using System.Collections;
 using System.Collections.Generic;
 using System.Runtime.InteropServices;
 using System.Runtime.CompilerServices;
+using System.Threading;
 using System.Globalization;
 
 namespace System {
@@ -46,8 +49,13 @@ namespace System {
 	[ClassInterface (ClassInterfaceType.None)]
 	[ComVisible (true)]
 	[ComDefaultInterface (typeof (_Type))]
+	[StructLayout (LayoutKind.Sequential)]
+#if MOBILE
+	public abstract class Type : MemberInfo, IReflect {
+#else
 	public abstract class Type : MemberInfo, IReflect, _Type {
-		
+#endif
+
 		internal RuntimeTypeHandle _impl;
 
 		public static readonly char Delimiter = '.';
@@ -66,11 +74,11 @@ namespace System {
 			string name = (string) filterCriteria;
 			if (name == null || name.Length == 0 )
 				return false; // because m.Name cannot be null or empty
-				
-			if (name [name.Length-1] == '*')
-				return string.CompareOrdinal (name, 0, m.Name, 0, name.Length-1) == 0;
-
-           	return name.Equals (m.Name);            	
+			
+			if (name [name.Length - 1] == '*')
+				return m.Name.StartsWithOrdinalUnchecked (name.Substring (0, name.Length - 1));
+			
+			return m.Name == name;
 		}
 
 		static bool FilterNameIgnoreCase_impl (MemberInfo m, object filterCriteria)
@@ -79,15 +87,18 @@ namespace System {
 			if (name == null || name.Length == 0 )
 				return false; // because m.Name cannot be null or empty
 				
-			if (name [name.Length-1] == '*')
-				return string.Compare (name, 0, m.Name, 0, name.Length-1, StringComparison.OrdinalIgnoreCase) == 0;
-
-			return string.Equals (name, m.Name, StringComparison.OrdinalIgnoreCase);
+			if (name [name.Length - 1] == '*')
+				return m.Name.StartsWithOrdinalCaseInsensitiveUnchecked (name.Substring (0, name.Length - 1));
+			
+			return string.CompareOrdinalCaseInsensitiveUnchecked (m.Name, name) == 0;
 		}
 
 		static bool FilterAttribute_impl (MemberInfo m, object filterCriteria)
 		{
-			int flags = ((IConvertible)filterCriteria).ToInt32 (null);
+			if (!(filterCriteria is int))
+				throw new InvalidFilterCriteriaException ("Int32 value is expected for filter criteria");
+
+			int flags = (int) filterCriteria;
 			if (m is MethodInfo)
 				return ((int)((MethodInfo)m).Attributes & flags) != 0;
 			if (m is FieldInfo)
@@ -220,6 +231,12 @@ namespace System {
 				return IsCOMObjectImpl ();
 			}
 		}
+		
+		public virtual bool IsConstructedGenericType {
+			get {
+				throw new NotImplementedException ();
+			}
+		}
 
 		public bool IsContextful {
 			get {
@@ -228,9 +245,7 @@ namespace System {
 		}
 
 		public
-#if NET_4_0
 		virtual
-#endif
 		bool IsEnum {
 			get {
 				return IsSubclassOf (typeof (Enum));
@@ -334,9 +349,7 @@ namespace System {
 		}
 
 		public
-#if NET_4_0
 		virtual
-#endif
 		bool IsSerializable {
 			get {
 				if ((Attributes & TypeAttributes.Serializable) != 0)
@@ -424,20 +437,9 @@ namespace System {
 
 		public override bool Equals (object o)
 		{
-#if NET_4_0
 			return Equals (o as Type);
-#else
-			if (o == this)
-				return true;
-
-			Type me = UnderlyingSystemType;
-			if (me == null)
-				return false;
-			return me.EqualsInternal (o as Type);
-#endif
 		}
 
-#if NET_4_0
 		public virtual bool Equals (Type o)
 		{
 			if ((object)o == (object)this)
@@ -455,21 +457,6 @@ namespace System {
 				return true;
 			return me.EqualsInternal (o);
 		}		
-#else
-		public bool Equals (Type o)
-		{
-
-			if (o == this)
-				return true;
-			if (o == null)
-				return false;
-			Type me = UnderlyingSystemType;
-			if (me == null)
-				return false;
-			return me.EqualsInternal (o.UnderlyingSystemType);
-		}
-#endif
-#if NET_4_0
 		[MonoTODO ("Implement it properly once 4.0 impl details are known.")]
 		public static bool operator == (Type left, Type right)
 		{
@@ -502,11 +489,19 @@ namespace System {
 
 			var fields = GetFields (BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static);
 
-			string [] result = new string [fields.Length];
-			for (int i = 0; i < fields.Length; ++i)
-				result [i] = fields [i].Name;
+			string [] names = new string [fields.Length];
+			if (0 != names.Length) {
+				for (int i = 0; i < fields.Length; ++i)
+					names [i] = fields [i].Name;
+					
+				var et = GetEnumUnderlyingType ();
+				var values = Array.CreateInstance (et, names.Length);
+				for (int i = 0; i < fields.Length; ++i)
+					values.SetValue (fields [i].GetValue (null), i);
+				MonoEnumInfo.SortEnums (et, values, names);
+			}
 
-			return result;
+			return names;
 		}
 
 		static NotImplementedException CreateNIE () {
@@ -520,81 +515,14 @@ namespace System {
 			throw CreateNIE ();
 		}
 
-		bool IsValidEnumType (Type type) {
-			return (type.IsPrimitive && type != typeof (bool) && type != typeof (double) && type != typeof (float)) || type.IsEnum;
+		public virtual string GetEnumName (object value)
+		{
+			return Enum.GetName (this, value);
 		}
 
-		[MonoInternalNote ("Reimplement this in MonoType for bonus speed")]
-		public virtual string GetEnumName (object value) {
-			if (value == null)
-				throw new ArgumentException ("Value is null", "value");
-			if (!IsValidEnumType (value.GetType ()))
-				throw new ArgumentException ("Value is not the enum or a valid enum underlying type", "value");
-			if (!IsEnum)
-				throw new ArgumentException ("Type is not an enumeration", "enumType");
-
-			object obj = null;
-			var fields = GetFields (BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static);
-			
-			for (int i = 0; i < fields.Length; ++i) {
-				var fv = fields [i].GetValue (null);
-				if (obj == null) {
-					try {
-						//XXX we can't use 'this' as argument as it might be an UserType
-						obj = Enum.ToObject (fv.GetType (), value);
-					} catch (OverflowException) {
-						return null;
-					} catch (InvalidCastException) {
-						throw new ArgumentException ("Value is not valid", "value");
-					}
-				}
-				if (fv.Equals (obj))
-					return fields [i].Name;
-			}
-
-			return null;
-		}
-
-		[MonoInternalNote ("Reimplement this in MonoType for bonus speed")]
-		public virtual bool IsEnumDefined (object value) {
-			if (value == null)
-				throw new ArgumentException ("Value is null", "value");
-			if (!IsEnum)
-				throw new ArgumentException ("Type is not an enumeration", "enumType");
-
-			Type vt = value.GetType ();
-			if (!IsValidEnumType (vt) && vt != typeof (string))
-				throw new InvalidOperationException ("Value is not the enum or a valid enum underlying type");
-
-			var fields = GetFields (BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static);
-
-			if (value is string) {
-				for (int i = 0; i < fields.Length; ++i) {
-					if (fields [i].Name.Equals (value))
-						return true;
-				}
-			} else {
-				if (vt != this && vt != GetEnumUnderlyingType ())
-					throw new ArgumentException ("Value is not the enum or a valid enum underlying type", "value");
-
-				object obj = null;
-				for (int i = 0; i < fields.Length; ++i) {
-					var fv = fields [i].GetValue (null);
-					if (obj == null) {
-						try {
-							//XXX we can't use 'this' as argument as it might be an UserType
-							obj = Enum.ToObject (fv.GetType (), value);
-						} catch (OverflowException) {
-							return false;
-						} catch (InvalidCastException) {
-							throw new ArgumentException ("Value is not valid", "value");
-						}
-					}
-					if (fv.Equals (obj))
-						return true;
-				}
-			}
-			return false;
+		public virtual bool IsEnumDefined (object value)
+		{
+			return Enum.IsDefined (this, value);
 		}
 	
 		public static Type GetType (string typeName, Func<AssemblyName,Assembly> assemblyResolver, Func<Assembly,string,bool,Type> typeResolver)
@@ -627,7 +555,6 @@ namespace System {
 		{
 			get { throw CreateNIE (); }
 		}
-#endif
 		
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		internal extern bool EqualsInternal (Type type);
@@ -684,15 +611,15 @@ namespace System {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		internal extern static TypeCode GetTypeCodeInternal (Type type);
 
-#if NET_4_0
 		protected virtual
-#endif
 		TypeCode GetTypeCodeImpl () {
 			Type type = this;
 			if (type is MonoType)
 				return GetTypeCodeInternal (type);
+#if !FULL_AOT_RUNTIME
 			if (type is TypeBuilder)
 				return ((TypeBuilder)type).GetTypeCodeInternal ();
+#endif
 
 			type = type.UnderlyingSystemType;
 
@@ -709,28 +636,82 @@ namespace System {
 			return type.GetTypeCodeImpl ();
 		}
 
-		[MonoTODO("This operation is currently not supported by Mono")]
+#if !FULL_AOT_RUNTIME
+		private static Dictionary<Guid, Type> clsid_types;
+		private static AssemblyBuilder clsid_assemblybuilder;
+#endif
+
+		[MonoTODO("COM servers only work on Windows")]
 		public static Type GetTypeFromCLSID (Guid clsid)
 		{
-			throw new NotImplementedException ();
+			return GetTypeFromCLSID (clsid, null, true);
 		}
 
-		[MonoTODO("This operation is currently not supported by Mono")]
+		[MonoTODO("COM servers only work on Windows")]
 		public static Type GetTypeFromCLSID (Guid clsid, bool throwOnError)
 		{
-			throw new NotImplementedException ();
+			return GetTypeFromCLSID (clsid, null, throwOnError);
 		}
 
-		[MonoTODO("This operation is currently not supported by Mono")]
+		[MonoTODO("COM servers only work on Windows")]
 		public static Type GetTypeFromCLSID (Guid clsid, string server)
 		{
-			throw new NotImplementedException ();
+			return GetTypeFromCLSID (clsid, server, true);
 		}
 
-		[MonoTODO("This operation is currently not supported by Mono")]
+		[MonoTODO("COM servers only work on Windows")]
 		public static Type GetTypeFromCLSID (Guid clsid, string server, bool throwOnError)
 		{
+#if !FULL_AOT_RUNTIME
+			Type result;
+
+			if (clsid_types == null)
+			{
+				Dictionary<Guid, Type> new_clsid_types = new Dictionary<Guid, Type> ();
+				Interlocked.CompareExchange<Dictionary<Guid, Type>>(
+					ref clsid_types, new_clsid_types, null);
+			}
+
+			lock (clsid_types) {
+				if (clsid_types.TryGetValue(clsid, out result))
+					return result;
+
+				if (clsid_assemblybuilder == null)
+				{
+					AssemblyName assemblyname = new AssemblyName ();
+					assemblyname.Name = "GetTypeFromCLSIDDummyAssembly";
+					clsid_assemblybuilder = AppDomain.CurrentDomain.DefineDynamicAssembly (
+						assemblyname, AssemblyBuilderAccess.Run);
+				}
+				ModuleBuilder modulebuilder = clsid_assemblybuilder.DefineDynamicModule (
+					clsid.ToString ());
+
+				TypeBuilder typebuilder = modulebuilder.DefineType ("System.__ComObject",
+					TypeAttributes.Public | TypeAttributes.Class, typeof(System.__ComObject));
+
+				Type[] guidattrtypes = new Type[] { typeof(string) };
+
+				CustomAttributeBuilder customattr = new CustomAttributeBuilder (
+					typeof(GuidAttribute).GetConstructor (guidattrtypes),
+					new object[] { clsid.ToString () });
+
+				typebuilder.SetCustomAttribute (customattr);
+
+				customattr = new CustomAttributeBuilder (
+					typeof(ComImportAttribute).GetConstructor (EmptyTypes),
+					new object[0] {});
+
+				typebuilder.SetCustomAttribute (customattr);
+
+				result = typebuilder.CreateType ();
+
+				clsid_types.Add(clsid, result);
+
+				return result;
+			}
+#else
 			throw new NotImplementedException ();
+#endif
 		}
 
 		public static Type GetTypeFromHandle (RuntimeTypeHandle handle)
@@ -828,25 +809,9 @@ namespace System {
 		internal static extern void GetInterfaceMapData (Type t, Type iface, out MethodInfo[] targets, out MethodInfo[] methods);
 
 		[ComVisible (true)]
-		public virtual InterfaceMapping GetInterfaceMap (Type interfaceType) {
-			if (!IsSystemType)
-				throw new NotSupportedException ("Derived classes must provide an implementation.");
-			if (!interfaceType.IsSystemType)
-				throw new ArgumentException ("interfaceType", "Type is an user type");
-			InterfaceMapping res;
-			if (interfaceType == null)
-				throw new ArgumentNullException ("interfaceType");
-			if (!interfaceType.IsInterface)
-				throw new ArgumentException (Locale.GetText ("Argument must be an interface."), "interfaceType");
-			if (IsInterface)
-				throw new ArgumentException ("'this' type cannot be an interface itself");
-			res.TargetType = this;
-			res.InterfaceType = interfaceType;
-			GetInterfaceMapData (this, interfaceType, out res.TargetMethods, out res.InterfaceMethods);
-			if (res.TargetMethods == null)
-				throw new ArgumentException (Locale.GetText ("Interface not found"), "interfaceType");
-
-			return res;
+		public virtual InterfaceMapping GetInterfaceMap (Type interfaceType)
+		{
+			throw new NotSupportedException ();
 		}
 
 		public abstract Type[] GetInterfaces ();
@@ -859,8 +824,10 @@ namespace System {
 			if (Equals (c))
 				return true;
 
+#if !FULL_AOT_RUNTIME
 			if (c is TypeBuilder)
 				return ((TypeBuilder)c).IsAssignableTo (this);
+#endif
 
 			/* Handle user defined type classes */
 			if (!IsSystemType) {
@@ -1208,10 +1175,10 @@ namespace System {
 		{
 			object [] att = GetCustomAttributes (typeof (DefaultMemberAttribute), true);
 			if (att.Length == 0)
-				return new MemberInfo [0];
+				return EmptyArray<MemberInfo>.Value;
 
 			MemberInfo [] member = GetMember (((DefaultMemberAttribute) att [0]).MemberName);
-			return (member != null) ? member : new MemberInfo [0];
+			return (member != null) ? member : EmptyArray<MemberInfo>.Value;
 		}
 
 		public virtual MemberInfo[] FindMembers (MemberTypes memberType, BindingFlags bindingAttr,
@@ -1220,8 +1187,6 @@ namespace System {
 			MemberInfo[] result;
 			ArrayList l = new ArrayList ();
 
-			// Console.WriteLine ("FindMembers for {0} (Type: {1}): {2}",
-			// this.FullName, this.GetType().FullName, this.obj_address());
 			if ((memberType & MemberTypes.Method) != 0) {
 				MethodInfo[] c = GetMethods (bindingAttr);
 				if (filter != null) {
@@ -1346,11 +1311,15 @@ namespace System {
 			return FullName;
 		}
 
-		internal virtual bool IsCompilerContext {
-			get {
-				AssemblyBuilder builder = Assembly as AssemblyBuilder;
-				return builder != null && builder.IsCompilerContext;
-			}
+		internal static bool ShouldPrintFullName (Type type)
+		{
+			while (type.HasElementType)
+				type = type.GetElementType ();
+
+			if (type == typeof (void) || type.IsNested)
+				return false;
+
+			return !type.IsPrimitive;
 		}
 
 		internal virtual Type InternalResolve ()
@@ -1363,6 +1332,12 @@ namespace System {
 				return _impl.Value != IntPtr.Zero;
 			}
 		}
+		
+		public virtual Type[] GenericTypeArguments {
+			get {
+				return IsGenericType ? GetGenericArguments () : EmptyTypes;
+			}
+		}
 
 		public virtual Type[] GetGenericArguments ()
 		{
@@ -1390,29 +1365,10 @@ namespace System {
 			[MethodImplAttribute(MethodImplOptions.InternalCall)]
 			get;
 		}
-
+		
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		static extern Type MakeGenericType (Type gt, Type [] types);
 
-		static AssemblyBuilder PeelAssemblyBuilder (Type type)
-		{
-			if (type.Assembly is AssemblyBuilder)
-				return (AssemblyBuilder)type.Assembly;
-
-			if (type.HasElementType)
-				return PeelAssemblyBuilder (type.GetElementType ());
-
-			if (!type.IsGenericType || type.IsGenericParameter || type.IsGenericTypeDefinition)
-				return null;
-
-			foreach (Type arg in type.GetGenericArguments ()) {
-				AssemblyBuilder ab = PeelAssemblyBuilder (arg);
-				if (ab != null)
-					return ab;
-			}
-			return null;
-		}
-
 		public virtual Type MakeGenericType (params Type[] typeArguments)
 		{
 			if (IsUserType)
@@ -1425,7 +1381,6 @@ namespace System {
 				throw new ArgumentException (String.Format ("The type or method has {0} generic parameter(s) but {1} generic argument(s) where provided. A generic argument must be provided for each generic parameter.", GetGenericArguments ().Length, typeArguments.Length), "typeArguments");
 
 			bool hasUserType = false;
-			AssemblyBuilder compilerContext = null;
 
 			Type[] systemTypes = new Type[typeArguments.Length];
 			for (int i = 0; i < typeArguments.Length; ++i) {
@@ -1435,15 +1390,15 @@ namespace System {
 
 				if (!(t is MonoType))
 					hasUserType = true;
-				if (t.IsCompilerContext)
-					compilerContext = PeelAssemblyBuilder (t);
 				systemTypes [i] = t;
 			}
 
 			if (hasUserType) {
-				if (compilerContext != null)
-					return compilerContext.MakeGenericType (this, typeArguments);
+#if FULL_AOT_RUNTIME
+				throw new NotSupportedException ("User types are not supported under full aot");
+#else
 				return new MonoGenericClass (this, typeArguments);
+#endif
 			}
 
 			Type res = MakeGenericType (this, systemTypes);
@@ -1587,11 +1542,7 @@ namespace System {
 
 		public virtual StructLayoutAttribute StructLayoutAttribute {
 			get {
-#if NET_4_0
 				throw new NotSupportedException ();
-#else
-				return GetStructLayoutAttribute ();
-#endif
 			}
 		}
 		
@@ -1650,28 +1601,34 @@ namespace System {
 		}			
 
 
-#if NET_4_0
 		public virtual bool IsEquivalentTo (Type other)
 		{
 			return this == other;
 		}
-#endif
 
 		/* 
 		 * Return whenever this object is an instance of a user defined subclass
 		 * of System.Type or an instance of TypeDelegator.
+		 * A user defined type is not simply the opposite of a system type.
+		 * It's any class that's neither a SRE or runtime baked type.
 		 */
-		internal bool IsUserType {
+		internal virtual bool IsUserType {
 			get {
-				/* 
-				 * subclasses cannot modify _impl so if it is zero, it means the
-				 * type is not created by the runtime.
-				 */
-				return _impl.Value == IntPtr.Zero &&
-					(GetType ().Assembly != typeof (Type).Assembly || GetType () == typeof (TypeDelegator));
+				return true;
 			}
 		}
 
+		internal Type GetRootElementType()
+		{
+			Type rootElementType = this;
+
+			while (rootElementType.HasElementType)
+				rootElementType = rootElementType.GetElementType();
+
+			return rootElementType;
+		}
+
+#if !MOBILE
 		void _Type.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
 		{
 			throw new NotImplementedException ();
@@ -1691,5 +1648,6 @@ namespace System {
 		{
 			throw new NotImplementedException ();
 		}
+#endif
 	}
 }
