@@ -7,12 +7,15 @@
 // 
 // (C) Ville Palo
 // (c) 2003 Ximian, Inc. (http://www.ximian.com)
+// Copyright 2011 Xamarin Inc (http://www.xamarin.com).
 // 
 
 using NUnit.Framework;
 using System;
 using System.IO;
+using System.Runtime.InteropServices;
 using System.Text;
+using System.Threading;
 
 namespace MonoTests.System.IO
 {
@@ -21,6 +24,10 @@ namespace MonoTests.System.IO
 	{
 		string TempFolder = Path.Combine (Path.GetTempPath (), "MonoTests.System.IO.Tests");
 		static readonly char DSC = Path.DirectorySeparatorChar;
+		static bool MacOSX = false;
+
+		[DllImport ("libc")]
+		static extern int uname (IntPtr buf);
 
 		[TearDown]
 		public void TearDown ()
@@ -36,6 +43,13 @@ namespace MonoTests.System.IO
 				Directory.Delete (TempFolder, true);
 
 			Directory.CreateDirectory (TempFolder);
+#if !MOBILE			
+			// from XplatUI.cs
+			IntPtr buf = Marshal.AllocHGlobal (8192);
+			if (uname (buf) == 0)
+				MacOSX = Marshal.PtrToStringAnsi (buf) == "Darwin";
+			Marshal.FreeHGlobal (buf);
+#endif
 		}
 
 		public void TestCtr ()
@@ -102,6 +116,9 @@ namespace MonoTests.System.IO
 				Assert.Fail ("#B1");
 			} catch (FileNotFoundException ex) {
 				Assert.AreEqual (typeof (FileNotFoundException), ex.GetType (), "#B2");
+				// under OSX 'var' is a symlink to 'private/var'
+				if (MacOSX)
+					path = "/private" + path;
 				Assert.AreEqual (path, ex.FileName, "#B3");
 				Assert.IsNull (ex.InnerException, "#B4");
 				Assert.IsNotNull (ex.Message, "#B5");
@@ -147,6 +164,9 @@ namespace MonoTests.System.IO
 				Assert.Fail ("#B1");
 			} catch (FileNotFoundException ex) {
 				Assert.AreEqual (typeof (FileNotFoundException), ex.GetType (), "#B2");
+				// under OSX 'var' is a symlink to 'private/var'
+				if (MacOSX)
+					path = "/private" + path;
 				Assert.AreEqual (path, ex.FileName, "#B3");
 				Assert.IsNull (ex.InnerException, "#B4");
 				Assert.IsNotNull (ex.Message, "#B5");
@@ -324,14 +344,8 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
-#if NET_2_0
 		// FileShare.Inheritable is ignored, but file does not exist
 		[ExpectedException (typeof (FileNotFoundException))]
-#else
-		// share: Enum value was out of legal range.
-		// (FileShare.Inheritable is not valid)
-		[ExpectedException (typeof (ArgumentOutOfRangeException))]
-#endif
 		public void CtorArgumentOutOfRangeException3 ()
 		{
 			string path = TempFolder + DSC + "CtorArgumentOutOfRangeException1";
@@ -406,14 +420,8 @@ namespace MonoTests.System.IO
 
 
 		[Test]
-#if NET_2_0
 		// FileShare.Inheritable is ignored, but file does not exist
 		[ExpectedException (typeof (FileNotFoundException))]
-#else
-		// share: Enum value was out of legal range.
-		// (FileShare.Inheritable is not valid)
-		[ExpectedException (typeof (ArgumentOutOfRangeException))]
-#endif
 		public void CtorArgumentOutOfRangeException5 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "temp";
@@ -698,7 +706,6 @@ namespace MonoTests.System.IO
 			}
 		}
 
-#if !TARGET_JVM // No support IntPtr file handles under TARGET_JVM
 		[Test, ExpectedException (typeof (IOException))]
 		public void CtorIOException2 ()
 		{
@@ -710,9 +717,7 @@ namespace MonoTests.System.IO
 					stream.Close ();
 			}
 		}
-#endif // TARGET_JVM
 
-		[Category("TargetJvmNotSupported")] // File sharing not supported for TARGET_JVM
 		[Test, ExpectedException (typeof (IOException))]
 		public void CtorIOException ()
 		{
@@ -760,7 +765,6 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
-		[Category("TargetJvmNotSupported")] // File sharing not supported for TARGET_JVM
 		[ExpectedException (typeof (IOException))]
 		public void CtorAccess1Read2Write ()
 		{
@@ -786,7 +790,6 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
-		[Category("TargetJvmNotSupported")] // File sharing not supported for TARGET_JVM
 		[ExpectedException (typeof (IOException))]
 		public void CtorAccess1Write2Write ()
 		{
@@ -821,6 +824,37 @@ namespace MonoTests.System.IO
 			}
 		}
 
+		[Test] // bug #79250
+		public void FileShare_Delete ()
+		{
+			string fn = Path.Combine (TempFolder, "temp");
+			
+			using (Stream s = new FileStream (fn, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Delete)) {
+				s.Write (new byte [1] { 0x5 }, 0, 1);
+				File.Delete (fn);
+			}
+
+			using (Stream s = new FileStream (fn, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite | FileShare.Delete)) {
+				s.Write (new byte [1] { 0x5 }, 0, 1);
+				File.Delete (fn);
+			}
+
+			using (Stream s = new FileStream (fn, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Write | FileShare.Delete)) {
+				s.Write (new byte [1] { 0x5 }, 0, 1);
+				File.Delete (fn);
+			}
+
+			using (Stream s = new FileStream (fn, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Read | FileShare.Delete)) {
+				s.Write (new byte [1] { 0x5 }, 0, 1);
+				File.Delete (fn);
+			}
+
+			using (Stream s = new FileStream (fn, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Inheritable | FileShare.Delete)) {
+				s.Write (new byte [1] { 0x5 }, 0, 1);
+				File.Delete (fn);
+			}
+		}
+
 		[Test]
 		public void Write ()
 		{
@@ -983,7 +1017,6 @@ namespace MonoTests.System.IO
 			DeleteFile (path);
 		}
 
-		[Category("TargetJvmNotSupported")] // File locking not supported for TARGET_JVM
 		public void TestLock ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "TestLock";
@@ -1272,7 +1305,6 @@ namespace MonoTests.System.IO
 			}
 		}
 
-#if !TARGET_JVM // No support IntPtr file handles under TARGET_JVM
 		// Check that the stream is flushed even when it doesn't own the
 		// handle
 		[Test]
@@ -1292,7 +1324,6 @@ namespace MonoTests.System.IO
 			Assert.AreEqual ((int) '1', s.ReadByte ());
 			s.Close ();
 		}
-#endif // TARGET_JVM
 
 		private void DeleteFile (string path)
 		{
@@ -1408,14 +1439,12 @@ namespace MonoTests.System.IO
 			}
 		}
 
-#if !TARGET_JVM // No support IntPtr file handles under TARGET_JVM
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
 		public void Constructor_InvalidFileHandle ()
 		{
 			new FileStream ((IntPtr) (-1L), FileAccess.Read);
 		}
-#endif // TARGET_JVM
 
 		[Test]
 		public void PositionAfterSetLength ()
@@ -1454,7 +1483,6 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
-		[Category("TargetJvmNotSupported")] // Async IO not supported for TARGET_JVM
 		[ExpectedException (typeof (ObjectDisposedException))]
 		public void BeginRead_Disposed ()
 		{
@@ -1466,7 +1494,6 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
-		[Category("TargetJvmNotSupported")] // Async IO not supported for TARGET_JVM
 		[ExpectedException (typeof (ObjectDisposedException))]
 		public void BeginWrite_Disposed ()
 		{
@@ -1477,8 +1504,36 @@ namespace MonoTests.System.IO
 			stream.EndWrite (stream.BeginWrite (new byte[8], 0, 8, null, null));
 		}
 
+		static IAsyncResult DoBeginWrite(Stream stream, ManualResetEvent mre, byte[] RandomBuffer)
+		{
+			return stream.BeginWrite (RandomBuffer, 0, RandomBuffer.Length, ar => {
+				stream.EndWrite (ar);
+
+				// we don't supply an ManualResetEvent so this will throw an NRE on the second run
+				// which nunit-console will ignore (but other test runners don't like that)
+				if (mre == null)
+					return;
+
+				DoBeginWrite (stream, null, RandomBuffer).AsyncWaitHandle.WaitOne ();
+				mre.Set ();
+			}, null);
+		}
+
+		[Test]
+		public void BeginWrite_Recursive ()
+		{
+			string path = TempFolder + Path.DirectorySeparatorChar + "temp";
+			DeleteFile (path);
+	
+			using (FileStream stream = new FileStream (path, FileMode.OpenOrCreate, FileAccess.Write)) {
+				var mre = new ManualResetEvent (false);	
+				var RandomBuffer = new byte[1024];			
+				DoBeginWrite (stream, mre, RandomBuffer);
+				Assert.IsTrue (mre.WaitOne (5000), "#1");
+			}
+		}
+
 		[Test]
-		[Category("TargetJvmNotSupported")] // File locking not supported for TARGET_JVM
 		[ExpectedException (typeof (ObjectDisposedException))]
 		public void Lock_Disposed ()
 		{
@@ -1490,7 +1545,6 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
-		[Category("TargetJvmNotSupported")] // File locking not supported for TARGET_JVM
 		[ExpectedException (typeof (ObjectDisposedException))]
 		public void Unlock_Disposed ()
 		{
@@ -1553,9 +1607,8 @@ namespace MonoTests.System.IO
 			}
 		}
 
-#if NET_2_0
-		[Category("TargetJvmNotSupported")] // FileOptions.DeleteOnClose not supported for TARGET_JVM
-		[Test] public void DeleteOnClose ()
+		[Test]
+		public void DeleteOnClose ()
 		{
 			string path = TempFolder + DSC + "created.txt";
 			DeleteFile (path);
@@ -1566,7 +1619,41 @@ namespace MonoTests.System.IO
 			Assert.AreEqual (false, File.Exists (path), "DOC#2");
 			
 		}
+
+#if !MOBILE
+		[Test]
+		public void WriteWithExposedHandle ()
+		{
+			string path = TempFolder + DSC + "exposed-handle.txt";
+			FileStream fs1 = null;
+			FileStream fs2 = null;
+			DeleteFile (path);
+			
+			try {
+				fs1 = new FileStream (path, FileMode.Create);
+				fs2 = new FileStream (fs1.SafeFileHandle, FileAccess.ReadWrite);
+				fs1.WriteByte (Convert.ToByte ('H'));
+				fs1.WriteByte (Convert.ToByte ('E'));
+				fs1.WriteByte (Convert.ToByte ('L'));
+				fs2.WriteByte (Convert.ToByte ('L'));
+				fs2.WriteByte (Convert.ToByte ('O'));
+				long fs1Pos = fs1.Position;
+				fs1.Flush ();
+				fs2.Flush (); 
+				fs1.Close ();
+				fs2.Close ();
+
+				var check = Encoding.ASCII.GetString (File.ReadAllBytes (path));
+				Assert.AreEqual ("HELLO", check, "EXPOSED#1");
+				Assert.AreEqual (5, fs1Pos, "EXPOSED#2");
+			} finally {
+				if (fs1 != null)
+					fs1.Close ();
+				if (fs2 != null)
+					fs2.Close ();
+				DeleteFile (path);
+			}
+		}
 #endif
 	}
 }
-
