@@ -2,8 +2,10 @@
 // AssemblyBuilderTest.cs - NUnit Test Cases for the AssemblyBuilder class
 //
 // Zoltan Varga (vargaz@freemail.hu)
+// Andres G. Aragoneses (andres@7digital.com)
 //
 // (C) Ximian, Inc.  http://www.ximian.com
+// (C) 7digital Media, Ltd. http://www.7digital.com
 //
 //
 
@@ -76,12 +78,16 @@ public class AssemblyBuilderTest
 		return assemblyName;
 	}
 
-	private AssemblyBuilder genAssembly ()
+	private AssemblyBuilder genAssembly (AssemblyBuilderAccess access)
 	{
 		return domain.DefineDynamicAssembly (genAssemblyName (),
-						     AssemblyBuilderAccess.RunAndSave,
+						     access,
 						     tempDir);
 	}
+	private AssemblyBuilder genAssembly ()
+	{
+		return genAssembly (AssemblyBuilderAccess.RunAndSave);
+	}
 
 	private MethodInfo genEntryFunction (AssemblyBuilder assembly)
 	{
@@ -93,7 +99,6 @@ public class AssemblyBuilderTest
 		return mb;
 	}
 
-#if NET_2_0
 	[Test]
 	[Category ("NotWorking")]
 	public void ManifestModule ()
@@ -118,7 +123,6 @@ public class AssemblyBuilderTest
 		Assert.AreSame (mb2, ab.ManifestModule, "#C3");
 		Assert.AreSame (mb1, mb2, "#C4");
 	}
-#endif
 
 	[Test]
 	[ExpectedException (typeof (NotSupportedException))]
@@ -276,14 +280,12 @@ public class AssemblyBuilderTest
 		Assert.IsNull (ab.GetDynamicModule ("FOO4"));
 	}
 
-#if NET_1_1
 	[Test]
 	public void TestImageRuntimeVersion ()
 	{
 		string version = ab.ImageRuntimeVersion;
 		Assert.IsTrue (version.Length > 0);
 	}
-#endif
 
 	[Test]
 	public void TestAddResourceFile_Name_Null ()
@@ -779,6 +781,10 @@ public class AssemblyBuilderTest
 		try {
 			ab.Save ("lib.dll");
 			Assert.Fail ("#A1");
+#if NET_4_0
+		} catch (CultureNotFoundException ex) {
+		}
+#else
 		} catch (ArgumentException ex) {
 			// Culture name doesnotexist is not supported
 			Assert.AreEqual (typeof (ArgumentException), ex.GetType (), "#A2");
@@ -787,6 +793,7 @@ public class AssemblyBuilderTest
 			Assert.IsTrue (ex.Message.IndexOf ("doesnotexist") != -1, "#A5");
 			Assert.AreEqual ("name", ex.ParamName, "#A6");
 		}
+#endif
 
 		ab = AppDomain.CurrentDomain.DefineDynamicAssembly (aname,
 			AssemblyBuilderAccess.RunAndSave, tempDir);
@@ -802,6 +809,10 @@ public class AssemblyBuilderTest
 		try {
 			ab.Save ("lib.dll");
 			Assert.Fail ("#B1");
+#if NET_4_0
+		} catch (CultureNotFoundException ex) {
+		}
+#else
 		} catch (ArgumentException ex) {
 			// Culture name neutral is not supported
 			Assert.AreEqual (typeof (ArgumentException), ex.GetType (), "#B2");
@@ -810,6 +821,7 @@ public class AssemblyBuilderTest
 			Assert.IsTrue (ex.Message.IndexOf ("neutral") != -1, "#B5");
 			Assert.AreEqual ("name", ex.ParamName, "#B6");
 		}
+#endif
 	}
 
 	[Test] // DefineVersionInfoResource ()
@@ -891,6 +903,10 @@ public class AssemblyBuilderTest
 		try {
 			ab.Save ("lib.dll");
 			Assert.Fail ("#A1");
+#if NET_4_0
+		} catch (CultureNotFoundException ex) {
+		}
+#else
 		} catch (ArgumentException ex) {
 			// Culture name doesnotexist is not supported
 			Assert.AreEqual (typeof (ArgumentException), ex.GetType (), "#A2");
@@ -899,6 +915,7 @@ public class AssemblyBuilderTest
 			Assert.IsTrue (ex.Message.IndexOf ("doesnotexist") != -1, "#A5");
 			Assert.AreEqual ("name", ex.ParamName, "#A6");
 		}
+#endif
 
 		ab = AppDomain.CurrentDomain.DefineDynamicAssembly (aname,
 			AssemblyBuilderAccess.RunAndSave, tempDir);
@@ -914,6 +931,10 @@ public class AssemblyBuilderTest
 		try {
 			ab.Save ("lib.dll");
 			Assert.Fail ("#B1");
+#if NET_4_0
+		} catch (CultureNotFoundException ex) {
+		}
+#else
 		} catch (ArgumentException ex) {
 			// Culture name neutral is not supported
 			Assert.AreEqual (typeof (ArgumentException), ex.GetType (), "#B2");
@@ -922,6 +943,7 @@ public class AssemblyBuilderTest
 			Assert.IsTrue (ex.Message.IndexOf ("neutral") != -1, "#B5");
 			Assert.AreEqual ("name", ex.ParamName, "#B6");
 		}
+#endif
 	}
 
 	[Test] // DefineVersionInfoResource (String, String, String, String, String)
@@ -1056,28 +1078,14 @@ public class AssemblyBuilderTest
 		filename = Path.Combine (tempDir, "TestSetCustomAttributeA.dll");
 		check = AssemblyName.GetAssemblyName (filename);
 		Assert.AreEqual (CultureInfo.InvariantCulture, check.CultureInfo, "#A1");
-#if NET_2_0
 		Assert.AreEqual (AssemblyNameFlags.None, check.Flags, "#A2");
-#else
-		Assert.AreEqual (AssemblyNameFlags.PublicKey, check.Flags, "#A2");
-#endif
 		Assert.AreEqual ("TestSetCustomAttributeA, Version=0.0.0.0, Culture=neutral, PublicKeyToken=null", check.FullName, "#A3");
-#if NET_2_0
 		Assert.IsNull (check.GetPublicKey (), "#A4");
-#else
-		Assert.AreEqual (new byte [0], check.GetPublicKey (), "#A4");
-#endif
-#if NET_2_0
 		Assert.AreEqual (new byte [0], check.GetPublicKeyToken (), "#A5");
-#else
-		Assert.IsNull (check.GetPublicKeyToken (), "#A5");
-#endif
 		Assert.AreEqual (AssemblyHashAlgorithm.SHA1, check.HashAlgorithm, "#A6");
 		Assert.IsNull (check.KeyPair, "#A7");
 		Assert.AreEqual ("TestSetCustomAttributeA", check.Name, "#A8");
-#if NET_2_0
 		//Assert.AreEqual (ProcessorArchitecture.MSIL, check.ProcessorArchitecture, "#A9");
-#endif
 		Assert.AreEqual (new Version (0, 0, 0, 0), check.Version, "#A10");
 		Assert.AreEqual (AssemblyVersionCompatibility.SameMachine, check.VersionCompatibility, "#A11");
 
@@ -1106,9 +1114,7 @@ public class AssemblyBuilderTest
 		an.CultureInfo = new CultureInfo ("nl-BE");
 		an.Flags = AssemblyNameFlags.Retargetable;
 		an.Name = "TestSetCustomAttributeB";
-#if NET_2_0
 		an.ProcessorArchitecture = ProcessorArchitecture.IA64;
-#endif
 		an.Version = new Version (1, 3, 5, 7);
 		an.VersionCompatibility = AssemblyVersionCompatibility.SameDomain;
 
@@ -1131,28 +1137,14 @@ public class AssemblyBuilderTest
 		filename = Path.Combine (tempDir, "TestSetCustomAttributeB.dll");
 		check = AssemblyName.GetAssemblyName (filename);
 		Assert.AreEqual ("nl-BE", check.CultureInfo.Name, "#B1");
-#if NET_2_0
 		Assert.AreEqual (AssemblyNameFlags.Retargetable, check.Flags, "#B2");
-#else
-		Assert.AreEqual (AssemblyNameFlags.PublicKey | AssemblyNameFlags.Retargetable, check.Flags, "#B2");
-#endif
 		Assert.AreEqual ("TestSetCustomAttributeB, Version=1.3.5.7, Culture=nl-BE, PublicKeyToken=null, Retargetable=Yes", check.FullName, "#B3");
-#if NET_2_0
 		Assert.IsNull (check.GetPublicKey (), "#B4");
-#else
-		Assert.AreEqual (new byte [0], check.GetPublicKey (), "#B4");
-#endif
-#if NET_2_0
 		Assert.AreEqual (new byte [0], check.GetPublicKeyToken (), "#B5");
-#else
-		Assert.IsNull (check.GetPublicKeyToken (), "#B5");
-#endif
 		Assert.AreEqual (AssemblyHashAlgorithm.SHA1, check.HashAlgorithm, "#B6");
 		Assert.IsNull (check.KeyPair, "#B7");
 		Assert.AreEqual ("TestSetCustomAttributeB", check.Name, "#B8");
-#if NET_2_0
 		//Assert.AreEqual (ProcessorArchitecture.MSIL, check.ProcessorArchitecture, "#B9");
-#endif
 		Assert.AreEqual (new Version (1, 3, 5, 7), check.Version, "#B10");
 		Assert.AreEqual (AssemblyVersionCompatibility.SameMachine, check.VersionCompatibility, "#B11");
 
@@ -1244,17 +1236,10 @@ public class AssemblyBuilderTest
 		AssemblyName check = AssemblyName.GetAssemblyName (filename);
 		// no exception is thrown (file not found)
 		// because it's not AssemblyBuilder.Save job to do the signing :-/
-#if NET_2_0
 		Assert.AreEqual (AssemblyNameFlags.None, check.Flags, "#1");
 		Assert.IsNull (check.GetPublicKey (), "#2");
 		Assert.IsNotNull (check.GetPublicKeyToken (), "#3a");
 		Assert.AreEqual (0, check.GetPublicKeyToken ().Length, "#3b");
-#else
-		Assert.AreEqual (AssemblyNameFlags.PublicKey, check.Flags, "#1");
-		Assert.IsNotNull (check.GetPublicKey (), "#2a");
-		Assert.AreEqual (0, check.GetPublicKey ().Length, "#2b");
-		Assert.IsNull (check.GetPublicKeyToken (), "#3");
-#endif
 		Assert.IsTrue (check.FullName.IndexOf ("Version=0.0.0.0") != -1, "#4");
 		Assert.IsTrue (check.FullName.IndexOf ("Culture=neutral") != -1, "#5");
 		Assert.IsTrue (check.FullName.IndexOf ("PublicKeyToken=null") != -1, "#6");
@@ -1276,17 +1261,10 @@ public class AssemblyBuilderTest
 		AssemblyName check = AssemblyName.GetAssemblyName (filename);
 		// no public key is inserted into the assembly
 		// because it's not AssemblyBuilder.Save job to do the signing :-/
-#if NET_2_0
 		Assert.AreEqual (AssemblyNameFlags.None, check.Flags, "#1");
 		Assert.IsNull (check.GetPublicKey (), "#2");
 		Assert.IsNotNull (check.GetPublicKeyToken (), "#3a");
 		Assert.AreEqual (0, check.GetPublicKeyToken ().Length, "#3b");
-#else
-		Assert.AreEqual (AssemblyNameFlags.PublicKey, check.Flags, "#1");
-		Assert.IsNotNull (check.GetPublicKey (), "#2a");
-		Assert.AreEqual (0, check.GetPublicKey ().Length, "#2b");
-		Assert.IsNull (check.GetPublicKeyToken (), "#3");
-#endif
 		Assert.IsTrue (check.FullName.IndexOf ("Version=0.0.0.0") != -1, "#4");
 		Assert.IsTrue (check.FullName.IndexOf ("Culture=neutral") != -1, "#5");
 		Assert.IsTrue (check.FullName.IndexOf ("PublicKeyToken=null") != -1, "#6");
@@ -1440,14 +1418,21 @@ public class AssemblyBuilderTest
 			Assert.IsNotNull (refs [0].CultureInfo, "#D2:CultureInfo");
 			Assert.IsNull (refs [0].EscapedCodeBase, "#D2:EscapedCodeBase");
 			Assert.AreEqual (AssemblyNameFlags.None, refs [0].Flags, "#D2:Flags");
-			Assert.AreEqual (Consts.AssemblyCorlib, refs [0].FullName, "#D2:FullName");
+			Assert.AreEqual (typeof (object).FullName, refs [0].FullName, "#D2:FullName");
 			Assert.AreEqual (AssemblyHashAlgorithm.SHA1, refs [0].HashAlgorithm, "#D2:HashAlgorithm");
 			Assert.IsNull (refs [0].KeyPair, "#D2:KeyPair");
 			Assert.AreEqual ("mscorlib", refs [0].Name, "#D2:Name");
-#if NET_2_0
 			Assert.AreEqual (ProcessorArchitecture.None, refs [0].ProcessorArchitecture, "#D2:PA");
+
+			string FxVersion;
+#if MOBILE
+			FxVersion = "2.0.5.0;";
+#elif NET_4_0
+			FxVersion = "4.0.0.0;";
+#else
+			FxVersion = "2.0.0.0;";
 #endif
-			Assert.AreEqual (new Version (Consts.FxVersion), refs [0].Version, "#D2:Version");
+			Assert.AreEqual (new Version (FxVersion), refs [0].Version, "#D2:Version");
 			Assert.AreEqual (AssemblyVersionCompatibility.SameMachine,
 				refs [0].VersionCompatibility, "#D2:VersionCompatibility");
 			Assert.IsNull (refs [0].GetPublicKey (), "#D2:GetPublicKey");
@@ -1474,9 +1459,7 @@ public class AssemblyBuilderTest
 			Assert.AreEqual (AssemblyHashAlgorithm.SHA1, refs [0].HashAlgorithm, "#E2:HashAlgorithm");
 			Assert.IsNull (refs [0].KeyPair, "#E2:KeyPair");
 			Assert.AreEqual (an1.Name, refs [0].Name, "#E2:Name");
-#if NET_2_0
 			Assert.AreEqual (ProcessorArchitecture.None, refs [0].ProcessorArchitecture, "#E2:PA");
-#endif
 			Assert.AreEqual (new Version (3, 0, 0, 0), refs [0].Version, "#E2:Version");
 			Assert.AreEqual (AssemblyVersionCompatibility.SameMachine,
 				refs [0].VersionCompatibility, "#E2:VersionCompatibility");
@@ -1504,9 +1487,7 @@ public class AssemblyBuilderTest
 			Assert.AreEqual (AssemblyHashAlgorithm.SHA1, refs [0].HashAlgorithm, "#F2:HashAlgorithm");
 			Assert.IsNull (refs [0].KeyPair, "#F2:KeyPair");
 			Assert.AreEqual (an1.Name, refs [0].Name, "#F2:Name");
-#if NET_2_0
 			Assert.AreEqual (ProcessorArchitecture.None, refs [0].ProcessorArchitecture, "#F2:PA");
-#endif
 			Assert.AreEqual (new Version (3, 0, 0, 0), refs [0].Version, "#F2:Version");
 			Assert.AreEqual (AssemblyVersionCompatibility.SameMachine,
 				refs [0].VersionCompatibility, "#F2:VersionCompatibility");
@@ -1524,9 +1505,7 @@ public class AssemblyBuilderTest
 			Assert.AreEqual (AssemblyHashAlgorithm.SHA1, refs [1].HashAlgorithm, "#F3:HashAlgorithm");
 			Assert.IsNull (refs [1].KeyPair, "#F3:KeyPair");
 			Assert.AreEqual (an2.Name, refs [1].Name, "#F3:Name");
-#if NET_2_0
 			Assert.AreEqual (ProcessorArchitecture.None, refs [1].ProcessorArchitecture, "#F3:PA");
-#endif
 			Assert.AreEqual (new Version (1, 2, 3, 4), refs [1].Version, "#F3:Version");
 			Assert.AreEqual (AssemblyVersionCompatibility.SameMachine,
 				refs [1].VersionCompatibility, "#F3:VersionCompatibility");
@@ -1583,13 +1562,7 @@ public class AssemblyBuilderTest
 		Assert.IsTrue (abName.GetPublicKey ().Length > 0, "#B3b");
 		Assert.IsNotNull (abName.GetPublicKeyToken (), "#B4a");
 		Assert.IsTrue (abName.GetPublicKeyToken ().Length > 0, "#B4b");
-#if NET_2_0
 		Assert.AreEqual (fullName, abName.FullName, "#B5");
-#else
-		//Assert.AreEqual ("AssemblyNameTest_PublicKey, Version=1.2.3.4, PublicKeyToken=0eea7ce65f35f2d8", abName.FullName, "#B5");
-		Assert.IsTrue (abName.FullName.IndexOf ("AssemblyNameTest_PublicKey, Version=1.2.3.4") != -1, "#B5a");
-		Assert.IsTrue (abName.FullName.IndexOf ("PublicKeyToken=0eea7ce65f35f2d8") != -1, "#B5b");
-#endif
 
 		ab.Save ("AssemblyNameTest_PublicKey.dll");
 		AssemblyName bakedName = AssemblyName.GetAssemblyName (Path.Combine(
@@ -1620,18 +1593,10 @@ public class AssemblyBuilderTest
 
 		AssemblyName abName = ab.GetName ();
 		Assert.IsNotNull (abName.CultureInfo != null, "#B1");
-#if NET_2_0
 		Assert.IsTrue (abName.CultureInfo != CultureInfo.InvariantCulture, "#B2a");
 		Assert.AreEqual (CultureInfo.InvariantCulture.LCID, abName.CultureInfo.LCID, "#B2a");
 		Assert.AreEqual (AssemblyNameFlags.PublicKey, abName.Flags, "#B3");
 		Assert.AreEqual (fullName, abName.FullName, "#B4");
-#else
-		Assert.AreEqual (CultureInfo.InvariantCulture, abName.CultureInfo, "#B2");
-		Assert.AreEqual (AssemblyNameFlags.PublicKey, abName.Flags, "#B3");
-		//Assert.AreEqual ("AssemblyNameTest_MoreCultureInfo, Version=1.2.3.4, PublicKeyToken=0eea7ce65f35f2d8", abName.FullName, "#B4");
-		Assert.IsTrue (abName.FullName.IndexOf ("AssemblyNameTest_MoreCultureInfo, Version=1.2.3.4") != -1, "#B4a");
-		Assert.IsTrue (abName.FullName.IndexOf ("PublicKeyToken=0eea7ce65f35f2d8") != -1, "#B4b");
-#endif
 
 		ab.Save ("AssemblyNameTest_MoreCultureInfo.dll");
 
@@ -1640,12 +1605,8 @@ public class AssemblyBuilderTest
 
 		Assert.IsNotNull (bakedName.CultureInfo, "#C1");
 
-#if NET_2_0
 		Assert.IsTrue (abName.CultureInfo != CultureInfo.InvariantCulture, "#C2a");
 		Assert.AreEqual (CultureInfo.InvariantCulture.LCID, abName.CultureInfo.LCID, "#C2b");
-#else
-		Assert.AreEqual (CultureInfo.InvariantCulture, bakedName.CultureInfo, "#C2");
-#endif
 		Assert.AreEqual (fullName, bakedName.FullName, "#C3");
 	}
 
@@ -1684,11 +1645,10 @@ public class AssemblyBuilderTest
 	}
 
 	[Test]
-	[Category ("NotDotNet")]
 	public void GetType_IgnoreCase ()
 	{
 		TypeBuilder tb = mb.DefineType ("Foo.Test2", TypeAttributes.Public, typeof (object));
-		// the previous line throws a TypeLoadException under MS 1.1 SP1
+		tb.CreateType ();
 
 		Type t;
 
@@ -1702,6 +1662,66 @@ public class AssemblyBuilderTest
 		Assert.AreEqual ("Test2", t.Name, "#3");
 	}
 
+
+	[Test]
+	public void TestGetType ()
+	{
+		TypeBuilder tb = mb.DefineType ("Test", TypeAttributes.Public);
+
+		Assert.IsNull (ab.GetType ("Test", false, true), "#1");
+		try {
+			ab.GetType ("Test", true, true);
+			Assert.Fail ("#2");
+		} catch (TypeLoadException) { }
+
+		var res = tb.CreateType ();
+
+		Assert.AreSame (res, ab.GetType ("Test", false, true), "#3");
+	}
+
+	[Test]
+	public void GetModule ()
+	{
+		var ab = genAssembly ();
+		Assert.IsNull (ab.GetModule ("Foo"), "#1");
+
+		var modA = ab.DefineDynamicModule ("Foo");
+		var modB = ab.DefineDynamicModule ("Bar");
+
+		Assert.AreSame (modA, ab.GetModule ("Foo"), "#2"); 
+		Assert.AreSame (modB, ab.GetModule ("Bar"), "#3"); 
+		Assert.IsNull (ab.GetModule ("FooBar"), "#4");
+	}
+	
+	[Test]
+	public void GetModules2 ()
+	{
+		//XXX this is not the v4 behavior since it returns
+		//the manifest module in the place of the first one
+		var ab = genAssembly ();
+		var modA = ab.DefineDynamicModule ("Foo");
+		var modB = ab.DefineDynamicModule ("Bar");
+		Assert.AreEqual (2, ab.GetModules ().Length, "#1");
+		Assert.AreSame (modA, ab.GetModules () [0], "#2");
+		Assert.AreSame (modB, ab.GetModules () [1], "#3");
+	}
+
+	[Test]
+	[Category ("NotDotNet")] // MS returns the real deal
+	public void GetReferencedAssemblies_Trivial ()
+	{
+		Assert.IsNotNull (ab.GetReferencedAssemblies (), "#1");
+	}
+	
+	[Test]
+	public void GetLoadedModules ()
+	{
+		var res = ab.GetLoadedModules (true);
+		Assert.IsNotNull (res, "#1");
+		Assert.AreEqual (1, res.Length, "#2");
+		Assert.AreEqual (mb, res [0], "#3");
+	}
+
 	[ExpectedException (typeof (TypeLoadException))]
 	public void GetCustomAttributes_NotCreated ()
 	{
@@ -1717,6 +1737,67 @@ public class AssemblyBuilderTest
 		ab.GetCustomAttributes (true);
 	}
 
+
+	[Test]
+	public void GetTypesWithUnfinishedTypeBuilder ()
+	{
+		AssemblyBuilder ab = genAssembly ();
+		ModuleBuilder mb = ab.DefineDynamicModule("tester", "tester.dll", false);
+		mb.DefineType ("K").CreateType ();
+		var tb = mb.DefineType ("T");
+
+		try {
+			ab.GetTypes ();
+			Assert.Fail ("#1");
+		} catch (ReflectionTypeLoadException ex) {
+			Assert.AreEqual (1, ex.Types.Length, "#2");
+			Assert.AreEqual (1, ex.LoaderExceptions.Length, "#3");
+			Assert.IsNull (ex.Types [0], "#4");
+			Assert.IsTrue (ex.LoaderExceptions [0] is TypeLoadException, "#5");
+		}
+
+		tb.CreateType ();
+		var types = ab.GetTypes ();
+		Assert.AreEqual (2, types.Length, "#5");
+		foreach (var t in types)
+			Assert.IsFalse (t is TypeBuilder, "#6_" + t.Name);
+	}
+
+	[Test]
+	public void DynamicAssemblyGenerationInCurrentDomainShouldNotChangeTheOrderOfCurrentDomainGetAssemblies ()
+	{
+		var initialPosition = GetAssemblyPositionForType (GetType ());
+		DefineDynamicAssembly (AppDomain.CurrentDomain);
+
+		var currentPosition = GetAssemblyPositionForType (GetType ());
+		Assert.AreEqual (initialPosition, currentPosition);
+	}
+
+	static void DefineDynamicAssembly (AppDomain domain)
+	{
+		AssemblyName assemblyName = new AssemblyName ();
+		assemblyName.Name = "MyDynamicAssembly";
+
+		AssemblyBuilder assemblyBuilder = domain.DefineDynamicAssembly (assemblyName, AssemblyBuilderAccess.Run);
+		ModuleBuilder moduleBuilder = assemblyBuilder.DefineDynamicModule ("MyDynamicModule");
+		TypeBuilder typeBuilder = moduleBuilder.DefineType ("MyDynamicType", TypeAttributes.Public);
+		ConstructorBuilder constructorBuilder = typeBuilder.DefineConstructor (MethodAttributes.Public, CallingConventions.Standard, null);
+		ILGenerator ilGenerator = constructorBuilder.GetILGenerator ();
+		ilGenerator.EmitWriteLine ("MyDynamicType instantiated!");
+		ilGenerator.Emit (OpCodes.Ret);
+		typeBuilder.CreateType ();
+	}
+
+	static int GetAssemblyPositionForType (Type type)
+	{
+		var assemblies = AppDomain.CurrentDomain.GetAssemblies ();
+		for (int i = 0; i < assemblies.Length; i++)
+			if (type.Assembly == assemblies [i])
+				return i;
+		return -1;
+	}
+
+
 	private static void AssertAssemblyName (string tempDir, AssemblyName assemblyName, string abName, string fullName)
 	{
 		AppDomain currentDomain = AppDomain.CurrentDomain;
@@ -1727,25 +1808,17 @@ public class AssemblyBuilderTest
 				assemblyName, AssemblyBuilderAccess.Save, tempDir);
 			ab.Save (assemblyName.Name + ".dll");
 
-#if NET_2_0
 			// on .NET 2.0, the full name of the AssemblyBuilder matches the
 			// fully qualified assembly name
 			Assert.AreEqual (fullName, ab.FullName, "#1");
-#else
-			//Assert.AreEqual (abName, ab.FullName, "#1");
-#endif
 
 			AssemblyName an = ab.GetName ();
 
 			Assert.AreEqual (AssemblyNameFlags.PublicKey, an.Flags, "#2");
 			Assert.IsNotNull (an.GetPublicKey (), "#3a");
 			Assert.AreEqual (0, an.GetPublicKey ().Length, "#3b");
-#if NET_2_0
 			Assert.IsNotNull (an.GetPublicKeyToken (), "#4a");
 			Assert.AreEqual (0, an.GetPublicKeyToken ().Length, "#4b");
-#else
-			Assert.IsNull (an.GetPublicKeyToken (), "#4");
-#endif
 
 			// load assembly in separate domain, so we can clean-up after the 
 			// test
@@ -1780,5 +1853,25 @@ public class AssemblyBuilderTest
 			Assert.AreEqual (_assemblyName, assemblyName.ToString ());
 		}
 	}
+
+
+	[Test]//Bug #7126
+	public void CannotCreateInstanceOfSaveOnlyAssembly ()
+	{
+		var asm_builder = genAssembly (AssemblyBuilderAccess.Save);
+        var mod_builder = asm_builder.DefineDynamicModule("Foo", "Foo.dll");
+
+        var type_builder = mod_builder.DefineType("Foo",
+                TypeAttributes.Public | TypeAttributes.Sealed |
+                TypeAttributes.Class | TypeAttributes.BeforeFieldInit);
+
+        var type = type_builder.CreateType();
+
+		try {
+			Activator.CreateInstance(type);
+			Assert.Fail ("Cannot create instance of save only type");
+		} catch (NotSupportedException e) {
+		}
+     }
 }
 }
