@@ -101,6 +101,88 @@ namespace MonoTests.System.Reflection
 		}
 	}
 
+	class MethodInfoWrapper : MethodInfo
+	{
+		private readonly MethodInfo method;
+		
+		public MethodInfoWrapper (MethodInfo method)
+		{
+			this.method = method;
+		}
+		
+		public override object[] GetCustomAttributes (bool inherit)
+		{
+			return method.GetCustomAttributes (inherit);
+		}
+		
+		public override bool IsDefined (Type attributeType, bool inherit)
+		{
+			return method.IsDefined (attributeType, inherit);
+		}
+		
+		public override ParameterInfo[] GetParameters ()
+		{
+			return method.GetParameters ();
+		}
+		
+		public override MethodImplAttributes GetMethodImplementationFlags ()
+		{
+			return method.GetMethodImplementationFlags ();
+		}
+		
+		public override object Invoke (object obj, BindingFlags invokeAttr, Binder binder, object[] parameters, CultureInfo culture)
+		{
+			return method.Invoke (obj, invokeAttr, binder, parameters, culture);
+		}
+		
+		public override MethodInfo GetBaseDefinition ()
+		{
+			return method.GetBaseDefinition ();
+		}
+		
+		public override ICustomAttributeProvider ReturnTypeCustomAttributes {
+			get { return method.ReturnTypeCustomAttributes; }
+		}
+		
+		public override string Name {
+			get { return method.Name; }
+		}
+		
+		public override Type ReturnType {
+			get { return method.ReturnType; }
+		}
+		
+		public override Type DeclaringType {
+			get { return method.DeclaringType; }
+		}
+		
+		public override Type ReflectedType {
+			get { return method.ReflectedType; }
+		}
+		
+		public override RuntimeMethodHandle MethodHandle {
+			get { return method.MethodHandle; }
+		}
+		
+		public override MethodAttributes Attributes {
+			get { return method.Attributes; }
+		}
+		
+		public override object[] GetCustomAttributes (Type attributeType, bool inherit)
+		{
+			return method.GetCustomAttributes (attributeType, inherit);
+		}
+	}
+
+	class DefaultValues
+	{
+		public int Value;
+		public DefaultValues (int i = 5)
+		{
+			Value = i;
+		}
+	}
+
 	[TestFixture]
 	public class BinderTest
 	{
@@ -113,6 +195,16 @@ namespace MonoTests.System.Reflection
 			var o = Activator.CreateInstance (typeof (ParamsArrayTest), new object[] { test_args });
 			Assert.IsNotNull (o, "#A1");
 		}
+
+		[Test]
+		public void DefaultParameter ()
+		{
+			var o = Activator.CreateInstance (typeof (DefaultValues),
+				BindingFlags.CreateInstance | BindingFlags.Public | BindingFlags.Instance | BindingFlags.OptionalParamBinding,
+				null, null, null);
+			var a = o as DefaultValues;
+			Assert.AreEqual (5, a.Value);
+		}
 		
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
@@ -130,6 +222,13 @@ namespace MonoTests.System.Reflection
 			binder.SelectProperty (0, new PropertyInfo [] {}, null, null, null);
 		}
 
+		[Test]
+		[ExpectedException (typeof (NotSupportedException))]
+		public void ChangeTypeOnDefaultBinder ()
+		{
+			binder.ChangeType (null, null, null);
+		}
+
 		[Test]
 		[ExpectedException (typeof (AmbiguousMatchException))]
 		public void AmbiguousProperty1 () // Bug 58381
@@ -271,19 +370,11 @@ namespace MonoTests.System.Reflection
 			MethodInfo mi_run = type.GetMethod ("Run", flags, binder,
 				new Type [] { typeof (int) }, null);
 			Assert.IsFalse (mi_run.GetParameters () [0].ParameterType.IsByRef, "#A1");
-#if NET_2_0
 			MethodInfo mi_run_ref = type.GetMethod ("Run", flags, binder,
 				new Type [] { typeof (int).MakeByRefType () }, null);
-#else
-			MethodInfo mi_run_ref = type.GetMethod ("RunV1", flags);
-#endif
 			Assert.IsTrue (mi_run_ref.GetParameters () [0].ParameterType.IsByRef, "#A2");
 
-#if NET_2_0
 			ref_int = typeof (int).MakeByRefType ();
-#else
-			ref_int = mi_run_ref.GetParameters () [0].ParameterType;
-#endif
 
 			match = new MethodBase [] { mi_run_ref };
 			types = new Type [] { typeof (int) };
@@ -469,12 +560,8 @@ namespace MonoTests.System.Reflection
 			MethodInfo mi_run = type.GetMethod ("Run", flags, binder,
 				new Type [] { typeof (int) }, null);
 			Assert.IsFalse (mi_run.GetParameters () [0].ParameterType.IsByRef, "#A1");
-#if NET_2_0
 			MethodInfo mi_run_ref = type.GetMethod ("Run", flags, binder,
 				new Type [] { typeof (int).MakeByRefType () }, null);
-#else
-			MethodInfo mi_run_ref = type.GetMethod ("RunV1", flags);
-#endif
 			Assert.IsTrue (mi_run_ref.GetParameters () [0].ParameterType.IsByRef, "#A2");
 
 			match = new MethodBase [] { mi_run };
@@ -876,6 +963,32 @@ namespace MonoTests.System.Reflection
 			Assert.AreEqual ("Hello\nExtra\nWorld\n", sw.ToString ());
 		}
 
+		[Test] // #1321
+		public void BindToMethodNamedArgs_2 ()
+		{
+			StringWriter sw = new StringWriter ();
+			sw.NewLine = "\n";
+
+			object[] argValues = new object [] {5, "AB", sw};
+			string [] argNames = new string [] {"second", "first", "output"};
+
+			typeof (BinderTest).InvokeMember ("TestMethod",
+					BindingFlags.InvokeMethod,
+					null,
+					null,
+					argValues,
+					null,
+					null,
+					argNames);
+
+			Assert.AreEqual ("AB\n5\n", sw.ToString ());
+		}
+
+		public static void TestMethod (string first, int second, TextWriter output) {
+			output.WriteLine (first);
+			output.WriteLine (second);
+		}
+
 		public class Bug41691
 		{
 			public static void PrintName (string lastName, string firstName, string extra, TextWriter output)
@@ -923,7 +1036,6 @@ namespace MonoTests.System.Reflection
 			Assert.AreEqual (3, bug42457_2, "#6");
 		}
 
-#if NET_2_0
 		[Test]
 		public void NullableArg () {
 			MethodInfo method = (typeof (BinderTest)).GetMethod("SetA", new [] {typeof (Int32)});
@@ -933,7 +1045,6 @@ namespace MonoTests.System.Reflection
 		public int SetA(Int32? a) {
 			return (int)a;
 		}
-#endif
 
 		static void MethodWithLongParam(long param)
 		{
@@ -954,19 +1065,16 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-		[Category ("NotDotNet")]
-		public void TestParamsAttribute2_Mono ()
+		public void TestParamsAttribute_1 ()
 		{
-			MethodInfo mi = typeof (BinderTest).GetMethod ("params_method1", BindingFlags.Static|BindingFlags.Public, null, new Type [] { typeof (object), typeof (object), typeof (object) }, null);
-			Assert.IsNotNull (mi, "#1");
-			Assert.AreEqual (typeof (object []), mi.GetParameters () [1].ParameterType, "#2");
+			MethodInfo mi = typeof (BinderTest).GetMethod ("params_method1", BindingFlags.Static | BindingFlags.Public, null, new Type [] { typeof (object), typeof (object), typeof (object) }, null);
+			Assert.IsNull (mi, "#1");
 		}
 
 		[Test]
-		[Category ("NotWorking")]
-		public void TestParamsAttribute2_MS ()
+		public void TestParamsAttribute_2 ()
 		{
-			MethodInfo mi = typeof (BinderTest).GetMethod ("params_method1", BindingFlags.Static | BindingFlags.Public, null, new Type [] { typeof (object), typeof (object), typeof (object) }, null);
+			MethodInfo mi = typeof (BinderTest).GetMethod ("params_method2", BindingFlags.Static | BindingFlags.Public, null, Type.EmptyTypes, null);
 			Assert.IsNull (mi, "#1");
 		}
 
@@ -982,6 +1090,10 @@ namespace MonoTests.System.Reflection
 		{
 		}
 
+		public static void params_method2 (params string[] args)
+		{
+		}	
+
 		public static double DoubleMethod (double d) {
 			return d;
 		}
@@ -1229,11 +1341,7 @@ namespace MonoTests.System.Reflection
 			{
 			}
 
-#if NET_2_0
 			public void Run (out int i)
-#else
-			public void RunV1 (out int i)
-#endif
 			{
 				i = 0;
 			}
@@ -1368,5 +1476,18 @@ namespace MonoTests.System.Reflection
 				null, // target
 				new object [] { CultureInfo.CurrentCulture, "foo{0}{1}", "bar", "baz" }));
 		}
+
+		public static void CustomMethodType_Helper ()
+		{
+		}
+
+		[Test]
+		public void CustomMethodType ()
+		{
+			var method = new MethodInfoWrapper (GetType ().GetMethod ("CustomMethodType_Helper"));
+
+			var res = Type.DefaultBinder.SelectMethod (BindingFlags.Static | BindingFlags.Public, new[] { method }, Type.EmptyTypes, new ParameterModifier[0]);
+			Assert.AreSame (method, res);
+		}
 	}
 }
