@@ -31,28 +31,29 @@ using NUnit.Framework;
 using System;
 using System.Threading;
 using System.Reflection;
+#if !MONOTOUCH
 using System.Reflection.Emit;
+#endif
 using System.Runtime.InteropServices;
 using System.Runtime.CompilerServices;
 
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 
 namespace A.B.C {
+	// Disable expected warning
+#pragma warning disable 169
 	public struct MethodInfoTestStruct {
 		int p;
 	}
+#pragma warning restore 169
 }
 namespace MonoTests.System.Reflection
 {
 	[TestFixture]
 	public class MethodInfoTest
 	{
-#if !TARGET_JVM
 		[DllImport ("libfoo", EntryPoint="foo", CharSet=CharSet.Unicode, ExactSpelling=false, PreserveSig=true, SetLastError=true, BestFitMapping=true, ThrowOnUnmappableChar=true)]
 		public static extern void dllImportMethod ();
-#endif
 		[MethodImplAttribute(MethodImplOptions.PreserveSig)]
 		public void preserveSigMethod ()
 		{
@@ -80,7 +81,20 @@ namespace MonoTests.System.Reflection
 			}
 		}
 
-#if NET_2_0
+		[Test]
+		public void TestInvokeByRefReturnMethod ()
+		{
+			try {
+				MethodInfo m = typeof (int[]).GetMethod ("Address");
+				m.Invoke (new int[1], new object[] { 0 });
+				Assert.Fail ("#1");
+			} catch (NotSupportedException e) {
+				Assert.AreEqual (typeof (NotSupportedException), e.GetType (), "#2");
+				Assert.IsNull (e.InnerException, "#3");
+				Assert.IsNotNull (e.Message, "#4");
+			}
+		}
+
 		[Test]
 		public void PseudoCustomAttributes ()
 		{
@@ -112,14 +126,12 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-		[Category ("TargetJvmNotWorking")]
 		public void ReturnTypePseudoCustomAttributes ()
 		{
 			MethodInfo mi = typeof (MethodInfoTest).GetMethod ("ReturnTypeMarshalAs");
 
 			Assert.IsTrue (mi.ReturnTypeCustomAttributes.GetCustomAttributes (typeof (MarshalAsAttribute), true).Length == 1);
 		}
-#endif
 
 		public static int foo (int i, int j)
 		{
@@ -157,9 +169,6 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-#if ONLY_1_1
-		[Category ("NotDotNet")] // #A2 fails on MS.NET 1.x
-#endif
 		public void ByrefVtypeInvoke ()
 		{
 			MethodInfo mi = typeof (MethodInfoTest).GetMethod ("ByrefVtype");
@@ -177,7 +186,7 @@ namespace MonoTests.System.Reflection
 			Assert.AreEqual (5, args[0], "#B2");
 		}
 
-		public void HeyHey (out string out1, ref string ref1)
+		public void HeyHey (out string out1, ref DateTime ref1)
 		{
 			out1 = null;
 		}
@@ -199,18 +208,10 @@ namespace MonoTests.System.Reflection
 				method.Invoke (null, new object [0]);
 				Assert.Fail ("#1");
 			}
-#if NET_2_0
 			catch (ThreadAbortException ex) {
 				Thread.ResetAbort ();
 				Assert.IsNull (ex.InnerException, "#2");
 			}
-#else
-			catch (TargetInvocationException ex) {
-				Thread.ResetAbort ();
-				Assert.IsNotNull (ex.InnerException, "#2");
-				Assert.AreEqual (typeof (ThreadAbortException), ex.InnerException.GetType (), "#3");
-			}
-#endif
 		}
 
 		public static void AbortIt ()
@@ -221,7 +222,7 @@ namespace MonoTests.System.Reflection
 		[Test] // bug #76541
 		public void ToStringByRef ()
 		{
-			Assert.AreEqual ("Void HeyHey(System.String ByRef, System.String ByRef)",
+			Assert.AreEqual ("Void HeyHey(System.String ByRef, System.DateTime ByRef)",
 				this.GetType ().GetMethod ("HeyHey").ToString ());
 		}
 		
@@ -238,11 +239,9 @@ namespace MonoTests.System.Reflection
 			Assert.AreEqual ("Int32* PtrFunc(Int32*)", this.GetType ().GetMethod ("PtrFunc").ToString ());
 		}
 
-
-#if NET_2_0
 		public struct SimpleStruct
 		{
-			int a;
+			public int a;
 		}
 
 		public static unsafe SimpleStruct* PtrFunc2 (SimpleStruct* a, A.B.C.MethodInfoTestStruct *b)
@@ -262,7 +261,6 @@ namespace MonoTests.System.Reflection
 			Assert.AreEqual ("System.Collections.ObjectModel.ReadOnlyCollection`1[T] AsReadOnly[T](T[])",
 				typeof (Array).GetMethod ("AsReadOnly").ToString ());
 		}
-#endif
 
 		class GBD_A         { public virtual     void f () {} }
 		class GBD_B : GBD_A { public override    void f () {} }
@@ -303,8 +301,6 @@ namespace MonoTests.System.Reflection
 			Assert.AreSame (inheritedMethod, baseMethod);
 		}
 
-#if NET_2_0
-#if !TARGET_JVM // MethodBody is not supported for TARGET_JVM
 		[Test]
 		public void GetMethodBody_Abstract ()
 		{
@@ -350,6 +346,9 @@ namespace MonoTests.System.Reflection
 		[Test]
 		public void GetMethodBody ()
 		{
+#if MONOTOUCH && !DEBUG
+			Assert.Ignore ("Release app (on devices) are stripped of (managed) IL so this test would fail");
+#endif
 			MethodBody mb = typeof (MethodInfoTest).GetMethod ("locals_method").GetMethodBody ();
 
 			Assert.IsTrue (mb.InitLocals, "#1");
@@ -366,7 +365,6 @@ namespace MonoTests.System.Reflection
 			else
 				Assert.AreEqual (false, locals [1].IsPinned, "#6");
 		}
-#endif // TARGET_JVM
 
 		public int return_parameter_test ()
 		{
@@ -397,7 +395,6 @@ namespace MonoTests.System.Reflection
 			//Assert.IsTrue (pi.IsRetval, "#3");
 		}
 
-#if !TARGET_JVM // ReflectionOnly is not supported yet on TARGET_JVM
 		[Test]
 			public void InvokeOnRefOnlyAssembly ()
 		{
@@ -415,7 +412,6 @@ namespace MonoTests.System.Reflection
 				Assert.IsNotNull (ex.Message, "#4");
 			}
 		}
-#endif // TARGET_JVM
 
 		[Test]
 		[ExpectedException (typeof (TargetInvocationException))]
@@ -538,7 +534,7 @@ namespace MonoTests.System.Reflection
 			} catch (InvalidOperationException ex) {
 			}
 		}
-
+#if !MONOTOUCH
 		public TFoo SimpleGenericMethod2<TFoo, TBar> () { return default (TFoo); }
 		/*Test for the uggly broken behavior of SRE.*/
 		[Test]
@@ -557,7 +553,7 @@ namespace MonoTests.System.Reflection
 			/*broken ReturnType*/
 			Assert.AreSame (gmi.GetGenericArguments () [0], ins.ReturnType, "#2");
 		}
-
+#endif
 		public static int? pass_nullable (int? i)
 		{
 			return i;
@@ -691,6 +687,21 @@ namespace MonoTests.System.Reflection
 			Assert.AreSame (gmd, oi);
 		}
 
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+#if MOBILE
+		[Category ("NotWorking")] // #10552
+#endif
+		public void MakeGenericMethodRespectConstraints ()
+		{
+			var m = typeof (MethodInfoTest).GetMethod ("TestMethod");
+			m.MakeGenericMethod (typeof (Type));
+		}
+
+		public void TestMethod <T> () where T : Exception
+		{
+		}
+
 		public class MyList<T>
 		{
 			public TOutput ConvertAll<TOutput> (Foo<T,TOutput> arg)
@@ -713,7 +724,6 @@ namespace MonoTests.System.Reflection
 			{
 			}
 		}
-#endif
 #if NET_4_0
 		interface IMethodInvoke<out T>
 		{
@@ -740,9 +750,65 @@ namespace MonoTests.System.Reflection
 		}
 #endif
 
+
+		public int? Bug12856 ()
+		{
+			return null;
+		}
+
+		[Test] //Bug #12856
+		public void MethodToStringShouldPrintFullNameOfGenericStructs ()
+		{
+			var m = GetType ().GetMethod ("Bug12856");
+			Assert.AreEqual ("System.Nullable`1[System.Int32] Bug12856()", m.ToString (), "#1");
+		}
+
+#if !MONOTOUCH
+		class GenericClass<T>
+		{
+			public void Method ()
+			{
+				T lv = default(T);
+				Console.WriteLine(lv);
+			}
+
+			public void Method2<K> (T a0, K a1)
+			{
+				T var0 = a0;
+				K var1 = a1;
+				Console.WriteLine (var0);
+				Console.WriteLine (var1);
+			}
+		}
+
+		[Test]
+		public void TestLocalVariableTypes ()
+		{
+			var typeofT = typeof (GenericClass<>).GetGenericArguments () [0];
+			var typeofK = typeof (GenericClass<>).GetMethod ("Method2").GetGenericArguments () [0];
+
+			var type = typeof (GenericClass<>).GetMethod("Method").GetMethodBody().LocalVariables[0].LocalType;
+			Assert.AreEqual (typeofT, type);
+			Assert.AreEqual (typeof (GenericClass<>), type.DeclaringType);
+
+			type = typeof (GenericClass<>).GetMethod("Method2").GetMethodBody().LocalVariables[0].LocalType;
+			Assert.AreEqual (typeofT, type);
+			Assert.AreEqual (typeof (GenericClass<>), type.DeclaringType);
+
+			type = typeof (GenericClass<>).GetMethod("Method2").GetMethodBody().LocalVariables[1].LocalType;
+			Assert.AreEqual (typeofK, type);
+			Assert.AreEqual (typeof (GenericClass<>), type.DeclaringType);
+
+			type = typeof (GenericClass<int>).GetMethod("Method2").GetMethodBody().LocalVariables[0].LocalType;
+			Assert.AreEqual (typeof (int), type);
+
+			type = typeof (GenericClass<int>).GetMethod("Method2").GetMethodBody().LocalVariables[1].LocalType;
+			Assert.AreEqual (typeofK, type);
+			Assert.AreEqual (typeof (GenericClass<>), type.DeclaringType);
+		}
+#endif
 	}
 	
-#if NET_2_0
 	// Helper class
 	class RefOnlyMethodClass 
 	{
@@ -766,5 +832,4 @@ namespace MonoTests.System.Reflection
 			set { _myList = value; }
 		}
 	}
-#endif
 }
