@@ -28,7 +28,7 @@ namespace MonoTests.System.Runtime.InteropServices
 			public int field;
 		}
 
-		class ClsNoLayout {
+		public class ClsNoLayout {
 			public int field;
 		}
 
@@ -173,12 +173,20 @@ namespace MonoTests.System.Runtime.InteropServices
 			Marshal.FreeHGlobal (ptr);
 		}
 
-		struct Foo {
-			int a;
-			static int b;
-			long c;
-			static char d;
-			int e;
+		[Test]
+		public void AllocCoTaskMemZeroSize ()
+		{
+			IntPtr ptr = Marshal.AllocCoTaskMem (0);
+			Assert.IsTrue (ptr != IntPtr.Zero);
+			Marshal.FreeCoTaskMem (ptr);
+		}
+
+		public struct Foo {
+			public int a;
+			public static int b;
+			public long c;
+			public static char d;
+			public int e;
 		}
 
 		[Test]
@@ -497,7 +505,6 @@ namespace MonoTests.System.Runtime.InteropServices
 				Marshal.FreeCoTaskMem (ptr);
 			}
 		}
-#if NET_2_0
 		private const string NotSupported = "Not supported before Windows 2000 Service Pack 3";
 		private static char[] PlainText = new char[] { 'a', 'b', 'c' };
 		private static byte[] AsciiPlainText = new byte[] { (byte) 'a', (byte) 'b', (byte) 'c' };
@@ -663,7 +670,6 @@ namespace MonoTests.System.Runtime.InteropServices
 				Assert.Ignore (NotSupported);
 			}
 		}
-#endif
 
 #if !NET_2_1
 		[Test]
@@ -780,7 +786,6 @@ namespace MonoTests.System.Runtime.InteropServices
 			Assert.IsNull (Marshal.PtrToStructure (IntPtr.Zero, typeof (SimpleStruct2)));
 		}
 		
-#if NET_2_0
 		[Test]
 		public void TestGetExceptionForHR ()
 		{
@@ -793,7 +798,6 @@ namespace MonoTests.System.Runtime.InteropServices
 			ex = Marshal.GetExceptionForHR (E_INVALIDARG);
 			Assert.AreEqual (typeof (ArgumentException), ex.GetType (), "E_INVALIDARG");
 		}
-#endif
 		bool RunningOnUnix {
 			get {
 				int p = (int) Environment.OSVersion.Platform;
