@@ -8,6 +8,7 @@
 //
 // Copyright 2001 Ximian, Inc (http://www.ximian.com)
 // Copyright 2004-2010 Novell, Inc
+// Copyright 2011 Xamarin Inc
 //
 //
 
@@ -38,11 +39,13 @@ namespace Mono.CSharp {
 		ArrayType = 1 << 19,
 		PointerType = 1 << 20,
 		InternalCompilerType = 1 << 21,
+		MissingType = 1 << 22,
+		Void = 1 << 23,
+		Namespace = 1 << 24,
 
 		NestedMask = Class | Struct | Delegate | Enum | Interface,
 		GenericMask = Method | Class | Struct | Delegate | Interface,
-		MaskType = Constructor | Event | Field | Method | Property | Indexer | Operator | Destructor | NestedMask,
-		All = MaskType
+		MaskType = Constructor | Event | Field | Method | Property | Indexer | Operator | Destructor | NestedMask
 	}
 
 	[Flags]
@@ -68,17 +71,7 @@ namespace Mono.CSharp {
 		public readonly MemberKind Kind;
 		public readonly AParametersCollection Parameters;
 		public readonly TypeSpec MemberType;
-
-		int arity; // -1 to ignore the check
-
-		private MemberFilter (string name, MemberKind kind)
-		{
-			Name = name;
-			Kind = kind;
-			Parameters = null;
-			MemberType = null;
-			arity = -1;
-		}
+		public readonly int Arity; // -1 to ignore the check
 
 		public MemberFilter (MethodSpec m)
 		{
@@ -86,7 +79,7 @@ namespace Mono.CSharp {
 			Kind = MemberKind.Method;
 			Parameters = m.Parameters;
 			MemberType = m.ReturnType;
-			arity = m.Arity;
+			Arity = m.Arity;
 		}
 
 		public MemberFilter (string name, int arity, MemberKind kind, AParametersCollection param, TypeSpec type)
@@ -95,7 +88,7 @@ namespace Mono.CSharp {
 			Kind = kind;
 			Parameters = param;
 			MemberType = type;
-			this.arity = arity;
+			this.Arity = arity;
 		}
 
 		public static MemberFilter Constructor (AParametersCollection param)
@@ -128,7 +121,7 @@ namespace Mono.CSharp {
 				return false;
 
 			// Check arity when not disabled
-			if (arity >= 0 && arity != other.Arity)
+			if (Arity >= 0 && Arity != other.Arity)
 				return false;
 
 			if (Parameters != null) {
@@ -168,6 +161,7 @@ namespace Mono.CSharp {
 	//
 	public class MemberCache
 	{
+		[Flags]
 		enum StateFlags
 		{
 			HasConversionOperator = 1 << 1,
@@ -177,7 +171,7 @@ namespace Mono.CSharp {
 		readonly Dictionary<string, IList<MemberSpec>> member_hash;
 		Dictionary<string, MemberSpec[]> locase_members;
 		IList<MethodSpec> missing_abstract;
-		StateFlags state;
+		StateFlags state;	// TODO: Move to TypeSpec or ITypeDefinition
 
 		public static readonly string IndexerNameAlias = "<this>";
 
@@ -232,7 +226,7 @@ namespace Mono.CSharp {
 						continue;
 
 					if (list is MemberSpec[]) {
-						list = new List<MemberSpec> () { list [0] };
+						list = new List<MemberSpec> { list [0] };
 						member_hash[entry.Key] = list;
 					}
 
@@ -246,6 +240,8 @@ namespace Mono.CSharp {
 		// contain all base interface members, so the Lookup code
 		// can use simple inheritance rules.
 		//
+		// Does not work recursively because of generic interfaces
+		//
 		public void AddInterface (TypeSpec iface)
 		{
 			var cache = iface.MemberCache;
@@ -264,6 +260,13 @@ namespace Mono.CSharp {
 				}
 
 				foreach (var ce in entry.Value) {
+					//
+					// When two or more different base interfaces implemenent common
+					// interface
+					//
+					// I : IA, IFoo
+					// IA : IFoo
+					//
 					if (list.Contains (ce))
 						continue;
 
@@ -271,12 +274,6 @@ namespace Mono.CSharp {
 						member_hash[entry.Key] = list;
 				}
 			}
-
-			// Add also all base interfaces
-			if (iface.Interfaces != null) {
-				foreach (var base_iface in iface.Interfaces)
-					AddInterface (base_iface);
-			}
 		}
 
 		public void AddMember (InterfaceMemberBase imb, string exlicitName, MemberSpec ms)
@@ -284,7 +281,7 @@ namespace Mono.CSharp {
 			// Explicit names cannot be looked-up but can be used for
 			// collision checking (no name mangling needed)
 			if (imb.IsExplicitImpl)
-				AddMember (exlicitName, ms);
+				AddMember (exlicitName, ms, false);
 			else
 				AddMember (ms);
 		}
@@ -294,23 +291,36 @@ namespace Mono.CSharp {
 		//
 		public void AddMember (MemberSpec ms)
 		{
-			AddMember (GetLookupName (ms), ms);
+			AddMember (GetLookupName (ms), ms, false);
 		}
 
-		void AddMember (string name, MemberSpec member)
+		void AddMember (string name, MemberSpec member, bool removeHiddenMembers)
 		{
 			if (member.Kind == MemberKind.Operator) {
 				var dt = member.DeclaringType;
-				if (dt == TypeManager.string_type || dt == TypeManager.delegate_type || dt == TypeManager.multicast_delegate_type) {
-					// Some core types have user operators but they cannot be used as normal
-					// user operators as they are predefined and therefore having different
-					// rules (e.g. binary operators) by not setting the flag we hide them for
-					// user conversions
-					// TODO: Should I do this for all core types ?
-				} else if (name == Operator.GetMetadataName (Operator.OpType.Implicit) || name == Operator.GetMetadataName (Operator.OpType.Explicit)) {
-					state |= StateFlags.HasConversionOperator;
-				} else {
-					state |= StateFlags.HasUserOperator;
+
+
+				//
+				// Some core types have user operators but they cannot be used like normal
+				// user operators as they are predefined and therefore having different
+				// rules (e.g. binary operators) by not setting the flag we hide them for
+				// user conversions
+				//
+				if (!BuiltinTypeSpec.IsPrimitiveType (dt) || dt.BuiltinType == BuiltinTypeSpec.Type.Char) {
+					switch (dt.BuiltinType) {
+					case BuiltinTypeSpec.Type.String:
+					case BuiltinTypeSpec.Type.Delegate:
+					case BuiltinTypeSpec.Type.MulticastDelegate:
+						break;
+					default:
+						if (name == Operator.GetMetadataName (Operator.OpType.Implicit) || name == Operator.GetMetadataName (Operator.OpType.Explicit)) {
+							state |= StateFlags.HasConversionOperator;
+						} else {
+							state |= StateFlags.HasUserOperator;
+						}
+
+						break;
+					}
 				}
 			}
 
@@ -320,12 +330,12 @@ namespace Mono.CSharp {
 				return;
 			}
 
-			if (member.DeclaringType.IsInterface) {
+			if (removeHiddenMembers && member.DeclaringType.IsInterface) {
 				if (AddInterfaceMember (member, ref list))
 					member_hash[name] = list;
 			} else {
-				if (list is MemberSpec[]) {
-					list = new List<MemberSpec> () { list[0] };
+				if (list.Count == 1) {
+					list = new List<MemberSpec> { list[0] };
 					member_hash[name] = list;
 				}
 
@@ -333,6 +343,11 @@ namespace Mono.CSharp {
 			}
 		}
 
+		public void AddMemberImported (MemberSpec ms)
+		{
+			AddMember (GetLookupName (ms), ms, true);
+		}
+
 		//
 		// Ignores any base interface member which can be hidden
 		// by this interface
@@ -360,7 +375,7 @@ namespace Mono.CSharp {
 				}
 
 				if (member.DeclaringType.ImplementsInterface (entry.DeclaringType, false)) {
-					if (existing is MemberSpec[]) {
+					if (existing.Count == 1) {
 						existing = new MemberSpec[] { member };
 						return true;
 					}
@@ -374,8 +389,8 @@ namespace Mono.CSharp {
 					return false;
 			}
 
-			if (existing is MemberSpec[]) {
-				existing = new List<MemberSpec> () { existing[0], member };
+			if (existing.Count == 1) {
+				existing = new List<MemberSpec> { existing[0], member };
 				return true;
 			}
 
@@ -424,12 +439,15 @@ namespace Mono.CSharp {
 		// A special method to work with member lookup only. It returns a list of all members named @name
 		// starting from @container. It's very performance sensitive
 		//
-		public static IList<MemberSpec> FindMembers (TypeSpec container, string name, bool declaredOnly)
+		// declaredOnlyClass cannot be used interfaces. Manual filtering is required because names are
+		// compacted
+		//
+		public static IList<MemberSpec> FindMembers (TypeSpec container, string name, bool declaredOnlyClass)
 		{
 			IList<MemberSpec> applicable;
 
 			do {
-				if (container.MemberCache.member_hash.TryGetValue (name, out applicable) || declaredOnly)
+				if (container.MemberCache.member_hash.TryGetValue (name, out applicable) || declaredOnlyClass)
 					return applicable;
 
 				container = container.BaseType;
@@ -446,6 +464,7 @@ namespace Mono.CSharp {
 			IList<MemberSpec> applicable;
 			TypeSpec best_match = null;
 			do {
+#if !FULL_AOT_RUNTIME
 				// TODO: Don't know how to handle this yet
 				// When resolving base type of nested type, parent type must have
 				// base type resolved to scan full hierarchy correctly
@@ -453,7 +472,8 @@ namespace Mono.CSharp {
 				// based on type definition
 				var tc = container.MemberDefinition as TypeContainer;
 				if (tc != null)
-					tc.DefineType ();
+					tc.DefineContainer ();
+#endif
 
 				if (container.MemberCacheTypes.member_hash.TryGetValue (name, out applicable)) {
 					for (int i = applicable.Count - 1; i >= 0; i--) {
@@ -484,7 +504,7 @@ namespace Mono.CSharp {
 		//
 		// Looks for extension methods with defined name and extension type
 		//
-		public List<MethodSpec> FindExtensionMethods (TypeContainer invocationType, TypeSpec extensionType, string name, int arity)
+		public List<MethodSpec> FindExtensionMethods (IMemberContext invocationContext, string name, int arity)
 		{
 			IList<MemberSpec> entries;
 			if (!member_hash.TryGetValue (name, out entries))
@@ -499,10 +519,13 @@ namespace Mono.CSharp {
 				if (!ms.IsExtensionMethod)
 					continue;
 
-				if (!ms.IsAccessible (invocationType.CurrentType))
+				if (!ms.IsAccessible (invocationContext))
 					continue;
 
-				if ((ms.DeclaringType.Modifiers & Modifiers.INTERNAL) != 0 && !ms.DeclaringType.MemberDefinition.IsInternalAsPublic (invocationType.DeclaringAssembly))
+				//
+				// Extension methods cannot be nested hence checking parent is enough
+				//
+				if ((ms.DeclaringType.Modifiers & Modifiers.INTERNAL) != 0 && !ms.DeclaringType.MemberDefinition.IsInternalAsPublic (invocationContext.Module.DeclaringAssembly))
 					continue;
 
 				if (candidates == null)
@@ -517,7 +540,7 @@ namespace Mono.CSharp {
 		// Returns base members of @member member if no exact match is found @bestCandidate returns
 		// the best match
 		//
-		public static MemberSpec FindBaseMember (MemberCore member, out MemberSpec bestCandidate)
+		public static MemberSpec FindBaseMember (MemberCore member, out MemberSpec bestCandidate, ref bool overrides)
 		{
 			bestCandidate = null;
 			var container = member.Parent.PartialContainer.Definition;
@@ -530,26 +553,24 @@ namespace Mono.CSharp {
 			}
 
 			string name = GetLookupName (member);
-			IList<MemberSpec> applicable;
 			var member_param = member is IParametersMember ? ((IParametersMember) member).Parameters : null;
 
 			var mkind = GetMemberCoreKind (member);
 			bool member_with_accessors = mkind == MemberKind.Indexer || mkind == MemberKind.Property;
 
+			IList<MemberSpec> applicable;
+			MemberSpec ambig_candidate = null;
+
 			do {
 				if (container.MemberCache.member_hash.TryGetValue (name, out applicable)) {
 					for (int i = 0; i < applicable.Count; ++i) {
 						var entry = applicable [i];
 
-						if ((entry.Modifiers & Modifiers.PRIVATE) != 0)
-							continue;
-
-						if ((entry.Modifiers & Modifiers.AccessibilityMask) == Modifiers.INTERNAL &&
-							!entry.DeclaringType.MemberDefinition.IsInternalAsPublic (member.Module.DeclaringAssembly))
+						if ((entry.Modifiers & Modifiers.PUBLIC) == 0 && !entry.IsAccessible (member))
 							continue;
 
 						//
-						// Is the member of same kind ?
+						// Isn't the member of same kind ?
 						//
 						if ((entry.Kind & ~MemberKind.Destructor & mkind & MemberKind.MaskType) == 0) {
 							// Destructors are ignored as they cannot be overridden by user
@@ -581,31 +602,39 @@ namespace Mono.CSharp {
 						}
 
 						//
-						// Skip override members with accessors they may not fully implement the base member
+						// Skip override for member with accessors. It may not fully implement the base member
+						// but keep flag we found an implementation in case the base member is abstract
 						//
-						if (member_with_accessors) {
-							if ((entry.Modifiers & (Modifiers.OVERRIDE | Modifiers.SEALED)) == Modifiers.OVERRIDE) {
-								//
-								// Set candidate to member override to flag we found an implementation
-								//
-								bestCandidate = entry;
-								continue;
-							}
-						} else {
-							bestCandidate = null;
+						if (member_with_accessors && ((entry.Modifiers & (Modifiers.OVERRIDE | Modifiers.SEALED)) == Modifiers.OVERRIDE)) {
+							//
+							// Set candidate to override implementation to flag we found an implementation
+							//
+							overrides = true;
+							continue;
 						}
 
+						//
+						// For members with parameters we can encounter an ambiguous candidates (they match exactly)
+						// because generic type parameters could be inflated into same types
+						//
+						if (ambig_candidate == null && (entry.Kind & mkind & (MemberKind.Method | MemberKind.Indexer)) != 0) {
+							bestCandidate = null;
+							ambig_candidate = entry;
+							continue;
+						}
+
+						bestCandidate = ambig_candidate;
 						return entry;
 					}
 				}
 
-				if (container.IsInterface)
+				if (container.IsInterface || ambig_candidate != null)
 					break;
 
 				container = container.BaseType;
 			} while (container != null);
 
-			return null;
+			return ambig_candidate;
 		}
 
 		//
@@ -657,7 +686,66 @@ namespace Mono.CSharp {
 			throw new NotImplementedException (member.GetType ().ToString ());
 		}
 
-		public static IList<MemberSpec> GetCompletitionMembers (TypeSpec container, string name)
+		public static List<FieldSpec> GetAllFieldsForDefiniteAssignment (TypeSpec container, IMemberContext context)
+		{
+			List<FieldSpec> fields = null;
+			bool imported = container.MemberDefinition.IsImported;
+			foreach (var entry in container.MemberCache.member_hash) {
+				foreach (var name_entry in entry.Value) {
+					if (name_entry.Kind != MemberKind.Field)
+						continue;
+
+					if ((name_entry.Modifiers & Modifiers.STATIC) != 0)
+						continue;
+
+					//
+					// Fixed size buffers are not subject to definite assignment checking
+					//
+					if (name_entry is FixedFieldSpec || name_entry is ConstSpec)
+						continue;
+
+					var fs = (FieldSpec) name_entry;
+					if (imported && ShouldIgnoreFieldForDefiniteAssignment (fs, context))
+						continue;
+
+					//if ((fs.Modifiers & (Modifiers.BACKING_FIELD) != 0)
+					//	continue;
+
+					if (fields == null)
+						fields = new List<FieldSpec> ();
+
+					fields.Add (fs);
+					break;
+				}
+			}
+
+			return fields ?? new List<FieldSpec> (0);
+		}
+
+		static bool ShouldIgnoreFieldForDefiniteAssignment (FieldSpec fs, IMemberContext context)
+		{
+			//
+			// LAMESPEC: This mimics csc quirk where definitive assignment is not done
+			// for all kinds of imported non-public struct fields
+			//
+			var mod = fs.Modifiers;
+			if ((mod & Modifiers.PRIVATE) == 0 && ((mod & Modifiers.INTERNAL) != 0 && fs.DeclaringType.MemberDefinition.IsInternalAsPublic (context.Module.DeclaringAssembly)))
+				return false;
+
+			//
+			// Ignore reference type fields except when type is an array or type parameter
+			//
+			var type = fs.MemberType;
+			switch (type.Kind) {
+			case MemberKind.ArrayType:
+			case MemberKind.TypeParameter:
+				return false;
+			default:
+				return TypeSpec.IsReferenceType (type);
+			}
+		}
+
+		public static IList<MemberSpec> GetCompletitionMembers (IMemberContext ctx, TypeSpec container, string name)
 		{
 			var matches = new List<MemberSpec> ();
 			foreach (var entry in container.MemberCache.member_hash) {
@@ -668,7 +756,7 @@ namespace Mono.CSharp {
 					if ((name_entry.Kind & (MemberKind.Constructor | MemberKind.Destructor | MemberKind.Operator)) != 0)
 						continue;
 
-					if (!name_entry.IsAccessible (InternalType.FakeInternalType))
+					if (!name_entry.IsAccessible (ctx))
 						continue;
 
 					if (name == null || name_entry.Name.StartsWith (name)) {
@@ -683,7 +771,7 @@ namespace Mono.CSharp {
 		//
 		// Returns members of @iface only, base members are ignored
 		//
-		public static IList<MethodSpec> GetInterfaceMethods (TypeSpec iface)
+		public static List<MethodSpec> GetInterfaceMethods (TypeSpec iface)
 		{
 			//
 			// MemberCache flatten interfaces, therefore in cases like this one
@@ -729,10 +817,11 @@ namespace Mono.CSharp {
 						if ((name_entry.Modifiers & Modifiers.ABSTRACT) == 0)
 							continue;
 
-						if (name_entry.Kind != MemberKind.Method)
+						var ms = name_entry as MethodSpec;
+						if (ms == null)
 							continue;
 
-						abstract_methods.Add ((MethodSpec) name_entry);
+						abstract_methods.Add (ms);
 					}
 				}
 
@@ -775,6 +864,12 @@ namespace Mono.CSharp {
 						if ((item.Modifiers & (Modifiers.OVERRIDE | Modifiers.VIRTUAL)) == 0)
 							continue;
 
+						//
+						// Abstract override does not override anything
+						//
+						if ((item.Modifiers & Modifiers.ABSTRACT) != 0)
+							continue;
+
 						if (filter.Equals (item)) {
 							--not_implemented_count;
 							abstract_methods [i] = null;
@@ -823,7 +918,7 @@ namespace Mono.CSharp {
 				return IndexerNameAlias;
 
 			if (mc is Constructor)
-				return Constructor.ConstructorName;
+				return mc.IsStatic ? Constructor.TypeConstructorName : Constructor.ConstructorName;
 
 			return mc.MemberName.Name;
 		}
@@ -834,7 +929,7 @@ namespace Mono.CSharp {
 		public static IList<MemberSpec> GetUserOperator (TypeSpec container, Operator.OpType op, bool declaredOnly)
 		{
 			IList<MemberSpec> found = null;
-
+			bool shared_list = true;
 			IList<MemberSpec> applicable;
 			do {
 				var mc = container.MemberCache;
@@ -864,10 +959,13 @@ namespace Mono.CSharp {
 									found = new List<MemberSpec> ();
 									found.Add (applicable[i]);
 								} else {
-									var prev = found as List<MemberSpec>;
-									if (prev == null) {
+									List<MemberSpec> prev;
+									if (shared_list) {
+										shared_list = false;
 										prev = new List<MemberSpec> (found.Count + 1);
 										prev.AddRange (found);
+									} else {
+										prev = (List<MemberSpec>) found;
 									}
 
 									prev.Add (applicable[i]);
@@ -876,12 +974,16 @@ namespace Mono.CSharp {
 						} else {
 							if (found == null) {
 								found = applicable;
+								shared_list = true;
 							} else {
-								var merged = found as List<MemberSpec>;
-								if (merged == null) {
+								List<MemberSpec> merged;
+								if (shared_list) {
+									shared_list = false;
 									merged = new List<MemberSpec> (found.Count + applicable.Count);
 									merged.AddRange (found);
 									found = merged;
+								} else {
+									merged = (List<MemberSpec>) found;
 								}
 
 								merged.AddRange (applicable);
@@ -1034,6 +1136,53 @@ namespace Mono.CSharp {
 			}
 		}
 
+		//
+		// Removes hidden base members of an interface. For compiled interfaces we cannot
+		// do name filtering during Add (as we do for import) because we need all base
+		// names to be valid during type definition.
+		// Add replaces hidden base member with current one which means any name collision
+		// (CS0108) of non-first name would be unnoticed because the name was replaced
+		// with the one from compiled type
+		//
+		public void RemoveHiddenMembers (TypeSpec container)
+		{
+			foreach (var entry in member_hash) {
+				var values = entry.Value;
+
+				int container_members_start_at = 0;
+				while (values[container_members_start_at].DeclaringType != container && ++container_members_start_at < entry.Value.Count);
+
+				if (container_members_start_at == 0 || container_members_start_at == values.Count)
+					continue;
+
+				for (int i = 0; i < container_members_start_at; ++i) {
+					var member = values[i];
+
+					if (!container.ImplementsInterface (member.DeclaringType, false))
+						continue;
+
+					var member_param = member is IParametersMember ? ((IParametersMember) member).Parameters : ParametersCompiled.EmptyReadOnlyParameters;
+
+					for (int ii = container_members_start_at; ii < values.Count; ++ii) {
+						var container_entry = values[ii];
+
+						if (container_entry.Arity != member.Arity)
+							continue;
+
+						if (container_entry is IParametersMember) {
+							if (!TypeSpecComparer.Override.IsEqual (((IParametersMember) container_entry).Parameters, member_param))
+								continue;
+						}
+
+						values.RemoveAt (i);
+						--container_members_start_at;
+						--ii;
+						--i;
+					}
+				}
+			}
+		}
+
 		//
 		// Checks all appropriate container members for CLS compliance
 		//
@@ -1045,8 +1194,9 @@ namespace Mono.CSharp {
 			if (container.BaseType == null) {
 				locase_members = new Dictionary<string, MemberSpec[]> (member_hash.Count); // StringComparer.OrdinalIgnoreCase);
 			} else {
-				container.BaseType.MemberCache.VerifyClsCompliance (container.BaseType, report);
-				locase_members = new Dictionary<string, MemberSpec[]> (container.BaseType.MemberCache.locase_members); //, StringComparer.OrdinalIgnoreCase);
+				var btype = container.BaseType.GetDefinition ();
+				btype.MemberCache.VerifyClsCompliance (btype, report);
+				locase_members = new Dictionary<string, MemberSpec[]> (btype.MemberCache.locase_members); //, StringComparer.OrdinalIgnoreCase);
 			}
 
 			var is_imported_type = container.MemberDefinition.IsImported;
@@ -1062,13 +1212,17 @@ namespace Mono.CSharp {
 					if ((name_entry.Kind & MemberKind.MaskType) == 0)
 						continue;
 
-					if (name_entry.MemberDefinition.IsNotCLSCompliant ())
+					if (name_entry.MemberDefinition.CLSAttributeValue == false)
 					    continue;
 
-					IParametersMember p_a = name_entry as IParametersMember;
-					if (p_a != null && !name_entry.IsAccessor) {
-						if (!is_imported_type) {
+					IParametersMember p_a = null;
+					if (!is_imported_type) {
+						p_a = name_entry as IParametersMember;
+						if (p_a != null && !name_entry.IsAccessor) {
 							var p_a_pd = p_a.Parameters;
+							//
+							// Check differing overloads in @container
+							//
 							for (int ii = i + 1; ii < entry.Value.Count; ++ii) {
 								var checked_entry = entry.Value[ii];
 								IParametersMember p_b = checked_entry as IParametersMember;
@@ -1083,24 +1237,7 @@ namespace Mono.CSharp {
 
 								var res = ParametersCompiled.IsSameClsSignature (p_a.Parameters, p_b.Parameters);
 								if (res != 0) {
-									var last = GetLaterDefinedMember (checked_entry, name_entry);
-									if (last == checked_entry.MemberDefinition) {
-										report.SymbolRelatedToPreviousError (name_entry);
-									} else {
-										report.SymbolRelatedToPreviousError (checked_entry);
-									}
-
-									if ((res & 1) != 0) {
-										report.Warning (3006, 1, last.Location,
-												"Overloaded method `{0}' differing only in ref or out, or in array rank, is not CLS-compliant",
-												name_entry.GetSignatureForError ());
-									}
-
-									if ((res & 2) != 0) {
-										report.Warning (3007, 1, last.Location,
-											"Overloaded method `{0}' differing only by unnamed array types is not CLS-compliant",
-											name_entry.GetSignatureForError ());
-									}
+									ReportOverloadedMethodClsDifference (name_entry, checked_entry, res, report);
 								}
 							}
 						}
@@ -1118,11 +1255,26 @@ namespace Mono.CSharp {
 					} else {
 						bool same_names_only = true;
 						foreach (var f in found) {
-							if (f.Name == name_entry.Name)
-								continue;
+							if (f.Name == name_entry.Name) {
+								if (p_a != null) {
+									IParametersMember p_b = f as IParametersMember;
+									if (p_b == null)
+										continue;
 
-//							if (f.IsAccessor && name_entry.IsAccessor)
-//								continue;
+									if (p_a.Parameters.Count != p_b.Parameters.Count)
+										continue;
+
+									if (f.IsAccessor)
+										continue;
+
+									var res = ParametersCompiled.IsSameClsSignature (p_a.Parameters, p_b.Parameters);
+									if (res != 0) {
+										ReportOverloadedMethodClsDifference (f, name_entry, res, report);
+									}
+								}
+
+								continue;
+							}
 
 							same_names_only = false;
 							if (!is_imported_type) {
@@ -1161,12 +1313,37 @@ namespace Mono.CSharp {
 			if (mc_b == null)
 				return mc_a;
 
+			if (a.DeclaringType.MemberDefinition != b.DeclaringType.MemberDefinition)
+				return mc_b;
+
 			if (mc_a.Location.File != mc_a.Location.File)
 				return mc_b;
 
 			return mc_b.Location.Row > mc_a.Location.Row ? mc_b : mc_a;
 		}
 
+		static void ReportOverloadedMethodClsDifference (MemberSpec a, MemberSpec b, int res, Report report)
+		{
+			var last = GetLaterDefinedMember (a, b);
+			if (last == a.MemberDefinition) {
+				report.SymbolRelatedToPreviousError (b);
+			} else {
+				report.SymbolRelatedToPreviousError (a);
+			}
+
+			if ((res & 1) != 0) {
+				report.Warning (3006, 1, last.Location,
+						"Overloaded method `{0}' differing only in ref or out, or in array rank, is not CLS-compliant",
+						last.GetSignatureForError ());
+			}
+
+			if ((res & 2) != 0) {
+				report.Warning (3007, 1, last.Location,
+					"Overloaded method `{0}' differing only by unnamed array types is not CLS-compliant",
+					last.GetSignatureForError ());
+			}
+		}
+
 		public bool CheckExistingMembersOverloads (MemberCore member, AParametersCollection parameters)
 		{
 			var name = GetLookupName (member);
@@ -1209,8 +1386,10 @@ namespace Mono.CSharp {
 						type_a = parameters.Types [ii];
 						type_b = p_types [ii];
 
-						if ((pd.FixedParameters [ii].ModFlags & Parameter.Modifier.ISBYREF) !=
-							(parameters.FixedParameters [ii].ModFlags & Parameter.Modifier.ISBYREF))
+						var a_byref = (pd.FixedParameters[ii].ModFlags & Parameter.Modifier.RefOutMask) != 0;
+						var b_byref = (parameters.FixedParameters[ii].ModFlags & Parameter.Modifier.RefOutMask) != 0;
+
+						if (a_byref != b_byref)
 							break;
 
 					} while (TypeSpecComparer.Override.IsEqual (type_a, type_b) && ii-- != 0);
@@ -1229,7 +1408,9 @@ namespace Mono.CSharp {
 					//
 					if (pd != null && member is MethodCore) {
 						ii = method_param_count;
-						while (ii-- != 0 && parameters.FixedParameters[ii].ModFlags == pd.FixedParameters[ii].ModFlags &&
+						while (ii-- != 0 &&
+							(parameters.FixedParameters[ii].ModFlags & Parameter.Modifier.ModifierMask) ==
+							(pd.FixedParameters[ii].ModFlags & Parameter.Modifier.ModifierMask) &&
 							parameters.ExtensionMethodType == pd.ExtensionMethodType) ;
 
 						if (ii >= 0) {
@@ -1284,9 +1465,12 @@ namespace Mono.CSharp {
 									"A partial method declaration and partial method implementation must be both `static' or neither");
 							}
 
-							Report.SymbolRelatedToPreviousError (ce);
-							Report.Error (764, member.Location,
-								"A partial method declaration and partial method implementation must be both `unsafe' or neither");
+							if ((method_a.ModFlags & Modifiers.UNSAFE) != (method_b.ModFlags & Modifiers.UNSAFE)) {
+								Report.SymbolRelatedToPreviousError (ce);
+								Report.Error (764, member.Location,
+									"A partial method declaration and partial method implementation must be both `unsafe' or neither");
+							}
+
 							return false;
 						}
 
