@@ -29,6 +29,8 @@
 
 using System;
 using System.Collections;
+using System.Linq;
+
 using Mono.Cecil;
 using Mono.Cecil.Cil;
 
@@ -36,9 +38,13 @@ namespace Mono.Linker.Steps {
 
 	public class MarkStep : IStep {
 
-		LinkContext _context;
-		Queue _methods;
-		ArrayList _virtual_methods;
+		protected LinkContext _context;
+		protected Queue _methods;
+		protected ArrayList _virtual_methods;
+
+		public AnnotationStore Annotations {
+			get { return _context.Annotations; }
+		}
 
 		public MarkStep ()
 		{
@@ -46,7 +52,7 @@ namespace Mono.Linker.Steps {
 			_virtual_methods = new ArrayList ();
 		}
 
-		public void Process (LinkContext context)
+		public virtual void Process (LinkContext context)
 		{
 			_context = context;
 
@@ -79,8 +85,13 @@ namespace Mono.Linker.Steps {
 				InitializeFields (type);
 			if (type.HasMethods)
 				InitializeMethods (type.Methods);
-			if (type.HasConstructors)
-				InitializeMethods (type.Constructors);
+
+			if (type.HasNestedTypes) {
+				foreach (var nested in type.NestedTypes) {
+					if (Annotations.IsMarked (nested))
+						InitializeType (nested);
+				}
+			}
 		}
 
 		void InitializeFields (TypeDefinition type)
@@ -157,27 +168,16 @@ namespace Mono.Linker.Steps {
 			ProcessVirtualMethod (method);
 		}
 
-		void MarkMethodBody (MethodBody body)
+		void MarkMarshalSpec (IMarshalInfoProvider spec)
 		{
-			foreach (VariableDefinition var in body.Variables)
-				MarkType (var.VariableType);
+			if (!spec.HasMarshalInfo)
+				return;
 
-			foreach (ExceptionHandler eh in body.ExceptionHandlers)
-				if (eh.Type == ExceptionHandlerType.Catch)
-					MarkType (eh.CatchType);
-
-			foreach (Instruction instruction in body.Instructions)
-				MarkInstruction (instruction);
-		}
-
-		void MarkMarshalSpec (IHasMarshalSpec spec)
-		{
-			CustomMarshalerSpec marshaler = spec.MarshalSpec as CustomMarshalerSpec;
+			var marshaler = spec.MarshalInfo as CustomMarshalInfo;
 			if (marshaler == null)
 				return;
 
-			TypeDefinition type = _context.GetType (marshaler.ManagedType);
-			MarkType (type);
+			MarkType (marshaler.ManagedType);
 		}
 
 		void MarkCustomAttributes (ICustomAttributeProvider provider)
@@ -189,19 +189,11 @@ namespace Mono.Linker.Steps {
 				MarkCustomAttribute (ca);
 		}
 
-		void MarkCustomAttribute (CustomAttribute ca)
+		protected virtual void MarkCustomAttribute (CustomAttribute ca)
 		{
 			MarkMethod (ca.Constructor);
 
-			if (!ca.Resolved) {
-				ca = ca.Clone ();
-				ca.Resolve ();
-			}
-
-			if (!ca.Resolved)
-				return;
-
-			MarkCustomAttributeParameters (ca);
+			MarkCustomAttributeArguments (ca);
 
 			TypeReference constructor_type = ca.Constructor.DeclaringType;
 			TypeDefinition type = constructor_type.Resolve ();
@@ -212,26 +204,80 @@ namespace Mono.Linker.Steps {
 			MarkCustomAttributeFields (ca, type);
 		}
 
-		void MarkCustomAttributeProperties (CustomAttribute ca, TypeDefinition attribute)
+		protected void MarkSecurityDeclarations (ISecurityDeclarationProvider provider)
 		{
-			foreach (DictionaryEntry de in ca.Properties) {
-				string propertyname = (string) de.Key;
+			// most security declarations are removed (if linked) but user code might still have some
+			// and if the attribtues references types then they need to be marked too
+			if ((provider == null) || !provider.HasSecurityDeclarations)
+				return;
 
-				PropertyDefinition property = GetProperty (attribute, propertyname);
-				if (property != null)
-					MarkMethod (property.SetMethod);
+			foreach (var sd in provider.SecurityDeclarations)
+				MarkSecurityDeclaration (sd);
+		}
 
-				TypeReference propType = ca.GetPropertyType (propertyname);
-				MarkIfType (propType, de.Value);
-			}
+		protected virtual void MarkSecurityDeclaration (SecurityDeclaration sd)
+		{
+			if (!sd.HasSecurityAttributes)
+				return;
+			
+			foreach (var sa in sd.SecurityAttributes)
+				MarkSecurityAttribute (sa);
+		}
+
+		protected virtual void MarkSecurityAttribute (SecurityAttribute sa)
+		{
+			TypeReference security_type = sa.AttributeType;
+			TypeDefinition type = security_type.Resolve ();
+			if (type == null)
+				throw new ResolutionException (security_type);
+			
+			MarkType (security_type);
+			MarkSecurityAttributeProperties (sa, type);
+			MarkSecurityAttributeFields (sa, type);
+		}
+
+		protected void MarkSecurityAttributeProperties (SecurityAttribute sa, TypeDefinition attribute)
+		{
+			if (!sa.HasProperties)
+				return;
+
+			foreach (var named_argument in sa.Properties)
+				MarkCustomAttributeProperty (named_argument, attribute);
+		}
+
+		protected void MarkSecurityAttributeFields (SecurityAttribute sa, TypeDefinition attribute)
+		{
+			if (!sa.HasFields)
+				return;
+
+			foreach (var named_argument in sa.Fields)
+				MarkCustomAttributeField (named_argument, attribute);
+		}
+
+		protected void MarkCustomAttributeProperties (CustomAttribute ca, TypeDefinition attribute)
+		{
+			if (!ca.HasProperties)
+				return;
+
+			foreach (var named_argument in ca.Properties)
+				MarkCustomAttributeProperty (named_argument, attribute);
+		}
+
+		protected void MarkCustomAttributeProperty (CustomAttributeNamedArgument namedArgument, TypeDefinition attribute)
+		{
+			PropertyDefinition property = GetProperty (attribute, namedArgument.Name);
+			if (property != null)
+				MarkMethod (property.SetMethod);
+
+			MarkIfType (namedArgument.Argument);
 		}
 
 		PropertyDefinition GetProperty (TypeDefinition type, string propertyname)
 		{
 			while (type != null) {
-				PropertyDefinition [] properties = type.Properties.GetProperties (propertyname);
-				if (properties != null && properties.Length != 0 && properties [0].SetMethod != null)
-					return properties [0];
+				PropertyDefinition property = type.Properties.FirstOrDefault (p => p.Name == propertyname);
+				if (property != null)
+					return property;
 
 				type = type.BaseType != null ? ResolveTypeDefinition (type.BaseType) : null;
 			}
@@ -239,24 +285,28 @@ namespace Mono.Linker.Steps {
 			return null;
 		}
 
-		void MarkCustomAttributeFields (CustomAttribute ca, TypeDefinition attribute)
+		protected void MarkCustomAttributeFields (CustomAttribute ca, TypeDefinition attribute)
 		{
-			foreach (DictionaryEntry de in ca.Fields) {
-				string fieldname = (string) de.Key;
+			if (!ca.HasFields)
+				return;
 
-				FieldDefinition field = GetField (attribute, fieldname);
-				if (field != null)
-					MarkField (field);
+			foreach (var named_argument in ca.Fields)
+				MarkCustomAttributeField (named_argument, attribute);
+		}
 
-				TypeReference fieldType = ca.GetFieldType (fieldname);
-				MarkIfType (fieldType, de.Value);
-			}
+		protected void MarkCustomAttributeField (CustomAttributeNamedArgument namedArgument, TypeDefinition attribute)
+		{
+			FieldDefinition field = GetField (attribute, namedArgument.Name);
+			if (field != null)
+				MarkField (field);
+
+			MarkIfType (namedArgument.Argument);
 		}
 
 		FieldDefinition GetField (TypeDefinition type, string fieldname)
 		{
 			while (type != null) {
-				FieldDefinition field = type.Fields.GetField (fieldname);
+				FieldDefinition field = type.Fields.FirstOrDefault (f => f.Name == fieldname);
 				if (field != null)
 					return field;
 
@@ -266,25 +316,46 @@ namespace Mono.Linker.Steps {
 			return null;
 		}
 
-		void MarkCustomAttributeParameters (CustomAttribute ca)
+		void MarkCustomAttributeArguments (CustomAttribute ca)
 		{
-			for (int i = 0; i < ca.Constructor.Parameters.Count; i++) {
-				ParameterDefinition param = ca.Constructor.Parameters [i];
-				MarkIfType (param.ParameterType, ca.ConstructorParameters [i]);
+			if (!ca.HasConstructorArguments)
+				return;
+
+			foreach (var argument in ca.ConstructorArguments)
+				MarkIfType (argument);
+		}
+
+		void MarkIfType (CustomAttributeArgument argument)
+		{
+			var at = argument.Type;
+			if (at.IsArray) {
+				var et = at.GetElementType ();
+				if (et.Namespace != "System" || et.Name != "Type")
+					return;
+
+				MarkType (et);
+				foreach (var cac in (CustomAttributeArgument[]) argument.Value)
+					MarkWithResolvedScope ((TypeReference) cac.Value);
+			} else if (at.Namespace == "System" && at.Name == "Type") {
+				MarkType (argument.Type);
+				MarkWithResolvedScope ((TypeReference) argument.Value);
 			}
 		}
 
-		void MarkIfType (TypeReference slotType, object value)
+		// custom attributes encoding means it's possible to have a scope that will point into a PCL facade
+		// even if we (just before saving) will resolve all type references (bug #26752)
+		void MarkWithResolvedScope (TypeReference type)
 		{
-			if (slotType.FullName != Constants.Type)
+			// we cannot set the Scope of a TypeSpecification so there's no point in resolving it
+			if ((type == null) || (type is TypeSpecification))
 				return;
-
-			TypeDefinition type = _context.GetType ((string) value);
-
+			var td = type.Resolve ();
+			if (td != null)
+				type.Scope = td.Scope;
 			MarkType (type);
 		}
 
-		protected static bool CheckProcessed (IAnnotationProvider provider)
+		protected bool CheckProcessed (IMetadataTokenProvider provider)
 		{
 			if (Annotations.IsProcessed (provider))
 				return true;
@@ -293,22 +364,42 @@ namespace Mono.Linker.Steps {
 			return false;
 		}
 
-		void MarkAssembly (AssemblyDefinition assembly)
+		protected void MarkAssembly (AssemblyDefinition assembly)
 		{
 			if (CheckProcessed (assembly))
 				return;
 
+			ProcessModule (assembly);
+
 			MarkCustomAttributes (assembly);
+			MarkSecurityDeclarations (assembly);
 
 			foreach (ModuleDefinition module in assembly.Modules)
 				MarkCustomAttributes (module);
 		}
 
-		void MarkField (FieldReference reference)
+		void ProcessModule (AssemblyDefinition assembly)
+		{
+			// Pre-mark <Module> if there is any methods as they need to be executed 
+			// at assembly load time
+			foreach (TypeDefinition type in assembly.MainModule.Types)
+			{
+				if (type.Name == "<Module>" && type.HasMethods)
+				{
+					MarkType (type);
+					break;
+				}
+			}
+		}
+
+		protected void MarkField (FieldReference reference)
 		{
 //			if (IgnoreScope (reference.DeclaringType.Scope))
 //				return;
 
+			if (reference.DeclaringType is GenericInstanceType)
+				MarkType (reference.DeclaringType);
+
 			FieldDefinition field = ResolveFieldDefinition (reference);
 
 			if (field == null)
@@ -342,22 +433,28 @@ namespace Mono.Linker.Steps {
 
 		void MarkScope (IMetadataScope scope)
 		{
-			IAnnotationProvider provider = scope as IAnnotationProvider;
+			var provider = scope as IMetadataTokenProvider;
 			if (provider == null)
 				return;
 
 			Annotations.Mark (provider);
 		}
 
-		protected virtual void MarkType (TypeReference reference)
+		protected virtual void MarkSerializable (TypeDefinition type)
+		{
+			MarkDefaultConstructor (type);
+			MarkMethodsIf (type.Methods, IsSpecialSerializationConstructorPredicate);
+		}
+
+		protected virtual TypeDefinition MarkType (TypeReference reference)
 		{
 			if (reference == null)
-				return;
+				return null;
 
 			reference = GetOriginalType (reference);
 
 			if (reference is GenericParameter)
-				return;
+				return null;
 
 //			if (IgnoreScope (reference.Scope))
 //				return;
@@ -368,44 +465,44 @@ namespace Mono.Linker.Steps {
 				throw new ResolutionException (reference);
 
 			if (CheckProcessed (type))
-				return;
+				return null;
 
 			MarkScope (type.Scope);
 			MarkType (type.BaseType);
 			MarkType (type.DeclaringType);
 			MarkCustomAttributes (type);
+			MarkSecurityDeclarations (type);
 
 			if (IsMulticastDelegate (type)) {
-				MarkMethodCollection (type.Constructors);
 				MarkMethodCollection (type.Methods);
 			}
 
-			if (IsSerializable (type) && type.HasConstructors) {
-				MarkMethodsIf (type.Constructors, IsDefaultConstructorPredicate);
-				MarkMethodsIf (type.Constructors, IsSpecialSerializationConstructorPredicate);
-			}
+			if (IsSerializable (type))
+				MarkSerializable (type);
 
 			MarkTypeSpecialCustomAttributes (type);
 
 			MarkGenericParameterProvider (type);
 
-			if (type.IsValueType)
-				MarkFields (type);
+			// keep fields for value-types and for classes with LayoutKind.Sequential or Explicit
+			if (type.IsValueType || !type.IsAutoLayout)
+				MarkFields (type, type.IsEnum);
 
 			if (type.HasInterfaces) {
 				foreach (TypeReference iface in type.Interfaces)
 					MarkType (iface);
 			}
 
-			if (type.HasMethods)
+			if (type.HasMethods) {
 				MarkMethodsIf (type.Methods, IsVirtualAndHasPreservedParent);
-
-			if (type.HasConstructors)
-				MarkMethodsIf (type.Constructors, IsStaticConstructorPredicate);
+				MarkMethodsIf (type.Methods, IsStaticConstructorPredicate);
+			}
 
 			Annotations.Mark (type);
 
 			ApplyPreserveInfo (type);
+
+			return type;
 		}
 
 		void MarkTypeSpecialCustomAttributes (TypeDefinition type)
@@ -449,15 +546,15 @@ namespace Mono.Linker.Steps {
 		{
 			argument = null;
 
-			if (!attribute.Resolved || attribute.ConstructorParameters.Count < 1)
+			if (attribute.ConstructorArguments.Count < 1)
 				return false;
 
-			argument = attribute.ConstructorParameters [0] as string;
+			argument = attribute.ConstructorArguments [0].Value as string;
 
 			return argument != null;
 		}
 
-		void MarkNamedMethod (TypeDefinition type, string method_name)
+		protected void MarkNamedMethod (TypeDefinition type, string method_name)
 		{
 			if (!type.HasMethods)
 				return;
@@ -550,7 +647,7 @@ namespace Mono.Linker.Steps {
 			if (!IsConstructor (method))
 				return false;
 
-			ParameterDefinitionCollection parameters = method.Parameters;
+			var parameters = method.Parameters;
 			if (parameters.Count != 2)
 				return false;
 
@@ -571,21 +668,27 @@ namespace Mono.Linker.Steps {
 
 		static bool IsDefaultConstructor (MethodDefinition method)
 		{
-			return IsConstructor (method) && method.Parameters.Count == 0;
+			return IsConstructor (method) && !method.HasParameters;
 		}
 
 		static bool IsConstructor (MethodDefinition method)
 		{
-			return method.Name == MethodDefinition.Ctor && method.IsSpecialName &&
-				method.IsRuntimeSpecialName;
+			return method.IsConstructor && !method.IsStatic;
+		}
+
+		protected void MarkDefaultConstructor (TypeDefinition type)
+		{
+			if ((type == null) || !type.HasMethods)
+				return;
+
+			MarkMethodsIf (type.Methods, IsDefaultConstructorPredicate);
 		}
 
 		static MethodPredicate IsStaticConstructorPredicate = new MethodPredicate (IsStaticConstructor);
 
 		static bool IsStaticConstructor (MethodDefinition method)
 		{
-			return method.Name == MethodDefinition.Cctor && method.IsSpecialName &&
-				method.IsRuntimeSpecialName;
+			return method.IsConstructor && method.IsStatic;
 		}
 
 		static bool IsSerializable (TypeDefinition td)
@@ -598,7 +701,7 @@ namespace Mono.Linker.Steps {
 			return td.BaseType != null && td.BaseType.FullName == "System.MulticastDelegate";
 		}
 
-		TypeDefinition ResolveTypeDefinition (TypeReference type)
+		protected TypeDefinition ResolveTypeDefinition (TypeReference type)
 		{
 			TypeDefinition td = type as TypeDefinition;
 			if (td == null)
@@ -614,7 +717,7 @@ namespace Mono.Linker.Steps {
 				if (git != null)
 					MarkGenericArguments (git);
 
-				ModType mod = type as ModType;
+				var mod = type as IModifierType;
 				if (mod != null)
 					MarkModifierType (mod);
 
@@ -624,7 +727,7 @@ namespace Mono.Linker.Steps {
 			return type;
 		}
 
-		void MarkModifierType (ModType mod)
+		void MarkModifierType (IModifierType mod)
 		{
 			MarkType (mod.ModifierType);
 		}
@@ -661,7 +764,7 @@ namespace Mono.Linker.Steps {
 				if (argument_definition == null)
 					continue;
 
-				MarkMethodsIf (argument_definition.Constructors, ctor => !ctor.IsStatic && !ctor.HasParameters);
+				MarkMethodsIf (argument_definition.Methods, ctor => !ctor.IsStatic && !ctor.HasParameters);
 			}
 		}
 
@@ -669,11 +772,11 @@ namespace Mono.Linker.Steps {
 		{
 			var method = instance as GenericInstanceMethod;
 			if (method != null)
-				return method.ElementMethod;
+				return ResolveMethodDefinition (method.ElementMethod);
 
 			var type = instance as GenericInstanceType;
 			if (type != null)
-				return type.ElementType;
+				return ResolveTypeDefinition (type.ElementType);
 
 			return null;
 		}
@@ -687,11 +790,11 @@ namespace Mono.Linker.Steps {
 
 			switch (Annotations.GetPreserve (type)) {
 			case TypePreserve.All:
-				MarkFields (type);
+				MarkFields (type, true);
 				MarkMethods (type);
 				break;
 			case TypePreserve.Fields:
-				MarkFields (type);
+				MarkFields (type, true);
 				break;
 			case TypePreserve.Methods:
 				MarkMethods (type);
@@ -705,25 +808,34 @@ namespace Mono.Linker.Steps {
 			if (list == null)
 				return;
 
-			foreach (MethodDefinition method in list)
-				MarkMethod (method);
+			MarkMethodCollection (list);
 		}
 
-		void MarkFields (TypeDefinition type)
+		void ApplyPreserveMethods (MethodDefinition method)
+		{
+			var list = Annotations.GetPreservedMethods (method);
+			if (list == null)
+				return;
+
+			MarkMethodCollection (list);
+		}
+
+		protected void MarkFields (TypeDefinition type, bool includeStatic)
 		{
 			if (!type.HasFields)
 				return;
 
-			foreach (FieldDefinition field in type.Fields)
+			foreach (FieldDefinition field in type.Fields) {
+				if (!includeStatic && field.IsStatic)
+					continue;
 				MarkField (field);
+			}
 		}
 
-		void MarkMethods (TypeDefinition type)
+		protected virtual void MarkMethods (TypeDefinition type)
 		{
 			if (type.HasMethods)
 				MarkMethodCollection (type.Methods);
-			if (type.HasConstructors)
-				MarkMethodCollection (type.Constructors);
 		}
 
 		void MarkMethodCollection (IEnumerable methods)
@@ -732,12 +844,15 @@ namespace Mono.Linker.Steps {
 				MarkMethod (method);
 		}
 
-		void MarkMethod (MethodReference reference)
+		protected virtual MethodDefinition MarkMethod (MethodReference reference)
 		{
 			reference = GetOriginalMethod (reference);
 
 			if (reference.DeclaringType is ArrayType)
-				return;
+				return null;
+
+			if (reference.DeclaringType is GenericInstanceType)
+				MarkType (reference.DeclaringType);
 
 //			if (IgnoreScope (reference.DeclaringType.Scope))
 //				return;
@@ -751,6 +866,7 @@ namespace Mono.Linker.Steps {
 				Annotations.SetAction (method, MethodAction.Parse);
 
 			EnqueueMethod (method);
+			return method;
 		}
 
 		AssemblyDefinition ResolveAssembly (IMetadataScope scope)
@@ -760,7 +876,7 @@ namespace Mono.Linker.Steps {
 			return assembly;
 		}
 
-		MethodReference GetOriginalMethod (MethodReference method)
+		protected MethodReference GetOriginalMethod (MethodReference method)
 		{
 			while (method is MethodSpecification) {
 				GenericInstanceMethod gim = method as GenericInstanceMethod;
@@ -782,13 +898,14 @@ namespace Mono.Linker.Steps {
 			return md;
 		}
 
-		void ProcessMethod (MethodDefinition method)
+		protected virtual void ProcessMethod (MethodDefinition method)
 		{
 			if (CheckProcessed (method))
 				return;
 
 			MarkType (method.DeclaringType);
 			MarkCustomAttributes (method);
+			MarkSecurityDeclarations (method);
 
 			MarkGenericParameterProvider (method);
 
@@ -817,14 +934,16 @@ namespace Mono.Linker.Steps {
 
 			MarkBaseMethods (method);
 
-			MarkType (method.ReturnType.ReturnType);
-			MarkCustomAttributes (method.ReturnType);
-			MarkMarshalSpec (method.ReturnType);
+			MarkType (method.ReturnType);
+			MarkCustomAttributes (method.MethodReturnType);
+			MarkMarshalSpec (method.MethodReturnType);
 
 			if (ShouldParseMethodBody (method))
 				MarkMethodBody (method.Body);
 
 			Annotations.Mark (method);
+
+			ApplyPreserveMethods (method);
 		}
 
 		void MarkBaseMethods (MethodDefinition method)
@@ -834,6 +953,9 @@ namespace Mono.Linker.Steps {
 				return;
 
 			foreach (MethodDefinition base_method in base_methods) {
+				if (base_method.DeclaringType.IsInterface && !method.DeclaringType.IsInterface)
+					continue;
+
 				MarkMethod (base_method);
 				MarkBaseMethods (base_method);
 			}
@@ -849,7 +971,7 @@ namespace Mono.Linker.Steps {
 				(Annotations.GetAction (assembly) == AssemblyAction.Link && Annotations.GetAction (method) == MethodAction.Parse));
 		}
 
-		static bool IsPropertyMethod (MethodDefinition md)
+		static internal bool IsPropertyMethod (MethodDefinition md)
 		{
 			return (md.SemanticsAttributes & MethodSemanticsAttributes.Getter) != 0 ||
 				(md.SemanticsAttributes & MethodSemanticsAttributes.Setter) != 0;
@@ -862,7 +984,7 @@ namespace Mono.Linker.Steps {
 				(md.SemanticsAttributes & MethodSemanticsAttributes.RemoveOn) != 0;
 		}
 
-		static PropertyDefinition GetProperty (MethodDefinition md)
+		static internal PropertyDefinition GetProperty (MethodDefinition md)
 		{
 			TypeDefinition declaringType = (TypeDefinition) md.DeclaringType;
 			foreach (PropertyDefinition prop in declaringType.Properties)
@@ -882,12 +1004,12 @@ namespace Mono.Linker.Steps {
 			return null;
 		}
 
-		void MarkProperty (PropertyDefinition prop)
+		protected void MarkProperty (PropertyDefinition prop)
 		{
 			MarkCustomAttributes (prop);
 		}
 
-		void MarkEvent (EventDefinition evt)
+		protected void MarkEvent (EventDefinition evt)
 		{
 			MarkCustomAttributes (evt);
 			MarkMethodIfNotNull (evt.AddMethod);
@@ -903,7 +1025,20 @@ namespace Mono.Linker.Steps {
 			MarkMethod (method);
 		}
 
-		void MarkInstruction (Instruction instruction)
+		protected virtual void MarkMethodBody (MethodBody body)
+		{
+			foreach (VariableDefinition var in body.Variables)
+				MarkType (var.VariableType);
+
+			foreach (ExceptionHandler eh in body.ExceptionHandlers)
+				if (eh.HandlerType == ExceptionHandlerType.Catch)
+					MarkType (eh.CatchType);
+
+			foreach (Instruction instruction in body.Instructions)
+				MarkInstruction (instruction);
+		}
+
+		protected virtual void MarkInstruction (Instruction instruction)
 		{
 			switch (instruction.OpCode.OperandType) {
 			case OperandType.InlineField:
