@@ -19,9 +19,7 @@ using System.Text;
 using IKVM.Reflection;
 
 
-#if NET_4_5
 using System.Threading.Tasks;
-#endif
 
 class MakeBundle {
 	static string output = "a.out";
@@ -704,6 +702,7 @@ void          mono_register_config_for_assembly (const char* assembly_name, cons
 			return system (cmdLine);
 		}
 		
+#if XAMARIN_ANDROID
 		// on Windows, we have to pipe the output of a
 		// `cmd` interpolation to dos2unix, because the shell does not
 		// strip the CRLFs generated by the native pkg-config distributed
@@ -715,9 +714,14 @@ void          mono_register_config_for_assembly (const char* assembly_name, cons
 		if (use_dos2unix == null) {
 			use_dos2unix = false;
 			try {
-				var dos2unix = Process.Start ("dos2unix");
+				var info = new ProcessStartInfo ("dos2unix");
+				info.CreateNoWindow = true;
+				info.RedirectStandardInput = true;
+				info.UseShellExecute = false;
+				var dos2unix = Process.Start (info);
 				dos2unix.StandardInput.WriteLine ("aaa");
 				dos2unix.StandardInput.WriteLine ("\u0004");
+				dos2unix.StandardInput.Close ();
 				dos2unix.WaitForExit ();
 				if (dos2unix.ExitCode == 0)
 					use_dos2unix = true;
@@ -727,6 +731,7 @@ void          mono_register_config_for_assembly (const char* assembly_name, cons
 		}
 		// and if there is no dos2unix, just run cmd /c.
 		if (use_dos2unix == false) {
+#endif
 			Console.WriteLine (cmdLine);
 			ProcessStartInfo dos2unix = new ProcessStartInfo ();
 			dos2unix.UseShellExecute = false;
@@ -737,15 +742,19 @@ void          mono_register_config_for_assembly (const char* assembly_name, cons
 				p.WaitForExit ();
 				return p.ExitCode;
 			}
+#if XAMARIN_ANDROID
 		}
+#endif
 
 		StringBuilder b = new StringBuilder ();
 		int count = 0;
 		for (int i = 0; i < cmdLine.Length; i++) {
 			if (cmdLine [i] == '`') {
+#if XAMARIN_ANDROID
 				if (count % 2 != 0) {
 					b.Append ("|dos2unix");
 				}
+#endif
 				count++;
 			}
 			b.Append (cmdLine [i]);
