@@ -6,6 +6,7 @@
 #define S390X_H
 #include <glib.h>
 #include <assert.h>
+#include <limits.h>
 
 #define FLOAT_REGS 	2	/* No. float registers for parms    */
 #define GENERAL_REGS 	5	/* No. general registers for parms  */
@@ -136,16 +137,21 @@ typedef enum {
 	s390_fpc = 256,
 } S390SpecialRegister;
 
-#define s390_is_imm16(val) 		((glong)val >= (glong)-(1<<15) && \
-					 (glong)val <= (glong)((1<<15)-1))
-#define s390_is_uimm16(val) 		((glong)val >= 0 && (glong)val <= 65535)
+#define s390_is_imm16(val) 		((glong)val >= (glong) SHRT_MIN && \
+					 (glong)val <= (glong) SHRT_MAX)
+#define s390_is_imm32(val) 		((glong)val >= (glong) INT_MIN && \
+					 (glong)val <= (glong) INT_MAX)
+#define s390_is_uimm16(val) 		((glong)val >= 0 && (glong)val <= (glong) USHRT_MAX)
+#define s390_is_uimm32(val) 		((glong)val >= 0 && (glong)val <= (glong) UINT_MAX)
 #define s390_is_uimm20(val) 		((glong)val >= 0 && (glong)val <= 1048575)
 #define s390_is_imm20(val) 		((glong)val >= -524288 && (glong)val <= 524287)
-#define s390_is_imm12(val)		((glong)val >= (glong)-(1<<11) && \
-					 (glong)val <= (glong)((1<<11)-1))
+#define s390_is_imm12(val)		((glong)val >= (glong)-4096 && \
+					 (glong)val <= (glong)4095)
 #define s390_is_uimm12(val)		((glong)val >= 0 && (glong)val <= 4095)
 
 #define STK_BASE			s390_r15
+#define S390_SP				s390_r15
+#define S390_FP				s390_r11
 #define S390_MINIMAL_STACK_SIZE		160
 #define S390_REG_SAVE_OFFSET 		48
 #define S390_PARM_SAVE_OFFSET 		16
@@ -335,7 +341,7 @@ typedef struct {
 
 typedef struct {
 	char 	op1;
-	char	r1 : 4;
+	char	m1 : 4;
 	char	op2 : 4;
 	short	i2;
 } RI_Format;
@@ -347,7 +353,36 @@ typedef struct {
 	short	i2;
 	char	xx;
 	char	op2;
-} RIE_Format;
+} RIE_Format_1;
+
+typedef struct {
+	char 	op1;
+	char	r1 : 4;
+	char	r3 : 4;
+	short	i2;
+	char	m2 : 4;
+	char    xx : 4;
+	char	op2;
+} RIE_Format_2;
+
+typedef struct {
+	char 	op1;
+	char	r1 : 4;
+	char	r3 : 4;
+	short	d;
+	char	i;
+	char	op2;
+} RIE_Format_3;
+
+typedef struct {
+	char 	op1;
+	char	r1 : 4;
+	char	yy : 4;
+	short	i2;
+	char	m3 : 4;
+	char	xx : 4;
+	char	op2;
+} RIE_Format_4;
 
 typedef struct {
 	char 	op1;
@@ -425,21 +460,31 @@ typedef struct {
 
 typedef struct {
 	short	op;
-	char	b1 : 4;
+	short	tb1 : 4;
 	short	d1 : 12;
-	char	b2 : 4;
+	short	b2 : 4;
 	short	d2 : 12;
 } __attribute__ ((packed)) SSE_Format;
 
+typedef struct {
+	short	op;
+	char	r3 : 4;
+	char	o2 : 4;
+	short	b1 : 4;
+	short	d1 : 12;
+	short	b2 : 4;
+	short	d2 : 12;
+} __attribute__ ((packed)) SSF_Format;
+
 #define s390_emit16(c, x) do 			\
 {						\
-	*((guint16 *) c) = x; 			\
+	*((guint16 *) c) = (guint16) x;		\
 	c += sizeof(guint16);			\
 } while(0)
 
 #define s390_emit32(c, x) do 			\
 {						\
-	*((guint32 *) c) = x; 			\
+	*((guint32 *) c) = (guint32) x;		\
 	c += sizeof(guint32);			\
 } while(0)
 
@@ -507,12 +552,33 @@ typedef struct {
 
 #define S390_RI(c,opc,g1,m2)		s390_emit32(c, ((opc >> 4) << 24 | (g1) << 20 | (opc & 0x0f) << 16 | (m2 & 0xffff)))
 
-#define S390_RIE(c,opc,g1,g3,m2) do				\
+#define S390_RIE_1(c,opc,g1,g3,m2) do				\
 {								\
 	s390_emit16(c, ((opc & 0xff00) | (g1) << 4 | g3));	\
 	s390_emit32(c, ((m2) << 16 | (opc & 0xff)));		\
 } while (0)
 
+#define S390_RIE_2(c,opc,g1,g2,m3,v) do				\
+{								\
+	s390_emit16(c, ((opc & 0xff00) | (g1) << 4 | g3));	\
+	s390_emit16(c, (v));					\
+	s390_emit16(c, ((m2) << 12 | (opc & 0xff)));		\
+} while (0)
+
+#define S390_RIE_3(c,opc,g1,i,m3,d) do				\
+{								\
+	s390_emit16(c, ((opc & 0xff00) | (g1) << 4 | m3));	\
+	s390_emit16(c, (d));					\
+	s390_emit16(c, ((i) << 8 | (opc & 0xff)));		\
+} while (0)
+
+#define S390_RIE_4(c,opc,g1,i2,m3) do				\
+{								\
+	s390_emit16(c, ((opc & 0xff00) | (g1) << 4);		\
+	s390_emit16(c, (i2));					\
+	s390_emit16(c, ((m3) << 12 | (opc & 0xff)));		\
+} while (0)
+
 #define S390_RIL_1(c,opc,g1,m2) do					\
 {									\
 	s390_emit16(c, ((opc >> 4) << 8 | (g1) << 4 | (opc & 0xf)));	\
@@ -525,6 +591,20 @@ typedef struct {
 	s390_emit32(c, m2);						\
 } while (0)
 
+#define S390_RIS(c,opc,r,i,m3,b,d) do				\
+{								\
+	s390_emit16(c, ((opc, & 0xff00) | (r1) << 4) | (r2));	\
+	s390_emit16(c, ((b) << 12) | (d));			\
+	s390_emit16(c, ((i) << 4) | ((opc) & 0xff));		\
+}
+
+#define S390_RRS(c,opc,r1,r2,m3,b,d) do				\
+{								\
+	s390_emit16(c, ((opc, & 0xff00) | (r1) << 4) | (r2));	\
+	s390_emit16(c, ((b) << 12) | (d));			\
+	s390_emit16(c, ((m3) << 12) | ((opc) & 0xff));		\
+}
+
 #define S390_SI(c,opc,s1,p1,m2)		s390_emit32(c, (opc << 24 | (m2) << 16 | (s1) << 12 | ((p1) & 0xfff)));
 
 #define S390_SIY(c,opc,s1,p1,m2) do				\
@@ -571,31 +651,71 @@ typedef struct {
 	s390_emit16(c, ((s2) << 12 | ((p2) & 0xfff)));	\
 } while (0)
 
+#define S390_SSF(c,opc,r3,s1,p1,s2,p2) do				\
+{									\
+	s390_emit16(c, (((opc) & 0xff00) << 8) | ((r3) << 4) | 		\
+			((opc) & 0xf));					\
+	s390_emit16(c, ((s1) << 12 | ((p1) & 0xfff)));			\
+	s390_emit16(c, ((s2) << 12 | ((p2) & 0xfff)));			\
+} while (0)
+
 #define s390_a(c, r, x, b, d)		S390_RX(c, 0x5a, r, x, b, d)
 #define s390_adb(c, r, x, b, d)		S390_RXE(c, 0xed1a, r, x, b, d)
 #define s390_adbr(c, r1, r2)		S390_RRE(c, 0xb31a, r1, r2)
 #define s390_aebr(c, r1, r2)		S390_RRE(c, 0xb30a, r1, r2)
+#define s390_afi(c, r, v)		S390_RIL_1(c, 0xc29, r, v);
 #define s390_ag(c, r, x, b, d)		S390_RXY(c, 0xe308, r, x, b, d)
 #define s390_agf(c, r, x, b, d)		S390_RXY(c, 0xe318, r, x, b, d)
+#define s390_agfi(c, r, v)		S390_RIL_1(c, 0xc28, r, v)
+#define s390_afgr(c, r1, r2)		S390_RRE(c, 0xb918, r1, r2)
 #define s390_aghi(c, r, v)		S390_RI(c, 0xa7b, r, v)
+#define s390_aghik(c, r, v)		S390_RIE_1(c, 0xecd9, r, v)
 #define s390_agr(c, r1, r2)		S390_RRE(c, 0xb908, r1, r2)
+#define s390_agrk(c, r1, r2, r3)	S390_RRF_1(c, 0xb9e8, r1, r2, r3)
+#define s390_agsi(c, r, v)		S390_SIY(c, 0xeb7a, r v)
+#define s390_ahhhr(c, r1, r2, r3)	S390_RRF_1(c, 0xb9c8, r1, r2, r3)
+#define s390_ahhlr(c, r1, r2, r3)	S390_RRF_1(c, 0xb9d8, r1, r2, r3)
 #define s390_ahi(c, r, v)		S390_RI(c, 0xa7a, r, v)
+#define s390_ahik(c, r, v)		S390_RIE_1(c, 0xecd8, r, v)
+#define s390_ahy(c, r, x, b, d)		S390_RXY(c, 0xe37a, r, b, d)
+#define s390_aih(c, r, v)		S390_RIL_1(c, 0xcc8, r, v)
+#define s390_al(c, r, x, b, d)		S390_RX(c, 0x5e, r, x, b, d)
+#define s390_alc(c, r, x, b, d)		S390_RXY(c, 0xe398, r, x, b, d)
+#define s390_alcg(c, r, x, b, d)	S390_RXY(c, 0xe388, r, x, b, d)
 #define s390_alcgr(c, r1, r2)		S390_RRE(c, 0xb988, r1, r2)
 #define s390_alcr(c, r1, r2)		S390_RRE(c, 0xb998, r1, r2)
-#define s390_al(c, r, x, b, d)		S390_RX(c, 0x5e, r, x, b, d)
+#define s390_alfi(c, r, v)		S390_RIL_1(c, 0xc2b, r, v)
 #define s390_alg(c, r, x, b, d)		S390_RXY(c, 0xe30a, r, x, b, d)
 #define s390_algf(c, r, x, b, d)	S390_RXY(c, 0xe31a, r, x, b, d)
+#define s390_algfi(c, r, v)		S390_RIL_1(c, 0xc2a, r, v)
+#define s390_algfr(c, r1, r2)		S390_RRE(c, 0xb91a, r1, r2)
+#define s390_alghsik(c, r, v)		S390_RIE_1(c, 0xecd8, r, v)
 #define s390_algr(c, r1, r2)		S390_RRE(c, 0xb90a, r1, r2)
+#define s390_algsi(c, r, v)		S390_SIY(c, 0xeb7e, r, v)
+#define s390_alhhhr(c, r1, r2, r3)	S390_RRF_1(c, 0xb9ca, r1, r2, r3)
+#define s390_alhhlr(c, r1, r2, r3)	S390_RRF_1(c, 0xb9da, r1, r2, r3)
+#define s390_alhsik(c, r, v)		S390_RIE_1(c, 0xecda, r, v)
 #define s390_alr(c, r1, r2)		S390_RR(c, 0x1e, r1, r2)
+#define s390_alrk(c, r1, r2)		S390_RRF(c, 0xb9fa, r1, r2)
+#define s390_alsi(c, r, v)		S390_SIY(c, 0xeb6e, r, v)
+#define s390_alsih(c, r, v)		S390_RIL_1(c, 0xcca, r, v)
+#define s390_alsihn(c, r, v)		S390_RIL_1(c, 0xccb, r, v)
+#define s390_aly(c, r, x, b, d)		S390_RXY(c, 0xe35e, r, x, b, d)
 #define s390_ar(c, r1, r2)		S390_RR(c, 0x1a, r1, r2)
+#define s390_ark(c, r1, r2, r3)		S390_RRF_1(c, 0xb9f8, r1, r2, r3)
+#define s390_asi(c, r, v)		S390_SIY(c, 0xeb6a, r, v)
+#define s390_ay(c, r, x, b, d)		S390_RXY(c, 0xe35a, r, x, b, d)
 #define s390_basr(c, r1, r2)		S390_RR(c, 0x0d, r1, r2)
 #define s390_bctr(c, r1, r2)		S390_RR(c, 0x06, r1, r2)
 #define s390_bctrg(c, r1, r2)		S390_RRE(c, 0xb946, r1, r2)
+#define s390_bnzr(c, r)			S390_RR(c, 0x07, 0x07, r)
 #define s390_bras(c, r, o)		S390_RI(c, 0xa75, r, o)
 #define s390_brasl(c, r, o)		S390_RIL_1(c, 0xc05, r, o)
 #define s390_brc(c, m, d)		S390_RI(c, 0xa74, m, d)
+#define s390_brcl(c, m, d)		S390_RIL_2(c, 0xc04, m, d)
 #define s390_br(c, r)			S390_RR(c, 0x07, 0xf, r)
 #define s390_break(c)			S390_RR(c, 0, 0, 0)
+#define s390_bzr(c, r)			S390_RR(c, 0x07, 0x08, r)
 #define s390_c(c, r, x, b, d)		S390_RX(c, 0x59, r, x, b, d)
 #define s390_cdb(c, r, x, b, d)		S390_RXE(c, 0xed19, r, x, b, d)
 #define s390_cdbr(c, r1, r2)		S390_RRE(c, 0xb319, r1, r2)
@@ -605,19 +725,46 @@ typedef struct {
 #define s390_cdsg(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb3e, r1, r2, b, d)
 #define s390_cdsy(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb31, r1, r2, b, d)
 #define s390_cebr(c, r1, r2)		S390_RRE(c, 0xb309, r1, r2)
+#define s390_cegbr(c, r1, r2)		S390_RRE(c, 0xb3a4, r1, r2)
 #define s390_cfdbr(c, r1, m, r2)	S390_RRF_2(c, 0xb399, r1, m, r2)
+#define s390_cfi(c, r, v)		S390_RIL_1(c, 0xc2d, r, v)
 #define s390_cgdbr(c, r1, m, r2)	S390_RRF_2(c, 0xb3a9, r1, m, r2)
 #define s390_cg(c, r, x, b, d)		S390_RXY(c, 0xe320, r, x, b, d)
+#define s390_cgfi(c, r, v)		S390_RIL_1(c, 0xc2c, r, v)
+#define s390_cgfrl(c, r, v)		S390_RIL_1(c, 0xc6c, r, v)
 #define s390_cghi(c, r, i)		S390_RI(c, 0xa7f, r, i)
+#define s390_cgib(c, r, i, m, b, d)	S390_RIS(c, 0xecfc, r, i, m, b, d)
+#define s390_cgij(c, r, i, m, d)	S390_RIE_3(c, 0xec7c, r, i, m, d)
+#define s390_cgit(c, r, i, m)		S390_RIE_4(c, 0xec70, r, i m);
 #define s390_cgr(c, r1, r2)		S390_RRE(c, 0xb920, r1, r2)
+#define s390_cgrb(c, r1, r2, m3, b, d)	S390_RRS(c, 0xece4, r1, r2, m3, b, d)
+#define s390_cgrj(c, r1, r2, m3, v)	S390_RIE_2(c, 0xec64, r1, r2, m3, v)
+#define s390_cgrl(c, r, v)		S390_RIL_1(c, 0xc68, r, v)
 #define s390_chi(c, r, i)		S390_RI(c, 0xa7e, r, i)
+#define s390_cib(c, r, i, m, b, d)	S390_RIS(c, 0xecfe, r, i, m, b, d)
+#define s390_cij(c, r, i, m, d)		S390_RIE_3(c, 0xec7e, r, i, m, d)
+#define s390_cit(c, r, i, m)		S390_RIE_4(c, 0xec72, r, i m);
 #define s390_cl(c, r, x, b, d)		S390_RX(c, 0x55, r, x, b, d)
 #define s390_clg(c, r, x, b, d)		S390_RXY(c, 0xe321, r, x, b, d)
+#define s390_clgib(c, r, i, m, b, d)	S390_RIS(c, 0xecfd, r, i, m, b, d)
+#define s390_clgij(c, r, i, b)		S390_RIE_3(c, 0xec7d, r, i, m, d)
 #define s390_clgr(c, r1, r2)		S390_RRE(c, 0xb921, r1, r2)
+#define s390_clgrj(c, r1, r2, m, v)	S390_RIE_2(c, 0xec65, r1, r2, m, v)
+#define s390_clgrb(c, r1, r2, m3, b, d)	S390_RRS(c, 0xece5, r1, r2, m3, b, d)
+#define s390_clib(c, r, i, m, b, d)	S390_RIS(c, 0xecff, r, i, m, b, d)
+#define s390_clij(c, r, i, b)		S390_RIE_3(c, 0xec7f, r, i, m, d)
 #define s390_clr(c, r1, r2)		S390_RR(c, 0x15, r1, r2)
+#define s390_clrb(c, r1, r2, m3, b, d)	S390_RRS(c, 0xecf7, r1, r2, m3, b, d)
+#define s390_clrj(c, r1, r2, m, v)	S390_RIE_2(c, 0xec77, r1, r2, m, v)
 #define s390_cr(c, r1, r2)		S390_RR(c, 0x19, r1, r2)
+#define s390_crb(c, r1, r2, m3, b, d)	S390_RRS(c, 0xecf6, r1, r2, m3, b, d)
+#define s390_crj(c, r1, r2, m3, v)	S390_RIE_2(c, 0xec76, r1, r2, m3, v)
+#define s390_crl(c, r, v)		S390_RIL_1(c, 0xc6d, r, v)
+#define s390_crt(c, r1, r2, m3)		S390_RRF_2(c, 0xb972, r1, r2, m3);
+#define s390_cgrt(c, r1, r2, m3)	S390_RRF_2(c, 0xb960, r1, r2, m3);
 #define s390_cs(c, r1, r2, b, d)	S390_RX(c, 0xba, r1, r2, b, d)
 #define s390_csg(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb30, r1, r2, b, d)
+#define s390_csst(c, d1, b1, d2, b2, r)	S390_SSF(c, 0xc82, b1, d1, b2, d2, r)
 #define s390_csy(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb14, r1, r2, b, d)
 #define s390_ddbr(c, r1, r2)		S390_RRE(c, 0xb31d, r1, r2)
 #define s390_debr(c, r1, r2)		S390_RRE(c, 0xb30d, r1, r2)
@@ -632,9 +779,15 @@ typedef struct {
 #define s390_icm(c, r, m, b, d)		S390_RX(c, 0xbf, r, m, b, d)
 #define s390_icmy(c, r, x, b, d)	S390_RXY(c, 0xeb81, r, x, b, d)
 #define s390_icy(c, r, x, b, d)		S390_RXY(c, 0xe373, r, x, b, d)
+#define s390_iihf(c, r, v)		S390_RIL_1(c, 0xc08, r, v)
+#define s390_iihh(c, r, v)		S390_RI(c, 0xa50, r, v)
+#define s390_iihl(c, r, v)		S390_RI(c, 0xa51, r, v)
+#define s390_iilf(c, r, v)		S390_RIL_1(c, 0xc09, r, v)
+#define s390_iilh(c, r, v)		S390_RI(c, 0xa52, r, v)
+#define s390_iill(c, r, v)		S390_RI(c, 0xa53, r, v)
 #define s390_j(c,d)			s390_brc(c, S390_CC_UN, d)
 #define s390_jc(c, m, d)		s390_brc(c, m, d)
-#define s390_jcl(c, m, d)		S390_RIL_2(c, 0xc04, m, d)
+#define s390_jcl(c, m, d)		s390_brcl(c, m, d)
 #define s390_jcy(c, d)			s390_brc(c, S390_CC_CY, d)
 #define s390_je(c, d)			s390_brc(c, S390_CC_EQ, d)
 #define s390_jeo(c, d)			s390_brc(c, S390_CC_ZR|S390_CC_OV, d)
@@ -652,6 +805,24 @@ typedef struct {
 #define s390_jno(c, d)			s390_brc(c, S390_CC_NO, d)
 #define s390_jp(c, d)			s390_brc(c, S390_CC_GT, d)
 #define s390_jz(c, d)			s390_brc(c, S390_CC_ZR, d)
+#define s390_jg(c,d)			s390_brcl(c, S390_CC_UN, d)
+#define s390_jgcy(c, d)			s390_brcl(c, S390_CC_CY, d)
+#define s390_jge(c, d)			s390_brcl(c, S390_CC_EQ, d)
+#define s390_jgeo(c, d)			s390_brcl(c, S390_CC_ZR|S390_CC_OV, d)
+#define s390_jgh(c, d)			s390_brcl(c, S390_CC_GT, d)
+#define s390_jgho(c, d)			s390_brcl(c, S390_CC_GT|S390_CC_OV, d)
+#define s390_jgl(c, d)			s390_brcl(c, S390_CC_LT, d)
+#define s390_jglo(c, d)			s390_brcl(c, S390_CC_LT|S390_CC_OV, d)
+#define s390_jgm(c, d)			s390_brcl(c, S390_CC_LT, d)
+#define s390_jgnc(c, d)			s390_brcl(c, S390_CC_NC, d)
+#define s390_jgne(c, d)			s390_brcl(c, S390_CC_NZ, d)
+#define s390_jgnh(c, d)			s390_brcl(c, S390_CC_LE, d)
+#define s390_jgnl(c, d)			s390_brcl(c, S390_CC_GE, d)
+#define s390_jgnz(c, d)			s390_brcl(c, S390_CC_NZ, d)
+#define s390_jgo(c, d)			s390_brcl(c, S390_CC_OV, d)
+#define s390_jgno(c, d)			s390_brcl(c, S390_CC_NO, d)
+#define s390_jgp(c, d)			s390_brcl(c, S390_CC_GT, d)
+#define s390_jgz(c, d)			s390_brcl(c, S390_CC_ZR, d)
 #define s390_l(c, r, x, b, d)		S390_RX(c, 0x58, r, x, b, d)
 #define s390_ly(c, r, x, b, d)		S390_RXY(c, 0xe358, r, x, b, d)
 #define s390_la(c, r, x, b, d)		S390_RX(c, 0x41, r, x, b, d)
@@ -659,6 +830,7 @@ typedef struct {
 #define s390_lam(c, r1, r2, b, d)	S390_RS_1(c, 0x9a, r1, r2, b, d)
 #define s390_larl(c, r, o)		S390_RIL_1(c, 0xc00, r, o)
 #define s390_lb(c, r, x, b, d)		S390_RXY(c, 0xe376, r, x, b, d)
+#define s390_lbr(c, r1, r2)		S390_RRE(c, 0xb926, r1, r2)
 #define s390_lcdbr(c, r1, r2)		S390_RRE(c, 0xb313, r1, r2)
 #define s390_lcgr(c, r1, r2)		S390_RRE(c, 0xb903, r1, r2)
 #define s390_lcr(c, r1, r2)		S390_RR(c, 0x13, r1, r2)
@@ -666,26 +838,44 @@ typedef struct {
 #define s390_ldy(c, r, x, b, d)		S390_RXY(c, 0xed65, r, x, b, d)
 #define s390_ldeb(c, r, x, b, d)	S390_RXE(c, 0xed04, r, x, b, d)
 #define s390_ldebr(c, r1, r2)		S390_RRE(c, 0xb304, r1, r2)
+#define s390_ldgr(c, r1, r2)		S390_RRE(c, 0xb3c1, r1, r2)
 #define s390_ldr(c, r1, r2)		S390_RR(c, 0x28, r1, r2)
 #define s390_le(c, f, x, b, d)		S390_RX(c, 0x78, f, x, b, d)
 #define s390_ledbr(c, r1, r2)		S390_RRE(c, 0xb344, r1, r2)
 #define s390_ler(c, r1, r2)		S390_RR(c, 0x38, r1, r2)
 #define s390_ley(c, r, x, b, d)		S390_RXY(c, 0xed64, r, x, b, d)
-#define s390_lgb(c, r, x, b, d)		S390_RXY(c, 0xe377, r, x, b, d)
 #define s390_lg(c, r, x, b, d)		S390_RXY(c, 0xe304, r, x, b, d)
+#define s390_lgb(c, r, x, b, d)		S390_RXY(c, 0xe377, r, x, b, d)
+#define s390_lgbr(c, r1, r2)		S390_RRE(c, 0xb906, r1, r2)
+#define s390_lgdr(c, r1, r2)		S390_RRE(c, 0xb3cd, r1, r2)
 #define s390_lgf(c, r, x, b, d)		S390_RXY(c, 0xe314, r, x, b, d)
+#define s390_lgfi(c, r, v)		S390_RIL_1(c, 0xc01, r, v)
+#define s390_lgfrl(c, r1, d)		S390_RIL_1(c, 0xc4c, r1, d)
 #define s390_lgfr(c, r1, r2)		S390_RRE(c, 0xb914, r1, r2)
 #define s390_lgh(c, r, x, b, d)		S390_RXY(c, 0xe315, r, x, b, d)
 #define s390_lghi(c, r, v)		S390_RI(c, 0xa79, r, v)
+#define s390_lghr(c, r1, r2)		S390_RRE(c, 0xb907, r1, r2)
 #define s390_lgr(c, r1, r2)		S390_RRE(c, 0xb904, r1, r2)
+#define s390_lgrl(c, r1, d)		S390_RIL_1(c, 0xc48, r1, d)
 #define s390_lh(c, r, x, b, d)		S390_RX(c, 0x48, r, x, b, d)
+#define s390_lhr(c, r1, r2)		S390_RRE(c, 0xb927, r1, r2)
 #define s390_lhg(c, r, x, b, d)		S390_RXY(c, 0xe315, r, x, b, d)
 #define s390_lhi(c, r, v)		S390_RI(c, 0xa78, r, v)
 #define s390_lhy(c, r, x, b, d)		S390_RXY(c, 0xe378, r, x, b, d)
+#define s390_llcr(c, r1, r2)		S390_RRE(c, 0xb994, r1, r2)
 #define s390_llgc(c, r, x, b, d)	S390_RXY(c, 0xe390, r, x, b, d)
+#define s390_llgcr(c, r1, r2)		S390_RRE(c, 0xb984, r1, r2)
 #define s390_llgf(c, r, x, b, d)	S390_RXY(c, 0xe316, r, x, b, d)
 #define s390_llgfr(c, r1, r2)		S390_RRE(c, 0xb916, r1, r2)
 #define s390_llgh(c, r, x, b, d)	S390_RXY(c, 0xe391, r, x, b, d)
+#define s390_llghr(c, r1, r2)		S390_RRE(c, 0xb985, r1, r2)
+#define s390_llhr(c, r1, r2)		S390_RRE(c, 0xb995, r1, r2)
+#define s390_llihf(c, r, v)		S390_RIL_1(c, 0xc0e, r, v)
+#define s390_llihh(c, r, v)		S390_RI(c, 0xa5c, r, v)
+#define s390_llihl(c, r, v)		S390_RI(c, 0xa5d, r, v)
+#define s390_llilf(c, r, v)		S390_RIL_1(c, 0xc0f, r, v)
+#define s390_llilh(c, r, v)		S390_RI(c, 0xa5e, r, v)
+#define s390_llill(c, r, v)		S390_RI(c, 0xa5f, r, v)
 #define s390_lm(c, r1, r2, b, d)	S390_RS_1(c, 0x98, r1, r2, b, d)
 #define s390_lmg(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb04, r1, r2, b, d)
 #define s390_lndbr(c, r1, r2)		S390_RRE(c, 0xb311, r1, r2)
@@ -695,6 +885,7 @@ typedef struct {
 #define s390_lpgr(c, r1, r2)		S390_RRE(c, 0xb900, r1, r2)
 #define s390_lpr(c, r1, r2)		S390_RR(c, 0x10, r1, r2)
 #define s390_lr(c, r1, r2)		S390_RR(c, 0x18, r1, r2)
+#define s390_lrl(c, r1, d)		S390_RIL_1(c, 0xc4d, r1, d)
 #define s390_ltgfr(c, r1, r2)		S390_RRE(c, 0xb912, r1, r2)
 #define s390_ltgr(c, r1, r2)		S390_RRE(c, 0xb902, r1, r2)
 #define s390_ltr(c, r1, r2)		S390_RR(c, 0x12, r1, r2)
@@ -703,23 +894,45 @@ typedef struct {
 #define s390_m(c, r, x, b, d)		S390_RX(c, 0x5c, r, x, b, d)
 #define s390_mdbr(c, r1, r2)		S390_RRE(c, 0xb31c, r1, r2)
 #define s390_meebr(c, r1, r2)		S390_RRE(c, 0xb317, r1, r2)
+#define s390_mfy(c, r, x, b, d)		S390_RXY(c, 0xe35c, r, x, b, d)
 #define s390_mlgr(c, r1, r2)		S390_RRE(c, 0xb986, r1, r2)
 #define s390_mlr(c, r1, r2)		S390_RRE(c, 0xb996, r1, r2)
 #define s390_mr(c, r1, r2)		S390_RR(c, 0x1c, r1, r2)
 #define s390_ms(c, r, x, b, d)		S390_RX(c, 0x71, r, x, b, d)
+#define s390_msi(c, r, v)		S390_RIL_1(c, 0xc21, r, v)
 #define s390_msgfr(c, r1, r2)		S390_RRE(c, 0xb91c, r1, r2)
+#define s390_msgi(c, r, v)		S390_RIL_1(c, 0xc20, r, v)
 #define s390_msgr(c, r1, r2)		S390_RRE(c, 0xb90c, r1, r2)
 #define s390_msr(c, r1, r2)		S390_RRE(c, 0xb252, r1, r2)
 #define s390_mvc(c, l, b1, d1, b2, d2)	S390_SS_1(c, 0xd2, l, b1, d1, b2, d2)
 #define s390_mvcl(c, r1, r2)		S390_RR(c, 0x0e, r1, r2)
 #define s390_mvcle(c, r1, r3, d2, b2)	S390_RS_1(c, 0xa8, r1, r3, d2, b2)
 #define s390_n(c, r, x, b, d)		S390_RX(c, 0x54, r, x, b, d)
+#define s390_nc(c, l, b1, d1, b2, d2)	S390_SS_1(c, 0xd4, l, b1, d1, b2, d2)
 #define s390_ng(c, r, x, b, d)		S390_RXY(c, 0xe380, r, x, b, d)
 #define s390_ngr(c, r1, r2)		S390_RRE(c, 0xb980, r1, r2)
+#define s390_ngrk(c, r1, r2, r3)	S390_RRF_1(c, 0xb9e4, r1, r2, r3)
+#define s390_ni(c, b, d, v)		S390_SI(c, 0x94, b, d, v) 
+#define s390_nihf(c, r, v)		S390_RIL_1(c, 0xc0a, r, v)
+#define s390_nihh(c, r, v)		S390_RI(c, 0xa54, r, v)
+#define s390_nihl(c, r, v)		S390_RI(c, 0xa55, r, v)
+#define s390_nilf(c, r, v)		S390_RIL_1(c, 0xc0b, r, v)
 #define s390_nilh(c, r, v)		S390_RI(c, 0xa56, r, v)
 #define s390_nill(c, r, v)		S390_RI(c, 0xa57, r, v)
+#define s390_niy(c, b, d, v)		S390_SIY(c, 0xeb54, b, d, v) 
+#define s390_nop(c)  			S390_RR(c, 0x07, 0x0, 0)
+#define s390_mem(c)  			S390_RR(c, 0x07, 0xe, 0)
 #define s390_nr(c, r1, r2)		S390_RR(c, 0x14, r1, r2)
+#define s390_nrk(c, r1, r2)		S390_RRF_1(c, 0xb9f4, r1, r2)
+#define s390_ny(c, r, x, b, d)		S390_RRY(c, 0xe354, r1, r2)
 #define s390_o(c, r, x, b, d)		S390_RX(c, 0x56, r, x, b, d)
+#define s390_oihf(c, r, v)		S390_RIL_1(c, 0xc0c, r, v)
+#define s390_oihh(c, r, v)		S390_RI(c, 0xa58, r, v)
+#define s390_oihl(c, r, v)		S390_RI(c, 0xa59, r, v)
+#define s390_oilf(c, r, v)		S390_RIL_1(c, 0xc0d, r, v)
+#define s390_oilh(c, r, v)		S390_RI(c, 0xa5a, r, v)
+#define s390_oill(c, r, v)		S390_RI(c, 0xa5b` r, v)
+#define s390_oiy(c, b, d, v)		S390_SIY(c, 0xeb56 b, d, v) 
 #define s390_og(c, r, x, b, d)		S390_RXY(c, 0xe381, r, x, b, d)
 #define s390_ogr(c, r1, r2)		S390_RRE(c, 0xb981, r1, r2)
 #define s390_or(c, r1, r2)		S390_RR(c, 0x16, r1, r2)
@@ -730,16 +943,19 @@ typedef struct {
 #define s390_sg(c, r, x, b, d)		S390_RXY(c, 0xe309, r, x, b, d)
 #define s390_sgf(c, r, x, b, d)		S390_RXY(c, 0xe319, r, x, b, d)
 #define s390_sgr(c, r1, r2)		S390_RRE(c, 0xb909, r1, r2)
+#define s390_sl(c, r, x, b, d)		S390_RX(c, 0x5f, r, x, b, d)
 #define s390_sla(c, r, b, d)		S390_RS_3(c, 0x8b, r, b, d) 
 #define s390_slag(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb0b, r1, r2, b, d) 
 #define s390_slbg(c, r, x, b, d)	S390_RXY(c, 0xe389, r, x, b, d)
 #define s390_slbgr(c, r1, r2)		S390_RRE(c, 0xb989, r1, r2)
 #define s390_slbr(c, r1, r2)		S390_RRE(c, 0xb999, r1, r2)
-#define s390_sl(c, r, x, b, d)		S390_RX(c, 0x5f, r, x, b, d)
 #define s390_slda(c, r, b, d)		S390_RS_3(c, 0x8f, r, b, d) 
 #define s390_sldl(c, r, b, d)		S390_RS_3(c, 0x8d, r, b, d) 
+#define s390_slfi(c, r, v)		S390_RIL_1(c, 0xc25, r, v)
 #define s390_slg(c, r, x, b, d)		S390_RXY(c, 0xe30b, r, x, b, d)
 #define s390_slgf(c, r, x, b, d)	S390_RXY(c, 0xe31b, r, x, b, d)
+#define s390_slgfr(c, r1, r2)		S390_RRE(c, 0xb91b, r1, r2)
+#define s390_slgfi(c, r, v)		S390_RIL_1(c, 0xc24, r, v)
 #define s390_slgr(c, r1, r2)		S390_RRE(c, 0xb90b, r1, r2)
 #define s390_sll(c, r, b, d)		S390_RS_3(c, 0x89, r, b, d) 
 #define s390_sllg(c, r1, r2, b, d)	S390_RSY_1(c, 0xeb0d, r1, r2, b, d) 
@@ -765,7 +981,6 @@ typedef struct {
 #define s390_stey(c, r, x, b, d)	S390_RXY(c, 0xed66, r, x, b, d)
 #define s390_stfpc(c, b, d)		S390_S(c, 0xb29c, b, d)
 #define s390_stg(c, r, x, b, d)		S390_RXY(c, 0xe324, r, x, b, d)
-#define s390_stg(c, r, x, b, d)		S390_RXY(c, 0xe324, r, x, b, d)
 #define s390_sth(c, r, x, b, d)		S390_RX(c, 0x40, r, x, b, d)
 #define s390_sthy(c, r, x, b, d)	S390_RXY(c, 0xe370, r, x, b, d)
 #define s390_stm(c, r1, r2, b, d)	S390_RS_1(c, 0x90, r1, r2, b, d)
@@ -774,7 +989,10 @@ typedef struct {
 #define s390_tcdb(c, r, x, b, d)	S390_RXE(c, 0xed11, r, x, b, d)
 #define s390_tceb(c, r, x, b, d)	S390_RXE(c, 0xed10, r, x, b, d)
 #define s390_x(c, r, x, b, d)		S390_RX(c, 0x57, r, x, b, d)
+#define s390_xihf(c, r, v)		S390_RIL_1(c, 0xc06, r, v)
+#define s390_xilf(c, r, v)		S390_RIL_1(c, 0xc07, r, v)
 #define s390_xg(c, r, x, b, d)		S390_RXY(c, 0xe382, r, x, b, d)
 #define s390_xgr(c, r1, r2)		S390_RRE(c, 0xb982, r1, r2)
 #define s390_xr(c, r1, r2)		S390_RR(c, 0x17, r1, r2)
+#define s390_xy(c, r, x, b, d)		S390_RXY(c, 0xe357, r, x, b, d)
 #endif
