@@ -30,6 +30,7 @@
 #include <mono/metadata/loader.h>
 #include <mono/metadata/assembly.h>
 #include <mono/metadata/appdomain.h>
+#include <mono/utils/bsearch.h>
 
 static void     setup_filter          (MonoImage *image);
 static gboolean should_include_type   (int idx);
@@ -835,6 +836,7 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 	}
 
 	for (i = start; i < end; i++){
+		MonoError error;
 		MonoMethodSignature *ms;
 		MonoGenericContainer *container;
 		char *flags, *impl_flags;
@@ -853,17 +855,23 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 		mono_metadata_decode_blob_size (sig, &sig);
 
 		container = mono_metadata_load_generic_params (m, MONO_TOKEN_METHOD_DEF | (i + 1), type_container);
-		if (container)
-			mono_metadata_load_generic_param_constraints (m, MONO_TOKEN_METHOD_DEF | (i + 1), container);
-		else 
+		if (container) {
+			MonoError error;
+			mono_metadata_load_generic_param_constraints_checked (m, MONO_TOKEN_METHOD_DEF | (i + 1), container, &error);
+			g_assert (mono_error_ok (&error)); /*FIXME don't swallow the error message*/
+		} else {
 			container = type_container;
+		}
 
-		ms = mono_metadata_parse_method_signature_full (m, container, i + 1, sig, &sig);
+		ms = mono_metadata_parse_method_signature_full (m, container, i + 1, sig, &sig, &error);
 		if (ms != NULL){
 			sig_str = dis_stringify_method_signature (m, ms, i + 1, container, FALSE);
 			method_name = mono_metadata_string_heap (m, cols [MONO_METHOD_NAME]);
-		} else
+		} else {
+			sig_str = NULL;
 			method_name = g_strdup ("<NULL METHOD SIGNATURE>");
+			mono_error_cleanup (&error);
+		}
 
 		fprintf (output, "    // method line %d\n", i + 1);
 		fprintf (output, "    .method %s", flags);
@@ -1107,7 +1115,7 @@ dis_interfaces (MonoImage *m, guint32 typedef_row, MonoGenericContainer *contain
 	loc.col_idx = MONO_INTERFACEIMPL_CLASS;
 	loc.idx = typedef_row;
 
-	if (!bsearch (&loc, table->base, table->rows, table->row_size, table_locator))
+	if (!mono_binary_search (&loc, table->base, table->rows, table->row_size, table_locator))
 		return;
 
 	start = loc.result;
@@ -1178,8 +1186,11 @@ dis_type (MonoImage *m, int n, int is_nested, int forward)
 	}
 
 	container = mono_metadata_load_generic_params (m, MONO_TOKEN_TYPE_DEF | (n + 1), NULL);
-	if (container)
-		mono_metadata_load_generic_param_constraints (m, MONO_TOKEN_TYPE_DEF | (n + 1), container);
+	if (container) {
+		MonoError error;
+		mono_metadata_load_generic_param_constraints_checked (m, MONO_TOKEN_TYPE_DEF | (n + 1), container, &error);
+		g_assert (mono_error_ok (&error)); /*FIXME don't swallow the error message*/
+	}
 
 	esname = get_escaped_name (name);
 	if ((cols [MONO_TYPEDEF_FLAGS] & TYPE_ATTRIBUTE_CLASS_SEMANTIC_MASK) == TYPE_ATTRIBUTE_CLASS){
@@ -1678,7 +1689,7 @@ table_includes (TableFilter *tf, int idx)
 {
 	if (!tf->count)
 		return FALSE;
-	return bsearch (&idx, tf->elems, tf->count, sizeof (int), int_cmp) != NULL;
+	return mono_binary_search (&idx, tf->elems, tf->count, sizeof (int), int_cmp) != NULL;
 }
 
 static gboolean 
