@@ -4,7 +4,8 @@
  * Author:
  *	Dick Porter (dick@ximian.com)
  *
- * (C) 2002-2006 Novell, Inc.
+ * (C) 2002-2011 Novell, Inc.
+ * Copyright 2011 Xamarin Inc
  */
 
 #include <config.h>
@@ -12,6 +13,9 @@
 #include <pthread.h>
 #include <errno.h>
 #include <unistd.h>
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
+#endif
 #include <string.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
@@ -27,20 +31,28 @@
 #  include <dirent.h>
 #endif
 #include <sys/stat.h>
+#ifdef HAVE_SYS_RESOURCE_H
+#  include <sys/resource.h>
+#endif
 
 #include <mono/io-layer/wapi.h>
 #include <mono/io-layer/wapi-private.h>
 #include <mono/io-layer/handles-private.h>
-#include <mono/io-layer/mono-mutex.h>
 #include <mono/io-layer/misc-private.h>
 #include <mono/io-layer/shared.h>
 #include <mono/io-layer/collection.h>
 #include <mono/io-layer/process-private.h>
-#include <mono/io-layer/critical-section-private.h>
 
-#undef DEBUG
+#include <mono/utils/mono-mutex.h>
+#include <mono/utils/mono-proclib.h>
 #undef DEBUG_REFS
 
+#if 0
+#define DEBUG(...) g_message(__VA_ARGS__)
+#else
+#define DEBUG(...)
+#endif
+
 static void (*_wapi_handle_ops_get_close_func (WapiHandleType type))(gpointer, gpointer);
 
 static WapiHandleCapability handle_caps[WAPI_HANDLE_COUNT]={0};
@@ -124,10 +136,10 @@ struct _WapiFileShareLayout *_wapi_fileshare_layout = NULL;
  * 4MB array.
  */
 static GHashTable *file_share_hash;
-static CRITICAL_SECTION file_share_hash_mutex;
+static mono_mutex_t file_share_hash_mutex;
 
-#define file_share_hash_lock() EnterCriticalSection (&file_share_hash_mutex)
-#define file_share_hash_unlock() LeaveCriticalSection (&file_share_hash_mutex)
+#define file_share_hash_lock() mono_mutex_lock (&file_share_hash_mutex)
+#define file_share_hash_unlock() mono_mutex_unlock (&file_share_hash_mutex)
 
 guint32 _wapi_fd_reserve;
 
@@ -152,7 +164,6 @@ gboolean _wapi_has_shut_down = FALSE;
 static pid_t _wapi_pid;
 static mono_once_t pid_init_once = MONO_ONCE_INIT;
 
-static gpointer _wapi_handle_real_new (WapiHandleType type, gpointer handle_specific);
 static void _wapi_handle_unref_full (gpointer handle, gboolean ignore_private_busy_handles);
 
 static void pid_init (void)
@@ -168,14 +179,12 @@ pid_t _wapi_getpid (void)
 }
 
 
-static mono_mutex_t scan_mutex = MONO_MUTEX_INITIALIZER;
+static mono_mutex_t scan_mutex;
 
 static void handle_cleanup (void)
 {
 	int i, j, k;
 	
-	_wapi_process_signal_self ();
-
 	/* Every shared handle we were using ought really to be closed
 	 * by now, but to make sure just blow them all away.  The
 	 * exiting finalizer thread in particular races us to the
@@ -186,33 +195,10 @@ static void handle_cleanup (void)
 	for(i = SLOT_INDEX (0); _wapi_private_handles[i] != NULL; i++) {
 		for(j = SLOT_OFFSET (0); j < _WAPI_HANDLE_INITIAL_COUNT; j++) {
 			struct _WapiHandleUnshared *handle_data = &_wapi_private_handles[i][j];
-			int type = handle_data->type;
 			gpointer handle = GINT_TO_POINTER (i*_WAPI_HANDLE_INITIAL_COUNT+j);
-			
-			if (_WAPI_SHARED_HANDLE (type)) {
-				if (type == WAPI_HANDLE_THREAD) {
-					/* Special-case thread handles
-					 * because they need extra
-					 * cleanup.  This also avoids
-					 * a race condition between
-					 * the application exit and
-					 * the finalizer thread - if
-					 * it finishes up between now
-					 * and actual app termination
-					 * it will find all its handle
-					 * details have been blown
-					 * away, so this sets those
-					 * anyway.
-					 */
-					g_assert (0); /*This condition is freaking impossible*/
-					_wapi_thread_set_termination_details (handle, 0);
-				}
-			}
-				
+
 			for(k = handle_data->ref; k > 0; k--) {
-#ifdef DEBUG
-				g_message ("%s: unreffing %s handle %p", __func__, _wapi_handle_typename[type], handle);
-#endif
+				DEBUG ("%s: unreffing %s handle %p", __func__, _wapi_handle_typename[type], handle);
 					
 				_wapi_handle_unref_full (handle, TRUE);
 			}
@@ -226,31 +212,31 @@ static void handle_cleanup (void)
 
 	if (file_share_hash) {
 		g_hash_table_destroy (file_share_hash);
-		DeleteCriticalSection (&file_share_hash_mutex);
+		mono_mutex_destroy (&file_share_hash_mutex);
 	}
 
 	for (i = 0; i < _WAPI_PRIVATE_MAX_SLOTS; ++i)
 		g_free (_wapi_private_handles [i]);
 }
 
-void _wapi_cleanup ()
+int
+wapi_getdtablesize (void)
 {
-	g_assert (_wapi_has_shut_down == FALSE);
-	
-	_wapi_has_shut_down = TRUE;
-
-	_wapi_critical_section_cleanup ();
-	_wapi_error_cleanup ();
-	_wapi_thread_cleanup ();
+	return eg_getdtablesize ();
 }
 
-static mono_once_t shared_init_once = MONO_ONCE_INIT;
-static void shared_init (void)
+/*
+ * wapi_init:
+ *
+ *   Initialize the io-layer.
+ */
+void
+wapi_init (void)
 {
 	g_assert ((sizeof (handle_ops) / sizeof (handle_ops[0]))
 		  == WAPI_HANDLE_COUNT);
-	
-	_wapi_fd_reserve = getdtablesize();
+
+	_wapi_fd_reserve = wapi_getdtablesize ();
 
 	/* This is needed by the code in _wapi_handle_new_internal */
 	_wapi_fd_reserve = (_wapi_fd_reserve + (_WAPI_HANDLE_INITIAL_COUNT - 1)) & ~(_WAPI_HANDLE_INITIAL_COUNT - 1);
@@ -284,19 +270,28 @@ static void shared_init (void)
 	if (_wapi_shm_enabled ())
 		_wapi_collection_init ();
 #endif
+	_wapi_io_init ();
+	mono_mutex_init (&scan_mutex);
 
-	/* Can't call wapi_handle_new as it calls us recursively */
-	_wapi_global_signal_handle = _wapi_handle_real_new (WAPI_HANDLE_EVENT, NULL);
+	_wapi_global_signal_handle = _wapi_handle_new (WAPI_HANDLE_EVENT, NULL);
 
 	_wapi_global_signal_cond = &_WAPI_PRIVATE_HANDLES (GPOINTER_TO_UINT (_wapi_global_signal_handle)).signal_cond;
 	_wapi_global_signal_mutex = &_WAPI_PRIVATE_HANDLES (GPOINTER_TO_UINT (_wapi_global_signal_handle)).signal_mutex;
 
-	/* Using g_atexit here instead of an explicit function call in
-	 * a cleanup routine lets us cope when a third-party library
-	 * calls exit (eg if an X client loses the connection to its
-	 * server.)
-	 */
-	g_atexit (handle_cleanup);
+	wapi_processes_init ();
+}
+
+void
+wapi_cleanup (void)
+{
+	g_assert (_wapi_has_shut_down == FALSE);
+	
+	_wapi_has_shut_down = TRUE;
+
+	_wapi_error_cleanup ();
+	_wapi_thread_cleanup ();
+	wapi_processes_cleanup ();
+	handle_cleanup ();
 }
 
 static void _wapi_handle_init_shared (struct _WapiHandleShared *handle,
@@ -315,10 +310,57 @@ static void _wapi_handle_init_shared (struct _WapiHandleShared *handle,
 	}
 }
 
+static size_t _wapi_handle_struct_size (WapiHandleType type)
+{
+	size_t type_size;
+
+	switch (type) {
+		case WAPI_HANDLE_FILE: case WAPI_HANDLE_CONSOLE: case WAPI_HANDLE_PIPE:
+			type_size = sizeof (struct _WapiHandle_file);
+			break;
+		case WAPI_HANDLE_THREAD:
+			type_size = sizeof (struct _WapiHandle_thread);
+			break;
+		case WAPI_HANDLE_SEM:
+			type_size = sizeof (struct _WapiHandle_sem);
+			break;
+		case WAPI_HANDLE_MUTEX:
+			type_size = sizeof (struct _WapiHandle_mutex);
+			break;
+		case WAPI_HANDLE_EVENT:
+			type_size = sizeof (struct _WapiHandle_event);
+			break;
+		case WAPI_HANDLE_SOCKET:
+			type_size = sizeof (struct _WapiHandle_socket);
+			break;
+		case WAPI_HANDLE_FIND:
+			type_size = sizeof (struct _WapiHandle_find);
+			break;
+		case WAPI_HANDLE_PROCESS:
+			type_size = sizeof (struct _WapiHandle_process);
+			break;
+		case WAPI_HANDLE_NAMEDMUTEX:
+			type_size = sizeof (struct _WapiHandle_namedmutex);
+			break;
+		case WAPI_HANDLE_NAMEDSEM:
+			type_size = sizeof (struct _WapiHandle_namedsem);
+			break;
+		case WAPI_HANDLE_NAMEDEVENT:
+			type_size = sizeof (struct _WapiHandle_namedevent);
+			break;
+
+		default:
+			g_error ("Unknown WapiHandleType: %d\n", type);
+	}
+
+	return type_size;
+}
+
 static void _wapi_handle_init (struct _WapiHandleUnshared *handle,
 			       WapiHandleType type, gpointer handle_specific)
 {
 	int thr_ret;
+	int type_size;
 	
 	g_assert (_wapi_has_shut_down == FALSE);
 	
@@ -330,12 +372,13 @@ static void _wapi_handle_init (struct _WapiHandleUnshared *handle,
 		thr_ret = pthread_cond_init (&handle->signal_cond, NULL);
 		g_assert (thr_ret == 0);
 				
-		thr_ret = mono_mutex_init (&handle->signal_mutex, NULL);
+		thr_ret = mono_mutex_init (&handle->signal_mutex);
 		g_assert (thr_ret == 0);
 
 		if (handle_specific != NULL) {
+			type_size = _wapi_handle_struct_size (type);
 			memcpy (&handle->u, handle_specific,
-				sizeof (handle->u));
+				type_size);
 		}
 	}
 }
@@ -447,21 +490,20 @@ again:
 	return(0);
 }
 
-static gpointer _wapi_handle_real_new (WapiHandleType type, gpointer handle_specific)
+gpointer 
+_wapi_handle_new (WapiHandleType type, gpointer handle_specific)
 {
 	guint32 handle_idx = 0;
 	gpointer handle;
 	int thr_ret;
-	
-#ifdef DEBUG
-	g_message ("%s: Creating new handle of type %s", __func__,
+
+	g_assert (_wapi_has_shut_down == FALSE);
+		
+	DEBUG ("%s: Creating new handle of type %s", __func__,
 		   _wapi_handle_typename[type]);
-#endif
 
 	g_assert(!_WAPI_FD_HANDLE(type));
 	
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 		
@@ -481,7 +523,6 @@ static gpointer _wapi_handle_real_new (WapiHandleType type, gpointer handle_spec
 	
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 
 	if (handle_idx == 0) {
 		/* We ran out of slots */
@@ -494,9 +535,7 @@ static gpointer _wapi_handle_real_new (WapiHandleType type, gpointer handle_spec
 	
 	handle = GUINT_TO_POINTER (handle_idx);
 
-#ifdef DEBUG
-	g_message ("%s: Allocated new handle %p", __func__, handle);
-#endif
+	DEBUG ("%s: Allocated new handle %p", __func__, handle);
 	
 	if (_WAPI_SHARED_HANDLE(type)) {
 		/* Add the shared section too */
@@ -505,7 +544,6 @@ static gpointer _wapi_handle_real_new (WapiHandleType type, gpointer handle_spec
 		ref = _wapi_handle_new_shared (type, handle_specific);
 		if (ref == 0) {
 			_wapi_handle_collect ();
-			_wapi_process_reap ();
 			ref = _wapi_handle_new_shared (type, handle_specific);
 			if (ref == 0) {
 				/* FIXME: grow the arrays */
@@ -515,25 +553,14 @@ static gpointer _wapi_handle_real_new (WapiHandleType type, gpointer handle_spec
 		}
 		
 		_WAPI_PRIVATE_HANDLES(handle_idx).u.shared.offset = ref;
-#ifdef DEBUG
-		g_message ("%s: New shared handle at offset 0x%x", __func__,
+		DEBUG ("%s: New shared handle at offset 0x%x", __func__,
 			   ref);
-#endif
 	}
 		
 done:
 	return(handle);
 }
 
-gpointer _wapi_handle_new (WapiHandleType type, gpointer handle_specific)
-{
-	g_assert (_wapi_has_shut_down == FALSE);
-	
-	mono_once (&shared_init_once, shared_init);
-
-	return _wapi_handle_real_new (type, handle_specific);
-}
-
 gpointer _wapi_handle_new_from_offset (WapiHandleType type, guint32 offset,
 				       gboolean timestamp)
 {
@@ -541,16 +568,11 @@ gpointer _wapi_handle_new_from_offset (WapiHandleType type, guint32 offset,
 	gpointer handle = INVALID_HANDLE_VALUE;
 	int thr_ret, i, k;
 	struct _WapiHandleShared *shared;
-	guint32 now = (guint32)(time (NULL) & 0xFFFFFFFF);
 	
 	g_assert (_wapi_has_shut_down == FALSE);
 	
-	mono_once (&shared_init_once, shared_init);
-
-#ifdef DEBUG
-	g_message ("%s: Creating new handle of type %s to offset %d", __func__,
+	DEBUG ("%s: Creating new handle of type %s to offset %d", __func__,
 		   _wapi_handle_typename[type], offset);
-#endif
 
 	g_assert(!_WAPI_FD_HANDLE(type));
 	g_assert(_WAPI_SHARED_HANDLE(type));
@@ -558,12 +580,11 @@ gpointer _wapi_handle_new_from_offset (WapiHandleType type, guint32 offset,
 
 	shared = &_wapi_shared_layout->handles[offset];
 	if (timestamp) {
+		guint32 now = (guint32)(time (NULL) & 0xFFFFFFFF);
 		/* Bump up the timestamp for this offset */
 		InterlockedExchange ((gint32 *)&shared->timestamp, now);
 	}
 		
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 
@@ -584,15 +605,12 @@ gpointer _wapi_handle_new_from_offset (WapiHandleType type, guint32 offset,
 first_pass_done:
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 
 	if (handle != INVALID_HANDLE_VALUE) {
 		_wapi_handle_ref (handle);
 
-#ifdef DEBUG
-		g_message ("%s: Returning old handle %p referencing 0x%x",
+		DEBUG ("%s: Returning old handle %p referencing 0x%x",
 			   __func__, handle, offset);
-#endif
 		return (handle);
 	}
 
@@ -602,24 +620,18 @@ first_pass_done:
 	
 	if (shared->type == WAPI_HANDLE_UNUSED) {
 		/* Someone deleted this handle while we were working */
-#ifdef DEBUG
-		g_message ("%s: Handle at 0x%x unused", __func__, offset);
-#endif
+		DEBUG ("%s: Handle at 0x%x unused", __func__, offset);
 		goto done;
 	}
 
 	if (shared->type != type) {
-#ifdef DEBUG
-		g_message ("%s: Wrong type at %d 0x%x! Found %s wanted %s",
+		DEBUG ("%s: Wrong type at %d 0x%x! Found %s wanted %s",
 			   __func__, offset, offset,
 			   _wapi_handle_typename[shared->type],
 			   _wapi_handle_typename[type]);
-#endif
 		goto done;
 	}
 	
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 	
@@ -635,7 +647,6 @@ first_pass_done:
 		
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 		
 	/* Make sure we left the space for fd mappings */
 	g_assert (handle_idx >= _wapi_fd_reserve);
@@ -645,9 +656,7 @@ first_pass_done:
 	_WAPI_PRIVATE_HANDLES(handle_idx).u.shared.offset = offset;
 	InterlockedIncrement ((gint32 *)&shared->handle_refs);
 	
-#ifdef DEBUG
-	g_message ("%s: Allocated new handle %p referencing 0x%x (shared refs %d)", __func__, handle, offset, shared->handle_refs);
-#endif
+	DEBUG ("%s: Allocated new handle %p referencing 0x%x (shared refs %d)", __func__, handle, offset, shared->handle_refs);
 	
 done:
 	_wapi_handle_unlock_shared_handles ();
@@ -660,8 +669,6 @@ init_handles_slot (int idx)
 {
 	int thr_ret;
 
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-						  (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 
@@ -673,7 +680,6 @@ init_handles_slot (int idx)
 
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 }
 
 gpointer _wapi_handle_new_fd (WapiHandleType type, int fd,
@@ -684,20 +690,14 @@ gpointer _wapi_handle_new_fd (WapiHandleType type, int fd,
 	
 	g_assert (_wapi_has_shut_down == FALSE);
 	
-	mono_once (&shared_init_once, shared_init);
-	
-#ifdef DEBUG
-	g_message ("%s: Creating new handle of type %s", __func__,
+	DEBUG ("%s: Creating new handle of type %s", __func__,
 		   _wapi_handle_typename[type]);
-#endif
 	
 	g_assert(_WAPI_FD_HANDLE(type));
 	g_assert(!_WAPI_SHARED_HANDLE(type));
 	
 	if (fd >= _wapi_fd_reserve) {
-#ifdef DEBUG
-		g_message ("%s: fd %d is too big", __func__, fd);
-#endif
+		DEBUG ("%s: fd %d is too big", __func__, fd);
 
 		return(GUINT_TO_POINTER (_WAPI_HANDLE_INVALID));
 	}
@@ -709,17 +709,13 @@ gpointer _wapi_handle_new_fd (WapiHandleType type, int fd,
 	handle = &_WAPI_PRIVATE_HANDLES(fd);
 	
 	if (handle->type != WAPI_HANDLE_UNUSED) {
-#ifdef DEBUG
-		g_message ("%s: fd %d is already in use!", __func__, fd);
-#endif
+		DEBUG ("%s: fd %d is already in use!", __func__, fd);
 		/* FIXME: clean up this handle?  We can't do anything
 		 * with the fd, cos thats the new one
 		 */
 	}
 
-#ifdef DEBUG
-	g_message ("%s: Assigning new fd handle %d", __func__, fd);
-#endif
+	DEBUG ("%s: Assigning new fd handle %d", __func__, fd);
 
 	/* Prevent file share entries racing with us, when the file
 	 * handle is only half initialised
@@ -789,8 +785,6 @@ _wapi_handle_foreach (WapiHandleType type,
 	guint32 i, k;
 	int thr_ret;
 
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 
@@ -810,7 +804,6 @@ _wapi_handle_foreach (WapiHandleType type,
 
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 }
 
 /* This might list some shared handles twice if they are already
@@ -819,6 +812,7 @@ _wapi_handle_foreach (WapiHandleType type,
  * unreffed if the check function returns FALSE, so callers must not
  * rely on the handle persisting (unless the check function returns
  * TRUE)
+ * The caller owns the returned handle.
  */
 gpointer _wapi_search_handle (WapiHandleType type,
 			      gboolean (*check)(gpointer test, gpointer user),
@@ -833,8 +827,6 @@ gpointer _wapi_search_handle (WapiHandleType type,
 	gboolean found = FALSE;
 	int thr_ret;
 
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 	
@@ -862,13 +854,10 @@ gpointer _wapi_search_handle (WapiHandleType type,
 
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 
 	if (!found && search_shared && _WAPI_SHARED_HANDLE (type)) {
 		/* Not found yet, so search the shared memory too */
-#ifdef DEBUG
-		g_message ("%s: Looking at other shared handles...", __func__);
-#endif
+		DEBUG ("%s: Looking at other shared handles...", __func__);
 
 		for (i = 0; i < _WAPI_HANDLE_INITIAL_COUNT; i++) {
 			shared = &_wapi_shared_layout->handles[i];
@@ -889,9 +878,7 @@ gpointer _wapi_search_handle (WapiHandleType type,
 					continue;
 				}
 				
-#ifdef DEBUG
-				g_message ("%s: Opened tmp handle %p (type %s) from offset %d", __func__, ret, _wapi_handle_typename[type], i);
-#endif
+				DEBUG ("%s: Opened tmp handle %p (type %s) from offset %d", __func__, ret, _wapi_handle_typename[type], i);
 
 				/* It's possible that the shared part
 				 * of this handle has now been blown
@@ -968,10 +955,8 @@ gint32 _wapi_search_handle_namespace (WapiHandleType type,
 	
 	g_assert(_WAPI_SHARED_HANDLE(type));
 	
-#ifdef DEBUG
-	g_message ("%s: Lookup for handle named [%s] type %s", __func__,
+	DEBUG ("%s: Lookup for handle named [%s] type %s", __func__,
 		   utf8_name, _wapi_handle_typename[type]);
-#endif
 
 	/* Do a handle collection before starting to look, so that any
 	 * stale cruft gets removed
@@ -994,28 +979,20 @@ gint32 _wapi_search_handle_namespace (WapiHandleType type,
 			continue;
 		}
 
-#ifdef DEBUG
-		g_message ("%s: found a shared namespace handle at 0x%x (type %s)", __func__, i, _wapi_handle_typename[shared_handle_data->type]);
-#endif
+		DEBUG ("%s: found a shared namespace handle at 0x%x (type %s)", __func__, i, _wapi_handle_typename[shared_handle_data->type]);
 
 		sharedns=(WapiSharedNamespace *)&shared_handle_data->u;
 			
-#ifdef DEBUG
-		g_message ("%s: name is [%s]", __func__, sharedns->name);
-#endif
+		DEBUG ("%s: name is [%s]", __func__, sharedns->name);
 
 		if (strcmp (sharedns->name, utf8_name) == 0) {
 			if (shared_handle_data->type != type) {
 				/* Its the wrong type, so fail now */
-#ifdef DEBUG
-				g_message ("%s: handle 0x%x matches name but is wrong type: %s", __func__, i, _wapi_handle_typename[shared_handle_data->type]);
-#endif
+				DEBUG ("%s: handle 0x%x matches name but is wrong type: %s", __func__, i, _wapi_handle_typename[shared_handle_data->type]);
 				ret = -1;
 				goto done;
 			} else {
-#ifdef DEBUG
-				g_message ("%s: handle 0x%x matches name and type", __func__, i);
-#endif
+				DEBUG ("%s: handle 0x%x matches name and type", __func__, i);
 				ret = i;
 				goto done;
 			}
@@ -1031,7 +1008,6 @@ done:
 void _wapi_handle_ref (gpointer handle)
 {
 	guint32 idx = GPOINTER_TO_UINT(handle);
-	guint32 now = (guint32)(time (NULL) & 0xFFFFFFFF);
 	struct _WapiHandleUnshared *handle_data;
 
 	if (!_WAPI_PRIVATE_VALID_SLOT (idx)) {
@@ -1055,7 +1031,7 @@ void _wapi_handle_ref (gpointer handle)
 	 */
 	if (_WAPI_SHARED_HANDLE(handle_data->type)) {
 		struct _WapiHandleShared *shared_data = &_wapi_shared_layout->handles[handle_data->u.shared.offset];
-		
+		guint32 now = (guint32)(time (NULL) & 0xFFFFFFFF);
 		InterlockedExchange ((gint32 *)&shared_data->timestamp, now);
 	}
 	
@@ -1120,12 +1096,9 @@ static void _wapi_handle_unref_full (gpointer handle, gboolean ignore_private_bu
 			g_assert (thr_ret == 0);
 		}
 		
-		pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup, (void *)&scan_mutex);
 		thr_ret = mono_mutex_lock (&scan_mutex);
 
-#ifdef DEBUG
-		g_message ("%s: Destroying handle %p", __func__, handle);
-#endif
+		DEBUG ("%s: Destroying handle %p", __func__, handle);
 		
 		memcpy (&handle_data, &_WAPI_PRIVATE_HANDLES(idx),
 			sizeof (struct _WapiHandleUnshared));
@@ -1178,7 +1151,6 @@ static void _wapi_handle_unref_full (gpointer handle, gboolean ignore_private_bu
 
 		thr_ret = mono_mutex_unlock (&scan_mutex);
 		g_assert (thr_ret == 0);
-		pthread_cleanup_pop (0);
 
 		if (early_exit)
 			return;
@@ -1219,10 +1191,8 @@ gboolean _wapi_handle_test_capabilities (gpointer handle,
 	
 	type = _WAPI_PRIVATE_HANDLES(idx).type;
 
-#ifdef DEBUG
-	g_message ("%s: testing 0x%x against 0x%x (%d)", __func__,
+	DEBUG ("%s: testing 0x%x against 0x%x (%d)", __func__,
 		   handle_caps[type], caps, handle_caps[type] & caps);
-#endif
 	
 	return((handle_caps[type] & caps) != 0);
 }
@@ -1306,7 +1276,7 @@ gboolean _wapi_handle_ops_isowned (gpointer handle)
 	}
 }
 
-guint32 _wapi_handle_ops_special_wait (gpointer handle, guint32 timeout)
+guint32 _wapi_handle_ops_special_wait (gpointer handle, guint32 timeout, gboolean alertable)
 {
 	guint32 idx = GPOINTER_TO_UINT(handle);
 	WapiHandleType type;
@@ -1319,7 +1289,7 @@ guint32 _wapi_handle_ops_special_wait (gpointer handle, guint32 timeout)
 	
 	if (handle_ops[type] != NULL &&
 	    handle_ops[type]->special_wait != NULL) {
-		return(handle_ops[type]->special_wait (handle, timeout));
+		return(handle_ops[type]->special_wait (handle, timeout, alertable));
 	} else {
 		return(WAIT_FAILED);
 	}
@@ -1393,7 +1363,7 @@ gboolean DuplicateHandle (gpointer srcprocess, gpointer src,
 	if (src == _WAPI_PROCESS_CURRENT) {
 		*target = _wapi_process_duplicate ();
 	} else if (src == _WAPI_THREAD_CURRENT) {
-		*target = _wapi_thread_duplicate ();
+		g_assert_not_reached ();
 	} else {
 		_wapi_handle_ref (src);
 		*target = src;
@@ -1422,9 +1392,7 @@ again:
 		gpointer handle = handles[i];
 		guint32 idx = GPOINTER_TO_UINT(handle);
 
-#ifdef DEBUG
-		g_message ("%s: attempting to lock %p", __func__, handle);
-#endif
+		DEBUG ("%s: attempting to lock %p", __func__, handle);
 
 		type = _WAPI_PRIVATE_HANDLES(idx).type;
 
@@ -1433,10 +1401,8 @@ again:
 		if (thr_ret != 0) {
 			/* Bummer */
 			
-#ifdef DEBUG
-			g_message ("%s: attempt failed for %p: %s", __func__,
+			DEBUG ("%s: attempt failed for %p: %s", __func__,
 				   handle, strerror (thr_ret));
-#endif
 
 			thr_ret = _wapi_handle_unlock_shared_handles ();
 			g_assert (thr_ret == 0);
@@ -1460,19 +1426,15 @@ again:
 				iter=1;
 			}
 			
-#ifdef DEBUG
-			g_message ("%s: Backing off for %d ms", __func__,
+			DEBUG ("%s: Backing off for %d ms", __func__,
 				   iter*10);
-#endif
 			_wapi_handle_spin (10 * iter);
 			
 			goto again;
 		}
 	}
 	
-#ifdef DEBUG
-	g_message ("%s: Locked all handles", __func__);
-#endif
+	DEBUG ("%s: Locked all handles", __func__);
 
 	count=0;
 	*lowest=numhandles;
@@ -1483,9 +1445,7 @@ again:
 		
 		type = _WAPI_PRIVATE_HANDLES(idx).type;
 
-#ifdef DEBUG
-		g_message ("%s: Checking handle %p", __func__, handle);
-#endif
+		DEBUG ("%s: Checking handle %p", __func__, handle);
 
 		if(((_wapi_handle_test_capabilities (handle, WAPI_HANDLE_CAP_OWN)==TRUE) &&
 		    (_wapi_handle_ops_isowned (handle) == TRUE)) ||
@@ -1495,19 +1455,15 @@ again:
 		    _WAPI_PRIVATE_HANDLES(idx).signalled == TRUE)) {
 			count++;
 			
-#ifdef DEBUG
-			g_message ("%s: Handle %p signalled", __func__,
+			DEBUG ("%s: Handle %p signalled", __func__,
 				   handle);
-#endif
 			if(*lowest>i) {
 				*lowest=i;
 			}
 		}
 	}
 	
-#ifdef DEBUG
-	g_message ("%s: %d event handles signalled", __func__, count);
-#endif
+	DEBUG ("%s: %d event handles signalled", __func__, count);
 
 	if ((waitall == TRUE && count == numhandles) ||
 	    (waitall == FALSE && count > 0)) {
@@ -1516,9 +1472,7 @@ again:
 		ret=FALSE;
 	}
 	
-#ifdef DEBUG
-	g_message ("%s: Returning %d", __func__, ret);
-#endif
+	DEBUG ("%s: Returning %d", __func__, ret);
 
 	*retcount=count;
 	
@@ -1536,9 +1490,7 @@ void _wapi_handle_unlock_handles (guint32 numhandles, gpointer *handles)
 	for(i=0; i<numhandles; i++) {
 		gpointer handle = handles[i];
 		
-#ifdef DEBUG
-		g_message ("%s: unlocking handle %p", __func__, handle);
-#endif
+		DEBUG ("%s: unlocking handle %p", __func__, handle);
 
 		thr_ret = _wapi_handle_unlock_handle (handle);
 		g_assert (thr_ret == 0);
@@ -1590,9 +1542,7 @@ int _wapi_handle_timedwait_signal (struct timespec *timeout, gboolean poll)
 
 int _wapi_handle_wait_signal_handle (gpointer handle, gboolean alertable)
 {
-#ifdef DEBUG
-	g_message ("%s: waiting for %p", __func__, handle);
-#endif
+	DEBUG ("%s: waiting for %p", __func__, handle);
 	
 	return _wapi_handle_timedwait_signal_handle (handle, NULL, alertable, FALSE);
 }
@@ -1600,10 +1550,8 @@ int _wapi_handle_wait_signal_handle (gpointer handle, gboolean alertable)
 int _wapi_handle_timedwait_signal_handle (gpointer handle,
 										  struct timespec *timeout, gboolean alertable, gboolean poll)
 {
-#ifdef DEBUG
-	g_message ("%s: waiting for %p (type %s)", __func__, handle,
+	DEBUG ("%s: waiting for %p (type %s)", __func__, handle,
 		   _wapi_handle_typename[_wapi_handle_type (handle)]);
-#endif
 	
 	if (_WAPI_SHARED_HANDLE (_wapi_handle_type (handle))) {
 		if (WAPI_SHARED_HANDLE_DATA(handle).signalled == TRUE) {
@@ -1668,6 +1616,7 @@ _wapi_free_share_info (_WapiFileShare *share_info)
 		file_share_hash_lock ();
 		g_hash_table_remove (file_share_hash, share_info);
 		file_share_hash_unlock ();
+		/* The hashtable dtor frees share_info */
 	} else {
 		memset (share_info, '\0', sizeof(struct _WapiFileShare));
 	}
@@ -1690,7 +1639,7 @@ wapi_share_info_hash (gconstpointer data)
 	return s->inode;
 }
 
-gboolean _wapi_handle_get_or_set_share (dev_t device, ino_t inode,
+gboolean _wapi_handle_get_or_set_share (guint64 device, guint64 inode,
 					guint32 new_sharemode,
 					guint32 new_access,
 					guint32 *old_sharemode,
@@ -1721,7 +1670,7 @@ gboolean _wapi_handle_get_or_set_share (dev_t device, ino_t inode,
 		 */
 		if (!file_share_hash) {
 			file_share_hash = g_hash_table_new_full (wapi_share_info_hash, wapi_share_info_equal, NULL, g_free);
-			InitializeCriticalSection (&file_share_hash_mutex);
+			mono_mutex_init_recursive (&file_share_hash_mutex);
 		}
 			
 		tmp.device = device;
@@ -1829,6 +1778,9 @@ gboolean _wapi_handle_get_or_set_share (dev_t device, ino_t inode,
  */
 static void _wapi_handle_check_share_by_pid (struct _WapiFileShare *share_info)
 {
+#if defined(__native_client__)
+	g_assert_not_reached ();
+#else
 	if (kill (share_info->opened_by_pid, 0) == -1 &&
 	    (errno == ESRCH ||
 	     errno == EPERM)) {
@@ -1836,12 +1788,11 @@ static void _wapi_handle_check_share_by_pid (struct _WapiFileShare *share_info)
 		 * owned by someone else) so mark this share info as
 		 * dead
 		 */
-#ifdef DEBUG
-		g_message ("%s: Didn't find it, destroying entry", __func__);
-#endif
+		DEBUG ("%s: Didn't find it, destroying entry", __func__);
 
-		memset (share_info, '\0', sizeof(struct _WapiFileShare));
+		_wapi_free_share_info (share_info);
 	}
+#endif
 }
 
 #ifdef __linux__
@@ -1855,8 +1806,6 @@ static void _wapi_handle_check_share_by_pid (struct _WapiFileShare *share_info)
 void _wapi_handle_check_share (struct _WapiFileShare *share_info, int fd)
 {
 	gboolean found = FALSE, proc_fds = FALSE;
-	pid_t self = _wapi_getpid ();
-	int pid;
 	int thr_ret, i;
 	
 	/* Prevents entries from expiring under us if we remove this
@@ -1888,10 +1837,8 @@ void _wapi_handle_check_share (struct _WapiFileShare *share_info, int fd)
 				struct _WapiHandle_file *file_handle = &handle->u.file;
 
 				if (file_handle->share_info == share_info) {
-#ifdef DEBUG
-					g_message ("%s: handle 0x%x has this file open!",
+					DEBUG ("%s: handle 0x%x has this file open!",
 							   __func__, i);
-#endif
 
 					goto done;
 				}
@@ -1899,78 +1846,13 @@ void _wapi_handle_check_share (struct _WapiFileShare *share_info, int fd)
 		}
 	}
 
-	for (i = 0; i < _WAPI_HANDLE_INITIAL_COUNT; i++) {
-		struct _WapiHandleShared *shared;
-		struct _WapiHandle_process *process_handle;
-
-		shared = &_wapi_shared_layout->handles[i];
-		
-		if (shared->type == WAPI_HANDLE_PROCESS) {
-			DIR *fd_dir;
-			struct dirent *fd_entry;
-			char subdir[_POSIX_PATH_MAX];
-
-			process_handle = &shared->u.process;
-			pid = process_handle->id;
-		
-			/* Look in /proc/<pid>/fd/ but ignore
-			 * /proc/<our pid>/fd/<fd>, as we have the
-			 * file open too
-			 */
-			g_snprintf (subdir, _POSIX_PATH_MAX, "/proc/%d/fd",
-				    pid);
-			
-			fd_dir = opendir (subdir);
-			if (fd_dir == NULL) {
-				continue;
-			}
-
-#ifdef DEBUG
-			g_message ("%s: Looking in %s", __func__, subdir);
-#endif
-			
-			proc_fds = TRUE;
-			
-			while ((fd_entry = readdir (fd_dir)) != NULL) {
-				char path[_POSIX_PATH_MAX];
-				struct stat link_stat;
-				
-				if (!strcmp (fd_entry->d_name, ".") ||
-				    !strcmp (fd_entry->d_name, "..") ||
-				    (pid == self &&
-				     fd == atoi (fd_entry->d_name))) {
-					continue;
-				}
-
-				g_snprintf (path, _POSIX_PATH_MAX,
-					    "/proc/%d/fd/%s", pid,
-					    fd_entry->d_name);
-				
-				stat (path, &link_stat);
-				if (link_stat.st_dev == share_info->device &&
-				    link_stat.st_ino == share_info->inode) {
-#ifdef DEBUG
-					g_message ("%s:  Found it at %s",
-						   __func__, path);
-#endif
-
-					found = TRUE;
-				}
-			}
-			
-			closedir (fd_dir);
-		}
-	}
-
 	if (proc_fds == FALSE) {
 		_wapi_handle_check_share_by_pid (share_info);
 	} else if (found == FALSE) {
 		/* Blank out this entry, as it is stale */
-#ifdef DEBUG
-		g_message ("%s: Didn't find it, destroying entry", __func__);
-#endif
+		DEBUG ("%s: Didn't find it, destroying entry", __func__);
 
-		memset (share_info, '\0', sizeof(struct _WapiFileShare));
+		_wapi_free_share_info (share_info);
 	}
 
 done:
@@ -2008,8 +1890,6 @@ void _wapi_handle_dump (void)
 	guint32 i, k;
 	int thr_ret;
 	
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	g_assert (thr_ret == 0);
 	
@@ -2035,7 +1915,6 @@ void _wapi_handle_dump (void)
 
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 }
 
 static void _wapi_shared_details (gpointer handle_info)
@@ -2058,8 +1937,6 @@ void _wapi_handle_update_refs (void)
 	thr_ret = _wapi_shm_sem_lock (_WAPI_SHARED_SEM_FILESHARE);
 	g_assert(thr_ret == 0);
 
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&scan_mutex);
 	thr_ret = mono_mutex_lock (&scan_mutex);
 	
 	for(i = SLOT_INDEX (0); i < _wapi_private_handle_slot_count; i++) {
@@ -2070,29 +1947,21 @@ void _wapi_handle_update_refs (void)
 				if (_WAPI_SHARED_HANDLE(handle->type)) {
 					struct _WapiHandleShared *shared_data;
 				
-#ifdef DEBUG
-					g_message ("%s: (%d) handle 0x%x is SHARED (%s)", __func__, _wapi_getpid (), i * _WAPI_HANDLE_INITIAL_COUNT + k, _wapi_handle_typename[handle->type]);
-#endif
+					DEBUG ("%s: (%d) handle 0x%x is SHARED (%s)", __func__, _wapi_getpid (), i * _WAPI_HANDLE_INITIAL_COUNT + k, _wapi_handle_typename[handle->type]);
 
 					shared_data = &_wapi_shared_layout->handles[handle->u.shared.offset];
 
-#ifdef DEBUG
-					g_message ("%s: (%d) Updating timestamp of handle 0x%x", __func__, _wapi_getpid (), handle->u.shared.offset);
-#endif
+					DEBUG ("%s: (%d) Updating timestamp of handle 0x%x", __func__, _wapi_getpid (), handle->u.shared.offset);
 
 					InterlockedExchange ((gint32 *)&shared_data->timestamp, now);
 				} else if (handle->type == WAPI_HANDLE_FILE) {
 					struct _WapiHandle_file *file_handle = &handle->u.file;
 				
-#ifdef DEBUG
-					g_message ("%s: (%d) handle 0x%x is FILE", __func__, _wapi_getpid (), i * _WAPI_HANDLE_INITIAL_COUNT + k);
-#endif
+					DEBUG ("%s: (%d) handle 0x%x is FILE", __func__, _wapi_getpid (), i * _WAPI_HANDLE_INITIAL_COUNT + k);
 				
 					g_assert (file_handle->share_info != NULL);
 
-#ifdef DEBUG
-					g_message ("%s: (%d) Inc refs on fileshare 0x%x", __func__, _wapi_getpid (), (file_handle->share_info - &_wapi_fileshare_layout->share_info[0]) / sizeof(struct _WapiFileShare));
-#endif
+					DEBUG ("%s: (%d) Inc refs on fileshare 0x%x", __func__, _wapi_getpid (), (file_handle->share_info - &_wapi_fileshare_layout->share_info[0]) / sizeof(struct _WapiFileShare));
 
 					InterlockedExchange ((gint32 *)&file_handle->share_info->timestamp, now);
 				}
@@ -2102,7 +1971,6 @@ void _wapi_handle_update_refs (void)
 
 	thr_ret = mono_mutex_unlock (&scan_mutex);
 	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
 	
 	thr_ret = _wapi_shm_sem_unlock (_WAPI_SHARED_SEM_FILESHARE);
 
