@@ -6,6 +6,7 @@
  *
  * Copyright 2001-2003 Ximian, Inc (http://www.ximian.com)
  * Copyright 2004-2009 Novell, Inc (http://www.novell.com)
+ * Copyright 2011 Xamarin, Inc (http://www.xamarin.com)
  */
 #include <config.h>
 #include <stdio.h>
@@ -32,6 +33,10 @@
 #include <mono/utils/mono-path.h>
 #include <mono/metadata/reflection.h>
 #include <mono/metadata/coree.h>
+#include <mono/metadata/cil-coff.h>
+#include <mono/utils/mono-io-portability.h>
+#include <mono/utils/atomic.h>
+#include <mono/utils/mono-mutex.h>
 
 #ifndef HOST_WIN32
 #include <sys/types.h>
@@ -52,6 +57,7 @@ typedef struct  {
 /* the default search path is empty, the first slot is replaced with the computed value */
 static const char*
 default_path [] = {
+	NULL,
 	NULL,
 	NULL
 };
@@ -62,8 +68,12 @@ static char **assemblies_path = NULL;
 /* Contains the list of directories that point to auxiliary GACs */
 static char **extra_gac_paths = NULL;
 
+#ifndef DISABLE_ASSEMBLY_REMAPPING
 /* The list of system assemblies what will be remapped to the running
  * runtime version. WARNING: this list must be sorted.
+ * The integer number is an index in the MonoRuntimeInfo structure, whose
+ * values can be found in domain.c - supported_runtimes. Look there
+ * to understand what remapping will be made.
  */
 static const AssemblyVersionMap framework_assemblies [] = {
 	{"Accessibility", 0},
@@ -99,6 +109,8 @@ static const AssemblyVersionMap framework_assemblies [] = {
 	{"System.Data", 0},
 	{"System.Data.Linq", 2},
 	{"System.Data.OracleClient", 0},
+	{"System.Data.Services", 2},
+	{"System.Data.Services.Client", 2},
 	{"System.Data.SqlXml", 0},
 	{"System.Design", 0},
 	{"System.DirectoryServices", 0},
@@ -107,20 +119,28 @@ static const AssemblyVersionMap framework_assemblies [] = {
 	{"System.EnterpriseServices", 0},
 	{"System.Management", 0},
 	{"System.Messaging", 0},
+	{"System.Net", 2},
 	{"System.Runtime.Remoting", 0},
+	{"System.Runtime.Serialization", 3},
 	{"System.Runtime.Serialization.Formatters.Soap", 0},
 	{"System.Security", 0},
+	{"System.ServiceModel.Web", 2},
 	{"System.ServiceProcess", 0},
+	{"System.Transactions", 0},
 	{"System.Web", 0},
 	{"System.Web.Abstractions", 2},
+	{"System.Web.DynamicData", 2},
 	{"System.Web.Extensions", 2},
 	{"System.Web.Mobile", 0},
 	{"System.Web.Routing", 2},
 	{"System.Web.Services", 0},
 	{"System.Windows.Forms", 0},
 	{"System.Xml", 0},
+	{"System.Xml.Linq", 2},
+	{"WindowsBase", 3},
 	{"mscorlib", 0}
 };
+#endif
 
 /*
  * keeps track of loaded assemblies
@@ -150,18 +170,22 @@ mono_set_corlib_data (void *data, size_t size)
 #endif
 
 /* This protects loaded_assemblies and image->references */
-#define mono_assemblies_lock() EnterCriticalSection (&assemblies_mutex)
-#define mono_assemblies_unlock() LeaveCriticalSection (&assemblies_mutex)
-static CRITICAL_SECTION assemblies_mutex;
+#define mono_assemblies_lock() mono_mutex_lock (&assemblies_mutex)
+#define mono_assemblies_unlock() mono_mutex_unlock (&assemblies_mutex)
+static mono_mutex_t assemblies_mutex;
 
 /* If defined, points to the bundled assembly information */
 const MonoBundledAssembly **bundles;
 
+static mono_mutex_t assembly_binding_mutex;
+
 /* Loaded assembly binding info */
 static GSList *loaded_assembly_bindings = NULL;
 
 static MonoAssembly*
-mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, gboolean refonly, gboolean postload);
+mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, MonoAssembly *requesting, gboolean refonly, gboolean postload);
+static MonoAssembly*
+mono_assembly_load_full_internal (MonoAssemblyName *aname, MonoAssembly *requesting, const char *basedir, MonoImageOpenStatus *status, gboolean refonly);
 static MonoBoolean
 mono_assembly_is_in_gac (const gchar *filanem);
 
@@ -195,27 +219,37 @@ mono_public_tokens_are_equal (const unsigned char *pubt1, const unsigned char *p
 	return memcmp (pubt1, pubt2, 16) == 0;
 }
 
-static void
-check_path_env (void)
+/**
+ * mono_set_assemblies_path:
+ * @path: list of paths that contain directories where Mono will look for assemblies
+ *
+ * Use this method to override the standard assembly lookup system and
+ * override any assemblies coming from the GAC.  This is the method
+ * that supports the MONO_PATH variable.
+ *
+ * Notice that MONO_PATH and this method are really a very bad idea as
+ * it prevents the GAC from working and it prevents the standard
+ * resolution mechanisms from working.  Nonetheless, for some debugging
+ * situations and bootstrapping setups, this is useful to have. 
+ */
+void
+mono_set_assemblies_path (const char* path)
 {
-	const char *path;
 	char **splitted, **dest;
-	
-	path = g_getenv ("MONO_PATH");
-	if (!path)
-		return;
 
 	splitted = g_strsplit (path, G_SEARCHPATH_SEPARATOR_S, 1000);
 	if (assemblies_path)
 		g_strfreev (assemblies_path);
 	assemblies_path = dest = splitted;
-	while (*splitted){
-		if (**splitted)
-			*dest++ = *splitted;
+	while (*splitted) {
+		char *tmp = *splitted;
+		if (*tmp)
+			*dest++ = mono_path_canonicalize (tmp);
+		g_free (tmp);
 		splitted++;
 	}
 	*dest = *splitted;
-	
+
 	if (g_getenv ("MONO_DEBUG") == NULL)
 		return;
 
@@ -228,6 +262,27 @@ check_path_env (void)
 	}
 }
 
+/* Native Client can't get this info from an environment variable so */
+/* it's passed in to the runtime, or set manually by embedding code. */
+#ifdef __native_client__
+char* nacl_mono_path = NULL;
+#endif
+
+static void
+check_path_env (void)
+{
+	const char* path;
+	path = g_getenv ("MONO_PATH");
+#ifdef __native_client__
+	if (!path)
+		path = nacl_mono_path;
+#endif
+	if (!path || assemblies_path != NULL)
+		return;
+
+	mono_set_assemblies_path(path);
+}
+
 static void
 check_extra_gac_path_env (void) {
 	const char *path;
@@ -529,7 +584,7 @@ compute_base (char *path)
 		return NULL;
 
 	/* Not a well known Mono executable, we are embedded, cant guess the base  */
-	if (strcmp (p, "/mono") && strcmp (p, "/monodis") && strcmp (p, "/mint") && strcmp (p, "/monodiet"))
+	if (strcmp (p, "/mono") && strcmp (p, "/mono-boehm") && strcmp (p, "/mono-sgen") && strcmp (p, "/pedump") && strcmp (p, "/monodis"))
 		return NULL;
 	    
 	*p = 0;
@@ -549,7 +604,7 @@ fallback (void)
 	mono_set_dirs (MONO_ASSEMBLIES, MONO_CFG_DIR);
 }
 
-static void
+static G_GNUC_UNUSED void
 set_dirs (char *exe)
 {
 	char *base;
@@ -566,7 +621,7 @@ set_dirs (char *exe)
 
 	config = g_build_filename (base, "etc", NULL);
 	lib = g_build_filename (base, "lib", NULL);
-	mono = g_build_filename (lib, "mono/1.0", NULL);
+	mono = g_build_filename (lib, "mono/2.0", NULL);
 	if (stat (mono, &buf) == -1)
 		fallback ();
 	else {
@@ -606,6 +661,7 @@ mono_set_rootdir (void)
 		gchar buf[4096];
  		guint buf_size = sizeof (buf);
  
+		name = NULL;
  		if (_NSGetExecutablePath (buf, &buf_size) == 0)
  			name = g_strdup (buf);
  
@@ -613,8 +669,6 @@ mono_set_rootdir (void)
  			fallback ();
  			return;
  		}
-
-		name = mono_path_resolve_symlinks (name);
  	}
 #endif
 
@@ -687,7 +741,20 @@ mono_assemblies_init (void)
 	check_path_env ();
 	check_extra_gac_path_env ();
 
-	InitializeCriticalSection (&assemblies_mutex);
+	mono_mutex_init_recursive (&assemblies_mutex);
+	mono_mutex_init (&assembly_binding_mutex);
+}
+
+static void
+mono_assembly_binding_lock (void)
+{
+	mono_locks_mutex_acquire (&assembly_binding_mutex, AssemblyBindingLock);
+}
+
+static void
+mono_assembly_binding_unlock (void)
+{
+	mono_locks_mutex_release (&assembly_binding_mutex, AssemblyBindingLock);
 }
 
 gboolean
@@ -695,6 +762,7 @@ mono_assembly_fill_assembly_name (MonoImage *image, MonoAssemblyName *aname)
 {
 	MonoTableInfo *t = &image->tables [MONO_TABLE_ASSEMBLY];
 	guint32 cols [MONO_ASSEMBLY_SIZE];
+	gint32 machine, flags;
 
 	if (!t->rows)
 		return FALSE;
@@ -739,6 +807,31 @@ mono_assembly_fill_assembly_name (MonoImage *image, MonoAssemblyName *aname)
 	else
 		aname->public_key = 0;
 
+	machine = ((MonoCLIImageInfo*)(image->image_info))->cli_header.coff.coff_machine;
+	flags = ((MonoCLIImageInfo*)(image->image_info))->cli_cli_header.ch_flags;
+	switch (machine) {
+	case COFF_MACHINE_I386:
+		/* https://bugzilla.xamarin.com/show_bug.cgi?id=17632 */
+		if (flags & (CLI_FLAGS_32BITREQUIRED|CLI_FLAGS_PREFERRED32BIT))
+			aname->arch = MONO_PROCESSOR_ARCHITECTURE_X86;
+		else if ((flags & 0x70) == 0x70)
+			aname->arch = MONO_PROCESSOR_ARCHITECTURE_NONE;
+		else
+			aname->arch = MONO_PROCESSOR_ARCHITECTURE_MSIL;
+		break;
+	case COFF_MACHINE_IA64:
+		aname->arch = MONO_PROCESSOR_ARCHITECTURE_IA64;
+		break;
+	case COFF_MACHINE_AMD64:
+		aname->arch = MONO_PROCESSOR_ARCHITECTURE_AMD64;
+		break;
+	case COFF_MACHINE_ARM:
+		aname->arch = MONO_PROCESSOR_ARCHITECTURE_ARM;
+		break;
+	default:
+		break;
+	}
+
 	return TRUE;
 }
 
@@ -755,9 +848,11 @@ mono_assembly_fill_assembly_name (MonoImage *image, MonoAssemblyName *aname)
 char*
 mono_stringify_assembly_name (MonoAssemblyName *aname)
 {
+	const char *quote = (aname->name && g_ascii_isspace (aname->name [0])) ? "\"" : "";
+
 	return g_strdup_printf (
-		"%s, Version=%d.%d.%d.%d, Culture=%s, PublicKeyToken=%s%s",
-		aname->name,
+		"%s%s%s, Version=%d.%d.%d.%d, Culture=%s, PublicKeyToken=%s%s",
+		quote, aname->name, quote,
 		aname->major, aname->minor, aname->build, aname->revision,
 		aname->culture && *aname->culture? aname->culture: "neutral",
 		aname->public_key_token [0] ? (char *)aname->public_key_token : "null",
@@ -796,6 +891,60 @@ mono_assembly_addref (MonoAssembly *assembly)
 	InterlockedIncrement (&assembly->ref_count);
 }
 
+/*
+ * CAUTION: This table must be kept in sync with
+ *          ivkm/reflect/Fusion.cs
+ */
+
+#define SILVERLIGHT_KEY "7cec85d7bea7798e"
+#define WINFX_KEY "31bf3856ad364e35"
+#define ECMA_KEY "b77a5c561934e089"
+#define MSFINAL_KEY "b03f5f7f11d50a3a"
+
+typedef struct {
+	const char *name;
+	const char *from;
+	const char *to;
+} KeyRemapEntry;
+
+static KeyRemapEntry key_remap_table[] = {
+	{ "Microsoft.CSharp", WINFX_KEY, MSFINAL_KEY },
+	{ "System", SILVERLIGHT_KEY, ECMA_KEY },
+	{ "System.ComponentModel.Composition", WINFX_KEY, ECMA_KEY },
+	{ "System.ComponentModel.DataAnnotations", "ddd0da4d3e678217", WINFX_KEY },
+	{ "System.Core", SILVERLIGHT_KEY, ECMA_KEY },
+	// FIXME: MS uses MSFINAL_KEY for .NET 4.5
+	{ "System.Net", SILVERLIGHT_KEY, MSFINAL_KEY },
+	{ "System.Numerics", WINFX_KEY, ECMA_KEY },
+	{ "System.Runtime.Serialization", SILVERLIGHT_KEY, ECMA_KEY },
+	{ "System.ServiceModel", WINFX_KEY, ECMA_KEY },
+	{ "System.ServiceModel.Web", SILVERLIGHT_KEY, WINFX_KEY },
+	{ "System.Windows", SILVERLIGHT_KEY, MSFINAL_KEY },
+	{ "System.Xml", SILVERLIGHT_KEY, ECMA_KEY },
+	{ "System.Xml.Linq", WINFX_KEY, ECMA_KEY },
+	{ "System.Xml.Serialization", WINFX_KEY, ECMA_KEY }
+};
+
+static void
+remap_keys (MonoAssemblyName *aname)
+{
+	int i;
+	for (i = 0; i < G_N_ELEMENTS (key_remap_table); i++) {
+		const KeyRemapEntry *entry = &key_remap_table [i];
+
+		if (strcmp (aname->name, entry->name) ||
+		    !mono_public_tokens_are_equal (aname->public_key_token, (const unsigned char*) entry->from))
+			continue;
+
+		memcpy (aname->public_key_token, entry->to, MONO_PUBLIC_KEY_TOKEN_LENGTH);
+		     
+		mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY,
+			    "Remapped public key token of retargetable assembly %s from %s to %s",
+			    aname->name, entry->from, entry->to);
+		return;
+	}
+}
+
 static MonoAssemblyName *
 mono_assembly_remap_version (MonoAssemblyName *aname, MonoAssemblyName *dest_aname)
 {
@@ -803,8 +952,40 @@ mono_assembly_remap_version (MonoAssemblyName *aname, MonoAssemblyName *dest_ana
 	int pos, first, last;
 
 	if (aname->name == NULL) return aname;
+
 	current_runtime = mono_get_runtime_info ();
 
+	if (aname->flags & ASSEMBLYREF_RETARGETABLE_FLAG) {
+		const AssemblyVersionSet* vset;
+
+		/* Remap to current runtime */
+		vset = &current_runtime->version_sets [0];
+
+		memcpy (dest_aname, aname, sizeof(MonoAssemblyName));
+		dest_aname->major = vset->major;
+		dest_aname->minor = vset->minor;
+		dest_aname->build = vset->build;
+		dest_aname->revision = vset->revision;
+		dest_aname->flags &= ~ASSEMBLYREF_RETARGETABLE_FLAG;
+
+		/* Remap assembly name */
+		if (!strcmp (aname->name, "System.Net"))
+			dest_aname->name = g_strdup ("System");
+		
+		remap_keys (dest_aname);
+
+		mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY,
+					"The request to load the retargetable assembly %s v%d.%d.%d.%d was remapped to %s v%d.%d.%d.%d",
+					aname->name,
+					aname->major, aname->minor, aname->build, aname->revision,
+					dest_aname->name,
+					vset->major, vset->minor, vset->build, vset->revision
+					);
+
+		return dest_aname;
+	}
+	
+#ifndef DISABLE_ASSEMBLY_REMAPPING
 	first = 0;
 	last = G_N_ELEMENTS (framework_assemblies) - 1;
 	
@@ -842,6 +1023,8 @@ mono_assembly_remap_version (MonoAssemblyName *aname, MonoAssemblyName *dest_ana
 			first = pos + 1;
 		}
 	}
+#endif
+
 	return aname;
 }
 
@@ -897,6 +1080,7 @@ mono_assembly_load_reference (MonoImage *image, int index)
 		MonoTableInfo *t = &image->tables [MONO_TABLE_ASSEMBLYREF];
 	
 		image->references = g_new0 (MonoAssembly *, t->rows + 1);
+		image->nreferences = t->rows;
 	}
 	reference = image->references [index];
 	mono_assemblies_unlock ();
@@ -908,12 +1092,12 @@ mono_assembly_load_reference (MonoImage *image, int index)
 	if (image->assembly && image->assembly->ref_only) {
 		/* We use the loaded corlib */
 		if (!strcmp (aname.name, "mscorlib"))
-			reference = mono_assembly_load_full (&aname, image->assembly->basedir, &status, FALSE);
+			reference = mono_assembly_load_full_internal (&aname, image->assembly, image->assembly->basedir, &status, FALSE);
 		else {
 			reference = mono_assembly_loaded_full (&aname, TRUE);
 			if (!reference)
 				/* Try a postload search hook */
-				reference = mono_assembly_invoke_search_hook_internal (&aname, TRUE, TRUE);
+				reference = mono_assembly_invoke_search_hook_internal (&aname, image->assembly, TRUE, TRUE);
 		}
 
 		/*
@@ -922,8 +1106,17 @@ mono_assembly_load_reference (MonoImage *image, int index)
 		*/
 		if (!reference)
 			reference = REFERENCE_MISSING;
-	} else
-		reference = mono_assembly_load (&aname, image->assembly? image->assembly->basedir: NULL, &status);
+	} else {
+		/* we first try without setting the basedir: this can eventually result in a ResolveAssembly
+		 * event which is the MS .net compatible behaviour (the assemblyresolve_event3.cs test has been fixed
+		 * accordingly, it would fail on the MS runtime before).
+		 * The second load attempt has the basedir set to keep compatibility with the old mono behavior, for
+		 * example bug-349190.2.cs and who knows how much more code in the wild.
+		 */
+		reference = mono_assembly_load_full_internal (&aname, image->assembly, NULL, &status, FALSE);
+		if (!reference && image->assembly)
+			reference = mono_assembly_load_full_internal (&aname, image->assembly, image->assembly->basedir, &status, FALSE);
+	}
 
 	if (reference == NULL){
 		char *extra_msg;
@@ -960,11 +1153,11 @@ mono_assembly_load_reference (MonoImage *image, int index)
 		if (reference != REFERENCE_MISSING){
 			mono_assembly_addref (reference);
 			if (image->assembly)
-				mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Assembly Ref addref %s %p -> %s %p: %d\n",
+				mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Assembly Ref addref %s[%p] -> %s[%p]: %d",
 				    image->assembly->aname.name, image->assembly, reference->aname.name, reference, reference->ref_count);
 		} else {
 			if (image->assembly)
-				mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Failed to load assembly %s %p\n",
+				mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Failed to load assembly %s[%p]\n",
 				    image->assembly->aname.name, image->assembly);
 		}
 		
@@ -1041,13 +1234,36 @@ struct AssemblySearchHook {
 AssemblySearchHook *assembly_search_hook = NULL;
 
 static MonoAssembly*
-mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, gboolean refonly, gboolean postload)
+mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, MonoAssembly *requesting, gboolean refonly, gboolean postload)
 {
 	AssemblySearchHook *hook;
 
 	for (hook = assembly_search_hook; hook; hook = hook->next) {
 		if ((hook->refonly == refonly) && (hook->postload == postload)) {
-			MonoAssembly *ass = hook->func (aname, hook->user_data);
+			MonoAssembly *ass;
+			/**
+			  * A little explanation is in order here.
+			  *
+			  * The default postload search hook needs to know the requesting assembly to report it to managed code.
+			  * The embedding API exposes a search hook that doesn't take such argument.
+			  *
+			  * The original fix would call the default search hook before all the registered ones and pass
+			  * the requesting assembly to it. It works but broke a very suddle embedding API aspect that some users
+			  * rely on. Which is the ordering between user hooks and the default runtime hook.
+			  *
+			  * Registering the hook after mono_jit_init would let your hook run before the default one and
+			  * when using it to handle non standard app layouts this could save your app from a massive amount
+			  * of syscalls that the default hook does when probing all sorts of places. Slow targets with horrible IO
+			  * are all using this trick and if we broke this assumption they would be very disapointed at us.
+			  *
+			  * So what's the fix? We register the default hook using regular means and special case it when iterating
+			  * over the registered hooks. This preserves ordering and enables managed resolve hooks to get the requesting
+			  * assembly.
+			  */
+			if (hook->func == (void*)mono_domain_assembly_postload_search)
+				ass = mono_domain_assembly_postload_search (aname, requesting, refonly);
+			else
+				ass = hook->func (aname, hook->user_data);
 			if (ass)
 				return ass;
 		}
@@ -1059,7 +1275,7 @@ mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, gboolean ref
 MonoAssembly*
 mono_assembly_invoke_search_hook (MonoAssemblyName *aname)
 {
-	return mono_assembly_invoke_search_hook_internal (aname, FALSE, FALSE);
+	return mono_assembly_invoke_search_hook_internal (aname, NULL, FALSE, FALSE);
 }
 
 static void
@@ -1291,11 +1507,13 @@ mono_assembly_open_from_bundle (const char *filename, MonoImageOpenStatus *statu
 		}
 	}
 	mono_assemblies_unlock ();
-	g_free (name);
 	if (image) {
 		mono_image_addref (image);
+		mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Assembly Loader loaded assembly from bundle: '%s'.", name);
+		g_free (name);
 		return image;
 	}
+	g_free (name);
 	return NULL;
 }
 
@@ -1307,6 +1525,7 @@ mono_assembly_open_full (const char *filename, MonoImageOpenStatus *status, gboo
 	MonoImageOpenStatus def_status;
 	gchar *fname;
 	gchar *new_fname;
+	gboolean loaded_from_bundle;
 	
 	g_return_val_if_fail (filename != NULL, NULL);
 
@@ -1358,8 +1577,11 @@ mono_assembly_open_full (const char *filename, MonoImageOpenStatus *status, gboo
 	
 	image = NULL;
 
-	if (bundles != NULL)
+	loaded_from_bundle = FALSE;
+	if (bundles != NULL) {
 		image = mono_assembly_open_from_bundle (fname, status, refonly);
+		loaded_from_bundle = image != NULL;
+	}
 
 	if (!image)
 		image = mono_image_open_full (fname, status, refonly);
@@ -1382,7 +1604,8 @@ mono_assembly_open_full (const char *filename, MonoImageOpenStatus *status, gboo
 	ass = mono_assembly_load_from_full (image, fname, status, refonly);
 
 	if (ass) {
-		mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY,
+		if (!loaded_from_bundle)
+			mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY,
 				"Assembly Loader loaded assembly from location: '%s'.", filename);
 		if (!refonly)
 			mono_config_for_assembly (ass->image);
@@ -1559,14 +1782,14 @@ mono_assembly_load_from_full (MonoImage *image, const char*fname,
 	/* Add a non-temporary reference because of ass->image */
 	mono_image_addref (image);
 
-	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Image addref %s %p -> %s %p: %d\n", ass->aname.name, ass, image->name, image, image->ref_count);
+	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Image addref %s[%p] -> %s[%p]: %d", ass->aname.name, ass, image->name, image, image->ref_count);
 
 	/* 
 	 * The load hooks might take locks so we can't call them while holding the
 	 * assemblies lock.
 	 */
 	if (ass->aname.name) {
-		ass2 = mono_assembly_invoke_search_hook_internal (&ass->aname, refonly, FALSE);
+		ass2 = mono_assembly_invoke_search_hook_internal (&ass->aname, NULL, refonly, FALSE);
 		if (ass2) {
 			g_free (ass);
 			g_free (base_dir);
@@ -1635,7 +1858,7 @@ mono_assembly_name_free (MonoAssemblyName *aname)
 }
 
 static gboolean
-parse_public_key (const gchar *key, gchar** pubkey)
+parse_public_key (const gchar *key, gchar** pubkey, gboolean *is_ecma)
 {
 	const gchar *pkey;
 	gchar header [16], val, *arr;
@@ -1644,7 +1867,16 @@ parse_public_key (const gchar *key, gchar** pubkey)
 	keylen = strlen (key) >> 1;
 	if (keylen < 1)
 		return FALSE;
-	
+
+	/* allow the ECMA standard key */
+	if (strcmp (key, "00000000000000000400000000000000") == 0) {
+		if (pubkey) {
+			*pubkey = g_strdup (key);
+			*is_ecma = TRUE;
+		}
+		return TRUE;
+	}
+	*is_ecma = FALSE;
 	val = g_ascii_xdigit_value (key [0]) << 4;
 	val |= g_ascii_xdigit_value (key [1]);
 	switch (val) {
@@ -1767,10 +1999,20 @@ build_assembly_name (const char *name, const char *version, const char *culture,
 	}
 
 	if (key) {
-		if (strcmp (key, "null") == 0 || !parse_public_key (key, &pkey)) {
+		gboolean is_ecma;
+		if (strcmp (key, "null") == 0 || !parse_public_key (key, &pkey, &is_ecma)) {
 			mono_assembly_name_free (aname);
 			return FALSE;
 		}
+
+		if (is_ecma) {
+			if (save_public_key)
+				aname->public_key = (guint8*)pkey;
+			else
+				g_free (pkey);
+			g_strlcpy ((gchar*)aname->public_key_token, "b77a5c561934e089", MONO_PUBLIC_KEY_TOKEN_LENGTH);
+			return TRUE;
+		}
 		
 		len = mono_metadata_decode_blob_size ((const gchar *) pkey, (const gchar **) &pkeyptr);
 		// We also need to generate the key token
@@ -1805,6 +2047,25 @@ parse_assembly_directory_name (const char *name, const char *dirname, MonoAssemb
 	return res;
 }
 
+static gboolean
+split_key_value (const gchar *pair, gchar **key, guint32 *keylen, gchar **value)
+{
+	char *eqsign = strchr (pair, '=');
+	if (!eqsign) {
+		*key = NULL;
+		*keylen = 0;
+		*value = NULL;
+		return FALSE;
+	}
+
+	*key = (gchar*)pair;
+	*keylen = eqsign - *key;
+	while (*keylen > 0 && g_ascii_isspace ((*key) [*keylen - 1]))
+		(*keylen)--;
+	*value = g_strstrip (eqsign + 1);
+	return TRUE;
+}
+
 gboolean
 mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboolean save_public_key, gboolean *is_version_defined, gboolean *is_token_defined)
 {
@@ -1815,13 +2076,14 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 	gchar *key = NULL;
 	gchar *retargetable = NULL;
 	gboolean res;
-	gchar *value;
+	gchar *value, *part_name;
+	guint32 part_name_len;
 	gchar **parts;
 	gchar **tmp;
 	gboolean version_defined;
 	gboolean token_defined;
 	guint32 flags = 0;
-	guint32 arch = PROCESSOR_ARCHITECTURE_NONE;
+	guint32 arch = MONO_PROCESSOR_ARCHITECTURE_NONE;
 
 	if (!is_version_defined)
 		is_version_defined = &version_defined;
@@ -1841,10 +2103,12 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 	tmp++;
 
 	while (*tmp) {
-		value = g_strstrip (*tmp);
-		if (!g_ascii_strncasecmp (value, "Version=", 8)) {
+		if (!split_key_value (g_strstrip (*tmp), &part_name, &part_name_len, &value))
+			goto cleanup_and_fail;
+
+		if (part_name_len == 7 && !g_ascii_strncasecmp (part_name, "Version", part_name_len)) {
 			*is_version_defined = TRUE;
-			version = g_strstrip (value + 8);
+			version = value;
 			if (strlen (version) == 0) {
 				goto cleanup_and_fail;
 			}
@@ -1852,8 +2116,8 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 			continue;
 		}
 
-		if (!g_ascii_strncasecmp (value, "Culture=", 8)) {
-			culture = g_strstrip (value + 8);
+		if (part_name_len == 7 && !g_ascii_strncasecmp (part_name, "Culture", part_name_len)) {
+			culture = value;
 			if (strlen (culture) == 0) {
 				goto cleanup_and_fail;
 			}
@@ -1861,9 +2125,9 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 			continue;
 		}
 
-		if (!g_ascii_strncasecmp (value, "PublicKeyToken=", 15)) {
+		if (part_name_len == 14 && !g_ascii_strncasecmp (part_name, "PublicKeyToken", part_name_len)) {
 			*is_token_defined = TRUE;
-			token = g_strstrip (value + 15);
+			token = value;
 			if (strlen (token) == 0) {
 				goto cleanup_and_fail;
 			}
@@ -1871,8 +2135,8 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 			continue;
 		}
 
-		if (!g_ascii_strncasecmp (value, "PublicKey=", 10)) {
-			key = g_strstrip (value + 10);
+		if (part_name_len == 9 && !g_ascii_strncasecmp (part_name, "PublicKey", part_name_len)) {
+			key = value;
 			if (strlen (key) == 0) {
 				goto cleanup_and_fail;
 			}
@@ -1880,8 +2144,8 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 			continue;
 		}
 
-		if (!g_ascii_strncasecmp (value, "Retargetable=", 13)) {
-			retargetable = g_strstrip (value + 13);
+		if (part_name_len == 12 && !g_ascii_strncasecmp (part_name, "Retargetable", part_name_len)) {
+			retargetable = value;
 			if (strlen (retargetable) == 0) {
 				goto cleanup_and_fail;
 			}
@@ -1894,18 +2158,15 @@ mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboole
 			continue;
 		}
 
-		if (!g_ascii_strncasecmp (value, "ProcessorArchitecture=", 22)) {
-			char *s = g_strstrip (value + 22);
-			if (!g_ascii_strcasecmp (s, "None"))
-				arch = PROCESSOR_ARCHITECTURE_NONE;
-			else if (!g_ascii_strcasecmp (s, "MSIL"))
-				arch = PROCESSOR_ARCHITECTURE_MSIL;
-			else if (!g_ascii_strcasecmp (s, "X86"))
-				arch = PROCESSOR_ARCHITECTURE_X86;
-			else if (!g_ascii_strcasecmp (s, "IA64"))
-				arch = PROCESSOR_ARCHITECTURE_IA64;
-			else if (!g_ascii_strcasecmp (s, "AMD64"))
-				arch = PROCESSOR_ARCHITECTURE_AMD64;
+		if (part_name_len == 21 && !g_ascii_strncasecmp (part_name, "ProcessorArchitecture", part_name_len)) {
+			if (!g_ascii_strcasecmp (value, "MSIL"))
+				arch = MONO_PROCESSOR_ARCHITECTURE_MSIL;
+			else if (!g_ascii_strcasecmp (value, "X86"))
+				arch = MONO_PROCESSOR_ARCHITECTURE_X86;
+			else if (!g_ascii_strcasecmp (value, "IA64"))
+				arch = MONO_PROCESSOR_ARCHITECTURE_IA64;
+			else if (!g_ascii_strcasecmp (value, "AMD64"))
+				arch = MONO_PROCESSOR_ARCHITECTURE_AMD64;
 			else
 				goto cleanup_and_fail;
 			tmp++;
@@ -2073,7 +2334,8 @@ MonoAssembly*
 mono_assembly_load_with_partial_name (const char *name, MonoImageOpenStatus *status)
 {
 	MonoAssembly *res;
-	MonoAssemblyName *aname, base_name, maped_aname;
+	MonoAssemblyName *aname, base_name;
+	MonoAssemblyName mapped_aname;
 	gchar *fullname, *gacpath;
 	gchar **paths;
 
@@ -2088,7 +2350,7 @@ mono_assembly_load_with_partial_name (const char *name, MonoImageOpenStatus *sta
 	 * correct version for system assemblies.
 	 */ 
 	if ((aname->major | aname->minor | aname->build | aname->revision) == 0)
-		aname = mono_assembly_remap_version (aname, &maped_aname);
+		aname = mono_assembly_remap_version (aname, &mapped_aname);
 	
 	res = mono_assembly_loaded (aname);
 	if (res) {
@@ -2130,7 +2392,7 @@ mono_assembly_load_with_partial_name (const char *name, MonoImageOpenStatus *sta
 		res->in_gac = TRUE;
 	else {
 		MonoDomain *domain = mono_domain_get ();
-		MonoReflectionAssembly *refasm = mono_try_assembly_resolve (domain, mono_string_new (domain, name), FALSE);
+		MonoReflectionAssembly *refasm = mono_try_assembly_resolve (domain, mono_string_new (domain, name), NULL, FALSE);
 		if (refasm)
 			res = refasm->assembly;
 	}
@@ -2271,7 +2533,7 @@ mono_assembly_bind_version (MonoAssemblyBindingInfo *info, MonoAssemblyName *ana
 	return dest_name;
 }
 
-/* LOCKING: Assumes that we are already locked */
+/* LOCKING: assembly_binding lock must be held */
 static MonoAssemblyBindingInfo*
 search_binding_loaded (MonoAssemblyName *aname)
 {
@@ -2348,16 +2610,23 @@ assembly_binding_info_parsed (MonoAssemblyBindingInfo *info, void *user_data)
 	domain->assembly_bindings = g_slist_append_mempool (domain->mp, domain->assembly_bindings, info_copy);
 }
 
+static int
+get_version_number (int major, int minor)
+{
+	return major * 256 + minor;
+}
+
 static inline gboolean
 info_major_minor_in_range (MonoAssemblyBindingInfo *info, MonoAssemblyName *aname)
 {
+	int aname_version_number = get_version_number (aname->major, aname->minor);
 	if (!info->has_old_version_bottom)
 		return FALSE;
 
-	if (info->old_version_bottom.major > aname->major || info->old_version_bottom.minor > aname->minor)
+	if (get_version_number (info->old_version_bottom.major, info->old_version_bottom.minor) > aname_version_number)
 		return FALSE;
 
-	if (info->has_old_version_top && (info->old_version_top.major < aname->major || info->old_version_top.minor < aname->minor))
+	if (info->has_old_version_top && get_version_number (info->old_version_top.major, info->old_version_top.minor) < aname_version_number)
 		return FALSE;
 
 	/* This is not the nicest way to do it, but it's a by-product of the way parsing is done */
@@ -2407,15 +2676,17 @@ mono_assembly_apply_binding (MonoAssemblyName *aname, MonoAssemblyName *dest_nam
 		return aname;
 
 	domain = mono_domain_get ();
-	mono_loader_lock ();
+
+	mono_assembly_binding_lock ();
 	info = search_binding_loaded (aname);
+	mono_assembly_binding_unlock ();
+
 	if (!info) {
 		mono_domain_lock (domain);
 		info = get_per_domain_assembly_binding_info (domain, aname);
 		mono_domain_unlock (domain);
 	}
 
-	mono_loader_unlock ();
 	if (info) {
 		if (!check_policy_versions (info, aname))
 			return aname;
@@ -2427,19 +2698,29 @@ mono_assembly_apply_binding (MonoAssemblyName *aname, MonoAssemblyName *dest_nam
 	if (domain && domain->setup && domain->setup->configuration_file) {
 		mono_domain_lock (domain);
 		if (!domain->assembly_bindings_parsed) {
-			gchar *domain_config_file = mono_string_to_utf8 (domain->setup->configuration_file);
+			gchar *domain_config_file_name = mono_string_to_utf8 (domain->setup->configuration_file);
+			gchar *domain_config_file_path = mono_portability_find_file (domain_config_file_name, TRUE);
 
-			mono_config_parse_assembly_bindings (domain_config_file, aname->major, aname->minor, domain, assembly_binding_info_parsed);
+			if (!domain_config_file_path)
+				domain_config_file_path = domain_config_file_name;
+			
+			mono_config_parse_assembly_bindings (domain_config_file_path, aname->major, aname->minor, domain, assembly_binding_info_parsed);
 			domain->assembly_bindings_parsed = TRUE;
-			g_free (domain_config_file);
+			if (domain_config_file_name != domain_config_file_path)
+				g_free (domain_config_file_name);
+			g_free (domain_config_file_path);
+		}
+
+		info2 = get_per_domain_assembly_binding_info (domain, aname);
+
+		if (info2) {
+			info = g_memdup (info2, sizeof (MonoAssemblyBindingInfo));
+			info->name = g_strdup (info2->name);
+			info->culture = g_strdup (info2->culture);
+			info->domain_id = domain->domain_id;
 		}
-		mono_domain_unlock (domain);
 
-		mono_loader_lock ();
-		mono_domain_lock (domain);
-		info = get_per_domain_assembly_binding_info (domain, aname);
 		mono_domain_unlock (domain);
-		mono_loader_unlock ();
 	}
 
 	if (!info) {
@@ -2463,7 +2744,7 @@ mono_assembly_apply_binding (MonoAssemblyName *aname, MonoAssemblyName *dest_nam
 		g_strlcpy ((char *)info->public_key_token, (const char *)aname->public_key_token, MONO_PUBLIC_KEY_TOKEN_LENGTH);
 	}
 	
-	mono_loader_lock ();
+	mono_assembly_binding_lock ();
 	info2 = search_binding_loaded (aname);
 	if (info2) {
 		/* This binding was added by another thread 
@@ -2475,7 +2756,7 @@ mono_assembly_apply_binding (MonoAssemblyName *aname, MonoAssemblyName *dest_nam
 	} else
 		loaded_assembly_bindings = g_slist_prepend (loaded_assembly_bindings, info);
 		
-	mono_loader_unlock ();
+	mono_assembly_binding_unlock ();
 	
 	if (!info->is_valid || !check_policy_versions (info, aname))
 		return aname;
@@ -2560,6 +2841,7 @@ MonoAssembly*
 mono_assembly_load_corlib (const MonoRuntimeInfo *runtime, MonoImageOpenStatus *status)
 {
 	char *corlib_file;
+	MonoAssemblyName *aname;
 
 	if (corlib) {
 		/* g_print ("corlib already loaded\n"); */
@@ -2581,7 +2863,14 @@ mono_assembly_load_corlib (const MonoRuntimeInfo *runtime, MonoImageOpenStatus *
 			return corlib;
 	}
 #endif
-	
+
+	aname = mono_assembly_name_new ("mscorlib.dll");
+	corlib = invoke_assembly_preload_hook (aname, assemblies_path);
+	mono_assembly_name_free (aname);
+	g_free (aname);
+	if (corlib != NULL)
+		return corlib;
+
 	if (assemblies_path) {
 		corlib = load_in_path ("mscorlib.dll", (const char**)assemblies_path, status, FALSE);
 		if (corlib)
@@ -2600,7 +2889,10 @@ mono_assembly_load_corlib (const MonoRuntimeInfo *runtime, MonoImageOpenStatus *
 	}
 	corlib = load_in_path (corlib_file, default_path, status, FALSE);
 	g_free (corlib_file);
-
+	
+	if (corlib && !strcmp (runtime->framework_version, "4.5"))
+		default_path [1] = g_strdup_printf ("%s/mono/4.5/Facades", default_path [0]);
+		
 	return corlib;
 }
 
@@ -2612,7 +2904,8 @@ mono_assembly_load_full_nosearch (MonoAssemblyName *aname,
 {
 	MonoAssembly *result;
 	char *fullpath, *filename;
-	MonoAssemblyName maped_aname, maped_name_pp;
+	MonoAssemblyName maped_aname;
+	MonoAssemblyName maped_name_pp;
 	int ext_index;
 	const char *ext;
 	int len;
@@ -2679,6 +2972,17 @@ mono_assembly_load_full_nosearch (MonoAssemblyName *aname,
 	return result;
 }
 
+MonoAssembly*
+mono_assembly_load_full_internal (MonoAssemblyName *aname, MonoAssembly *requesting, const char *basedir, MonoImageOpenStatus *status, gboolean refonly)
+{
+	MonoAssembly *result = mono_assembly_load_full_nosearch (aname, basedir, status, refonly);
+
+	if (!result)
+		/* Try a postload search hook */
+		result = mono_assembly_invoke_search_hook_internal (aname, requesting, refonly, TRUE);
+	return result;
+}
+
 /**
  * mono_assembly_load_full:
  * @aname: A MonoAssemblyName with the assembly name to load.
@@ -2698,12 +3002,7 @@ mono_assembly_load_full_nosearch (MonoAssemblyName *aname,
 MonoAssembly*
 mono_assembly_load_full (MonoAssemblyName *aname, const char *basedir, MonoImageOpenStatus *status, gboolean refonly)
 {
-	MonoAssembly *result = mono_assembly_load_full_nosearch (aname, basedir, status, refonly);
-	
-	if (!result)
-		/* Try a postload search hook */
-		result = mono_assembly_invoke_search_hook_internal (aname, refonly, TRUE);
-	return result;
+	return mono_assembly_load_full_internal (aname, NULL, basedir, status, refonly);
 }
 
 /**
@@ -2721,9 +3020,9 @@ mono_assembly_load_full (MonoAssemblyName *aname, const char *basedir, MonoImage
 MonoAssembly*
 mono_assembly_load (MonoAssemblyName *aname, const char *basedir, MonoImageOpenStatus *status)
 {
-	return mono_assembly_load_full (aname, basedir, status, FALSE);
+	return mono_assembly_load_full_internal (aname, NULL, basedir, status, FALSE);
 }
-	
+
 MonoAssembly*
 mono_assembly_loaded_full (MonoAssemblyName *aname, gboolean refonly)
 {
@@ -2732,7 +3031,7 @@ mono_assembly_loaded_full (MonoAssemblyName *aname, gboolean refonly)
 
 	aname = mono_assembly_remap_version (aname, &maped_aname);
 
-	res = mono_assembly_invoke_search_hook_internal (aname, refonly, FALSE);
+	res = mono_assembly_invoke_search_hook_internal (aname, NULL, refonly, FALSE);
 
 	return res;
 }
@@ -2750,6 +3049,21 @@ mono_assembly_loaded (MonoAssemblyName *aname)
 	return mono_assembly_loaded_full (aname, FALSE);
 }
 
+void
+mono_assembly_release_gc_roots (MonoAssembly *assembly)
+{
+	if (assembly == NULL || assembly == REFERENCE_MISSING)
+		return;
+
+	if (assembly_is_dynamic (assembly)) {
+		int i;
+		MonoDynamicImage *dynimg = (MonoDynamicImage *)assembly->image;
+		for (i = 0; i < dynimg->image.module_count; ++i)
+			mono_dynamic_image_release_gc_roots ((MonoDynamicImage *)dynimg->image.modules [i]);
+		mono_dynamic_image_release_gc_roots (dynimg);
+	}
+}
+
 /*
  * Returns whether mono_assembly_close_finish() must be called as
  * well.  See comment for mono_image_close_except_pools() for why we
@@ -2804,7 +3118,7 @@ mono_assembly_close_finish (MonoAssembly *assembly)
 	if (assembly->image)
 		mono_image_close_finish (assembly->image);
 
-	if (assembly->dynamic) {
+	if (assembly_is_dynamic (assembly)) {
 		g_free ((char*)assembly->aname.culture);
 	} else {
 		g_free (assembly);
@@ -2859,7 +3173,8 @@ mono_assemblies_cleanup (void)
 {
 	GSList *l;
 
-	DeleteCriticalSection (&assemblies_mutex);
+	mono_mutex_destroy (&assemblies_mutex);
+	mono_mutex_destroy (&assembly_binding_mutex);
 
 	for (l = loaded_assembly_bindings; l; l = l->next) {
 		MonoAssemblyBindingInfo *info = l->data;
@@ -2874,6 +3189,30 @@ mono_assemblies_cleanup (void)
 	free_assembly_preload_hooks ();
 }
 
+/*LOCKING takes the assembly_binding lock*/
+void
+mono_assembly_cleanup_domain_bindings (guint32 domain_id)
+{
+	GSList **iter;
+
+	mono_assembly_binding_lock ();
+	iter = &loaded_assembly_bindings;
+	while (*iter) {
+		GSList *l = *iter;
+		MonoAssemblyBindingInfo *info = l->data;
+
+		if (info->domain_id == domain_id) {
+			*iter = l->next;
+			mono_assembly_binding_info_free (info);
+			g_free (info);
+			g_slist_free_1 (l);
+		} else {
+			iter = &l->next;
+		}
+	}
+	mono_assembly_binding_unlock ();
+}
+
 /*
  * Holds the assembly of the application, for
  * System.Diagnostics.Process::MainModule
