@@ -27,15 +27,17 @@
 #include "marshal.h"
 #include "coree.h"
 #include <mono/io-layer/io-layer.h>
-#include <mono/utils/mono-logger.h>
+#include <mono/utils/mono-logger-internal.h>
 #include <mono/utils/mono-path.h>
 #include <mono/utils/mono-mmap.h>
 #include <mono/utils/mono-io-portability.h>
+#include <mono/utils/atomic.h>
 #include <mono/metadata/class-internals.h>
 #include <mono/metadata/assembly.h>
 #include <mono/metadata/object-internals.h>
 #include <mono/metadata/security-core-clr.h>
 #include <mono/metadata/verify-internals.h>
+#include <mono/metadata/verify.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef HAVE_UNISTD_H
@@ -52,10 +54,61 @@ static GHashTable *loaded_images_refonly_hash;
 
 static gboolean debug_assembly_unload = FALSE;
 
-#define mono_images_lock() if (mutex_inited) EnterCriticalSection (&images_mutex)
-#define mono_images_unlock() if (mutex_inited) LeaveCriticalSection (&images_mutex)
+#define mono_images_lock() if (mutex_inited) mono_mutex_lock (&images_mutex)
+#define mono_images_unlock() if (mutex_inited) mono_mutex_unlock (&images_mutex)
 static gboolean mutex_inited;
-static CRITICAL_SECTION images_mutex;
+static mono_mutex_t images_mutex;
+
+typedef struct ImageUnloadHook ImageUnloadHook;
+struct ImageUnloadHook {
+	MonoImageUnloadFunc func;
+	gpointer user_data;
+};
+
+GSList *image_unload_hooks;
+
+void
+mono_install_image_unload_hook (MonoImageUnloadFunc func, gpointer user_data)
+{
+	ImageUnloadHook *hook;
+	
+	g_return_if_fail (func != NULL);
+
+	hook = g_new0 (ImageUnloadHook, 1);
+	hook->func = func;
+	hook->user_data = user_data;
+	image_unload_hooks = g_slist_prepend (image_unload_hooks, hook);
+}
+
+void
+mono_remove_image_unload_hook (MonoImageUnloadFunc func, gpointer user_data)
+{
+	GSList *l;
+	ImageUnloadHook *hook;
+
+	for (l = image_unload_hooks; l; l = l->next) {
+		hook = l->data;
+
+		if (hook->func == func && hook->user_data == user_data) {
+			g_free (hook);
+			image_unload_hooks = g_slist_delete_link (image_unload_hooks, l);
+			break;
+		}
+	}
+}
+
+static void
+mono_image_invoke_unload_hook (MonoImage *image)
+{
+	GSList *l;
+	ImageUnloadHook *hook;
+
+	for (l = image_unload_hooks; l; l = l->next) {
+		hook = l->data;
+
+		hook->func (image, hook->user_data);
+	}
+}
 
 /* returns offset relative to image->raw_data */
 guint32
@@ -69,7 +122,7 @@ mono_cli_rva_image_map (MonoImage *image, guint32 addr)
 	for (i = 0; i < top; i++){
 		if ((addr >= tables->st_virtual_address) &&
 		    (addr < tables->st_virtual_address + tables->st_raw_data_size)){
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 			if (image->is_module_handle)
 				return addr;
 #endif
@@ -98,7 +151,16 @@ mono_image_rva_map (MonoImage *image, guint32 addr)
 	const int top = iinfo->cli_section_count;
 	MonoSectionTable *tables = iinfo->cli_section_tables;
 	int i;
-	
+
+#ifdef HOST_WIN32
+	if (image->is_module_handle) {
+		if (addr && addr < image->raw_data_len)
+			return image->raw_data + addr;
+		else
+			return NULL;
+	}
+#endif
+
 	for (i = 0; i < top; i++){
 		if ((addr >= tables->st_virtual_address) &&
 		    (addr < tables->st_virtual_address + tables->st_raw_data_size)){
@@ -106,10 +168,6 @@ mono_image_rva_map (MonoImage *image, guint32 addr)
 				if (!mono_image_ensure_section_idx (image, i))
 					return NULL;
 			}
-#ifdef PLATFORM_WIN32
-			if (image->is_module_handle)
-				return image->raw_data + addr;
-#endif
 			return (char*)iinfo->cli_sections [i] +
 				(addr - tables->st_virtual_address);
 		}
@@ -126,7 +184,7 @@ mono_image_rva_map (MonoImage *image, guint32 addr)
 void
 mono_images_init (void)
 {
-	InitializeCriticalSection (&images_mutex);
+	mono_mutex_init_recursive (&images_mutex);
 
 	loaded_images_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	loaded_images_refonly_hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -144,7 +202,14 @@ mono_images_init (void)
 void
 mono_images_cleanup (void)
 {
-	DeleteCriticalSection (&images_mutex);
+	GHashTableIter iter;
+	MonoImage *image;
+
+	mono_mutex_destroy (&images_mutex);
+
+	g_hash_table_iter_init (&iter, loaded_images_hash);
+	while (g_hash_table_iter_next (&iter, NULL, (void**)&image))
+		mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Assembly image '%s' still loaded at shutdown.", image->name);
 
 	g_hash_table_destroy (loaded_images_hash);
 	g_hash_table_destroy (loaded_images_refonly_hash);
@@ -180,7 +245,7 @@ mono_image_ensure_section_idx (MonoImage *image, int section)
 
 	if (sect->st_raw_data_ptr + sect->st_raw_data_size > image->raw_data_len)
 		return FALSE;
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	if (image->is_module_handle)
 		iinfo->cli_sections [section] = image->raw_data + sect->st_virtual_address;
 	else
@@ -469,11 +534,7 @@ mono_image_check_for_module_cctor (MonoImage *image)
 	MonoTableInfo *t, *mt;
 	t = &image->tables [MONO_TABLE_TYPEDEF];
 	mt = &image->tables [MONO_TABLE_METHOD];
-	if (mono_framework_version () == 1) {
-		image->checked_module_cctor = TRUE;
-		return;
-	}
-	if (image->dynamic) {
+	if (image_is_dynamic (image)) {
 		/* FIXME: */
 		image->checked_module_cctor = TRUE;
 		return;
@@ -534,7 +595,7 @@ mono_image_load_module (MonoImage *image, int idx)
 	GList *list_iter, *valid_modules = NULL;
 	MonoImageOpenStatus status;
 
-	if ((image->module_count == 0) || (idx > image->module_count))
+	if ((image->module_count == 0) || (idx > image->module_count || idx <= 0))
 		return NULL;
 	if (image->modules_loaded [idx - 1])
 		return image->modules [idx - 1];
@@ -573,7 +634,7 @@ mono_image_load_module (MonoImage *image, int idx)
 			if (image->modules [idx - 1]) {
 				mono_image_addref (image->modules [idx - 1]);
 				image->modules [idx - 1]->assembly = image->assembly;
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 				if (image->modules [idx - 1]->is_module_handle)
 					mono_image_fixup_vtable (image->modules [idx - 1]);
 #endif
@@ -610,12 +671,15 @@ class_next_value (gpointer value)
 void
 mono_image_init (MonoImage *image)
 {
+	mono_mutex_init_recursive (&image->lock);
+	mono_mutex_init_recursive (&image->szarray_cache_lock);
+
 	image->mempool = mono_mempool_new_size (512);
 	mono_internal_hash_table_init (&image->class_cache,
 				       g_direct_hash,
 				       class_key_extract,
 				       class_next_value);
-	image->field_cache = g_hash_table_new (NULL, NULL);
+	image->field_cache = mono_conc_hashtable_new (&image->lock, NULL, NULL);
 
 	image->typespec_cache = g_hash_table_new (NULL, NULL);
 	image->memberref_signatures = g_hash_table_new (NULL, NULL);
@@ -623,8 +687,6 @@ mono_image_init (MonoImage *image)
 	image->method_signatures = g_hash_table_new (NULL, NULL);
 
 	image->property_hash = mono_property_hash_new ();
-	InitializeCriticalSection (&image->lock);
-	InitializeCriticalSection (&image->szarray_cache_lock);
 }
 
 #if G_BYTE_ORDER != G_LITTLE_ENDIAN
@@ -647,7 +709,7 @@ do_load_header (MonoImage *image, MonoDotNetHeader *header, int offset)
 {
 	MonoDotNetHeader64 header64;
 
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	if (!image->is_module_handle)
 #endif
 	if (offset + sizeof (MonoDotNetHeader32) > image->raw_data_len)
@@ -768,7 +830,7 @@ do_load_header (MonoImage *image, MonoDotNetHeader *header, int offset)
  	SWAPPDE (header->datadir.pe_cli_header);
 	SWAPPDE (header->datadir.pe_reserved);
 
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	if (image->is_module_handle)
 		image->raw_data_len = header->nt.pe_image_size;
 #endif
@@ -787,7 +849,7 @@ mono_image_load_pe_data (MonoImage *image)
 	iinfo = image->image_info;
 	header = &iinfo->cli_header;
 
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	if (!image->is_module_handle)
 #endif
 	if (offset + sizeof (msdos) > image->raw_data_len)
@@ -881,6 +943,7 @@ do_mono_image_load (MonoImage *image, MonoImageOpenStatus *status,
 {
 	MonoCLIImageInfo *iinfo;
 	MonoDotNetHeader *header;
+	GSList *errors = NULL;
 
 	mono_profiler_module_event (image, MONO_PROFILE_START_LOAD);
 
@@ -895,7 +958,7 @@ do_mono_image_load (MonoImage *image, MonoImageOpenStatus *status,
 	if (care_about_pecoff == FALSE)
 		goto done;
 
-	if (!mono_verifier_verify_pe_data (image, NULL))
+	if (!mono_verifier_verify_pe_data (image, &errors))
 		goto invalid_image;
 
 	if (!mono_image_load_pe_data (image))
@@ -905,13 +968,13 @@ do_mono_image_load (MonoImage *image, MonoImageOpenStatus *status,
 		goto done;
 	}
 
-	if (!mono_verifier_verify_cli_data (image, NULL))
+	if (!mono_verifier_verify_cli_data (image, &errors))
 		goto invalid_image;
 
 	if (!mono_image_load_cli_data (image))
 		goto invalid_image;
 
-	if (!mono_verifier_verify_table_data (image, NULL))
+	if (!mono_verifier_verify_table_data (image, &errors))
 		goto invalid_image;
 
 	mono_image_load_names (image);
@@ -926,9 +989,14 @@ done:
 	return image;
 
 invalid_image:
+	if (errors) {
+		MonoVerifyInfo *info = errors->data;
+		g_warning ("Could not load image %s due to %s", image->name, info->message);
+		mono_free_verify_list (errors);
+	}
 	mono_profiler_module_loaded (image, MONO_PROFILE_FAILED);
 	mono_image_close (image);
-		return NULL;
+	return NULL;
 }
 
 static MonoImage *
@@ -959,6 +1027,12 @@ do_mono_image_open (const char *fname, MonoImageOpenStatus *status,
 	image->raw_buffer_used = TRUE;
 	image->raw_data_len = mono_file_map_size (filed);
 	image->raw_data = mono_file_map (image->raw_data_len, MONO_MMAP_READ|MONO_MMAP_PRIVATE, mono_file_map_fd (filed), 0, &image->raw_data_handle);
+#if defined(HAVE_MMAP) && !defined (HOST_WIN32)
+	if (!image->raw_data) {
+		image->fileio_used = TRUE;
+		image->raw_data = mono_file_map_fileio (image->raw_data_len, MONO_MMAP_READ|MONO_MMAP_PRIVATE, mono_file_map_fd (filed), 0, &image->raw_data_handle);
+	}
+#endif
 	if (!image->raw_data) {
 		mono_file_map_close (filed);
 		g_free (image);
@@ -1117,7 +1191,7 @@ mono_image_open_from_data (char *data, guint32 data_len, gboolean need_copy, Mon
 	return mono_image_open_from_data_full (data, data_len, need_copy, status, FALSE);
 }
 
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 /* fname is not duplicated. */
 MonoImage*
 mono_image_open_from_module_handle (HMODULE module_handle, char* fname, gboolean has_entry_point, MonoImageOpenStatus* status)
@@ -1151,7 +1225,7 @@ mono_image_open_full (const char *fname, MonoImageOpenStatus *status, gboolean r
 	
 	g_return_val_if_fail (fname != NULL, NULL);
 	
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	/* Load modules using LoadLibrary. */
 	if (!refonly && coree_module_handle) {
 		HMODULE module_handle;
@@ -1200,8 +1274,12 @@ mono_image_open_full (const char *fname, MonoImageOpenStatus *status, gboolean r
 			if (status) {
 				if (last_error == ERROR_BAD_EXE_FORMAT || last_error == STATUS_INVALID_IMAGE_FORMAT)
 					*status = MONO_IMAGE_IMAGE_INVALID;
-				else
-					*status = MONO_IMAGE_ERROR_ERRNO;
+				else {
+					if (last_error == ERROR_FILE_NOT_FOUND || last_error == ERROR_PATH_NOT_FOUND)
+						errno = ENOENT;
+					else
+						errno = 0;
+				}
 			}
 			return NULL;
 		}
@@ -1299,7 +1377,7 @@ mono_image_open_raw (const char *fname, MonoImageOpenStatus *status)
 void
 mono_image_fixup_vtable (MonoImage *image)
 {
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	MonoCLIImageInfo *iinfo;
 	MonoPEDirEntry *de;
 	MonoVTableFixup *vtfixup;
@@ -1361,12 +1439,6 @@ free_mr_signatures (gpointer key, gpointer val, gpointer user_data)
 }
 */
 
-static void
-free_remoting_wrappers (gpointer key, gpointer val, gpointer user_data)
-{
-	g_free (val);
-}
-
 static void
 free_array_cache_entry (gpointer key, gpointer val, gpointer user_data)
 {
@@ -1404,21 +1476,21 @@ free_hash (GHashTable *hash)
 		g_hash_table_destroy (hash);
 }
 
-/**
- * mono_image_close:
- * @image: The image file we wish to close
- *
- * Closes an image file, deallocates all memory consumed and
- * unmaps all possible sections of the file
+/*
+ * Returns whether mono_image_close_finish() must be called as well.
+ * We must unload images in two steps because clearing the domain in
+ * SGen requires the class metadata to be intact, but we need to free
+ * the mono_g_hash_tables in case a collection occurs during domain
+ * unloading and the roots would trip up the GC.
  */
-void
-mono_image_close (MonoImage *image)
+gboolean
+mono_image_close_except_pools (MonoImage *image)
 {
 	MonoImage *image2;
 	GHashTable *loaded_images;
 	int i;
 
-	g_return_if_fail (image != NULL);
+	g_return_val_if_fail (image != NULL, FALSE);
 
 	/* 
 	 * Atomically decrement the refcount and remove ourselves from the hash tables, so
@@ -1428,7 +1500,7 @@ mono_image_close (MonoImage *image)
 
 	if (InterlockedDecrement (&image->ref_count) > 0) {
 		mono_images_unlock ();
-		return;
+		return FALSE;
 	}
 
 	loaded_images = image->ref_only ? loaded_images_refonly_hash : loaded_images_hash;
@@ -1442,14 +1514,14 @@ mono_image_close (MonoImage *image)
 
 	mono_images_unlock ();
 
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	if (image->is_module_handle && image->has_entry_point) {
 		mono_images_lock ();
 		if (image->ref_count == 0) {
 			/* Image will be closed by _CorDllMain. */
 			FreeLibrary ((HMODULE) image->raw_data);
 			mono_images_unlock ();
-			return;
+			return FALSE;
 		}
 		mono_images_unlock ();
 	}
@@ -1459,6 +1531,8 @@ mono_image_close (MonoImage *image)
 
 	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY, "Unloading image %s [%p].", image->name, image);
 
+	mono_image_invoke_unload_hook (image);
+
 	mono_metadata_clean_for_image (image);
 
 	/*
@@ -1466,20 +1540,21 @@ mono_image_close (MonoImage *image)
 	 * assemblies, so we can't release these references in mono_assembly_close () since the
 	 * MonoImage might outlive its associated MonoAssembly.
 	 */
-	if (image->references && !image->dynamic) {
-		MonoTableInfo *t = &image->tables [MONO_TABLE_ASSEMBLYREF];
-		int i;
-
-		for (i = 0; i < t->rows; i++) {
-			if (image->references [i])
-				mono_assembly_close (image->references [i]);
+	if (image->references && !image_is_dynamic (image)) {
+		for (i = 0; i < image->nreferences; i++) {
+			if (image->references [i] && image->references [i] != REFERENCE_MISSING) {
+				if (!mono_assembly_close_except_image_pools (image->references [i]))
+					image->references [i] = NULL;
+			}
+		}
+	} else {
+		if (image->references) {
+			g_free (image->references);
+			image->references = NULL;
 		}
-
-		g_free (image->references);
-		image->references = NULL;
 	}
 
-#ifdef PLATFORM_WIN32
+#ifdef HOST_WIN32
 	mono_images_lock ();
 	if (image->is_module_handle && !image->has_entry_point)
 		FreeLibrary ((HMODULE) image->raw_data);
@@ -1487,8 +1562,14 @@ mono_image_close (MonoImage *image)
 #endif
 
 	if (image->raw_buffer_used) {
-		if (image->raw_data != NULL)
-			mono_file_unmap (image->raw_data, image->raw_data_handle);
+		if (image->raw_data != NULL) {
+#ifndef HOST_WIN32
+			if (image->fileio_used)
+				mono_file_unmap_fileio (image->raw_data, image->raw_data_handle);
+			else
+#endif
+				mono_file_unmap (image->raw_data, image->raw_data_handle);
+		}
 	}
 	
 	if (image->raw_data_allocated) {
@@ -1522,7 +1603,7 @@ mono_image_close (MonoImage *image)
 	if (image->methodref_cache)
 		g_hash_table_destroy (image->methodref_cache);
 	mono_internal_hash_table_destroy (&image->class_cache);
-	g_hash_table_destroy (image->field_cache);
+	mono_conc_hashtable_destroy (image->field_cache);
 	if (image->array_cache) {
 		g_hash_table_foreach (image->array_cache, free_array_cache_entry, NULL);
 		g_hash_table_destroy (image->array_cache);
@@ -1537,15 +1618,23 @@ mono_image_close (MonoImage *image)
 	}
 
 	free_hash (image->native_wrapper_cache);
+	free_hash (image->native_wrapper_aot_cache);
+	free_hash (image->native_wrapper_check_cache);
+	free_hash (image->native_wrapper_aot_check_cache);
+	free_hash (image->native_func_wrapper_cache);
 	free_hash (image->managed_wrapper_cache);
 	free_hash (image->delegate_begin_invoke_cache);
 	free_hash (image->delegate_end_invoke_cache);
 	free_hash (image->delegate_invoke_cache);
 	free_hash (image->delegate_abstract_invoke_cache);
-	if (image->remoting_invoke_cache)
-		g_hash_table_foreach (image->remoting_invoke_cache, free_remoting_wrappers, NULL);
+	free_hash (image->delegate_bound_static_invoke_cache);
+	free_hash (image->delegate_invoke_generic_cache);
+	free_hash (image->delegate_begin_invoke_generic_cache);
+	free_hash (image->delegate_end_invoke_generic_cache);
+	free_hash (image->synchronized_generic_cache);
 	free_hash (image->remoting_invoke_cache);
 	free_hash (image->runtime_invoke_cache);
+	free_hash (image->runtime_invoke_vtype_cache);
 	free_hash (image->runtime_invoke_direct_cache);
 	free_hash (image->runtime_invoke_vcall_cache);
 	free_hash (image->synchronized_cache);
@@ -1560,24 +1649,31 @@ mono_image_close (MonoImage *image)
 	free_hash (image->castclass_cache);
 	free_hash (image->proxy_isinst_cache);
 	free_hash (image->thunk_invoke_cache);
+	free_hash (image->var_cache_slow);
+	free_hash (image->mvar_cache_slow);
+	free_hash (image->wrapper_param_names);
+	free_hash (image->native_wrapper_aot_cache);
+	free_hash (image->pinvoke_scopes);
+	free_hash (image->pinvoke_scope_filenames);
+	free_hash (image->gsharedvt_types);
 
 	/* The ownership of signatures is not well defined */
-	//g_hash_table_foreach (image->memberref_signatures, free_mr_signatures, NULL);
 	g_hash_table_destroy (image->memberref_signatures);
-	//g_hash_table_foreach (image->helper_signatures, free_mr_signatures, NULL);
 	g_hash_table_destroy (image->helper_signatures);
 	g_hash_table_destroy (image->method_signatures);
 
-	if (image->generic_class_cache)
-		g_hash_table_destroy (image->generic_class_cache);
-
 	if (image->rgctx_template_hash)
 		g_hash_table_destroy (image->rgctx_template_hash);
 
 	if (image->property_hash)
 		mono_property_hash_destroy (image->property_hash);
 
-	g_slist_free (image->reflection_info_unregister_classes);
+	/*
+	reflection_info_unregister_classes is only required by dynamic images, which will not be properly
+	cleared during shutdown as we don't perform regular appdomain unload for the root one.
+	*/
+	g_assert (!image->reflection_info_unregister_classes || mono_runtime_is_shutting_down ());
+	image->reflection_info_unregister_classes = NULL;
 
 	if (image->interface_bitset) {
 		mono_unload_interface_ids (image->interface_bitset);
@@ -1593,23 +1689,57 @@ mono_image_close (MonoImage *image)
 		g_free (image->image_info);
 	}
 
+	for (i = 0; i < image->module_count; ++i) {
+		if (image->modules [i]) {
+			if (!mono_image_close_except_pools (image->modules [i]))
+				image->modules [i] = NULL;
+		}
+	}
+	if (image->modules_loaded)
+		g_free (image->modules_loaded);
+
+	mono_mutex_destroy (&image->szarray_cache_lock);
+	mono_mutex_destroy (&image->lock);
+
+	/*g_print ("destroy image %p (dynamic: %d)\n", image, image->dynamic);*/
+	if (image_is_dynamic (image)) {
+		/* Dynamic images are GC_MALLOCed */
+		g_free ((char*)image->module_name);
+		mono_dynamic_image_free ((MonoDynamicImage*)image);
+	}
+
+	mono_profiler_module_event (image, MONO_PROFILE_END_UNLOAD);
+
+	return TRUE;
+}
+
+void
+mono_image_close_finish (MonoImage *image)
+{
+	int i;
+
+	if (image->references && !image_is_dynamic (image)) {
+		for (i = 0; i < image->nreferences; i++) {
+			if (image->references [i] && image->references [i] != REFERENCE_MISSING)
+				mono_assembly_close_finish (image->references [i]);
+		}
+
+		g_free (image->references);
+		image->references = NULL;
+	}
+
 	for (i = 0; i < image->module_count; ++i) {
 		if (image->modules [i])
-			mono_image_close (image->modules [i]);
+			mono_image_close_finish (image->modules [i]);
 	}
 	if (image->modules)
 		g_free (image->modules);
-	if (image->modules_loaded)
-		g_free (image->modules_loaded);
-	if (image->references)
-		g_free (image->references);
+
+#ifndef DISABLE_PERFCOUNTERS
 	mono_perfcounters->loader_bytes -= mono_mempool_get_allocated (image->mempool);
+#endif
 
-	DeleteCriticalSection (&image->szarray_cache_lock);
-	DeleteCriticalSection (&image->lock);
-
-	/*g_print ("destroy image %p (dynamic: %d)\n", image, image->dynamic);*/
-	if (!image->dynamic) {
+	if (!image_is_dynamic (image)) {
 		if (debug_assembly_unload)
 			mono_mempool_invalidate (image->mempool);
 		else {
@@ -1617,16 +1747,27 @@ mono_image_close (MonoImage *image)
 			g_free (image);
 		}
 	} else {
-		/* Dynamic images are GC_MALLOCed */
-		g_free ((char*)image->module_name);
-		mono_dynamic_image_free ((MonoDynamicImage*)image);
 		if (debug_assembly_unload)
 			mono_mempool_invalidate (image->mempool);
-		else
+		else {
 			mono_mempool_destroy (image->mempool);
+			mono_dynamic_image_free_image ((MonoDynamicImage*)image);
+		}
 	}
+}
 
-	mono_profiler_module_event (image, MONO_PROFILE_END_UNLOAD);
+/**
+ * mono_image_close:
+ * @image: The image file we wish to close
+ *
+ * Closes an image file, deallocates all memory consumed and
+ * unmaps all possible sections of the file
+ */
+void
+mono_image_close (MonoImage *image)
+{
+	if (mono_image_close_except_pools (image))
+		mono_image_close_finish (image);
 }
 
 /** 
@@ -1861,21 +2002,27 @@ mono_image_load_file_for_image (MonoImage *image, int fileidx)
 	if (fileidx < 1 || fileidx > t->rows)
 		return NULL;
 
-	mono_loader_lock ();
+	mono_image_lock (image);
 	if (image->files && image->files [fileidx - 1]) {
-		mono_loader_unlock ();
+		mono_image_unlock (image);
 		return image->files [fileidx - 1];
 	}
 
-	if (!image->files)
-		image->files = g_new0 (MonoImage*, t->rows);
-
 	fname_id = mono_metadata_decode_row_col (t, fileidx - 1, MONO_FILE_NAME);
 	fname = mono_metadata_string_heap (image, fname_id);
 	base_dir = g_path_get_dirname (image->name);
 	name = g_build_filename (base_dir, fname, NULL);
 	res = mono_image_open (name, NULL);
-	if (res) {
+	if (!res)
+		goto done;
+
+	mono_image_lock (image);
+	if (image->files && image->files [fileidx - 1]) {
+		MonoImage *old = res;
+		res = image->files [fileidx - 1];
+		mono_loader_unlock ();
+		mono_image_close (old);
+	} else {
 		int i;
 		/* g_print ("loaded file %s from %s (%p)\n", name, image->name, image->assembly); */
 		res->assembly = image->assembly;
@@ -1884,13 +2031,18 @@ mono_image_load_file_for_image (MonoImage *image, int fileidx)
 				res->modules [i]->assembly = image->assembly;
 		}
 
+		if (!image->files)
+			image->files = g_new0 (MonoImage*, t->rows);
 		image->files [fileidx - 1] = res;
-#ifdef PLATFORM_WIN32
+		mono_loader_unlock ();
+		/* vtable fixup can't happen with the image lock held */
+#ifdef HOST_WIN32
 		if (res->is_module_handle)
 			mono_image_fixup_vtable (res);
 #endif
 	}
-	mono_loader_unlock ();
+
+done:
 	g_free (name);
 	g_free (base_dir);
 	return res;
@@ -1969,7 +2121,7 @@ mono_image_get_public_key (MonoImage *image, guint32 *size)
 	const char *pubkey;
 	guint32 len, tok;
 
-	if (image->dynamic) {
+	if (image_is_dynamic (image)) {
 		if (size)
 			*size = ((MonoDynamicImage*)image)->public_key_len;
 		return (char*)((MonoDynamicImage*)image)->public_key;
@@ -2066,7 +2218,7 @@ mono_image_get_assembly (MonoImage *image)
 gboolean
 mono_image_is_dynamic (MonoImage *image)
 {
-	return image->dynamic;
+	return image_is_dynamic (image);
 }
 
 /**
@@ -2094,7 +2246,9 @@ mono_image_alloc (MonoImage *image, guint size)
 {
 	gpointer res;
 
+#ifndef DISABLE_PERFCOUNTERS
 	mono_perfcounters->loader_bytes += size;
+#endif
 	mono_image_lock (image);
 	res = mono_mempool_alloc (image->mempool, size);
 	mono_image_unlock (image);
@@ -2107,7 +2261,9 @@ mono_image_alloc0 (MonoImage *image, guint size)
 {
 	gpointer res;
 
+#ifndef DISABLE_PERFCOUNTERS
 	mono_perfcounters->loader_bytes += size;
+#endif
 	mono_image_lock (image);
 	res = mono_mempool_alloc0 (image->mempool, size);
 	mono_image_unlock (image);
@@ -2120,7 +2276,9 @@ mono_image_strdup (MonoImage *image, const char *s)
 {
 	char *res;
 
+#ifndef DISABLE_PERFCOUNTERS
 	mono_perfcounters->loader_bytes += strlen (s);
+#endif
 	mono_image_lock (image);
 	res = mono_mempool_strdup (image->mempool, s);
 	mono_image_unlock (image);
@@ -2219,3 +2377,13 @@ mono_image_property_remove (MonoImage *image, gpointer subject)
 	mono_property_hash_remove_object (image->property_hash, subject);
  	mono_image_unlock (image);
 }
+
+void
+mono_image_append_class_to_reflection_info_set (MonoClass *class)
+{
+	MonoImage *image = class->image;
+	g_assert (image_is_dynamic (image));
+	mono_image_lock (image);
+	image->reflection_info_unregister_classes = g_slist_prepend_mempool (image->mempool, image->reflection_info_unregister_classes, class);
+	mono_image_unlock (image);
+}
