@@ -18,6 +18,7 @@
 #include <mono/metadata/opcodes.h>
 #include <mono/metadata/reflection.h>
 #include <mono/metadata/method-builder.h>
+#include <mono/metadata/remoting.h>
 
 #define mono_marshal_find_bitfield_offset(type, elem, byte_offset, bitmask) \
 	do { \
@@ -76,14 +77,131 @@ typedef enum {
 	MARSHAL_ACTION_MANAGED_CONV_RESULT
 } MarshalAction;
 
+/*
+ * This is an extension of the MONO_WRAPPER_ enum to avoid adding more elements to that
+ * enum.
+ */
+typedef enum {
+	WRAPPER_SUBTYPE_NONE,
+	/* Subtypes of MONO_WRAPPER_MANAGED_TO_MANAGED */
+	WRAPPER_SUBTYPE_ELEMENT_ADDR,
+	WRAPPER_SUBTYPE_STRING_CTOR,
+	/* Subtypes of MONO_WRAPPER_STELEMREF */
+	WRAPPER_SUBTYPE_VIRTUAL_STELEMREF,
+	/* Subtypes of MONO_WRAPPER_UNKNOWN */
+	WRAPPER_SUBTYPE_FAST_MONITOR_ENTER,
+	WRAPPER_SUBTYPE_FAST_MONITOR_ENTER_V4,
+	WRAPPER_SUBTYPE_FAST_MONITOR_EXIT,
+	WRAPPER_SUBTYPE_PTR_TO_STRUCTURE,
+	WRAPPER_SUBTYPE_STRUCTURE_TO_PTR,
+	/* Subtypes of MONO_WRAPPER_CASTCLASS */
+	WRAPPER_SUBTYPE_CASTCLASS_WITH_CACHE,
+	WRAPPER_SUBTYPE_ISINST_WITH_CACHE,
+	/* Subtypes of MONO_WRAPPER_RUNTIME_INVOKE */
+	WRAPPER_SUBTYPE_RUNTIME_INVOKE_NORMAL,
+	WRAPPER_SUBTYPE_RUNTIME_INVOKE_DYNAMIC,
+	WRAPPER_SUBTYPE_RUNTIME_INVOKE_DIRECT,
+	WRAPPER_SUBTYPE_RUNTIME_INVOKE_VIRTUAL,
+	/* Subtypes of MONO_WRAPPER_MANAGED_TO_NATIVE */
+	WRAPPER_SUBTYPE_ICALL_WRAPPER,
+	WRAPPER_SUBTYPE_NATIVE_FUNC_AOT,
+	WRAPPER_SUBTYPE_PINVOKE,
+	/* Subtypes of MONO_WRAPPER_UNKNOWN */
+	WRAPPER_SUBTYPE_SYNCHRONIZED_INNER,
+	WRAPPER_SUBTYPE_GSHAREDVT_IN,
+	WRAPPER_SUBTYPE_GSHAREDVT_OUT,
+	WRAPPER_SUBTYPE_ARRAY_ACCESSOR,
+	/* Subtypes of MONO_WRAPPER_MANAGED_TO_MANAGED */
+	WRAPPER_SUBTYPE_GENERIC_ARRAY_HELPER,
+	/* Subtypes of MONO_WRAPPER_DELEGATE_INVOKE */
+	WRAPPER_SUBTYPE_DELEGATE_INVOKE_VIRTUAL,
+	WRAPPER_SUBTYPE_DELEGATE_INVOKE_BOUND
+} WrapperSubtype;
+
+typedef struct {
+	MonoMethod *method;
+	MonoClass *klass;
+} NativeToManagedWrapperInfo;
+
+typedef struct {
+	MonoMethod *method;
+} StringCtorWrapperInfo;
+
+typedef struct {
+	int kind;
+} VirtualStelemrefWrapperInfo;
+
 typedef struct {
 	guint32 rank, elem_size;
 } ElementAddrWrapperInfo;
 
+typedef struct {
+	MonoMethod *method;
+	/* For WRAPPER_SUBTYPE_RUNTIME_INVOKE_NORMAL */
+	MonoMethodSignature *sig;
+} RuntimeInvokeWrapperInfo;
+
+typedef struct {
+	MonoMethod *method;
+} ManagedToNativeWrapperInfo;
+
+typedef struct {
+	MonoMethod *method;
+} SynchronizedInnerWrapperInfo;
+
+typedef struct {
+	MonoMethod *method;
+} GenericArrayHelperWrapperInfo;
+
+typedef struct {
+	gpointer func;
+} ICallWrapperInfo;
+
+typedef struct {
+	MonoMethod *method;
+} ArrayAccessorWrapperInfo;
+
+typedef struct {
+	MonoClass *klass;
+} ProxyWrapperInfo;
+
+/*
+ * This structure contains additional information to uniquely identify a given wrapper
+ * method. It can be retrieved by mono_marshal_get_wrapper_info () for certain types
+ * of wrappers, i.e. ones which do not have a 1-1 association with a method/class.
+ */
+typedef struct {
+	WrapperSubtype subtype;
+	union {
+		/* RUNTIME_INVOKE_... */
+		RuntimeInvokeWrapperInfo runtime_invoke;
+		/* STRING_CTOR */
+		StringCtorWrapperInfo string_ctor;
+		/* ELEMENT_ADDR */
+		ElementAddrWrapperInfo element_addr;
+		/* VIRTUAL_STELEMREF */
+		VirtualStelemrefWrapperInfo virtual_stelemref;
+		/* MONO_WRAPPER_NATIVE_TO_MANAGED */
+		NativeToManagedWrapperInfo native_to_managed;
+		/* MONO_WRAPPER_MANAGED_TO_NATIVE */
+		ManagedToNativeWrapperInfo managed_to_native;
+		/* SYNCHRONIZED_INNER */
+		SynchronizedInnerWrapperInfo synchronized_inner;
+		/* GENERIC_ARRAY_HELPER */
+		GenericArrayHelperWrapperInfo generic_array_helper;
+		/* ICALL_WRAPPER */
+		ICallWrapperInfo icall;
+		/* ARRAY_ACCESSOR */
+		ArrayAccessorWrapperInfo array_accessor;
+		/* PROXY_ISINST etc. */
+		ProxyWrapperInfo proxy;
+	} d;
+} WrapperInfo;
+
 G_BEGIN_DECLS
 
 /*type of the function pointer of methods returned by mono_marshal_get_runtime_invoke*/
-typedef MonoObject *(*RuntimeInvokeFunction) (MonoObject *this, void **params, MonoObject **exc, void* compiled_method);
+typedef MonoObject *(*RuntimeInvokeFunction) (MonoObject *this_obj, void **params, MonoObject **exc, void* compiled_method);
 
 typedef void (*RuntimeInvokeDynamicFunction) (void *args, MonoObject **exc, void* compiled_method);
 
@@ -92,6 +210,9 @@ typedef void (*RuntimeInvokeDynamicFunction) (void *args, MonoObject **exc, void
 void
 mono_marshal_init (void) MONO_INTERNAL;
 
+void
+mono_marshal_init_tls (void) MONO_INTERNAL;
+
 void
 mono_marshal_cleanup (void) MONO_INTERNAL;
 
@@ -169,24 +290,15 @@ mono_type_to_stind (MonoType *type) MONO_INTERNAL;
 MonoMethod *
 mono_marshal_method_from_wrapper (MonoMethod *wrapper) MONO_INTERNAL;
 
+WrapperInfo*
+mono_wrapper_info_create (MonoMethodBuilder *mb, WrapperSubtype subtype) MONO_INTERNAL;
+
 void
 mono_marshal_set_wrapper_info (MonoMethod *method, gpointer data) MONO_INTERNAL;
 
 gpointer
 mono_marshal_get_wrapper_info (MonoMethod *wrapper) MONO_INTERNAL;
 
-MonoMethod *
-mono_marshal_get_remoting_invoke (MonoMethod *method) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_xappdomain_invoke (MonoMethod *method) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_remoting_invoke_for_target (MonoMethod *method, MonoRemotingTarget target_type) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_remoting_invoke_with_check (MonoMethod *method) MONO_INTERNAL;
-
 MonoMethod *
 mono_marshal_get_delegate_begin_invoke (MonoMethod *method) MONO_INTERNAL;
 
@@ -197,7 +309,7 @@ MonoMethod *
 mono_marshal_get_delegate_invoke (MonoMethod *method, MonoDelegate *del) MONO_INTERNAL;
 
 MonoMethod *
-mono_marshal_get_runtime_invoke (MonoMethod *method, gboolean virtual) MONO_INTERNAL;
+mono_marshal_get_runtime_invoke (MonoMethod *method, gboolean is_virtual) MONO_INTERNAL;
 
 MonoMethod*
 mono_marshal_get_runtime_invoke_dynamic (void) MONO_INTERNAL;
@@ -220,61 +332,76 @@ mono_marshal_get_native_wrapper (MonoMethod *method, gboolean check_exceptions,
 MonoMethod *
 mono_marshal_get_native_func_wrapper (MonoImage *image, MonoMethodSignature *sig, MonoMethodPInvoke *piinfo, MonoMarshalSpec **mspecs, gpointer func) MONO_INTERNAL;
 
+MonoMethod*
+mono_marshal_get_native_func_wrapper_aot (MonoClass *klass) MONO_INTERNAL;
+
 MonoMethod *
 mono_marshal_get_struct_to_ptr (MonoClass *klass) MONO_INTERNAL;
 
 MonoMethod *
 mono_marshal_get_ptr_to_struct (MonoClass *klass) MONO_INTERNAL;
 
-MonoMethod *
-mono_marshal_get_stfld_wrapper (MonoType *type) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_ldfld_wrapper (MonoType *type) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_ldflda_wrapper (MonoType *type) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_ldfld_remote_wrapper (MonoClass *klass) MONO_INTERNAL;
-
-MonoMethod *
-mono_marshal_get_stfld_remote_wrapper (MonoClass *klass) MONO_INTERNAL;
-
 MonoMethod *
 mono_marshal_get_synchronized_wrapper (MonoMethod *method) MONO_INTERNAL;
 
+MonoMethod *
+mono_marshal_get_synchronized_inner_wrapper (MonoMethod *method) MONO_INTERNAL;
+
 MonoMethod *
 mono_marshal_get_unbox_wrapper (MonoMethod *method) MONO_INTERNAL;
 
+MonoMethod *
+mono_marshal_get_castclass_with_cache (void) MONO_INTERNAL;
+
+MonoMethod *
+mono_marshal_get_isinst_with_cache (void) MONO_INTERNAL;
+
 MonoMethod *
 mono_marshal_get_isinst (MonoClass *klass) MONO_INTERNAL;
 
 MonoMethod *
 mono_marshal_get_castclass (MonoClass *klass) MONO_INTERNAL;
 
-MonoMethod *
-mono_marshal_get_proxy_cancast (MonoClass *klass) MONO_INTERNAL;
-
 MonoMethod *
 mono_marshal_get_stelemref (void) MONO_INTERNAL;
 
+MonoMethod*
+mono_marshal_get_virtual_stelemref (MonoClass *array_class) MONO_INTERNAL;
+
+MonoMethod**
+mono_marshal_get_virtual_stelemref_wrappers (int *nwrappers) MONO_INTERNAL;
+
 MonoMethod*
 mono_marshal_get_array_address (int rank, int elem_size) MONO_INTERNAL;
 
 MonoMethod *
-mono_marshal_get_generic_array_helper (MonoClass *class, MonoClass *iface,
+mono_marshal_get_array_accessor_wrapper (MonoMethod *method) MONO_INTERNAL;
+
+MonoMethod *
+mono_marshal_get_generic_array_helper (MonoClass *klass, MonoClass *iface,
 				       gchar *name, MonoMethod *method) MONO_INTERNAL;
 
 MonoMethod *
 mono_marshal_get_thunk_invoke_wrapper (MonoMethod *method) MONO_INTERNAL;
 
+MonoMethod*
+mono_marshal_get_gsharedvt_in_wrapper (void) MONO_INTERNAL;
+
+MonoMethod*
+mono_marshal_get_gsharedvt_out_wrapper (void) MONO_INTERNAL;
+
 void
 mono_marshal_free_dynamic_wrappers (MonoMethod *method) MONO_INTERNAL;
 
 void
 mono_marshal_free_inflated_wrappers (MonoMethod *method) MONO_INTERNAL;
 
+void
+mono_marshal_lock_internal (void) MONO_INTERNAL;
+
+void
+mono_marshal_unlock_internal (void) MONO_INTERNAL;
+
 /* marshaling internal calls */
 
 void * 
@@ -289,9 +416,6 @@ mono_marshal_free_array (gpointer *ptr, int size) MONO_INTERNAL;
 gboolean 
 mono_marshal_free_ccw (MonoObject* obj) MONO_INTERNAL;
 
-MonoObject *
-mono_marshal_xdomain_copy_value (MonoObject *val) MONO_INTERNAL;
-
 void
 cominterop_release_all_rcws (void) MONO_INTERNAL; 
 
@@ -303,36 +427,6 @@ void
 ves_icall_System_Runtime_InteropServices_Marshal_copy_from_unmanaged (gpointer src, gint32 start_index,
 								      MonoArray *dest, gint32 length) MONO_INTERNAL;
 
-gpointer
-ves_icall_System_Runtime_InteropServices_Marshal_ReadIntPtr (gpointer ptr, gint32 offset) MONO_INTERNAL;
-
-unsigned char
-ves_icall_System_Runtime_InteropServices_Marshal_ReadByte (gpointer ptr, gint32 offset) MONO_INTERNAL;
-
-gint16
-ves_icall_System_Runtime_InteropServices_Marshal_ReadInt16 (gpointer ptr, gint32 offset) MONO_INTERNAL;
-
-gint32
-ves_icall_System_Runtime_InteropServices_Marshal_ReadInt32 (gpointer ptr, gint32 offset) MONO_INTERNAL;
-
-gint64
-ves_icall_System_Runtime_InteropServices_Marshal_ReadInt64 (gpointer ptr, gint32 offset) MONO_INTERNAL;
-
-void
-ves_icall_System_Runtime_InteropServices_Marshal_WriteByte (gpointer ptr, gint32 offset, unsigned char val) MONO_INTERNAL;
-
-void
-ves_icall_System_Runtime_InteropServices_Marshal_WriteIntPtr (gpointer ptr, gint32 offset, gpointer val) MONO_INTERNAL;
-
-void
-ves_icall_System_Runtime_InteropServices_Marshal_WriteInt16 (gpointer ptr, gint32 offset, gint16 val) MONO_INTERNAL;
-
-void
-ves_icall_System_Runtime_InteropServices_Marshal_WriteInt32 (gpointer ptr, gint32 offset, gint32 val) MONO_INTERNAL;
-
-void
-ves_icall_System_Runtime_InteropServices_Marshal_WriteInt64 (gpointer ptr, gint32 offset, gint64 val) MONO_INTERNAL;
-
 MonoString *
 ves_icall_System_Runtime_InteropServices_Marshal_PtrToStringAnsi (char *ptr) MONO_INTERNAL;
 
@@ -450,16 +544,16 @@ ves_icall_Mono_Interop_ComInteropProxy_AddProxy (gpointer pUnk, MonoComInteropPr
 MonoComInteropProxy*
 ves_icall_Mono_Interop_ComInteropProxy_FindProxy (gpointer pUnk) MONO_INTERNAL;
 
-void
+MONO_API void
 mono_win32_compat_CopyMemory (gpointer dest, gconstpointer source, gsize length);
 
-void
+MONO_API void
 mono_win32_compat_FillMemory (gpointer dest, gsize length, guchar fill);
 
-void
+MONO_API void
 mono_win32_compat_MoveMemory (gpointer dest, gconstpointer source, gsize length);
 
-void
+MONO_API void
 mono_win32_compat_ZeroMemory (gpointer dest, gsize length);
 
 void
@@ -468,10 +562,10 @@ mono_marshal_find_nonzero_bit_offset (guint8 *buf, int len, int *byte_offset, gu
 MonoMethodSignature*
 mono_signature_no_pinvoke (MonoMethod *method) MONO_INTERNAL;
 
-/* Called from cominterop.c */
+/* Called from cominterop.c/remoting.c */
 
 void
-mono_marshal_emit_native_wrapper (MonoImage *image, MonoMethodBuilder *mb, MonoMethodSignature *sig, MonoMethodPInvoke *piinfo, MonoMarshalSpec **mspecs, gpointer func, gboolean aot, gboolean check_exceptions) MONO_INTERNAL;
+mono_marshal_emit_native_wrapper (MonoImage *image, MonoMethodBuilder *mb, MonoMethodSignature *sig, MonoMethodPInvoke *piinfo, MonoMarshalSpec **mspecs, gpointer func, gboolean aot, gboolean check_exceptions, gboolean func_param) MONO_INTERNAL;
 
 void
 mono_marshal_emit_managed_wrapper (MonoMethodBuilder *mb, MonoMethodSignature *invoke_sig, MonoMarshalSpec **mspecs, EmitMarshalContext* m, MonoMethod *method, uint32_t target_handle) MONO_INTERNAL;
@@ -489,6 +583,30 @@ mono_mb_create_and_cache (GHashTable *cache, gpointer key,
 void
 mono_marshal_emit_thread_interrupt_checkpoint (MonoMethodBuilder *mb) MONO_INTERNAL;
 
+void
+mono_marshal_emit_thread_force_interrupt_checkpoint (MonoMethodBuilder *mb) MONO_INTERNAL;
+
+void
+mono_marshal_use_aot_wrappers (gboolean use) MONO_INTERNAL;
+
+MonoObject *
+mono_marshal_xdomain_copy_value (MonoObject *val) MONO_INTERNAL;
+
+int
+mono_mb_emit_save_args (MonoMethodBuilder *mb, MonoMethodSignature *sig, gboolean save_this) MONO_INTERNAL;
+
+void
+mono_mb_emit_restore_result (MonoMethodBuilder *mb, MonoType *return_type) MONO_INTERNAL;
+
+MonoMethod*
+mono_mb_create (MonoMethodBuilder *mb, MonoMethodSignature *sig,
+				int max_stack, WrapperInfo *info) MONO_INTERNAL;
+
+MonoMethod*
+mono_mb_create_and_cache_full (GHashTable *cache, gpointer key,
+							   MonoMethodBuilder *mb, MonoMethodSignature *sig,
+							   int max_stack, WrapperInfo *info, gboolean *out_found) MONO_INTERNAL;
+
 G_END_DECLS
 
 #endif /* __MONO_MARSHAL_H__ */
