@@ -50,7 +50,6 @@
 #endif
 #include <stdio.h>
 #include <string.h>
-#include <signal.h>
 #include <errno.h>
 #include <assert.h>
 #ifdef __MACH__
@@ -71,6 +70,7 @@
 #include "metadata/sgen-archdep.h"
 #include "metadata/sgen-bridge.h"
 #include "metadata/sgen-memory-governor.h"
+#include "metadata/sgen-pinning.h"
 #include "metadata/mono-gc.h"
 #include "metadata/method-builder.h"
 #include "metadata/profiler-private.h"
@@ -207,7 +207,7 @@ dump_alloc_records (void)
 	printf ("------------------------------------DUMP RECORDS----------------------------\n");
 	for (i = 0; i < next_record; ++i) {
 		AllocRecord *rec = alloc_records + i;
-		printf ("obj [%p, %p] size %zd reason %s seq %d tid %zx\n", rec->address, rec_end (rec), rec->size, get_reason_name (rec), rec->seq, (size_t)rec->tid);
+		printf ("obj [%p, %p] size %d reason %s seq %d tid %x\n", rec->address, rec_end (rec), (int)rec->size, get_reason_name (rec), rec->seq, (size_t)rec->tid);
 	}
 }
 
@@ -234,7 +234,7 @@ verify_alloc_records (void)
 			hole_size = rec->address - rec_end (prev);
 			max_hole = MAX (max_hole, hole_size);
 		}
-		printf ("obj [%p, %p] size %zd hole to prev %d reason %s seq %d tid %zx\n", rec->address, rec_end (rec), rec->size, hole_size, get_reason_name (rec), rec->seq, (size_t)rec->tid);
+		printf ("obj [%p, %p] size %d hole to prev %d reason %s seq %d tid %zx\n", rec->address, rec_end (rec), (int)rec->size, hole_size, get_reason_name (rec), rec->seq, (size_t)rec->tid);
 		prev = rec;
 	}
 	printf ("SUMMARY total alloc'd %d holes %d max_hole %d\n", total, holes, max_hole);
@@ -747,12 +747,12 @@ fragment_list_reverse (SgenFragmentAllocator *allocator)
 }
 
 mword
-sgen_build_nursery_fragments (GCMemSection *nursery_section, void **start, size_t num_entries, SgenGrayQueue *unpin_queue)
+sgen_build_nursery_fragments (GCMemSection *nursery_section, SgenGrayQueue *unpin_queue)
 {
 	char *frag_start, *frag_end;
 	size_t frag_size;
-	size_t i = 0;
 	SgenFragment *frags_ranges;
+	void **pin_start, **pin_entry, **pin_end;
 
 #ifdef NALLOC_DEBUG
 	reset_alloc_records ();
@@ -769,26 +769,30 @@ sgen_build_nursery_fragments (GCMemSection *nursery_section, void **start, size_
 	/* clear scan starts */
 	memset (nursery_section->scan_starts, 0, nursery_section->num_scan_start * sizeof (gpointer));
 
-	while (i < num_entries || frags_ranges) {
+	pin_start = pin_entry = sgen_pinning_get_entry (nursery_section->pin_queue_first_entry);
+	pin_end = sgen_pinning_get_entry (nursery_section->pin_queue_last_entry);
+
+	while (pin_entry < pin_end || frags_ranges) {
 		char *addr0, *addr1;
 		size_t size;
 		SgenFragment *last_frag = NULL;
 
 		addr0 = addr1 = sgen_nursery_end;
-		if (i < num_entries)
-			addr0 = start [i];
+		if (pin_entry < pin_end)
+			addr0 = *pin_entry;
 		if (frags_ranges)
 			addr1 = frags_ranges->fragment_start;
 
 		if (addr0 < addr1) {
 			if (unpin_queue)
-				GRAY_OBJECT_ENQUEUE (unpin_queue, addr0);
+				GRAY_OBJECT_ENQUEUE (unpin_queue, addr0, sgen_obj_get_descriptor_safe (addr0));
 			else
 				SGEN_UNPIN_OBJECT (addr0);
+			size = SGEN_ALIGN_UP (sgen_safe_object_get_size ((MonoObject*)addr0));
+			CANARIFY_SIZE (size);
 			sgen_set_nursery_scan_start (addr0);
 			frag_end = addr0;
-			size = SGEN_ALIGN_UP (sgen_safe_object_get_size ((MonoObject*)addr0));
-			++i;
+			++pin_entry;
 		} else {
 			frag_end = addr1;
 			size = frags_ranges->fragment_next - addr1;
@@ -808,7 +812,7 @@ sgen_build_nursery_fragments (GCMemSection *nursery_section, void **start, size_
 
 		frag_size = size;
 #ifdef NALLOC_DEBUG
-		add_alloc_record (start [i], frag_size, PINNING);
+		add_alloc_record (*pin_entry, frag_size, PINNING);
 #endif
 		frag_start = frag_end + frag_size;
 	}
@@ -829,9 +833,10 @@ sgen_build_nursery_fragments (GCMemSection *nursery_section, void **start, size_
 	sgen_minor_collector.build_fragments_finish (&mutator_allocator);
 
 	if (!unmask (mutator_allocator.alloc_head)) {
-		SGEN_LOG (1, "Nursery fully pinned (%zd)", num_entries);
-		for (i = 0; i < num_entries; ++i) {
-			SGEN_LOG (3, "Bastard pinning obj %p (%s), size: %zd", start [i], sgen_safe_name (start [i]), sgen_safe_object_get_size (start [i]));
+		SGEN_LOG (1, "Nursery fully pinned");
+		for (pin_entry = pin_start; pin_entry < pin_end; ++pin_entry) {
+			void *p = *pin_entry;
+			SGEN_LOG (3, "Bastard pinning obj %p (%s), size: %zd", p, sgen_safe_name (p), sgen_safe_object_get_size (p));
 		}
 	}
 	return fragment_total;
@@ -871,7 +876,7 @@ sgen_can_alloc_size (size_t size)
 void*
 sgen_nursery_alloc (size_t size)
 {
-	SGEN_ASSERT (1, size >= sizeof (MonoObject) && size <= SGEN_MAX_SMALL_OBJ_SIZE, "Invalid nursery object size");
+	SGEN_ASSERT (1, size >= sizeof (MonoObject) && size <= (SGEN_MAX_SMALL_OBJ_SIZE + CANARY_SIZE), "Invalid nursery object size");
 
 	SGEN_LOG (4, "Searching nursery for size: %zd", size);
 	size = SGEN_ALIGN_UP (size);
