@@ -5,6 +5,7 @@
  *   Patrik Torstensson (Patrik.Torstesson at gmail.com)
  *
  * (C) 2005 Ximian, Inc.  http://www.ximian.com
+ * Copyright 2011 Xamarin Inc.  http://www.xamarin.com
  */
  #include "mini.h"
 
@@ -32,8 +33,7 @@ mono_bb_is_fall_through (MonoCompile *cfg, MonoBasicBlock *bb)
 MonoInst *
 mono_branch_optimize_exception_target (MonoCompile *cfg, MonoBasicBlock *bb, const char * exname)
 {
-	MonoMethod *method = cfg->method;
-	MonoMethodHeader *header = mono_method_get_header (method);
+	MonoMethodHeader *header = cfg->header;
 	MonoExceptionClause *clause;
 	MonoClass *exclass;
 	int i;
@@ -94,6 +94,9 @@ mono_branch_optimize_exception_target (MonoCompile *cfg, MonoBasicBlock *bb, con
 						return jump;
 					} 
 
+					return NULL;
+				} else {
+					/* Branching to an outer clause could skip inner clauses */
 					return NULL;
 				}
 			} else {
@@ -132,7 +135,7 @@ static const int long_cmov_opcodes [] = {
 	OP_CMOV_LGT_UN
 };
 
-static int
+static G_GNUC_UNUSED int
 br_to_br_un (int opcode)
 {
 	switch (opcode) {
@@ -275,6 +278,7 @@ mono_if_conversion (MonoCompile *cfg)
 #ifdef MONO_ARCH_HAVE_CMOV_OPS
 	MonoBasicBlock *bb;
 	gboolean changed = FALSE;
+	int filter = FILTER_NOP | FILTER_IL_SEQ_POINT;
 
 	if (!(cfg->opt & MONO_OPT_CMOV))
 		return;
@@ -314,14 +318,15 @@ mono_if_conversion (MonoCompile *cfg)
 			int dreg, tmp_reg;
 			CompType comp_type;
 
-			if (bb->last_ins && (bb->last_ins->opcode == OP_BR_REG || bb->last_ins->opcode == OP_BR))
+			branch = mono_bb_last_inst (bb, filter);
+
+			if (!branch || branch->opcode == OP_BR_REG || branch->opcode == OP_BR)
 				continue;
 
 			/* Find the compare instruction */
-			if (!bb->last_ins || !bb->last_ins->prev)
+			compare = mono_inst_prev (branch, filter);
+			if (!compare)
 				continue;
-			branch = bb->last_ins;
-			compare = branch->prev;
 
 			if (!MONO_IS_COND_BRANCH_OP (branch))
 				/* This can happen if a cond branch is optimized away */
@@ -335,22 +340,19 @@ mono_if_conversion (MonoCompile *cfg)
 			 * variable.
 			 */
 			/* FIXME: Get rid of the nops earlier */
-			ins1 = true_bb->code;
-			while (ins1 && ins1->opcode == OP_NOP)
-				ins1 = ins1->next;
-			ins2 = false_bb->code;
-			while (ins2 && ins2->opcode == OP_NOP)
-				ins2 = ins2->next;
+			ins1 = mono_bb_first_inst (true_bb, filter);
+			ins2 = mono_bb_first_inst (false_bb, filter);
+
 			if (!(ins1 && ins2 && ins1->dreg == ins2->dreg && ins1->dreg != -1))
 				continue;
 
 			simple = TRUE;
 			for (tmp = ins1->next; tmp; tmp = tmp->next)
-				if (!((tmp->opcode == OP_NOP) || (tmp->opcode == OP_BR)))
+				if (!((tmp->opcode == OP_NOP) || (tmp->opcode == OP_IL_SEQ_POINT) || (tmp->opcode == OP_BR)))
 					simple = FALSE;
 					
 			for (tmp = ins2->next; tmp; tmp = tmp->next)
-				if (!((tmp->opcode == OP_NOP) || (tmp->opcode == OP_BR)))
+				if (!((tmp->opcode == OP_NOP) || (tmp->opcode == OP_IL_SEQ_POINT) || (tmp->opcode == OP_BR)))
 					simple = FALSE;
 
 			if (!simple)
@@ -378,7 +380,7 @@ mono_if_conversion (MonoCompile *cfg)
 			if (cfg->verbose_level > 2) {
 				printf ("\tBranch -> CMove optimization in BB%d on\n", bb->block_num);
 				printf ("\t\t"); mono_print_ins (compare);
-				printf ("\t\t"); mono_print_ins (compare->next);
+				printf ("\t\t"); mono_print_ins (mono_inst_next (compare, filter));
 				printf ("\t\t"); mono_print_ins (ins1);
 				printf ("\t\t"); mono_print_ins (ins2);
 			}
@@ -486,15 +488,15 @@ mono_if_conversion (MonoCompile *cfg)
 				next_bb = bb2;
 			}
 
-			ins1 = code_bb->code;
+			ins1 = mono_bb_first_inst (code_bb, filter);
 
 			if (!ins1)
 				continue;
 
 			/* Check that code_bb is simple */
 			simple = TRUE;
-			for (tmp = ins1->next; tmp; tmp = tmp->next)
-				if (!((tmp->opcode == OP_NOP) || (tmp->opcode == OP_BR)))
+			for (tmp = ins1; tmp; tmp = tmp->next)
+				if (!((tmp->opcode == OP_NOP) || (tmp->opcode == OP_IL_SEQ_POINT) || (tmp->opcode == OP_BR)))
 					simple = FALSE;
 
 			if (!simple)
@@ -504,15 +506,15 @@ mono_if_conversion (MonoCompile *cfg)
 			if (!MONO_INS_HAS_NO_SIDE_EFFECT (ins1))
 				continue;
 
-			if (bb->last_ins && bb->last_ins->opcode == OP_BR_REG)
+			branch = mono_bb_last_inst (bb, filter);
+
+			if (!branch || branch->opcode == OP_BR_REG)
 				continue;
 
 			/* Find the compare instruction */
-
-			if (!bb->last_ins || !bb->last_ins->prev)
+			compare = mono_inst_prev (branch, filter);
+			if (!compare)
 				continue;
-			branch = bb->last_ins;
-			compare = branch->prev;
 
 			if (!MONO_IS_COND_BRANCH_OP (branch))
 				/* This can happen if a cond branch is optimized away */
@@ -532,10 +534,17 @@ mono_if_conversion (MonoCompile *cfg)
 			if (cfg->ret && ins1->dreg == cfg->ret->dreg)
 				continue;
 
+			if (!(cfg->opt & MONO_OPT_DEADCE))
+				/* 
+				 * It is possible that dreg is never set before, so we can't use
+				 * it as an sreg of the cmov instruction (#582322).
+				 */
+				continue;
+
 			if (cfg->verbose_level > 2) {
 				printf ("\tBranch -> CMove optimization (2) in BB%d on\n", bb->block_num);
 				printf ("\t\t"); mono_print_ins (compare);
-				printf ("\t\t"); mono_print_ins (compare->next);
+				printf ("\t\t"); mono_print_ins (mono_inst_next (compare, filter));
 				printf ("\t\t"); mono_print_ins (ins1);
 			}
 
@@ -618,7 +627,7 @@ mono_if_conversion (MonoCompile *cfg)
 	 */
 	for (bb = cfg->bb_entry; bb; bb = bb->next_bb) {
 		MonoBasicBlock *bb1, *bb2, *true_bb, *false_bb, *next_bb;
-		MonoInst *branch1, *branch2, *compare1, *ins;
+		MonoInst *branch1, *branch2, *compare1, *ins, *next;
 
 		/* Look for the IR code generated from if (<var> < 0 || v > <limit>)
 		 * after branch opts which is:
@@ -644,14 +653,14 @@ mono_if_conversion (MonoCompile *cfg)
 		next_bb = bb2;
 
 		/* Check first branch */
-		branch1 = bb->last_ins;
+		branch1 = mono_bb_last_inst (bb, filter);
 		if (!(branch1 && ((branch1->opcode == OP_IBLT) || (branch1->opcode == OP_LBLT)) && (branch1->inst_false_bb == next_bb)))
 			continue;
 
 		true_bb = branch1->inst_true_bb;
 
 		/* Check second branch */
-		branch2 = next_bb->last_ins;
+		branch2 = mono_bb_last_inst (next_bb, filter);
 		if (!branch2)
 			continue;
 
@@ -664,19 +673,20 @@ mono_if_conversion (MonoCompile *cfg)
 			continue;
 
 		/* Check first compare */
-		compare1 = bb->last_ins->prev;
+		compare1 = mono_inst_prev (mono_bb_last_inst (bb, filter), filter);
 		if (!(compare1 && ((compare1->opcode == OP_ICOMPARE_IMM) || (compare1->opcode == OP_LCOMPARE_IMM)) && compare1->inst_imm == 0))
 			continue;
 
 		/* Check second bblock */
-		ins = next_bb->code;
+		ins = mono_bb_first_inst (next_bb, filter);
 		if (!ins)
 			continue;
-		if (((ins->opcode == OP_ICOMPARE_IMM) || (ins->opcode == OP_LCOMPARE_IMM)) && ins->sreg1 == compare1->sreg1 && ins->next == branch2) {
+		next = mono_inst_next (ins, filter);
+		if (((ins->opcode == OP_ICOMPARE_IMM) || (ins->opcode == OP_LCOMPARE_IMM)) && ins->sreg1 == compare1->sreg1 && next == branch2) {
 			/* The second arg must be positive */
 			if (ins->inst_imm < 0)
 				continue;
-		} else if (((ins->opcode == OP_LDLEN) || (ins->opcode == OP_STRLEN)) && ins->dreg != compare1->sreg1 && ins->next && ins->next->opcode == OP_ICOMPARE && ins->next->sreg1 == compare1->sreg1 && ins->next->sreg2 == ins->dreg && ins->next->next == branch2) {
+		} else if (((ins->opcode == OP_LDLEN) || (ins->opcode == OP_STRLEN)) && ins->dreg != compare1->sreg1 && next && next->opcode == OP_ICOMPARE && next->sreg1 == compare1->sreg1 && next->sreg2 == ins->dreg && mono_inst_next (next, filter) == branch2) {
 			/* Another common case: if (index < 0 || index > arr.Length) */
 		} else {
 			continue;
@@ -685,7 +695,7 @@ mono_if_conversion (MonoCompile *cfg)
 		if (cfg->verbose_level > 2) {
 			printf ("\tSigned->unsigned compare optimization in BB%d on\n", bb->block_num);
 			printf ("\t\t"); mono_print_ins (compare1);
-			printf ("\t\t"); mono_print_ins (compare1->next);
+			printf ("\t\t"); mono_print_ins (mono_inst_next (compare1, filter));
 			printf ("\t\t"); mono_print_ins (ins);
 		}
 
@@ -745,7 +755,8 @@ mono_if_conversion (MonoCompile *cfg)
 		mono_handle_global_vregs (cfg);
 		if (cfg->opt & (MONO_OPT_CONSPROP | MONO_OPT_COPYPROP))
 			mono_local_cprop (cfg);
-		mono_local_deadce (cfg);
+		if (cfg->opt & MONO_OPT_DEADCE)
+			mono_local_deadce (cfg);
 	}
 #endif
 }
@@ -805,6 +816,15 @@ replace_in_block (MonoBasicBlock *bb, MonoBasicBlock *orig, MonoBasicBlock *repl
 static void
 replace_out_block_in_code (MonoBasicBlock *bb, MonoBasicBlock *orig, MonoBasicBlock *repl) {
 	MonoInst *ins;
+
+#if defined(__native_client_codegen__)
+	/* Need to maintain this flag for the new block because */
+	/* we can't jump indirectly to a non-aligned block.     */
+	if (orig->flags & BB_INDIRECT_JUMP_TARGET)
+	{
+		repl->flags |= BB_INDIRECT_JUMP_TARGET;
+	}
+#endif
 	
 	for (ins = bb->code; ins != NULL; ins = ins->next) {
 		switch (ins->opcode) {
@@ -865,6 +885,7 @@ remove_block_if_useless (MonoCompile *cfg, MonoBasicBlock *bb, MonoBasicBlock *p
 	MONO_BB_FOR_EACH_INS (bb, inst) {
 		switch (inst->opcode) {
 		case OP_NOP:
+		case OP_IL_SEQ_POINT:
 			break;
 		case OP_BR:
 			target_bb = inst->inst_target_bb;
@@ -951,6 +972,12 @@ mono_merge_basic_blocks (MonoCompile *cfg, MonoBasicBlock *bb, MonoBasicBlock *b
 	bb->has_array_access |= bbn->has_array_access;
 	bb->extended |= bbn->extended;
 
+	/* Compute prev_bb if possible to avoid the linear search below */
+	prev_bb = NULL;
+	for (i = 0; i < bbn->in_count; ++i)
+		if (bbn->in_bb [0]->next_bb == bbn)
+			prev_bb = bbn->in_bb [0];
+
 	mono_unlink_bblock (cfg, bb, bbn);
 	for (i = 0; i < bbn->out_count; ++i)
 		mono_link_bblock (cfg, bb, bbn->out_bb [i]);
@@ -958,21 +985,27 @@ mono_merge_basic_blocks (MonoCompile *cfg, MonoBasicBlock *bb, MonoBasicBlock *b
 		mono_unlink_bblock (cfg, bbn, bbn->out_bb [0]);
 
 	/* Handle the branch at the end of the bb */
-	for (inst = bb->code; inst != NULL; inst = inst->next) {
-		if (inst->opcode == OP_CALL_HANDLER) {
-			g_assert (inst->inst_target_bb == bbn);
-			NULLIFY_INS (inst);
+	if (bb->has_call_handler) {
+		for (inst = bb->code; inst != NULL; inst = inst->next) {
+			if (inst->opcode == OP_CALL_HANDLER) {
+				g_assert (inst->inst_target_bb == bbn);
+				NULLIFY_INS (inst);
+			}
 		}
-		if (MONO_IS_JUMP_TABLE (inst)) {
-			int i;
-			MonoJumpInfoBBTable *table = MONO_JUMP_TABLE_FROM_INS (inst);
-			for (i = 0; i < table->table_size; i++ ) {
-				/* Might be already NULL from a previous merge */
-				if (table->table [i])
-					g_assert (table->table [i] == bbn);
-				table->table [i] = NULL;
+	}
+	if (bb->has_jump_table) {
+		for (inst = bb->code; inst != NULL; inst = inst->next) {
+			if (MONO_IS_JUMP_TABLE (inst)) {
+				int i;
+				MonoJumpInfoBBTable *table = MONO_JUMP_TABLE_FROM_INS (inst);
+				for (i = 0; i < table->table_size; i++ ) {
+					/* Might be already NULL from a previous merge */
+					if (table->table [i])
+						g_assert (table->table [i] == bbn);
+					table->table [i] = NULL;
+				}
+				/* Can't nullify this as later instructions depend on it */
 			}
-			/* Can't nullify this as later instructions depend on it */
 		}
 	}
 	if (bb->last_ins && MONO_IS_COND_BRANCH_OP (bb->last_ins)) {
@@ -983,6 +1016,9 @@ mono_merge_basic_blocks (MonoCompile *cfg, MonoBasicBlock *bb, MonoBasicBlock *b
 		NULLIFY_INS (bb->last_ins);
 	}
 
+	bb->has_call_handler |= bbn->has_call_handler;
+	bb->has_jump_table |= bbn->has_jump_table;
+
 	if (bb->last_ins) {
 		if (bbn->code) {
 			bb->last_ins->next = bbn->code;
@@ -994,8 +1030,10 @@ mono_merge_basic_blocks (MonoCompile *cfg, MonoBasicBlock *bb, MonoBasicBlock *b
 		bb->last_ins = bbn->last_ins;
 	}
 
-	for (prev_bb = cfg->bb_entry; prev_bb && prev_bb->next_bb != bbn; prev_bb = prev_bb->next_bb)
-		;
+	if (!prev_bb) {
+		for (prev_bb = cfg->bb_entry; prev_bb && prev_bb->next_bb != bbn; prev_bb = prev_bb->next_bb)
+			;
+	}
 	if (prev_bb) {
 		prev_bb->next_bb = bbn->next_bb;
 	} else {
@@ -1205,6 +1243,8 @@ mono_optimize_branches (MonoCompile *cfg)
 	int i, changed = FALSE;
 	MonoBasicBlock *bb, *bbn;
 	guint32 niterations;
+	MonoInst *bbn_first_inst;
+	int filter = FILTER_IL_SEQ_POINT;
 
 	/*
 	 * Some crazy loops could cause the code below to go into an infinite
@@ -1261,8 +1301,8 @@ mono_optimize_branches (MonoCompile *cfg)
 					/* the block are in sequence anyway ... */
 
 					/* branches to the following block can be removed */
-					if (bb->last_ins && bb->last_ins->opcode == OP_BR) {
-						bb->last_ins->opcode = OP_NOP;
+					if (bb->last_ins && bb->last_ins->opcode == OP_BR && !bbn->out_of_line) {
+						NULLIFY_INS (bb->last_ins);
 						changed = TRUE;
 						if (cfg->verbose_level > 2)
 							g_print ("br removal triggered %d -> %d\n", bb->block_num, bbn->block_num);
@@ -1301,17 +1341,18 @@ mono_optimize_branches (MonoCompile *cfg)
 
 				if (bb->last_ins && bb->last_ins->opcode == OP_BR) {
 					bbn = bb->last_ins->inst_target_bb;
-					if (bb->region == bbn->region && bbn->code && bbn->code->opcode == OP_BR &&
-						bbn->code->inst_target_bb != bbn &&
-					    bbn->code->inst_target_bb->region == bb->region) {
+					bbn_first_inst = mono_bb_first_inst (bbn, filter);
+					if (bb->region == bbn->region && bbn_first_inst && bbn_first_inst->opcode == OP_BR &&
+						bbn_first_inst->inst_target_bb != bbn &&
+						bbn_first_inst->inst_target_bb->region == bb->region) {
 						
 						if (cfg->verbose_level > 2)
-							g_print ("branch to branch triggered %d -> %d -> %d\n", bb->block_num, bbn->block_num, bbn->code->inst_target_bb->block_num);
+							g_print ("branch to branch triggered %d -> %d -> %d\n", bb->block_num, bbn->block_num, bbn_first_inst->inst_target_bb->block_num);
 
 						replace_in_block (bbn, bb, NULL);
-						replace_out_block (bb, bbn, bbn->code->inst_target_bb);
-						mono_link_bblock (cfg, bb, bbn->code->inst_target_bb);
-						bb->last_ins->inst_target_bb = bbn->code->inst_target_bb;
+						replace_out_block (bb, bbn, bbn_first_inst->inst_target_bb);
+						mono_link_bblock (cfg, bb, bbn_first_inst->inst_target_bb);
+						bb->last_ins->inst_target_bb = bbn_first_inst->inst_target_bb;
 						changed = TRUE;
 						continue;
 					}
@@ -1347,12 +1388,13 @@ mono_optimize_branches (MonoCompile *cfg)
 						continue;
 					}
 					bbn = bb->last_ins->inst_true_bb;
-					if (bb->region == bbn->region && bbn->code && bbn->code->opcode == OP_BR &&
-					    bbn->code->inst_target_bb->region == bb->region) {
+					bbn_first_inst = mono_bb_first_inst (bbn, filter);
+					if (bb->region == bbn->region && bbn_first_inst && bbn_first_inst->opcode == OP_BR &&
+					    bbn_first_inst->inst_target_bb->region == bb->region) {
 						if (cfg->verbose_level > 2)		
 							g_print ("cbranch1 to branch triggered %d -> (%d) %d (0x%02x)\n", 
-								 bb->block_num, bbn->block_num, bbn->code->inst_target_bb->block_num, 
-								 bbn->code->opcode);
+								 bb->block_num, bbn->block_num, bbn_first_inst->inst_target_bb->block_num, 
+								 bbn_first_inst->opcode);
 
 						/* 
 						 * Unlink, then relink bblocks to avoid various
@@ -1362,7 +1404,7 @@ mono_optimize_branches (MonoCompile *cfg)
 						mono_unlink_bblock (cfg, bb, bb->last_ins->inst_true_bb);
 						mono_unlink_bblock (cfg, bb, bb->last_ins->inst_false_bb);
 
-						bb->last_ins->inst_true_bb = bbn->code->inst_target_bb;
+						bb->last_ins->inst_true_bb = bbn_first_inst->inst_target_bb;
 
 						mono_link_bblock (cfg, bb, bb->last_ins->inst_true_bb);
 						mono_link_bblock (cfg, bb, bb->last_ins->inst_false_bb);
@@ -1372,17 +1414,18 @@ mono_optimize_branches (MonoCompile *cfg)
 					}
 
 					bbn = bb->last_ins->inst_false_bb;
-					if (bbn && bb->region == bbn->region && bbn->code && bbn->code->opcode == OP_BR &&
-					    bbn->code->inst_target_bb->region == bb->region) {
+					bbn_first_inst = mono_bb_first_inst (bbn, filter);
+					if (bbn && bb->region == bbn->region && bbn_first_inst && bbn_first_inst->opcode == OP_BR &&
+						bbn_first_inst->inst_target_bb->region == bb->region) {
 						if (cfg->verbose_level > 2)
 							g_print ("cbranch2 to branch triggered %d -> (%d) %d (0x%02x)\n", 
-								 bb->block_num, bbn->block_num, bbn->code->inst_target_bb->block_num, 
-								 bbn->code->opcode);
+								 bb->block_num, bbn->block_num, bbn_first_inst->inst_target_bb->block_num, 
+								 bbn_first_inst->opcode);
 
 						mono_unlink_bblock (cfg, bb, bb->last_ins->inst_true_bb);
 						mono_unlink_bblock (cfg, bb, bb->last_ins->inst_false_bb);
 
-						bb->last_ins->inst_false_bb = bbn->code->inst_target_bb;
+						bb->last_ins->inst_false_bb = bbn_first_inst->inst_target_bb;
 
 						mono_link_bblock (cfg, bb, bb->last_ins->inst_true_bb);
 						mono_link_bblock (cfg, bb, bb->last_ins->inst_false_bb);
@@ -1410,7 +1453,7 @@ mono_optimize_branches (MonoCompile *cfg)
 				}
 
 				if (bb->last_ins && MONO_IS_COND_BRANCH_NOFP (bb->last_ins)) {
-					if (bb->last_ins->inst_false_bb && bb->last_ins->inst_false_bb->out_of_line && (bb->region == bb->last_ins->inst_false_bb->region)) {
+					if (bb->last_ins->inst_false_bb && bb->last_ins->inst_false_bb->out_of_line && (bb->region == bb->last_ins->inst_false_bb->region) && !cfg->disable_out_of_line_bblocks) {
 						/* Reverse the branch */
 						bb->last_ins->opcode = mono_reverse_branch_op (bb->last_ins->opcode);
 						bbn = bb->last_ins->inst_false_bb;
