@@ -16,17 +16,9 @@
 #include <mono/metadata/appdomain.h>
 #include <mono/metadata/threads-types.h>
 
-#define _IN_THE_MONO_DEBUGGER
 #include <mono/metadata/mono-debug-debugger.h>
-#include "debug-mini.h"
 
-#ifdef HAVE_VALGRIND_H
-#include <valgrind/valgrind.h>
-#endif
-
-#ifdef MONO_DEBUGGER_SUPPORTED
-#include <libgc/include/libgc-mono-debugger.h>
-#endif
+#include <mono/utils/valgrind.h>
 
 typedef struct {
 	guint32 index;
@@ -41,54 +33,6 @@ typedef struct
 	guint32 breakpoint_id;
 } MiniDebugMethodInfo;
 
-typedef struct {
-	MonoObject *last_exception;
-	guint32 stopped_on_exception : 1;
-	guint32 stopped_on_unhandled : 1;
-} MonoDebuggerExceptionState;
-
-struct _MonoDebuggerThreadInfo {
-	guint64 tid;
-	guint64 lmf_addr;
-	guint64 end_stack;
-
-	guint64 extended_notifications;
-
-	/* Next pointer. */
-	MonoDebuggerThreadInfo *next;
-
-	/*
-	 * The stack bounds are only used when reading a core file.
-	 */
-	guint64 stack_start;
-	guint64 signal_stack_start;
-	guint32 stack_size;
-	guint32 signal_stack_size;
-
-	MonoDebuggerExceptionState exception_state;
-
-	/*
-	 * The debugger doesn't access anything beyond this point.
-	 */
-	MonoJitTlsData *jit_tls;
-	MonoThread *thread;
-};
-
-typedef struct {
-	gpointer stack_pointer;
-	MonoObject *exception_obj;
-	guint32 stop;
-	guint32 stop_unhandled;
-} MonoDebuggerExceptionInfo;
-
-typedef enum {
-	MONO_DEBUGGER_EXCEPTION_ACTION_NONE		= 0,
-	MONO_DEBUGGER_EXCEPTION_ACTION_STOP		= 1,
-	MONO_DEBUGGER_EXCEPTION_ACTION_STOP_UNHANDLED	= 2
-} MonoDebuggerExceptionAction;
-
-MonoDebuggerThreadInfo *mono_debugger_thread_table = NULL;
-
 static inline void
 record_line_number (MiniDebugMethodInfo *info, guint32 address, guint32 offset)
 {
@@ -106,7 +50,7 @@ mono_debug_init_method (MonoCompile *cfg, MonoBasicBlock *start_block, guint32 b
 {
 	MiniDebugMethodInfo *info;
 
-	if (mono_debug_format == MONO_DEBUG_FORMAT_NONE)
+	if (!mono_debug_enabled ())
 		return;
 
 	info = g_new0 (MiniDebugMethodInfo, 1);
@@ -128,7 +72,7 @@ mono_debug_open_method (MonoCompile *cfg)
 
 	mono_class_init (cfg->method->klass);
 
-	header = mono_method_get_header (cfg->method);
+	header = cfg->header;
 	g_assert (header);
 	
 	info->jit = jit = g_new0 (MonoDebugMethodJitInfo, 1);
@@ -146,10 +90,24 @@ write_variable (MonoInst *inst, MonoDebugVarInfo *var)
 		var->index = inst->dreg | MONO_DEBUG_VAR_ADDRESS_MODE_REGISTER;
 	else if (inst->flags & MONO_INST_IS_DEAD)
 		var->index = MONO_DEBUG_VAR_ADDRESS_MODE_DEAD;
-	else {
+	else if (inst->opcode == OP_REGOFFSET) {
 		/* the debug interface needs fixing to allow 0(%base) address */
 		var->index = inst->inst_basereg | MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET;
 		var->offset = inst->inst_offset;
+	} else if (inst->opcode == OP_GSHAREDVT_ARG_REGOFFSET) {
+		var->index = inst->inst_basereg | MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET_INDIR;
+		var->offset = inst->inst_offset;
+	} else if (inst->opcode == OP_GSHAREDVT_LOCAL) {
+		var->index = inst->inst_imm | MONO_DEBUG_VAR_ADDRESS_MODE_GSHAREDVT_LOCAL;
+	} else if (inst->opcode == OP_VTARG_ADDR) {
+		MonoInst *vtaddr;
+
+		vtaddr = inst->inst_left;
+		g_assert (vtaddr->opcode == OP_REGOFFSET);
+		var->offset = vtaddr->inst_offset;
+		var->index  = vtaddr->inst_basereg | MONO_DEBUG_VAR_ADDRESS_MODE_VTADDR;
+	} else {
+		g_assert_not_reached ();
 	}
 }
 
@@ -251,6 +209,7 @@ mono_debug_add_vg_method (MonoMethod *method, MonoDebugMethodJitInfo *jit)
 
 	g_free (addresses);
 	g_free (lines);
+	mono_metadata_free_mh (header);
 #endif /* VALGRIND_ADD_LINE_INFO */
 }
 
@@ -273,7 +232,7 @@ mono_debug_close_method (MonoCompile *cfg)
 	}
 
 	method = cfg->method;
-	header = mono_method_get_header (method);
+	header = cfg->header;
 	sig = mono_method_signature (method);
 
 	jit = info->jit;
@@ -298,24 +257,39 @@ mono_debug_close_method (MonoCompile *cfg)
 	for (i = 0; i < jit->num_params; i++)
 		write_variable (cfg->args [i + sig->hasthis], &jit->params [i]);
 
+	if (cfg->gsharedvt_info_var) {
+		jit->gsharedvt_info_var = g_new0 (MonoDebugVarInfo, 1);
+		jit->gsharedvt_locals_var = g_new0 (MonoDebugVarInfo, 1);
+		write_variable (cfg->gsharedvt_info_var, jit->gsharedvt_info_var);
+		write_variable (cfg->gsharedvt_locals_var, jit->gsharedvt_locals_var);
+	}
+
 	jit->num_line_numbers = info->line_numbers->len;
 	jit->line_numbers = g_new0 (MonoDebugLineNumberEntry, jit->num_line_numbers);
 
 	for (i = 0; i < jit->num_line_numbers; i++)
 		jit->line_numbers [i] = g_array_index (info->line_numbers, MonoDebugLineNumberEntry, i);
 
-	debug_info = mono_debug_add_method (method, jit, cfg->domain);
+	debug_info = mono_debug_add_method (cfg->method_to_register, jit, cfg->domain);
 
 	mono_debug_add_vg_method (method, jit);
 
-	if (info->breakpoint_id)
-		mono_debugger_breakpoint_callback (method, info->breakpoint_id);
-
-	mono_debugger_check_breakpoints (method, debug_info);
-
 	mono_debug_free_method_jit_info (jit);
-	g_array_free (info->line_numbers, TRUE);
-	g_free (info);
+	mono_debug_free_method (cfg);
+}
+
+void
+mono_debug_free_method (MonoCompile *cfg)
+{
+	MiniDebugMethodInfo *info;
+
+	info = (MiniDebugMethodInfo *) cfg->debug_info;
+	if (info) {
+		if (info->line_numbers)
+			g_array_free (info->line_numbers, TRUE);
+		g_free (info);
+		cfg->debug_info = NULL;	
+	}
 }
 
 void
@@ -329,7 +303,7 @@ mono_debug_record_line_number (MonoCompile *cfg, MonoInst *ins, guint32 address)
 	if (!info || !info->jit || !ins->cil_code)
 		return;
 
-	header = mono_method_get_header (cfg->method);
+	header = cfg->header;
 	g_assert (header);
 
 	if ((ins->cil_code < header->code) ||
@@ -356,7 +330,7 @@ mono_debug_open_block (MonoCompile *cfg, MonoBasicBlock *bb, guint32 address)
 	if (!info || !info->jit || !bb->cil_code)
 		return;
 
-	header = mono_method_get_header (cfg->method);
+	header = cfg->header;
 	g_assert (header);
 
 	if ((bb->cil_code < header->code) ||
@@ -449,8 +423,11 @@ serialize_variable (MonoDebugVarInfo *var, guint8 *p, guint8 **endbuf)
 	case MONO_DEBUG_VAR_ADDRESS_MODE_REGISTER:
 		break;
 	case MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET:
+	case MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET_INDIR:
+	case MONO_DEBUG_VAR_ADDRESS_MODE_VTADDR:
 		encode_value (var->offset, p, &p);
 		break;
+	case MONO_DEBUG_VAR_ADDRESS_MODE_GSHAREDVT_LOCAL:
 	case MONO_DEBUG_VAR_ADDRESS_MODE_DEAD:
 		break;
 	default:
@@ -489,6 +466,14 @@ mono_debug_serialize_debug_info (MonoCompile *cfg, guint8 **out_buf, guint32 *bu
 	for (i = 0; i < jit->num_locals; i++)
 		serialize_variable (&jit->locals [i], p, &p);
 
+	if (jit->gsharedvt_info_var) {
+		encode_value (1, p, &p);
+		serialize_variable (jit->gsharedvt_info_var, p, &p);
+		serialize_variable (jit->gsharedvt_locals_var, p, &p);
+	} else {
+		encode_value (0, p, &p);
+	}
+
 	encode_value (jit->num_line_numbers, p, &p);
 
 	prev_offset = 0;
@@ -521,8 +506,11 @@ deserialize_variable (MonoDebugVarInfo *var, guint8 *p, guint8 **endbuf)
 	case MONO_DEBUG_VAR_ADDRESS_MODE_REGISTER:
 		break;
 	case MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET:
+	case MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET_INDIR:
+	case MONO_DEBUG_VAR_ADDRESS_MODE_VTADDR:
 		var->offset = decode_value (p, &p);
 		break;
+	case MONO_DEBUG_VAR_ADDRESS_MODE_GSHAREDVT_LOCAL:
 	case MONO_DEBUG_VAR_ADDRESS_MODE_DEAD:
 		break;
 	default:
@@ -566,6 +554,13 @@ deserialize_debug_info (MonoMethod *method, guint8 *code_start, guint8 *buf, gui
 	for (i = 0; i < jit->num_locals; i++)
 		deserialize_variable (&jit->locals [i], p, &p);
 
+	if (decode_value (p, &p)) {
+		jit->gsharedvt_info_var = g_new0 (MonoDebugVarInfo, 1);
+		jit->gsharedvt_locals_var = g_new0 (MonoDebugVarInfo, 1);
+		deserialize_variable (jit->gsharedvt_info_var, p, &p);
+		deserialize_variable (jit->gsharedvt_locals_var, p, &p);
+	}
+
 	jit->num_line_numbers = decode_value (p, &p);
 	jit->line_numbers = g_new0 (MonoDebugLineNumberEntry, jit->num_line_numbers);
 
@@ -584,6 +579,7 @@ deserialize_debug_info (MonoMethod *method, guint8 *code_start, guint8 *buf, gui
 		prev_native_offset = native_offset;
 	}
 
+	mono_metadata_free_mh (header);
 	return jit;
 }
 
@@ -593,7 +589,7 @@ mono_debug_add_aot_method (MonoDomain *domain, MonoMethod *method, guint8 *code_
 {
 	MonoDebugMethodJitInfo *jit;
 
-	if (mono_debug_format == MONO_DEBUG_FORMAT_NONE)
+	if (!mono_debug_enabled ())
 		return;
 
 	if ((method->iflags & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL) ||
@@ -615,15 +611,6 @@ mono_debug_add_aot_method (MonoDomain *domain, MonoMethod *method, guint8 *code_
 	mono_debug_free_method_jit_info (jit);
 }
 
-void
-mono_debug_add_icall_wrapper (MonoMethod *method, MonoJitICallInfo* callinfo)
-{
-	if (mono_debug_format == MONO_DEBUG_FORMAT_NONE)
-		return;
-
-	// mono_debug_add_wrapper (method, callinfo->wrapper, callinfo->func);
-}
-
 static void
 print_var_info (MonoDebugVarInfo *info, int idx, const char *name, const char *type)
 {
@@ -634,6 +621,15 @@ print_var_info (MonoDebugVarInfo *info, int idx, const char *name, const char *t
 	case MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET:
 		g_print ("%s %s (%d) in memory: base register %s + %d\n", type, name, idx, mono_arch_regname (info->index & (~MONO_DEBUG_VAR_ADDRESS_MODE_FLAGS)), info->offset);
 		break;
+	case MONO_DEBUG_VAR_ADDRESS_MODE_REGOFFSET_INDIR:
+		g_print ("%s %s (%d) in indir memory: base register %s + %d\n", type, name, idx, mono_arch_regname (info->index & (~MONO_DEBUG_VAR_ADDRESS_MODE_FLAGS)), info->offset);
+		break;
+	case MONO_DEBUG_VAR_ADDRESS_MODE_GSHAREDVT_LOCAL:
+		g_print ("%s %s (%d) gsharedvt local.\n", type, name, idx);
+		break;
+	case MONO_DEBUG_VAR_ADDRESS_MODE_VTADDR:
+		g_print ("%s %s (%d) vt address: base register %s + %d\n", type, name, idx, mono_arch_regname (info->index & (~MONO_DEBUG_VAR_ADDRESS_MODE_FLAGS)), info->offset);
+		break;
 	case MONO_DEBUG_VAR_ADDRESS_MODE_TWO_REGISTERS:
 	default:
 		g_assert_not_reached ();
@@ -662,14 +658,14 @@ mono_debug_print_vars (gpointer ip, gboolean only_arguments)
 	if (!ji)
 		return;
 
-	jit = mono_debug_find_method (mono_jit_info_get_method (ji), domain);
+	jit = mono_debug_find_method (jinfo_get_method (ji), domain);
 	if (!jit)
 		return;
 
 	if (only_arguments) {
 		char **names;
 		names = g_new (char *, jit->num_params);
-		mono_method_get_param_names (mono_jit_info_get_method (ji), (const char **) names);
+		mono_method_get_param_names (jinfo_get_method (ji), (const char **) names);
 		if (jit->this_var)
 			print_var_info (jit->this_var, 0, "this", "Arg");
 		for (i = 0; i < jit->num_params; ++i) {
@@ -692,9 +688,9 @@ mono_debug_print_vars (gpointer ip, gboolean only_arguments)
  * breakpoint when the method is JITed.
  */
 
-static GPtrArray *breakpoints = NULL;
+static GPtrArray *breakpoints;
 
-int
+static int
 mono_debugger_insert_breakpoint_full (MonoMethodDesc *desc)
 {
 	static int last_breakpoint_id = 0;
@@ -712,29 +708,7 @@ mono_debugger_insert_breakpoint_full (MonoMethodDesc *desc)
 	return info->index;
 }
 
-int
-mono_debugger_remove_breakpoint (int breakpoint_id)
-{
-	int i;
-
-	if (!breakpoints)
-		return 0;
-
-	for (i = 0; i < breakpoints->len; i++) {
-		MiniDebugBreakpointInfo *info = g_ptr_array_index (breakpoints, i);
-
-		if (info->index != breakpoint_id)
-			continue;
-
-		mono_method_desc_free (info->desc);
-		g_ptr_array_remove (breakpoints, info);
-		g_free (info);
-		return 1;
-	}
-
-	return 0;
-}
-
+/*FIXME This is part of the public API by accident, remove it from there when possible. */
 int
 mono_debugger_insert_breakpoint (const gchar *method_name, gboolean include_namespace)
 {
@@ -747,12 +721,13 @@ mono_debugger_insert_breakpoint (const gchar *method_name, gboolean include_name
 	return mono_debugger_insert_breakpoint_full (desc);
 }
 
+/*FIXME This is part of the public API by accident, remove it from there when possible. */
 int
 mono_debugger_method_has_breakpoint (MonoMethod *method)
 {
 	int i;
 
-	if (!breakpoints || (method->wrapper_type != MONO_WRAPPER_NONE))
+	if (!breakpoints)
 		return 0;
 
 	for (i = 0; i < breakpoints->len; i++) {
@@ -766,424 +741,3 @@ mono_debugger_method_has_breakpoint (MonoMethod *method)
 
 	return 0;
 }
-
-void
-mono_debugger_breakpoint_callback (MonoMethod *method, guint32 index)
-{
-	mono_debugger_event (MONO_DEBUGGER_EVENT_JIT_BREAKPOINT, (guint64) (gsize) method, index);
-}
-
-void
-mono_debugger_thread_created (gsize tid, MonoThread *thread, MonoJitTlsData *jit_tls)
-{
-#ifdef MONO_DEBUGGER_SUPPORTED
-	size_t stsize = 0;
-	guint8 *staddr = NULL;
-	MonoDebuggerThreadInfo *info;
-
-	if (mono_debug_format == MONO_DEBUG_FORMAT_NONE)
-		return;
-
-	mono_debugger_lock ();
-
-	mono_thread_get_stack_bounds (&staddr, &stsize);
-
-	info = g_new0 (MonoDebuggerThreadInfo, 1);
-	info->tid = tid;
-	info->thread = thread;
-	info->stack_start = (guint64) (gsize) staddr;
-	info->signal_stack_start = (guint64) (gsize) jit_tls->signal_stack;
-	info->stack_size = stsize;
-	info->signal_stack_size = jit_tls->signal_stack_size;
-	info->end_stack = (guint64) (gsize) GC_mono_debugger_get_stack_ptr ();
-	info->lmf_addr = (guint64) (gsize) mono_get_lmf_addr ();
-	info->jit_tls = jit_tls;
-
-	info->next = mono_debugger_thread_table;
-	mono_debugger_thread_table = info;
-
-	mono_debugger_event (MONO_DEBUGGER_EVENT_THREAD_CREATED,
-			     tid, (guint64) (gsize) info);
-
-	mono_debugger_unlock ();
-#endif /* MONO_DEBUGGER_SUPPORTED */
-}
-
-void
-mono_debugger_thread_cleanup (MonoJitTlsData *jit_tls)
-{
-#ifdef MONO_DEBUGGER_SUPPORTED
-	MonoDebuggerThreadInfo **ptr;
-
-	if (mono_debug_format == MONO_DEBUG_FORMAT_NONE)
-		return;
-
-	mono_debugger_lock ();
-
-	for (ptr = &mono_debugger_thread_table; *ptr; ptr = &(*ptr)->next) {
-		MonoDebuggerThreadInfo *info = *ptr;
-
-		if (info->jit_tls != jit_tls)
-			continue;
-
-		mono_debugger_event (MONO_DEBUGGER_EVENT_THREAD_CLEANUP,
-				     info->tid, (guint64) (gsize) info);
-
-		*ptr = info->next;
-		g_free (info);
-		break;
-	}
-
-	mono_debugger_unlock ();
-#endif
-}
-
-void
-mono_debugger_extended_notification (MonoDebuggerEvent event, guint64 data, guint64 arg)
-{
-#ifdef MONO_DEBUGGER_SUPPORTED
-	MonoDebuggerThreadInfo **ptr;
-	MonoThread *thread = mono_thread_current ();
-
-	if (!mono_debug_using_mono_debugger ())
-		return;
-
-	mono_debugger_lock ();
-
-	for (ptr = &mono_debugger_thread_table; *ptr; ptr = &(*ptr)->next) {
-		MonoDebuggerThreadInfo *info = *ptr;
-
-		if (info->thread != thread)
-			continue;
-
-		if ((info->extended_notifications & (int) event) == 0)
-			continue;
-
-		mono_debugger_event (event, data, arg);
-	}
-
-	mono_debugger_unlock ();
-#endif
-}
-
-void
-mono_debugger_trampoline_compiled (MonoMethod *method, const guint8 *code)
-{
-	mono_debugger_extended_notification (MONO_DEBUGGER_EVENT_TRAMPOLINE,
-					     (guint64) (gsize) method, (guint64) (gsize) code);
-}
-
-#if MONO_DEBUGGER_SUPPORTED
-static MonoDebuggerThreadInfo *
-find_debugger_thread_info (MonoThread *thread)
-{
-	MonoDebuggerThreadInfo **ptr;
-
-	for (ptr = &mono_debugger_thread_table; *ptr; ptr = &(*ptr)->next) {
-		MonoDebuggerThreadInfo *info = *ptr;
-
-		if (info->thread == thread)
-			return info;
-	}
-
-	return NULL;
-}
-#endif
-
-static MonoDebuggerExceptionAction
-_mono_debugger_throw_exception (gpointer addr, gpointer stack, MonoObject *exc)
-{
-#ifdef MONO_DEBUGGER_SUPPORTED
-	MonoDebuggerExceptionInfo exc_info;
-	MonoDebuggerThreadInfo *thread_info;
-
-	if (!mono_debug_using_mono_debugger ())
-		return MONO_DEBUGGER_EXCEPTION_ACTION_NONE;
-
-	mono_debugger_lock ();
-
-	thread_info = find_debugger_thread_info (mono_thread_current ());
-	if (!thread_info) {
-		mono_debugger_unlock ();
-		return MONO_DEBUGGER_EXCEPTION_ACTION_NONE;
-	}
-
-	if (thread_info->exception_state.stopped_on_exception ||
-	    thread_info->exception_state.stopped_on_unhandled) {
-		thread_info->exception_state.stopped_on_exception = 0;
-		mono_debugger_unlock ();
-		return MONO_DEBUGGER_EXCEPTION_ACTION_NONE;
-	}
-
-	/* Protect the exception object from being garbage collected. */
-
-	thread_info->exception_state.stopped_on_unhandled = 0;
-	thread_info->exception_state.stopped_on_exception = 1;
-	thread_info->exception_state.last_exception = exc;
-
-	/*
-	 * Backwards compatibility:
-	 *
-	 * Older debugger versions only know `exc_info.stop' and older runtime versions check
-	 * `exc_info.stop != 0'.
-	 *
-	 * The debugger must check for `mono_debug_debugger_version >= 5' before accessing the
-	 * `stop_unhandled' field.
-	 */
-
-	exc_info.stack_pointer = stack;
-	exc_info.exception_obj = exc;
-	exc_info.stop = 0;
-	exc_info.stop_unhandled = 0;
-
-	mono_debugger_event (MONO_DEBUGGER_EVENT_THROW_EXCEPTION, (guint64) (gsize) &exc_info,
-			     (guint64) (gsize) addr);
-
-	if (!exc_info.stop) {
-		thread_info->exception_state.stopped_on_exception = 0;
-		thread_info->exception_state.last_exception = NULL;
-	} 
-
-	mono_debugger_unlock ();
-
-	if (exc_info.stop)
-		return MONO_DEBUGGER_EXCEPTION_ACTION_STOP;
-	else if (exc_info.stop_unhandled)
-		return MONO_DEBUGGER_EXCEPTION_ACTION_STOP_UNHANDLED;
-#endif
-
-	return MONO_DEBUGGER_EXCEPTION_ACTION_NONE;
-}
-
-static gboolean
-_mono_debugger_unhandled_exception (gpointer addr, gpointer stack, MonoObject *exc)
-{
-#ifdef MONO_DEBUGGER_SUPPORTED
-	MonoDebuggerThreadInfo *thread_info;
-
-	if (!mono_debug_using_mono_debugger ())
-		return FALSE;
-
-	if (exc) {
-		const gchar *name = mono_class_get_name (mono_object_get_class (exc));
-		if (!strcmp (name, "ThreadAbortException"))
-			return FALSE;
-	}
-
-	mono_debugger_lock ();
-
-	thread_info = find_debugger_thread_info (mono_thread_current ());
-	if (!thread_info) {
-		mono_debugger_unlock ();
-		return FALSE;
-	}
-
-	if (thread_info->exception_state.stopped_on_unhandled) {
-		thread_info->exception_state.stopped_on_unhandled = 0;
-		mono_debugger_unlock ();
-		return FALSE;
-	}
-
-	thread_info->exception_state.stopped_on_unhandled = 1;
-	thread_info->exception_state.last_exception = exc;
-
-	mono_debugger_event (MONO_DEBUGGER_EVENT_UNHANDLED_EXCEPTION,
-			     (guint64) (gsize) exc, (guint64) (gsize) addr);
-
-	return TRUE;
-#else
-	return FALSE;
-#endif
-}
-
-/*
- * mono_debugger_call_exception_handler:
- *
- * Called from mono_handle_exception_internal() to tell the debugger that we're about
- * to invoke an exception handler.
- *
- * The debugger may choose to set a breakpoint at @addr.  This is used if the user is
- * single-stepping from a `try' into a `catch' block, for instance.
- */
-
-void
-mono_debugger_call_exception_handler (gpointer addr, gpointer stack, MonoObject *exc)
-{
-#ifdef MONO_DEBUGGER_SUPPORTED
-	MonoDebuggerThreadInfo *thread_info;
-	MonoDebuggerExceptionInfo exc_info;
-
-	if (!mono_debug_using_mono_debugger ())
-		return;
-
-	mono_debugger_lock ();
-
-	thread_info = find_debugger_thread_info (mono_thread_current ());
-	if (!thread_info) {
-		mono_debugger_unlock ();
-		return;
-	}
-
-	// Prevent the object from being finalized.
-	thread_info->exception_state.last_exception = exc;
-
-	exc_info.stack_pointer = stack;
-	exc_info.exception_obj = exc;
-	exc_info.stop = 0;
-	exc_info.stop_unhandled = 0;
-
-	mono_debugger_event (MONO_DEBUGGER_EVENT_HANDLE_EXCEPTION, (guint64) (gsize) &exc_info,
-			     (guint64) (gsize) addr);
-
-	mono_debugger_unlock ();
-#endif
-}
-
-/*
- * mono_debugger_handle_exception:
- *
- *  Notify the debugger about exceptions.  Returns TRUE if the debugger wants us to stop
- *  at the exception and FALSE to resume with the normal exception handling.
- *
- *  The arch code is responsible to setup @ctx in a way that MONO_CONTEXT_GET_IP () and
- *  MONO_CONTEXT_GET_SP () point to the throw instruction; ie. before executing the
- *  `callq throw' instruction.
- */
-gboolean
-mono_debugger_handle_exception (MonoContext *ctx, MonoObject *obj)
-{
-	MonoDebuggerExceptionAction action;
-
-	if (!mono_debug_using_mono_debugger ())
-		return FALSE;
-
-	if (!obj) {
-		MonoException *ex = mono_get_exception_null_reference ();
-		MONO_OBJECT_SETREF (ex, message, mono_string_new (mono_domain_get (), "Object reference not set to an instance of an object"));
-		obj = (MonoObject *)ex;
-	} 
-
-	action = _mono_debugger_throw_exception (MONO_CONTEXT_GET_IP (ctx), MONO_CONTEXT_GET_SP (ctx), obj);
-
-	if (action == MONO_DEBUGGER_EXCEPTION_ACTION_STOP) {
-		/*
-		 * The debugger wants us to stop on the `throw' instruction.
-		 * By the time we get here, it already inserted a breakpoint there.
-		 */
-		return TRUE;
-	} else if (action == MONO_DEBUGGER_EXCEPTION_ACTION_STOP_UNHANDLED) {
-		MonoContext ctx_cp = *ctx;
-
-		/*
-		 * The debugger wants us to stop only if this exception is user-unhandled.
-		 */
-
-		if (!mono_handle_exception (&ctx_cp, obj, MONO_CONTEXT_GET_IP (ctx), TRUE)) {
-			/*
-			 * The exception is user-unhandled - tell the debugger to stop.
-			 */
-			return _mono_debugger_unhandled_exception (MONO_CONTEXT_GET_IP (ctx), MONO_CONTEXT_GET_SP (ctx), obj);
-		}
-
-		/*
-		 * The exception is catched somewhere - resume with the normal exception handling and don't
-		 * stop in the debugger.
-		 */
-	}
-
-	return FALSE;
-}
-
-#ifdef MONO_DEBUGGER_SUPPORTED
-
-static gchar *
-get_exception_message (MonoObject *exc)
-{
-	char *message = NULL;
-	MonoString *str; 
-	MonoMethod *method;
-	MonoClass *klass;
-	gint i;
-
-	if (mono_object_isinst (exc, mono_defaults.exception_class)) {
-		klass = exc->vtable->klass;
-		method = NULL;
-		while (klass && method == NULL) {
-			for (i = 0; i < klass->method.count; ++i) {
-				method = klass->methods [i];
-				if (!strcmp ("ToString", method->name) &&
-				    mono_method_signature (method)->param_count == 0 &&
-				    method->flags & METHOD_ATTRIBUTE_VIRTUAL &&
-				    method->flags & METHOD_ATTRIBUTE_PUBLIC) {
-					break;
-				}
-				method = NULL;
-			}
-			
-			if (method == NULL)
-				klass = klass->parent;
-		}
-
-		g_assert (method);
-
-		str = (MonoString *) mono_runtime_invoke (method, exc, NULL, NULL);
-		if (str)
-			message = mono_string_to_utf8 (str);
-	}
-
-	return message;
-}
-
-MonoObject *
-mono_debugger_runtime_invoke (MonoMethod *method, void *obj, void **params, MonoObject **exc)
-{
-	MonoDebuggerThreadInfo *thread_info;
-	MonoDebuggerExceptionState saved_exception_state;
-	MonoObject *retval;
-	gchar *message;
-
-	mono_debugger_lock ();
-
-	thread_info = find_debugger_thread_info (mono_thread_current ());
-	if (!thread_info) {
-		mono_debugger_unlock ();
-		return NULL;
-	}
-
-	saved_exception_state = thread_info->exception_state;
-
-	thread_info->exception_state.last_exception = NULL;
-	thread_info->exception_state.stopped_on_unhandled = 0;
-	thread_info->exception_state.stopped_on_exception = 0;
-
-	mono_debugger_unlock ();
-
-	if (!strcmp (method->name, ".ctor")) {
-		retval = obj = mono_object_new (mono_domain_get (), method->klass);
-
-		mono_runtime_invoke (method, obj, params, exc);
-	} else
-		retval = mono_runtime_invoke (method, obj, params, exc);
-
-	mono_debugger_lock ();
-
-	thread_info = find_debugger_thread_info (mono_thread_current ());
-	if (thread_info)
-		thread_info->exception_state = saved_exception_state;
-
-	mono_debugger_unlock ();
-
-	if (!exc || (*exc == NULL))
-		return retval;
-
-	retval = *exc;
-	message = get_exception_message (*exc);
-	if (message) {
-		*exc = (MonoObject *) mono_string_new_wrapper (message);
-		g_free (message);
-	}
-
-	return retval;
-}
-
-#endif
