@@ -20,13 +20,20 @@
 #if defined (HAVE_SYS_ZLIB)
 #include <zlib.h>
 #endif
+#include <glib.h>
 #include <mono/metadata/profiler.h>
 #include <mono/metadata/object.h>
 #include <mono/metadata/debug-helpers.h>
+#include <mono/utils/mono-counters.h>
 
 #define HASH_SIZE 9371
 #define SMALL_HASH_SIZE 31
 
+#if defined(__native_client__) || defined(__native_client_codegen__)
+volatile int __nacl_thread_suspension_needed = 0;
+void __nacl_suspend_thread_if_needed() {}
+#endif
+
 static int debug = 0;
 static int collect_traces = 0;
 static int show_traces = 0;
@@ -39,6 +46,7 @@ static uint64_t find_size = 0;
 static const char* find_name = NULL;
 static uint64_t time_from = 0;
 static uint64_t time_to = 0xffffffffffffffffULL;
+static int use_time_filter = 0;
 static uint64_t startup_time = 0;
 static FILE* outfile = NULL;
 
@@ -83,6 +91,418 @@ pstrdup (const char *s)
 	return p;
 }
 
+typedef struct _CounterValue CounterValue;
+struct _CounterValue {
+	uint64_t timestamp;
+	unsigned char *buffer;
+	CounterValue *next;
+};
+
+typedef struct _Counter Counter;
+struct _Counter {
+	int index;
+	const char *section;
+	const char *name;
+	int type;
+	int unit;
+	int variance;
+	CounterValue *values;
+	CounterValue *values_last;
+};
+
+typedef struct _CounterList CounterList;
+struct _CounterList {
+	Counter *counter;
+	CounterList *next;
+};
+
+typedef struct _CounterSection CounterSection;
+struct _CounterSection {
+	const char *value;
+	CounterList *counters;
+	CounterList *counters_last;
+	CounterSection *next;
+};
+
+typedef struct _CounterTimestamp CounterTimestamp;
+struct _CounterTimestamp {
+	uint64_t value;
+	CounterSection *sections;
+	CounterSection *sections_last;
+	CounterTimestamp *next;
+};
+
+static CounterList *counters = NULL;
+static CounterSection *counters_sections = NULL;
+static CounterTimestamp *counters_timestamps = NULL;
+
+enum {
+	COUNTERS_SORT_TIME,
+	COUNTERS_SORT_CATEGORY
+};
+
+static int counters_sort_mode = COUNTERS_SORT_TIME;
+
+static void
+add_counter_to_section (Counter *counter)
+{
+	CounterSection *csection, *s;
+	CounterList *clist;
+
+	clist = calloc (1, sizeof (CounterList));
+	clist->counter = counter;
+
+	for (csection = counters_sections; csection; csection = csection->next) {
+		if (strcmp (csection->value, counter->section) == 0) {
+			/* If section exist */
+			if (!csection->counters)
+				csection->counters = clist;
+			else
+				csection->counters_last->next = clist;
+			csection->counters_last = clist;
+			return;
+		}
+	}
+
+	/* If section does not exist */
+	csection = calloc (1, sizeof (CounterSection));
+	csection->value = counter->section;
+	csection->counters = clist;
+	csection->counters_last = clist;
+
+	if (!counters_sections) {
+		counters_sections = csection;
+	} else {
+		s = counters_sections;
+		while (s->next)
+			s = s->next;
+		s->next = csection;
+	}
+}
+
+static void
+add_counter (const char *section, const char *name, int type, int unit, int variance, int index)
+{
+	CounterList *list, *l;
+	Counter *counter;
+
+	for (list = counters; list; list = list->next)
+		if (list->counter->index == index)
+			return;
+
+	counter = calloc (1, sizeof (Counter));
+	counter->section = section;
+	counter->name = name;
+	counter->type = type;
+	counter->unit = unit;
+	counter->variance = variance;
+	counter->index = index;
+
+	list = calloc (1, sizeof (CounterList));
+	list->counter = counter;
+
+	if (!counters) {
+		counters = list;
+	} else {
+		l = counters;
+		while (l->next)
+			l = l->next;
+		l->next = list;
+	}
+
+	if (counters_sort_mode == COUNTERS_SORT_CATEGORY || !verbose)
+		add_counter_to_section (counter);
+}
+
+static void
+add_counter_to_timestamp (uint64_t timestamp, Counter *counter)
+{
+	CounterTimestamp *ctimestamp, *t;
+	CounterSection *csection;
+	CounterList *clist;
+
+	clist = calloc (1, sizeof (CounterList));
+	clist->counter = counter;
+
+	for (ctimestamp = counters_timestamps; ctimestamp; ctimestamp = ctimestamp->next) {
+		if (ctimestamp->value == timestamp) {
+			for (csection = ctimestamp->sections; csection; csection = csection->next) {
+				if (strcmp (csection->value, counter->section) == 0) {
+					/* if timestamp exist and section exist */
+					if (!csection->counters)
+						csection->counters = clist;
+					else
+						csection->counters_last->next = clist;
+					csection->counters_last = clist;
+					return;
+				}
+			}
+
+			/* if timestamp exist and section does not exist */
+			csection = calloc (1, sizeof (CounterSection));
+			csection->value = counter->section;
+			csection->counters = clist;
+			csection->counters_last = clist;
+
+			if (!ctimestamp->sections)
+				ctimestamp->sections = csection;
+			else
+				ctimestamp->sections_last->next = csection;
+			ctimestamp->sections_last = csection;
+			return;
+		}
+	}
+
+	/* If timestamp do not exist and section does not exist */
+	csection = calloc (1, sizeof (CounterSection));
+	csection->value = counter->section;
+	csection->counters = clist;
+	csection->counters_last = clist;
+
+	ctimestamp = calloc (1, sizeof (CounterTimestamp));
+	ctimestamp->value = timestamp;
+	ctimestamp->sections = csection;
+	ctimestamp->sections_last = csection;
+
+	if (!counters_timestamps) {
+		counters_timestamps = ctimestamp;
+	} else {
+		t = counters_timestamps;
+		while (t->next)
+			t = t->next;
+		t->next = ctimestamp;
+	}
+}
+
+static void
+add_counter_value (int index, CounterValue *value)
+{
+	CounterList *list;
+
+	for (list = counters; list; list = list->next) {
+		if (list->counter->index == index) {
+			if (!list->counter->values)
+				list->counter->values = value;
+			else
+				list->counter->values_last->next = value;
+			list->counter->values_last = value;
+
+			if (counters_sort_mode == COUNTERS_SORT_TIME)
+				add_counter_to_timestamp (value->timestamp, list->counter);
+
+			return;
+		}
+	}
+}
+
+static const char*
+section_name (int section)
+{
+	switch (section) {
+	case MONO_COUNTER_JIT: return "Mono JIT";
+	case MONO_COUNTER_GC: return "Mono GC";
+	case MONO_COUNTER_METADATA: return "Mono Metadata";
+	case MONO_COUNTER_GENERICS: return "Mono Generics";
+	case MONO_COUNTER_SECURITY: return "Mono Security";
+	case MONO_COUNTER_RUNTIME: return "Mono Runtime";
+	case MONO_COUNTER_SYSTEM: return "Mono System";
+	default: return "<unknown>";
+	}
+}
+
+static const char*
+type_name (int type)
+{
+	switch (type) {
+	case MONO_COUNTER_INT: return "Int";
+	case MONO_COUNTER_UINT: return "UInt";
+	case MONO_COUNTER_WORD: return "Word";
+	case MONO_COUNTER_LONG: return "Long";
+	case MONO_COUNTER_ULONG: return "ULong";
+	case MONO_COUNTER_DOUBLE: return "Double";
+	case MONO_COUNTER_STRING: return "String";
+	case MONO_COUNTER_TIME_INTERVAL: return "Time Interval";
+	default: return "<unknown>";
+	}
+}
+
+static const char*
+unit_name (int unit)
+{
+	switch (unit) {
+	case MONO_COUNTER_RAW: return "Raw";
+	case MONO_COUNTER_BYTES: return "Bytes";
+	case MONO_COUNTER_TIME: return "Time";
+	case MONO_COUNTER_COUNT: return "Count";
+	case MONO_COUNTER_PERCENTAGE: return "Percentage";
+	default: return "<unknown>";
+	}
+}
+
+static const char*
+variance_name (int variance)
+{
+	switch (variance) {
+	case MONO_COUNTER_MONOTONIC: return "Monotonic";
+	case MONO_COUNTER_CONSTANT: return "Constant";
+	case MONO_COUNTER_VARIABLE: return "Variable";
+	default: return "<unknown>";
+	}
+}
+
+static void
+dump_counters_value (Counter *counter, const char *key_format, const char *key, void *value)
+{
+	char format[32];
+
+	if (value == NULL) {
+		snprintf (format, sizeof (format), "%s : %%s\n", key_format);
+		fprintf (outfile, format, key, "<null>");
+	} else {
+		switch (counter->type) {
+		case MONO_COUNTER_INT:
+#if SIZEOF_VOID_P == 4
+		case MONO_COUNTER_WORD:
+#endif
+			snprintf (format, sizeof (format), "%s : %%d\n", key_format);
+			fprintf (outfile, format, key, *(int32_t*)value);
+			break;
+		case MONO_COUNTER_UINT:
+			snprintf (format, sizeof (format), "%s : %%u\n", key_format);
+			fprintf (outfile, format, key, *(uint32_t*)value);
+			break;
+		case MONO_COUNTER_LONG:
+#if SIZEOF_VOID_P == 8
+		case MONO_COUNTER_WORD:
+#endif
+		case MONO_COUNTER_TIME_INTERVAL:
+			if (counter->type == MONO_COUNTER_LONG && counter->unit == MONO_COUNTER_TIME) {
+				snprintf (format, sizeof (format), "%s : %%0.3fms\n", key_format);
+				fprintf (outfile, format, key, (double)*(int64_t*)value / 10000.0);
+			} else if (counter->type == MONO_COUNTER_TIME_INTERVAL) {
+				snprintf (format, sizeof (format), "%s : %%0.3fms\n", key_format);
+				fprintf (outfile, format, key, (double)*(int64_t*)value / 1000.0);
+			} else {
+				snprintf (format, sizeof (format), "%s : %%u\n", key_format);
+				fprintf (outfile, format, key, *(int64_t*)value);
+			}
+			break;
+		case MONO_COUNTER_ULONG:
+			snprintf (format, sizeof (format), "%s : %%llu\n", key_format);
+			fprintf (outfile, format, key, *(uint64_t*)value);
+			break;
+		case MONO_COUNTER_DOUBLE:
+			snprintf (format, sizeof (format), "%s : %%f\n", key_format);
+			fprintf (outfile, format, key, *(double*)value);
+			break;
+		case MONO_COUNTER_STRING:
+			snprintf (format, sizeof (format), "%s : %%s\n", key_format);
+			fprintf (outfile, format, key, *(char*)value);
+			break;
+		}
+	}
+}
+
+static void
+dump_counters (void)
+{
+	Counter *counter;
+	CounterValue *cvalue;
+	CounterTimestamp *ctimestamp;
+	CounterSection *csection;
+	CounterList *clist;
+	char strtimestamp[17];
+	int i, section_printed;
+
+	fprintf (outfile, "\nCounters:\n");
+
+	if (!verbose) {
+		char counters_to_print[][64] = {
+			"Methods from AOT",
+			"Methods JITted using mono JIT",
+			"Methods JITted using LLVM",
+			"Total time spent JITting (sec)",
+			"User Time",
+			"System Time",
+			"Total Time",
+			"Working Set",
+			"Private Bytes",
+			"Virtual Bytes",
+			"Page Faults",
+			"CPU Load Average - 1min",
+			"CPU Load Average - 5min",
+			"CPU Load Average - 15min",
+			""
+		};
+
+		for (csection = counters_sections; csection; csection = csection->next) {
+			section_printed = 0;
+
+			for (clist = csection->counters; clist; clist = clist->next) {
+				counter = clist->counter;
+				if (!counter->values_last)
+					continue;
+
+				for (i = 0; counters_to_print [i][0] != 0; i++) {
+					if (strcmp (counters_to_print [i], counter->name) == 0) {
+						if (!section_printed) {
+							fprintf (outfile, "\t%s:\n", csection->value);
+							section_printed = 1;
+						}
+
+						dump_counters_value (counter, "\t\t%-30s", counter->name, counter->values_last->buffer);
+						break;
+					}
+				}
+			}
+		}
+	} else if (counters_sort_mode == COUNTERS_SORT_TIME) {
+		for (ctimestamp = counters_timestamps; ctimestamp; ctimestamp = ctimestamp->next) {
+			fprintf (outfile, "\t%llu:%02llu:%02llu:%02llu.%03llu:\n",
+				(unsigned long long) (ctimestamp->value / 1000 / 60 / 60 / 24 % 1000),
+				(unsigned long long) (ctimestamp->value / 1000 / 60 / 60 % 24),
+				(unsigned long long) (ctimestamp->value / 1000 / 60 % 60),
+				(unsigned long long) (ctimestamp->value / 1000 % 60),
+				(unsigned long long) (ctimestamp->value % 1000));
+
+			for (csection = ctimestamp->sections; csection; csection = csection->next) {
+				fprintf (outfile, "\t\t%s:\n", csection->value);
+
+				for (clist = csection->counters; clist; clist = clist->next) {
+					counter = clist->counter;
+					for (cvalue = counter->values; cvalue; cvalue = cvalue->next) {
+						if (cvalue->timestamp != ctimestamp->value)
+							continue;
+
+						dump_counters_value (counter, "\t\t\t%-30s", counter->name, cvalue->buffer);
+					}
+				}
+			}
+		}
+	} else if (counters_sort_mode == COUNTERS_SORT_CATEGORY) {
+		for (csection = counters_sections; csection; csection = csection->next) {
+			fprintf (outfile, "\t%s:\n", csection->value);
+
+			for (clist = csection->counters; clist; clist = clist->next) {
+				counter = clist->counter;
+				fprintf (outfile, "\t\t%s: [type: %s, unit: %s, variance: %s]\n",
+					counter->name, type_name (counter->type), unit_name (counter->unit), variance_name (counter->variance));
+
+				for (cvalue = counter->values; cvalue; cvalue = cvalue->next) {
+					snprintf (strtimestamp, sizeof (strtimestamp), "%llu:%02llu:%02llu:%02llu.%03llu",
+						(unsigned long long) (cvalue->timestamp / 1000 / 60 / 60 / 24 % 1000),
+						(unsigned long long) (cvalue->timestamp / 1000 / 60 / 60 % 24),
+						(unsigned long long) (cvalue->timestamp / 1000 / 60 % 60),
+						(unsigned long long) (cvalue->timestamp / 1000 % 60),
+						(unsigned long long) (cvalue->timestamp % 1000));
+
+					dump_counters_value (counter, "\t\t\t%s", strtimestamp, cvalue->buffer);
+				}
+			}
+		}
+	}
+}
+
 static int num_images;
 typedef struct _ImageDesc ImageDesc;
 struct _ImageDesc {
@@ -183,6 +603,7 @@ struct _MethodDesc {
 	int len;
 	int recurse_count;
 	int sample_hits;
+	int ignore_jit; /* when this is set, we collect the metadata but don't count this method fot jit time and code size, when filtering events */
 	uint64_t calls;
 	uint64_t total_time;
 	uint64_t callee_time;
@@ -438,9 +859,9 @@ static void
 print_usym (UnmanagedSymbol* um)
 {
 	if (um->parent)
-		fprintf (outfile, "\t%6d %6.2f %-36s in %s\n", um->sample_hits, um->sample_hits*100.0/num_stat_samples, um->name, um->parent->name);
+		fprintf (outfile, "\t%6zd %6.2f %-36s in %s\n", um->sample_hits, um->sample_hits*100.0/num_stat_samples, um->name, um->parent->name);
 	else
-		fprintf (outfile, "\t%6d %6.2f %s\n", um->sample_hits, um->sample_hits*100.0/num_stat_samples, um->name);
+		fprintf (outfile, "\t%6zd %6.2f %s\n", um->sample_hits, um->sample_hits*100.0/num_stat_samples, um->name);
 }
 
 static int
@@ -919,7 +1340,7 @@ heap_shot_mark_objects (HeapShot *hs)
 			fprintf (outfile, "object %p (%s) unmarked\n", (void*)hs->objects_hash [i], hs->objects_hash [i]->hklass->klass->name);
 		}
 	}
-	fprintf (outfile, "Total unmarked: %d/%d\n", num_unmarked, hs->objects_count);
+	fprintf (outfile, "Total unmarked: %zd/%zd\n", num_unmarked, hs->objects_count);
 	free (marks);
 }
 
@@ -1012,7 +1433,7 @@ typedef struct _ThreadContext ThreadContext;
 typedef struct {
 	FILE *file;
 #if defined (HAVE_SYS_ZLIB)
-	gzFile *gzfile;
+	gzFile gzfile;
 #endif
 	unsigned char *buf;
 	int size;
@@ -1287,6 +1708,7 @@ static int gc_resizes;
 typedef struct {
 	uint64_t created;
 	uint64_t destroyed;
+	uint64_t live;
 	uint64_t max_live;
 	TraceDesc traces;
 } HandleInfo;
@@ -1425,7 +1847,7 @@ tracked_creation (uintptr_t obj, ClassDesc *cd, uint64_t size, BackTrace *bt, ui
 	for (i = 0; i < num_tracked_objects; ++i) {
 		if (tracked_objects [i] != obj)
 			continue;
-		fprintf (outfile, "Object %p created (%s, %llu bytes) at %.3f secs.\n", (void*)obj, cd->name, size, (timestamp - startup_time)/1000000000.0);
+		fprintf (outfile, "Object %p created (%s, %llu bytes) at %.3f secs.\n", (void*)obj, cd->name, (unsigned long long) size, (timestamp - startup_time)/1000000000.0);
 		if (bt && bt->count) {
 			int k;
 			for (k = 0; k < bt->count; ++k)
@@ -1474,6 +1896,36 @@ found_object (uintptr_t obj)
 	tracked_objects [num_tracked_objects - 1] = obj;
 }
 
+static int num_jit_helpers = 0;
+static int jit_helpers_code_size = 0;
+
+static const char*
+code_buffer_desc (int type)
+{
+	switch (type) {
+	case MONO_PROFILER_CODE_BUFFER_METHOD:
+		return "method";
+	case MONO_PROFILER_CODE_BUFFER_METHOD_TRAMPOLINE:
+		return "method trampoline";
+	case MONO_PROFILER_CODE_BUFFER_UNBOX_TRAMPOLINE:
+		return "unbox trampoline";
+	case MONO_PROFILER_CODE_BUFFER_IMT_TRAMPOLINE:
+		return "imt trampoline";
+	case MONO_PROFILER_CODE_BUFFER_GENERICS_TRAMPOLINE:
+		return "generics trampoline";
+	case MONO_PROFILER_CODE_BUFFER_SPECIFIC_TRAMPOLINE:
+		return "specific trampoline";
+	case MONO_PROFILER_CODE_BUFFER_HELPER:
+		return "misc helper";
+	case MONO_PROFILER_CODE_BUFFER_MONITOR:
+		return "monitor/lock";
+	case MONO_PROFILER_CODE_BUFFER_DELEGATE_INVOKE:
+		return "delegate invoke";
+	default:
+		return "unspecified";
+	}
+}
+
 #define OBJ_ADDR(diff) ((obj_base + diff) << 3)
 #define LOG_TIME(base,diff) /*fprintf("outfile, time %llu + %llu near offset %d\n", base, diff, p - ctx->buf)*/
 
@@ -1514,13 +1966,13 @@ decode_buffer (ProfContext *ctx)
 	thread_id = read_int64 (p + 32);
 	method_base = read_int64 (p + 40);
 	if (debug)
-		fprintf (outfile, "buf: thread:%x, len: %d, time: %llu, file offset: %llu\n", thread_id, len, time_base, file_offset);
+		fprintf (outfile, "buf: thread:%zx, len: %d, time: %llu, file offset: %llu\n", thread_id, len, (unsigned long long) time_base, (unsigned long long) file_offset);
 	thread = load_thread (ctx, thread_id);
 	if (!load_data (ctx, len))
 		return 0;
 	if (!startup_time) {
 		startup_time = time_base;
-		if (time_from) {
+		if (use_time_filter) {
 			time_from += startup_time;
 			time_to += startup_time;
 		}
@@ -1541,7 +1993,7 @@ decode_buffer (ProfContext *ctx)
 			if (subtype == TYPE_GC_RESIZE) {
 				uint64_t new_size = decode_uleb128 (p, &p);
 				if (debug)
-					fprintf (outfile, "gc heap resized to %llu\n", new_size);
+					fprintf (outfile, "gc heap resized to %llu\n", (unsigned long long) new_size);
 				gc_resizes++;
 				if (new_size > max_heap_size)
 					max_heap_size = new_size;
@@ -1549,7 +2001,7 @@ decode_buffer (ProfContext *ctx)
 				uint64_t ev = decode_uleb128 (p, &p);
 				int gen = decode_uleb128 (p, &p);
 				if (debug)
-					fprintf (outfile, "gc event for gen%d: %s at %llu (thread: 0x%x)\n", gen, gc_event_name (ev), time_base, thread->thread_id);
+					fprintf (outfile, "gc event for gen%d: %s at %llu (thread: 0x%zx)\n", gen, gc_event_name (ev), (unsigned long long) time_base, thread->thread_id);
 				if (gen > 2) {
 					fprintf (outfile, "incorrect gc gen: %d\n", gen);
 					break;
@@ -1582,10 +2034,11 @@ decode_buffer (ProfContext *ctx)
 				if (htype > 3)
 					return 0;
 				handle_info [htype].created++;
+				handle_info [htype].live++;
 				add_trace_thread (thread, &handle_info [htype].traces, 1);
 				/* FIXME: we don't take into account timing here */
-				if (handle_info [htype].created > handle_info [htype].max_live)
-					handle_info [htype].max_live = handle_info [htype].created;
+				if (handle_info [htype].live > handle_info [htype].max_live)
+					handle_info [htype].max_live = handle_info [htype].live;
 				if (num_tracked_objects)
 					track_handle (OBJ_ADDR (objdiff), htype, handle);
 				if (debug)
@@ -1595,7 +2048,8 @@ decode_buffer (ProfContext *ctx)
 				uint32_t handle = decode_uleb128 (p, &p);
 				if (htype > 3)
 					return 0;
-				handle_info [htype].created--;
+				handle_info [htype].destroyed ++;
+				handle_info [htype].live--;
 				if (debug)
 					fprintf (outfile, "handle (%s) %u destroyed\n", get_handle_name (htype), handle);
 			}
@@ -1616,7 +2070,7 @@ decode_buffer (ProfContext *ctx)
 					return 0;
 				}
 				if (debug)
-					fprintf (outfile, "loaded class %p (%s in %p) at %llu\n", (void*)(ptr_base + ptrdiff), p, (void*)(ptr_base + imptrdiff), time_base);
+					fprintf (outfile, "loaded class %p (%s in %p) at %llu\n", (void*)(ptr_base + ptrdiff), p, (void*)(ptr_base + imptrdiff), (unsigned long long) time_base);
 				if (!error)
 					add_class (ptr_base + ptrdiff, (char*)p);
 				while (*p) p++;
@@ -1628,7 +2082,7 @@ decode_buffer (ProfContext *ctx)
 					return 0;
 				}
 				if (debug)
-					fprintf (outfile, "loaded image %p (%s) at %llu\n", (void*)(ptr_base + ptrdiff), p, time_base);
+					fprintf (outfile, "loaded image %p (%s) at %llu\n", (void*)(ptr_base + ptrdiff), p, (unsigned long long) time_base);
 				if (!error)
 					add_image (ptr_base + ptrdiff, (char*)p);
 				while (*p) p++;
@@ -1640,7 +2094,7 @@ decode_buffer (ProfContext *ctx)
 					fprintf (outfile, "non-zero flags in thread\n");
 					return 0;
 				}
-				nt = get_thread (ctx, ptr_base * ptrdiff);
+				nt = get_thread (ctx, ptr_base + ptrdiff);
 				nt->name = pstrdup ((char*)p);
 				if (debug)
 					fprintf (outfile, "thread %p named: %s\n", (void*)(ptr_base + ptrdiff), p);
@@ -1663,7 +2117,7 @@ decode_buffer (ProfContext *ctx)
 			LOG_TIME (time_base, tdiff);
 			time_base += tdiff;
 			if (debug)
-				fprintf (outfile, "alloced object %p, size %llu (%s) at %llu\n", (void*)OBJ_ADDR (objdiff), len, lookup_class (ptr_base + ptrdiff)->name, time_base);
+				fprintf (outfile, "alloced object %p, size %llu (%s) at %llu\n", (void*)OBJ_ADDR (objdiff), (unsigned long long) len, lookup_class (ptr_base + ptrdiff)->name, (unsigned long long) time_base);
 			if (has_bt) {
 				num_bt = 8;
 				frames = decode_bt (sframes, &num_bt, p, &p, ptr_base);
@@ -1703,15 +2157,20 @@ decode_buffer (ProfContext *ctx)
 			if (subtype == TYPE_JIT) {
 				intptr_t codediff = decode_sleb128 (p, &p);
 				int codelen = decode_uleb128 (p, &p);
+				MethodDesc *jitted_method;
 				if (debug)
 					fprintf (outfile, "jitted method %p (%s), size: %d, code: %p\n", (void*)(method_base), p, codelen, (void*)(ptr_base + codediff));
-				add_method (method_base, (char*)p, ptr_base + codediff, codelen);
+				jitted_method = add_method (method_base, (char*)p, ptr_base + codediff, codelen);
+				if (!(time_base >= time_from && time_base < time_to))
+					jitted_method->ignore_jit = 1;
 				while (*p) p++;
 				p++;
 			} else {
 				MethodDesc *method;
 				if ((thread_filter && thread_filter != thread->thread_id))
 					break;
+				if (!(time_base >= time_from && time_base < time_to))
+					break;
 				method = lookup_method (method_base);
 				if (subtype == TYPE_ENTER) {
 					add_trace_thread (thread, &method->traces, 1);
@@ -1727,13 +2186,13 @@ decode_buffer (ProfContext *ctx)
 		case TYPE_HEAP: {
 			int subtype = *p & 0xf0;
 			if (subtype == TYPE_HEAP_OBJECT) {
-				HeapObjectDesc *ho;
+				HeapObjectDesc *ho = NULL;
 				int i;
 				intptr_t objdiff = decode_sleb128 (p + 1, &p);
 				intptr_t ptrdiff = decode_sleb128 (p, &p);
 				uint64_t size = decode_uleb128 (p, &p);
 				uintptr_t num = decode_uleb128 (p, &p);
-				uintptr_t ref_offset;
+				uintptr_t ref_offset = 0;
 				uintptr_t last_obj_offset = 0;
 				ClassDesc *cd = lookup_class (ptr_base + ptrdiff);
 				if (size) {
@@ -1760,10 +2219,10 @@ decode_buffer (ProfContext *ctx)
 						track_obj_reference (OBJ_ADDR (obj1diff), OBJ_ADDR (objdiff), cd);
 				}
 				if (debug && size)
-					fprintf (outfile, "traced object %p, size %llu (%s), refs: %d\n", (void*)OBJ_ADDR (objdiff), size, cd->name, num);
+					fprintf (outfile, "traced object %p, size %llu (%s), refs: %zd\n", (void*)OBJ_ADDR (objdiff), (unsigned long long) size, cd->name, num);
 			} else if (subtype == TYPE_HEAP_ROOT) {
 				uintptr_t num = decode_uleb128 (p + 1, &p);
-				uintptr_t gc_num = decode_uleb128 (p, &p);
+				uintptr_t gc_num G_GNUC_UNUSED = decode_uleb128 (p, &p);
 				int i;
 				for (i = 0; i < num; ++i) {
 					intptr_t objdiff = decode_sleb128 (p, &p);
@@ -1826,6 +2285,8 @@ decode_buffer (ProfContext *ctx)
 			LOG_TIME (time_base, tdiff);
 			time_base += tdiff;
 			record = (!thread_filter || thread_filter == thread->thread_id);
+			if (!(time_base >= time_from && time_base < time_to))
+				record = 0;
 			if (event == MONO_PROFILER_MONITOR_CONTENTION) {
 				MonitorDesc *mdesc = lookup_monitor (OBJ_ADDR (objdiff));
 				if (record) {
@@ -1888,6 +2349,8 @@ decode_buffer (ProfContext *ctx)
 			LOG_TIME (time_base, tdiff);
 			time_base += tdiff;
 			record = (!thread_filter || thread_filter == thread->thread_id);
+			if (!(time_base >= time_from && time_base < time_to))
+				record = 0;
 			if (subtype == TYPE_CLAUSE) {
 				int clause_type = decode_uleb128 (p, &p);
 				int clause_num = decode_uleb128 (p, &p);
@@ -1921,6 +2384,30 @@ decode_buffer (ProfContext *ctx)
 			}
 			break;
 		}
+		case TYPE_RUNTIME: {
+			int subtype = *p & 0xf0;
+			uint64_t tdiff = decode_uleb128 (p + 1, &p);
+			LOG_TIME (time_base, tdiff);
+			time_base += tdiff;
+			if (subtype == TYPE_JITHELPER) {
+				int type = decode_uleb128 (p, &p);
+				intptr_t codediff = decode_sleb128 (p, &p);
+				int codelen = decode_uleb128 (p, &p);
+				const char *name;
+				if (type == MONO_PROFILER_CODE_BUFFER_SPECIFIC_TRAMPOLINE) {
+					name = (void*)p;
+					while (*p) p++;
+						p++;
+				} else {
+					name = code_buffer_desc (type);
+				}
+				num_jit_helpers++;
+				jit_helpers_code_size += codelen;
+				if (debug)
+					fprintf (outfile, "jit helper %s, size: %d, code: %p\n", name, codelen, (void*)(ptr_base + codediff));
+			}
+			break;
+		}
 		case TYPE_SAMPLE: {
 			int subtype = *p & 0xf0;
 			if (subtype == TYPE_SAMPLE_HIT) {
@@ -1930,10 +2417,24 @@ decode_buffer (ProfContext *ctx)
 				int count = decode_uleb128 (p, &p);
 				for (i = 0; i < count; ++i) {
 					uintptr_t ip = ptr_base + decode_sleb128 (p, &p);
-					add_stat_sample (sample_type, ip);
+					if ((tstamp >= time_from && tstamp < time_to))
+						add_stat_sample (sample_type, ip);
 					if (debug)
 						fprintf (outfile, "sample hit, type: %d at %p\n", sample_type, (void*)ip);
 				}
+				if (ctx->data_version > 5) {
+					count = decode_uleb128 (p, &p);
+					for (i = 0; i < count; ++i) {
+						MethodDesc *method;
+						int64_t ptrdiff = decode_sleb128 (p, &p);
+						int il_offset = decode_sleb128 (p, &p);
+						int native_offset = decode_sleb128 (p, &p);
+						method_base += ptrdiff;
+						method = lookup_method (method_base);
+						if (debug)
+							fprintf (outfile, "sample hit bt %d: %s at IL offset %d (native: %d)\n", i, method->name, il_offset, native_offset);
+					}
+				}
 			} else if (subtype == TYPE_SAMPLE_USYM) {
 				/* un unmanaged symbol description */
 				uintptr_t addr = ptr_base + decode_sleb128 (p + 1, &p);
@@ -1949,7 +2450,7 @@ decode_buffer (ProfContext *ctx)
 				/* un unmanaged binary loaded in memory */
 				uint64_t tdiff = decode_uleb128 (p + 1, &p);
 				uintptr_t addr = decode_sleb128 (p, &p);
-				uint64_t offset = decode_uleb128 (p, &p);
+				uint64_t offset G_GNUC_UNUSED = decode_uleb128 (p, &p);
 				uintptr_t size = decode_uleb128 (p, &p);
 				char *name;
 				LOG_TIME (time_base, tdiff);
@@ -1960,13 +2461,101 @@ decode_buffer (ProfContext *ctx)
 					fprintf (outfile, "unmanaged binary %s at %p\n", name, (void*)addr);
 				while (*p) p++;
 				p++;
+			} else if (subtype == TYPE_SAMPLE_COUNTERS_DESC) {
+				uint64_t i, len = decode_uleb128 (p + 1, &p);
+				for (i = 0; i < len; i++) {
+					uint64_t type, unit, variance, index;
+					uint64_t section = decode_uleb128 (p, &p);
+					char *section_str, *name;
+					if (section != MONO_COUNTER_PERFCOUNTERS) {
+						section_str = (char*) section_name (section);
+					} else {
+						section_str = pstrdup ((char*)p);
+						while (*p++);
+					}
+					name = pstrdup ((char*)p);
+					while (*p++);
+					type = decode_uleb128 (p, &p);
+					unit = decode_uleb128 (p, &p);
+					variance = decode_uleb128 (p, &p);
+					index = decode_uleb128 (p, &p);
+					add_counter (section_str, name, (int)type, (int)unit, (int)variance, (int)index);
+				}
+			} else if (subtype == TYPE_SAMPLE_COUNTERS) {
+				int i;
+				CounterValue *value, *previous = NULL;
+				CounterList *list;
+				uint64_t timestamp = decode_uleb128 (p + 1, &p);
+				uint64_t time_between = timestamp / 1000 * 1000 * 1000 * 1000 + startup_time;
+				while (1) {
+					uint64_t type, index = decode_uleb128 (p, &p);
+					if (index == 0)
+						break;
+
+					for (list = counters; list; list = list->next) {
+						if (list->counter->index == (int)index) {
+							previous = list->counter->values_last;
+							break;
+						}
+					}
+
+					type = decode_uleb128 (p, &p);
+
+					value = calloc (1, sizeof (CounterValue));
+					value->timestamp = timestamp;
+
+					switch (type) {
+					case MONO_COUNTER_INT:
+#if SIZEOF_VOID_P == 4
+					case MONO_COUNTER_WORD:
+#endif
+						value->buffer = malloc (sizeof (int32_t));
+						*(int32_t*)value->buffer = (int32_t)decode_sleb128 (p, &p) + (previous ? (*(int32_t*)previous->buffer) : 0);
+						break;
+					case MONO_COUNTER_UINT:
+						value->buffer = malloc (sizeof (uint32_t));
+						*(uint32_t*)value->buffer = (uint32_t)decode_uleb128 (p, &p) + (previous ? (*(uint32_t*)previous->buffer) : 0);
+						break;
+					case MONO_COUNTER_LONG:
+#if SIZEOF_VOID_P == 8
+					case MONO_COUNTER_WORD:
+#endif
+					case MONO_COUNTER_TIME_INTERVAL:
+						value->buffer = malloc (sizeof (int64_t));
+						*(int64_t*)value->buffer = (int64_t)decode_sleb128 (p, &p) + (previous ? (*(int64_t*)previous->buffer) : 0);
+						break;
+					case MONO_COUNTER_ULONG:
+						value->buffer = malloc (sizeof (uint64_t));
+						*(uint64_t*)value->buffer = (uint64_t)decode_uleb128 (p, &p) + (previous ? (*(uint64_t*)previous->buffer) : 0);
+						break;
+					case MONO_COUNTER_DOUBLE:
+						value->buffer = malloc (sizeof (double));
+#if TARGET_BYTE_ORDER == G_LITTLE_ENDIAN
+						for (i = 0; i < sizeof (double); i++)
+#else
+						for (i = sizeof (double) - 1; i >= 0; i--)
+#endif
+							value->buffer[i] = *p++;
+						break;
+					case MONO_COUNTER_STRING:
+						if (*p++ == 0) {
+							value->buffer = NULL;
+						} else {
+							value->buffer = (unsigned char*) pstrdup ((char*)p);
+							while (*p++);
+						}
+						break;
+					}
+					if (time_between >= time_from && time_between <= time_to)
+						add_counter_value (index, value);
+				}
 			} else {
 				return 0;
 			}
 			break;
 		}
 		default:
-			fprintf (outfile, "unhandled profiler event: 0x%x at file offset: %llu + %d (len: %d\n)\n", *p, file_offset, p - ctx->buf, len);
+			fprintf (outfile, "unhandled profiler event: 0x%x at file offset: %llu + %lld (len: %d\n)\n", *p, (unsigned long long) file_offset, (long long) (p - ctx->buf), len);
 			exit (1);
 		}
 	}
@@ -2070,7 +2659,7 @@ dump_traces (TraceDesc *traces, const char *desc)
 		bt = traces->traces [j].bt;
 		if (!bt->count)
 			continue;
-		fprintf (outfile, "\t%llu %s from:\n", traces->traces [j].count, desc);
+		fprintf (outfile, "\t%llu %s from:\n", (unsigned long long) traces->traces [j].count, desc);
 		for (k = 0; k < bt->count; ++k)
 			fprintf (outfile, "\t\t%s\n", bt->methods [k]->name);
 	}
@@ -2091,12 +2680,12 @@ dump_exceptions (void)
 {
 	int i;
 	fprintf (outfile, "\nException summary\n");
-	fprintf (outfile, "\tThrows: %llu\n", throw_count);
+	fprintf (outfile, "\tThrows: %llu\n", (unsigned long long) throw_count);
 	dump_traces (&exc_traces, "throws");
 	for (i = 0; i <= MONO_EXCEPTION_CLAUSE_FAULT; ++i) {
 		if (!clause_summary [i])
 			continue;
-		fprintf (outfile, "\tExecuted %s clauses: %llu\n", clause_name (i), clause_summary [i]);
+		fprintf (outfile, "\tExecuted %s clauses: %llu\n", clause_name (i), (unsigned long long) clause_summary [i]);
 	}
 }
 
@@ -2136,9 +2725,9 @@ dump_monitors (void)
 			mdesc->wait_time/1000000000.0, mdesc->max_wait_time/1000000000.0, mdesc->wait_time/1000000000.0/mdesc->contentions);
 		dump_traces (&mdesc->traces, "contentions");
 	}
-	fprintf (outfile, "\tLock contentions: %llu\n", monitor_contention);
-	fprintf (outfile, "\tLock acquired: %llu\n", monitor_acquired);
-	fprintf (outfile, "\tLock failures: %llu\n", monitor_failed);
+	fprintf (outfile, "\tLock contentions: %llu\n", (unsigned long long) monitor_contention);
+	fprintf (outfile, "\tLock acquired: %llu\n", (unsigned long long) monitor_acquired);
+	fprintf (outfile, "\tLock failures: %llu\n", (unsigned long long) monitor_failed);
 }
 
 static void
@@ -2147,20 +2736,25 @@ dump_gcs (void)
 	int i;
 	fprintf (outfile, "\nGC summary\n");
 	fprintf (outfile, "\tGC resizes: %d\n", gc_resizes);
-	fprintf (outfile, "\tMax heap size: %llu\n", max_heap_size);
-	fprintf (outfile, "\tObject moves: %llu\n", gc_object_moves);
+	fprintf (outfile, "\tMax heap size: %llu\n", (unsigned long long) max_heap_size);
+	fprintf (outfile, "\tObject moves: %llu\n", (unsigned long long) gc_object_moves);
 	for (i = 0; i < 3; ++i) {
 		if (!gc_info [i].count)
 			continue;
 		fprintf (outfile, "\tGen%d collections: %d, max time: %lluus, total time: %lluus, average: %lluus\n",
-			i, gc_info [i].count, gc_info [i].max_time / 1000, gc_info [i].total_time / 1000,
-			gc_info [i].total_time / gc_info [i].count / 1000);
+			i, gc_info [i].count,
+			(unsigned long long) (gc_info [i].max_time / 1000),
+			(unsigned long long) (gc_info [i].total_time / 1000),
+			(unsigned long long) (gc_info [i].total_time / gc_info [i].count / 1000));
 	}
 	for (i = 0; i < 3; ++i) {
 		if (!handle_info [i].max_live)
 			continue;
 		fprintf (outfile, "\tGC handles %s: created: %llu, destroyed: %llu, max: %llu\n",
-			get_handle_name (i), handle_info [i].created, handle_info [i].destroyed, handle_info [i].max_live);
+			get_handle_name (i),
+			(unsigned long long) (handle_info [i].created),
+			(unsigned long long) (handle_info [i].destroyed),
+			(unsigned long long) (handle_info [i].max_live));
 		dump_traces (&handle_info [i].traces, "created");
 	}
 }
@@ -2176,7 +2770,7 @@ dump_jit (void)
 	for (i = 0; i < HASH_SIZE; ++i) {
 		m = method_hash [i];
 		for (m = method_hash [i]; m; m = m->next) {
-			if (!m->code)
+			if (!m->code || m->ignore_jit)
 				continue;
 			compiled_methods++;
 			code_size += m->len;
@@ -2184,6 +2778,8 @@ dump_jit (void)
 	}
 	fprintf (outfile, "\tCompiled methods: %d\n", compiled_methods);
 	fprintf (outfile, "\tGenerated code size: %d\n", code_size);
+	fprintf (outfile, "\tJIT helpers: %d\n", num_jit_helpers);
+	fprintf (outfile, "\tJIT helpers code size: %d\n", jit_helpers_code_size);
 }
 
 static void
@@ -2214,11 +2810,15 @@ dump_allocations (void)
 			fprintf (outfile, "\nAllocation summary\n");
 			fprintf (outfile, "%10s %10s %8s Type name\n", "Bytes", "Count", "Average");
 		}
-		fprintf (outfile, "%10llu %10d %8llu %s\n", cd->alloc_size, cd->allocs, cd->alloc_size / cd->allocs, cd->name);
+		fprintf (outfile, "%10llu %10zd %8llu %s\n",
+			(unsigned long long) (cd->alloc_size),
+			cd->allocs,
+			(unsigned long long) (cd->alloc_size / cd->allocs),
+			cd->name);
 		dump_traces (&cd->traces, "bytes");
 	}
 	if (allocs)
-		fprintf (outfile, "Total memory allocated: %llu bytes in %d objects\n", size, allocs);
+		fprintf (outfile, "Total memory allocated: %llu bytes in %zd objects\n", (unsigned long long) size, allocs);
 }
 
 enum {
@@ -2304,11 +2904,15 @@ dump_methods (void)
 			fprintf (outfile, "\nMethod call summary\n");
 			fprintf (outfile, "%8s %8s %10s Method name\n", "Total(ms)", "Self(ms)", "Calls");
 		}
-		fprintf (outfile, "%8llu %8llu %10llu %s\n", msecs, smsecs, cd->calls, cd->name);
+		fprintf (outfile, "%8llu %8llu %10llu %s\n",
+			(unsigned long long) (msecs),
+			(unsigned long long) (smsecs),
+			(unsigned long long) (cd->calls),
+			cd->name);
 		dump_traces (&cd->traces, "calls");
 	}
 	if (calls)
-		fprintf (outfile, "Total calls: %llu\n", calls);
+		fprintf (outfile, "Total calls: %llu\n", (unsigned long long) calls);
 }
 
 static int
@@ -2353,7 +2957,9 @@ dump_rev_claases (HeapClassRevRef *revs, int count)
 		return;
 	for (j = 0; j < count; ++j) {
 		HeapClassDesc *cd = revs [j].klass;
-		fprintf (outfile, "\t\t%llu references from: %s\n", revs [j].count, cd->klass->name);
+		fprintf (outfile, "\t\t%llu references from: %s\n",
+			(unsigned long long) (revs [j].count),
+			cd->klass->name);
 	}
 }
 
@@ -2377,8 +2983,12 @@ heap_shot_summary (HeapShot *hs, int hs_num, HeapShot *last_hs)
 	}
 	hs->sorted = sorted;
 	qsort (sorted, ccount, sizeof (void*), compare_heap_class);
-	fprintf (outfile, "\n\tHeap shot %d at %.3f secs: size: %llu, object count: %llu, class count: %d, roots: %d\n",
-		hs_num, (hs->timestamp - startup_time)/1000000000.0, size, count, ccount, hs->num_roots);
+	fprintf (outfile, "\n\tHeap shot %d at %.3f secs: size: %llu, object count: %llu, class count: %d, roots: %zd\n",
+		hs_num,
+		(hs->timestamp - startup_time)/1000000000.0,
+		(unsigned long long) (size),
+		(unsigned long long) (count),
+		ccount, hs->num_roots);
 	if (!verbose && ccount > 30)
 		ccount = 30;
 	fprintf (outfile, "\t%10s %10s %8s Class name\n", "Bytes", "Count", "Average");
@@ -2389,11 +2999,15 @@ heap_shot_summary (HeapShot *hs, int hs_num, HeapShot *last_hs)
 		cd = sorted [i];
 		if (last_hs)
 			ocd = heap_class_lookup (last_hs, cd->klass);
-		fprintf (outfile, "\t%10llu %10llu %8llu %s", cd->total_size, cd->count, cd->total_size / cd->count, cd->klass->name);
+		fprintf (outfile, "\t%10llu %10llu %8llu %s",
+			(unsigned long long) (cd->total_size),
+			(unsigned long long) (cd->count),
+			(unsigned long long) (cd->total_size / cd->count),
+			cd->klass->name);
 		if (ocd) {
 			int64_t bdiff = cd->total_size - ocd->total_size;
 			int64_t cdiff = cd->count - ocd->count;
-			fprintf (outfile, " (bytes: %+lld, count: %+lld)\n", bdiff, cdiff);
+			fprintf (outfile, " (bytes: %+lld, count: %+lld)\n", (long long) bdiff, (long long) cdiff);
 		} else {
 			fprintf (outfile, "\n");
 		}
@@ -2408,7 +3022,7 @@ heap_shot_summary (HeapShot *hs, int hs_num, HeapShot *last_hs)
 		assert (cd->rev_count == k);
 		qsort (rev_sorted, cd->rev_count, sizeof (HeapClassRevRef), compare_rev_class);
 		if (cd->root_references)
-			fprintf (outfile, "\t\t%d root references (%d pinning)\n", cd->root_references, cd->pinned_references);
+			fprintf (outfile, "\t\t%zd root references (%zd pinning)\n", cd->root_references, cd->pinned_references);
 		dump_rev_claases (rev_sorted, cd->rev_count);
 		free (rev_sorted);
 	}
@@ -2463,7 +3077,7 @@ flush_context (ProfContext *ctx)
 	}
 }
 
-static const char *reports = "header,jit,gc,sample,alloc,call,metadata,exception,monitor,thread,heapshot";
+static const char *reports = "header,jit,gc,sample,alloc,call,metadata,exception,monitor,thread,heapshot,counters";
 
 static const char*
 match_option (const char *p, const char *opt)
@@ -2538,6 +3152,11 @@ print_reports (ProfContext *ctx, const char *reps, int parse_only)
 				dump_samples ();
 			continue;
 		}
+		if ((opt = match_option (p, "counters")) != p) {
+			if (!parse_only)
+				dump_counters ();
+			continue;
+		}
 		return 0;
 	}
 	return 1;
@@ -2572,6 +3191,8 @@ usage (void)
 	printf ("\t                     %s\n", reports);
 	printf ("\t--method-sort=MODE   sort methods according to MODE: total, self, calls\n");
 	printf ("\t--alloc-sort=MODE    sort allocations according to MODE: bytes, count\n");
+	printf ("\t--counters-sort=MODE sort counters according to MODE: time, category\n");
+	printf ("\t                     only accessible in verbose mode\n");
 	printf ("\t--track=OB1[,OB2...] track what happens to objects OBJ1, O2 etc.\n");
 	printf ("\t--find=FINDSPEC      find and track objects matching FINFSPEC, where FINDSPEC is:\n");
 	printf ("\t                     S:minimum_size or T:partial_name\n");
@@ -2615,6 +3236,16 @@ main (int argc, char *argv[])
 				usage ();
 				return 1;
 			}
+		} else if (strncmp ("--counters-sort=", argv [i], 16) == 0) {
+			const char *val = argv [i] + 16;
+			if (strcmp (val, "time") == 0) {
+				counters_sort_mode = COUNTERS_SORT_TIME;
+			} else if (strcmp (val, "category") == 0) {
+				counters_sort_mode = COUNTERS_SORT_CATEGORY;
+			} else {
+				usage ();
+				return 1;
+			}
 		} else if (strncmp ("--reports=", argv [i], 10) == 0) {
 			const char *val = argv [i] + 10;
 			if (!print_reports (NULL, val, 1)) {
@@ -2674,6 +3305,7 @@ main (int argc, char *argv[])
 			}
 			time_from = from_secs * 1000000000;
 			time_to = to_secs * 1000000000;
+			use_time_filter = 1;
 		} else if (strcmp ("--verbose", argv [i]) == 0) {
 			verbose++;
 		} else if (strcmp ("--traces", argv [i]) == 0) {
