@@ -8,6 +8,8 @@
  * Copyright 2011 Xamarin, Inc (http://www.xamarin.com)
  */
 
+#include <config.h>
+
 #include <mono/utils/mono-compiler.h>
 #include <mono/utils/mono-semaphore.h>
 #include <mono/utils/mono-threads.h>
@@ -15,6 +17,7 @@
 #include <mono/utils/hazard-pointer.h>
 #include <mono/utils/mono-memory-model.h>
 #include <mono/utils/mono-mmap.h>
+#include <mono/utils/atomic.h>
 
 #include <errno.h>
 
@@ -42,7 +45,12 @@ static MonoSemType global_suspend_semaphore;
 static size_t thread_info_size;
 static MonoThreadInfoCallbacks threads_callbacks;
 static MonoThreadInfoRuntimeCallbacks runtime_callbacks;
-static MonoNativeTlsKey thread_info_key, thread_exited_key, small_id_key;
+static MonoNativeTlsKey thread_info_key, thread_exited_key;
+#ifdef HAVE_KW_THREAD
+static __thread guint32 tls_small_id MONO_TLS_FAST;
+#else
+static MonoNativeTlsKey small_id_key;
+#endif
 static MonoLinkedListSet thread_list;
 static gboolean disable_new_interrupt = FALSE;
 static gboolean mono_threads_inited = FALSE;
@@ -121,7 +129,11 @@ int
 mono_thread_info_register_small_id (void)
 {
 	int small_id = mono_thread_small_id_alloc ();
+#ifdef HAVE_KW_THREAD
+	tls_small_id = small_id;
+#else
 	mono_native_tls_set_value (small_id_key, GUINT_TO_POINTER (small_id + 1));
+#endif
 	return small_id;
 }
 
@@ -144,7 +156,7 @@ register_thread (MonoThreadInfo *info, gpointer baseptr)
 
 	if (threads_callbacks.thread_register) {
 		if (threads_callbacks.thread_register (info, baseptr) == NULL) {
-			g_warning ("thread registation failed\n");
+			// g_warning ("thread registation failed\n");
 			g_free (info);
 			return NULL;
 		}
@@ -177,7 +189,9 @@ unregister_thread (void *arg)
 	 * TLS destruction order is not reliable so small_id might be cleaned up
 	 * before us.
 	 */
+#ifndef HAVE_KW_THREAD
 	mono_native_tls_set_value (small_id_key, GUINT_TO_POINTER (info->small_id + 1));
+#endif
 
 	info->thread_state = STATE_SHUTTING_DOWN;
 
@@ -274,10 +288,14 @@ mono_thread_info_current (void)
 int
 mono_thread_info_get_small_id (void)
 {
+#ifdef HAVE_KW_THREAD
+	return tls_small_id;
+#else
 	gpointer val = mono_native_tls_get_value (small_id_key);
 	if (!val)
 		return -1;
 	return GPOINTER_TO_INT (val) - 1;
+#endif
 }
 
 MonoLinkedListSet*
@@ -286,6 +304,31 @@ mono_thread_info_list_head (void)
 	return &thread_list;
 }
 
+/**
+ * mono_threads_attach_tools_thread
+ *
+ * Attach the current thread as a tool thread. DON'T USE THIS FUNCTION WITHOUT READING ALL DISCLAIMERS.
+ *
+ * A tools thread is a very special kind of thread that needs access to core runtime facilities but should
+ * not be counted as a regular thread for high order facilities such as executing managed code or accessing
+ * the managed heap.
+ *
+ * This is intended only to tools such as a profiler than needs to be able to use our lock-free support when
+ * doing things like resolving backtraces in their background processing thread.
+ */
+void
+mono_threads_attach_tools_thread (void)
+{
+	int dummy = 0;
+	MonoThreadInfo *info;
+
+	/* Must only be called once */
+	g_assert (!mono_native_tls_get_value (thread_info_key));
+
+	info = mono_thread_info_attach (&dummy);
+	info->tools_thread = TRUE;
+}
+
 MonoThreadInfo*
 mono_thread_info_attach (void *baseptr)
 {
@@ -359,7 +402,9 @@ mono_threads_init (MonoThreadInfoCallbacks *callbacks, size_t info_size)
 #endif
 	g_assert (res);
 
+#ifndef HAVE_KW_THREAD
 	res = mono_native_tls_alloc (&small_id_key, NULL);
+#endif
 	g_assert (res);
 
 	MONO_SEM_INIT (&global_suspend_semaphore, 1);
@@ -426,7 +471,7 @@ mono_thread_info_suspend_sync (MonoNativeThreadId tid, gboolean interrupt_kernel
 		return info;
 	}
 
-	if (!mono_threads_core_suspend (info)) {
+	if (!mono_threads_core_suspend (info, interrupt_kernel)) {
 		MONO_SEM_POST (&info->suspend_semaphore);
 		mono_hazard_pointer_clear (hp, 1);
 		*error_condition = "Could not suspend thread";
@@ -605,7 +650,7 @@ mono_thread_info_safe_suspend_sync (MonoNativeThreadId id, gboolean interrupt_ke
 	for (;;) {
 		const char *suspend_error = "Unknown error";
 		if (!(info = mono_thread_info_suspend_sync (id, interrupt_kernel, &suspend_error))) {
-			g_warning ("failed to suspend thread %p due to %s, hopefully it is dead", (gpointer)id, suspend_error);
+			// g_warning ("failed to suspend thread %p due to %s, hopefully it is dead", (gpointer)id, suspend_error);
 			mono_thread_info_suspend_unlock ();
 			return NULL;
 		}
@@ -614,7 +659,7 @@ mono_thread_info_safe_suspend_sync (MonoNativeThreadId id, gboolean interrupt_ke
 			break;
 
 		if (!mono_thread_info_core_resume (info)) {
-			g_warning ("failed to resume thread %p, hopefully it is dead", (gpointer)id);
+			// g_warning ("failed to resume thread %p, hopefully it is dead", (gpointer)id);
 			mono_hazard_pointer_clear (mono_hazard_pointer_get (), 1);
 			mono_thread_info_suspend_unlock ();
 			return NULL;
@@ -731,11 +776,13 @@ mono_thread_info_new_interrupt_enabled (void)
 #if defined (HAVE_BOEHM_GC) && !defined (USE_INCLUDED_LIBGC)
 	return FALSE;
 #endif
-	/*port not done*/
 #if defined(HOST_WIN32)
-	return FALSE;
+	return !disable_new_interrupt;
 #endif
-#if defined (__i386__)
+#if defined (__i386__) || defined(__x86_64__)
+	return !disable_new_interrupt;
+#endif
+#if defined(__arm__) || defined(__aarch64__)
 	return !disable_new_interrupt;
 #endif
 	return FALSE;
@@ -852,7 +899,7 @@ mono_thread_info_open_handle (void)
 }
 
 /*
- * mono_thread_info_open_handle:
+ * mono_threads_open_thread_handle:
  *
  *   Return a io-layer/win32 handle for the thread identified by HANDLE/TID.
  * The handle need to be closed by calling CloseHandle () when it is no
@@ -869,3 +916,65 @@ mono_thread_info_set_name (MonoNativeThreadId tid, const char *name)
 {
 	mono_threads_core_set_name (tid, name);
 }
+
+/*
+ * mono_thread_info_prepare_interrupt:
+ *
+ *   See wapi_prepare_interrupt ().
+ */
+gpointer
+mono_thread_info_prepare_interrupt (HANDLE thread_handle)
+{
+	return mono_threads_core_prepare_interrupt (thread_handle);
+}
+
+void
+mono_thread_info_finish_interrupt (gpointer wait_handle)
+{
+	mono_threads_core_finish_interrupt (wait_handle);
+}
+
+void
+mono_thread_info_interrupt (HANDLE thread_handle)
+{
+	gpointer wait_handle;
+
+	wait_handle = mono_thread_info_prepare_interrupt (thread_handle);
+	mono_thread_info_finish_interrupt (wait_handle);
+}
+	
+void
+mono_thread_info_self_interrupt (void)
+{
+	mono_threads_core_self_interrupt ();
+}
+
+void
+mono_thread_info_clear_interruption (void)
+{
+	mono_threads_core_clear_interruption ();
+}
+
+/* info must be self or be held in a hazard pointer. */
+gboolean
+mono_threads_add_async_job (MonoThreadInfo *info, MonoAsyncJob job)
+{
+	MonoAsyncJob old_job;
+	do {
+		old_job = info->service_requests;
+		if (old_job & job)
+			return FALSE;
+	} while (InterlockedCompareExchange (&info->service_requests, old_job | job, old_job) != old_job);
+	return TRUE;
+}
+
+MonoAsyncJob
+mono_threads_consume_async_jobs (void)
+{
+	MonoThreadInfo *info = (MonoThreadInfo*)mono_native_tls_get_value (thread_info_key);
+
+	if (!info)
+		return 0;
+
+	return InterlockedExchange (&info->service_requests, 0);
+}
